      SUBROUTINE CARIOCHEM(LON,LAT,NIV,NTRA,SOLID,
#ifdef LINO_3
     +                     NCARIO3,CARO3PAR,
#endif
#ifdef LINCO
     +                     NCARIOCO,CARCOPAR,
#endif
#ifdef LINHNO_3
     +                     NCARIOHNO3,CARHNO3PAR,
#endif
     +                     DLON,DLAT,XSDAY,XDAY,DT,
     +                     TRANAM,TGRIM,XPAM,XPHFM,
     +                     QQQM,TRANSINV)
C
CCCCC
C
C   Purpose : compute linearised ozone, carbon monoxide and
C             nitric acid chemistries
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C
C   Centre National de Recherches Meteorologiques
C   Groupe de Modelisation de Grande Echelle et du Climat
C   Equipe de Recherches sur l Atmosphere Moyenne
C
CCCCC
C
C  Original : H. Teyssedre, 11/2003
C
C  Modified : version etendue a CO et HNO3 (H. Teyssedre, 12/2007)
C
CCC
	USE CSTES_MOD
C
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: LON,LAT,NIV,NTRA,NCARIO3,NCARIOCO,NCARIOHNO3
      REAL (KIND=8) :: XSDAY,XDAY,DT
      REAL (KIND=8)  , DIMENSION (LON) :: DLON
      REAL (KIND=8)  , DIMENSION (LAT) :: DLAT
      REAL (KIND=8)  , DIMENSION (LON,LAT,NIV) :: XPAM,XPHFM,TGRIM
#ifdef LINO_3
      REAL (KIND=8)  , DIMENSION (LON,LAT,NIV,NCARIO3) :: CARO3PAR
#endif
#ifdef LINCO
      REAL (KIND=8)  , DIMENSION (LON,LAT,NIV,NCARIOCO) :: CARCOPAR
#endif
#ifdef LINHNO_3
      REAL (KIND=8)  , DIMENSION (LON,LAT,NIV,NCARIOHNO3) :: CARHNO3PAR
#endif
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV,NTRA) :: QQQM
      LOGICAL :: SOLID, TRANSINV
      CHARACTER(LEN=15) :: TRANAM(NTRA)
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: IO3,ITF,ICO,IHNO3,ITRA,ILON,ILAT,INIV
      REAL (KIND=8) :: XMO3,XMCO,XMHNO3
      REAL (KIND=8) :: ZALPHA,SZASEUIL,TSEUIL,DPSUP,DPINF
      REAL (KIND=8) :: ZCOEFF,ZRAPP,ZVALOZ,ZVALCO,ZVALNA
#ifdef LINO_3
      REAL (KIND=8) , ALLOCATABLE :: O3T(:,:,:),sza(:,:)
      REAL (KIND=8) , ALLOCATABLE :: CHET(:,:,:)
#endif
      REAL (KIND=8) , ALLOCATABLE :: ZANEX(:,:,:)
      REAL (KIND=8) :: TAU6H,UNSURTAU1,UNSURTAU2
C  TEMPORAL SCHEME (1 : IMPLICIT, 0 : EXPLICIT)
      PARAMETER (ZALPHA=1.)
C  COMPLETE CHEMISTRY USES SZAlim=94. SWITCH TO SZASEUIL=94
C  IF COMPATIBILITY IS NEEDED
      PARAMETER (SZASEUIL=87., TSEUIL=195.)	
      PARAMETER (TAU6H=6.*3600.)
C
      LOGICAL :: LTF
C
      DATA XMO3   / 0.048 /
      DATA XMCO   / 0.030 /
      DATA XMHNO3 / 0.063 /
C
#ifdef LINO_3
#ifdef NIV_60
#define CPP_ESP 'O_3'
#else
#define CPP_ESP 'O_x'
#endif     
C      
      ALLOCATE (O3T(LON,LAT,NIV))
      ALLOCATE (sza(LON,LAT))
      ALLOCATE (CHET(LON,LAT,NIV))
#endif
      ALLOCATE (ZANEX(LON,LAT,NIV))
C
#ifdef LINO_3
C
      iO3=0
      iTF=0
      DO itra=1,NTRA
        IF (TRANAM(itra).EQ.CPP_ESP) iO3=itra
        IF (TRANAM(itra).EQ.'TRACEUR_FROID') iTF=itra
      ENDDO
C
      IF (iO3.EQ.0) THEN
        PRINT*,'ABORT : ',CPP_ESP,' IS NOT IN CURRENT SCHEME!'
        CALL MOCAGE_STOP
      ENDIF
C
      IF (iTF.EQ.0) THEN
        LTF=.FALSE.
      ELSE
        LTF=.TRUE.
      ENDIF
C
      IF (LTF) THEN
        IF (NCARIO3.NE.9) THEN
          PRINT*,'ABORT : THIS VERSION OF CARIOCHEM IS NOT CONSISTENT'
          PRINT*,'WITH THE FILE OF COEFFICIENTS FOR ',CPP_ESP,' !!!'
          CALL MOCAGE_STOP
        ENDIF
      ELSE
        IF (NCARIO3.NE.8) THEN
          PRINT*,'ABORT : THIS VERSION OF CARIOCHEM IS NOT CONSISTENT'
          PRINT*,'WITH THE FILE OF COEFFICIENTS FOR ',CPP_ESP,' !!!'
          CALL MOCAGE_STOP
        ENDIF
      ENDIF
C
C Ozone volumic mixing ratio is converted to mass mixing ratio
C
      DO ilon=1,LON*LAT*NIV
        QQQM(ilon,1,1,iO3)=XMO3*QQQM(ilon,1,1,iO3)/MAIR
      ENDDO
#endif
#ifdef LINCO
C
      iCO=0
      DO itra=1,NTRA
        IF (TRANAM(itra).EQ.'CO') iCO=itra
      ENDDO
C
      IF (iCO.EQ.0) THEN
        PRINT*,'ABORT : CO IS NOT IN CURRENT SCHEME!'
        STOP
      ENDIF
C
      IF (NCARIOCO.NE.5) THEN
        PRINT*,'ABORT : THIS VERSION OF CARIOCHEM IS NOT CONSISTENT'
        PRINT*,'WITH THE FILE OF COEFFICIENTS FOR CO !!!'
        CALL MOCAGE_STOP
      ENDIF
C
C Carbon monoxide volumic mixing ratio is converted to mass mixing ratio
C
      DO ilon=1,LON*LAT*NIV
        QQQM(ilon,1,1,iCO)=XMCO*QQQM(ilon,1,1,iCO)/MAIR
      ENDDO
#endif
#ifdef LINHNO_3
C
      iHNO3=0
      DO itra=1,NTRA
        IF (TRANAM(itra).EQ.'HNO_3') iHNO3=itra
      ENDDO
C
      IF (iHNO3.EQ.0) THEN
        PRINT*,"ABORT : HNO_3 IS NOT IN CURRENT SCHEME!"
        STOP
      ENDIF
C
      IF (NCARIOHNO3.NE.5) THEN
        PRINT*,'ABORT : THIS VERSION OF CARIOCHEM IS NOT CONSISTENT'
        PRINT*,'WITH THE FILE OF COEFFICIENTS FOR HNO3 !!!'
        CALL MOCAGE_STOP
      ENDIF
C
C Nitric acid volumic mixing ratio is converted to mass mixing ratio
C
      DO ilon=1,LON*LAT*NIV
        QQQM(ilon,1,1,iHNO3)=XMHNO3*QQQM(ilon,1,1,iHNO3)/MAIR
      ENDDO
#endif
#ifdef LINO_3
C
C Activation coefficient for heterogenous chemistry parametrization
C
      DO ilon=1,LON*LAT*NIV
        CHET(ilon,1,1)=0.
      ENDDO
C
      IF (SOLID) THEN
        IF (LTF) THEN
C
C Cold tracer evolution (semi-implicit scheme)
C
          DO iniv=1,NIV
          DO ilat=1,LAT
          DO ilon=1,LON
            UNSURTAU1=0.
            UNSURTAU2=CARO3PAR(ilon,ilat,iniv,9)
            IF ((ABS(DLAT(ilat)*CONV).GE.45.).AND.
     +          (TGRIM(ilon,ilat,iniv).LE.TSEUIL)) THEN
              UNSURTAU1=1./TAU6H
              UNSURTAU2=0.
            ENDIF
            QQQM(ilon,ilat,iniv,iTF)=
     +          (QQQM(ilon,ilat,iniv,iTF)+DT*UNSURTAU1)/
     +          (1.+DT*(UNSURTAU1+UNSURTAU2))
          ENDDO
          ENDDO
          ENDDO
        ENDIF
C
C Solar zenith angle
C
        CALL ZENITH (DLON, DLAT, XSDAY, XDAY, sza,
     +               LON, LAT)
C
        IF (LTF) THEN
          DO iniv=1,NIV
          DO ilat=1,LAT
          DO ilon=1,LON
            IF (sza(ilon,ilat).LE.SZASEUIL) THEN
              CHET(ilon,ilat,iniv)=CARO3PAR(ilon,ilat,iniv,8)*
     +                             QQQM(ilon,ilat,iniv,iTF)*
     +                       (195./TGRIM(ilon,ilat,iniv))**4.5
            ENDIF
          ENDDO
          ENDDO
          ENDDO
        ELSE
          DO iniv=1,NIV
          DO ilat=1,LAT
          DO ilon=1,LON
            IF ( (sza(ilon,ilat).LE.SZASEUIL).AND.
     +           (TGRIM(ilon,ilat,iniv).LE.TSEUIL) ) THEN
              CHET(ilon,ilat,iniv)=CARO3PAR(ilon,ilat,iniv,8)
            ENDIF
          ENDDO
          ENDDO
          ENDDO
        ENDIF
C
      ENDIF
C
C Ozone column above gridpoint (for both 3D ozone model and
C ozone coefficient for parameterization)
C
      DO ilon=1,LON*LAT
        O3T(ilon,1,1)=CARO3PAR(ilon,1,1,6)
      ENDDO
C
      DO iniv=2,NIV
        DO ilon=1,LON*LAT
          DPSUP=XPAM (ilon,1,iniv  )-XPHFM(ilon,1,iniv-1)
          DPINF=XPHFM(ilon,1,iniv-1)-XPAM (ilon,1,iniv-1)
          O3T(ilon,1,iniv)=O3T(ilon,1,iniv-1) +
     +       ( DPSUP*MAX(1. E-11, QQQM(ilon,1,iniv  ,iO3)) +
     +         DPINF*MAX(1. E-11, QQQM(ilon,1,iniv-1,iO3)) )/GG
          CARO3PAR(ilon,1,iniv,6)=CARO3PAR(ilon,1,iniv-1,6) +
     +       ( DPSUP*MAX(1. E-11, CARO3PAR(ilon,1,iniv  ,1)) +
     +         DPINF*MAX(1. E-11, CARO3PAR(ilon,1,iniv-1,1)) )/GG
        ENDDO
      ENDDO
C
C Ozone tendancy
C
      DO ilon=1,LON*LAT*NIV
        ZANEX(ilon,1,1) = 
     +         ( CARO3PAR(ilon,1,1,2) +
     +           CARO3PAR(ilon,1,1,5)*
     +          (TGRIM(ilon,1,1)-CARO3PAR(ilon,1,1,4)) +
     +           CARO3PAR(ilon,1,1,7)*
     +          (O3T  (ilon,1,1)-CARO3PAR(ilon,1,1,6)) -
     +           CARO3PAR(ilon,1,1,3)*CARO3PAR(ilon,1,1,1) )*DT
        ZCOEFF = CARO3PAR(ilon,1,1,3) + CHET(ilon,1,1)
        ZRAPP  = 1. - ZALPHA*DT*ZCOEFF
        ZRAPP  = MAX(ZRAPP, 1. E-06)
        ZVALOZ = (1. + (1.-ZALPHA)*DT*ZCOEFF)*
     +           QQQM(ilon,1,1,iO3)
        ZANEX(ilon,1,1) = 
     +          (ZANEX(ilon,1,1)+ZVALOZ) / ZRAPP
        ZANEX(ilon,1,1) = ZANEX(ilon,1,1)-QQQM(ilon,1,1,iO3)
      ENDDO
#ifdef NIV_47
C
C tendancy is not applied at first level
C
      DO ilon=1,LON*LAT
        ZANEX(ilon,1,1)=0.
      ENDDO
#endif
C
C Apply tendancy and ozone mass mixing ratio is converted back
C to volumic mixing ratio
C
      IF (.NOT.TRANSINV) THEN
        DO ilon=1,LON*LAT*NIV
          QQQM(ilon,1,1,iO3)=MAX(1.e-30,
     +     (QQQM(ilon,1,1,iO3)+ZANEX(ilon,1,1))*MAIR/XMO3)
        ENDDO
      ELSE
        DO ilon=1,LON*LAT*NIV
          QQQM(ilon,1,1,iO3)=MAX(1.e-30,
     +     (QQQM(ilon,1,1,iO3)-ZANEX(ilon,1,1))*MAIR/XMO3)
        ENDDO
      ENDIF
C
      DEALLOCATE (O3T)
      DEALLOCATE (sza)
      DEALLOCATE (CHET)
C
#endif
#ifdef LINCO
C
C Carbon monoxide tendancy
C
      DO ilon=1,LON*LAT*NIV
        ZANEX(ilon,1,1) = 
     +         ( CARCOPAR(ilon,1,1,2) +
     +           CARCOPAR(ilon,1,1,5)*
     +          (TGRIM(ilon,1,1)-CARCOPAR(ilon,1,1,4)) -
     +           CARCOPAR(ilon,1,1,3)*CARCOPAR(ilon,1,1,1) )*DT
        ZCOEFF = CARCOPAR(ilon,1,1,3)
        ZRAPP  = 1. - ZALPHA*DT*ZCOEFF
        ZRAPP  = MAX(ZRAPP, 1. E-06)
        ZVALCO = (1. + (1.-ZALPHA)*DT*ZCOEFF)*
     +           QQQM(ilon,1,1,iCO)
        ZANEX(ilon,1,1) = 
     +          (ZANEX(ilon,1,1)+ZVALCO) / ZRAPP
        ZANEX(ilon,1,1) = ZANEX(ilon,1,1)-QQQM(ilon,1,1,iCO)
      ENDDO
#ifdef NIV_47
C
C tendancy is not applied at first level
C
      DO ilon=1,LON*LAT
        ZANEX(ilon,1,1)=0.
      ENDDO
#endif
C
C Apply tendancy and carbone moxide mass mixing ratio is converted back
C to volumic mixing ratio
C
      IF (.NOT.TRANSINV) THEN
        DO ilon=1,LON*LAT*NIV
          QQQM(ilon,1,1,iCO)=MAX(1.e-30,
     +     (QQQM(ilon,1,1,iCO)+ZANEX(ilon,1,1))*MAIR/XMCO)
        ENDDO
      ELSE
        DO ilon=1,LON*LAT*NIV
          QQQM(ilon,1,1,iCO)=MAX(1.e-30,
     +     (QQQM(ilon,1,1,iCO)-ZANEX(ilon,1,1))*MAIR/XMCO)
        ENDDO
      ENDIF
#endif
#ifdef LINHNO_3
C
C Nitric acid tendancy
C
      DO ilon=1,LON*LAT*NIV
        ZANEX(ilon,1,1) = 
     +         ( CARHNO3PAR(ilon,1,1,2) +
     +           CARHNO3PAR(ilon,1,1,5)*
     +          (TGRIM(ilon,1,1)-CARHNO3PAR(ilon,1,1,4)) -
     +           CARHNO3PAR(ilon,1,1,3)*CARHNO3PAR(ilon,1,1,1) )*DT
        ZCOEFF = CARHNO3PAR(ilon,1,1,3)
        ZRAPP  = 1. - ZALPHA*DT*ZCOEFF
        ZRAPP  = MAX(ZRAPP, 1. E-06)
        ZVALNA = (1. + (1.-ZALPHA)*DT*ZCOEFF)*
     +           QQQM(ilon,1,1,iHNO3)
        ZANEX(ilon,1,1) = 
     +          (ZANEX(ilon,1,1)+ZVALNA) / ZRAPP
        ZANEX(ilon,1,1) = ZANEX(ilon,1,1)-QQQM(ilon,1,1,iHNO3)
      ENDDO
#ifdef NIV_47
C
C tendancy is not applied at first level
C
      DO ilon=1,LON*LAT
        ZANEX(ilon,1,1)=0.
      ENDDO
#endif
C
C Apply tendancy and nitric acid mass mixing ratio is converted back
C to volumic mixing ratio
C
      IF (.NOT.TRANSINV) THEN
        DO ilon=1,LON*LAT*NIV
          QQQM(ilon,1,1,iHNO3)=MAX(1.e-30,
     +     (QQQM(ilon,1,1,iHNO3)+ZANEX(ilon,1,1))*MAIR/XMHNO3)
        ENDDO
      ELSE
        DO ilon=1,LON*LAT*NIV
          QQQM(ilon,1,1,iHNO3)=MAX(1.e-30,
     +     (QQQM(ilon,1,1,iHNO3)-ZANEX(ilon,1,1))*MAIR/XMHNO3)
        ENDDO
      ENDIF
#endif
      DEALLOCATE (ZANEX)
C
      RETURN
      END
C
