SUBROUTINE CH_CONSERV (QQQM,HHHM,lon,lat,niv,nivbas,ntra,nsls)
!
!!!!!
!
!   CONSERVATION DES ATOMES DE CHLORE, DE BROME ET D'AZOTE
!   SUR UN PAS DE TEMPS CHIMIQUE
!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!
!   Centre National de Recherches Meteorologiques
!   Groupe de Modelisation de Grande Echelle et du Climat
!   Equipe de Recherches sur l'Atmosphere Moyenne
!
!!!!!
!
!  Original : H. Teyssedre, 09/2003
!
!  Modified : H. Teyssedre, version Fortran-90, 11/2006
!
!!!!!
!
#ifdef MPI
USE CSTES_MOD
#endif
!
IMPLICIT NONE
!
!  subroutine arguments
!
INTEGER , INTENT(IN) :: lon,lat,niv,nivbas,ntra,nsls
!
REAL (KIND=8), INTENT(INOUT) :: QQQM(lon,lat,niv,ntra)
REAL (KIND=8), INTENT(INOUT) :: HHHM(lon,lat,niv,nsls)
!
!  local variables
!
INTEGER :: ilon,iniv,i1,i2
REAL (KIND=8) :: totN, totCl, totBr
REAL (KIND=8) :: rapN, rapCl, rapBr
!
#ifdef MPI
i1 = i_deb(myid) ; i2 = i_fin(myid)
#else
i1 = 1           ; i2 = lon*lat
#endif
!
!!!!!
!
!  TRANSPORTED LONG-LIVED SPECIES AND FAMILIES (ARRAY QM)
!
!!!!!
!
!       1 : N2O         2 : CH4         3 : H2O         4 : HNO3        5 : N2O5
!       6 : CO          7 : OClO        8 : HCl         9 : ClONO2     10 : HOCl
!      11 : Cl2        12 : H2O2       13 : ClNO2      14 : HBr        15 : BrON
!      16 : HNO4       17 : Cl2O2      18 : HOBr       19 : BrCl       20 : HCHO
!      21 : MO2        22 : OP1        23 : CFC11      24 : CFC12      25 : CFC1
!      26 : CCl4       27 : CH3CCl3    28 : CH3Cl      29 : HCFC22     30 : CH3B
!      31 : H1211      32 : H1301      33 : H2SO4      34 : PSC        35 : HONO
!      36 : DMS        37 : SO2        38 : ETH        39 : HC3        40 : HC5
!      41 : HC8        42 : ETE        43 : OLT        44 : OLI        45 : DIEN
!      46 : ISO        47 : API        48 : LIM        49 : TOL        50 : XYL
!      51 : CSL        52 : ALD        53 : KET        54 : GLY        55 : MGLY
!      56 : DCB        57 : MACR       58 : UDD        59 : HKET       60 : ONIT
!      61 : PAN        62 : TPAN       63 : OP2        64 : PAA        65 : ETHP
!      66 : HC3P       67 : HC5P       68 : HC8P       69 : ETEP       70 : OLTP
!      71 : OLIP       72 : ISOP       73 : APIP       74 : LIMP       75 : PHO
!      76 : TOLP       77 : XYLP       78 : CSLP       79 : ACO3       80 : TCO3
!      81 : KETP       82 : OLNN       83 : OLND       84 : XO2        85 : SULF
!      86 : Ox         87 : NOx        88 : ClOx       89 : BrOx       90 : NOy
!      91 : Cly        92 : Bry
!
!!!!!
!
!  SHORT-LIVED SPECIES AT INSTANTANEOUS EQUILIBRIUM (ARRAY HM)
!
!!!!!
!
!       1 : O3P         2 : O1D         3 : O3          4 : N           5 : NO
!       6 : NO2         7 : NO3         8 : ClO         9 : Cl         10 : BrO
!      11 : Br         12 : H          13 : OH         14 : HO2        15 : CH3
!      16 : CH3O       17 : ADDT       18 : ADDX       19 : ADDC       20 : PANs
!      21 : RONO2s     22 : NOy        23 : NOz        24 : ResN       25 : COVp
!      26 : COVpB      27 : COVtot     28 : HOx        29 : Oz
!
!$omp parallel do default(none) &
!$omp shared(QQQM,HHHM,i1,i2,niv,nivbas,ntra,nsls) &
!$omp private(ilon,iniv,TOTCL,RAPCL,TOTBR,RAPBR,TOTN,RAPN)
do iniv=1,nivbas
  do ilon=i1,i2
!
!================================================================
!  CHECK CONSERVATION FOR Cl
!================================================================
!
    totCl = + QQQM(ilon,1,iniv,7) + QQQM(ilon,1,iniv,8) +           &
              QQQM(ilon,1,iniv,9) + QQQM(ilon,1,iniv,10) +          &
          2.0*QQQM(ilon,1,iniv,11) + QQQM(ilon,1,iniv,13) +         &
          2.0*QQQM(ilon,1,iniv,17) + QQQM(ilon,1,iniv,88)
!
! adjust to keep mass constant :
!
    rapCl = MAX( 0. , QQQM(ilon,1,iniv,91) -                        &
                      QQQM(ilon,1,iniv,19) ) / totCl
!
    QQQM(ilon,1,iniv,7) = QQQM(ilon,1,iniv,7) * rapCl
    QQQM(ilon,1,iniv,8) = QQQM(ilon,1,iniv,8) * rapCl
    QQQM(ilon,1,iniv,9) = QQQM(ilon,1,iniv,9) * rapCl
    QQQM(ilon,1,iniv,10) = QQQM(ilon,1,iniv,10) * rapCl
    QQQM(ilon,1,iniv,11) = QQQM(ilon,1,iniv,11) * rapCl
    QQQM(ilon,1,iniv,13) = QQQM(ilon,1,iniv,13) * rapCl
    QQQM(ilon,1,iniv,17) = QQQM(ilon,1,iniv,17) * rapCl
    QQQM(ilon,1,iniv,88) = QQQM(ilon,1,iniv,88) * rapCl
    HHHM(ilon,1,iniv,8) = HHHM(ilon,1,iniv,8) * rapCl
    HHHM(ilon,1,iniv,9) = HHHM(ilon,1,iniv,9) * rapCl
!
!================================================================
!  CHECK CONSERVATION FOR N
!================================================================
!
    totN = + QQQM(ilon,1,iniv,4) + 2.0*QQQM(ilon,1,iniv,5) +       &
             QQQM(ilon,1,iniv,16) + QQQM(ilon,1,iniv,35) +         &
             QQQM(ilon,1,iniv,55) + QQQM(ilon,1,iniv,60) +         &
             QQQM(ilon,1,iniv,61) + QQQM(ilon,1,iniv,62) +         &
             QQQM(ilon,1,iniv,82) + QQQM(ilon,1,iniv,83) +         &
             QQQM(ilon,1,iniv,87)
!
! adjust to keep mass constant :
!
    rapN = MAX( 0.0 , QQQM(ilon,1,iniv,90) -                       &
                      QQQM(ilon,1,iniv,9) - QQQM(ilon,1,iniv,13) - &
                      QQQM(ilon,1,iniv,15) ) / totN
!
    QQQM(ilon,1,iniv,4) = QQQM(ilon,1,iniv,4) * rapN
    QQQM(ilon,1,iniv,5) = QQQM(ilon,1,iniv,5) * rapN
    QQQM(ilon,1,iniv,16) = QQQM(ilon,1,iniv,16) * rapN
    QQQM(ilon,1,iniv,35) = QQQM(ilon,1,iniv,35) * rapN
    QQQM(ilon,1,iniv,55) = QQQM(ilon,1,iniv,55) * rapN
    QQQM(ilon,1,iniv,60) = QQQM(ilon,1,iniv,60) * rapN
    QQQM(ilon,1,iniv,61) = QQQM(ilon,1,iniv,61) * rapN
    QQQM(ilon,1,iniv,62) = QQQM(ilon,1,iniv,62) * rapN
    QQQM(ilon,1,iniv,82) = QQQM(ilon,1,iniv,82) * rapN
    QQQM(ilon,1,iniv,83) = QQQM(ilon,1,iniv,83) * rapN
    QQQM(ilon,1,iniv,87) = QQQM(ilon,1,iniv,87) * rapN
    HHHM(ilon,1,iniv,4) = HHHM(ilon,1,iniv,4) * rapN
    HHHM(ilon,1,iniv,5) = HHHM(ilon,1,iniv,5) * rapN
    HHHM(ilon,1,iniv,6) = HHHM(ilon,1,iniv,6) * rapN
    HHHM(ilon,1,iniv,7) = HHHM(ilon,1,iniv,7) * rapN
!
!================================================================
!  CHECK CONSERVATION FOR Br
!================================================================
!
    totBr = + QQQM(ilon,1,iniv,14) + QQQM(ilon,1,iniv,15) +      &
              QQQM(ilon,1,iniv,18) + QQQM(ilon,1,iniv,19) +      &
              QQQM(ilon,1,iniv,89)
!
! adjust to keep mass constant :
!
    rapBr = MAX( 0.0 , QQQM(ilon,1,iniv,92) ) /totBr
!
    QQQM(ilon,1,iniv,14) = QQQM(ilon,1,iniv,14) * rapBr
    QQQM(ilon,1,iniv,15) = QQQM(ilon,1,iniv,15) * rapBr
    QQQM(ilon,1,iniv,18) = QQQM(ilon,1,iniv,18) * rapBr
    QQQM(ilon,1,iniv,19) = QQQM(ilon,1,iniv,19) * rapBr
    QQQM(ilon,1,iniv,87) = QQQM(ilon,1,iniv,87) * rapBr
    HHHM(ilon,1,iniv,10) = HHHM(ilon,1,iniv,10) * rapBr
    HHHM(ilon,1,iniv,11) = HHHM(ilon,1,iniv,11) * rapBr
!
  enddo
enddo
!$omp end parallel do
!
RETURN
END SUBROUTiNE CH_CONSERV
