SUBROUTINE CH_ELEM(QM0,QM,HM,imax,ntra,nsls)
!
!!!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_0 
!
!!! DATE : 15/05/2005
!
!
!   CONSERVATION DES ATOMES DE CHLORE, DE BROME ET D'AZOTE
!   SUR UN PAS DE TEMPS CHIMIQUE
!
!!!!!
!
!  Original : H. Teyssedre, 08/2000
!
!  Modified : D. Charpin, version Fortran-90, 08/2006
!
!!!!!
!
#ifdef MPI
USE CSTES_MOD
#endif
!
IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
INTEGER , INTENT(IN) :: imax, ntra, nsls
REAL (KIND=8) , DIMENSION (imax,ntra) , INTENT(IN)    :: QM0
REAL (KIND=8) , DIMENSION (imax,ntra) , INTENT(INOUT) :: QM
REAL (KIND=8) , DIMENSION (imax,nsls) , INTENT(INOUT) :: HM
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
INTEGER :: I,i1,i2
REAL (KIND=8) :: TOTCL0,TOTCL,RAPCL,TOTBR0,TOTBR,RAPBR
REAL (KIND=8) :: TOTN0,TOTN,RAPN
!
#ifdef MPI
i1 = i_deb(myid) ; i2 = i_fin(myid)
#else
i1 = 1           ; i2 = imax
#endif
!
!!!!!
!
!  TRANSPORTED LONG-LIVED SPECIES AND FAMILIES (ARRAY QM)
!
!!!!!
!
!       1 : N2O         2 : CH4         3 : H2O         4 : HNO3        5 : N2O5
!       6 : CO          7 : OClO        8 : HCl         9 : ClONO2     10 : HOCl
!      11 : Cl2        12 : H2O2       13 : ClNO2      14 : HBr        15 : BrON
!      16 : HNO4       17 : Cl2O2      18 : HOBr       19 : BrCl       20 : HCHO
!      21 : MO2        22 : OP1        23 : CFC11      24 : CFC12      25 : CFC1
!      26 : CCl4       27 : CH3CCl3    28 : CH3Cl      29 : HCFC22     30 : CH3B
!      31 : H1211      32 : H1301      33 : H2SO4      34 : PSC        35 : HONO
!      36 : DMS        37 : SO2        38 : ETH        39 : HC3        40 : HC5
!      41 : HC8        42 : ETE        43 : OLT        44 : OLI        45 : DIEN
!      46 : ISO        47 : API        48 : LIM        49 : TOL        50 : XYL
!      51 : CSL        52 : ALD        53 : KET        54 : GLY        55 : MGLY
!      56 : DCB        57 : MACR       58 : UDD        59 : HKET       60 : ONIT
!      61 : PAN        62 : TPAN       63 : OP2        64 : PAA        65 : ETHP
!      66 : HC3P       67 : HC5P       68 : HC8P       69 : ETEP       70 : OLTP
!      71 : OLIP       72 : ISOP       73 : APIP       74 : LIMP       75 : PHO
!      76 : TOLP       77 : XYLP       78 : CSLP       79 : ACO3       80 : TCO3
!      81 : KETP       82 : OLNN       83 : OLND       84 : XO2        85 : SULF
!      86 : Ox         87 : NOx        88 : ClOx       89 : BrOx
!
!!!!!
!
!  SHORT-LIVED SPECIES AT INSTANTANEOUS EQUILIBRIUM (ARRAY HM)
!
!!!!!
!
!       1 : O3P         2 : O1D         3 : O3          4 : N           5 : NO
!       6 : NO2         7 : NO3         8 : ClO         9 : Cl         10 : BrO
!      11 : Br         12 : H          13 : OH         14 : HO2        15 : CH3
!      16 : CH3O       17 : ADDT       18 : ADDX       19 : ADDC       20 : PANs
!      21 : RONO2s     22 : NOy        23 : NOz        24 : ResN       25 : COVp
!      26 : COVpB      27 : COVtot     28 : HOx        29 : Oz
!
!$omp parallel do default(none) &
!$omp shared(QM0,QM,HM,i1,i2,ntra,nsls) &
!$omp private(i,RAPBR,RAPCL,RAPN,TOTBR,TOTBR0,TOTCL,TOTCL0,TOTN,TOTN0)
!
do i = i1,i2
!
!================================================================
!  CHECK CONSERVATION FOR Cl ATOMS
!================================================================
!
! initial number of atoms :
!
  totCl0 = + QM0(i,7) + QM0(i,8) + QM0(i,9) + QM0(i,10) +              &
    2.0*QM0(i,11) + QM0(i,13) + 2.0*QM0(i,17) + QM0(i,19) +            &
    3.0*QM0(i,23) + 2.0*QM0(i,24) + 3.0*QM0(i,25) + 4.0*QM0(i,26) +    &
    3.0*QM0(i,27) + QM0(i,28) + QM0(i,29) + QM0(i,31) + QM0(i,88)
!
! final number of atoms :
!
  totCl = + QM(i,7) + QM(i,8) + QM(i,9) + QM(i,10) + 2.0*QM(i,11) +    &
    QM(i,13) + 2.0*QM(i,17) + QM(i,19) + 3.0*QM(i,23) +                &
    2.0*QM(i,24) + 3.0*QM(i,25) + 4.0*QM(i,26) + 3.0*QM(i,27) +        &
    QM(i,28) + QM(i,29) + QM(i,31) + QM(i,88)
!
! adjust to keep atom number constant :
!
  rapCl = totCl0 / totCl
!
  QM(i,7) = QM(i,7) * rapCl
  QM(i,8) = QM(i,8) * rapCl
  QM(i,9) = QM(i,9) * rapCl
  QM(i,10) = QM(i,10) * rapCl
  QM(i,11) = QM(i,11) * rapCl
  QM(i,13) = QM(i,13) * rapCl
  QM(i,17) = QM(i,17) * rapCl
  QM(i,19) = QM(i,19) * rapCl
#ifndef MASSy
  QM(i,23) = QM(i,23) * rapCl
  QM(i,24) = QM(i,24) * rapCl
  QM(i,25) = QM(i,25) * rapCl
  QM(i,26) = QM(i,26) * rapCl
  QM(i,27) = QM(i,27) * rapCl
  QM(i,28) = QM(i,28) * rapCl
  QM(i,29) = QM(i,29) * rapCl
  QM(i,31) = QM(i,31) * rapCl
#endif
  QM(i,88) = QM(i,88) * rapCl
  HM(i,8) = HM(i,8) * rapCl
  HM(i,9) = HM(i,9) * rapCl
!
!================================================================
!  CHECK CONSERVATION FOR Br ATOMS
!================================================================
!
! initial number of atoms :
!
  totBr0 = + QM0(i,14) + QM0(i,15) + QM0(i,18) + QM0(i,19) +             &
    QM0(i,30) + QM0(i,31) + QM0(i,32) + QM0(i,89)
!
! final number of atoms :
!
  totBr = + QM(i,14) + QM(i,15) + QM(i,18) + QM(i,19) + QM(i,30) +       &
    QM(i,31) + QM(i,32) + QM(i,89)
!
! adjust to keep atom number constant :
!
  rapBr = totBr0 / totBr
!
  QM(i,14) = QM(i,14) * rapBr
  QM(i,15) = QM(i,15) * rapBr
  QM(i,18) = QM(i,18) * rapBr
  QM(i,19) = QM(i,19) * rapBr
#ifndef MASSy
  QM(i,30) = QM(i,30) * rapBr
  QM(i,31) = QM(i,31) * rapBr
  QM(i,32) = QM(i,32) * rapBr
#endif
  QM(i,89) = QM(i,89) * rapBr
  HM(i,10) = HM(i,10) * rapBr
  HM(i,11) = HM(i,11) * rapBr
!
!================================================================
!  CHECK CONSERVATION FOR N ATOMS
!================================================================
!
! initial number of atoms :
!
  totN0 = + 2.0*QM0(i,1) + QM0(i,4) + 2.0*QM0(i,5) + QM0(i,9) +         &
    QM0(i,13) + QM0(i,15) + QM0(i,16) + QM0(i,35) + QM0(i,60) +         &
    QM0(i,61) + QM0(i,62) + QM0(i,82) + QM0(i,83) + QM0(i,87)
!
! final number of atoms :
!
  totN = + 2.0*QM(i,1) + QM(i,4) + 2.0*QM(i,5) + QM(i,9) +              &
    QM(i,13) + QM(i,15) + QM(i,16) + QM(i,35) + QM(i,60) +              &
    QM(i,61) + QM(i,62) + QM(i,82) + QM(i,83) + QM(i,87)
!
! adjust to keep atom number constant :
!
  rapN = totN0 / totN
!
#ifndef MASSy
  QM(i,1) = QM(i,1) * rapN
#endif
  QM(i,4) = QM(i,4) * rapN
  QM(i,5) = QM(i,5) * rapN
  QM(i,9) = QM(i,9) * rapN
  QM(i,13) = QM(i,13) * rapN
  QM(i,15) = QM(i,15) * rapN
  QM(i,16) = QM(i,16) * rapN
  QM(i,35) = QM(i,35) * rapN
  QM(i,60) = QM(i,60) * rapN
  QM(i,61) = QM(i,61) * rapN
  QM(i,62) = QM(i,62) * rapN
  QM(i,82) = QM(i,82) * rapN
  QM(i,83) = QM(i,83) * rapN
  QM(i,87) = QM(i,87) * rapN
  HM(i,4) = HM(i,4) * rapN
  HM(i,5) = HM(i,5) * rapN
  HM(i,6) = HM(i,6) * rapN
  HM(i,7) = HM(i,7) * rapN
!
!================================================================
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY
!
!          PANs = PAN + TPAN
!
!================================================================
!
  HM(i,20) =  + QM(i,61) + QM(i,62)
!
!================================================================
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY
!
!          RONO2s = ONIT
!
!================================================================
!
  HM(i,21) =  + QM(i,60)
!
!================================================================
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY
!
!          NOy = HNO3 + N2O5 + N2O5 + HNO4 + HONO + PAN + TPAN +
!                ONIT + OLNN + OLND + NO + NO2 + NO3
!
!================================================================
!
  HM(i,22) =  + QM(i,4) + QM(i,5) + QM(i,5) + QM(i,16) + QM(i,35) +       &
    QM(i,61) + QM(i,62) + QM(i,60) + QM(i,82) + QM(i,83) + HM(i,5) +      &
    HM(i,6) + HM(i,7)
!
!================================================================
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY
!
!          NOz = HNO3 + N2O5 + N2O5 + HNO4 + HONO + PAN + TPAN +
!                ONIT + OLNN + OLND + NO3
!
!================================================================
!
  HM(i,23) =  + QM(i,4) + QM(i,5) + QM(i,5) + QM(i,16) + QM(i,35) +       &
    QM(i,61) + QM(i,62) + QM(i,60) + QM(i,82) + QM(i,83) + HM(i,7)
!
!================================================================
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY
!
!          ResN = PAN + TPAN + ONIT
!
!================================================================
!
  HM(i,24) =  + QM(i,61) + QM(i,62) + QM(i,60)
!
!================================================================
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY
!
!          COVpA = HCHO + ETH + HC3 + HC5 + HC8 + ETE + OLT +
!                OLI + DIEN + TOL + XYL + CSL + ALD + KET
!
!================================================================
!
  HM(i,25) =  + QM(i,20) + QM(i,38) + QM(i,39) + QM(i,40) +          &
    QM(i,41) + QM(i,42) + QM(i,43) + QM(i,44) + QM(i,45) +           &
    QM(i,49) + QM(i,50) + QM(i,51) + QM(i,52) + QM(i,53)
!
!================================================================
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY
!
!          COVpB = ISO + API + LIM
!
!================================================================
!
  HM(i,26) =  + QM(i,46) + QM(i,47) + QM(i,48)
!
!================================================================
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY
!
!          COVtot = HCHO + ETH + HC3 + HC5 + HC8 + ETE + OLT +
!                OLI + DIEN + TOL + XYL + CSL + ALD + KET + ISO + API + LIM +
!                OP1 + GLY + MGLY + DCB + MACR + UDD + HKET + ONIT + PAN + TPAN
!                OP2 + PAA
!
!================================================================
!
  HM(i,27) =  + QM(i,20) + QM(i,38) + QM(i,39) + QM(i,40) +          &
    QM(i,41) + QM(i,42) + QM(i,43) + QM(i,44) + QM(i,45) +           &
    QM(i,49) + QM(i,50) + QM(i,51) + QM(i,52) + QM(i,53) +           &
    QM(i,46) + QM(i,47) + QM(i,48) + QM(i,22) + QM(i,54) +           &
    QM(i,55) + QM(i,56) + QM(i,57) + QM(i,58) + QM(i,59) +           &
    QM(i,60) + QM(i,61) + QM(i,62) + QM(i,63) + QM(i,64)
!
!================================================================
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY
!
!          HOx = HO2 + OH + XO2 + OLND + OLNN + KETP + TCO3 +
!                ACO3 + CSLP + XYLP + TOLP + LIMP + APIP + ISOP + OLIP + OLTP +
!                ETEP + HC8P + HC3P + HC5P + ETHP + MO2
!
!================================================================
!
  HM(i,28) =  + HM(i,14) + HM(i,13) + QM(i,84) + QM(i,83) +          &
    QM(i,82) + QM(i,81) + QM(i,80) + QM(i,79) + QM(i,78) +           &
    QM(i,77) + QM(i,76) + QM(i,74) + QM(i,73) + QM(i,72) +           &
    QM(i,71) + QM(i,70) + QM(i,69) + QM(i,68) + QM(i,66) +           &
    QM(i,67) + QM(i,65) + QM(i,21)
!
!================================================================
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY
!
!          Oz = O3 + NO2
!
!================================================================
!
  HM(i,29) =  + HM(i,3) + HM(i,6)
!
enddo
!$omp end parallel do
!
RETURN
END SUBROUTINE CH_ELEM
