SUBROUTINE CH_EQUIL(sza,k,hnm,QM,HM,partHNO3,partH2O,partHCl,partHBr,   &
                    imax,nk,ntra,nsls,iniv,NIVHOx,SZAlim,LSOL)
!
!!!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_0 
!
!!! DATE : 15/05/2005
!
!
!   CALCUL DE L'EQUILIBRE PHOTOCHIMIQUE POUR LES ESPECES A
!   COURTE DUREE DE VIE
!
!!!!!
!
!  Original : H. Teyssedre, V.-H. Peuch, 08/2000
!
!  Modified : D. Charpin, version Fortran-90, 08/2006
!
!!!!!
!
#ifdef MPI
      USE CSTES_MOD
#endif
!
      IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
      INTEGER , INTENT(IN) :: imax, nk, ntra, nsls, iniv, NIVHOx
      REAL (KIND=8) , INTENT(IN) :: SZAlim
      REAL (KIND=8) , DIMENSION (imax) , INTENT(IN) :: sza,hnm
      REAL (KIND=8) , DIMENSION (imax) , INTENT(IN) :: partHNO3,partH2O
      REAL (KIND=8) , DIMENSION (imax) , INTENT(IN) :: partHCl,partHBr
      REAL (KIND=8) , DIMENSION (imax,nk) , INTENT(IN) :: k
      REAL (KIND=8) , DIMENSION (imax,ntra) , INTENT(INOUT) :: QM
      REAL (KIND=8) , DIMENSION (imax,nsls) , INTENT(INOUT) :: HM
      LOGICAL , INTENT(IN) :: LSOL
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
      INTEGER :: I, i1, i2
!
      REAL (KIND=8) :: roxo3p,roxo1d,rox,aoxo3,aoxo3p,aoxo1d
      REAL (KIND=8) :: rnoxn,rnoxno,rnoxno3,rnox,anoxno2
      REAL (KIND=8) :: anoxn,anoxno,anoxno3,rcloxcl,rcloxclono2
      REAL (KIND=8) :: rcloxhocl,rcloxcl2o2,rcloxbrcl,rclox
      REAL (KIND=8) :: acloxclo,acloxcl,rbroxbr,rbroxbrono2
      REAL (KIND=8) :: rbroxhobr,rbroxbrcl,rbrox,abroxbro,abroxbr
      REAL (KIND=8) :: xproh,xpeoh,xprho2,xpeho2,argum
      REAL (KIND=8) :: rhoxh,rhoxho2,ahoxh,ahoxho2,ahoxoh
      REAL (KIND=8) :: xphox,xl1hox,xl2hox
      REAL (KIND=8) , DIMENSION (imax) :: gHNO3,gH2O
      REAL (KIND=8) , DIMENSION (imax) :: gHCl,gHBr
!
!!!!!
!
!  TRANSPORTED LONG-LIVED SPECIES AND FAMILIES (ARRAY QM)
!
!!!!!
!
!       1 : N2O         2 : CH4         3 : H2O         4 : HNO3        5 : N2O5
!       6 : CO          7 : OClO        8 : HCl         9 : ClONO2     10 : HOCl
!      11 : Cl2        12 : H2O2       13 : ClNO2      14 : HBr        15 : BrON
!      16 : HNO4       17 : Cl2O2      18 : HOBr       19 : BrCl       20 : HCHO
!      21 : MO2        22 : OP1        23 : CFC11      24 : CFC12      25 : CFC1
!      26 : CCl4       27 : CH3CCl3    28 : CH3Cl      29 : HCFC22     30 : CH3B
!      31 : H1211      32 : H1301      33 : H2SO4      34 : PSC        35 : HONO
!      36 : DMS        37 : SO2        38 : ETH        39 : HC3        40 : HC5
!      41 : HC8        42 : ETE        43 : OLT        44 : OLI        45 : DIEN
!      46 : ISO        47 : API        48 : LIM        49 : TOL        50 : XYL
!      51 : CSL        52 : ALD        53 : KET        54 : GLY        55 : MGLY
!      56 : DCB        57 : MACR       58 : UDD        59 : HKET       60 : ONIT
!      61 : PAN        62 : TPAN       63 : OP2        64 : PAA        65 : ETHP
!      66 : HC3P       67 : HC5P       68 : HC8P       69 : ETEP       70 : OLTP
!      71 : OLIP       72 : ISOP       73 : APIP       74 : LIMP       75 : PHO
!      76 : TOLP       77 : XYLP       78 : CSLP       79 : ACO3       80 : TCO3
!      81 : KETP       82 : OLNN       83 : OLND       84 : XO2        85 : SULF
!      86 : Ox         87 : NOx        88 : ClOx       89 : BrOx
!
!!!!!
!
!  SHORT-LIVED SPECIES AT INSTANTANEOUS EQUILIBRIUM (ARRAY HM)
!
!!!!!
!
!       1 : O3P         2 : O1D         3 : O3          4 : N           5 : NO
!       6 : NO2         7 : NO3         8 : ClO         9 : Cl         10 : BrO
!      11 : Br         12 : H          13 : OH         14 : HO2        15 : CH3
!      16 : CH3O       17 : ADDT       18 : ADDX       19 : ADDC       20 : PANs
!      21 : RONO2s     22 : NOy        23 : NOz        24 : ResN       25 : COVp
!      26 : COVpB      27 : COVtot     28 : HOx        29 : Oz
!
!!!!!
!
!  1. DAY-TIME CHEMISTRY (CHEMICAL STEADY STATE)
!
!!!!!
!
if (LSOL) then
  do i=1,imax
    gHNO3(i)=partHNO3(i)*QM(i,4)
    gH2O (i)=partH2O (i)*QM(i,3)
    gHCl (i)=partHCl (i)*QM(i,8)
    gHBr (i)=partHBr (i)*QM(i,14)
  enddo
else
  do i=1,imax
    gHNO3(i)=QM(i,4)
    gH2O (i)=QM(i,3)
    gHCl (i)=QM(i,8)
    gHBr (i)=QM(i,14)
  enddo
endif
!
#ifdef MPI
i1 = i_deb(myid) ; i2 = i_fin(myid)
#else
i1 = 1           ; i2 = imax
#endif
!
!$omp parallel default(none) &
!$omp private (i,rOxO3P,rOxO1D,rOx,aOxO3,aOxO3P,aOxO1D) &
!$omp private (rNOxN,rNOxNO,rNOxNO3,rNOx) &
!$omp private (aNOxNO2,aNOxN,aNOxNO,aNOxNO3) &
!$omp private (rClOxCl,rClOxClONO2,rClOxHOCl,rClOxCl2O2) &
!$omp private (rClOxBrCl,rClOx,aClOxClO,aClOxCl) &
!$omp private (rBrOxBr,rBrOxBrONO2,rBrOxHOBr,rBrOxBrCl,rBrOx) &
!$omp private (aBrOxBrO,aBrOxBr) &
!$omp private (rHOxH,rHOxHO2,argum) &
!$omp private (aHOxH,aHOxHO2,aHOxOH) &
!$omp private (xpHOx,xl1HOx,xl2HOx) &
!$omp private (xprOH,xpeOH,xprHO2,xpeHO2) &
!$omp shared  (i1,i2,sza,SZAlim,hnm,k,QM,HM,imax,iniv,LSOL) &
!$omp shared  (gHNO3,gH2O,gHCl,gHBr,NIVHOx)
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
    rOxO3P  = HM(i,1) / HM(i,3)
    rOxO1D  = HM(i,2) / HM(i,3)
!
!================================================================
!  FOR FAMILY Ox = O3P + O1D + O3
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k002 :  O3  -->  O1D + O2
!  k003 :  O3  -->  O3P + O2
!  k053 :  O3P + O2  -->  O3
!  k055 :  O1D + N2  -->  O3P + N2
!  k056 :  O1D + O2  -->  O3P + O2
!  k155 :  DIEN + O3  -->  0.90*HCHO + 0.39*MACR + 0.36*CO +
!                0.15*ORA1 + 0.09*O3P + 0.30*HO2 + 0.35*OLT + 0.28*OH + 0.05*H2
!                0.15*ACO3 + 0.03*MO2 + 0.02*KETP + 0.13*XO2 + 0.001*H2O2
!  k156 :  ISO + O3  -->  0.90*HCHO + 0.39*MACR + 0.36*CO +
!                0.15*ORA1 + 0.09*O3P + 0.30*HO2 + 0.35*OLT + 0.28*OH + 0.05*H2
!                0.15*ACO3 + 0.03*MO2 + 0.02*KETP + 0.13*XO2 + 0.001*H2O2
!================================================================
!
!  O3P / O3
!
    rOxO3P = ( + rOxO1D*( + k(i,55)*0.78*hnm(i) +                 &
      k(i,56)*0.21*hnm(i) ) + 0.09*( + k(i,155)*QM(i,45) +        &
      k(i,156)*QM(i,46) ) + k(i,3) ) / ( + k(i,53)*0.21*hnm(i) )
!
    rOxO3P = max( 0. , rOxO3P )
!
!  O1D / O3
!
    rOxO1D = ( + k(i,2) ) / ( + k(i,55)*0.78*hnm(i) +             &
      k(i,56)*0.21*hnm(i) )
!
    rOxO1D = max( 0. , rOxO1D )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rOx = 1. + rOxO3P + rOxO1D
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aOxO3 = 0.0
!
    HM(i,3) = aOxO3*HM(i,3) + (1.-aOxO3)*QM(i,86)/rOx
!
    aOxO3P = 0.0
!
    HM(i,1) = aOxO3P*HM(i,1) + (1.-aOxO3P)*HM(i,3)*rOxO3P
!
    aOxO1D = 0.0
!
    HM(i,2) = aOxO1D*HM(i,2) + (1.-aOxO1D)*HM(i,3)*rOxO1D
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
    rNOxN   = HM(i,4) / HM(i,6)
    rNOxNO  = HM(i,5) / HM(i,6)
    rNOxNO3 = HM(i,7) / HM(i,6)
!
!================================================================
!  FOR FAMILY NOx = N + NO + NO2 + NO3
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k001 :  NO2  -->  O3P + NO
!  k008 :  NO3  -->  NO + O2
!  k009 :  NO3  -->  NO2 + O3P
!  k052 :  NO  -->  N + O3P
!  k064 :  O3P + NO  -->  NO2
!  k065 :  O3P + NO2  -->  NO + O2
!  k066 :  O3P + NO2  -->  NO3
!  k069 :  OH + NO3  -->  NO2 + HO2
!  k070 :  HO2 + NO  -->  NO2 + OH
!  k074 :  HO2 + NO3  --> NO2 + OH
!  k078 :  O3 + NO  -->  NO2 + O2
!  k079 :  O3 + NO2  -->  NO3 + O2
!  k080 :  NO + NO + O2  -->  NO2 + NO2
!  k081 :  NO3 + NO  -->  NO2 + NO2
!  k082 :  NO3 + NO2  -->  NO + NO2 + O2
!  k083 :  NO3 + NO2  -->  N2O5  (destruction for NO3)
!  k085 :  NO3 + NO3  -->  NO2 + NO2 + O2
!  k134 :  DCB + NO3  -->  0.50*TCO3 + 0.50*HO2 + 0.50*XO2 +
!                0.25*GLY + 0.25*ALD + 0.50*NO2 + 0.03*KET + 0.25*MGLY +
!                0.50*HNO3
!  k151 :  TPAN + NO3  -->  PAN + HCHO + NO2 + XO2
!  k180 :  MO2 + NO  -->  CH3O + NO2
!  k181 :  ETHP + NO  -->  ALD + HO2 + NO2
!  k182 :  HC3P + NO  -->  0.047*HCHO + 0.233*ALD + 0.623*KET +
!                0.063*GLY + 0.742*HO2 + 0.150*MO2 + 0.048*ETHP + 0.048*XO2 +
!                0.059*ONIT + 0.941*NO2
!  k183 :  HC5P + NO  -->  0.021*HCHO + 0.211*ALD + 0.722*KET +
!                0.599*HO2 + 0.031*MO2 + 0.245*ETHP + 0.334*XO2 + 0.124*ONIT +
!                0.876*NO2
!  k184 :  HC8P + NO  -->  0.150*ALD + 0.642*KET + 0.133*ETHP +
!                0.261*ONIT + 0.739*NO2 + 0.606*HO2 + 0.416*XO2
!  k185 :  ETEP + NO  -->  1.6*HCHO + HO2 + NO2 + 0.2*ALD
!  k186 :  OLTP + NO  -->  0.94*ALD + HCHO + HO2 + NO2 + 0.06*KET
!  k187 :  OLIP + NO  -->  HO2 + 1.71*ALD + 0.29*KET + NO2
!  k188 :  ISOP + NO  -->  0.446*MACR + 0.354*OLT + 0.847*HO2 +
!                0.847*NO2 + 0.153*ONIT + 0.606*HCHO
!  k189 :  APIP + NO  -->  0.80*HO2 + 0.80*ALD + 0.80*NO2 + 0.80*KET + 0.20*ONIT
!  k190 :  LIMP + NO  -->  0.65*HO2 + 0.40*MACR + 0.25*OLI +
!                0.25*HCHO + 0.65*NO2 + 0.35*ONIT
!  k191 :  TOLP + NO  -->  0.95*NO2 + 0.95*HO2 + 1.20*GLY +
!                0.50*DCB + 0.05*ONIT + 0.65*MGLY
!  k192 :  XYLP + NO  -->  0.95*NO2 + 0.95*HO2 + 0.35*GLY +
!                0.95*DCB + 0.05*ONIT + 0.60*MGLY
!  k193 :  CSLP + NO  -->  GLY + MGLY + NO2 + HO2
!  k194 :  ACO3 + NO  -->  MO2 + NO2
!  k195 :  TCO3 + NO  -->  NO2 + ACO3 + HCHO
!  k196 :  KETP + NO  -->  0.54*MGLY + 0.46*ALD + 0.23*ACO3 +
!                0.77*HO2 + 0.16*XO2 + NO2
!  k197 :  OLNN + NO  -->  ONIT + NO2 + HO2
!  k198 :  OLND + NO  -->  0.287*HCHO + 1.24*ALD + 0.464*KET + 2.0*NO2
!  k263 :  MO2 + NO3  -->  HCHO + HO2 + NO2
!  k264 :  ETHP + NO3  -->  ALD + HO2 + NO2
!  k265 :  HC3P + NO3  -->  0.048*HCHO + 0.243*ALD + 0.670*KET +
!                0.063*GLY + 0.792*HO2 + 0.155*MO2 + 0.053*ETHP + 0.051*XO2 + NO
!  k266 :  HC5P + NO3  -->  0.021*HCHO + 0.239*ALD + 0.828*KET +
!                0.699*HO2 + 0.040*MO2 + 0.262*ETHP + 0.391*XO2 + NO2
!  k267 :  HC8P + NO3  -->  0.187*ALD + 0.880*KET + 0.155*ETHP +
!                NO2 + 0.845*HO2 + 0.587*XO2
!  k268 :  ETEP + NO3  -->  1.6*HCHO + HO2 + NO2 + 0.2*ALD
!  k269 :  OLTP + NO3  -->  0.94*ALD + HCHO + HO2 + NO2 + 0.06*KET
!  k270 :  OLIP + NO3  -->  HO2 + 1.71*ALD + 0.29*KET + NO2
!  k271 :  ISOP + NO3  -->  0.600*MACR + 0.400*OLT + HO2 + NO2 + 0.686*HCHO
!  k272 :  APIP + NO3  -->  HO2 + ALD + NO2 + KET
!  k273 :  LIMP + NO3  -->  HO2 + 0.60*MACR + 0.40*OLI + 0.40*HCHO + NO2
!  k274 :  TOLP + NO3  -->  NO2 + HO2 + 0.50*DCB + 1.30*GLY + 0.70*MGLY
!  k275 :  XYLP + NO3  -->  NO2 + HO2 + DCB + 0.74*GLY + 1.26*MGLY
!  k276 :  CSLP + NO3  -->  GLY + MGLY + NO2 + HO2
!  k277 :  ACO3 + NO3  -->  MO2 + NO2
!  k278 :  TCO3 + NO3  -->  NO2 + ACO3 + HCHO
!  k279 :  KETP + NO3  -->  0.54*MGLY + 0.46*ALD + 0.23*ACO3 +
!                0.77*HO2 + 0.16*XO2 + NO2
!  k280 :  OLNN + NO3  -->  ONIT + NO2 + HO2
!  k281 :  OLND + NO3  -->  0.280*HCHO + 1.24*ALD + 0.469*KET + 2.0*NO2
!  k286 :  XO2 + NO  -->  NO2
!  k287 :  XO2 + NO3  -->  NO2
!  k288 :  DMS + NO3  -->  SO2 + NO2
!  k302 :  N + NO  -->  N2 + O3P  (destruction for N)
!  k303 :  N + O2  -->  NO + O3P
!  k318 :  ClO + NO  -->  NO2 + Cl
!  k347 :  BrO + NO  -->  NO2 + Br
!================================================================
!
!  N / NO2
!
    rNOxN = ( + k(i,52)*rNOxNO ) / ( + k(i,302)*HM(i,5) +            &
      k(i,303)*0.21*hnm(i) )
!
    rNOxN = max( 0. , rNOxN )
!
!  NO3 / NO2
!
    rNOxNO3 = ( + k(i,66)*HM(i,1) + k(i,79)*HM(i,3) ) / ( +          &
      HM(i,6)*( + k(i,82) + k(i,83) ) + k(i,8) + k(i,9) +            &
      k(i,69)*HM(i,13) + k(i,74)*HM(i,14) + k(i,81)*HM(i,5) +        &
      2.0*k(i,85)*HM(i,7) + 0.50*k(i,134)*QM(i,56) +                 &
      k(i,151)*QM(i,62) + k(i,263)*QM(i,21) + k(i,264)*QM(i,65) +    &
      k(i,265)*QM(i,66) + k(i,266)*QM(i,67) + k(i,267)*QM(i,68) +    &
      k(i,268)*QM(i,69) + k(i,269)*QM(i,70) + k(i,270)*QM(i,71) +    &
      k(i,271)*QM(i,72) + k(i,272)*QM(i,73) + k(i,273)*QM(i,74) +    &
      k(i,274)*QM(i,76) + k(i,275)*QM(i,77) + k(i,276)*QM(i,78) +    &
      k(i,277)*QM(i,79) + k(i,278)*QM(i,80) + k(i,279)*QM(i,81) +    &
      k(i,280)*QM(i,82) + k(i,281)*QM(i,83) + k(i,287)*QM(i,84) +    &
      k(i,288)*QM(i,36) )
!
    rNOxNO3 = max( 0. , rNOxNO3 )
!
!  NO / NO2
!
    rNOxNO = ( + rNOxNO3*( + k(i,8) + k(i,82)*HM(i,6) ) + k(i,1) +     &
      k(i,65)*HM(i,1) + k(i,303)*rNOxN*0.21*hnm(i) ) / ( + 0.95*( +    &
      k(i,191)*QM(i,76) + k(i,192)*QM(i,77) ) + k(i,52) +              &
      k(i,64)*HM(i,1) + k(i,70)*HM(i,14) + k(i,78)*HM(i,3) +           &
      2.0*k(i,80)*0.21*hnm(i)*HM(i,5) + k(i,81)*HM(i,7) +              &
      k(i,180)*QM(i,21) + k(i,181)*QM(i,65) +                          &
      0.941*k(i,182)*QM(i,66) + 0.876*k(i,183)*QM(i,67) +              &
      0.739*k(i,184)*QM(i,68) + k(i,185)*QM(i,69) +                    &
      k(i,186)*QM(i,70) + k(i,187)*QM(i,71) +                          &
      0.847*k(i,188)*QM(i,72) + 0.80*k(i,189)*QM(i,73) +               &
      0.65*k(i,190)*QM(i,74) + k(i,193)*QM(i,78) + k(i,194)*QM(i,79) + &
      k(i,195)*QM(i,80) + k(i,196)*QM(i,81) + k(i,197)*QM(i,82) +      &
      k(i,198)*QM(i,83) + k(i,286)*QM(i,84) + k(i,318)*HM(i,8) +       &
      k(i,347)*HM(i,10) )
!
    rNOxNO = max( 0. , rNOxNO )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rNOx = 1. + rNOxN + rNOxNO + rNOxNO3
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aNOxNO2 = 0.0
!
    HM(i,6) = aNOxNO2*HM(i,6) + (1.-aNOxNO2)*QM(i,87)/rNOx
!
    aNOxN = 0.0
!
    HM(i,4) = aNOxN*HM(i,4) + (1.-aNOxN)*HM(i,6)*rNOxN
!
    aNOxNO = 0.0
!
    HM(i,5) = aNOxNO*HM(i,5) + (1.-aNOxNO)*HM(i,6)*rNOxNO
!
    aNOxNO3 = 0.0
!
    HM(i,7) = aNOxNO3*HM(i,7) + (1.-aNOxNO3)*HM(i,6)*rNOxNO3
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
    rClOxCl = HM(i,9) / HM(i,8)
!
!================================================================
!  FOR FAMILY ClOx = ClO + Cl
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k030 :  HOCl  -->  Cl + OH
!  k031 :  ClONO2  -->  Cl + NO3
!  k032 :  ClONO2  -->  ClO + NO2
!  k033 :  Cl2O2  -->  Cl + Cl + O2 
!  k041 :  BrCl  -->  Br + Cl
!  k314 :  MO2 + ClO  -->  CH3O + Cl + O2
!  k316 :  Cl + O3  -->  ClO + O2
!  k317 :  ClO + O3P  -->  Cl + O2
!  k318 :  ClO + NO  -->  NO2 + Cl
!  k319 :  Cl + CH4  -->  HCl + CH3  (destruction for Cl)
!  k321 :  Cl + HO2  -->  HCl + O2  (destruction for Cl)
!  k322 :  Cl + HO2  -->  OH + ClO  (destruction for Cl)
!  k323 :  ClO + OH  -->  Cl + HO2
!  k326 :  ClO + NO2  -->  ClONO2
!  k328 :  ClO + HO2  -->  HOCl + O2
!  k329 :  OH + HOCl  -->  H2O + ClO
!  k330 :  O3P + HOCl  -->  OH + ClO
!  k332 :  Cl + HOCl  -->  OH + Cl2  (destruction for Cl)
!  k333 :  ClO + ClO  -->  Cl2O2
!  k334 :  Cl2O2  -->  ClO + ClO
!  k349 :  BrO + ClO  -->  Br + Cl + O2
!  k350 :  BrO + ClO  -->  BrCl + O2
!  k364 :  ClONO2 + H2O  -->  HOCl + HNO3  (production for HOCl)
!  k365 :  ClONO2 + HCl  -->  Cl2 + HNO3  (destruction for ClONO2)
!  k368 :  HOCl + HCl  -->  Cl2 + H2O  (destruction for HOCl)
!================================================================
!
!  ClONO2 / ClO
!
    rClOxClONO2 = ( + k(i,326)*HM(i,6) ) / ( + k(i,31) + k(i,32) +      &
      k(i,364)*QM(i,3) + k(i,365)*QM(i,8) )
!
!  HOCl / ClO
!
    rClOxHOCl = ( + k(i,328)*HM(i,14) +                                 &
      k(i,364)*rClOxClONO2*QM(i,3) ) / ( + k(i,30) +                    &
      k(i,329)*HM(i,13) + k(i,330)*HM(i,1) + k(i,368)*QM(i,8) )
!
!  Cl2O2 / ClO
!
    rClOxCl2O2 = ( + k(i,333)*HM(i,8) ) / ( + k(i,33) + k(i,334) )
!
!  BrCl / ClO
!
    rClOxBrCl = ( + k(i,350)*HM(i,10) ) / ( + k(i,41) )
!
!  Cl / ClO
!
    rClOxCl = ( + k(i,30)*rClOxHOCl + k(i,31)*rClOxClONO2 +            &
      2.0*k(i,33)*rClOxCl2O2 + k(i,41)*rClOxBrCl + k(i,314)*QM(i,21) + &
      k(i,317)*HM(i,1) + k(i,318)*HM(i,5) + k(i,323)*HM(i,13) +        &
      k(i,349)*HM(i,10) ) / ( + HM(i,14)*( + k(i,321) +                &
      2.0*k(i,322) ) + k(i,316)*HM(i,3) + k(i,319)*QM(i,2) +           &
      k(i,332)*QM(i,10) )
!
    rClOxCl = max( 0. , rClOxCl )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rClOx = 1. + rClOxCl
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aClOxClO = 0.5
!
    HM(i,8) = aClOxClO*HM(i,8) + (1.-aClOxClO)*QM(i,88)/rClOx
!
    aClOxCl = 0.0
!
    HM(i,9) = aClOxCl*HM(i,9) + (1.-aClOxCl)*HM(i,8)*rClOxCl
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <=  SZAlim) then
!
    rBrOxBr = HM(i,11) / HM(i,10)
!
!================================================================
!  FOR FAMILY BrOx = BrO + Br
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k037 :  BrO  -->  Br + O3P
!  k038 :  BrONO2  -->  Br + NO3
!  k039 :  HOBr  -->  Br + OH
!  k041 :  BrCl  -->  Br + Cl
!  k315 :  HCHO + Br  -->  HO2 + CO + HBr  (destruction for Br)
!  k345 :  Br + O3  -->  BrO + O2
!  k346 :  BrO + O3P  -->  Br + O2
!  k347 :  BrO + NO  -->  NO2 + Br
!  k348 :  BrO + ClO  -->  OClO + Br
!  k349 :  BrO + ClO  -->  Br + Cl + O2
!  k350 :  BrO + ClO  -->  BrCl + O2
!  k351 :  BrO + BrO  -->  Br + Br + O2
!  k352 :  Br + HO2  -->  HBr + O2  (destruction for Br)
!  k354 :  BrO + OH  -->  Br + HO2
!  k356 :  BrO + NO2  -->  BrONO2
!  k357 :  BrO + HO2  -->  HOBr + O2
!  k358 :  HOBr + O3P  -->  OH + BrO
!================================================================
!
!  BrONO2 / BrO
!
    rBrOxBrONO2 = ( + k(i,356)*HM(i,6) ) / ( + k(i,38) )
!
!  HOBr / BrO
!
    rBrOxHOBr = ( + k(i,357)*HM(i,14) ) / ( + k(i,39) + k(i,358)*HM(i,1) )
!
!  BrCl / BrO
!
    rBrOxBrCl = ( + k(i,350)*HM(i,8) ) / ( + k(i,41) )
!
!  Br / BrO
!
    rBrOxBr = ( + HM(i,8)*( + k(i,348) + k(i,349) ) + k(i,37) +          &
      k(i,38)*rBrOxBrONO2 + k(i,39)*rBrOxHOBr + k(i,41)*rBrOxBrCl +      &
      k(i,346)*HM(i,1) + k(i,347)*HM(i,5) + 2.0*k(i,351)*HM(i,10) +      &
      k(i,354)*HM(i,13) ) / ( + k(i,315)*QM(i,20) + k(i,345)*HM(i,3) +   &
      k(i,352)*HM(i,14) )
!
    rBrOxBr = max( 0. , rBrOxBr )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rBrOx = 1. + rBrOxBr
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aBrOxBrO = 0.5
!
    HM(i,10) = aBrOxBrO*HM(i,10) + (1.-aBrOxBrO)*QM(i,89)/rBrOx
!
    aBrOxBr = 0.0
!
    HM(i,11) = aBrOxBr*HM(i,11) + (1.-aBrOxBr)*HM(i,10)*rBrOxBr
!
  endif
!
enddo
!$omp end do
!
if (iniv <= NIVHOx) then
!$omp do
!
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
      if (HM(i,13) < 1.0e-10) then
        rHOxH   = 0.0
        rHOxHO2 = 0.0
      else
        rHOxH   = HM(i,12) / HM(i,13)
        rHOxHO2 = HM(i,14) / HM(i,13)
      endif
!
!================================================================
!  FOR FAMILY HOx = H + OH + HO2
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k058 :  O3 + OH  -->  HO2 + O2
!  k059 :  O3 + HO2  -->  OH + O2 + O2
!  k061 :  H2O2 + OH  -->  HO2 + H2O
!  k069 :  OH + NO3  -->  NO2 + HO2
!  k070 :  HO2 + NO  -->  NO2 + OH
!  k074 :  HO2 + NO3  --> NO2 + OH
!  k086 :  OH + H2  -->  H2O + H
!  k087 :  OH + SO2  -->  H2SO4 + HO2
!  k088 :  CO + OH  -->  H + CO2
!  k095 :  HC5 + OH  --> KET + HO2 + H2O
!  k096 :  HC8 + OH  -->  0.951*HC8P + 0.025*ALD + 0.024*HKET + 0.049*HO2 + H2O
!  k105 :  TOL + OH  --> XO2 + HO2
!  k107 :  XYL + OH  --> XO2 + HO2
!  k110 :  CSL + OH  --> HO2 + XO2
!  k111 :  HCHO + OH  -->  HO2 + CO + H2O
!  k114 :  HKET + OH  -->  HO2 + MGLY + H2O
!  k115 :  GLY + OH  -->  HO2 + CO + CO + H2O
!  k117 :  MACR + OH  -->  0.51*TCO3 + 0.41*HKET + 0.08*MGLY +
!                0.41*CO + 0.08*HCHO + 0.49*HO2 + 0.49*XO2
!  k118 :  OH + DCB  -->  0.50*TCO3 + 0.50*HO2 + 0.50*XO2 +
!                0.35*UDD + 0.15*GLY + 0.15*MGLY
!  k119 :  OH + UDD  --> ALD + HO2
!  k120 :  OH + UDD  --> KET + HO2
!  k125 :  PAA + OH  --> HO2 + HCHO + XO2
!  k127 :  TPAN + OH  -->  0.60*HKET + 0.60*NO3 + 0.40*PAN +
!                0.40*HCHO + 0.40*HO2 + XO2
!  k291 :  H + O2  -->  HO2
!  k292 :  H + O3  -->  OH + O2
!  k294 :  O3P + OH  -->  O2 + H
!  k295 :  O3P + HO2  -->  OH + O2
!  k296 :  H + HO2  -->  OH + OH
!  k322 :  Cl + HO2  -->  OH + ClO
!  k323 :  ClO + OH  -->  Cl + HO2
!  k354 :  BrO + OH  -->  Br + HO2
!================================================================
!
!  H / OH
!
      rHOxH = ( + k(i,86)*5.e-7*hnm(i) + k(i,88)*QM(i,6) +              &
        k(i,294)*HM(i,1) ) / ( + k(i,291)*0.21*hnm(i) +                 &
        k(i,292)*HM(i,3) + k(i,296)*HM(i,14) )
!
      aHOxH = 0.0
!
      HM(i,12) = aHOxH*HM(i,12) + (1.-aHOxH)*HM(i,13)*rHOxH
!
!  HO2 / OH
!
      rHOxHO2 = ( + QM(i,58)*( + k(i,119) + k(i,120) ) +                &
        k(i,58)*HM(i,3) + k(i,61)*QM(i,12) + k(i,69)*HM(i,7) +          &
        k(i,87)*QM(i,37) + k(i,95)*QM(i,40) + 0.049*k(i,96)*QM(i,41) +  &
        k(i,105)*QM(i,49) + k(i,107)*QM(i,50) + k(i,110)*QM(i,51) +     &
        k(i,111)*QM(i,20) + k(i,114)*QM(i,59) + k(i,115)*QM(i,54) +     &
        0.49*k(i,117)*QM(i,57) + 0.50*k(i,118)*QM(i,56) +               &
        k(i,125)*QM(i,64) + 0.40*k(i,127)*QM(i,62) +                    &
        k(i,291)*rHOxH*0.21*hnm(i) + k(i,323)*HM(i,8) +                 &
        k(i,354)*HM(i,10) ) / ( + k(i,59)*HM(i,3) + k(i,70)*HM(i,5) +   &
        k(i,74)*HM(i,7) + k(i,295)*HM(i,1) + k(i,296)*HM(i,12) +        &
        k(i,322)*HM(i,9) )
!
      aHOxHO2 = 0.5
!
      HM(i,14) = aHOxHO2*HM(i,14) + (1.-aHOxHO2)*HM(i,13)*rHOxHO2
!
!  QUADRATIC LOSS FOR FAMILY HOx = H + OH + HO2
!
!  loss(HOx)**2 = + 2.0*k297*[H]*[HO2] + 2.0*k298*[H]*[HO2] +
!                2.0*k060*[OH]*[HO2] + 2.0*k300*[OH]*[OH] + 2.0*k301*[OH]*[OH] +
!                2.0*k062*[HO2]*[HO2] + 2.0*k063*[HO2]*[HO2]*[H2O]
!
      xl2HOx = + 2.0*( + k(i,297)*rHOxH*rHOxHO2 +                              &
        k(i,298)*rHOxH*rHOxHO2 + k(i,60)*rHOxHO2 + k(i,300) + k(i,301) +       &
        k(i,62)*rHOxHO2*rHOxHO2 + k(i,63)*rHOxHO2*rHOxHO2*gH2O(i) )
!
!  LINEAR LOSS FOR FAMILY HOx = H + OH + HO2
!
!  loss(HOx) = + k067*[OH]*[NO] + k068*[OH]*[NO2] +
!                k075*[OH]*[HONO] + k076*[OH]*[HNO3] + k077*[OH]*[HNO4] +
!                k091*[CH4]*[OH] + k092*[ETH]*[OH] + k094*[HC5]*[OH] +
!                k097*[ETE]*[OH] + k098*[OLT]*[OH] + k099*[OLI]*[OH] +
!                k100*[DIEN]*[OH] + k101*[ISO]*[OH] + k102*[API]*[OH] +
!                k103*[LIM]*[OH] + k104*[TOL]*[OH] + k106*[XYL]*[OH] +
!                k108*[CSL]*[OH] + k109*[CSL]*[OH] + k112*[ALD]*[OH] +
!                k113*[KET]*[OH] + k116*[MGLY]*[OH] + k121*[OP1]*[OH] +
!                k124*[PAA]*[OH] + k126*[PAN]*[OH] + k128*[ONIT]*[OH] +
!                k290*[DMS]*[OH] + k324*[ClO]*[OH] + k325*[OH]*[HCl] +
!                k329*[OH]*[HOCl] + k341*[OH]*[CH3CCl3] + k342*[OH]*[CH3Cl] +
!                k344*[OH]*[HCFC22] + k355*[OH]*[HBr] + k360*[OH]*[CH3Br] +
!                k071*[HO2]*[NO2] + k073*[HO2]*[NO3] + 0.951*k096*[HC8]*[OH] +
!                0.510*k117*[MACR]*[OH] + 0.500*k118*[OH]*[DCB] +
!                0.600*k127*[TPAN]*[OH] + k163*[PHO]*[HO2] + k199*[MO2]*[HO2] +
!                k200*[ETHP]*[HO2] + k201*[HC3P]*[HO2] + k202*[HC5P]*[HO2] +
!                k203*[HC8P]*[HO2] + k204*[ETEP]*[HO2] + k205*[OLTP]*[HO2] +
!                k206*[OLIP]*[HO2] + k207*[ISOP]*[HO2] + k208*[APIP]*[HO2] +
!                k209*[LIMP]*[HO2] + k210*[TOLP]*[HO2] + k211*[XYLP]*[HO2] +
!                k212*[CSLP]*[HO2] + k213*[ACO3]*[HO2] + k214*[ACO3]*[HO2] +
!                k215*[TCO3]*[HO2] + k216*[TCO3]*[HO2] + k217*[KETP]*[HO2] +
!                k218*[OLNN]*[HO2] + k219*[OLND]*[HO2] + k282*[XO2]*[HO2] +
!                k321*[Cl]*[HO2] + k328*[ClO]*[HO2] + k352*[Br]*[HO2] +
!                k357*[BrO]*[HO2]
!
      xl1HOx = + rHOxHO2*( + k(i,71)*HM(i,6) + k(i,73)*HM(i,7) +             &
        k(i,163)*QM(i,75) + k(i,199)*QM(i,21) + k(i,200)*QM(i,65) +          &
        k(i,201)*QM(i,66) + k(i,202)*QM(i,67) + k(i,203)*QM(i,68) +          &
        k(i,204)*QM(i,69) + k(i,205)*QM(i,70) + k(i,206)*QM(i,71) +          &
        k(i,207)*QM(i,72) + k(i,208)*QM(i,73) + k(i,209)*QM(i,74) +          &
        k(i,210)*QM(i,76) + k(i,211)*QM(i,77) + k(i,212)*QM(i,78) +          &
        k(i,213)*QM(i,79) + k(i,214)*QM(i,79) + k(i,215)*QM(i,80) +          &
        k(i,216)*QM(i,80) + k(i,217)*QM(i,81) + k(i,218)*QM(i,82) +          &
        k(i,219)*QM(i,83) + k(i,282)*QM(i,84) + k(i,321)*HM(i,9) +           &
        k(i,328)*HM(i,8) + k(i,352)*HM(i,11) + k(i,357)*HM(i,10) ) +         &
        HM(i,13)*( + 0.951*k(i,96)*QM(i,41) + 0.510*k(i,117)*QM(i,57) +      &
        0.500*k(i,118)*QM(i,56) + 0.600*k(i,127)*QM(i,62) ) +                &
        QM(i,51)*( + k(i,108) + k(i,109) ) + k(i,67)*HM(i,5) +               &
        k(i,68)*HM(i,6) + k(i,75)*QM(i,35) + k(i,76)*gHNO3(i) +              &
        k(i,77)*QM(i,16) + k(i,91)*QM(i,2) + k(i,92)*QM(i,38) +              &
        k(i,94)*QM(i,40) + k(i,97)*QM(i,42) + k(i,98)*QM(i,43) +             &
        k(i,99)*QM(i,44) + k(i,100)*QM(i,45) + k(i,101)*QM(i,46) +           &
        k(i,102)*QM(i,47) + k(i,103)*QM(i,48) + k(i,104)*QM(i,49) +          &
        k(i,106)*QM(i,50) + k(i,112)*QM(i,52) + k(i,113)*QM(i,53) +          &
        k(i,116)*QM(i,55) + k(i,121)*QM(i,22) + k(i,124)*QM(i,64) +          &
        k(i,126)*QM(i,61) + k(i,128)*QM(i,60) + k(i,290)*QM(i,36) +          &
        k(i,324)*HM(i,8) + k(i,325)*gHCl(i) +                                &
        k(i,329)*QM(i,10) + k(i,341)*QM(i,27) + k(i,342)*QM(i,28) +          &
        k(i,344)*QM(i,29) + k(i,355)*gHBr(i) + k(i,360)*QM(i,30)
!
!  PRODUCTION FOR FAMILY HOx = H + OH + HO2
!
!  prod(HOx) = + k012*[HCHO] + 2.0*k028*[H2O] + k034*[HCl] +
!                2.0*k293*[O1D]*[H2] + 2.0*k299*[H2]*[O3P] + k307*[O1D]*[CH4] +
!                k309*[CH3]*[O3P] + k320*[Cl]*[H2] + k004*[HONO] + k005*[HNO3] +
!                k007*[HNO4] + 2.0*k010*[H2O2] + k014*[OP1] + 2.0*k015*[OP2] +
!                k016*[PAA] + k030*[HOCl] + k039*[HOBr] + 2.0*k057*[O1D]*[H2O] +
!                0.02*k089*[ISO]*[O3P] + 0.12*k152*[ETE]*[O3] +
!                0.40*k153*[OLT]*[O3] + 0.63*k154*[OLI]*[O3] +
!                0.28*k155*[DIEN]*[O3] + 0.28*k156*[ISO]*[O3] +
!                0.85*k157*[API]*[O3] + 0.85*k158*[LIM]*[O3] +
!                0.07*k159*[MACR]*[O3] + 0.21*k160*[DCB]*[O3] +
!                0.036*k161*[TPAN]*[O3] + k167*[ADDT]*[O3] + k171*[ADDX]*[O3] +
!                k175*[ADDC]*[O3] + k306*[O1D]*[CH4] + k311*[HCHO]*[O3P] +
!                k330*[O3P]*[HOCl] + k332*[Cl]*[HOCl] + k353*[O1D]*[HBr] +
!                k358*[HOBr]*[O3P] + k006*[HNO4] + k013*[ALD] + 0.80*k019*[GLY] +
!                k020*[MGLY] + k021*[DCB] + k022*[ONIT] + k023*[ONIT] +
!                k024*[MACR] + k025*[HKET] + k072*[HNO4] +
!                0.28*k089*[ISO]*[O3P] + 0.381*k093*[HC3]*[OH] +
!                k129*[HCHO]*[NO3] + k131*[GLY]*[NO3] + 0.50*k134*[DCB]*[NO3] +
!                0.26*k152*[ETE]*[O3] + 0.25*k153*[OLT]*[O3] +
!                0.22*k154*[OLI]*[O3] + 0.30*k155*[DIEN]*[O3] +
!                0.30*k156*[ISO]*[O3] + 0.10*k157*[API]*[O3] +
!                0.10*k158*[LIM]*[O3] + 0.29*k159*[MACR]*[O3] +
!                0.29*k160*[DCB]*[O3] + 0.08*k161*[TPAN]*[O3] +
!                k166*[ADDT]*[O2] + k170*[ADDX]*[O2] + k174*[ADDC]*[O2] +
!                k181*[ETHP]*[NO] + 0.742*k182*[HC3P]*[NO] +
!                0.599*k183*[HC5P]*[NO] + 0.606*k184*[HC8P]*[NO] +
!                k185*[ETEP]*[NO] + k186*[OLTP]*[NO] + k187*[OLIP]*[NO] +
!                0.847*k188*[ISOP]*[NO] +
!                0.80*k189*[APIP]*[NO] + 0.65*k190*[LIMP]*[NO] +
!                0.95*k191*[TOLP]*[NO] + 0.95*k192*[XYLP]*[NO] +
!                k193*[CSLP]*[NO] + 0.77*k196*[KETP]*[NO] + k197*[OLNN]*[NO] +
!                k222*[ETHP]*[MO2] + 0.992*k223*[HC3P]*[MO2] +
!                0.946*k224*[HC5P]*[MO2] + 0.993*k225*[HC8P]*[MO2] +
!                k226*[ETEP]*[MO2] + k227*[OLTP]*[MO2] + k228*[OLIP]*[MO2] +
!                k229*[ISOP]*[MO2] + 2.0*k230*[APIP]*[MO2] +
!                2.00*k231*[LIMP]*[MO2] + k232*[TOLP]*[MO2] + k233*[XYLP]*[MO2] +
!                2.0*k234*[CSLP]*[MO2] + k235*[ACO3]*[MO2] + k237*[TCO3]*[MO2] +
!                0.88*k239*[KETP]*[MO2] + k240*[OLNN]*[MO2] +
!                0.500*k241*[OLND]*[MO2] + 0.500*k242*[ETHP]*[ACO3] +
!                0.488*k243*[HC3P]*[ACO3] + 0.438*k244*[HC5P]*[ACO3] +
!                0.489*k245*[HC8P]*[ACO3] + 0.5*k246*[ETEP]*[ACO3] +
!                0.501*k247*[OLTP]*[ACO3] + 0.510*k248*[OLIP]*[ACO3] +
!                0.506*k249*[ISOP]*[ACO3] + k250*[APIP]*[ACO3] +
!                k251*[LIMP]*[ACO3] + k252*[TOLP]*[ACO3] + k253*[XYLP]*[ACO3] +
!                k254*[CSLP]*[ACO3] + 0.38*k257*[KETP]*[ACO3] +
!                0.50*k258*[OLNN]*[ACO3] + k260*[OLNN]*[OLNN] +
!                0.500*k261*[OLNN]*[OLND] + k263*[MO2]*[NO3] +
!                k264*[ETHP]*[NO3] + 0.792*k265*[HC3P]*[NO3] +
!                0.699*k266*[HC5P]*[NO3] + 0.845*k267*[HC8P]*[NO3] +
!                k268*[ETEP]*[NO3] + k269*[OLTP]*[NO3] + k270*[OLIP]*[NO3] +
!                k271*[ISOP]*[NO3] + k272*[APIP]*[NO3] + k273*[LIMP]*[NO3] +
!                k274*[TOLP]*[NO3] + k275*[XYLP]*[NO3] + k276*[CSLP]*[NO3] +
!                0.77*k279*[KETP]*[NO3] + k280*[OLNN]*[NO3] + k283*[XO2]*[MO2] +
!                k313*[CH3O]*[O2]
!
      xpHOx = + HM(i,3)*( + 0.12*k(i,152)*QM(i,42) +                      &
        0.40*k(i,153)*QM(i,43) + 0.63*k(i,154)*QM(i,44) +                 &
        0.28*k(i,155)*QM(i,45) + 0.28*k(i,156)*QM(i,46) +                 &
        0.85*k(i,157)*QM(i,47) + 0.85*k(i,158)*QM(i,48) +                 &
        0.07*k(i,159)*QM(i,57) + 0.21*k(i,160)*QM(i,56) +                 &
        0.036*k(i,161)*QM(i,62) + k(i,167)*HM(i,17) +                     &
        k(i,171)*HM(i,18) + k(i,175)*HM(i,19) + 0.26*k(i,152)*QM(i,42) +  &
        0.25*k(i,153)*QM(i,43) + 0.22*k(i,154)*QM(i,44) +                 &
        0.30*k(i,155)*QM(i,45) + 0.30*k(i,156)*QM(i,46) +                 &
        0.10*k(i,157)*QM(i,47) + 0.10*k(i,158)*QM(i,48) +                 &
        0.29*k(i,159)*QM(i,57) + 0.29*k(i,160)*QM(i,56) +                 &
        0.08*k(i,161)*QM(i,62) ) + HM(i,5)*( + k(i,181)*QM(i,65) +        &
        0.742*k(i,182)*QM(i,66) + 0.599*k(i,183)*QM(i,67) +               &
        0.606*k(i,184)*QM(i,68) + k(i,185)*QM(i,69) +                     &
        k(i,186)*QM(i,70) + k(i,187)*QM(i,71) +                           &
        0.847*k(i,188)*QM(i,72) ) + HM(i,1)*( +                           &
        2.0*k(i,299)*5.e-7*hnm(i) + k(i,309)*HM(i,15) +                   &
        0.02*k(i,89)*QM(i,46) + k(i,311)*QM(i,20) + k(i,330)*QM(i,10) +   &
        k(i,358)*QM(i,18) + 0.28*k(i,89)*QM(i,46) ) + 2.0*( +             &
        k(i,28)*gH2O(i) + k(i,293)*HM(i,2)*5.e-7*hnm(i) +                 &
        k(i,10)*QM(i,12) + k(i,15)*QM(i,63) + k(i,57)*HM(i,2)*gH2O(i)) +  &
        HM(i,2)*( + k(i,307)*QM(i,2) + k(i,306)*QM(i,2) +                 &
        k(i,353)*gHBr(i) ) + QM(i,16)*( + k(i,7) +                        &
        k(i,6) + k(i,72) ) + HM(i,7)*( + k(i,129)*QM(i,20) +              &
        k(i,131)*QM(i,54) + 0.50*k(i,134)*QM(i,56) ) + 0.21*hnm(i)*( +    &
        k(i,166)*HM(i,17) + k(i,170)*HM(i,18) + k(i,174)*HM(i,19) ) +     &
        HM(i,9)*( + k(i,320)*5.e-7*hnm(i) + k(i,332)*QM(i,10) ) +         &
        QM(i,60)*( + k(i,22) + k(i,23) ) + k(i,12)*QM(i,20) +             &
        k(i,34)*gHCl(i) + k(i,4)*QM(i,35) + k(i,5)*gHNO3(i) +             &
        k(i,14)*QM(i,22) + k(i,16)*QM(i,64) + k(i,30)*QM(i,10) +          &
        k(i,39)*QM(i,18) + k(i,13)*QM(i,52) + 0.80*k(i,19)*QM(i,54) +     &
        k(i,20)*QM(i,55) + k(i,21)*QM(i,56) + k(i,24)*QM(i,57) +          &
        k(i,25)*QM(i,59) + 0.381*k(i,93)*QM(i,39)*HM(i,13) +              &
        QM(i,21)*( + k(i,222)*QM(i,65) + 0.992*k(i,223)*QM(i,66) +        &
        0.946*k(i,224)*QM(i,67) + 0.993*k(i,225)*QM(i,68) +               &
        k(i,226)*QM(i,69) + k(i,227)*QM(i,70) + k(i,228)*QM(i,71) +       &
        k(i,229)*QM(i,72) + 2.0*k(i,230)*QM(i,73) +                       &
        2.00*k(i,231)*QM(i,74) + k(i,232)*QM(i,76) + k(i,233)*QM(i,77) +  &
        2.0*k(i,234)*QM(i,78) + k(i,235)*QM(i,79) + k(i,237)*QM(i,80) +   &
        0.88*k(i,239)*QM(i,81) + k(i,240)*QM(i,82) +                      &
        0.500*k(i,241)*QM(i,83) + k(i,263)*HM(i,7) +                      &
        k(i,283)*QM(i,84) ) + QM(i,79)*( + 0.500*k(i,242)*QM(i,65) +      &
        0.488*k(i,243)*QM(i,66) + 0.438*k(i,244)*QM(i,67) +               &
        0.489*k(i,245)*QM(i,68) + 0.5*k(i,246)*QM(i,69) +                 &
        0.501*k(i,247)*QM(i,70) + 0.510*k(i,248)*QM(i,71) +               &
        0.506*k(i,249)*QM(i,72) + k(i,250)*QM(i,73) +                     &
        k(i,251)*QM(i,74) + k(i,252)*QM(i,76) + k(i,253)*QM(i,77) +       &
        k(i,254)*QM(i,78) + 0.38*k(i,257)*QM(i,81) +                      &
        0.50*k(i,258)*QM(i,82) ) + HM(i,7)*( + k(i,264)*QM(i,65) +        &
        0.792*k(i,265)*QM(i,66) + 0.699*k(i,266)*QM(i,67) +               &
        0.845*k(i,267)*QM(i,68) + k(i,268)*QM(i,69) +                     &
        k(i,269)*QM(i,70) + k(i,270)*QM(i,71) + k(i,271)*QM(i,72) +       &
        k(i,272)*QM(i,73) + k(i,273)*QM(i,74) + k(i,274)*QM(i,76) +       &
        k(i,275)*QM(i,77) + k(i,276)*QM(i,78) + 0.77*k(i,279)*QM(i,81) +  &
        k(i,280)*QM(i,82) ) + HM(i,5)*( + 0.80*k(i,189)*QM(i,73) +        &
        0.65*k(i,190)*QM(i,74) + 0.95*k(i,191)*QM(i,76) +                 &
        0.95*k(i,192)*QM(i,77) + k(i,193)*QM(i,78) +                      &
        0.77*k(i,196)*QM(i,81) + k(i,197)*QM(i,82) ) + QM(i,82)*( +       &
        k(i,260)*QM(i,82) + 0.500*k(i,261)*QM(i,83) ) +                   &
        k(i,313)*HM(i,16)*0.21*hnm(i)
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY PIVOT OH
!
      argum = max( 0.0 , xl1HOx**2. + 4.0*xl2HOx*xpHOx )
!
      aHOxOH = 0.5
!
      HM(i,13) = aHOxOH*HM(i,13) + (1.-aHOxOH)*(sqrt(argum) - xl1HOx)     &
        / (2.0*xl2HOx)
!
    endif
!
  enddo
!$omp end do
!
else
!
!================================================================
!  EXPLICIT CALCULATION FOR SHORT-LIVED SPECIES THAT ARE NOT
!  TRANSPORTED
!================================================================
!
!$omp do
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
!  d[H]/dt = 0.0
!
      HM(i,12) = max( 1.0e-10 , ( + 5.e-7*hnm(i)*( + k(i,86)*HM(i,13) +  &
        k(i,293)*HM(i,2) + k(i,299)*HM(i,1) + k(i,320)*HM(i,9) ) +       &
        HM(i,13)*( + k(i,88)*QM(i,6) + k(i,294)*HM(i,1) ) +              &
        k(i,12)*QM(i,20) + k(i,28)*gH2O(i) + k(i,34)*gHCl(i) +           &
        k(i,307)*HM(i,2)*QM(i,2) + k(i,309)*HM(i,15)*HM(i,1) ) / ( +     &
        HM(i,14)*( + k(i,296) + k(i,297) + k(i,298) ) +                  &
        k(i,291)*0.21*hnm(i) + k(i,292)*HM(i,3) ) )
!
    endif
!
  enddo
!$omp end do
!
!$omp do
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
!  d[OH]/dt = 0.0
!
      xprOH =  + HM(i,3)*( + k(i,59)*HM(i,14) +                           &
        0.12*k(i,152)*QM(i,42) + 0.40*k(i,153)*QM(i,43) +                 &
        0.63*k(i,154)*QM(i,44) + 0.28*k(i,155)*QM(i,45) +                 &
        0.28*k(i,156)*QM(i,46) + 0.85*k(i,157)*QM(i,47) +                 &
        0.85*k(i,158)*QM(i,48) + 0.07*k(i,159)*QM(i,57) +                 &
        0.21*k(i,160)*QM(i,56) + 0.036*k(i,161)*QM(i,62) +                &
        k(i,167)*HM(i,17) + k(i,171)*HM(i,18) + k(i,175)*HM(i,19) +       &
        k(i,292)*HM(i,12) ) + HM(i,1)*( + 0.02*k(i,89)*QM(i,46) +         &
        k(i,295)*HM(i,14) + k(i,299)*5.e-7*hnm(i) + k(i,311)*QM(i,20) +   &
        k(i,330)*QM(i,10) + k(i,358)*QM(i,18) ) + HM(i,2)*( +             &
        2.0*k(i,57)*gH2O(i) + k(i,293)*5.e-7*hnm(i) + k(i,306)*QM(i,2) +  &
        k(i,353)*gHBr(i) ) + HM(i,14)*( + k(i,70)*HM(i,5) +               &
        k(i,74)*HM(i,7) + 2.0*k(i,296)*HM(i,12) + k(i,322)*HM(i,9) ) +    &
        HM(i,13)*( + 0.036*k(i,93)*QM(i,39) + k(i,122)*QM(i,22) +         &
        0.49*k(i,123)*QM(i,63) ) + QM(i,10)*( + k(i,30) +                 &
        k(i,332)*HM(i,9) ) + k(i,4)*QM(i,35) + k(i,5)*gHNO3(i) +          &
        k(i,7)*QM(i,16) + 2.0*k(i,10)*QM(i,12) + k(i,14)*QM(i,22) +       &
        k(i,15)*QM(i,63) + k(i,16)*QM(i,64) + k(i,28)*gH2O(i) +           &
        k(i,39)*QM(i,18)
!
      xpeOH =  + QM(i,51)*( + k(i,108) + k(i,109) + k(i,110) ) +          &
        QM(i,40)*( + k(i,94) + k(i,95) ) + QM(i,49)*( + k(i,104) +        &
        k(i,105) ) + QM(i,50)*( + k(i,106) + k(i,107) ) + QM(i,58)*( +    &
        k(i,119) + k(i,120) ) + QM(i,22)*( + k(i,121) + k(i,122) ) +      &
        QM(i,64)*( + k(i,124) + k(i,125) ) + 2.0*( + k(i,300)*HM(i,13) +  &
        k(i,301)*HM(i,13) ) + HM(i,8)*( + k(i,323) + k(i,324) ) +         &
        k(i,58)*HM(i,3) + k(i,60)*HM(i,14) + k(i,61)*QM(i,12) +           &
        k(i,67)*HM(i,5) + k(i,68)*HM(i,6) + k(i,69)*HM(i,7) +             &
        k(i,75)*QM(i,35) + k(i,76)*gHNO3(i) + k(i,77)*QM(i,16) +          &
        k(i,86)*5.e-7*hnm(i) + k(i,87)*QM(i,37) + k(i,88)*QM(i,6) +       &
        k(i,91)*QM(i,2) + k(i,92)*QM(i,38) + k(i,93)*QM(i,39) +           &
        k(i,96)*QM(i,41) + k(i,97)*QM(i,42) + k(i,98)*QM(i,43) +          &
        k(i,99)*QM(i,44) + k(i,100)*QM(i,45) + k(i,101)*QM(i,46) +        &
        k(i,102)*QM(i,47) + k(i,103)*QM(i,48) + k(i,111)*QM(i,20) +       &
        k(i,112)*QM(i,52) + k(i,113)*QM(i,53) + k(i,114)*QM(i,59) +       &
        k(i,115)*QM(i,54) + k(i,116)*QM(i,55) + k(i,117)*QM(i,57) +       &
        k(i,118)*QM(i,56) + k(i,123)*QM(i,63) + k(i,126)*QM(i,61) +       &
        k(i,127)*QM(i,62) + k(i,128)*QM(i,60) + k(i,290)*QM(i,36) +       &
        k(i,294)*HM(i,1) + k(i,325)*gHCl(i) + k(i,329)*QM(i,10) +         &
        k(i,341)*QM(i,27) + k(i,342)*QM(i,28) + k(i,344)*QM(i,29) +       &
        k(i,354)*HM(i,10) + k(i,355)*gHBr(i) + k(i,360)*QM(i,30)
!
      HM(i,13) = max( 1.0e-10 , xprOH / xpeOH )
!
    endif
!
  enddo
!$omp end do
!
!$omp do
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
!  d[HO2]/dt = 0.0
!
      xprHO2 =  + HM(i,13)*( + k(i,58)*HM(i,3) + k(i,61)*QM(i,12) +         &
        k(i,69)*HM(i,7) + k(i,87)*QM(i,37) + 0.381*k(i,93)*QM(i,39) +       &
        k(i,95)*QM(i,40) + 0.049*k(i,96)*QM(i,41) + k(i,105)*QM(i,49) +     &
        k(i,107)*QM(i,50) + k(i,110)*QM(i,51) + k(i,111)*QM(i,20) +         &
        k(i,114)*QM(i,59) + k(i,115)*QM(i,54) + 0.49*k(i,117)*QM(i,57) +    &
        0.50*k(i,118)*QM(i,56) + k(i,119)*QM(i,58) + k(i,120)*QM(i,58) +    &
        k(i,125)*QM(i,64) + 0.40*k(i,127)*QM(i,62) ) + HM(i,5)*( +          &
        k(i,181)*QM(i,65) + 0.742*k(i,182)*QM(i,66) +                       &
        0.599*k(i,183)*QM(i,67) + 0.606*k(i,184)*QM(i,68) +                 &
        k(i,185)*QM(i,69) + k(i,186)*QM(i,70) + k(i,187)*QM(i,71) +         &
        0.847*k(i,188)*QM(i,72) + 0.80*k(i,189)*QM(i,73) +                  &
        0.65*k(i,190)*QM(i,74) + 0.95*k(i,191)*QM(i,76) +                   &
        0.95*k(i,192)*QM(i,77) + k(i,193)*QM(i,78) +                        &
        0.77*k(i,196)*QM(i,81) + k(i,197)*QM(i,82) ) + QM(i,21)*( +         &
        k(i,222)*QM(i,65) + 0.992*k(i,223)*QM(i,66) +                       &
        0.946*k(i,224)*QM(i,67) + 0.993*k(i,225)*QM(i,68) +                 &
        k(i,226)*QM(i,69) + k(i,227)*QM(i,70) + k(i,228)*QM(i,71) +         &
        k(i,229)*QM(i,72) + 2.0*k(i,230)*QM(i,73) +                         &
        2.00*k(i,231)*QM(i,74) + k(i,232)*QM(i,76) + k(i,233)*QM(i,77) +    &
        2.0*k(i,234)*QM(i,78) + k(i,235)*QM(i,79) +                         &
        k(i,237)*QM(i,80) ) + HM(i,3)*( + 0.26*k(i,152)*QM(i,42) +          &
        0.25*k(i,153)*QM(i,43) + 0.22*k(i,154)*QM(i,44) +                   &
        0.30*k(i,155)*QM(i,45) + 0.30*k(i,156)*QM(i,46) +                   &
        0.10*k(i,157)*QM(i,47) + 0.10*k(i,158)*QM(i,48) +                   &
        0.29*k(i,159)*QM(i,57) + 0.29*k(i,160)*QM(i,56) +                   &
        0.08*k(i,161)*QM(i,62) ) + HM(i,7)*( + k(i,129)*QM(i,20) +          &
        k(i,131)*QM(i,54) + 0.50*k(i,134)*QM(i,56) ) + 0.21*hnm(i)*( +      &
        k(i,166)*HM(i,17) + k(i,170)*HM(i,18) + k(i,174)*HM(i,19) ) +       &
        QM(i,16)*( + k(i,6) + k(i,72) ) + QM(i,60)*( + k(i,22) +            &
        k(i,23) ) + k(i,12)*QM(i,20) + k(i,13)*QM(i,52) +                   &
        k(i,15)*QM(i,63) + 0.80*k(i,19)*QM(i,54) + k(i,20)*QM(i,55) +       &
        k(i,21)*QM(i,56) + k(i,24)*QM(i,57) + k(i,25)*QM(i,59) +            &
        0.28*k(i,89)*QM(i,46)*HM(i,1)
      xprHO2 = xprHO2 + HM(i,7)*( + k(i,263)*QM(i,21) +                     &
        k(i,264)*QM(i,65) + 0.792*k(i,265)*QM(i,66) +                       &
        0.699*k(i,266)*QM(i,67) + 0.845*k(i,267)*QM(i,68) +                 &
        k(i,268)*QM(i,69) + k(i,269)*QM(i,70) + k(i,270)*QM(i,71) +         &
        k(i,271)*QM(i,72) + k(i,272)*QM(i,73) + k(i,273)*QM(i,74) +         &
        k(i,274)*QM(i,76) + k(i,275)*QM(i,77) + k(i,276)*QM(i,78) +         &
        0.77*k(i,279)*QM(i,81) + k(i,280)*QM(i,82) ) + QM(i,79)*( +         &
        0.500*k(i,242)*QM(i,65) + 0.488*k(i,243)*QM(i,66) +                 &
        0.438*k(i,244)*QM(i,67) + 0.489*k(i,245)*QM(i,68) +                 &
        0.5*k(i,246)*QM(i,69) + 0.501*k(i,247)*QM(i,70) +                   &
        0.510*k(i,248)*QM(i,71) + 0.506*k(i,249)*QM(i,72) +                 &
        k(i,250)*QM(i,73) + k(i,251)*QM(i,74) + k(i,252)*QM(i,76) +         &
        k(i,253)*QM(i,77) + k(i,254)*QM(i,78) + 0.38*k(i,257)*QM(i,81) +    &
        0.50*k(i,258)*QM(i,82) ) + QM(i,21)*( + 0.88*k(i,239)*QM(i,81) +    &
        k(i,240)*QM(i,82) + 0.500*k(i,241)*QM(i,83) +                       &
        k(i,283)*QM(i,84) ) + QM(i,20)*( + k(i,311)*HM(i,1) +               &
        k(i,312)*HM(i,9) + k(i,315)*HM(i,11) ) + QM(i,82)*( +               &
        k(i,260)*QM(i,82) + 0.500*k(i,261)*QM(i,83) ) + 0.21*hnm(i)*( +     &
        k(i,291)*HM(i,12) + k(i,313)*HM(i,16) ) + HM(i,13)*( +              &
        k(i,323)*HM(i,8) + k(i,354)*HM(i,10) )
!
      xpeHO2 =  + HM(i,12)*( + k(i,296) + k(i,297) + k(i,298) ) +           &
        2.0*( + k(i,62)*HM(i,14) + k(i,63)*HM(i,14)*gH2O(i) ) +             &
        HM(i,7)*( + k(i,73) + k(i,74) ) + QM(i,79)*( + k(i,213) +           &
        k(i,214) ) + QM(i,80)*( + k(i,215) + k(i,216) ) + HM(i,9)*( +       &
        k(i,321) + k(i,322) ) + k(i,59)*HM(i,3) + k(i,60)*HM(i,13) +        &
        k(i,70)*HM(i,5) + k(i,71)*HM(i,6) + k(i,163)*QM(i,75) +             &
        k(i,199)*QM(i,21) + k(i,200)*QM(i,65) + k(i,201)*QM(i,66) +         &
        k(i,202)*QM(i,67) + k(i,203)*QM(i,68) + k(i,204)*QM(i,69) +         &
        k(i,205)*QM(i,70) + k(i,206)*QM(i,71) + k(i,207)*QM(i,72) +         &
        k(i,208)*QM(i,73) + k(i,209)*QM(i,74) + k(i,210)*QM(i,76) +         &
        k(i,211)*QM(i,77) + k(i,212)*QM(i,78) + k(i,217)*QM(i,81) +         &
        k(i,218)*QM(i,82) + k(i,219)*QM(i,83) + k(i,282)*QM(i,84) +         &
        k(i,295)*HM(i,1) + k(i,328)*HM(i,8) + k(i,352)*HM(i,11) +           &
        k(i,357)*HM(i,10)
!
      HM(i,14) = max( 1.0e-10 , xprHO2 / xpeHO2 )
!
    endif
!
  enddo
  !$omp end do
!
endif
!
!================================================================
!  EXPLICIT CALCULATION FOR SHORT-LIVED SPECIES THAT ARE NOT
!  TRANSPORTED
!================================================================
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
!  d[CH3]/dt = 0.0
!
    HM(i,15) = max( 1.0e-10 , ( + QM(i,2)*( + k(i,91)*HM(i,13) +        &
      k(i,306)*HM(i,2) + k(i,319)*HM(i,9) ) ) / ( + k(i,309)*HM(i,1) +  &
      k(i,310)*0.21*hnm(i) ) )
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
!  d[CH3O]/dt = 0.0
!
    HM(i,16) = max( 1.0e-10 , ( + QM(i,21)*( + k(i,180)*HM(i,5) +      &
      2.0*k(i,220)*QM(i,21) + k(i,314)*HM(i,8) ) + k(i,14)*QM(i,22) +  &
      k(i,307)*HM(i,2)*QM(i,2) ) / ( + k(i,313)*0.21*hnm(i) ) )
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
!  d[ADDT]/dt = 0.0
!
    HM(i,17) = max( 1.0e-10 , ( + k(i,104)*QM(i,49)*HM(i,13) ) / ( +  &
      0.21*hnm(i)*( + k(i,165) + k(i,166) ) + k(i,164)*HM(i,6) +      &
      k(i,167)*HM(i,3) ) )
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
!  d[ADDX]/dt = 0.0
!
    HM(i,18) = max( 1.0e-10 , ( + k(i,106)*QM(i,50)*HM(i,13) ) / ( + &
      0.21*hnm(i)*( + k(i,169) + k(i,170) ) + k(i,168)*HM(i,6) +     &
      k(i,171)*HM(i,3) ) )
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
!  d[ADDC]/dt = 0.0
!
    HM(i,19) = max( 1.0e-10 , ( + k(i,108)*QM(i,51)*HM(i,13) ) / ( +  &
      0.21*hnm(i)*( + k(i,173) + k(i,174) ) + k(i,172)*HM(i,6) +      &
      k(i,175)*HM(i,3) ) )
!
  endif
!
enddo
!$omp end do
!
!!!!!
!
!  2. NIGHT-TIME CHEMISTRY
!
!!!!!
!
!$omp do
do i = i1,i2
!
  if (sza(i) > SZAlim) then
!
!================================================================
!  AT NIGHT, FAMILY Ox = O3P + O1D + O3
!  IS REDUCED TO O3 SPECIE
!================================================================
!
!  O3P
!
    HM(i,1) = 0.0
!
!  O1D
!
    HM(i,2) = 0.0
!
!  O3
!
    HM(i,3) = max( 0.0 , QM(i,86) )
!
!================================================================
!  FOR FAMILY NOx = N + NO + NO2 + NO3
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT AT NIGHT :
!
!  k069 :  OH + NO3  -->  NO2 + HO2
!  k070 :  HO2 + NO  -->  NO2 + OH
!  k074 :  HO2 + NO3  --> NO2 + OH
!  k078 :  O3 + NO  -->  NO2 + O2
!  k079 :  O3 + NO2  -->  NO3 + O2
!  k080 :  NO + NO + O2  -->  NO2 + NO2
!  k081 :  NO3 + NO  -->  NO2 + NO2
!  k082 :  NO3 + NO2  -->  NO + NO2 + O2
!  k083 :  NO3 + NO2  -->  N2O5  (destruction only)
!  k085 :  NO3 + NO3  -->  NO2 + NO2 + O2
!  k134 :  DCB + NO3  -->  0.50*TCO3 + 0.50*HO2 + 0.50*XO2 +
!                0.25*GLY + 0.25*ALD + 0.50*NO2 + 0.03*KET + 0.25*MGLY +
!                0.50*HNO3
!  k151 :  TPAN + NO3  -->  PAN + HCHO + NO2 + XO2
!  k180 :  MO2 + NO  -->  CH3O + NO2
!  k181 :  ETHP + NO  -->  ALD + HO2 + NO2
!  k182 :  HC3P + NO  -->  0.047*HCHO + 0.233*ALD + 0.623*KET +
!                0.063*GLY + 0.742*HO2 + 0.150*MO2 + 0.048*ETHP + 0.048*XO2 +
!                0.059*ONIT + 0.941*NO2
!  k183 :  HC5P + NO  -->  0.021*HCHO + 0.211*ALD + 0.722*KET +
!                0.599*HO2 + 0.031*MO2 + 0.245*ETHP + 0.334*XO2 + 0.124*ONIT +
!                0.876*NO2
!  k184 :  HC8P + NO  -->  0.150*ALD + 0.642*KET + 0.133*ETHP +
!                0.261*ONIT + 0.739*NO2 + 0.606*HO2 + 0.416*XO2
!  k185 :  ETEP + NO  -->  1.6*HCHO + HO2 + NO2 + 0.2*ALD
!  k186 :  OLTP + NO  -->  0.94*ALD + HCHO + HO2 + NO2 + 0.06*KET
!  k187 :  OLIP + NO  -->  HO2 + 1.71*ALD + 0.29*KET + NO2
!  k188 :  ISOP + NO  -->  0.446*MACR + 0.354*OLT + 0.847*HO2 +
!                0.847*NO2 + 0.153*ONIT + 0.606*HCHO
!  k189 :  APIP + NO  -->  0.80*HO2 + 0.80*ALD + 0.80*NO2 + 0.80*KET + 0.20*ONIT
!  k190 :  LIMP + NO  -->  0.65*HO2 + 0.40*MACR + 0.25*OLI +
!                0.25*HCHO + 0.65*NO2 + 0.35*ONIT
!  k191 :  TOLP + NO  -->  0.95*NO2 + 0.95*HO2 + 1.20*GLY +
!                0.50*DCB + 0.05*ONIT + 0.65*MGLY
!  k192 :  XYLP + NO  -->  0.95*NO2 + 0.95*HO2 + 0.35*GLY +
!                0.95*DCB + 0.05*ONIT + 0.60*MGLY
!  k193 :  CSLP + NO  -->  GLY + MGLY + NO2 + HO2
!  k194 :  ACO3 + NO  -->  MO2 + NO2
!  k195 :  TCO3 + NO  -->  NO2 + ACO3 + HCHO
!  k196 :  KETP + NO  -->  0.54*MGLY + 0.46*ALD + 0.23*ACO3 +
!                0.77*HO2 + 0.16*XO2 + NO2
!  k197 :  OLNN + NO  -->  ONIT + NO2 + HO2
!  k198 :  OLND + NO  -->  0.287*HCHO + 1.24*ALD + 0.464*KET + 2.0*NO2
!  k263 :  MO2 + NO3  -->  HCHO + HO2 + NO2
!  k264 :  ETHP + NO3  -->  ALD + HO2 + NO2
!  k265 :  HC3P + NO3  -->  0.048*HCHO + 0.243*ALD + 0.670*KET +
!                0.063*GLY + 0.792*HO2 + 0.155*MO2 + 0.053*ETHP + 0.051*XO2 + NO
!  k266 :  HC5P + NO3  -->  0.021*HCHO + 0.239*ALD + 0.828*KET +
!                0.699*HO2 + 0.040*MO2 + 0.262*ETHP + 0.391*XO2 + NO2
!  k267 :  HC8P + NO3  -->  0.187*ALD + 0.880*KET + 0.155*ETHP +
!                NO2 + 0.845*HO2 + 0.587*XO2
!  k268 :  ETEP + NO3  -->  1.6*HCHO + HO2 + NO2 + 0.2*ALD
!  k269 :  OLTP + NO3  -->  0.94*ALD + HCHO + HO2 + NO2 + 0.06*KET
!  k270 :  OLIP + NO3  -->  HO2 + 1.71*ALD + 0.29*KET + NO2
!  k271 :  ISOP + NO3  -->  0.600*MACR + 0.400*OLT + HO2 + NO2 + 0.686*HCHO
!  k272 :  APIP + NO3  -->  HO2 + ALD + NO2 + KET
!  k273 :  LIMP + NO3  -->  HO2 + 0.60*MACR + 0.40*OLI + 0.40*HCHO + NO2
!  k274 :  TOLP + NO3  -->  NO2 + HO2 + 0.50*DCB + 1.30*GLY + 0.70*MGLY
!  k275 :  XYLP + NO3  -->  NO2 + HO2 + DCB + 0.74*GLY + 1.26*MGLY
!  k276 :  CSLP + NO3  -->  GLY + MGLY + NO2 + HO2
!  k277 :  ACO3 + NO3  -->  MO2 + NO2
!  k278 :  TCO3 + NO3  -->  NO2 + ACO3 + HCHO
!  k279 :  KETP + NO3  -->  0.54*MGLY + 0.46*ALD + 0.23*ACO3 +
!                0.77*HO2 + 0.16*XO2 + NO2
!  k280 :  OLNN + NO3  -->  ONIT + NO2 + HO2
!  k281 :  OLND + NO3  -->  0.280*HCHO + 1.24*ALD + 0.469*KET + 2.0*NO2
!  k286 :  XO2 + NO  -->  NO2
!  k287 :  XO2 + NO3  -->  NO2
!  k288 :  DMS + NO3  -->  SO2 + NO2
!  k318 :  ClO + NO  -->  NO2 + Cl
!  k347 :  BrO + NO  -->  NO2 + Br
!================================================================
!
!  N
!
    HM(i,4) = 0.0
!
!  NO3 / NO2
!
    rNOxNO3 = ( + k(i,79)*HM(i,3) ) / ( + HM(i,6)*( + k(i,82) +         &
      k(i,83) ) + k(i,69)*HM(i,13) + k(i,74)*HM(i,14) +                 &
      k(i,81)*HM(i,5) + 2.0*k(i,85)*HM(i,7) + 0.50*k(i,134)*QM(i,56) +  &
      k(i,151)*QM(i,62) + k(i,263)*QM(i,21) + k(i,264)*QM(i,65) +       &
      k(i,265)*QM(i,66) + k(i,266)*QM(i,67) + k(i,267)*QM(i,68) +       &
      k(i,268)*QM(i,69) + k(i,269)*QM(i,70) + k(i,270)*QM(i,71) +       &
      k(i,271)*QM(i,72) + k(i,272)*QM(i,73) + k(i,273)*QM(i,74) +       &
      k(i,274)*QM(i,76) + k(i,275)*QM(i,77) + k(i,276)*QM(i,78) +       &
      k(i,277)*QM(i,79) + k(i,278)*QM(i,80) + k(i,279)*QM(i,81) +       &
      k(i,280)*QM(i,82) + k(i,281)*QM(i,83) + k(i,287)*QM(i,84) +       &
      k(i,288)*QM(i,36) )
!
    rNOxNO3 = max( 0. , rNOxNO3 )
!
!  NO / NO2
!
    rNOxNO = ( + k(i,82)*rNOxNO3*HM(i,6) ) / ( + 0.95*( +               &
      k(i,191)*QM(i,76) + k(i,192)*QM(i,77) ) + k(i,70)*HM(i,14) +      &
      k(i,78)*HM(i,3) + 2.0*k(i,80)*0.21*hnm(i)*HM(i,5) +               &
      k(i,81)*HM(i,7) + k(i,180)*QM(i,21) + k(i,181)*QM(i,65) +         &
      0.941*k(i,182)*QM(i,66) + 0.876*k(i,183)*QM(i,67) +               &
      0.739*k(i,184)*QM(i,68) + k(i,185)*QM(i,69) +                     &
      k(i,186)*QM(i,70) + k(i,187)*QM(i,71) +                           &
      0.847*k(i,188)*QM(i,72) + 0.80*k(i,189)*QM(i,73) +                &
      0.65*k(i,190)*QM(i,74) + k(i,193)*QM(i,78) + k(i,194)*QM(i,79) +  &
      k(i,195)*QM(i,80) + k(i,196)*QM(i,81) + k(i,197)*QM(i,82) +       &
      k(i,198)*QM(i,83) + k(i,286)*QM(i,84) + k(i,318)*HM(i,8) +        &
      k(i,347)*HM(i,10) )
!
    rNOxNO = max( 0. , rNOxNO )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY AT NIGHT :
!
    rNOx = 1. + rNOxNO + rNOxNO3
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS AT NIGHT :
!
    aNOxNO2 = 0.0
!
    HM(i,6) = aNOxNO2*HM(i,6) + (1.-aNOxNO2)*QM(i,87)/rNOx
!
    aNOxNO = 0.0
!
    HM(i,5) = aNOxNO*HM(i,5) + (1.-aNOxNO)*HM(i,6)*rNOxNO
!
    aNOxNO3 = 0.0
!
    HM(i,7) = aNOxNO3*HM(i,7) + (1.-aNOxNO3)*HM(i,6)*rNOxNO3
!
!================================================================
!  AT NIGHT, FAMILY ClOx = ClO + Cl
!  IS REDUCED TO ClO SPECIE
!================================================================
!
!  ClO
!
    HM(i,8) = max( 0.0 , QM(i,88) )
!
!  Cl
!
    HM(i,9) = 0.0
!
!================================================================
!  AT NIGHT, FAMILY BrOx = BrO + Br
!  IS REDUCED TO BrO SPECIE
!================================================================
!
!  BrO
!
    HM(i,10) = max( 0.0 , QM(i,89) )
!
!  Br
!
    HM(i,11) = 0.0
!
!  H
!
    HM(i,12) = 0.0
!
!  CH3
!
    HM(i,15) = 0.0
!
!  CH3O
!
    HM(i,16) = 0.0
!
!  ADDT
!
    HM(i,17) = 0.0
!
!  ADDX
!
    HM(i,18) = 0.0
!
!  ADDC
!
    HM(i,19) = 0.0
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) > SZAlim) then
!
!  d[OH]/dt = 0.0
!
    xprOH =  + HM(i,3)*( + k(i,59)*HM(i,14) +                           &
      0.12*k(i,152)*QM(i,42) + 0.40*k(i,153)*QM(i,43) +                 &
      0.63*k(i,154)*QM(i,44) + 0.28*k(i,155)*QM(i,45) +                 &
      0.28*k(i,156)*QM(i,46) + 0.85*k(i,157)*QM(i,47) +                 &
      0.85*k(i,158)*QM(i,48) + 0.07*k(i,159)*QM(i,57) +                 &
      0.21*k(i,160)*QM(i,56) + 0.036*k(i,161)*QM(i,62) ) +              &
      HM(i,13)*( + 0.036*k(i,93)*QM(i,39) + k(i,122)*QM(i,22) +         &
      0.49*k(i,123)*QM(i,63) ) + HM(i,14)*( + k(i,70)*HM(i,5) +         &
      k(i,74)*HM(i,7) )
!
    xpeOH =  + QM(i,51)*( + k(i,108) + k(i,109) + k(i,110) ) +          &
      QM(i,40)*( + k(i,94) + k(i,95) ) + QM(i,49)*( + k(i,104) +        &
      k(i,105) ) + QM(i,50)*( + k(i,106) + k(i,107) ) + QM(i,58)*( +    &
      k(i,119) + k(i,120) ) + QM(i,22)*( + k(i,121) + k(i,122) ) +      &
      QM(i,64)*( + k(i,124) + k(i,125) ) + 2.0*( + k(i,300)*HM(i,13) +  &
      k(i,301)*HM(i,13) ) + HM(i,8)*( + k(i,323) + k(i,324) ) +         &
      k(i,58)*HM(i,3) + k(i,60)*HM(i,14) + k(i,61)*QM(i,12) +           &
      k(i,67)*HM(i,5) + k(i,68)*HM(i,6) + k(i,69)*HM(i,7) +             &
      k(i,75)*QM(i,35) + k(i,76)*gHNO3(i) + k(i,77)*QM(i,16) +          &
      k(i,86)*5.e-7*hnm(i) + k(i,87)*QM(i,37) + k(i,88)*QM(i,6) +       &
      k(i,91)*QM(i,2) + k(i,92)*QM(i,38) + k(i,93)*QM(i,39) +           &
      k(i,96)*QM(i,41) + k(i,97)*QM(i,42) + k(i,98)*QM(i,43) +          &
      k(i,99)*QM(i,44) + k(i,100)*QM(i,45) + k(i,101)*QM(i,46) +        &
      k(i,102)*QM(i,47) + k(i,103)*QM(i,48) + k(i,111)*QM(i,20) +       &
      k(i,112)*QM(i,52) + k(i,113)*QM(i,53) + k(i,114)*QM(i,59) +       &
      k(i,115)*QM(i,54) + k(i,116)*QM(i,55) + k(i,117)*QM(i,57) +       &
      k(i,118)*QM(i,56) + k(i,123)*QM(i,63) + k(i,126)*QM(i,61) +       &
      k(i,127)*QM(i,62) + k(i,128)*QM(i,60) + k(i,290)*QM(i,36) +       &
      k(i,325)*gHCl(i) + k(i,329)*QM(i,10) + k(i,341)*QM(i,27) +        &
      k(i,342)*QM(i,28) + k(i,344)*QM(i,29) + k(i,354)*HM(i,10) +       &
      k(i,355)*gHBr(i) + k(i,360)*QM(i,30)
!
    HM(i,13) = max( 1.0e-10 , xprOH / xpeOH )
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) > SZAlim) then
!
!  d[HO2]/dt = 0.0
!
    xprHO2 =  + HM(i,13)*( + k(i,58)*HM(i,3) + k(i,61)*QM(i,12) +       &
      k(i,69)*HM(i,7) + k(i,87)*QM(i,37) + 0.381*k(i,93)*QM(i,39) +     &
      k(i,95)*QM(i,40) + 0.049*k(i,96)*QM(i,41) + k(i,105)*QM(i,49) +   &
      k(i,107)*QM(i,50) + k(i,110)*QM(i,51) + k(i,111)*QM(i,20) +       &
      k(i,114)*QM(i,59) + k(i,115)*QM(i,54) + 0.49*k(i,117)*QM(i,57) +  &
      0.50*k(i,118)*QM(i,56) + k(i,119)*QM(i,58) + k(i,120)*QM(i,58) +  &
      k(i,125)*QM(i,64) + 0.40*k(i,127)*QM(i,62) ) + QM(i,21)*( +       &
      k(i,222)*QM(i,65) + 0.992*k(i,223)*QM(i,66) +                     &
      0.946*k(i,224)*QM(i,67) + 0.993*k(i,225)*QM(i,68) +               &
      k(i,226)*QM(i,69) + k(i,227)*QM(i,70) + k(i,228)*QM(i,71) +       &
      k(i,229)*QM(i,72) + 2.0*k(i,230)*QM(i,73) +                       &
      2.00*k(i,231)*QM(i,74) + k(i,232)*QM(i,76) + k(i,233)*QM(i,77) +  &
      2.0*k(i,234)*QM(i,78) + k(i,235)*QM(i,79) + k(i,237)*QM(i,80) +   &
      0.88*k(i,239)*QM(i,81) + k(i,240)*QM(i,82) +                      &
      0.500*k(i,241)*QM(i,83) ) + HM(i,5)*( + k(i,181)*QM(i,65) +       &
      0.742*k(i,182)*QM(i,66) + 0.599*k(i,183)*QM(i,67) +               &
      0.606*k(i,184)*QM(i,68) + k(i,185)*QM(i,69) +                     &
      k(i,186)*QM(i,70) + k(i,187)*QM(i,71) +                           &
      0.847*k(i,188)*QM(i,72) + 0.80*k(i,189)*QM(i,73) +                &
      0.65*k(i,190)*QM(i,74) + 0.95*k(i,191)*QM(i,76) +                 &
      0.95*k(i,192)*QM(i,77) + k(i,193)*QM(i,78) +                      &
      0.77*k(i,196)*QM(i,81) + k(i,197)*QM(i,82) ) + QM(i,79)*( +       &
      0.500*k(i,242)*QM(i,65) + 0.488*k(i,243)*QM(i,66) +               &
      0.438*k(i,244)*QM(i,67) + 0.489*k(i,245)*QM(i,68) +               &
      0.5*k(i,246)*QM(i,69) + 0.501*k(i,247)*QM(i,70) +                 &
      0.510*k(i,248)*QM(i,71) + 0.506*k(i,249)*QM(i,72) +               &
      k(i,250)*QM(i,73) + k(i,251)*QM(i,74) + k(i,252)*QM(i,76) +       &
      k(i,253)*QM(i,77) ) + HM(i,3)*( + 0.26*k(i,152)*QM(i,42) +        &
      0.25*k(i,153)*QM(i,43) + 0.22*k(i,154)*QM(i,44) +                 &
      0.30*k(i,155)*QM(i,45) + 0.30*k(i,156)*QM(i,46) +                 &
      0.10*k(i,157)*QM(i,47) + 0.10*k(i,158)*QM(i,48) +                 &
      0.29*k(i,159)*QM(i,57) + 0.29*k(i,160)*QM(i,56) +                 &
      0.08*k(i,161)*QM(i,62) ) + HM(i,7)*( + k(i,129)*QM(i,20) +        &
      k(i,131)*QM(i,54) + 0.50*k(i,134)*QM(i,56) ) + k(i,72)*QM(i,16)
    xprHO2 = xprHO2 + HM(i,7)*( + k(i,263)*QM(i,21) +                   &
      k(i,264)*QM(i,65) + 0.792*k(i,265)*QM(i,66) +                     &
      0.699*k(i,266)*QM(i,67) + 0.845*k(i,267)*QM(i,68) +               &
      k(i,268)*QM(i,69) + k(i,269)*QM(i,70) + k(i,270)*QM(i,71) +       &
      k(i,271)*QM(i,72) + k(i,272)*QM(i,73) + k(i,273)*QM(i,74) +       &
      k(i,274)*QM(i,76) + k(i,275)*QM(i,77) + k(i,276)*QM(i,78) +       &
      0.77*k(i,279)*QM(i,81) + k(i,280)*QM(i,82) ) + QM(i,79)*( +       &
      k(i,254)*QM(i,78) + 0.38*k(i,257)*QM(i,81) +                      &
      0.50*k(i,258)*QM(i,82) ) + QM(i,82)*( + k(i,260)*QM(i,82) +       &
      0.500*k(i,261)*QM(i,83) ) + HM(i,13)*( + k(i,323)*HM(i,8) +       &
      k(i,354)*HM(i,10) ) + k(i,283)*QM(i,84)*QM(i,21)
!
    xpeHO2 =  + 2.0*( + k(i,62)*HM(i,14) + k(i,63)*HM(i,14)*gH2O(i) )   &
     + HM(i,7)*( + k(i,73) + k(i,74) ) +                                &
      QM(i,79)*( + k(i,213) + k(i,214) ) + QM(i,80)*( + k(i,215) +      &
      k(i,216) ) + k(i,59)*HM(i,3) + k(i,60)*HM(i,13) +                 &
      k(i,70)*HM(i,5) + k(i,71)*HM(i,6) + k(i,163)*QM(i,75) +           &
      k(i,199)*QM(i,21) + k(i,200)*QM(i,65) + k(i,201)*QM(i,66) +       &
      k(i,202)*QM(i,67) + k(i,203)*QM(i,68) + k(i,204)*QM(i,69) +       &
      k(i,205)*QM(i,70) + k(i,206)*QM(i,71) + k(i,207)*QM(i,72) +       &
      k(i,208)*QM(i,73) + k(i,209)*QM(i,74) + k(i,210)*QM(i,76) +       &
      k(i,211)*QM(i,77) + k(i,212)*QM(i,78) + k(i,217)*QM(i,81) +       &
      k(i,218)*QM(i,82) + k(i,219)*QM(i,83) + k(i,282)*QM(i,84) +       &
      k(i,328)*HM(i,8) + k(i,357)*HM(i,10)
!
    HM(i,14) = max( 1.0e-10 , xprHO2 / xpeHO2 )
!
  endif
!
enddo
!$omp end do
!
!$omp end parallel	
!
!
RETURN
END SUBROUTINE CH_EQUIL
