SUBROUTINE CH_PTERMS(k,jco2,hnm,day,QM,HM,pr,pe,                          &
                     partHNO3,partH2O,partHCl,partHBr,imax,nk,ntra,nsls)
!
!!!!!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_0 
!
!!! DATE : 15/05/2005
!
!
!   CALCUL DES TERMES DE PRODUCTION ET DE DESTRUCTION DES ESPECES A
!   LONGUE DUREE DE VIE ET DES FAMILLES TRANSPORTEES
!
!!!!!
!
!  Original : H. Teyssedre, V.-H. Peuch, 08/2000
!
!  Modified : D. Charpin, version Fortran-90, 08/2006
!
!!!!!
!
#ifdef MPI
      USE CSTES_MOD
#endif
!
      IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
      INTEGER , INTENT(IN) :: imax, nk, ntra, nsls
      REAL (KIND=8) , DIMENSION(imax,nk), INTENT(IN) :: k
      REAL (KIND=8) , DIMENSION(imax)   , INTENT(IN) :: jco2,hnm,day
      REAL (KIND=8) , DIMENSION(imax)   , INTENT(IN) :: partHNO3,partH2O
      REAL (KIND=8) , DIMENSION(imax)   , INTENT(IN) :: partHCl,partHBr
      REAL (KIND=8) , DIMENSION(imax,ntra) , INTENT(IN) :: QM
      REAL (KIND=8) , DIMENSION(imax,nsls) , INTENT(IN) :: HM
      REAL (KIND=8) , DIMENSION(imax,ntra) , INTENT(INOUT) :: pr,pe
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
      INTEGER :: i,j,i1,i2
!
      REAL (KIND=8) , PARAMETER :: rco2=360.e-06
!
!!!!!
!
!  TRANSPORTED LONG-LIVED SPECIES AND FAMILIES (ARRAY QM)
!
!!!!!
!
!       1 : N2O         2 : CH4         3 : H2O         4 : HNO3        5 : N2O5
!       6 : CO          7 : OClO        8 : HCl         9 : ClONO2     10 : HOCl
!      11 : Cl2        12 : H2O2       13 : ClNO2      14 : HBr        15 : BrON
!      16 : HNO4       17 : Cl2O2      18 : HOBr       19 : BrCl       20 : HCHO
!      21 : MO2        22 : OP1        23 : CFC11      24 : CFC12      25 : CFC1
!      26 : CCl4       27 : CH3CCl3    28 : CH3Cl      29 : HCFC22     30 : CH3B
!      31 : H1211      32 : H1301      33 : H2SO4      34 : PSC        35 : HONO
!      36 : DMS        37 : SO2        38 : ETH        39 : HC3        40 : HC5
!      41 : HC8        42 : ETE        43 : OLT        44 : OLI        45 : DIEN
!      46 : ISO        47 : API        48 : LIM        49 : TOL        50 : XYL
!      51 : CSL        52 : ALD        53 : KET        54 : GLY        55 : MGLY
!      56 : DCB        57 : MACR       58 : UDD        59 : HKET       60 : ONIT
!      61 : PAN        62 : TPAN       63 : OP2        64 : PAA        65 : ETHP
!      66 : HC3P       67 : HC5P       68 : HC8P       69 : ETEP       70 : OLTP
!      71 : OLIP       72 : ISOP       73 : APIP       74 : LIMP       75 : PHO
!      76 : TOLP       77 : XYLP       78 : CSLP       79 : ACO3       80 : TCO3
!      81 : KETP       82 : OLNN       83 : OLND       84 : XO2        85 : SULF
!      86 : Ox         87 : NOx        88 : ClOx       89 : BrOx
!
!!!!!
!
!  SHORT-LIVED SPECIES AT INSTANTANEOUS EQUILIBRIUM (ARRAY HM)
!
!!!!!
!
!       1 : O3P         2 : O1D         3 : O3          4 : N           5 : NO
!       6 : NO2         7 : NO3         8 : ClO         9 : Cl         10 : BrO
!      11 : Br         12 : H          13 : OH         14 : HO2        15 : CH3
!      16 : CH3O       17 : ADDT       18 : ADDX       19 : ADDC       20 : PANs
!      21 : RONO2s     22 : NOy        23 : NOz        24 : ResN       25 : COVp
!      26 : COVpB      27 : COVtot     28 : HOx        29 : Oz
!
!	
!  pr(N2O) = 0.0
!
!  pe(N2O) =  + k029 + k304*[O1D] + k305*[O1D]
!
!  pr(CH4) =  + 0.06*k153*[OLT]*[O3] + 0.07*k154*[OLI]*[O3]
!
!  pe(CH4) =  + k091*[OH] + k306*[O1D] + k307*[O1D] + k308*[O1D] + k319*[Cl]
!
!  pr(H2O) =  + k060*[OH]*[HO2] + k061*[H2O2]*[OH] +
!                k063*[HO2]*[HO2]*[H2O]*partH2O + k075*[OH]*[HONO] +
!                k076*[OH]*[HNO3]*partHNO3 + k077*[OH]*[HNO4] + k086*[OH]*[H2] +
!                k091*[CH4]*[OH] + k092*[ETH]*[OH] + k093*[HC3]*[OH] +
!                k094*[HC5]*[OH] + k095*[HC5]*[OH] + k096*[HC8]*[OH] +
!                k111*[HCHO]*[OH] + k112*[ALD]*[OH] + k113*[KET]*[OH] +
!                k114*[HKET]*[OH] + k115*[GLY]*[OH] + k116*[MGLY]*[OH] +
!                k121*[OP1]*[OH] + k122*[OP1]*[OH] + k126*[PAN]*[OH] +
!                k128*[ONIT]*[OH] + k298*[H]*[HO2] + k300*[OH]*[OH] +
!                k325*[OH]*[HCl]*partHCl + k329*[OH]*[HOCl] +
!                k341*[OH]*[CH3CCl3] + k342*[OH]*[CH3Cl] + k344*[OH]*[HCFC22] +
!                k355*[OH]*[HBr]*partHBr + k360*[OH]*[CH3Br] +
!                k368*[HOCl]*[HCl] + k370*[HOBr]*[HCl] + k371*[HOBr]*[HBr] +
!                k372*[HOCl]*[HBr]
!
!  pe(H2O) =  + k028 + k057*[O1D] + k063*[HO2]*[HO2] +
!                k364*[ClONO2] + k366*[N2O5] + k369*[BrONO2]
!
!  pr(HNO3) =  + k068*[OH]*[NO2] + k073*[HO2]*[NO3] +
!                k129*[HCHO]*[NO3] + k130*[ALD]*[NO3] + k131*[GLY]*[NO3] +
!                k132*[MGLY]*[NO3] + 0.20*k133*[MACR]*[NO3] +
!                0.50*k134*[DCB]*[NO3] + k135*[CSL]*[NO3] + k364*[ClONO2]*[H2O]
!                k365*[ClONO2]*[HCl] + 2.0*k366*[N2O5]*[H2O] +
!                k367*[N2O5]*[HCl] + k369*[BrONO2]*[H2O]
!
!  pe(HNO3) =  + k005 + k076*[OH]
!
!  pr(N2O5) =  + k083*[NO3]*[NO2]
!
!  pe(N2O5) =  + k027 + k084 + k366*[H2O] + k367*[HCl]
!
!  pr(CO) =  + k011*[HCHO] + k012*[HCHO] + k013*[ALD] +
!                1.87*k018*[GLY] + 1.55*k019*[GLY] + k020*[MGLY] + k024*[MACR] +
!                0.01*k089*[ISO]*[O3P] + 0.036*k093*[HC3]*[OH] +
!                k111*[HCHO]*[OH] + 2.0*k115*[GLY]*[OH] + k116*[MGLY]*[OH] +
!                0.41*k117*[MACR]*[OH] + k129*[HCHO]*[NO3] +
!                2.0*k131*[GLY]*[NO3] + k132*[MGLY]*[NO3] +
!                0.80*k133*[MACR]*[NO3] + 0.43*k152*[ETE]*[O3] +
!                0.37*k153*[OLT]*[O3] + 0.30*k154*[OLI]*[O3] +
!                0.36*k155*[DIEN]*[O3] + 0.36*k156*[ISO]*[O3] +
!                0.14*k157*[API]*[O3] + 0.14*k158*[LIM]*[O3] +
!                0.54*k159*[MACR]*[O3] + 0.66*k160*[DCB]*[O3] +
!                0.13*k161*[TPAN]*[O3] + k311*[HCHO]*[O3P] + k312*[HCHO]*[Cl] +
!                k315*[HCHO]*[Br]
!
!  pe(CO) =  + k088*[OH]
!
!  pr(OClO) =  + k348*[BrO]*[ClO]
!
!  pe(OClO) =  + k040
!
!  pr(HCl) =  + k312*[HCHO]*[Cl] + k319*[Cl]*[CH4] +
!                k320*[Cl]*[H2] + k321*[Cl]*[HO2] + k324*[ClO]*[OH]
!
!  pe(HCl) =  + k034 + k325*[OH] + k365*[ClONO2] + k367*[N2O5] +
!                k368*[HOCl] + k370*[HOBr]
!
!  pr(ClONO2) =  + k326*[ClO]*[NO2]
!
!  pe(ClONO2) =  + k031 + k032 + k327*[O3P] + k364*[H2O] + k365*[HCl]
!
!  pr(HOCl) =  + k328*[ClO]*[HO2] + k364*[ClONO2]*[H2O]
!
!  pe(HOCl) =  + k030 + k329*[OH] + k330*[O3P] + k332*[Cl] +
!                k368*[HCl] + k372*[HBr]
!
!  pr(Cl2) =  + k332*[Cl]*[HOCl] + k335*[ClO]*[ClO] +
!                k336*[ClO]*[ClO] + k365*[ClONO2]*[HCl] + k368*[HOCl]*[HCl]
!
!  pe(Cl2) =  + k035
!
!  pr(H2O2) =  + k062*[HO2]*[HO2] +
!                k063*[HO2]*[HO2]*[H2O]*partH2O + 0.006*k153*[OLT]*[O3] +
!                0.011*k154*[OLI]*[O3] + 0.001*k155*[DIEN]*[O3] +
!                0.001*k156*[ISO]*[O3] + 0.02*k157*[API]*[O3] +
!                0.02*k158*[LIM]*[O3] + k301*[OH]*[OH]
!
!  pe(H2O2) =  + k010 + k061*[OH]
!
!  pr(ClNO2) =  + k331*[Cl]*[NO2] + k367*[N2O5]*[HCl]
!
!  pe(ClNO2) =  + k036
!
!  pr(HBr) =  + k315*[HCHO]*[Br] + k352*[Br]*[HO2]
!
!  pe(HBr) =  + k353*[O1D] + k355*[OH] + k371*[HOBr] + k372*[HOCl]
!
!  pr(BrONO2) =  + k356*[BrO]*[NO2]
!
!  pe(BrONO2) =  + k038 + k369*[H2O]
!
!  pr(HNO4) =  + k071*[HO2]*[NO2]
!
!  pe(HNO4) =  + k006 + k007 + k072 + k077*[OH]
!
!  pr(Cl2O2) =  + k333*[ClO]*[ClO]
!
!  pe(Cl2O2) =  + k033 + k334
!
!  pr(HOBr) =  + k357*[BrO]*[HO2] + k369*[BrONO2]*[H2O]
!
!  pe(HOBr) =  + k039 + k358*[O3P] + k370*[HCl] + k371*[HBr]
!
!  pr(BrCl) =  + k350*[BrO]*[ClO] + k370*[HOBr]*[HCl] + k372*[HOCl]*[HBr]
!
!  pe(BrCl) =  + k041
!
!  pr(HCHO) =  + 0.13*k018*[GLY] + 0.45*k019*[GLY] +
!                k024*[MACR] + k025*[HKET] + 0.05*k089*[ISO]*[O3P] +
!                0.010*k093*[HC3]*[OH] + 0.08*k117*[MACR]*[OH] +
!                k122*[OP1]*[OH] + k125*[PAA]*[OH] + k126*[PAN]*[OH] +
!                0.40*k127*[TPAN]*[OH] + k151*[TPAN]*[NO3] + k152*[ETE]*[O3] +
!                0.64*k153*[OLT]*[O3] + 0.02*k154*[OLI]*[O3] +
!                0.90*k155*[DIEN]*[O3] + 0.90*k156*[ISO]*[O3] +
!                0.04*k158*[LIM]*[O3] + 0.40*k159*[MACR]*[O3] +
!                0.70*k161*[TPAN]*[O3] + 0.047*k182*[HC3P]*[NO] +
!                0.021*k183*[HC5P]*[NO] + 1.6*k185*[ETEP]*[NO] +
!                k186*[OLTP]*[NO] + 0.606*k188*[ISOP]*[NO] +
!                0.25*k190*[LIMP]*[NO] + k195*[TCO3]*[NO] +
!                0.287*k198*[OLND]*[NO] + k221*[MO2]*[MO2] +
!                0.75*k222*[ETHP]*[MO2] + 0.810*k223*[HC3P]*[MO2] +
!                0.829*k224*[HC5P]*[MO2] + 0.753*k225*[HC8P]*[MO2] +
!                1.55*k226*[ETEP]*[MO2] + 1.25*k227*[OLTP]*[MO2] +
!                0.755*k228*[OLIP]*[MO2] + 1.09*k229*[ISOP]*[MO2] +
!                k230*[APIP]*[MO2] + 1.40*k231*[LIMP]*[MO2] + k232*[TOLP]*[MO2]
!                k233*[XYLP]*[MO2] + k234*[CSLP]*[MO2] + k235*[ACO3]*[MO2] +
!                k236*[ACO3]*[MO2] + 2.0*k237*[TCO3]*[MO2] + k238*[TCO3]*[MO2] +
!                0.75*k239*[KETP]*[MO2] + 0.75*k240*[OLNN]*[MO2] +
!                0.960*k241*[OLND]*[MO2] + 0.091*k243*[HC3P]*[ACO3] +
!                0.076*k244*[HC5P]*[ACO3] + 0.8*k246*[ETEP]*[ACO3] +
!                0.501*k247*[OLTP]*[ACO3] + 0.340*k249*[ISOP]*[ACO3] +
!                0.40*k251*[LIMP]*[ACO3] + k256*[TCO3]*[ACO3] +
!                0.207*k259*[OLND]*[ACO3] + 0.202*k261*[OLNN]*[OLND] +
!                0.504*k262*[OLND]*[OLND] + k263*[MO2]*[NO3] +
!                0.048*k265*[HC3P]*[NO3] + 0.021*k266*[HC5P]*[NO3] +
!                1.6*k268*[ETEP]*[NO3] + k269*[OLTP]*[NO3] +
!                0.686*k271*[ISOP]*[NO3] + 0.40*k273*[LIMP]*[NO3] +
!                k278*[TCO3]*[NO3] + 0.280*k281*[OLND]*[NO3] + k283*[XO2]*[MO2]
!                k308*[O1D]*[CH4] + k309*[CH3]*[O3P] + k313*[CH3O]*[O2]
!
!  pe(HCHO) =  + k011 + k012 + k111*[OH] + k129*[NO3] +
!                k311*[O3P] + k312*[Cl] + k315*[Br]
!
!  pr(MO2) =  + k013*[ALD] + k016*[PAA] + k121*[OP1]*[OH] +
!                0.19*k153*[OLT]*[O3] + 0.23*k154*[OLI]*[O3] +
!                0.03*k155*[DIEN]*[O3] + 0.03*k156*[ISO]*[O3] +
!                0.150*k182*[HC3P]*[NO] + 0.031*k183*[HC5P]*[NO] +
!                k194*[ACO3]*[NO] + 0.007*k223*[HC3P]*[MO2] +
!                0.049*k224*[HC5P]*[MO2] + k235*[ACO3]*[MO2] +
!                0.5*k242*[ETHP]*[ACO3] + 0.508*k243*[HC3P]*[ACO3] +
!                0.554*k244*[HC5P]*[ACO3] + 0.507*k245*[HC8P]*[ACO3] +
!                0.5*k246*[ETEP]*[ACO3] + 0.501*k247*[OLTP]*[ACO3] +
!                0.510*k248*[OLIP]*[ACO3] + 0.506*k249*[ISOP]*[ACO3] +
!                k250*[APIP]*[ACO3] + k251*[LIMP]*[ACO3] + k252*[TOLP]*[ACO3] +
!                k253*[XYLP]*[ACO3] + k254*[CSLP]*[ACO3] +
!                2.0*k255*[ACO3]*[ACO3] + k256*[TCO3]*[ACO3] +
!                0.50*k257*[KETP]*[ACO3] + 0.50*k258*[OLNN]*[ACO3] +
!                0.516*k259*[OLND]*[ACO3] + 0.155*k265*[HC3P]*[NO3] +
!                0.040*k266*[HC5P]*[NO3] + k277*[ACO3]*[NO3] +
!                k284*[XO2]*[ACO3] + k310*[CH3]*[O2]
!
!  pe(MO2) =  + k180*[NO] + k199*[HO2] + 2.0*k220*[MO2] +
!                2.0*k221*[MO2] + k222*[ETHP] + k223*[HC3P] + k224*[HC5P] +
!                k225*[HC8P] + k226*[ETEP] + k227*[OLTP] + k228*[OLIP] +
!                k229*[ISOP] + k230*[APIP] + k231*[LIMP] + k232*[TOLP] +
!                k233*[XYLP] + k234*[CSLP] + k235*[ACO3] + k236*[ACO3] +
!                k237*[TCO3] + k238*[TCO3] + k239*[KETP] + k240*[OLNN] +
!                k241*[OLND] + k263*[NO3] + k283*[XO2] + k314*[ClO]
!
!  pr(OP1) =  + k199*[MO2]*[HO2]
!
!  pe(OP1) =  + k014 + k121*[OH] + k122*[OH]
!
!  pr(CFC11) = 0.0
!
!  pe(CFC11) =  + k042 + k337*[O1D]
!
!  pr(CFC12) = 0.0
!
!  pe(CFC12) =  + k043 + k338*[O1D]
!
!  pr(CFC113) = 0.0
!
!  pe(CFC113) =  + k044 + k339*[O1D]
!
!  pr(CCl4) = 0.0
!
!  pe(CCl4) =  + k045 + k340*[O1D]
!
!  pr(CH3CCl3) = 0.0
!
!  pe(CH3CCl3) =  + k046 + k341*[OH]
!
!  pr(CH3Cl) = 0.0
!
!  pe(CH3Cl) =  + k047 + k342*[OH]
!
!  pr(HCFC22) = 0.0
!
!  pe(HCFC22) =  + k048 + k343*[O1D] + k344*[OH]
!
!  pr(CH3Br) = 0.0
!
!  pe(CH3Br) =  + k049 + k359*[O1D] + k360*[OH]
!
!  pr(H1211) = 0.0
!
!  pe(H1211) =  + k050 + k361*[O1D]
!
!  pr(H1301) = 0.0
!
!  pe(H1301) =  + k051 + k362*[O1D]
!
!  pr(H2SO4) = 0.0
!
!  pe(H2SO4) = 0.0
!
!  pr(PSC) = 0.0
!
!  pe(PSC) = 0.0
!
!  pr(HONO) =  + k067*[OH]*[NO] + k164*[ADDT]*[NO2] +
!                k168*[ADDX]*[NO2] + k172*[ADDC]*[NO2]
!
!  pe(HONO) =  + k004 + k075*[OH]
!
!  pr(DMS) = 0.0
!
!  pe(DMS) =  + k288*[NO3] + k289*[O3P] + k290*[OH]
!
!  pr(SO2) =  + k288*[DMS]*[NO3] + k289*[DMS]*[O3P] + 0.8*k290*[DMS]*[OH]
!
!  pe(SO2) =  + k087*[OH]
!
!  pr(ETH) =  + 0.03*k153*[OLT]*[O3] + 0.06*k154*[OLI]*[O3]
!
!  pe(ETH) =  + k092*[OH]
!
!  pr(HC3) = 0.0
!
!  pe(HC3) =  + k093*[OH]
!
!  pr(HC5) = 0.0
!
!  pe(HC5) =  + k094*[OH] + k095*[OH]
!
!  pr(HC8) = 0.0
!
!  pe(HC8) =  + k096*[OH]
!
!  pr(ETE) = 0.0
!
!  pe(ETE) =  + k097*[OH] + k136*[NO3] + k137*[NO3] + k152*[O3]
!
!  pr(OLT) =  + 0.86*k089*[ISO]*[O3P] + 0.35*k155*[DIEN]*[O3] +
!                0.35*k156*[ISO]*[O3] + 0.46*k158*[LIM]*[O3] +
!                0.354*k188*[ISOP]*[NO] + 0.370*k229*[ISOP]*[MO2] +
!                0.229*k249*[ISOP]*[ACO3] + 0.400*k271*[ISOP]*[NO3]
!
!  pe(OLT) =  + k098*[OH] + k138*[NO3] + k139*[NO3] + k153*[O3]
!
!  pr(OLI) =  + 0.25*k190*[LIMP]*[NO] + 0.080*k229*[ISOP]*[MO2] +
!                0.40*k231*[LIMP]*[MO2] + 0.40*k251*[LIMP]*[ACO3] +
!                0.40*k273*[LIMP]*[NO3]
!
!  pe(OLI) =  + k099*[OH] + k140*[NO3] + k141*[NO3] + k154*[O3]
!
!  pr(DIEN) = 0.0
!
!  pe(DIEN) =  + k100*[OH] + k142*[NO3] + k143*[NO3] + k155*[O3]
!
!  pr(ISO) = 0.0
!
!  pe(ISO) =  + k089*[O3P] + k101*[OH] + k144*[NO3] + k145*[NO3] + k156*[O3]
!
!  pr(API) = 0.0
!
!  pe(API) =  + k102*[OH] + k146*[NO3] + k147*[NO3] + k157*[O3]
!
!  pr(LIM) = 0.0
!
!  pe(LIM) =  + k103*[OH] + k148*[NO3] + k149*[NO3] + k158*[O3]
!
!  pr(TOL) = 0.0
!
!  pe(TOL) =  + k104*[OH] + k105*[OH]
!
!  pr(XYL) = 0.0
!
!  pe(XYL) =  + k106*[OH] + k107*[OH]
!
!  pr(CSL) =  + 0.10*k162*[PHO]*[NO2] + k163*[PHO]*[HO2] +
!                k164*[ADDT]*[NO2] + k166*[ADDT]*[O2] + k167*[ADDT]*[O3] +
!                k168*[ADDX]*[NO2] + k170*[ADDX]*[O2] + k171*[ADDX]*[O3] +
!                k172*[ADDC]*[NO2] + k174*[ADDC]*[O2] + k175*[ADDC]*[O3]
!
!  pe(CSL) =  + k108*[OH] + k109*[OH] + k110*[OH] + k135*[NO3]
!
!  pr(ALD) =  + k015*[OP2] + k022*[ONIT] + k090*[MACR]*[O3P] +
!                0.335*k093*[HC3]*[OH] + 0.025*k096*[HC8]*[OH] +
!                k119*[OH]*[UDD] + 0.08*k123*[OP2]*[OH] + 0.25*k134*[DCB]*[NO3]
!                0.44*k153*[OLT]*[O3] + 0.99*k154*[OLI]*[O3] +
!                0.65*k157*[API]*[O3] + 0.16*k160*[DCB]*[O3] + k181*[ETHP]*[NO]
!                0.233*k182*[HC3P]*[NO] + 0.211*k183*[HC5P]*[NO] +
!                0.150*k184*[HC8P]*[NO] + 0.2*k185*[ETEP]*[NO] +
!                0.94*k186*[OLTP]*[NO] + 1.71*k187*[OLIP]*[NO] +
!                0.80*k189*[APIP]*[NO] + 0.46*k196*[KETP]*[NO] +
!                1.24*k198*[OLND]*[NO] + 0.75*k222*[ETHP]*[MO2] +
!                0.580*k223*[HC3P]*[MO2] + 0.523*k224*[HC5P]*[MO2] +
!                0.411*k225*[HC8P]*[MO2] + 0.35*k226*[ETEP]*[MO2] +
!                0.669*k227*[OLTP]*[MO2] + 0.932*k228*[OLIP]*[MO2] +
!                k230*[APIP]*[MO2] + 0.30*k239*[KETP]*[MO2] +
!                0.640*k241*[OLND]*[MO2] + k242*[ETHP]*[ACO3] +
!                0.724*k243*[HC3P]*[ACO3] + 0.677*k244*[HC5P]*[ACO3] +
!                0.497*k245*[HC8P]*[ACO3] + 0.6*k246*[ETEP]*[ACO3] +
!                0.859*k247*[OLTP]*[ACO3] + 0.941*k248*[OLIP]*[ACO3] +
!                k250*[APIP]*[ACO3] + 0.35*k257*[KETP]*[ACO3] +
!                0.650*k259*[OLND]*[ACO3] + 0.640*k261*[OLNN]*[OLND] +
!                1.21*k262*[OLND]*[OLND] + k264*[ETHP]*[NO3] +
!                0.243*k265*[HC3P]*[NO3] + 0.239*k266*[HC5P]*[NO3] +
!                0.187*k267*[HC8P]*[NO3] + 0.2*k268*[ETEP]*[NO3] +
!                0.94*k269*[OLTP]*[NO3] + 1.71*k270*[OLIP]*[NO3] +
!                k272*[APIP]*[NO3] + 0.46*k279*[KETP]*[NO3] +
!                1.24*k281*[OLND]*[NO3]
!
!  pe(ALD) =  + k013 + k112*[OH] + k130*[NO3]
!
!  pr(KET) =  + k023*[ONIT] + k095*[HC5]*[OH] + k120*[OH]*[UDD] +
!                0.41*k123*[OP2]*[OH] + 0.03*k134*[DCB]*[NO3] +
!                0.03*k153*[OLT]*[O3] + 0.16*k154*[OLI]*[O3] +
!                0.53*k157*[API]*[O3] + 0.623*k182*[HC3P]*[NO] +
!                0.722*k183*[HC5P]*[NO] + 0.642*k184*[HC8P]*[NO] +
!                0.06*k186*[OLTP]*[NO] + 0.29*k187*[OLIP]*[NO] +
!                0.80*k189*[APIP]*[NO] + 0.464*k198*[OLND]*[NO] +
!                0.018*k223*[HC3P]*[MO2] + 0.240*k224*[HC5P]*[MO2] +
!                0.419*k225*[HC8P]*[MO2] + 0.081*k227*[OLTP]*[MO2] +
!                0.313*k228*[OLIP]*[MO2] + k230*[APIP]*[MO2] +
!                0.149*k241*[OLND]*[MO2] + 0.127*k243*[HC3P]*[ACO3] +
!                0.330*k244*[HC5P]*[ACO3] + 0.581*k245*[HC8P]*[ACO3] +
!                0.141*k247*[OLTP]*[ACO3] + 0.569*k248*[OLIP]*[ACO3] +
!                k250*[APIP]*[ACO3] + 0.11*k257*[KETP]*[ACO3] +
!                0.167*k259*[OLND]*[ACO3] + 0.149*k261*[OLNN]*[OLND] +
!                0.285*k262*[OLND]*[OLND] + 0.670*k265*[HC3P]*[NO3] +
!                0.828*k266*[HC5P]*[NO3] + 0.880*k267*[HC8P]*[NO3] +
!                0.06*k269*[OLTP]*[NO3] + 0.29*k270*[OLIP]*[NO3] +
!                k272*[APIP]*[NO3] + 0.469*k281*[OLND]*[NO3]
!
!  pe(KET) =  + k017 + k113*[OH]
!
!  pr(GLY) =  + 0.036*k093*[HC3]*[OH] + 0.15*k118*[OH]*[DCB] +
!                0.25*k134*[DCB]*[NO3] + 0.50*k160*[DCB]*[O3] +
!                0.063*k182*[HC3P]*[NO] + 1.20*k191*[TOLP]*[NO] +
!                0.35*k192*[XYLP]*[NO] + k193*[CSLP]*[NO] +
!                0.119*k223*[HC3P]*[MO2] + 0.65*k232*[TOLP]*[MO2] +
!                0.37*k233*[XYLP]*[MO2] + k234*[CSLP]*[MO2] +
!                0.100*k243*[HC3P]*[ACO3] + 0.65*k252*[TOLP]*[ACO3] +
!                0.37*k253*[XYLP]*[ACO3] + k254*[CSLP]*[ACO3] +
!                0.063*k265*[HC3P]*[NO3] + 1.30*k274*[TOLP]*[NO3] +
!                0.74*k275*[XYLP]*[NO3] + k276*[CSLP]*[NO3]
!
!  pe(GLY) =  + k018 + k019 + k115*[OH] + k131*[NO3]
!
!  pr(MGLY) =  + k114*[HKET]*[OH] + 0.08*k117*[MACR]*[OH] +
!                0.15*k118*[OH]*[DCB] + 0.25*k134*[DCB]*[NO3] +
!                0.60*k159*[MACR]*[O3] + 0.62*k160*[DCB]*[O3] +
!                0.65*k191*[TOLP]*[NO] + 0.60*k192*[XYLP]*[NO] +
!                k193*[CSLP]*[NO] + 0.54*k196*[KETP]*[NO] +
!                0.005*k223*[HC3P]*[MO2] + 0.35*k232*[TOLP]*[MO2] +
!                0.63*k233*[XYLP]*[MO2] + k234*[CSLP]*[MO2] +
!                0.40*k239*[KETP]*[MO2] + 0.004*k243*[HC3P]*[ACO3] +
!                0.35*k252*[TOLP]*[ACO3] + 0.63*k253*[XYLP]*[ACO3] +
!                k254*[CSLP]*[ACO3] + 0.54*k257*[KETP]*[ACO3] +
!                0.70*k274*[TOLP]*[NO3] + 1.26*k275*[XYLP]*[NO3] +
!                k276*[CSLP]*[NO3] + 0.54*k279*[KETP]*[NO3]
!
!  pe(MGLY) =  + k020 + k116*[OH] + k132*[NO3]
!
!  pr(DCB) =  + 0.13*k089*[ISO]*[O3P] + 0.50*k191*[TOLP]*[NO] +
!                0.95*k192*[XYLP]*[NO] + k232*[TOLP]*[MO2] + k233*[XYLP]*[MO2] +
!                k252*[TOLP]*[ACO3] + k253*[XYLP]*[ACO3] +
!                0.50*k274*[TOLP]*[NO3] + k275*[XYLP]*[NO3]
!
!  pe(DCB) =  + k021 + k118*[OH] + k134*[NO3] + k160*[O3]
!
!  pr(MACR) =  + k142*[DIEN]*[NO3] + k144*[ISO]*[NO3] +
!                0.39*k155*[DIEN]*[O3] + 0.39*k156*[ISO]*[O3] +
!                0.79*k158*[LIM]*[O3] + 0.446*k188*[ISOP]*[NO] +
!                0.40*k190*[LIMP]*[NO] + 0.550*k229*[ISOP]*[MO2] +
!                0.60*k231*[LIMP]*[MO2] + 0.771*k249*[ISOP]*[ACO3] +
!                0.60*k251*[LIMP]*[ACO3] + 0.600*k271*[ISOP]*[NO3] +
!                0.60*k273*[LIMP]*[NO3]
!
!  pe(MACR) =  + k024 + k090*[O3P] + k117*[OH] + k133*[NO3] + k159*[O3]
!
!  pr(UDD) =  + 0.35*k118*[OH]*[DCB]
!
!  pe(UDD) =  + k119*[OH] + k120*[OH]
!
!  pr(HKET) =  + 0.024*k096*[HC8]*[OH] + 0.41*k117*[MACR]*[OH] +
!                0.60*k127*[TPAN]*[OH] + 0.30*k239*[KETP]*[MO2]
!
!  pe(HKET) =  + k025 + k114*[OH]
!
!  pr(ONIT) =  + k150*[TPAN]*[NO3] + k162*[PHO]*[NO2] +
!                0.059*k182*[HC3P]*[NO] + 0.124*k183*[HC5P]*[NO] +
!                0.261*k184*[HC8P]*[NO] + 0.153*k188*[ISOP]*[NO] +
!                0.20*k189*[APIP]*[NO] + 0.35*k190*[LIMP]*[NO] +
!                0.05*k191*[TOLP]*[NO] + 0.05*k192*[XYLP]*[NO] +
!                k197*[OLNN]*[NO] + k218*[OLNN]*[HO2] + k219*[OLND]*[HO2] +
!                k240*[OLNN]*[MO2] + 0.500*k241*[OLND]*[MO2] +
!                k258*[OLNN]*[ACO3] + 0.484*k259*[OLND]*[ACO3] +
!                2.0*k260*[OLNN]*[OLNN] + 1.50*k261*[OLNN]*[OLND] +
!                k262*[OLND]*[OLND] + k280*[OLNN]*[NO3]
!
!  pe(ONIT) =  + k022 + k023 + k128*[OH]
!
!  pr(PAN) =  + 0.40*k127*[TPAN]*[OH] + k151*[TPAN]*[NO3] +
!                0.30*k161*[TPAN]*[O3] + k176*[ACO3]*[NO2]
!
!  pe(PAN) =  + k126*[OH] + k177
!
!  pr(TPAN) =  + k178*[TCO3]*[NO2]
!
!  pe(TPAN) =  + k127*[OH] + k150*[NO3] + k151*[NO3] + k161*[O3] + k179
!
!  pr(OP2) =  + 0.13*k159*[MACR]*[O3] + k200*[ETHP]*[HO2] +
!                k201*[HC3P]*[HO2] + k202*[HC5P]*[HO2] + k203*[HC8P]*[HO2] +
!                k204*[ETEP]*[HO2] + k205*[OLTP]*[HO2] + k206*[OLIP]*[HO2] +
!                k207*[ISOP]*[HO2] + k208*[APIP]*[HO2] + k209*[LIMP]*[HO2] +
!                k210*[TOLP]*[HO2] + k211*[XYLP]*[HO2] + k212*[CSLP]*[HO2] +
!                k215*[TCO3]*[HO2] + k217*[KETP]*[HO2] + k282*[XO2]*[HO2]
!
!  pe(OP2) =  + k015 + k123*[OH]
!
!  pr(PAA) =  + 0.11*k160*[DCB]*[O3] + k213*[ACO3]*[HO2]
!
!  pe(PAA) =  + k016 + k124*[OH] + k125*[OH]
!
!  pr(ETHP) =  + k017*[KET] + k092*[ETH]*[OH] +
!                0.10*k153*[OLT]*[O3] + 0.18*k154*[OLI]*[O3] +
!                0.20*k157*[API]*[O3] + 0.16*k158*[LIM]*[O3] +
!                0.048*k182*[HC3P]*[NO] + 0.245*k183*[HC5P]*[NO] +
!                0.133*k184*[HC8P]*[NO] + 0.014*k224*[HC5P]*[MO2] +
!                0.013*k225*[HC8P]*[MO2] + 0.006*k243*[HC3P]*[ACO3] +
!                0.018*k244*[HC5P]*[ACO3] + 0.015*k245*[HC8P]*[ACO3] +
!                0.053*k265*[HC3P]*[NO3] + 0.262*k266*[HC5P]*[NO3] +
!                0.155*k267*[HC8P]*[NO3]
!
!  pe(ETHP) =  + k181*[NO] + k200*[HO2] + k222*[MO2] + k242*[ACO3] + k264*[NO3]
!
!  pr(HC3P) =  + 0.583*k093*[HC3]*[OH] + 0.44*k123*[OP2]*[OH] + k128*[ONIT]*[OH]
!
!  pe(HC3P) =  + k182*[NO] + k201*[HO2] + k223*[MO2] + k243*[ACO3] + k265*[NO3]
!
!  pr(HC5P) =  + k094*[HC5]*[OH]
!
!  pe(HC5P) =  + k183*[NO] + k202*[HO2] + k224*[MO2] + k244*[ACO3] + k266*[NO3]
!
!  pr(HC8P) =  + 0.951*k096*[HC8]*[OH]
!
!  pe(HC8P) =  + k184*[NO] + k203*[HO2] + k225*[MO2] + k245*[ACO3] + k267*[NO3]
!
!  pr(ETEP) =  + k097*[ETE]*[OH]
!
!  pe(ETEP) =  + k185*[NO] + k204*[HO2] + k226*[MO2] + k246*[ACO3] + k268*[NO3]
!
!  pr(OLTP) =  + k098*[OLT]*[OH]
!
!  pe(OLTP) =  + k186*[NO] + k205*[HO2] + k227*[MO2] + k247*[ACO3] + k269*[NO3]
!
!  pr(OLIP) =  + k099*[OLI]*[OH]
!
!  pe(OLIP) =  + k187*[NO] + k206*[HO2] + k228*[MO2] + k248*[ACO3] + k270*[NO3]
!
!  pr(ISOP) =  + k100*[DIEN]*[OH] + k101*[ISO]*[OH]
!
!  pe(ISOP) =  + k188*[NO] + k207*[HO2] + k229*[MO2] + k249*[ACO3] + k271*[NO3]
!
!  pr(APIP) =  + k102*[API]*[OH]
!
!  pe(APIP) =  + k189*[NO] + k208*[HO2] + k230*[MO2] + k250*[ACO3] + k272*[NO3]
!
!  pr(LIMP) =  + k103*[LIM]*[OH]
!
!  pe(LIMP) =  + k190*[NO] + k209*[HO2] + k231*[MO2] + k251*[ACO3] + k273*[NO3]
!
!  pr(PHO) =  + k109*[CSL]*[OH] + k135*[CSL]*[NO3]
!
!  pe(PHO) =  + k162*[NO2] + k163*[HO2]
!
!  pr(TOLP) =  + k165*[ADDT]*[O2]
!
!  pe(TOLP) =  + k191*[NO] + k210*[HO2] + k232*[MO2] + k252*[ACO3] + k274*[NO3]
!
!  pr(XYLP) =  + k169*[ADDX]*[O2]
!
!  pe(XYLP) =  + k192*[NO] + k211*[HO2] + k233*[MO2] + k253*[ACO3] + k275*[NO3]
!
!  pr(CSLP) =  + k173*[ADDC]*[O2]
!
!  pe(CSLP) =  + k193*[NO] + k212*[HO2] + k234*[MO2] + k254*[ACO3] + k276*[NO3]
!
!  pr(ACO3) =  + k017*[KET] + k020*[MGLY] + k024*[MACR] +
!                k025*[HKET] + k112*[ALD]*[OH] + k116*[MGLY]*[OH] +
!                k124*[PAA]*[OH] + k130*[ALD]*[NO3] + k132*[MGLY]*[NO3] +
!                0.15*k155*[DIEN]*[O3] + 0.15*k156*[ISO]*[O3] +
!                0.13*k159*[MACR]*[O3] + 0.28*k160*[DCB]*[O3] +
!                0.70*k161*[TPAN]*[O3] + k177*[PAN] + k195*[TCO3]*[NO] +
!                0.23*k196*[KETP]*[NO] + k237*[TCO3]*[MO2] +
!                0.12*k239*[KETP]*[MO2] + k256*[TCO3]*[ACO3] +
!                0.12*k257*[KETP]*[ACO3] + k278*[TCO3]*[NO3] +
!                0.23*k279*[KETP]*[NO3]
!
!  pe(ACO3) =  + k176*[NO2] + k194*[NO] + k213*[HO2] +
!                k214*[HO2] + k235*[MO2] + k236*[MO2] + k242*[ETHP] +
!                k243*[HC3P] + k244*[HC5P] + k245*[HC8P] + k246*[ETEP] +
!                k247*[OLTP] + k248*[OLIP] + k249*[ISOP] + k250*[APIP] +
!                k251*[LIMP] + k252*[TOLP] + k253*[XYLP] + k254*[CSLP] +
!                2.0*k255*[ACO3] + k256*[TCO3] + k257*[KETP] + k258*[OLNN] +
!                k259*[OLND] + k277*[NO3] + k284*[XO2]
!
!  pr(TCO3) =  + k021*[DCB] + 0.51*k117*[MACR]*[OH] +
!                0.50*k118*[OH]*[DCB] + 0.20*k133*[MACR]*[NO3] +
!                0.50*k134*[DCB]*[NO3] + k179*[TPAN]
!
!  pe(TCO3) =  + k178*[NO2] + k195*[NO] + k215*[HO2] +
!                k216*[HO2] + k237*[MO2] + k238*[MO2] + k256*[ACO3] + k278*[NO3]
!
!  pr(KETP) =  + k113*[KET]*[OH] + 0.03*k153*[OLT]*[O3] +
!                0.12*k154*[OLI]*[O3] + 0.02*k155*[DIEN]*[O3] +
!                0.02*k156*[ISO]*[O3] + 0.42*k157*[API]*[O3] +
!                0.42*k158*[LIM]*[O3]
!
!  pe(KETP) =  + k196*[NO] + k217*[HO2] + k239*[MO2] + k257*[ACO3] + k279*[NO3]
!
!  pr(OLNN) =  + 0.80*k133*[MACR]*[NO3] + k136*[ETE]*[NO3] +
!                k138*[OLT]*[NO3] + k140*[OLI]*[NO3] + k142*[DIEN]*[NO3] +
!                k144*[ISO]*[NO3] + k146*[API]*[NO3] + k148*[LIM]*[NO3]
!
!  pe(OLNN) =  + k197*[NO] + k218*[HO2] + k240*[MO2] +
!                k258*[ACO3] + 2.0*k260*[OLNN] + k261*[OLND] + k280*[NO3]
!
!  pr(OLND) =  + k137*[ETE]*[NO3] + k139*[OLT]*[NO3] +
!                k141*[OLI]*[NO3] + k143*[DIEN]*[NO3] + k145*[ISO]*[NO3] +
!                k147*[API]*[NO3] + k149*[LIM]*[NO3]
!
!  pe(OLND) =  + k198*[NO] + k219*[HO2] + k241*[MO2] +
!                k259*[ACO3] + k261*[OLNN] + 2.0*k262*[OLND] + k281*[NO3]
!
!  pr(XO2) =  + 0.15*k089*[ISO]*[O3P] + k105*[TOL]*[OH] +
!                k107*[XYL]*[OH] + k110*[CSL]*[OH] + 0.49*k117*[MACR]*[OH] +
!                0.50*k118*[OH]*[DCB] + 0.07*k123*[OP2]*[OH] + k125*[PAA]*[OH] +
!                k126*[PAN]*[OH] + k127*[TPAN]*[OH] + 0.50*k134*[DCB]*[NO3] +
!                k150*[TPAN]*[NO3] + k151*[TPAN]*[NO3] + 0.13*k155*[DIEN]*[O3] +
!                0.13*k156*[ISO]*[O3] + 0.048*k182*[HC3P]*[NO] +
!                0.334*k183*[HC5P]*[NO] + 0.416*k184*[HC8P]*[NO] +
!                0.16*k196*[KETP]*[NO] + 0.085*k223*[HC3P]*[MO2] +
!                0.245*k224*[HC5P]*[MO2] + 0.322*k225*[HC8P]*[MO2] +
!                0.08*k239*[KETP]*[MO2] + 0.071*k243*[HC3P]*[ACO3] +
!                0.237*k244*[HC5P]*[ACO3] + 0.318*k245*[HC8P]*[ACO3] +
!                0.08*k257*[KETP]*[ACO3] + 0.051*k265*[HC3P]*[NO3] +
!                0.391*k266*[HC5P]*[NO3] + 0.587*k267*[HC8P]*[NO3] +
!                0.16*k279*[KETP]*[NO3]
!
!  pe(XO2) =  + k282*[HO2] + k283*[MO2] + k284*[ACO3] +
!                2.0*k285*[XO2] + k286*[NO] + k287*[NO3]
!
!  pr(SULF) =  + k087*[OH]*[SO2]
!
!  pe(SULF) = 0.0
!
!  pr(Ox) = 0.0
!              + k001*[NO2] + k009*[NO3] + 2.0*k026*[O2] +
!                k037*[BrO] + k040*[OClO] + k052*[NO] + k298*[H]*[HO2] +
!                k300*[OH]*[OH] + k302*[N]*[NO] + k303*[N]*[O2] + k029*[N2O] +
!                k214*[ACO3]*[HO2] + k216*[TCO3]*[HO2]
!
!  pe(Ox) =  + k317*[ClO]*[O3P] +
!                2.0*k333*[ClO]*[ClO]*k033/(k033 + k334) +
!                min(k030*[HOCl],k328*[HO2]*[ClO]) +
!                2.0*min(k041*[BrCl],k350*[BrO]*[ClO]) +
!                2.0*k349*[BrO]*[ClO]*day + min(k039*[HOBr],k357*[HO2]*[BrO]) +
!                2.0*k351*[BrO]*[BrO]*day + 2.0*(k032 + k031)*[ClONO2]
!              + 2.0*k054*[O3P]*[O3] + k064*[O3P]*[NO] +
!                k065*[O3P]*[NO2] + k066*[O3P]*[NO2] + k089*[ISO]*[O3P] +
!                k090*[MACR]*[O3P] + 0.910000*k155*[DIEN]*[O3] +
!                0.910000*k156*[ISO]*[O3] + k289*[DMS]*[O3P] + k294*[O3P]*[OH] +
!                k295*[O3P]*[HO2] + k299*[H2]*[O3P] + k309*[CH3]*[O3P] +
!                k311*[HCHO]*[O3P] + k317*[ClO]*[O3P] + k327*[O3P]*[ClONO2] +
!                k330*[O3P]*[HOCl] + k346*[BrO]*[O3P] + k358*[HOBr]*[O3P] +
!                2.0*k363*[O3P]*[O3P] + k057*[O1D]*[H2O] + k293*[O1D]*[H2] +
!                k304*[O1D]*[N2O] + k305*[O1D]*[N2O] + k306*[O1D]*[CH4] +
!                k307*[O1D]*[CH4] + k308*[O1D]*[CH4] + k337*[O1D]*[CFC11] +
!                k338*[O1D]*[CFC12] + k339*[O1D]*[CFC113] + k340*[O1D]*[CCl4] +
!                k343*[O1D]*[HCFC22] + k353*[O1D]*[HBr] + k359*[O1D]*[CH3Br] +
!                k361*[O1D]*[H1211] + k362*[O1D]*[H1301] + k058*[O3]*[OH] +
!                k059*[O3]*[HO2] + k078*[O3]*[NO] + k079*[O3]*[NO2] +
!                k152*[ETE]*[O3] + k153*[OLT]*[O3] + k154*[OLI]*[O3] +
!                k157*[API]*[O3] + k158*[LIM]*[O3] + k159*[MACR]*[O3] +
!                k160*[DCB]*[O3] + k161*[TPAN]*[O3] + k167*[ADDT]*[O3] +
!                k171*[ADDX]*[O3] + k175*[ADDC]*[O3] + k292*[O3]*[H] +
!                k316*[Cl]*[O3] + k345*[Br]*[O3]
!
!  pr(NOx) =  + 2.0*k027*[N2O5] + k327*[O3P]*[ClONO2]
!              + k004*[HONO] + 2.0*k305*[O1D]*[N2O] +
!                k005*[HNO3]*partHNO3 + k006*[HNO4] + k022*[ONIT] + k023*[ONIT]
!                k032*[ClONO2] + k036*[ClNO2] + k072*[HNO4] + k075*[OH]*[HONO] +
!                k077*[OH]*[HNO4] + 2.0*k084*[N2O5] + k128*[ONIT]*[OH] +
!                0.70*k161*[TPAN]*[O3] + k177*[PAN] + k179*[TPAN] +
!                1.000000*k198*[OLND]*[NO] + 0.500*k241*[OLND]*[MO2] +
!                0.516*k259*[OLND]*[ACO3] + 0.500*k261*[OLNN]*[OLND] +
!                k262*[OLND]*[OLND] + 1.000000*k281*[OLND]*[NO3] + k007*[HNO4] +
!                k031*[ClONO2] + k038*[BrONO2] + k076*[OH]*[HNO3]*partHNO3 +
!                k126*[PAN]*[OH] + 0.60*k127*[TPAN]*[OH]
!
!  pe(NOx) = 0.0
!              + 2.0*k302*[N]*[NO] + k067*[OH]*[NO] +
!                k068*[OH]*[NO2] + k071*[HO2]*[NO2] + 2.0*k083*[NO3]*[NO2] +
!                0.500000*k134*[DCB]*[NO3] + k162*[PHO]*[NO2] +
!                k164*[ADDT]*[NO2] + k168*[ADDX]*[NO2] + k172*[ADDC]*[NO2] +
!                k176*[ACO3]*[NO2] + k178*[TCO3]*[NO2] +
!                0.059000*k182*[HC3P]*[NO] + 0.124000*k183*[HC5P]*[NO] +
!                0.261000*k184*[HC8P]*[NO] + 0.153000*k188*[ISOP]*[NO] +
!                0.200000*k189*[APIP]*[NO] + 0.350000*k190*[LIMP]*[NO] +
!                0.050000*k191*[TOLP]*[NO] + 0.050000*k192*[XYLP]*[NO] +
!                k326*[ClO]*[NO2] + k331*[Cl]*[NO2] + k356*[BrO]*[NO2] +
!                k073*[HO2]*[NO3] + k129*[HCHO]*[NO3] + k130*[ALD]*[NO3] +
!                k131*[GLY]*[NO3] + k132*[MGLY]*[NO3] + k133*[MACR]*[NO3] +
!                k135*[CSL]*[NO3] + k136*[ETE]*[NO3] + k137*[ETE]*[NO3] +
!                k138*[OLT]*[NO3] + k139*[OLT]*[NO3] + k140*[OLI]*[NO3] +
!                k141*[OLI]*[NO3] + k142*[DIEN]*[NO3] + k143*[DIEN]*[NO3] +
!                k144*[ISO]*[NO3] + k145*[ISO]*[NO3] + k146*[API]*[NO3] +
!                k147*[API]*[NO3] + k148*[LIM]*[NO3] + k149*[LIM]*[NO3]
!
!  pr(ClOx) =  + 3.0*k042*[CFC11] + 2.0*k043*[CFC12] +
!                3.0*k044*[CFC113] + 4.0*k045*[CCl4] + 3.0*k046*[CH3CCl3] +
!                k047*[CH3Cl] + k048*[HCFC22] + k327*[O3P]*[ClONO2] +
!                3.0*k337*[O1D]*[CFC11] + 2.0*k338*[O1D]*[CFC12] +
!                3.0*k339*[O1D]*[CFC113] + 4.0*k340*[O1D]*[CCl4] +
!                3.0*k341*[OH]*[CH3CCl3] + k342*[OH]*[CH3Cl] +
!                k343*[O1D]*[HCFC22] + k344*[OH]*[HCFC22]
!              + k040*[OClO] + k034*[HCl]*partHCl +
!                2.0*k035*[Cl2] + k036*[ClNO2] + k325*[OH]*[HCl]*partHCl +
!                k031*[ClONO2] + k032*[ClONO2] + k030*[HOCl] + k329*[OH]*[HOCl]
!                k330*[O3P]*[HOCl] + 2.0*k033*[Cl2O2] + 2.0*k334*[Cl2O2] +
!                k041*[BrCl]
!
!  pe(ClOx) = 0.0
!              + k324*[ClO]*[OH] + 2.0*k335*[ClO]*[ClO] +
!                2.0*k336*[ClO]*[ClO] + k348*[BrO]*[ClO] + k312*[HCHO]*[Cl] +
!                k319*[Cl]*[CH4] + k320*[Cl]*[H2] + k321*[Cl]*[HO2] +
!                k331*[Cl]*[NO2] + k332*[Cl]*[HOCl] + k326*[ClO]*[NO2] +
!                k328*[ClO]*[HO2] + 2.0*k333*[ClO]*[ClO] + k350*[BrO]*[ClO]
!
!  pr(BrOx) =  + k049*[CH3Br] + k050*[H1211] + k051*[H1301] +
!                k359*[O1D]*[CH3Br] + k361*[O1D]*[H1211] + k362*[O1D]*[H1301] +
!                2.0*k371*[HOBr]*[HBr]
!              + k353*[O1D]*[HBr]*partHBr +
!                k355*[OH]*[HBr]*partHBr + k038*[BrONO2] + k039*[HOBr] +
!                k358*[HOBr]*[O3P] + k041*[BrCl]
!
!  pe(BrOx) = 0.0
!              + k315*[HCHO]*[Br] + k352*[Br]*[HO2] +
!                k356*[BrO]*[NO2] + k357*[BrO]*[HO2] + k350*[BrO]*[ClO]
!
!
#ifdef MPI
i1 = i_deb(myid) ; i2 = i_fin(myid)
#else
i1 = 1           ; i2 = imax
#endif
!
!$omp parallel default(none) &
!$omp private (i,j) &
!$omp shared  (i1,i2,ntra,k,jco2,hnm,day,QM,HM,pr,pe) &
!$omp shared  (partHNO3,partH2O,partHCl,partHBr,imax)
!
!$omp do
do j=1,ntra
do i = i1,i2
  pr(i,j) = 0.0
  pe(i,j) = 0.0
enddo
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  pe(i,1) =  + HM(i,2)*( + k(i,304) + k(i,305) ) + k(i,29)
!
  pr(i,2) =  + HM(i,3)*( + 0.06*k(i,153)*QM(i,43) + 0.07*k(i,154)*QM(i,44) )
!
  pe(i,2) =  + HM(i,2)*( + k(i,306) + k(i,307) + k(i,308) ) +                &
    k(i,91)*HM(i,13) + k(i,319)*HM(i,9)
!
  pe(i,4) = ( + k(i,5) + k(i,76)*HM(i,13))*partHNO3(i)
!
  pr(i,5) =  + k(i,83)*HM(i,7)*HM(i,6)
!
  pe(i,5) =  + k(i,27) + k(i,84) + k(i,366)*QM(i,3) + k(i,367)*QM(i,8)
!
  pe(i,6) =  + k(i,88)*HM(i,13)
!
  pr(i,7) =  + k(i,348)*HM(i,10)*HM(i,8)
!
  pe(i,7) =  + k(i,40)
!
  pr(i,8) =  + HM(i,9)*( + k(i,312)*QM(i,20) + k(i,319)*QM(i,2) +           &
    k(i,320)*5.e-7*hnm(i) + k(i,321)*HM(i,14) ) + k(i,324)*HM(i,8)*HM(i,13)
!
  pr(i,9) =  + k(i,326)*HM(i,8)*HM(i,6)
!
  pe(i,9) =  + k(i,31) + k(i,32) + k(i,327)*HM(i,1) + k(i,364)*QM(i,3) +     &
    k(i,365)*QM(i,8)
!
  pr(i,10) =  + k(i,328)*HM(i,8)*HM(i,14) + k(i,364)*QM(i,9)*QM(i,3)
!
  pr(i,11) =  + QM(i,10)*( + k(i,332)*HM(i,9) +                              &
    k(i,368)*QM(i,8) ) + HM(i,8)*( + k(i,335)*HM(i,8) +                      &
    k(i,336)*HM(i,8) ) + k(i,365)*QM(i,9)*QM(i,8)
!
  pe(i,11) =  + k(i,35)
!
  pe(i,12) =  + k(i,10) + k(i,61)*HM(i,13)
!
  pr(i,13) =  + k(i,331)*HM(i,9)*HM(i,6) + k(i,367)*QM(i,5)*QM(i,8)
!
  pe(i,13) =  + k(i,36)
!
  pr(i,14) =  + HM(i,11)*( + k(i,315)*QM(i,20) + k(i,352)*HM(i,14) )
!
  pe(i,14) = ( + k(i,353)*HM(i,2) + k(i,355)*HM(i,13) )*partHBr(i) +        &
     k(i,371)*QM(i,18) + k(i,372)*QM(i,10)
!
  pr(i,15) =  + k(i,356)*HM(i,10)*HM(i,6)
!
  pe(i,15) =  + k(i,38) + k(i,369)*QM(i,3)
!
  pr(i,16) =  + k(i,71)*HM(i,14)*HM(i,6)
!
  pe(i,16) =  + k(i,6) + k(i,7) + k(i,72) + k(i,77)*HM(i,13)
!
  pr(i,17) =  + k(i,333)*HM(i,8)*HM(i,8)
!
  pe(i,17) =  + k(i,33) + k(i,334)
!
  pr(i,18) =  + k(i,357)*HM(i,10)*HM(i,14) + k(i,369)*QM(i,15)*QM(i,3)
!
  pe(i,18) =  + k(i,39) + k(i,358)*HM(i,1) + k(i,370)*QM(i,8) +              &
    k(i,371)*QM(i,14)
!
  pr(i,19) =  + k(i,350)*HM(i,10)*HM(i,8) + k(i,370)*QM(i,18)*QM(i,8) +      &
    k(i,372)*QM(i,10)*QM(i,14)
!
  pe(i,19) =  + k(i,41)
!
  pr(i,22) =  + k(i,199)*QM(i,21)*HM(i,14)
!
  pe(i,22) =  + HM(i,13)*( + k(i,121) + k(i,122) ) + k(i,14)
!
  pe(i,23) =  + k(i,42) + k(i,337)*HM(i,2)
!
  pe(i,24) =  + k(i,43) + k(i,338)*HM(i,2)
!
  pe(i,25) =  + k(i,44) + k(i,339)*HM(i,2)
!
  pe(i,26) =  + k(i,45) + k(i,340)*HM(i,2)
!
  pe(i,27) =  + k(i,46) + k(i,341)*HM(i,13)
!
  pe(i,28) =  + k(i,47) + k(i,342)*HM(i,13)
!
  pe(i,29) =  + k(i,48) + k(i,343)*HM(i,2) + k(i,344)*HM(i,13)
!
  pe(i,30) =  + k(i,49) + k(i,359)*HM(i,2) + k(i,360)*HM(i,13)
!
  pe(i,31) =  + k(i,50) + k(i,361)*HM(i,2)
!
  pe(i,32) =  + k(i,51) + k(i,362)*HM(i,2)
!
  pr(i,35) =  + HM(i,6)*( + k(i,164)*HM(i,17) + k(i,168)*HM(i,18) +         &
    k(i,172)*HM(i,19) ) + k(i,67)*HM(i,13)*HM(i,5)
!
  pe(i,35) =  + k(i,4) + k(i,75)*HM(i,13)
!
  pe(i,36) =  + k(i,288)*HM(i,7) + k(i,289)*HM(i,1) + k(i,290)*HM(i,13)
!
  pr(i,37) =  + QM(i,36)*( + k(i,288)*HM(i,7) + k(i,289)*HM(i,1) +          &
    0.8*k(i,290)*HM(i,13) )
!
  pe(i,37) =  + k(i,87)*HM(i,13)
!
  pr(i,38) =  + HM(i,3)*( + 0.03*k(i,153)*QM(i,43) + 0.06*k(i,154)*QM(i,44) )
!
  pe(i,38) =  + k(i,92)*HM(i,13)
!
  pe(i,39) =  + k(i,93)*HM(i,13)
!
  pe(i,40) =  + HM(i,13)*( + k(i,94) + k(i,95) )
!
  pe(i,41) =  + k(i,96)*HM(i,13)
!
  pe(i,42) =  + HM(i,7)*( + k(i,136) + k(i,137) ) + k(i,97)*HM(i,13) +      &
    k(i,152)*HM(i,3)
!
  pe(i,43) =  + HM(i,7)*( + k(i,138) + k(i,139) ) + k(i,98)*HM(i,13) +       &
    k(i,153)*HM(i,3)
!
  pr(i,44) =  + QM(i,74)*( + 0.25*k(i,190)*HM(i,5) +                         &
    0.40*k(i,231)*QM(i,21) + 0.40*k(i,251)*QM(i,79) +                        &
    0.40*k(i,273)*HM(i,7) ) + 0.080*k(i,229)*QM(i,72)*QM(i,21)
!
  pe(i,44) =  + HM(i,7)*( + k(i,140) + k(i,141) ) +                          &
    k(i,99)*HM(i,13) + k(i,154)*HM(i,3)
!
  pe(i,45) =  + HM(i,7)*( + k(i,142) + k(i,143) ) +                          &
    k(i,100)*HM(i,13) + k(i,155)*HM(i,3)
!
  pe(i,46) =  + HM(i,7)*( + k(i,144) + k(i,145) ) +                          &
    k(i,89)*HM(i,1) + k(i,101)*HM(i,13) + k(i,156)*HM(i,3)
!
  pe(i,47) =  + HM(i,7)*( + k(i,146) + k(i,147) ) +                          &
    k(i,102)*HM(i,13) + k(i,157)*HM(i,3)
!
  pe(i,48) =  + HM(i,7)*( + k(i,148) + k(i,149) ) +                          &
    k(i,103)*HM(i,13) + k(i,158)*HM(i,3)
!
  pe(i,49) =  + HM(i,13)*( + k(i,104) + k(i,105) )
!
  pe(i,50) =  + HM(i,13)*( + k(i,106) + k(i,107) )
!
  pe(i,51) =  + HM(i,13)*( + k(i,108) + k(i,109) + k(i,110) ) + k(i,135)*HM(i,7)
!
  pe(i,52) =  + k(i,13) + k(i,112)*HM(i,13) + k(i,130)*HM(i,7)
!
  pe(i,53) =  + k(i,17) + k(i,113)*HM(i,13)
!
  pe(i,54) =  + k(i,18) + k(i,19) + k(i,115)*HM(i,13) + k(i,131)*HM(i,7)
!
  pe(i,55) =  + k(i,20) + k(i,116)*HM(i,13) + k(i,132)*HM(i,7)
!
  pe(i,56) =  + k(i,21) + k(i,118)*HM(i,13) + k(i,134)*HM(i,7) +            &
    k(i,160)*HM(i,3)
!
  pe(i,57) =  + k(i,24) + k(i,90)*HM(i,1) + k(i,117)*HM(i,13) +             &
    k(i,133)*HM(i,7) + k(i,159)*HM(i,3)
!
  pr(i,58) =  + 0.35*k(i,118)*HM(i,13)*QM(i,56)
!
  pe(i,58) =  + HM(i,13)*( + k(i,119) + k(i,120) )
!
  pr(i,59) =  + HM(i,13)*( + 0.024*k(i,96)*QM(i,41) +                       &
    0.41*k(i,117)*QM(i,57) + 0.60*k(i,127)*QM(i,62) ) +                     &
    0.30*k(i,239)*QM(i,81)*QM(i,21)
!
  pe(i,59) =  + k(i,25) + k(i,114)*HM(i,13)
!
  pe(i,60) =  + k(i,22) + k(i,23) + k(i,128)*HM(i,13)
!
  pr(i,61) =  + QM(i,62)*( + 0.40*k(i,127)*HM(i,13) +                       &
    k(i,151)*HM(i,7) + 0.30*k(i,161)*HM(i,3) ) +                            &
    k(i,176)*QM(i,79)*HM(i,6)
!
  pe(i,61) =  + k(i,126)*HM(i,13) + k(i,177)
!
  pr(i,62) =  + k(i,178)*QM(i,80)*HM(i,6)
!
  pe(i,62) =  + HM(i,7)*( + k(i,150) + k(i,151) ) +                         &
    k(i,127)*HM(i,13) + k(i,161)*HM(i,3) + k(i,179)
!
  pe(i,63) =  + k(i,15) + k(i,123)*HM(i,13)
!
  pr(i,64) =  + 0.11*k(i,160)*QM(i,56)*HM(i,3) + k(i,213)*QM(i,79)*HM(i,14)
!
  pe(i,64) =  + HM(i,13)*( + k(i,124) + k(i,125) ) + k(i,16)
!
  pe(i,65) =  + k(i,181)*HM(i,5) + k(i,200)*HM(i,14) +                       &
    k(i,222)*QM(i,21) + k(i,242)*QM(i,79) + k(i,264)*HM(i,7)
!
  pr(i,66) =  + HM(i,13)*( + 0.583*k(i,93)*QM(i,39) +                        &
    0.44*k(i,123)*QM(i,63) + k(i,128)*QM(i,60) )
!
  pe(i,66) =  + k(i,182)*HM(i,5) + k(i,201)*HM(i,14) +                       &
    k(i,223)*QM(i,21) + k(i,243)*QM(i,79) + k(i,265)*HM(i,7)
!
  pr(i,67) =  + k(i,94)*QM(i,40)*HM(i,13)
!
  pe(i,67) =  + k(i,183)*HM(i,5) + k(i,202)*HM(i,14) +                       &
    k(i,224)*QM(i,21) + k(i,244)*QM(i,79) + k(i,266)*HM(i,7)
!
  pr(i,68) =  + 0.951*k(i,96)*QM(i,41)*HM(i,13)
!
  pe(i,68) =  + k(i,184)*HM(i,5) + k(i,203)*HM(i,14) +                       &
    k(i,225)*QM(i,21) + k(i,245)*QM(i,79) + k(i,267)*HM(i,7)
!
  pr(i,69) =  + k(i,97)*QM(i,42)*HM(i,13)
!
  pe(i,69) =  + k(i,185)*HM(i,5) + k(i,204)*HM(i,14) +                       &
    k(i,226)*QM(i,21) + k(i,246)*QM(i,79) + k(i,268)*HM(i,7)
!
  pr(i,70) =  + k(i,98)*QM(i,43)*HM(i,13)
!
  pe(i,70) =  + k(i,186)*HM(i,5) + k(i,205)*HM(i,14) +                       &
    k(i,227)*QM(i,21) + k(i,247)*QM(i,79) + k(i,269)*HM(i,7)
!
  pr(i,71) =  + k(i,99)*QM(i,44)*HM(i,13)
!
  pe(i,71) =  + k(i,187)*HM(i,5) + k(i,206)*HM(i,14) +                       &
    k(i,228)*QM(i,21) + k(i,248)*QM(i,79) + k(i,270)*HM(i,7)
!
  pr(i,72) =  + HM(i,13)*( + k(i,100)*QM(i,45) + k(i,101)*QM(i,46) )
!
  pe(i,72) =  + k(i,188)*HM(i,5) + k(i,207)*HM(i,14) +                       &
    k(i,229)*QM(i,21) + k(i,249)*QM(i,79) + k(i,271)*HM(i,7)
!
  pr(i,73) =  + k(i,102)*QM(i,47)*HM(i,13)
!
  pe(i,73) =  + k(i,189)*HM(i,5) + k(i,208)*HM(i,14) +                       &
    k(i,230)*QM(i,21) + k(i,250)*QM(i,79) + k(i,272)*HM(i,7)
!
  pr(i,74) =  + k(i,103)*QM(i,48)*HM(i,13)
!
  pe(i,74) =  + k(i,190)*HM(i,5) + k(i,209)*HM(i,14) +                       &
    k(i,231)*QM(i,21) + k(i,251)*QM(i,79) + k(i,273)*HM(i,7)
!
  pr(i,75) =  + QM(i,51)*( + k(i,109)*HM(i,13) + k(i,135)*HM(i,7) )
!
  pe(i,75) =  + k(i,162)*HM(i,6) + k(i,163)*HM(i,14)
!
  pr(i,76) =  + k(i,165)*HM(i,17)*0.21*hnm(i)
!
  pe(i,76) =  + k(i,191)*HM(i,5) + k(i,210)*HM(i,14) +                       &
    k(i,232)*QM(i,21) + k(i,252)*QM(i,79) + k(i,274)*HM(i,7)
!
  pr(i,77) =  + k(i,169)*HM(i,18)*0.21*hnm(i)
!
  pe(i,77) =  + k(i,192)*HM(i,5) + k(i,211)*HM(i,14) +                       &
    k(i,233)*QM(i,21) + k(i,253)*QM(i,79) + k(i,275)*HM(i,7)
!
  pr(i,78) =  + k(i,173)*HM(i,19)*0.21*hnm(i)
!
  pe(i,78) =  + k(i,193)*HM(i,5) + k(i,212)*HM(i,14) +                       &
    k(i,234)*QM(i,21) + k(i,254)*QM(i,79) + k(i,276)*HM(i,7)
!
  pe(i,81) =  + k(i,196)*HM(i,5) + k(i,217)*HM(i,14) +                       &
    k(i,239)*QM(i,21) + k(i,257)*QM(i,79) + k(i,279)*HM(i,7)
!
  pr(i,85) =  + k(i,87)*HM(i,13)*QM(i,37)
!
  pe(i,3) = ( + k(i,28) + k(i,57)*HM(i,2) + k(i,63)*HM(i,14)*HM(i,14)        &
    )*partH2O(i) + k(i,364)*QM(i,9) + k(i,366)*QM(i,5) + k(i,369)*QM(i,15)
!
  pe(i,8) = ( + k(i,34) + k(i,325)*HM(i,13) )*partHCl(i) +                   &
    k(i,365)*QM(i,9) + k(i,367)*QM(i,5) + k(i,368)*QM(i,10) + k(i,370)*QM(i,18)
!
  pe(i,10) =  + k(i,30) + k(i,329)*HM(i,13) + k(i,330)*HM(i,1) +             &
    k(i,332)*HM(i,9) + k(i,368)*QM(i,8) + k(i,372)*QM(i,14)
!
  pr(i,12) =  + HM(i,3)*( + 0.006*k(i,153)*QM(i,43) +                        &
    0.011*k(i,154)*QM(i,44) + 0.001*k(i,155)*QM(i,45) +                      & 
    0.001*k(i,156)*QM(i,46) + 0.02*k(i,157)*QM(i,47) +                       &
    0.02*k(i,158)*QM(i,48) ) + HM(i,14)*( + k(i,62)*HM(i,14) +               &
    k(i,63)*HM(i,14)*QM(i,3)*partH2O(i) ) + k(i,301)*HM(i,13)*HM(i,13)
!
  pe(i,20) =  + k(i,11) + k(i,12) + k(i,111)*HM(i,13) +                      &
    k(i,129)*HM(i,7) + k(i,311)*HM(i,1) + k(i,312)*HM(i,9) + k(i,315)*HM(i,11)
!
  pr(i,43) =  + QM(i,72)*( + 0.354*k(i,188)*HM(i,5) +                        &
    0.370*k(i,229)*QM(i,21) + 0.229*k(i,249)*QM(i,79) +                      &
    0.400*k(i,271)*HM(i,7) ) + HM(i,3)*( + 0.35*k(i,155)*QM(i,45) +          &
    0.35*k(i,156)*QM(i,46) + 0.46*k(i,158)*QM(i,48) ) +                      &
    0.86*k(i,89)*QM(i,46)*HM(i,1)
!
  pr(i,56) =  + QM(i,76)*( + 0.50*k(i,191)*HM(i,5) +                         &
    k(i,232)*QM(i,21) + k(i,252)*QM(i,79) +                                  &
    0.50*k(i,274)*HM(i,7) ) + QM(i,77)*( + 0.95*k(i,192)*HM(i,5) +           &
    k(i,233)*QM(i,21) + k(i,253)*QM(i,79) + k(i,275)*HM(i,7) ) +             &
    0.13*k(i,89)*QM(i,46)*HM(i,1)
!
  pr(i,80) =  + QM(i,56)*( + k(i,21) + 0.50*k(i,118)*HM(i,13) +              &
    0.50*k(i,134)*HM(i,7) ) + QM(i,57)*( + 0.51*k(i,117)*HM(i,13) +          &
    0.20*k(i,133)*HM(i,7) ) + k(i,179)*QM(i,62)
!
  pe(i,80) =  + HM(i,14)*( + k(i,215) + k(i,216) ) + QM(i,21)*( +            &
    k(i,237) + k(i,238) ) + k(i,178)*HM(i,6) + k(i,195)*HM(i,5) +            &
    k(i,256)*QM(i,79) + k(i,278)*HM(i,7)
!
  pr(i,81) =  + HM(i,3)*( + 0.03*k(i,153)*QM(i,43) +                         &
    0.12*k(i,154)*QM(i,44) + 0.02*k(i,155)*QM(i,45) +                        &
    0.02*k(i,156)*QM(i,46) + 0.42*k(i,157)*QM(i,47) +                        &
    0.42*k(i,158)*QM(i,48) ) + k(i,113)*QM(i,53)*HM(i,13)
!
  pr(i,82) =  + HM(i,7)*( + 0.80*k(i,133)*QM(i,57) +                         &
    k(i,136)*QM(i,42) + k(i,138)*QM(i,43) + k(i,140)*QM(i,44) +              &
    k(i,142)*QM(i,45) + k(i,144)*QM(i,46) + k(i,146)*QM(i,47) +              &
    k(i,148)*QM(i,48) )
!
  pe(i,82) =  + k(i,197)*HM(i,5) + k(i,218)*HM(i,14) +                       &
    k(i,240)*QM(i,21) + k(i,258)*QM(i,79) + 2.0*k(i,260)*QM(i,82) +          &
    k(i,261)*QM(i,83) + k(i,280)*HM(i,7)
!
  pr(i,83) =  + HM(i,7)*( + k(i,137)*QM(i,42) + k(i,139)*QM(i,43) +          &
    k(i,141)*QM(i,44) + k(i,143)*QM(i,45) + k(i,145)*QM(i,46) +              &
    k(i,147)*QM(i,47) + k(i,149)*QM(i,48) )
!
  pe(i,83) =  + k(i,198)*HM(i,5) + k(i,219)*HM(i,14) +                       &
    k(i,241)*QM(i,21) + k(i,259)*QM(i,79) + k(i,261)*QM(i,82) +              &
    2.0*k(i,262)*QM(i,83) + k(i,281)*HM(i,7)
!
  pe(i,84) =  + k(i,282)*HM(i,14) + k(i,283)*QM(i,21) +                      &
    k(i,284)*QM(i,79) + 2.0*k(i,285)*QM(i,84) + k(i,286)*HM(i,5) +           &
    k(i,287)*HM(i,7)
!
  pr(i,3) =  + HM(i,13)*( + k(i,60)*HM(i,14) + k(i,61)*QM(i,12) +            &
    k(i,75)*QM(i,35) + k(i,76)*QM(i,4)*partHNO3(i) +                         &
    k(i,77)*QM(i,16) + k(i,86)*5.e-7*hnm(i) + k(i,91)*QM(i,2) +              &
    k(i,92)*QM(i,38) + k(i,93)*QM(i,39) + k(i,94)*QM(i,40) +                 &
    k(i,95)*QM(i,40) + k(i,96)*QM(i,41) + k(i,111)*QM(i,20) +                &
    k(i,112)*QM(i,52) + k(i,113)*QM(i,53) + k(i,114)*QM(i,59) +              &
    k(i,115)*QM(i,54) + k(i,116)*QM(i,55) + k(i,121)*QM(i,22) +              &
    k(i,122)*QM(i,22) + k(i,126)*QM(i,61) + k(i,128)*QM(i,60) +              &
    k(i,300)*HM(i,13) + k(i,325)*QM(i,8)*partHCl(i) +                        &
    k(i,329)*QM(i,10) + k(i,341)*QM(i,27) + k(i,342)*QM(i,28) +              &
    k(i,344)*QM(i,29) + k(i,355)*QM(i,14)*partHBr(i) +                       &
    k(i,360)*QM(i,30) ) + HM(i,14)*( +                                       &
    k(i,63)*HM(i,14)*QM(i,3)*partH2O(i) + k(i,298)*HM(i,12) ) +              &
    QM(i,10)*( + k(i,368)*QM(i,8) + k(i,372)*QM(i,14) ) +                    &
    QM(i,18)*( + k(i,370)*QM(i,8) + k(i,371)*QM(i,14) )
!
  pr(i,4) =  + HM(i,7)*( + k(i,73)*HM(i,14) + k(i,129)*QM(i,20) +            &
    k(i,130)*QM(i,52) + k(i,131)*QM(i,54) + k(i,132)*QM(i,55) +              &
    0.20*k(i,133)*QM(i,57) + 0.50*k(i,134)*QM(i,56) +                        &
    k(i,135)*QM(i,51) ) + QM(i,3)*( + k(i,364)*QM(i,9) +                     &
    2.0*k(i,366)*QM(i,5) + k(i,369)*QM(i,15) ) + QM(i,8)*( +                 &
    k(i,365)*QM(i,9) + k(i,367)*QM(i,5) ) + k(i,68)*HM(i,13)*HM(i,6)
!
  pr(i,6) =  + HM(i,3)*( + 0.43*k(i,152)*QM(i,42) +                          &
    0.37*k(i,153)*QM(i,43) + 0.30*k(i,154)*QM(i,44) +                        &
    0.36*k(i,155)*QM(i,45) + 0.36*k(i,156)*QM(i,46) +                        &
    0.14*k(i,157)*QM(i,47) + 0.14*k(i,158)*QM(i,48) +                        &
    0.54*k(i,159)*QM(i,57) + 0.66*k(i,160)*QM(i,56) +                        &
    0.13*k(i,161)*QM(i,62) ) + QM(i,20)*( + k(i,11) + k(i,12) +              &
    k(i,111)*HM(i,13) + k(i,129)*HM(i,7) + k(i,311)*HM(i,1) +                &
    k(i,312)*HM(i,9) + k(i,315)*HM(i,11) ) + QM(i,54)*( +                    &
    1.87*k(i,18) + 1.55*k(i,19) + 2.0*k(i,115)*HM(i,13) +                    &
    2.0*k(i,131)*HM(i,7) ) + QM(i,55)*( + k(i,20) +                          &
    k(i,116)*HM(i,13) + k(i,132)*HM(i,7) ) + QM(i,57)*( + k(i,24) +          &
    0.41*k(i,117)*HM(i,13) + 0.80*k(i,133)*HM(i,7) ) +                       &
    k(i,13)*QM(i,52) + 0.01*k(i,89)*QM(i,46)*HM(i,1) +                       &
#ifdef NIV_60
    jco2(i)*rco2*hnm(i) +                                                    &
#endif
    0.036*k(i,93)*QM(i,39)*HM(i,13)
!
  pr(i,20) =  + QM(i,21)*( + k(i,221)*QM(i,21) +                             &
    0.75*k(i,222)*QM(i,65) + 0.810*k(i,223)*QM(i,66) +                       &
    0.829*k(i,224)*QM(i,67) + 0.753*k(i,225)*QM(i,68) +                      &
    1.55*k(i,226)*QM(i,69) + 1.25*k(i,227)*QM(i,70) +                        &
    0.755*k(i,228)*QM(i,71) + 1.09*k(i,229)*QM(i,72) +                       &
    k(i,230)*QM(i,73) + 1.40*k(i,231)*QM(i,74) + k(i,232)*QM(i,76) +         &
    k(i,233)*QM(i,77) + k(i,234)*QM(i,78) + k(i,235)*QM(i,79) +              &
    k(i,236)*QM(i,79) + 2.0*k(i,237)*QM(i,80) + k(i,238)*QM(i,80) +          &
    0.75*k(i,239)*QM(i,81) + 0.75*k(i,240)*QM(i,82) +                        &
    0.960*k(i,241)*QM(i,83) + k(i,263)*HM(i,7) +                             &
    k(i,283)*QM(i,84) ) + HM(i,7)*( + k(i,151)*QM(i,62) +                    &
    0.048*k(i,265)*QM(i,66) + 0.021*k(i,266)*QM(i,67) +                      &
    1.6*k(i,268)*QM(i,69) + k(i,269)*QM(i,70) +                              &
    0.686*k(i,271)*QM(i,72) + 0.40*k(i,273)*QM(i,74) +                       &
    k(i,278)*QM(i,80) + 0.280*k(i,281)*QM(i,83) ) + HM(i,3)*( +              &
    k(i,152)*QM(i,42) + 0.64*k(i,153)*QM(i,43) +                             &
    0.02*k(i,154)*QM(i,44) + 0.90*k(i,155)*QM(i,45) +                        &
    0.90*k(i,156)*QM(i,46) + 0.04*k(i,158)*QM(i,48) +                        &
    0.40*k(i,159)*QM(i,57) + 0.70*k(i,161)*QM(i,62) ) + HM(i,5)*( +          &
    0.047*k(i,182)*QM(i,66) + 0.021*k(i,183)*QM(i,67) +                      &
    1.6*k(i,185)*QM(i,69) + k(i,186)*QM(i,70) +                              &
    0.606*k(i,188)*QM(i,72) + 0.25*k(i,190)*QM(i,74) +                       &
    k(i,195)*QM(i,80) + 0.287*k(i,198)*QM(i,83) ) + QM(i,79)*( +             &
    0.091*k(i,243)*QM(i,66) + 0.076*k(i,244)*QM(i,67) +                      &
    0.8*k(i,246)*QM(i,69) + 0.501*k(i,247)*QM(i,70) +                        &
    0.340*k(i,249)*QM(i,72) + 0.40*k(i,251)*QM(i,74) +                       &
    k(i,256)*QM(i,80) + 0.207*k(i,259)*QM(i,83) ) + HM(i,13)*( +             &
    0.010*k(i,93)*QM(i,39) + 0.08*k(i,117)*QM(i,57) +                        &
    k(i,122)*QM(i,22) + k(i,125)*QM(i,64) + k(i,126)*QM(i,61) +              &
    0.40*k(i,127)*QM(i,62) ) + QM(i,54)*( + 0.13*k(i,18) +                   &
    0.45*k(i,19) ) + HM(i,1)*( + 0.05*k(i,89)*QM(i,46) +                     &
    k(i,309)*HM(i,15) ) + QM(i,83)*( + 0.202*k(i,261)*QM(i,82) +             &
    0.504*k(i,262)*QM(i,83) ) + k(i,24)*QM(i,57) +                           &
    k(i,25)*QM(i,59) + k(i,308)*HM(i,2)*QM(i,2) +                            &
    k(i,313)*HM(i,16)*0.21*hnm(i)
!
  pr(i,21) =  + QM(i,79)*( + k(i,194)*HM(i,5) + k(i,235)*QM(i,21) +          &
    0.5*k(i,242)*QM(i,65) + 0.508*k(i,243)*QM(i,66) +                        &
    0.554*k(i,244)*QM(i,67) + 0.507*k(i,245)*QM(i,68) +                      &
    0.5*k(i,246)*QM(i,69) + 0.501*k(i,247)*QM(i,70) +                        &
    0.510*k(i,248)*QM(i,71) + 0.506*k(i,249)*QM(i,72) +                      &
    k(i,250)*QM(i,73) + k(i,251)*QM(i,74) + k(i,252)*QM(i,76) +              &
    k(i,253)*QM(i,77) + k(i,254)*QM(i,78) + 2.0*k(i,255)*QM(i,79) +          &
    k(i,256)*QM(i,80) + 0.50*k(i,257)*QM(i,81) +                             &
    0.50*k(i,258)*QM(i,82) + 0.516*k(i,259)*QM(i,83) +                       &
    k(i,277)*HM(i,7) + k(i,284)*QM(i,84) ) + HM(i,3)*( +                     &
    0.19*k(i,153)*QM(i,43) + 0.23*k(i,154)*QM(i,44) +                        &
    0.03*k(i,155)*QM(i,45) + 0.03*k(i,156)*QM(i,46) ) + QM(i,66)*( +         &
    0.150*k(i,182)*HM(i,5) + 0.007*k(i,223)*QM(i,21) +                       &
    0.155*k(i,265)*HM(i,7) ) + QM(i,67)*( + 0.031*k(i,183)*HM(i,5) +         &
    0.049*k(i,224)*QM(i,21) + 0.040*k(i,266)*HM(i,7) ) +                     &
    k(i,13)*QM(i,52) + k(i,16)*QM(i,64) +                                    &
    k(i,121)*QM(i,22)*HM(i,13) + k(i,310)*HM(i,15)*0.21*hnm(i)
!
  pe(i,21) =  + 2.0*( + k(i,220)*QM(i,21) + k(i,221)*QM(i,21) ) +            &
    QM(i,79)*( + k(i,235) + k(i,236) ) + QM(i,80)*( + k(i,237) +             &
    k(i,238) ) + k(i,180)*HM(i,5) + k(i,199)*HM(i,14) +                      &
    k(i,222)*QM(i,65) + k(i,223)*QM(i,66) + k(i,224)*QM(i,67) +              &
    k(i,225)*QM(i,68) + k(i,226)*QM(i,69) + k(i,227)*QM(i,70) +              &
    k(i,228)*QM(i,71) + k(i,229)*QM(i,72) + k(i,230)*QM(i,73) +              &
    k(i,231)*QM(i,74) + k(i,232)*QM(i,76) + k(i,233)*QM(i,77) +              &
    k(i,234)*QM(i,78) + k(i,239)*QM(i,81) + k(i,240)*QM(i,82) +              &
    k(i,241)*QM(i,83) + k(i,263)*HM(i,7) + k(i,283)*QM(i,84) +               &
    k(i,314)*HM(i,8)
!
  pr(i,51) =  + HM(i,6)*( + 0.10*k(i,162)*QM(i,75) +                         &
    k(i,164)*HM(i,17) + k(i,168)*HM(i,18) + k(i,172)*HM(i,19) ) +            &
    0.21*hnm(i)*( + k(i,166)*HM(i,17) + k(i,170)*HM(i,18) +                  &
    k(i,174)*HM(i,19) ) + HM(i,3)*( + k(i,167)*HM(i,17) +                    &
    k(i,171)*HM(i,18) + k(i,175)*HM(i,19) ) +                                &
    k(i,163)*QM(i,75)*HM(i,14)
!
  pr(i,52) =  + HM(i,7)*( + 0.25*k(i,134)*QM(i,56) +                         &
    k(i,264)*QM(i,65) + 0.243*k(i,265)*QM(i,66) +                            &
    0.239*k(i,266)*QM(i,67) + 0.187*k(i,267)*QM(i,68) +                      &
    0.2*k(i,268)*QM(i,69) + 0.94*k(i,269)*QM(i,70) +                         &
    1.71*k(i,270)*QM(i,71) + k(i,272)*QM(i,73) +                             &
    0.46*k(i,279)*QM(i,81) + 1.24*k(i,281)*QM(i,83) ) + HM(i,5)*( +          &
    k(i,181)*QM(i,65) + 0.233*k(i,182)*QM(i,66) +                            &
    0.211*k(i,183)*QM(i,67) + 0.150*k(i,184)*QM(i,68) +                      &
    0.2*k(i,185)*QM(i,69) + 0.94*k(i,186)*QM(i,70) +                         &
    1.71*k(i,187)*QM(i,71) + 0.80*k(i,189)*QM(i,73) +                        &
    0.46*k(i,196)*QM(i,81) + 1.24*k(i,198)*QM(i,83) ) + QM(i,21)*( +         &
    0.75*k(i,222)*QM(i,65) + 0.580*k(i,223)*QM(i,66) +                       &
    0.523*k(i,224)*QM(i,67) + 0.411*k(i,225)*QM(i,68) +                      &
    0.35*k(i,226)*QM(i,69) + 0.669*k(i,227)*QM(i,70) +                       &
    0.932*k(i,228)*QM(i,71) + k(i,230)*QM(i,73) +                            &
    0.30*k(i,239)*QM(i,81) + 0.640*k(i,241)*QM(i,83) ) +                     &
    QM(i,79)*( + k(i,242)*QM(i,65) + 0.724*k(i,243)*QM(i,66) +               &
    0.677*k(i,244)*QM(i,67) + 0.497*k(i,245)*QM(i,68) +                      &
    0.6*k(i,246)*QM(i,69) + 0.859*k(i,247)*QM(i,70) +                        &
    0.941*k(i,248)*QM(i,71) + k(i,250)*QM(i,73) +                            &
    0.35*k(i,257)*QM(i,81) + 0.650*k(i,259)*QM(i,83) ) +                     &
    HM(i,13)*( + 0.335*k(i,93)*QM(i,39) + 0.025*k(i,96)*QM(i,41) +           &
    k(i,119)*QM(i,58) + 0.08*k(i,123)*QM(i,63) ) + HM(i,3)*( +               &
    0.44*k(i,153)*QM(i,43) + 0.99*k(i,154)*QM(i,44) +                        &
    0.65*k(i,157)*QM(i,47) + 0.16*k(i,160)*QM(i,56) ) + QM(i,83)*( +         &
    0.640*k(i,261)*QM(i,82) + 1.21*k(i,262)*QM(i,83) ) +                     &
    k(i,15)*QM(i,63) + k(i,22)*QM(i,60) + k(i,90)*QM(i,57)*HM(i,1)
!
  pr(i,53) =  + HM(i,7)*( + 0.03*k(i,134)*QM(i,56) +           &
    0.670*k(i,265)*QM(i,66) + 0.828*k(i,266)*QM(i,67) +           &
    0.880*k(i,267)*QM(i,68) + 0.06*k(i,269)*QM(i,70) +           &
    0.29*k(i,270)*QM(i,71) + k(i,272)*QM(i,73) +           &
    0.469*k(i,281)*QM(i,83) ) + QM(i,79)*( +           &
    0.127*k(i,243)*QM(i,66) + 0.330*k(i,244)*QM(i,67) +           &
    0.581*k(i,245)*QM(i,68) + 0.141*k(i,247)*QM(i,70) +           &
    0.569*k(i,248)*QM(i,71) + k(i,250)*QM(i,73) +           &
    0.11*k(i,257)*QM(i,81) + 0.167*k(i,259)*QM(i,83) ) + HM(i,5)*( +           &
    0.623*k(i,182)*QM(i,66) + 0.722*k(i,183)*QM(i,67) +           &
    0.642*k(i,184)*QM(i,68) + 0.06*k(i,186)*QM(i,70) +           &
    0.29*k(i,187)*QM(i,71) + 0.80*k(i,189)*QM(i,73) +           &
    0.464*k(i,198)*QM(i,83) ) + QM(i,21)*( +           &
    0.018*k(i,223)*QM(i,66) + 0.240*k(i,224)*QM(i,67) +           &
    0.419*k(i,225)*QM(i,68) + 0.081*k(i,227)*QM(i,70) +           &
    0.313*k(i,228)*QM(i,71) + k(i,230)*QM(i,73) +           &
    0.149*k(i,241)*QM(i,83) ) + HM(i,13)*( + k(i,95)*QM(i,40) +           &
    k(i,120)*QM(i,58) + 0.41*k(i,123)*QM(i,63) ) + HM(i,3)*( +           &
    0.03*k(i,153)*QM(i,43) + 0.16*k(i,154)*QM(i,44) +           &
    0.53*k(i,157)*QM(i,47) ) + QM(i,83)*( +           &
    0.149*k(i,261)*QM(i,82) + 0.285*k(i,262)*QM(i,83) ) +           &
    k(i,23)*QM(i,60)
!
  pr(i,54) =  + HM(i,7)*( + 0.25*k(i,134)*QM(i,56) +           &
    0.063*k(i,265)*QM(i,66) + 1.30*k(i,274)*QM(i,76) +           &
    0.74*k(i,275)*QM(i,77) + k(i,276)*QM(i,78) ) + HM(i,5)*( +           &
    0.063*k(i,182)*QM(i,66) + 1.20*k(i,191)*QM(i,76) +           &
    0.35*k(i,192)*QM(i,77) + k(i,193)*QM(i,78) ) + QM(i,21)*( +           &
    0.119*k(i,223)*QM(i,66) + 0.65*k(i,232)*QM(i,76) +           &
    0.37*k(i,233)*QM(i,77) + k(i,234)*QM(i,78) ) + QM(i,79)*( +           &
    0.100*k(i,243)*QM(i,66) + 0.65*k(i,252)*QM(i,76) +           &
    0.37*k(i,253)*QM(i,77) + k(i,254)*QM(i,78) ) + HM(i,13)*( +           &
    0.036*k(i,93)*QM(i,39) + 0.15*k(i,118)*QM(i,56) ) +           &
    0.50*k(i,160)*QM(i,56)*HM(i,3)
!
  pr(i,55) =  + HM(i,7)*( + 0.25*k(i,134)*QM(i,56) +           &
    0.70*k(i,274)*QM(i,76) + 1.26*k(i,275)*QM(i,77) +           &
    k(i,276)*QM(i,78) + 0.54*k(i,279)*QM(i,81) ) + QM(i,21)*( +           &
    0.005*k(i,223)*QM(i,66) + 0.35*k(i,232)*QM(i,76) +           &
    0.63*k(i,233)*QM(i,77) + k(i,234)*QM(i,78) +           &
    0.40*k(i,239)*QM(i,81) ) + QM(i,79)*( +           &
    0.004*k(i,243)*QM(i,66) + 0.35*k(i,252)*QM(i,76) +           &
    0.63*k(i,253)*QM(i,77) + k(i,254)*QM(i,78) +           &
    0.54*k(i,257)*QM(i,81) ) + HM(i,5)*( + 0.65*k(i,191)*QM(i,76) +           &
    0.60*k(i,192)*QM(i,77) + k(i,193)*QM(i,78) +           &
    0.54*k(i,196)*QM(i,81) ) + HM(i,13)*( + k(i,114)*QM(i,59) +           &
    0.08*k(i,117)*QM(i,57) + 0.15*k(i,118)*QM(i,56) ) + HM(i,3)*( +           &
    0.60*k(i,159)*QM(i,57) + 0.62*k(i,160)*QM(i,56) )
!
  pr(i,57) =  + HM(i,7)*( + k(i,142)*QM(i,45) + k(i,144)*QM(i,46) +           &
    0.600*k(i,271)*QM(i,72) + 0.60*k(i,273)*QM(i,74) ) + HM(i,3)*( +           &
    0.39*k(i,155)*QM(i,45) + 0.39*k(i,156)*QM(i,46) +           &
    0.79*k(i,158)*QM(i,48) ) + QM(i,72)*( + 0.446*k(i,188)*HM(i,5) +           &
    0.550*k(i,229)*QM(i,21) + 0.771*k(i,249)*QM(i,79) ) +           &
    QM(i,74)*( + 0.40*k(i,190)*HM(i,5) + 0.60*k(i,231)*QM(i,21) +           &
    0.60*k(i,251)*QM(i,79) )
!
  pr(i,60) =  + HM(i,5)*( + 0.059*k(i,182)*QM(i,66) +           &
    0.124*k(i,183)*QM(i,67) + 0.261*k(i,184)*QM(i,68) +           &
    0.153*k(i,188)*QM(i,72) + 0.20*k(i,189)*QM(i,73) +           &
    0.35*k(i,190)*QM(i,74) + 0.05*k(i,191)*QM(i,76) +           &
    0.05*k(i,192)*QM(i,77) + k(i,197)*QM(i,82) ) + QM(i,82)*( +           &
    k(i,218)*HM(i,14) + k(i,240)*QM(i,21) + k(i,258)*QM(i,79) +           &
    2.0*k(i,260)*QM(i,82) + 1.50*k(i,261)*QM(i,83) +           &
    k(i,280)*HM(i,7) ) + QM(i,83)*( + k(i,219)*HM(i,14) +           &
    0.500*k(i,241)*QM(i,21) + 0.484*k(i,259)*QM(i,79) +           &
    k(i,262)*QM(i,83) ) + k(i,150)*QM(i,62)*HM(i,7) +           &
    k(i,162)*QM(i,75)*HM(i,6)
!
  pr(i,63) =  + HM(i,14)*( + k(i,200)*QM(i,65) +           &
    k(i,201)*QM(i,66) + k(i,202)*QM(i,67) + k(i,203)*QM(i,68) +           &
    k(i,204)*QM(i,69) + k(i,205)*QM(i,70) + k(i,206)*QM(i,71) +           &
    k(i,207)*QM(i,72) + k(i,208)*QM(i,73) + k(i,209)*QM(i,74) +           &
    k(i,210)*QM(i,76) + k(i,211)*QM(i,77) + k(i,212)*QM(i,78) +           &
    k(i,215)*QM(i,80) + k(i,217)*QM(i,81) + k(i,282)*QM(i,84) ) +           &
    0.13*k(i,159)*QM(i,57)*HM(i,3)
!
  pr(i,65) =  + HM(i,3)*( + 0.10*k(i,153)*QM(i,43) +           &
    0.18*k(i,154)*QM(i,44) + 0.20*k(i,157)*QM(i,47) +           &
    0.16*k(i,158)*QM(i,48) ) + QM(i,67)*( + 0.245*k(i,183)*HM(i,5) +           &
    0.014*k(i,224)*QM(i,21) + 0.018*k(i,244)*QM(i,79) +           &
    0.262*k(i,266)*HM(i,7) ) + QM(i,68)*( + 0.133*k(i,184)*HM(i,5) +           &
    0.013*k(i,225)*QM(i,21) + 0.015*k(i,245)*QM(i,79) +           &
    0.155*k(i,267)*HM(i,7) ) + QM(i,66)*( + 0.048*k(i,182)*HM(i,5) +           &
    0.006*k(i,243)*QM(i,79) + 0.053*k(i,265)*HM(i,7) ) +           &
    k(i,17)*QM(i,53) + k(i,92)*QM(i,38)*HM(i,13)
!
  pr(i,79) =  + HM(i,3)*( + 0.15*k(i,155)*QM(i,45) +           &
    0.15*k(i,156)*QM(i,46) + 0.13*k(i,159)*QM(i,57) +           &
    0.28*k(i,160)*QM(i,56) + 0.70*k(i,161)*QM(i,62) ) + HM(i,7)*( +           &
    k(i,130)*QM(i,52) + k(i,132)*QM(i,55) + k(i,278)*QM(i,80) +           &
    0.23*k(i,279)*QM(i,81) ) + HM(i,13)*( + k(i,112)*QM(i,52) +           &
    k(i,116)*QM(i,55) + k(i,124)*QM(i,64) ) + QM(i,80)*( +           &
    k(i,195)*HM(i,5) + k(i,237)*QM(i,21) + k(i,256)*QM(i,79) ) +           &
    QM(i,81)*( + 0.23*k(i,196)*HM(i,5) + 0.12*k(i,239)*QM(i,21) +           &
    0.12*k(i,257)*QM(i,79) ) + k(i,17)*QM(i,53) + k(i,20)*QM(i,55) +           &
    k(i,24)*QM(i,57) + k(i,25)*QM(i,59) + k(i,177)*QM(i,61)
!
  pe(i,79) =  + HM(i,14)*( + k(i,213) + k(i,214) ) + QM(i,21)*( +           &
    k(i,235) + k(i,236) ) + k(i,176)*HM(i,6) + k(i,194)*HM(i,5) +           &
    k(i,242)*QM(i,65) + k(i,243)*QM(i,66) + k(i,244)*QM(i,67) +           &
    k(i,245)*QM(i,68) + k(i,246)*QM(i,69) + k(i,247)*QM(i,70) +           &
    k(i,248)*QM(i,71) + k(i,249)*QM(i,72) + k(i,250)*QM(i,73) +           &
    k(i,251)*QM(i,74) + k(i,252)*QM(i,76) + k(i,253)*QM(i,77) +           &
    k(i,254)*QM(i,78) + 2.0*k(i,255)*QM(i,79) + k(i,256)*QM(i,80) +           &
    k(i,257)*QM(i,81) + k(i,258)*QM(i,82) + k(i,259)*QM(i,83) +           &
    k(i,277)*HM(i,7) + k(i,284)*QM(i,84)
!
  pr(i,84) =  + HM(i,13)*( + k(i,105)*QM(i,49) +           &
    k(i,107)*QM(i,50) + k(i,110)*QM(i,51) + 0.49*k(i,117)*QM(i,57) +           &
    0.50*k(i,118)*QM(i,56) + 0.07*k(i,123)*QM(i,63) +           &
    k(i,125)*QM(i,64) + k(i,126)*QM(i,61) + k(i,127)*QM(i,62) ) +           &
    HM(i,7)*( + 0.50*k(i,134)*QM(i,56) + k(i,150)*QM(i,62) +           &
    k(i,151)*QM(i,62) + 0.051*k(i,265)*QM(i,66) +           &
    0.391*k(i,266)*QM(i,67) + 0.587*k(i,267)*QM(i,68) +           &
    0.16*k(i,279)*QM(i,81) ) + HM(i,5)*( + 0.048*k(i,182)*QM(i,66) +           &
    0.334*k(i,183)*QM(i,67) + 0.416*k(i,184)*QM(i,68) +           &
    0.16*k(i,196)*QM(i,81) ) + QM(i,21)*( +           &
    0.085*k(i,223)*QM(i,66) + 0.245*k(i,224)*QM(i,67) +           &
    0.322*k(i,225)*QM(i,68) + 0.08*k(i,239)*QM(i,81) ) +           &
    QM(i,79)*( + 0.071*k(i,243)*QM(i,66) + 0.237*k(i,244)*QM(i,67) +           &
    0.318*k(i,245)*QM(i,68) + 0.08*k(i,257)*QM(i,81) ) +           &
    QM(i,46)*( + 0.15*k(i,89)*HM(i,1) + 0.13*k(i,156)*HM(i,3) ) +           &
    0.13*k(i,155)*QM(i,45)*HM(i,3)
!
  pr(i,86) = + HM(i,14)*( + k(i,298)*HM(i,12) + k(i,214)*QM(i,79) +           &
    k(i,216)*QM(i,80) ) + 0.21*hnm(i)*( + 2.0*k(i,26) +           &
    k(i,303)*HM(i,4) ) + HM(i,5)*( + k(i,52) + k(i,302)*HM(i,4) ) +           &
    k(i,1)*HM(i,6) + k(i,9)*HM(i,7) + k(i,37)*HM(i,10) +           &
    k(i,40)*QM(i,7) + k(i,300)*HM(i,13)*HM(i,13) + k(i,29)*QM(i,1)
!
  pe(i,86) =  + k(i,317)*HM(i,8)*HM(i,1) +           &
    2.0*k(i,333)*HM(i,8)*HM(i,8)*k(i,33)/(k(i,33) + k(i,334)) +           &
    MIN(k(i,30)*QM(i,10),k(i,328)*HM(i,14)*HM(i,8)) +           &
    2.0*MIN(k(i,41)*QM(i,19),k(i,350)*HM(i,10)*HM(i,8)) +           &
    2.0*k(i,349)*HM(i,10)*HM(i,8)*day(i) +           &
    MIN(k(i,39)*QM(i,18),k(i,357)*HM(i,14)*HM(i,10)) +           &
    2.0*k(i,351)*HM(i,10)*HM(i,10)*day(i) + 2.0*(k(i,32) +           &
    k(i,31))*QM(i,9)
  pe(i,86) = pe(i,86) + HM(i,3)*( + 2.0*k(i,54)*HM(i,1) +           &
    0.910000*k(i,155)*QM(i,45) + 0.910000*k(i,156)*QM(i,46) +           &
    k(i,58)*HM(i,13) + k(i,59)*HM(i,14) + k(i,78)*HM(i,5) +           &
    k(i,79)*HM(i,6) + k(i,152)*QM(i,42) + k(i,153)*QM(i,43) +           &
    k(i,154)*QM(i,44) + k(i,157)*QM(i,47) + k(i,158)*QM(i,48) +           &
    k(i,159)*QM(i,57) + k(i,160)*QM(i,56) + k(i,161)*QM(i,62) +           &
    k(i,167)*HM(i,17) + k(i,171)*HM(i,18) + k(i,175)*HM(i,19) +           &
    k(i,292)*HM(i,12) + k(i,316)*HM(i,9) + k(i,345)*HM(i,11) ) +           &
    HM(i,1)*( + k(i,64)*HM(i,5) + k(i,65)*HM(i,6) +           &
    k(i,66)*HM(i,6) + k(i,89)*QM(i,46) + k(i,90)*QM(i,57) +           &
    k(i,289)*QM(i,36) + k(i,294)*HM(i,13) + k(i,295)*HM(i,14) +           &
    k(i,299)*5.e-7*hnm(i) + k(i,309)*HM(i,15) + k(i,311)*QM(i,20) +           &
    k(i,317)*HM(i,8) + k(i,327)*QM(i,9) + k(i,330)*QM(i,10) +           &
    k(i,346)*HM(i,10) + k(i,358)*QM(i,18) + 2.0*k(i,363)*HM(i,1) ) +           &
    HM(i,2)*( + k(i,57)*QM(i,3) + k(i,293)*5.e-7*hnm(i) +           &
    k(i,304)*QM(i,1) + k(i,305)*QM(i,1) + k(i,306)*QM(i,2) +           &
    k(i,307)*QM(i,2) + k(i,308)*QM(i,2) + k(i,337)*QM(i,23) +           &
    k(i,338)*QM(i,24) + k(i,339)*QM(i,25) + k(i,340)*QM(i,26) +           &
    k(i,343)*QM(i,29) + k(i,353)*QM(i,14) + k(i,359)*QM(i,30) +           &
    k(i,361)*QM(i,31) + k(i,362)*QM(i,32) )
!
  pe(i,86) = pe(i,86) / QM(i,86)
!
  pr(i,87) =  + 2.0*k(i,27)*QM(i,5) + k(i,327)*HM(i,1)*QM(i,9)
  pr(i,87) = pr(i,87) + HM(i,13)*( + k(i,75)*QM(i,35) +           &
    k(i,77)*QM(i,16) + k(i,128)*QM(i,60) +           &
    k(i,76)*QM(i,4)*partHNO3(i) + k(i,126)*QM(i,61) +           &
    0.60*k(i,127)*QM(i,62) ) + QM(i,83)*( +           &
    1.000000*k(i,198)*HM(i,5) + 0.500*k(i,241)*QM(i,21) +           &
    0.516*k(i,259)*QM(i,79) + 0.500*k(i,261)*QM(i,82) +           &
    k(i,262)*QM(i,83) + 1.000000*k(i,281)*HM(i,7) ) + QM(i,16)*( +           &
    k(i,6) + k(i,72) + k(i,7) ) + 2.0*( + k(i,305)*HM(i,2)*QM(i,1) +           &
    k(i,84)*QM(i,5) ) + QM(i,60)*( + k(i,22) + k(i,23) ) +           &
    QM(i,9)*( + k(i,32) + k(i,31) ) + QM(i,62)*( +           &
    0.70*k(i,161)*HM(i,3) + k(i,179) ) + k(i,4)*QM(i,35) +           &
    k(i,5)*QM(i,4)*partHNO3(i) + k(i,36)*QM(i,13) +           &
    k(i,177)*QM(i,61) + k(i,38)*QM(i,15)
!
  pe(i,87) = + HM(i,7)*( + 2.0*k(i,83)*HM(i,6) +           &
    0.500000*k(i,134)*QM(i,56) + k(i,73)*HM(i,14) +           &
    k(i,129)*QM(i,20) + k(i,130)*QM(i,52) + k(i,131)*QM(i,54) +           &
    k(i,132)*QM(i,55) + k(i,133)*QM(i,57) + k(i,135)*QM(i,51) +           &
    k(i,136)*QM(i,42) + k(i,137)*QM(i,42) + k(i,138)*QM(i,43) +           &
    k(i,139)*QM(i,43) + k(i,140)*QM(i,44) + k(i,141)*QM(i,44) +           &
    k(i,142)*QM(i,45) + k(i,143)*QM(i,45) + k(i,144)*QM(i,46) +           &
    k(i,145)*QM(i,46) + k(i,146)*QM(i,47) + k(i,147)*QM(i,47) +           &
    k(i,148)*QM(i,48) + k(i,149)*QM(i,48) ) + HM(i,6)*( +           &
    k(i,68)*HM(i,13) + k(i,71)*HM(i,14) + k(i,162)*QM(i,75) +           &
    k(i,164)*HM(i,17) + k(i,168)*HM(i,18) + k(i,172)*HM(i,19) +           &
    k(i,176)*QM(i,79) + k(i,178)*QM(i,80) + k(i,326)*HM(i,8) +           &
    k(i,331)*HM(i,9) + k(i,356)*HM(i,10) ) + HM(i,5)*( +           &
    2.0*k(i,302)*HM(i,4) + k(i,67)*HM(i,13) +           &
    0.059000*k(i,182)*QM(i,66) + 0.124000*k(i,183)*QM(i,67) +               &
    0.261000*k(i,184)*QM(i,68) + 0.153000*k(i,188)*QM(i,72) +               &
    0.200000*k(i,189)*QM(i,73) + 0.350000*k(i,190)*QM(i,74) +               &
    0.050000*k(i,191)*QM(i,76) + 0.050000*k(i,192)*QM(i,77) )
!
  pe(i,87) = pe(i,87) / QM(i,87)
!
  pr(i,88) =  + 3.0*( + k(i,42)*QM(i,23) + k(i,44)*QM(i,25) +               &
    k(i,46)*QM(i,27) + k(i,337)*HM(i,2)*QM(i,23) +               &
    k(i,339)*HM(i,2)*QM(i,25) + k(i,341)*HM(i,13)*QM(i,27) ) +               &
    QM(i,29)*( + k(i,48) + k(i,343)*HM(i,2) + k(i,344)*HM(i,13) ) +         &
    2.0*( + k(i,43)*QM(i,24) + k(i,338)*HM(i,2)*QM(i,24) ) + 4.0*( +         &
    k(i,45)*QM(i,26) + k(i,340)*HM(i,2)*QM(i,26) ) + QM(i,28)*( +         &
    k(i,47) + k(i,342)*HM(i,13) ) + k(i,327)*HM(i,1)*QM(i,9)
  pr(i,88) = pr(i,88) + 2.0*( + k(i,35)*QM(i,11) +         &
    k(i,33)*QM(i,17) + k(i,334)*QM(i,17) ) + QM(i,10)*( + k(i,30) +         &
    k(i,329)*HM(i,13) + k(i,330)*HM(i,1) ) + QM(i,8)*( +         &
    k(i,34)*partHCl(i) + k(i,325)*HM(i,13)*partHCl(i) ) +         &
    QM(i,9)*( + k(i,31) + k(i,32) ) + k(i,40)*QM(i,7) +         &
    k(i,36)*QM(i,13) + k(i,41)*QM(i,19)
!
  pe(i,88) = + HM(i,8)*( + k(i,324)*HM(i,13) +         &
    2.0*k(i,335)*HM(i,8) + 2.0*k(i,336)*HM(i,8) +         &
    k(i,348)*HM(i,10) + k(i,326)*HM(i,6) + k(i,328)*HM(i,14) +         &
    2.0*k(i,333)*HM(i,8) + k(i,350)*HM(i,10) ) + HM(i,9)*( +         &
    k(i,312)*QM(i,20) + k(i,319)*QM(i,2) + k(i,320)*5.e-7*hnm(i) +         &
    k(i,321)*HM(i,14) + k(i,331)*HM(i,6) + k(i,332)*QM(i,10) )
!
  pe(i,88) = pe(i,88) / QM(i,88)
!
  pr(i,89) =  + HM(i,2)*( + k(i,359)*QM(i,30) + k(i,361)*QM(i,31) +         &
    k(i,362)*QM(i,32) ) + k(i,49)*QM(i,30) + k(i,50)*QM(i,31) +         &
    k(i,51)*QM(i,32) + 2.0*k(i,371)*QM(i,18)*QM(i,14)
  pr(i,89) = pr(i,89) + QM(i,14)*( + k(i,353)*HM(i,2)*partHBr(i) +         &
    k(i,355)*HM(i,13)*partHBr(i) ) + QM(i,18)*( + k(i,39) +         &
    k(i,358)*HM(i,1) ) + k(i,38)*QM(i,15) + k(i,41)*QM(i,19)
!
  pe(i,89) = + HM(i,10)*( + k(i,356)*HM(i,6) + k(i,357)*HM(i,14) +          &
    k(i,350)*HM(i,8) ) + HM(i,11)*( + k(i,315)*QM(i,20) +                   &
    k(i,352)*HM(i,14) )
!
  pe(i,89) = pe(i,89) / QM(i,89)
#ifdef MASSy
!
  pr(i,90) = + 2.0*k(i,305)*HM(i,2)*QM(i,1)
!
  pe(i,90) = + 2.0*k(i,302)*HM(i,4)*HM(i,5)
!
  pe(i,90) = pe(i,90) / QM(i,90)
!
  pr(i,91) =                   &
    3.0*( + k(i,42) + k(i,337)*HM(i,2) )*QM(i,23) +                   &
    2.0*( + k(i,43) + k(i,338)*HM(i,2) )*QM(i,24) +                   &
    3.0*( + k(i,44) + k(i,339)*HM(i,2) )*QM(i,25) +                   &
    4.0*( + k(i,45) + k(i,340)*HM(i,2) )*QM(i,26) +                   &
    3.0*( + k(i,46) + k(i,341)*HM(i,13) )*QM(i,27) +                   &
        ( + k(i,47) + k(i,342)*HM(i,13) )*QM(i,28) +                   &
        ( + k(i,48) + k(i,343)*HM(i,2) +                   &
            k(i,344)*HM(i,13) )*QM(i,29)
!
  pr(i,92) =                   &
    HM(i,2)*( + k(i,359)*QM(i,30) + k(i,361)*QM(i,31) +                   &
    k(i,362)*QM(i,32) ) + k(i,49)*QM(i,30) + k(i,50)*QM(i,31) +           &
    k(i,51)*QM(i,32)
#endif
!
enddo
!$omp end do
!
!$omp end parallel
!
!	   
RETURN
END SUBROUTINE CH_PTERMS
