SUBROUTINE KRATES(pm,t,hnm,tj,het,k,imax,nphot,nhet,nk)


!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_0 
!
!CC DATE : 15/05/2005
!
!
!   CALCUL DES CONTANTES DE VITESSE DES REACTIONS CHIMIQUES
!
!CCCC
!
!  Original : H. Teyssedre, V.-H. Peuch, 08/2000
!
!  Modified : D. Charpin, version Fortran-90, 08/2006



IMPLICIT NONE

!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~

INTEGER :: imax, nphot, nhet, nk
REAL (KIND=8) , DIMENSION (imax) :: pm,t,hnm
REAL (KIND=8) , DIMENSION (imax,nphot):: tj
REAL (KIND=8) , DIMENSION (imax,nhet):: het
REAL (KIND=8) , DIMENSION (imax,nk) :: k

!     Variables locales.
!     ~~~~~~~~~~~~~~~~~

INTEGER :: I
REAL (KIND=8) :: XK0,XE0,XN0,XKINF,XEINF,XMINF,XF,ZK0M
REAL (KIND=8) :: ZKINF,ZFACT,XAEQ,XBEQ


!$omp parallel default(none) &
!$omp private(i,xaeq,xbeq,xk0,xE0,xn0,xkinf,xEinf,xminf,xF,zk0m,zkinf,zfact) &
!$omp shared(imax,k,pm,tj,t,het,hnm)

!$omp do
do i=1,imax

   !  NO2  -->  O3P + NO

   k(i,1) = tj(i,001)

   !  O3  -->  O1D + O2

   k(i,2) = tj(i,002)

   !  O3  -->  O3P + O2

   k(i,3) = tj(i,003)

   !  HONO  -->  OH + NO

   k(i,4) = tj(i,004)

   !  HNO3  -->  OH + NO2

   k(i,5) = tj(i,005)

   !  HNO4  -->  HO2 + NO2

   k(i,6) = 0.65*tj(i,006)

   !  HNO4  -->  OH + NO3

   k(i,7) = 0.35*tj(i,006)

   !  NO3  -->  NO + O2

   k(i,8) = tj(i,007)

   !  NO3  -->  NO2 + O3P

   k(i,9) = tj(i,008)

   !  H2O2  -->  OH + OH

   k(i,10) = tj(i,009)

   !  HCHO  -->  H2 + CO

   k(i,11) = tj(i,010)

   !  HCHO  -->  H + HO2 + CO

   k(i,12) = tj(i,011)

   !  ALD  -->  MO2 + HO2 + CO

   k(i,13) = tj(i,012)

   !  OP1  -->  CH3O + OH

   k(i,14) = tj(i,013)

   !  OP2  -->  ALD + HO2 + OH

   k(i,15) = tj(i,014)

   !  PAA  -->  MO2 + OH

   k(i,16) = tj(i,015)

   !  KET  -->  ACO3 + ETHP

   k(i,17) = tj(i,016)

   !  GLY  -->  0.13*HCHO + 1.87*CO + 0.87*H2

   k(i,18) = tj(i,017)

   !  GLY  -->  0.45*HCHO + 1.55*CO + 0.80*HO2 + 0.15*H2

   k(i,19) = tj(i,018)

   !  MGLY  -->  ACO3 + HO2 + CO

   k(i,20) = tj(i,019)

   !  DCB  -->  TCO3 + HO2

   k(i,21) = tj(i,020)

   !  ONIT  --> ALD + HO2 + NO2

   k(i,22) = 0.20*tj(i,021)

   !  ONIT  --> KET + HO2 + NO2

   k(i,23) = 0.80*tj(i,021)

   !  MACR  -->  HCHO + ACO3 + CO + HO2

   k(i,24) = tj(i,022)

   !  HKET  -->  ACO3 + HCHO + HO2

   k(i,25) = tj(i,023)

   !  O2  -->  O3P + O3P

   k(i,26) = tj(i,024)

   !  N2O5  -->  NOx + NOx + XX

   k(i,27) = tj(i,025)

   !  H2O  -->  H + OH

   k(i,28) = tj(i,026)

   !  N2O  -->  N2 + O1D

   k(i,29) = tj(i,027)

   !  HOCl  -->  Cl + OH

   k(i,30) = tj(i,028)

   !  ClONO2  -->  Cl + NO3

   k(i,31) = tj(i,029)

   !  ClONO2  -->  ClO + NO2

   k(i,32) = tj(i,030)

   !  Cl2O2  -->  Cl + Cl + O2

   k(i,33) = tj(i,031)

   !  HCl  -->  H + Cl

   k(i,34) = tj(i,032)

   !  Cl2  -->  Cl + Cl

   k(i,35) = tj(i,033)

   !  ClNO2  -->  Cl + NO2

   k(i,36) = tj(i,034)

   !  BrO  -->  Br + O3P

   k(i,37) = tj(i,035)

   !  BrONO2  -->  Br + NO3

   k(i,38) = tj(i,036)

   !  HOBr  -->  Br + OH

   k(i,39) = tj(i,037)

   !  OClO  -->  O3P + ClO

   k(i,40) = tj(i,038)

   !  BrCl  -->  Br + Cl

   k(i,41) = tj(i,039)

   !  CFC11  -->  ClOx + ClOx + ClOx + XX

   k(i,42) = tj(i,040)

   !  CFC12  -->  ClOx + ClOx + XX

   k(i,43) = tj(i,041)

   !  CFC113  -->  ClOx + ClOx + ClOx + XX

   k(i,44) = tj(i,042)

   !  CCl4  -->  ClOx + ClOx + ClOx + ClOx + XX

   k(i,45) = tj(i,043)

   !  CH3CCl3  -->  ClOx + ClOx + ClOx + XX

   k(i,46) = tj(i,044)

   !  CH3Cl  -->  ClOx + XX

   k(i,47) = tj(i,045)

   !  HCFC22  -->  ClOx + XX

   k(i,48) = tj(i,046)

   !  CH3Br  -->  BrOx + XX

   k(i,49) = tj(i,047)

   !  H1211  -->  BrOx + XX

   k(i,50) = tj(i,048)

enddo
!$omp end do

!$omp do
do i=1,imax

   !  H1301  -->  BrOx + XX

   k(i,51) = tj(i,049)

   !  NO  -->  N + O3P

   k(i,52) = tj(i,050)

   !  O3P + O2  -->  O3

   k(i,53) = hnm(i)*6.00E-34*(t(i)/300)**(-2.3)

   !  O3P + O3  -->  O2 + O2

   k(i,54) = 8.00E-12*exp(-(2060.0/t(i)))

   !  O1D + N2  -->  O3P + N2

   k(i,55) = 1.80E-11*exp(-(-110.0/t(i)))

   !  O1D + O2  -->  O3P + O2

   k(i,56) = 3.20E-11*exp(-(-70.0/t(i)))

   !  O1D + H2O  -->  OH + OH

   k(i,57) = 2.20E-10

   !  O3 + OH  -->  HO2 + O2

   k(i,58) = 1.60E-12*exp(-(940.0/t(i)))

   !  O3 + HO2  -->  OH + O2 + O2

   k(i,59) = 1.10E-14*exp(-(500.0/t(i)))

   !  OH + HO2  -->  H2O + O2

   k(i,60) = 4.80E-11*exp(-(-250.0/t(i)))

   !  H2O2 + OH  -->  HO2 + H2O

   k(i,61) = 2.90E-12*exp(-(160.0/t(i)))

   !  HO2 + HO2  -->  H2O2 + O2

   k(i,62) = 2.3E-13*EXP(600./t(i))

   ! HO2 + HO2 + H2O --> H2O2 + H2O + O2

   k(i,63) = 3.22E-34*EXP(2800./t(i))+2.38E-30*1.E-24*hnm(i)*EXP(3200./t(i))

   !  O3P + NO  -->  NO2

   xk0   = 9.00E-32
   xE0   = 0.0
   xn0   = 1.5
   xkinf = 3.00E-11
   xEinf = 0.0
   xminf = 0.0
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,64) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  O3P + NO2  -->  NO + O2

   k(i,65) = 6.50E-12*exp(-(-120.0/t(i)))

   !  O3P + NO2  -->  NO3

   xk0   = 9.00E-32
   xE0   = 0.0
   xn0   = 2.0
   xkinf = 2.20E-11
   xEinf = 0.0
   xminf = 0.0
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,66) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  OH + NO  -->  HONO

   xk0   = 7.00E-31
   xE0   = 0.0
   xn0   = 2.6
   xkinf = 3.60E-11
   xEinf = 0.0
   xminf = 0.1
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,67) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  OH + NO2  -->  HNO3

   xk0   = 2.50E-30
   xE0   = 0.0
   xn0   = 4.4
   xkinf = 1.60E-11
   xEinf = 0.0
   xminf = 1.7
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,68) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  OH + NO3  -->  NO2 + HO2

   k(i,69) = 2.20E-11

   !  HO2 + NO  -->  NO2 + OH

   k(i,70) = 3.50E-12*exp(-(-250.0/t(i)))

   !  HO2 + NO2  -->  HNO4

   xk0   = 1.80E-31
   xE0   = 0.0
   xn0   = 3.2
   xkinf = 4.70E-12
   xEinf = 0.0
   xminf = 1.4
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,71) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  HNO4  -->  HO2 + NO2

   xk0   = 1.80E-31
   xn0   = 3.2
   xkinf = 4.70E-12
   xminf = 1.4
   xaeq  = 2.10E-27
   xbeq  = 10900.0
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
   zkinf = xkinf*((t(i)/300.)**(-xminf))
   zfact = 0.6**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,72) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact

   !  HO2 + NO3  --> HNO3

   k(i,73) = 0.3*3.50E-12

   !  HO2 + NO3  --> NO2 + OH

   k(i,74) = 0.7*3.50E-12

   !  OH + HONO  -->  H2O + NO2

   k(i,75) = 1.80E-11*exp(-(390.0/t(i)))

   !  OH + HNO3  -->  NO3 + H2O

   k(i,76) = (2.4E-14*EXP(460/t(i)))+(6.5E-34*EXP(1335/t(i))*hnm(i))  &
       /(1+(6.5E-34*EXP(1335/t(i))*hnm(i))/(2.7E-17*EXP(2199/t(i))))

   !  OH + HNO4  -->  NO2 + H2O + O2

   k(i,77) = 1.30E-12*exp(-(-380.0/t(i)))

   !  O3 + NO  -->  NO2 + O2

   k(i,78) = 2.00E-12*exp(-(1400.0/t(i)))

   !  O3 + NO2  -->  NO3 + O2

   k(i,79) = 1.20E-13*exp(-(2450.0/t(i)))

   !  NO + NO + O2  -->  NO2 + NO2

   k(i,80) = 3.30E-39*exp(-(-530.0/t(i)))

   !  NO3 + NO  -->  NO2 + NO2

   k(i,81) = 1.50E-11*exp(-(-170.0/t(i)))

   !  NO3 + NO2  -->  NO + NO2 + O2

   k(i,82) = 4.50E-14*exp(-(1260.0/t(i)))

   !  NO3 + NO2  -->  N2O5

   xk0   = 2.20E-30
   xE0   = 0.0
   xn0   = 3.9
   xkinf = 1.50E-12
   xEinf = 0.0
   xminf = 0.7
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,83) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  N2O5  -->  NO2 + NO3

   xk0   = 2.20E-30
   xn0   = 3.9
   xkinf = 1.50E-12
   xminf = 0.7
   xaeq  = 2.70E-27
   xbeq  = 11000.0
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
   zkinf = xkinf*((t(i)/300.)**(-xminf))
   zfact = 0.6**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,84) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact

   !  NO3 + NO3  -->  NO2 + NO2 + O2

   k(i,85) = 8.50E-13*exp(-(2450.0/t(i)))

   !  OH + H2  -->  H2O + H

   k(i,86) = 5.50E-12*exp(-(2000.0/t(i)))

   !  OH + SO2  -->  SULF + HO2

   xk0   = 3.00E-31
   xE0   = 0.0
   xn0   = 3.3
   xkinf = 1.50E-12
   xEinf = 0.0
   xminf = 0.0
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,87) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  CO + OH  -->  H + CO2

   k(i,88) = 1.5E-13*(1.+0.6*pm(i)/101325.)

   !  ISO + O3P  -->  0.86*OLT + 0.05*HCHO + 0.02*OH + 0.01*CO +
   !                0.13*DCB + 0.28*HO2 + 0.15*XO2

   k(i,89) = 6.00E-11

   !  MACR + O3P  -->  ALD

   k(i,90) = 1.59E-11*exp(-(-13.0/t(i)))

   !  CH4 + OH  -->  CH3 + H2O

   k(i,91) = 2.45e-12*exp(-1775./t(i))

   !  ETH + OH  -->  ETHP + H2O

   k(i,92) = t(i)*t(i)*1.51E-17*exp(-(492./t(i)))

   !  HC3 + OH  -->  0.583*HC3P + 0.381*HO2 + 0.335*ALD +
   !                0.036*ORA1 + 0.036*CO + 0.036*GLY + 0.036*OH + 0.010*HCHO + H2O

   k(i,93) = 5.26E-12*exp(-(260.0/t(i)))

   !  HC5 + OH  --> HC5P + H2O

   k(i,94) = 0.750*8.02E-12*exp(-(155.0/t(i)))

   !  HC5 + OH  --> KET + HO2 + H2O

   k(i,95) = 0.250*8.02E-12*exp(-(155.0/t(i)))

   !  HC8 + OH  -->  0.951*HC8P + 0.025*ALD + 0.024*HKET + 0.049*HO2 + H2O

   k(i,96) = 1.64E-11*exp(-(125.0/t(i)))

   !  ETE + OH  -->  ETEP

   k(i,97) = 1.96E-12*exp(-(-438.0/t(i)))

   !  OLT + OH  -->  OLTP

   k(i,98) = 5.72E-12*exp(-(-500.0/t(i)))

   !  OLI + OH  -->  OLIP

   k(i,99) = 1.33E-11*exp(-(-500.0/t(i)))

   !  DIEN + OH  -->  ISOP

   k(i,100) = 1.48E-11*exp(-(-448.0/t(i)))

enddo
!$omp end do

!$omp do
do i=1,imax

   !  ISO + OH  -->  ISOP

   k(i,101) = 2.54E-11*exp(-(-410.0/t(i)))

   !  API + OH  -->  APIP

   k(i,102) = 1.21E-11*exp(-(-444.0/t(i)))

   !  LIM + OH  -->  LIMP

   k(i,103) = 1.71E-10

   !  TOL + OH  --> ADDT

   k(i,104) = 0.90*1.81E-12*exp(-(-355.0/t(i)))

   !  TOL + OH  --> XO2 + HO2

   k(i,105) = 0.10*1.81E-12*exp(-(-355.0/t(i)))

   !  XYL + OH  --> ADDX

   k(i,106) = 0.90*7.30E-12*exp(-(-355.0/t(i)))

   !  XYL + OH  --> XO2 + HO2

   k(i,107) = 0.10*7.30E-12*exp(-(-355.0/t(i)))

   !  CSL + OH  --> ADDC

   k(i,108) = 0.85*6.00E-11

   !  CSL + OH  --> PHO

   k(i,109) = 0.10*6.00E-11

   !  CSL + OH  --> HO2 + XO2

   k(i,110) = 0.05*6.00E-11

   !  HCHO + OH  -->  HO2 + CO + H2O

   k(i,111) = 1.00E-11

   !  ALD + OH  -->  ACO3 + H2O

   k(i,112) = 5.55E-12*exp(-(-331.0/t(i)))

   !  KET + OH  -->  KETP + H2O

   k(i,113) = t(i)*t(i)*5.68E-18*exp(-(-92.0/t(i)))

   !  HKET + OH  -->  HO2 + MGLY + H2O

   k(i,114) = 3.00E-12

   !  GLY + OH  -->  HO2 + CO + CO + H2O

   k(i,115) = 1.14E-11

   !  MGLY + OH  -->  ACO3 + CO + H2O

   k(i,116) = 1.72E-11

   !  MACR + OH  -->  0.51*TCO3 + 0.41*HKET + 0.08*MGLY + 0.41*CO +
   !                0.08*HCHO + 0.49*HO2 + 0.49*XO2

   k(i,117) = 1.86E-11*exp(-(-175.0/t(i)))

   !  OH + DCB  -->  0.50*TCO3 + 0.50*HO2 + 0.50*XO2 + 0.35*UDD +
   !                0.15*GLY + 0.15*MGLY

   k(i,118) = 2.80E-11*exp(-(-175./t(i)))

   !  OH + UDD  --> ALD + HO2

   k(i,119) = 0.88*2.70E-10

   !  OH + UDD  --> KET + HO2

   k(i,120) = 0.12*2.70E-10

   !  OP1 + OH  --> MO2 + H2O

   k(i,121) = 0.7*3.8e-12*exp(200./t(i))

   !  OP1 + OH  --> HCHO + OH + H2O

   k(i,122) = 0.3*3.8e-12*exp(200./t(i))

   !  OP2 + OH  -->  0.44*HC3P + 0.08*ALD + 0.49*OH + 0.07*XO2 + 0.41*KET

   k(i,123) = 3.40E-12*exp(-(-190.0/t(i)))

   !  PAA + OH  --> ACO3

   k(i,124) = 0.65*2.93E-12*exp(-(-190.0/t(i)))

   !  PAA + OH  --> HO2 + HCHO + XO2

   k(i,125) = 0.35*2.93E-12*exp(-(-190.0/t(i)))

   !  PAN + OH  -->  HCHO + NO3 + XO2 + H2O

   k(i,126) = 4.00E-14

   !  TPAN + OH  -->  0.60*HKET + 0.60*NO3 + 0.40*PAN + 0.40*HCHO + 0.40*HO2 + XO2

   k(i,127) = 3.25E-13*exp(-(-500.0/t(i)))

   !  ONIT + OH  -->  HC3P + NO2 + H2O

   k(i,128) = 5.31E-12*exp(-(260.0/t(i)))

   !  HCHO + NO3  -->  HO2 + HNO3 + CO

   k(i,129) = 3.40E-13*exp(-(1900.0/t(i)))

   !  ALD + NO3  -->  ACO3 + HNO3

   k(i,130) = 1.40E-12*exp(-(1900.0/t(i)))

   !  GLY + NO3  -->  HNO3 + HO2 + CO + CO

   k(i,131) = 2.90E-12*exp(-(1900.0/t(i)))

   !  MGLY + NO3  -->  HNO3 + ACO3 + CO

   k(i,132) = 1.40E-12*exp(-(1900.0/t(i)))

   !  MACR + NO3  -->  0.20*TCO3 + 0.20*HNO3 + 0.80*OLNN + 0.80*CO

   k(i,133) = 8.27E-15*exp(-(150.0/t(i)))

   !  DCB + NO3  -->  0.50*TCO3 + 0.50*HO2 + 0.50*XO2 + 0.25*GLY +
   !                0.25*ALD + 0.50*NO2 + 0.03*KET + 0.25*MGLY + 0.50*HNO3

   k(i,134) = 2.87E-13*exp(-(1000.0/t(i)))

   !  CSL + NO3  -->  HNO3 + PHO

   k(i,135) = 2.20E-11

   !  ETE + NO3  --> OLNN

   k(i,136) = 0.80*t(i)*t(i)*4.88E-18*exp(-(2282.0/t(i)))

   !  ETE + NO3  --> OLND

   k(i,137) = 0.20*t(i)*t(i)*4.88E-18*exp(-(2282.0/t(i)))

   !  OLT + NO3  --> OLNN

   k(i,138) = 0.43*1.79E-13*exp(-(450.0/t(i)))

   !  OLT + NO3  --> OLND

   k(i,139) = 0.57*1.79E-13*exp(-(450.0/t(i)))

   !  OLI + NO3  --> OLNN

   k(i,140) = 0.11*8.64E-13*exp(-(-450./t(i)))

   !  OLI + NO3  --> OLND

   k(i,141) = 0.89*8.64E-13*exp(-(-450./t(i)))

   !  DIEN + NO3  --> OLNN + MACR

   k(i,142) = 0.90*1.00E-13

   !  DIEN + NO3  --> OLND

   k(i,143) = 0.10*1.00E-13

   !  ISO + NO3  --> OLNN + MACR

   k(i,144) = 0.90*4.00E-12*exp(-(446./t(i)))

   !  ISO + NO3  --> OLND

   k(i,145) = 0.10*4.00E-12*exp(-(446./t(i)))

   !  API + NO3  --> OLNN

   k(i,146) = 0.10*1.19E-12*exp(-(-490.0/t(i)))

   !  API + NO3  --> OLND

   k(i,147) = 0.90*1.19E-12*exp(-(-490.0/t(i)))

   !  LIM + NO3  --> OLNN

   k(i,148) = 0.13*1.22E-11

   !  LIM + NO3  --> OLND

   k(i,149) = 0.87*1.22E-11

   !  TPAN + NO3  -->  ONIT + NO3 + XO2

   k(i,150) = 0.60*2.20E-14*exp(-(500.0/t(i)))

enddo
!$omp end do

!$omp do
do i=1,imax

   !  TPAN + NO3  -->  PAN + HCHO + NO2 + XO2

   k(i,151) = 0.40*2.20E-14*exp(-(500.0/t(i)))

   !  ETE + O3  -->  HCHO + 0.43*CO + 0.37*ORA1 + 0.26*HO2 + 0.13*H2 + 0.12*OH

   k(i,152) = 9.14E-15*exp(-(2580.0/t(i)))

   !  OLT + O3  -->  0.64*HCHO + 0.44*ALD + 0.37*CO + 0.14*ORA1 +
   !                0.10*ORA2 + 0.25*HO2 + 0.40*OH + 0.03*KET + 0.03*KETP +
   !                0.06*CH4 + 0.05*H2 + 0.03*ETH + 0.006*H2O2 + 0.19*MO2 +
   !                0.10*ETHP

   k(i,153) = 4.33E-15*exp(-(1800.0/t(i)))

   !  OLI + O3  -->  0.02*HCHO + 0.99*ALD + 0.16*KET + 0.30*CO +
   !                0.011*H2O2 + 0.14*ORA2 + 0.07*CH4 + 0.22*HO2 + 0.63*OH +
   !                0.23*MO2 + 0.12*KETP + 0.06*ETH + 0.18*ETHP

   k(i,154) = 4.40E-15*exp(-(845.0/t(i)))

   !  DIEN + O3  -->  0.90*HCHO + 0.39*MACR + 0.36*CO + 0.15*ORA1 +
   !                0.09*O3P + 0.30*HO2 + 0.35*OLT + 0.28*OH + 0.05*H2 + 0.15*ACO3
   !                0.03*MO2 + 0.02*KETP + 0.13*XO2 + 0.001*H2O2

   k(i,155) = 1.34E-14*exp(-(2283.0/t(i)))

   !  ISO + O3  -->  0.90*HCHO + 0.39*MACR + 0.36*CO + 0.15*ORA1 +
   !                0.09*O3P + 0.30*HO2 + 0.35*OLT + 0.28*OH + 0.05*H2 + 0.15*ACO3
   !                0.03*MO2 + 0.02*KETP + 0.13*XO2 + 0.001*H2O2

   k(i,156) = 7.86E-15*exp(-(1913.0/t(i)))

   !  API + O3  -->  0.65*ALD + 0.53*KET + 0.14*CO + 0.20*ETHP +
   !                0.42*KETP + 0.85*OH + 0.10*HO2 + 0.02*H2O2

   k(i,157) = 1.01E-15*exp(-(732.0/t(i)))

   !  LIM + O3  -->  0.04*HCHO + 0.46*OLT + 0.14*CO + 0.16*ETHP +
   !                0.42*KETP + 0.85*OH + 0.10*HO2 + 0.02*H2O2 + 0.79*MACR +
   !                0.01*ORA1 + 0.07*ORA2

   k(i,158) = 2.00E-16

   !  MACR + O3  -->  0.40*HCHO + 0.60*MGLY + 0.13*ORA2 + 0.54*CO +
   !                0.08*H2 + 0.22*ORA1 + 0.29*HO2 + 0.07*OH + 0.13*OP2 + 0.13*ACO3

   k(i,159) = 1.36E-15*exp(-(2112.0/t(i)))

   !  DCB + O3  -->  0.21*OH + 0.29*HO2 + 0.66*CO + 0.50*GLY +
   !                0.62*MGLY + 0.28*ACO3 + 0.16*ALD + 0.11*PAA + 0.11*ORA1 +
   !                0.21*ORA2

   k(i,160) = 2.00E-18

   !  TPAN + O3  -->  0.70*HCHO + 0.30*PAN + 0.70*NO2 + 0.13*CO +
   !                0.04*H2 + 0.11*ORA1 + 0.08*HO2 + 0.036*OH + 0.70*ACO3

   k(i,161) = 2.46E-15*exp(-(1700.0/t(i)))

   !  PHO + NO2  -->  0.10*CSL + ONIT

   k(i,162) = 2.00E-11

   !  PHO + HO2  -->  CSL

   k(i,163) = 1.00E-11

   !  ADDT + NO2  -->  CSL + HONO

   k(i,164) = 3.60E-11

   !  ADDT + O2  --> TOLP

   k(i,165) = 0.98*1.66E-17*exp(-(-1044.0/t(i)))

   !  ADDT + O2  --> CSL + HO2

   k(i,166) = 0.02*1.66E-17*exp(-(-1044.0/t(i)))

   !  ADDT + O3  -->  CSL + OH

   k(i,167) = 5.00E-11

   !  ADDX + NO2  -->  CSL + HONO

   k(i,168) = 3.60E-11

   !  ADDX + O2  --> XYLP

   k(i,169) = 0.98*1.66E-17*exp(-(-1044.0/t(i)))

   !  ADDX + O2  --> CSL + HO2

   k(i,170) = 0.02*1.66E-17*exp(-(-1044.0/t(i)))

   !  ADDX + O3  -->  CSL + OH

   k(i,171) = 1.00E-11

   !  ADDC + NO2  -->  CSL + HONO

   k(i,172) = 3.60E-11

   !  ADDC + O2  --> CSLP

   k(i,173) = 0.98*1.66E-17*exp(-(-1044.0/t(i)))

   !  ADDC + O2  --> CSL + HO2

   k(i,174) = 0.02*1.66E-17*exp(-(-1044.0/t(i)))

   !  ADDC + O3  -->  CSL + OH

   k(i,175) = 5.00E-11

   !  ACO3 + NO2  -->  PAN

   xk0   = 9.70E-29
   xE0   = 0.0
   xn0   = 5.6
   xkinf = 9.30E-12
   xEinf = 0.0
   xminf = 1.5
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,176) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  PAN  -->  ACO3 + NO2

   xk0   = 9.70E-29
   xn0   = 5.6
   xkinf = 9.30E-12
   xminf = 1.5
   xaeq  = 8.60E-29
   xbeq  = 13954.0
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
   zkinf = xkinf*((t(i)/300.)**(-xminf))
   zfact = 0.6**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,177) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact

   !  TCO3 + NO2  -->  TPAN

   xk0   = 9.70E-29
   xE0   = 0.0
   xn0   = 5.6
   xkinf = 9.30E-12
   xEinf = 0.0
   xminf = 1.5
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,178) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  TPAN  -->  TCO3 + NO2

   xk0   = 9.70E-29
   xn0   = 5.6
   xkinf = 9.30E-12
   xminf = 1.5
   xaeq  = 8.60E-29
   xbeq  = 13954.0
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
   zkinf = xkinf*((t(i)/300.)**(-xminf))
   zfact = 0.6**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,179) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact

   !  MO2 + NO  -->  CH3O + NO2

   k(i,180) = 3.0e-12*exp(280./t(i))

   !  ETHP + NO  -->  ALD + HO2 + NO2

   k(i,181) = 8.70E-12

   !  HC3P + NO  -->  0.047*HCHO + 0.233*ALD + 0.623*KET +
   !                0.063*GLY + 0.742*HO2 + 0.150*MO2 + 0.048*ETHP + 0.048*XO2 +
   !                0.059*ONIT + 0.941*NO2

   k(i,182) = 4.00E-12

   !  HC5P + NO  -->  0.021*HCHO + 0.211*ALD + 0.722*KET +
   !                0.599*HO2 + 0.031*MO2 + 0.245*ETHP + 0.334*XO2 + 0.124*ONIT +
   !                0.876*NO2

   k(i,183) = 4.00E-12

   !  HC8P + NO  -->  0.150*ALD + 0.642*KET + 0.133*ETHP +
   !                0.261*ONIT + 0.739*NO2 + 0.606*HO2 + 0.416*XO2

   k(i,184) = 4.00E-12

   !  ETEP + NO  -->  1.6*HCHO + HO2 + NO2 + 0.2*ALD

   k(i,185) = 9.00E-12

   !  OLTP + NO  -->  0.94*ALD + HCHO + HO2 + NO2 + 0.06*KET

   k(i,186) = 4.00E-12

   !  OLIP + NO  -->  HO2 + 1.71*ALD + 0.29*KET + NO2

   k(i,187) = 4.00E-12

   !  ISOP + NO  -->  0.446*MACR + 0.354*OLT + 0.847*HO2 +
   !                0.847*NO2 + 0.153*ONIT + 0.606*HCHO

   k(i,188) = 4.00E-12

   !  APIP + NO  -->  0.80*HO2 + 0.80*ALD + 0.80*NO2 + 0.80*KET + 0.20*ONIT

   k(i,189) = 4.00E-12

   !  LIMP + NO  -->  0.65*HO2 + 0.40*MACR + 0.25*OLI + 0.25*HCHO +
   !                0.65*NO2 + 0.35*ONIT

   k(i,190) = 4.00E-12

   !  TOLP + NO  -->  0.95*NO2 + 0.95*HO2 + 1.20*GLY + 0.50*DCB +
   !                0.05*ONIT + 0.65*MGLY

   k(i,191) = 4.00E-12

   !  XYLP + NO  -->  0.95*NO2 + 0.95*HO2 + 0.35*GLY + 0.95*DCB +
   !                0.05*ONIT + 0.60*MGLY

   k(i,192) = 4.00E-12

   !  CSLP + NO  -->  GLY + MGLY + NO2 + HO2

   k(i,193) = 4.00E-12

   !  ACO3 + NO  -->  MO2 + NO2

   k(i,194) = 2.00E-11

   !  TCO3 + NO  -->  NO2 + ACO3 + HCHO

   k(i,195) = 2.00E-11

   !  KETP + NO  -->  0.54*MGLY + 0.46*ALD + 0.23*ACO3 + 0.77*HO2 + 0.16*XO2 + NO2

   k(i,196) = 4.00E-12

   !  OLNN + NO  -->  ONIT + NO2 + HO2

   k(i,197) = 4.00E-12

   !  OLND + NO  -->  0.287*HCHO + 1.24*ALD + 0.464*KET + 2.0*NO2

   k(i,198) = 4.00E-12

   !  MO2 + HO2  -->  OP1

   k(i,199) = 3.80E-13*exp(-(-800.0/t(i)))

   !  ETHP + HO2  -->  OP2

   k(i,200) = 7.50E-13*exp(-(-700.0/t(i)))

enddo
!$omp end do

!$omp do
do i=1,imax

   !  HC3P + HO2  -->  OP2

   k(i,201) = 1.66E-13*exp(-(-1300.0/t(i)))

   !  HC5P + HO2  -->  OP2

   k(i,202) = 1.66E-13*exp(-(-1300.0/t(i)))

   !  HC8P + HO2  -->  OP2

   k(i,203) = 1.66E-13*exp(-(-1300.0/t(i)))

   !  ETEP + HO2  -->  OP2

   k(i,204) = 1.90E-13*exp(-(-1300.0/t(i)))

   !  OLTP + HO2  -->  OP2

   k(i,205) = 1.66E-13*exp(-(-1300.0/t(i)))

   !  OLIP + HO2  -->  OP2

   k(i,206) = 1.66E-13*exp(-(-1300.0/t(i)))

   !  ISOP + HO2  -->  OP2

   k(i,207) = 1.28E-13*exp(-(-1300.0/t(i)))

   !  APIP + HO2  -->  OP2

   k(i,208) = 1.50E-11

   !  LIMP + HO2  -->  OP2

   k(i,209) = 1.50E-11

   !  TOLP + HO2  -->  OP2

   k(i,210) = 3.75E-13*exp(-(-980.0/t(i)))

   !  XYLP + HO2  -->  OP2

   k(i,211) = 3.75E-13*exp(-(-980.0/t(i)))

   !  CSLP + HO2  -->  OP2

   k(i,212) = 3.75E-13*exp(-(-980.0/t(i)))

   !  ACO3 + HO2  -->  PAA

   k(i,213) = 1.15E-12*exp(-(-550.0/t(i)))

   !  ACO3 + HO2  -->  ORA2 + O3

   k(i,214) = 3.86E-16*exp(-(-2640.0/t(i)))

   !  TCO3 + HO2  -->  OP2

   k(i,215) = 1.15E-12*exp(-(-550.0/t(i)))

   !  TCO3 + HO2  -->  ORA2 + O3

   k(i,216) = 3.86E-16*exp(-(-2640.0/t(i)))

   !  KETP + HO2  -->  OP2

   k(i,217) = 1.15E-13*exp(-(-1300.0/t(i)))

   !  OLNN + HO2  -->  ONIT

   k(i,218) = 1.66E-13*exp(-(-1300.0/t(i)))

   !  OLND + HO2  -->  ONIT

   k(i,219) = 1.66E-13*exp(-(-1300.0/t(i)))

   !  MO2 + MO2  -->  CH3O + CH3O + O2

   k(i,220) = 0.35*2.5e-13*exp(190./t(i))

   !  MO2 + MO2  -->  HCHO  + O2 + XX

   k(i,221) = 0.65*2.5e-13*exp(190./t(i))

   !  ETHP + MO2  -->  0.75*HCHO + HO2 + 0.75*ALD

   k(i,222) = 1.18E-13*exp(-(-158.0/t(i)))

   !  HC3P + MO2  -->  0.810*HCHO + 0.992*HO2 + 0.580*ALD +
   !                0.018*KET + 0.007*MO2 + 0.005*MGLY + 0.085*XO2 + 0.119*GLY

   k(i,223) = 9.46E-14*exp(-(-431.0/t(i)))

   !  HC5P + MO2  -->  0.829*HCHO + 0.946*HO2 + 0.523*ALD +
   !                0.240*KET + 0.014*ETHP + 0.245*XO2 + 0.049*MO2

   k(i,224) = 1.00E-13*exp(-(-467.0/t(i)))

   !  HC8P + MO2  -->  0.753*HCHO + 0.993*HO2 + 0.411*ALD +
   !                0.419*KET + 0.322*XO2 + 0.013*ETHP

   k(i,225) = 4.34E-14*exp(-(-633./t(i)))

   !  ETEP + MO2  -->  1.55*HCHO + HO2 + 0.35*ALD

   k(i,226) = 1.71E-13*exp(-(-708.0/t(i)))

   !  OLTP + MO2  -->  1.25*HCHO + HO2 + 0.669*ALD + 0.081*KET

   k(i,227) = 1.46E-13*exp(-(-708.0/t(i)))

   !  OLIP + MO2  -->  0.755*HCHO + HO2 + 0.932*ALD + 0.313*KET

   k(i,228) = 9.18E-14*exp(-(-708.0/t(i)))

   !  ISOP + MO2  -->  0.550*MACR + 0.370*OLT + HO2 + 0.080*OLI + 1.09*HCHO

   k(i,229) = 1.36E-13*exp(-(-708.0/t(i)))

   !  APIP + MO2  -->  HO2 + HO2 + ALD + HCHO + KET

   k(i,230) = 3.56E-14*exp(-(-708.0/t(i)))

   !  LIMP + MO2  -->  2.00*HO2 + 0.60*MACR + 0.40*OLI + 1.40*HCHO

   k(i,231) = 3.56E-14*exp(-(-708.0/t(i)))

   !  TOLP + MO2  -->  HCHO + HO2 + 0.65*GLY + DCB + 0.35*MGLY

   k(i,232) = 3.56E-14*exp(-(-708.0/t(i)))

   !  XYLP + MO2  -->  HCHO + HO2 + 0.37*GLY + DCB + 0.63*MGLY

   k(i,233) = 3.56E-14*exp(-(-708.0/t(i)))

   !  CSLP + MO2  -->  GLY + MGLY + HO2 + HO2 + HCHO

   k(i,234) = 3.56E-14*exp(-(-708.0/t(i)))

   !  ACO3 + MO2  -->  HCHO + HO2 + MO2

   k(i,235) = 3.21E-11*exp(-(440.0/t(i)))

   !  ACO3 + MO2  -->  HCHO + ORA2

   k(i,236) = 2.68E-16*exp(-(-2510.0/t(i)))

   !  TCO3 + MO2  -->  ACO3 + HCHO + HCHO + HO2

   k(i,237) = 3.21E-11*exp(-(440.0/t(i)))

   !  TCO3 + MO2  -->  HCHO + ORA2

   k(i,238) = 2.68E-16*exp(-(-2510.0/t(i)))

   !  KETP + MO2  -->  0.40*MGLY + 0.30*ALD + 0.30*HKET + 0.88*HO2 +
   !                0.08*XO2 + 0.12*ACO3 + 0.75*HCHO

   k(i,239) = 6.91E-13*exp(-(-508.0/t(i)))

   !  OLNN + MO2  -->  0.75*HCHO + HO2 + ONIT

   k(i,240) = 1.60E-13*exp(-(-708.0/t(i)))

   !  OLND + MO2  -->  0.960*HCHO + 0.500*HO2 + 0.640*ALD +
   !                0.149*KET + 0.500*NO2 + 0.500*ONIT

   k(i,241) = 9.68E-14*exp(-(-708.0/t(i)))

   !  ETHP + ACO3  -->  ALD + 0.500*HO2 + 0.5*MO2 + 0.5*ORA2

   k(i,242) = 1.03E-12*exp(-(-211.0/t(i)))

   !  HC3P + ACO3  -->  0.724*ALD + 0.127*KET + 0.488*HO2 +
   !                0.508*MO2 + 0.499*ORA2 + 0.091*HCHO + 0.006*ETHP + 0.071*XO2 +
   !                0.004*MGLY + 0.100*GLY

   k(i,243) = 6.90E-13*exp(-(-460.0/t(i)))

   !  HC5P + ACO3  -->  0.677*ALD + 0.330*KET + 0.438*HO2 +
   !                0.554*MO2 + 0.495*ORA2 + 0.076*HCHO + 0.018*ETHP + 0.237*XO2

   k(i,244) = 5.59E-13*exp(-(-522.0/t(i)))

   !  HC8P + ACO3  -->  0.497*ALD + 0.581*KET + 0.489*HO2 +
   !                0.507*MO2 + 0.495*ORA2 + 0.015*ETHP + 0.318*XO2

   k(i,245) = 2.47E-13*exp(-(-683./t(i)))

   !  ETEP + ACO3  -->  0.8*HCHO + 0.6*ALD + 0.5*HO2 + 0.5*MO2 + 0.5*ORA2

   k(i,246) = 9.48E-13*exp(-(-765.0/t(i)))

   !  OLTP + ACO3  -->  0.859*ALD + 0.501*HCHO + 0.501*HO2 +
   !                0.501*MO2 + 0.499*ORA2 + 0.141*KET

   k(i,247) = 8.11E-13*exp(-(-765.0/t(i)))

   !  OLIP + ACO3  -->  0.941*ALD + 0.510*HO2 + 0.510*MO2 + 0.569*KET + 0.490*ORA2

   k(i,248) = 5.09E-13*exp(-(-765.0/t(i)))

   !  ISOP + ACO3  -->  0.771*MACR + 0.229*OLT + 0.506*HO2 +
   !                0.494*ORA2 + 0.340*HCHO + 0.506*MO2

   k(i,249) = 7.60E-13*exp(-(-765.0/t(i)))

   !  APIP + ACO3  -->  HO2 + ALD + MO2 + KET

   k(i,250) = 7.40E-13*exp(-(-765.0/t(i)))

enddo
!$omp end do

!$omp do
do i=1,imax

   !  LIMP + ACO3  -->  HO2 + 0.60*MACR + 0.40*OLI + 0.40*HCHO + MO2

   k(i,251) = 7.40E-13*exp(-(-765.0/t(i)))

   !  TOLP + ACO3  -->  MO2 + HO2 + DCB + 0.65*GLY + 0.35*MGLY

   k(i,252) = 7.40E-13*exp(-(-765.0/t(i)))

   !  XYLP + ACO3  -->  MO2 + HO2 + DCB + 0.37*GLY + 0.63*MGLY

   k(i,253) = 7.40E-13*exp(-(-765.0/t(i)))

   !  CSLP + ACO3  -->  GLY + MGLY + HO2 + MO2

   k(i,254) = 7.40E-13*exp(-(-765.0/t(i)))

   !  ACO3 + ACO3  -->  MO2 + MO2

   k(i,255) = 2.80E-12*exp(-(-530.0/t(i)))

   !  TCO3 + ACO3  -->  MO2 + ACO3 + HCHO

   k(i,256) = 2.80E-12*exp(-(-530.0/t(i)))

   !  KETP + ACO3  -->  0.54*MGLY + 0.35*ALD + 0.11*KET +
   !                0.12*ACO3 + 0.38*HO2 + 0.08*XO2 + 0.50*MO2 + 0.50*ORA2

   k(i,257) = 7.51E-13*exp(-(-565.0/t(i)))

   !  OLNN + ACO3  -->  ONIT + 0.50*MO2 + 0.50*ORA2 + 0.50*HO2

   k(i,258) = 0.50*8.85E-13*exp(-(-765.0/t(i)))

   !  OLND + ACO3  -->  0.207*HCHO + 0.650*ALD + 0.167*KET +
   !                0.516*MO2 + 0.484*ORA2 + 0.484*ONIT + 0.516*NO2

   k(i,259) = 5.37E-13*exp(-(-765.0/t(i)))

   !  OLNN + OLNN  -->  ONIT + ONIT + HO2

   k(i,260) = 7.00E-14*exp(-(-1000.0/t(i)))

   !  OLNN + OLND  -->  0.202*HCHO + 0.640*ALD + 0.149*KET +
   !                0.500*HO2 + 0.500*NO2 + 1.50*ONIT

   k(i,261) = 4.25E-14*exp(-(-1000.0/t(i)))

   !  OLND + OLND  -->  0.504*HCHO + 1.21*ALD + 0.285*KET + ONIT + NO2

   k(i,262) = 2.96E-14*exp(-(-1000.0/t(i)))

   !  MO2 + NO3  -->  HCHO + HO2 + NO2

   k(i,263) = 1.20E-12

   !  ETHP + NO3  -->  ALD + HO2 + NO2

   k(i,264) = 1.20E-12

   !  HC3P + NO3  -->  0.048*HCHO + 0.243*ALD + 0.670*KET +
   !                0.063*GLY + 0.792*HO2 + 0.155*MO2 + 0.053*ETHP + 0.051*XO2 + NO

   k(i,265) = 1.20E-12

   !  HC5P + NO3  -->  0.021*HCHO + 0.239*ALD + 0.828*KET +
   !                0.699*HO2 + 0.040*MO2 + 0.262*ETHP + 0.391*XO2 + NO2

   k(i,266) = 1.20E-12

   !  HC8P + NO3  -->  0.187*ALD + 0.880*KET + 0.155*ETHP + NO2 +
   !                0.845*HO2 + 0.587*XO2

   k(i,267) = 1.20E-12

   !  ETEP + NO3  -->  1.6*HCHO + HO2 + NO2 + 0.2*ALD

   k(i,268) = 1.20E-12

   !  OLTP + NO3  -->  0.94*ALD + HCHO + HO2 + NO2 + 0.06*KET

   k(i,269) = 1.20E-12

   !  OLIP + NO3  -->  HO2 + 1.71*ALD + 0.29*KET + NO2

   k(i,270) = 1.20E-12

   !  ISOP + NO3  -->  0.600*MACR + 0.400*OLT + HO2 + NO2 + 0.686*HCHO

   k(i,271) = 1.20E-12

   !  APIP + NO3  -->  HO2 + ALD + NO2 + KET

   k(i,272) = 1.20E-12

   !  LIMP + NO3  -->  HO2 + 0.60*MACR + 0.40*OLI + 0.40*HCHO + NO2

   k(i,273) = 1.20E-12

   !  TOLP + NO3  -->  NO2 + HO2 + 0.50*DCB + 1.30*GLY + 0.70*MGLY

   k(i,274) = 1.20E-12

   !  XYLP + NO3  -->  NO2 + HO2 + DCB + 0.74*GLY + 1.26*MGLY

   k(i,275) = 1.20E-12

   !  CSLP + NO3  -->  GLY + MGLY + NO2 + HO2

   k(i,276) = 1.20E-12

   !  ACO3 + NO3  -->  MO2 + NO2

   k(i,277) = 4.00E-12

   !  TCO3 + NO3  -->  NO2 + ACO3 + HCHO

   k(i,278) = 4.00E-12

   !  KETP + NO3  -->  0.54*MGLY + 0.46*ALD + 0.23*ACO3 + 0.77*HO2 + 0.16*XO2 + NO2

   k(i,279) = 1.20E-12

   !  OLNN + NO3  -->  ONIT + NO2 + HO2

   k(i,280) = 1.20E-12

   !  OLND + NO3  -->  0.280*HCHO + 1.24*ALD + 0.469*KET + 2.0*NO2

   k(i,281) = 1.20E-12

   !  XO2 + HO2  -->  OP2

   k(i,282) = 1.66E-13*exp(-(-1300.0/t(i)))

   !  XO2 + MO2  -->  HCHO + HO2

   k(i,283) = 5.99E-15*exp(-(-1510.0/t(i)))

   !  XO2 + ACO3  -->  MO2

   k(i,284) = 3.40E-14*exp(-(-1560.0/t(i)))

   !  XO2 + XO2  -->

   k(i,285) = 7.13E-17*exp(-(-2950.0/t(i)))

   !  XO2 + NO  -->  NO2

   k(i,286) = 4.00E-12

   !  XO2 + NO3  -->  NO2

   k(i,287) = 1.20E-12

   !  DMS + NO3  -->  SO2 + NO2

   k(i,288) = 5.4E-13

   !  DMS + O3P  -->  SO2

   k(i,289) = 1.3E-11*exp(400.0/t(i))

   !  DMS + OH  -->  0.8*SO2 + 0.2*MSA

   k(i,290) = (t(i)*exp(-234.0/t(i))+8.4E-10*exp(7230./t(i)) &
       +2.68E-10*exp(7810./t(i)))/(1.04E11*t(i)+88.1*exp(7460./t(i)))

   !  H + O2  -->  HO2

   xk0   = 5.70E-32
   xE0   = 0.0
   xn0   = 1.6
   xkinf = 7.50E-11
   xEinf = 0.0
   xminf = 0.0
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,291) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  O3 + H  -->  OH + O2

   k(i,292) = 1.4e-10*exp(-470./t(i))

   !  O1D + H2  -->  OH + H

   k(i,293) = 1.1e-10

   !  O3P + OH  -->  O2 + H

   k(i,294) = 2.2e-11*exp(120./t(i))

   !  O3P + HO2  -->  OH + O2

   k(i,295) = 3.0e-11*exp(200./t(i))

   !  H + HO2  -->  OH + OH

   k(i,296) = 0.9*8.1e-11

   !  H + HO2  -->  H2 + O2

   k(i,297) = 0.08*8.1e-11

   !  H + HO2  -->  H2O + O3P

   k(i,298) = 0.02*8.1e-11

   !  H2 + O3P  -->  OH + H

   k(i,299) = 8.8e-12*exp(-4200./t(i))

   !  OH + OH  -->  H2O + O3P

   k(i,300) = 4.2e-12*exp(-240./t(i))

enddo
!$omp end do

!$omp do
do i=1,imax

   !   OH + OH   -->   H2O2

   xk0   = 6.20E-31
   xE0   = 0.
   xn0   = 1.0
   xkinf = 2.60E-11
   xEinf = 0.
   xminf = 0.0
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,301) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  N + NO  -->  N2 + O3P

   k(i,302) = 2.1e-11*exp(100./t(i))

   !  N + O2  -->  NO + O3P

   k(i,303) = 1.5e-11*exp(-3600./t(i))

   !  O1D + N2O  -->  N2 + O2

   k(i,304) = 4.9e-11

   !  O1D + N2O  -->  NO + NO

   k(i,305) = 6.7e-11

   !  O1D + CH4  -->  OH + CH3

   k(i,306) = 0.75*1.5e-10

   !  O1D + CH4  -->  CH3O + H

   k(i,307) = 0.20*1.5e-10

   !  O1D + CH4  -->  HCHO + H2

   k(i,308) = 0.05*1.5e-10

   !  CH3 + O3P  -->  HCHO + H

   k(i,309) = 1.1e-10

   !  CH3 + O2  -->  MO2

   xk0   = 4.50E-31
   xE0   = 0.0
   xn0   = 3.0
   xkinf = 1.80E-12
   xEinf = 0.0
   xminf = 1.7
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,310) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  HCHO + O3P  -->  HO2 + CO + OH

   k(i,311) = 3.4e-11*exp(-1600./t(i))

   !  HCHO + Cl  -->  HO2 + CO + HCl

   k(i,312) = 8.1e-11*exp(-30./t(i))

   !  CH3O + O2  -->  HCHO + HO2

   k(i,313) = 3.9e-14*exp(-900./t(i))

   !  MO2 + ClO  -->  CH3O + Cl + O2

   k(i,314) = 3.3e-12*exp(-115./t(i))

   !  HCHO + Br  -->  HO2 + CO + HBr

   k(i,315) = 1.7e-11*exp(-800./t(i))

   !  Cl + O3  -->  ClO + O2

   k(i,316) = 2.9e-11*exp(-260./t(i))

   !  ClO + O3P  -->  Cl + O2

   k(i,317) = 3.0e-11*exp(70./t(i))

   !  ClO + NO  -->  NO2 + Cl

   k(i,318) = 6.4e-12*exp(290./t(i))

   !  Cl + CH4  -->  HCl + CH3

   k(i,319) = 1.1e-11*exp(-1400./t(i))

   !  Cl + H2  -->  HCl + H

   k(i,320) = 3.7e-11*exp(-2300./t(i))

   !  Cl + HO2  -->  HCl + O2

   k(i,321) = 1.8e-11*exp(170./t(i))

   !  Cl + HO2  -->  OH + ClO

   k(i,322) = 4.1e-11*exp(-450./t(i))

   !  ClO + OH  -->  Cl + HO2

   k(i,323) = 0.92*1.1e-11*exp(120./t(i))

   !  ClO + OH  -->  HCl + O2

   k(i,324) = 0.08*1.1e-11*exp(120./t(i))

   !  OH + HCl  -->  H2O + Cl

   k(i,325) = 2.6e-12*exp(-350./t(i))

   !  ClO + NO2  -->  ClONO2

   xk0   = 1.80E-31
   xE0   = 0.0
   xn0   = 3.4
   xkinf = 1.50E-11
   xEinf = 0.0
   xminf = 1.9
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,326) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  O3P + ClONO2  -->  ClOx + NOx + XX

   k(i,327) = 2.9e-12*exp(-800./t(i))

   !  ClO + HO2  -->  HOCl + O2

   k(i,328) = 4.8e-13*exp(700./t(i))

   !  OH + HOCl  -->  H2O + ClO

   k(i,329) = 3.0e-12*exp(-500./t(i))

   !  O3P + HOCl  -->  OH + ClO

   k(i,330) = 1.7e-13

   !  Cl + NO2  -->  ClNO2

   xk0   = 1.80E-31
   xE0   = 0.0
   xn0   = 2.0
   xkinf = 1.00E-10
   xEinf = 0.0
   xminf = 1.0
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,331) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  Cl + HOCl  -->  OH + Cl2

   k(i,332) = 2.5e-12*exp(-130./t(i))

   !  ClO + ClO  -->  Cl2O2

   xk0   = 2.20E-32
   xE0   = 0.0
   xn0   = 3.1
   xkinf = 3.50E-12
   xEinf = 0.0
   xminf = 1.0
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,333) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  Cl2O2  -->  ClO + ClO

   xk0   = 2.20E-32
   xn0   = 3.1
   xkinf = 3.50E-12
   xminf = 1.0
   xaeq  = 1.30E-27
   xbeq  = 8744.0
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
   zkinf = xkinf*((t(i)/300.)**(-xminf))
   zfact = 0.6**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,334) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact

   !  ClO + ClO  -->  Cl2 + O2

   k(i,335) = 1.0e-12*exp(-1590./t(i))

   !  ClO + ClO  -->  Cl2 + O2

   k(i,336) = 3.0e-11*exp(-2450./t(i))

   !  O1D + CFC11  -->  ClOx + ClOx + ClOx + XX

   k(i,337) = 2.3e-10

   !  O1D + CFC12  -->  ClOx + ClOx + XX

   k(i,338) = 1.4e-10

   !  O1D + CFC113  -->  ClOx + ClOx + ClOx + XX

   k(i,339) = 2.0e-10

   !  O1D + CCl4  -->  ClOx + ClOx + ClOx + ClOx + XX

   k(i,340) = 3.3e-10

   !  OH + CH3CCl3  -->  ClOx + ClOx + ClOx + H2O + XX

   k(i,341) = 1.8e-12*exp(-1550./t(i))

   !  OH + CH3Cl  -->  ClOx + H2O + XX

   k(i,342) = 4.0e-12*exp(-1400./t(i))

   !  O1D + HCFC22  -->  ClOx + XX

   k(i,343) = 1.0e-10

   !  OH + HCFC22  -->  ClOx + H2O + XX

   k(i,344) = 1.9e-12*exp(-1550./t(i))

   !  Br + O3  -->  BrO + O2

   k(i,345) = 1.7e-11*exp(-800./t(i))

   !  BrO + O3P  -->  Br + O2

   k(i,346) = 1.9e-11*exp(230./t(i))

   !  BrO + NO  -->  NO2 + Br

   k(i,347) = 8.8e-12*exp(260./t(i))

   !  BrO + ClO  -->  OClO + Br

   k(i,348) = 1.6e-12*exp(430./t(i))

   !  BrO + ClO  -->  Br + Cl + O2

   k(i,349) = 2.9e-12*exp(220./t(i))

   !  BrO + ClO  -->  BrCl + O2

   k(i,350) = 5.8e-13*exp(170./t(i))

enddo
!$omp end do

!$omp do
do i=1,imax

   !  BrO + BrO  -->  Br + Br + O2

   k(i,351) = 1.5e-12*exp(230./t(i))

   !  Br + HO2  -->  HBr + O2

   k(i,352) = 1.5e-11*exp(-600./t(i))

   !  O1D + HBr  -->  OH + Br

   k(i,353) = 1.5e-10

   !  BrO + OH  -->  Br + HO2

   k(i,354) = 1.0e-11

   !  OH + HBr  -->  H2O + Br

   k(i,355) = 1.1e-11

   !  BrO + NO2  -->  BrONO2

   xk0   = 5.20E-31
   xE0   = 0.0
   xn0   = 3.2
   xkinf = 6.90E-12
   xEinf = 0.0
   xminf = 2.9
   xF    = 0.6
   zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
   zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
   zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
   k(i,356) = (zk0m/(1.+zk0m/zkinf))*zfact

   !  BrO + HO2  -->  HOBr + O2

   k(i,357) = 3.4e-12*exp(540./t(i))

   !  HOBr + O3P  -->  OH + BrO

   k(i,358) = 1.2e-10*exp(-430./t(i))

   !  O1D + CH3Br  -->  BrOx + XX

   k(i,359) = 1.8e-10

   !  OH + CH3Br  -->  CH2Br + H2O

   k(i,360) = 4.0e-12*exp(-1470./t(i))

   !  O1D + H1211  -->  BrOx + XX

   k(i,361) = 1.5e-10

   !  O1D + H1301  -->  BrOx + XX

   k(i,362) = 1.0e-10

   !  O3P + O3P  -->  O2

   k(i,363) = 4.23e-28*hnm(i)/(t(i)*t(i))

   !  ClONO2 + H2O  -->  HOCl + HNO3

   k(i,364) = het(i,1)

   !  ClONO2 + HCl  -->  Cl2 + HNO3

   k(i,365) = het(i,2)

   !  N2O5 + H2O  -->  HNO3 + HNO3

   k(i,366) = het(i,3)

   !  N2O5 + HCl  -->  ClNO2 + HNO3

   k(i,367) = het(i,4)

   !  HOCl + HCl  -->  Cl2 + H2O

   k(i,368) = het(i,5)

   !  BrONO2 + H2O  -->  HOBr + HNO3

   k(i,369) = het(i,6)

   !  HOBr + HCl  -->  BrCl + H2O

   k(i,370) = het(i,7)

   !  HOBr + HBr  -->  BrOx + BrOx + H2O

   k(i,371) = het(i,8)

   !  HOCl + HBr  -->  BrCl + H2O

   k(i,372) = het(i,9)

enddo
!$omp end do
!$omp end parallel

RETURN
END SUBROUTINE KRATES
