SUBROUTINE CH_ELEM(QM0,QM,HM,imax,ntra,nsls)
!
!!!!!
!
!   CONSERVATION DES ATOMES DE CHLORE, DE BROME ET D'AZOTE
!   SUR UN PAS DE TEMPS CHIMIQUE
!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!
!   Centre National de Recherches Meteorologiques
!   Groupe de Modelisation de Grande Echelle et du Climat
!   Equipe de Recherches sur l'Atmosphere Moyenne
!
!!!!!
!
!  Original : H. Teyssedre, 08/2000
!
!  Modified : H. Teyssedre, version Fortran-90, 11/2006
!
!!!!!
!
#ifdef MPI
USE CSTES_MOD
#endif
!
IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
INTEGER , INTENT(IN) :: imax, ntra, nsls
REAL (KIND=8) , DIMENSION (imax,ntra) , INTENT(IN)    :: QM0
REAL (KIND=8) , DIMENSION (imax,ntra) , INTENT(INOUT) :: QM
REAL (KIND=8) , DIMENSION (imax,nsls) , INTENT(INOUT) :: HM
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
INTEGER :: I,i1,i2
REAL (KIND=8) :: TOTCL0,TOTCL,RAPCL,TOTBR0,TOTBR,RAPBR
REAL (KIND=8) :: TOTN0,TOTN,RAPN
!
#ifdef MPI
i1 = i_deb(myid) ; i2 = i_fin(myid)
#else
i1 = 1           ; i2 = imax
#endif
!
!CCCC
!
!  TRANSPORTED LONG-LIVED SPECIES AND FAMILIES (ARRAY QM)
!
!CCCC
!
!       1 : N2O           2 : CH4           3 : H2O           4 : HNO3
!       5 : N2O5          6 : CO            7 : OClO          8 : HCl
!       9 : ClONO2       10 : HOCl         11 : Cl2          12 : H2O2
!      13 : ClNO2        14 : HBr          15 : BrONO2       16 : HNO4
!      17 : Cl2O2        18 : HOBr         19 : BrCl         20 : HCHO
!      21 : MO2          22 : OP1          23 : CFC11        24 : CFC12
!      25 : CFC113       26 : CCl4         27 : CH3CCl3      28 : CH3Cl
!      29 : HCFC22       30 : CH3Br        31 : H1211        32 : H1301
!      33 : H2SO4        34 : PSC          35 : HONO         36 : SO2
!      37 : ETH          38 : ALKANEbis    39 : ALKENEbis    40 : ISOTOT
!      41 : AROMATIC     42 : ALD          43 : KET          44 : MACR
!      45 : ONIT         46 : PAN          47 : OP2          48 : LINO3
!      49 : HC8P         50 : OLIP         51 : ISOP         52 : PHO
!      53 : TOLP         54 : ACO3         55 : OLNN         56 : XO2
!      57 : SULF         58 : Ox           59 : NOx          60 : ClOx
!      61 : BrOx         62 : NOy          63 : Cly          64 : Bry
!
!CCCC
!
!  SHORT-LIVED SPECIES AT INSTANTANEOUS EQUILIBRIUM (ARRAY HM)
!
!CCCC
!
!       1 : O3P           2 : O1D           3 : O3            4 : N
!       5 : NO            6 : NO2           7 : NO3           8 : ClO
!       9 : Cl           10 : BrO          11 : Br           12 : H
!      13 : OH           14 : HO2          15 : CH3          16 : CH3O
!      17 : ADDT
!
!$omp parallel do default(none) &
!$omp shared(QM0,QM,HM,i1,i2,ntra,nsls) &
!$omp private(i,RAPBR,RAPCL,RAPN,TOTBR,TOTBR0,TOTCL,TOTCL0,TOTN,TOTN0)
!
do i = i1,i2
!
!================================================================
!  CHECK CONSERVATION FOR Cl ATOMS
!================================================================
!
! initial number of atoms :
!
  totCl0 = + QM0(i,7) + QM0(i,8) + QM0(i,9) + QM0(i,10) +            &
    2.0*QM0(i,11) + QM0(i,13) + 2.0*QM0(i,17) + QM0(i,19) +          &
    3.0*QM0(i,23) + 2.0*QM0(i,24) + 3.0*QM0(i,25) + 4.0*QM0(i,26) +  &
    3.0*QM0(i,27) + QM0(i,28) + QM0(i,29) + QM0(i,31) + QM0(i,60)
!
! final number of atoms :
!
  totCl = + QM(i,7) + QM(i,8) + QM(i,9) + QM(i,10) + 2.0*QM(i,11) +  &
    QM(i,13) + 2.0*QM(i,17) + QM(i,19) + 3.0*QM(i,23) +              &
    2.0*QM(i,24) + 3.0*QM(i,25) + 4.0*QM(i,26) + 3.0*QM(i,27) +      &
    QM(i,28) + QM(i,29) + QM(i,31) + QM(i,60)
!
! adjust to keep atom number constant :
!
  rapCl = totCl0 / totCl
!
  QM(i,7) = QM(i,7) * rapCl
  QM(i,8) = QM(i,8) * rapCl
  QM(i,9) = QM(i,9) * rapCl
  QM(i,10) = QM(i,10) * rapCl
  QM(i,11) = QM(i,11) * rapCl
  QM(i,13) = QM(i,13) * rapCl
  QM(i,17) = QM(i,17) * rapCl
  QM(i,19) = QM(i,19) * rapCl
#ifndef MASSy
  QM(i,23) = QM(i,23) * rapCl
  QM(i,24) = QM(i,24) * rapCl
  QM(i,25) = QM(i,25) * rapCl
  QM(i,26) = QM(i,26) * rapCl
  QM(i,27) = QM(i,27) * rapCl
  QM(i,28) = QM(i,28) * rapCl
  QM(i,29) = QM(i,29) * rapCl
  QM(i,31) = QM(i,31) * rapCl
#endif
  QM(i,60) = QM(i,60) * rapCl
  HM(i,8) = HM(i,8) * rapCl
  HM(i,9) = HM(i,9) * rapCl
!
!================================================================
!  CHECK CONSERVATION FOR Br ATOMS
!================================================================
!
! initial number of atoms :
!
  totBr0 = + QM0(i,14) + QM0(i,15) + QM0(i,18) + QM0(i,19) +       &
    QM0(i,30) + QM0(i,31) + QM0(i,32) + QM0(i,61)
!
! final number of atoms :
!
  totBr = + QM(i,14) + QM(i,15) + QM(i,18) + QM(i,19) + QM(i,30) + &
    QM(i,31) + QM(i,32) + QM(i,61)
!
! adjust to keep atom number constant :
!
  rapBr = totBr0 / totBr
!
  QM(i,14) = QM(i,14) * rapBr
  QM(i,15) = QM(i,15) * rapBr
  QM(i,18) = QM(i,18) * rapBr
  QM(i,19) = QM(i,19) * rapBr
#ifndef MASSy
  QM(i,30) = QM(i,30) * rapBr
  QM(i,31) = QM(i,31) * rapBr
  QM(i,32) = QM(i,32) * rapBr
#endif
  QM(i,61) = QM(i,61) * rapBr
  HM(i,10) = HM(i,10) * rapBr
  HM(i,11) = HM(i,11) * rapBr
!
!================================================================
!  CHECK CONSERVATION FOR N ATOMS
!================================================================
!
! initial number of atoms :
!
  totN0 = + 2.0*QM0(i,1) + QM0(i,4) + 2.0*QM0(i,5) + QM0(i,9) +   &
    QM0(i,13) + QM0(i,15) + QM0(i,16) + QM0(i,35) + QM0(i,45) +   &
    QM0(i,46) + QM0(i,55) + QM0(i,59)
!
! final number of atoms :
!
  totN = + 2.0*QM(i,1) + QM(i,4) + 2.0*QM(i,5) + QM(i,9) +        &
    QM(i,13) + QM(i,15) + QM(i,16) + QM(i,35) + QM(i,45) +        &
    QM(i,46) + QM(i,55) + QM(i,59)
!
! adjust to keep atom number constant :
!
  rapN = totN0 / totN
!
#ifndef MASSy
  QM(i,1) = QM(i,1) * rapN
#endif
  QM(i,4) = QM(i,4) * rapN
  QM(i,5) = QM(i,5) * rapN
  QM(i,9) = QM(i,9) * rapN
  QM(i,13) = QM(i,13) * rapN
  QM(i,15) = QM(i,15) * rapN
  QM(i,16) = QM(i,16) * rapN
  QM(i,35) = QM(i,35) * rapN
  QM(i,45) = QM(i,45) * rapN
  QM(i,46) = QM(i,46) * rapN
  QM(i,55) = QM(i,55) * rapN
  QM(i,59) = QM(i,59) * rapN
  HM(i,4) = HM(i,4) * rapN
  HM(i,5) = HM(i,5) * rapN
  HM(i,6) = HM(i,6) * rapN
  HM(i,7) = HM(i,7) * rapN
!
enddo
!$omp end parallel do
!
RETURN
END SUBROUTINE CH_ELEM
