SUBROUTINE CH_EQUIL(sza,k,hnm,QM,HM,partHNO3,partH2O,partHCl,partHBr,    &
                    imax,nk,ntra,nsls,iniv,NIVHOx,SZAlim,LSOL)
!
!!!!!
!
!   CALCUL DE L'EQUILIBRE PHOTOCHIMIQUE POUR LES ESPECES A
!   COURTE DUREE DE VIE
!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!
!   Centre National de Recherches Meteorologiques
!   Groupe de Modelisation de Grande Echelle et du Climat
!   Equipe de Recherches sur l'Atmosphere Moyenne
!
!!!!!
!
!  Original : H. Teyssedre, V.-H. Peuch, 08/2000
!
!  Modified : H. Teyssedre, version Fortran-90, 11/2006
!
!!!!!
!
#ifdef MPI
      USE CSTES_MOD
#endif
!
      IMPLICIT NONE
!	
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
      INTEGER , INTENT(IN) :: imax, nk, ntra, nsls, iniv, NIVHOx
      REAL (KIND=8) , INTENT(IN) :: SZAlim
      REAL (KIND=8) , DIMENSION (imax) , INTENT(IN) :: sza,hnm
      REAL (KIND=8) , DIMENSION (imax) , INTENT(IN) :: partHNO3,partH2O
      REAL (KIND=8) , DIMENSION (imax) , INTENT(IN) :: partHCl,partHBr
      REAL (KIND=8) , DIMENSION (imax,nk) , INTENT(IN) :: k
      REAL (KIND=8) , DIMENSION (imax,ntra) , INTENT(INOUT) :: QM
      REAL (KIND=8) , DIMENSION (imax,nsls) , INTENT(INOUT) :: HM
      LOGICAL , INTENT(IN) :: LSOL
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
      INTEGER :: I, i1, i2
!
      REAL (KIND=8) :: roxo3p,roxo1d,rox,aoxo3,aoxo3p,aoxo1d
      REAL (KIND=8) :: rnoxn,rnoxno,rnoxno3,rnox,anoxno2
      REAL (KIND=8) :: anoxn,anoxno,anoxno3,rcloxcl,rcloxclono2
      REAL (KIND=8) :: rcloxhocl,rcloxcl2o2,rcloxbrcl,rclox
      REAL (KIND=8) :: acloxclo,acloxcl,rbroxbr,rbroxbrono2
      REAL (KIND=8) :: rbroxhobr,rbroxbrcl,rbrox,abroxbro,abroxbr
      REAL (KIND=8) :: xproh,xpeoh,xprho2,xpeho2,argum
      REAL (KIND=8) :: rhoxh,rhoxho2,ahoxh,ahoxho2,ahoxoh
      REAL (KIND=8) :: xphox,xl1hox,xl2hox
      REAL (KIND=8) , DIMENSION (imax) :: gHNO3,gH2O
      REAL (KIND=8) , DIMENSION (imax) :: gHCl,gHBr
!
!!!!!
!
!  TRANSPORTED LONG-LIVED SPECIES AND FAMILIES (ARRAY QM)
!
!!!!!
!
!       1 : N2O           2 : CH4           3 : H2O           4 : HNO3
!       5 : N2O5          6 : CO            7 : OClO          8 : HCl
!       9 : ClONO2       10 : HOCl         11 : Cl2          12 : H2O2
!      13 : ClNO2        14 : HBr          15 : BrONO2       16 : HNO4
!      17 : Cl2O2        18 : HOBr         19 : BrCl         20 : HCHO
!      21 : MO2          22 : OP1          23 : CFC11        24 : CFC12
!      25 : CFC113       26 : CCl4         27 : CH3CCl3      28 : CH3Cl
!      29 : HCFC22       30 : CH3Br        31 : H1211        32 : H1301
!      33 : H2SO4        34 : PSC          35 : HONO         36 : SO2
!      37 : ETH          38 : ALKANEbis    39 : ALKENEbis    40 : ISOTOT
!      41 : AROMATIC     42 : ALD          43 : KET          44 : MACR
!      45 : ONIT         46 : PAN          47 : OP2          48 : LINO3
!      49 : HC8P         50 : OLIP         51 : ISOP         52 : PHO
!      53 : TOLP         54 : ACO3         55 : OLNN         56 : XO2
!      57 : SULF         58 : Ox           59 : NOx          60 : ClOx
!      61 : BrOx         62 : NOy          63 : Cly          64 : Bry
!
!!!!!
!
!  SHORT-LIVED SPECIES AT INSTANTANEOUS EQUILIBRIUM (ARRAY HM)
!
!!!!!
!
!       1 : O3P           2 : O1D           3 : O3            4 : N
!       5 : NO            6 : NO2           7 : NO3           8 : ClO
!       9 : Cl           10 : BrO          11 : Br           12 : H
!      13 : OH           14 : HO2          15 : CH3          16 : CH3O
!      17 : ADDT
!
!
!!!!!
!
!  1. DAY-TIME CHEMISTRY (CHEMICAL STEADY STATE)
!
!!!!!
!
if (LSOL) then
  do i=1,imax
    gHNO3(i)=partHNO3(i)*QM(i,4)
    gH2O (i)=partH2O (i)*QM(i,3)
    gHCl (i)=partHCl (i)*QM(i,8)
    gHBr (i)=partHBr (i)*QM(i,14)
  enddo
else
  do i=1,imax
    gHNO3(i)=QM(i,4)
    gH2O (i)=QM(i,3)
    gHCl (i)=QM(i,8)
    gHBr (i)=QM(i,14)
  enddo
endif
!
#ifdef MPI
i1 = i_deb(myid) ; i2 = i_fin(myid)
#else
i1 = 1           ; i2 = imax
#endif
!
!$omp parallel default(none) &
!$omp private (i,rOxO3P,rOxO1D,rOx,aOxO3,aOxO3P,aOxO1D) &
!$omp private (rNOxN,rNOxNO,rNOxNO3,rNOx) &
!$omp private (aNOxNO2,aNOxN,aNOxNO,aNOxNO3) &
!$omp private (rClOxCl,rClOxClONO2,rClOxHOCl,rClOxCl2O2) &
!$omp private (rClOxBrCl,rClOx,aClOxClO,aClOxCl) &
!$omp private (rBrOxBr,rBrOxBrONO2,rBrOxHOBr,rBrOxBrCl,rBrOx) &
!$omp private (aBrOxBrO,aBrOxBr) &
!$omp private (rHOxH,rHOxHO2,argum) &
!$omp private (aHOxH,aHOxHO2,aHOxOH) &
!$omp private (xpHOx,xl1HOx,xl2HOx) &
!$omp private (xprOH,xpeOH,xprHO2,xpeHO2) &
!$omp shared  (i1,i2,sza,SZAlim,hnm,k,QM,HM,imax,iniv,LSOL) &
!$omp shared  (gHNO3,gH2O,gHCl,gHBr,NIVHOx)
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
    rOxO3P    = HM(i,1) / HM(i,3)
    rOxO1D    = HM(i,2) / HM(i,3)
!
!================================================================
!  FOR FAMILY Ox = O3P + O1D + O3
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k002 :  O3  -->  O1D + O2
!  k003 :  O3  -->  O3P + O2
!  k048 :  O3P + O2  -->  O3
!  k050 :  O1D + N2  -->  O3P + N2
!  k051 :  O1D + O2  -->  O3P + O2
!  k108 : ISOTOT + O3 --> 0.9*HCHO + 0.39253*MACR + 0.36*CO +
!                0.36404*ALKENEbis + 0.17*ACO3 + 0.03*MO2 + 0.15*ORA1 + 0.28*OH
!                0.3*HO2 + 0.001*H2O2 + 0.05*H2 + 0.13*XO2 + 0.09*O3P
!================================================================
!
!  O3P / O3
!
    rOxO3P = ( + rOxO1D*( + k(i,50)*0.78*hnm(i) +                        &
      k(i,51)*0.21*hnm(i) ) + k(i,3) + 0.09*k(i,108)*QM(i,40) ) / ( +    &
      k(i,48)*0.21*hnm(i) )
!
    rOxO3P = max( 0. , rOxO3P )
!
!  O1D / O3
!
    rOxO1D = ( + k(i,2) ) / ( + k(i,50)*0.78*hnm(i) + k(i,51)*0.21*hnm(i) )
!
    rOxO1D = max( 0. , rOxO1D )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rOx = 1. + rOxO3P + rOxO1D
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aOxO3 = 0.0
!
    HM(i,3) = aOxO3*HM(i,3) + (1.-aOxO3)*QM(i,58)/rOx
!
    aOxO3P = 0.0
!
    HM(i,1) = aOxO3P*HM(i,1) + (1.-aOxO3P)*HM(i,3)*rOxO3P
!
    aOxO1D = 0.0
!
    HM(i,2) = aOxO1D*HM(i,2) + (1.-aOxO1D)*HM(i,3)*rOxO1D
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!      
  if (sza(i) <= SZAlim) then
!
    rNOxN     = HM(i,4) / HM(i,6)
    rNOxNO    = HM(i,5) / HM(i,6)
    rNOxNO3   = HM(i,7) / HM(i,6)
!
!================================================================
!  FOR FAMILY NOx = N + NO + NO2 + NO3
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k001 :  NO2  -->  O3P + NO
!  k008 :  NO3  -->  NO + O2
!  k009 :  NO3  -->  NO2 + O3P
!  k047 :  NO  -->  N + O3P
!  k059 :  O3P + NO  -->  NO2
!  k060 :  O3P + NO2  -->  NO + O2
!  k061 :  O3P + NO2  -->  NO3
!  k064 :  OH + NO3  -->  NO2 + HO2
!  k065 :  HO2 + NO  -->  NO2 + OH
!  k069 :  HO2 + NO3  --> NO2 + OH
!  k073 :  O3 + NO  -->  NO2 + O2
!  k074 :  O3 + NO2  -->  NO3 + O2
!  k075 :  NO + NO + O2  -->  NO2 + NO2
!  k076 :  NO3 + NO  -->  NO2 + NO2
!  k077 :  NO3 + NO2  -->  NO + NO2 + O2
!  k078 :  NO3 + NO2  -->  N2O5  (destruction for NO3)
!  k080 :  NO3 + NO3  -->  NO2 + NO2 + O2
!  k102 : MACR + NO3 --> 0.90869*HNO3 + 0.42992*ACO3 +
!                0.10099*MACR + 0.05049*ALD + 0.00606*KET + 0.10099*NO2 +
!                0.10099*XO2 + 0.57975*HO2 + 1.28646*CO
!  k104 : ALKENEbis + NO3  -->  0.96421*OLNN (destruction for NO3)
!  k105 : ISOTOT + NO3  -->  OLNN (destruction for NO3)
!  k106 :  PAN + NO3 --> 0.6*ONIT + 0.6*NO3 + 0.4*PAN + 0.4*HCHO + 0.4*NO2 + XO2
!  k119 :  MO2 + NO  -->  CH3O + NO2
!  k120 : HC8P + NO --> 0.33628*ALD + 0.03065*HCHO +
!                0.54089*KET + 0.03609*MACR + 0.74716*HO2 + 0.09255*MO2 +
!                0.07839*HC8P + 0.12222*XO2 + 0.08116*ONIT + 0.91884*NO2
!  k121 : OLIP + NO --> 1.40361*HCHO + 0.41431*ALD + 0.04833*KET + HO2 + NO2
!  k122 : ISOP + NO --> 0.4489*MACR + 0.606*HCHO +
!                0.3682*ALKENEbis + 0.847*HO2 + 0.847*NO2 + 0.153*ONIT
!  k123 : TOLP + NO --> 0.95122*NO2 + 0.95122*HO2 + 2.08075*MACR + 0.04878*ONIT
!  k124 : ACO3 + NO --> 0.77298*MO2 + 0.10527*ACO3 +
!                0.06891*HCHO + 0.07273*ALD + 0.0855*MACR + 0.12175*HO2 +
!                0.02530*XO2 + NO2
!  k125 : OLNN + NO --> 0.16657*ONIT + 1.83342*NO2 +
!                0.16657*HO2 + 0.23919*HCHO + 1.03345*ALD + 0.38671*KET
!  k148 :  MO2 + NO3  -->  HCHO + HO2 + NO2
!  k149 : HC8P + NO3 --> 0.32478*ALD + 0.80993*HO2 +
!                0.03209*HCHO + 0.6412*KET + 0.03717*MACR + 0.09954*MO2 +
!                0.09067*HC8P + 0.16478*XO2 + NO2
!  k150 : OLIP + NO3 --> 1.45792*HCHO + 0.37131*ALD + 0.01531*KET + HO2 + NO2
!  k151 : ISOP + NO3 --> 0.60390*MACR + 0.41604*ALKENEbis +
!                0.686*HCHO + HO2 + NO2
!  k152 : TOLP + NO3 --> 2.83695*MACR + HO2 + NO2
!  k153 : ACO3 + NO3 --> 0.89016*MO2 + 0.04229*ACO3 +
!                0.04229*HCHO + 0.04744*MACR + 0.04036*ALD + 0.06756*HO2 +
!                0.01404*XO2 + NO2
!  k154 : OLNN + NO3 --> 0.28775*ONIT + 1.71225*NO2 +
!                0.28775*HO2 + 0.19943*HCHO + 0.88319*ALD + 0.33405*KET
!  k159 :  XO2 + NO  -->  NO2
!  k160 :  XO2 + NO3  -->  NO2
!  k172 :  N + NO  -->  N2 + O3P  (destruction for N)
!  k173 :  N + O2  -->  NO + O3P
!  k188 :  ClO + NO  -->  NO2 + Cl
!  k217 :  BrO + NO  -->  NO2 + Br
!================================================================
!
!  N / NO2
!
    rNOxN = ( + k(i,47)*rNOxNO ) / ( + k(i,172)*HM(i,5) + k(i,173)*0.21*hnm(i) )
!
    rNOxN = max( 0. , rNOxN )
!
!  NO / NO2
!
    rNOxNO = ( + rNOxNO3*( + k(i,8) + k(i,77)*HM(i,6) ) + k(i,1) +         &
      k(i,60)*HM(i,1) + k(i,173)*rNOxN*0.21*hnm(i) ) / ( + k(i,47) +       &
      k(i,59)*HM(i,1) + k(i,65)*HM(i,14) + k(i,73)*HM(i,3) +               &
      2.0*k(i,75)*0.21*hnm(i)*HM(i,5) + k(i,76)*HM(i,7) +                  &
      k(i,119)*QM(i,21) + 0.91884*k(i,120)*QM(i,49) +                      &
      k(i,121)*QM(i,50) + 0.847*k(i,122)*QM(i,51) +                        &
      0.95122*k(i,123)*QM(i,53) + k(i,124)*QM(i,54) +                      &
      k(i,125)*QM(i,55) + k(i,159)*QM(i,56) + k(i,188)*HM(i,8) +           &
      k(i,217)*HM(i,10) )
!
    rNOxNO = max( 0. , rNOxNO )
!
!  NO3 / NO2
!
    rNOxNO3 = ( + k(i,61)*HM(i,1) + k(i,74)*HM(i,3) ) / ( +                &
      HM(i,6)*( + k(i,77) + k(i,78) ) + k(i,8) + k(i,9) +                  &
      k(i,64)*HM(i,13) + k(i,69)*HM(i,14) + k(i,76)*HM(i,5) +              &
      2.0*k(i,80)*HM(i,7) + k(i,104)*QM(i,39) + k(i,105)*QM(i,40) +        &
      0.10099*k(i,102)*QM(i,44) + 0.4*k(i,106)*QM(i,46) +                  &
      k(i,148)*QM(i,21) + k(i,149)*QM(i,49) + k(i,150)*QM(i,50) +          &
      k(i,151)*QM(i,51) + k(i,152)*QM(i,53) + k(i,153)*QM(i,54) +          &
      k(i,154)*QM(i,55) + k(i,160)*QM(i,56) )
!
    rNOxNO3 = max( 0. , rNOxNO3 )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rNOx = 1. + rNOxN + rNOxNO + rNOxNO3
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aNOxNO2 = 0.0
!
    HM(i,6) = aNOxNO2*HM(i,6) + (1.-aNOxNO2)*QM(i,59)/rNOx
!
    aNOxN = 0.0
!
    HM(i,4) = aNOxN*HM(i,4) + (1.-aNOxN)*HM(i,6)*rNOxN
!
    aNOxNO = 0.0
!
    HM(i,5) = aNOxNO*HM(i,5) + (1.-aNOxNO)*HM(i,6)*rNOxNO
!
    aNOxNO3 = 0.0
!
    HM(i,7) = aNOxNO3*HM(i,7) + (1.-aNOxNO3)*HM(i,6)*rNOxNO3
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
    rClOxCl   = HM(i,9) / HM(i,8)
!
!================================================================
!  FOR FAMILY ClOx = ClO + Cl
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k025 :  HOCl  -->  Cl + OH
!  k026 :  ClONO2  -->  Cl + NO3
!  k027 :  ClONO2  -->  ClO + NO2
!  k028 :  Cl2O2  -->  Cl + Cl + O2
!  k036 :  BrCl  -->  Br + Cl
!  k184 :  MO2 + ClO  -->  CH3O + Cl + O2
!  k186 :  Cl + O3  -->  ClO + O2
!  k187 :  ClO + O3P  -->  Cl + O2
!  k188 :  ClO + NO  -->  NO2 + Cl
!  k189 :  Cl + CH4  -->  HCl + CH3  (destruction for Cl)
!  k191 :  Cl + HO2  -->  HCl + O2  (destruction for Cl)
!  k192 :  Cl + HO2  -->  OH + ClO  (destruction for Cl)
!  k193 :  ClO + OH  -->  Cl + HO2
!  k196 :  ClO + NO2  -->  ClONO2
!  k198 :  ClO + HO2  -->  HOCl + O2
!  k199 :  OH + HOCl  -->  H2O + ClO
!  k200 :  O3P + HOCl  -->  OH + ClO
!  k202 :  Cl + HOCl  -->  OH + Cl2  (destruction for Cl)
!  k203 :  ClO + ClO  -->  Cl2O2
!  k204 :  Cl2O2  -->  ClO + ClO
!  k219 :  BrO + ClO  -->  Br + Cl + O2
!  k220 :  BrO + ClO  -->  BrCl + O2
!  k234 :  ClONO2 + H2O  -->  HOCl + HNO3  (production for HOCl)
!  k235 :  ClONO2 + HCl  -->  Cl2 + HNO3  (destruction for ClONO2)
!  k238 :  HOCl + HCl  -->  Cl2 + H2O  (destruction for HOCl)
!================================================================
!
!  ClONO2 / ClO
!
    rClOxClONO2 = ( + k(i,196)*HM(i,6) ) / ( + k(i,26) + k(i,27) +     &
      k(i,234)*QM(i,3) + k(i,235)*QM(i,8) )
!
!  HOCl / ClO
!
    rClOxHOCl = ( + k(i,198)*HM(i,14) +                                &
      k(i,234)*rClOxClONO2*QM(i,3) ) / ( + k(i,25) +                   &
      k(i,199)*HM(i,13) + k(i,200)*HM(i,1) + k(i,238)*QM(i,8) )
!
!  Cl2O2 / ClO
!
    rClOxCl2O2 = ( + k(i,203)*HM(i,8) ) / ( + k(i,28) + k(i,204) )
!
!  BrCl / ClO
!
    rClOxBrCl = ( + k(i,220)*HM(i,10) ) / ( + k(i,36) )
!
!  Cl / ClO
!
    rClOxCl = ( + k(i,25)*rClOxHOCl + k(i,26)*rClOxClONO2 +            &
      2.0*k(i,28)*rClOxCl2O2 + k(i,36)*rClOxBrCl + k(i,184)*QM(i,21) + &
      k(i,187)*HM(i,1) + k(i,188)*HM(i,5) + k(i,193)*HM(i,13) +        &
      k(i,219)*HM(i,10) ) / ( + HM(i,14)*( + k(i,191) +                &
      k(i,192) ) + k(i,186)*HM(i,3) + k(i,189)*QM(i,2) +               &
      k(i,202)*QM(i,10) )
!
    rClOxCl = max( 0. , rClOxCl )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rClOx = 1. + rClOxCl
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aClOxClO = 0.5
!
    HM(i,8) = aClOxClO*HM(i,8) + (1.-aClOxClO)*QM(i,60)/rClOx
!
    aClOxCl = 0.0
!
    HM(i,9) = aClOxCl*HM(i,9) + (1.-aClOxCl)*HM(i,8)*rClOxCl
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
    rBrOxBr   = HM(i,11) / HM(i,10)
!
!================================================================
!  FOR FAMILY BrOx = BrO + Br
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k032 :  BrO  -->  Br + O3P
!  k033 :  BrONO2  -->  Br + NO3
!  k034 :  HOBr  -->  Br + OH
!  k036 :  BrCl  -->  Br + Cl
!  k185 :  HCHO + Br  -->  HO2 + CO + HBr  (destruction for Br)
!  k215 :  Br + O3  -->  BrO + O2
!  k216 :  BrO + O3P  -->  Br + O2
!  k217 :  BrO + NO  -->  NO2 + Br
!  k218 :  BrO + ClO  -->  OClO + Br
!  k219 :  BrO + ClO  -->  Br + Cl + O2
!  k220 :  BrO + ClO  -->  BrCl + O2
!  k221 :  BrO + BrO  -->  Br + Br + O2
!  k222 :  Br + HO2  -->  HBr + O2  (destruction for Br)
!  k224 :  BrO + OH  -->  Br + HO2
!  k226 :  BrO + NO2  -->  BrONO2
!  k227 :  BrO + HO2  -->  HOBr + O2
!  k228 :  HOBr + O3P  -->  OH + BrO
!================================================================
!
!  BrONO2 / BrO
!
    rBrOxBrONO2 = ( + k(i,226)*HM(i,6) ) / ( + k(i,33) )
!
!  HOBr / BrO
!
    rBrOxHOBr = ( + k(i,227)*HM(i,14) ) / ( + k(i,34) + k(i,228)*HM(i,1) )
!
!  BrCl / BrO
!
    rBrOxBrCl = ( + k(i,220)*HM(i,8) ) / ( + k(i,36) )
!
!  Br / BrO
!
    rBrOxBr = ( + HM(i,8)*( + k(i,218) + k(i,219) ) + k(i,32) +            &
      k(i,33)*rBrOxBrONO2 + k(i,34)*rBrOxHOBr + k(i,36)*rBrOxBrCl +        &
      k(i,216)*HM(i,1) + k(i,217)*HM(i,5) + 2.0*k(i,221)*HM(i,10) +        &
      k(i,224)*HM(i,13) ) / ( + k(i,185)*QM(i,20) + k(i,215)*HM(i,3) +     &
      k(i,222)*HM(i,14) )
!
      rBrOxBr = max( 0. , rBrOxBr )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rBrOx = 1. + rBrOxBr
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aBrOxBrO = 0.5
!
    HM(i,10) = aBrOxBrO*HM(i,10) + (1.-aBrOxBrO)*QM(i,61)/rBrOx
!
    aBrOxBr = 0.0
!
    HM(i,11) = aBrOxBr*HM(i,11) + (1.-aBrOxBr)*HM(i,10)*rBrOxBr
!
  endif
!
enddo
!$omp end do
!
if (iniv <= NIVHOx) then
!
!$omp do
!
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
      if (HM(i,13) < 1.0e-10) then
        rHOxH   = 0.0
        rHOxHO2 = 0.0
      else
        rHOxH   = HM(i,12) / HM(i,13)
        rHOxHO2 = HM(i,14) / HM(i,13)
      endif
!
!================================================================
!  FOR FAMILY HOx = H + OH + HO2
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k053 :  O3 + OH  -->  HO2 + O2
!  k054 :  O3 + HO2  -->  OH + O2 + O2
!  k056 :  H2O2 + OH  -->  HO2 + H2O
!  k064 :  OH + NO3  -->  NO2 + HO2
!  k065 :  HO2 + NO  -->  NO2 + OH
!  k069 :  HO2 + NO3  --> NO2 + OH
!  k081 :  OH + H2  -->  H2O + H
!  k082 :  OH + SO2  -->  SULF + HO2
!  k083 :  CO + OH  -->  H + CO2
!  k090 :  AROMATIC + OH --> 0.93480*ADDT + 0.10260*XO2 +
!                0.10260*HO2 + 0.00285*PHO
!  k091 :  HCHO + OH  -->  HO2 + CO + H2O
!  k094 : MACR + OH --> 0.53660*ACO3 + 0.14812*MACR +
!                1.03417*CO + 0.47963*HO2 + 0.05649*ALD + 0.0077*KET +
!                0.09179*XO2 + 0.76844*H2O
!  k098 :  PAN + OH  -->  0.53828*HCHO + 0.27962*MACR +
!                0.69128*NO3 + 0.30781*PAN + 0.30781*HO2 + 0.23046*H2O + XO2
!  k161 :  H + O2  -->  HO2
!  k162 :  O3 + H  -->  OH + O2
!  k164 :  O3P + OH  -->  O2 + H
!  k165 :  O3P + HO2  -->  OH + O2
!  k166 :  H + HO2  -->  OH + OH
!  k192 :  Cl + HO2  -->  OH + ClO
!  k193 :  ClO + OH  -->  Cl + HO2
!  k224 :  BrO + OH  -->  Br + HO2
!================================================================
!
!  H / OH
!
      rHOxH = ( + k(i,81)*5.e-7*hnm(i) + k(i,83)*QM(i,6) +                   &
        k(i,164)*HM(i,1) ) / ( + k(i,161)*0.21*hnm(i) +                      &
        k(i,162)*HM(i,3) + k(i,166)*HM(i,14) )
!
      aHOxH = 0.0
!
      HM(i,12) = aHOxH*HM(i,12) + (1.-aHOxH)*HM(i,13)*rHOxH
!
!  HO2 / OH
!
      rHOxHO2 = ( + k(i,53)*HM(i,3) + k(i,56)*QM(i,12) +                     &
        k(i,64)*HM(i,7) + k(i,82)*QM(i,36) + 0.10260*k(i,90)*QM(i,41) +      &
        k(i,91)*QM(i,20) + 0.47963*k(i,94)*QM(i,44) +                        &
        0.30781*k(i,98)*QM(i,46) + k(i,161)*rHOxH*0.21*hnm(i) +              &
        k(i,193)*HM(i,8) + k(i,224)*HM(i,10) ) / ( + k(i,54)*HM(i,3) +       &
        k(i,65)*HM(i,5) + k(i,69)*HM(i,7) + k(i,165)*HM(i,1) +               &
        k(i,166)*HM(i,12) + k(i,192)*HM(i,9) )
!
      aHOxHO2 = 0.5
!
      HM(i,14) = aHOxHO2*HM(i,14) + (1.-aHOxHO2)*HM(i,13)*rHOxHO2
!
!  QUADRATIC LOSS FOR FAMILY HOx = H + OH + HO2
!
!  loss(HOx)**2 = + 2.0*k167*[H]*[HO2] + 2.0*k168*[H]*[HO2] +
!                2.0*k055*[OH]*[HO2] + 2.0*k170*[OH]*[OH] + 2.0*k171*[OH]*[OH] +
!                2.0*k057*[HO2]*[HO2] + 2.0*k058*[HO2]*[HO2]*[H2O]
!
      xl2HOx = + 2.0*( + k(i,167)*rHOxH*rHOxHO2 +                            &
        k(i,168)*rHOxH*rHOxHO2 + k(i,55)*rHOxHO2 + k(i,170) + k(i,171) +     &
        k(i,57)*rHOxHO2*rHOxHO2 + k(i,58)*rHOxHO2*rHOxHO2*gH2O(i) )
!
!  LINEAR LOSS FOR FAMILY HOx = H + OH + HO2
!
!  loss(HOx) = + k062*[OH]*[NO] + k063*[OH]*[NO2] +
!                k070*[OH]*[HONO] + k071*[OH]*[HNO3] + k072*[OH]*[HNO4] +
!                k085*[CH4]*[OH] + k086*[ETH]*[OH] + k088*[ALKENEbis]*[OH] +
!                k089*[ISOTOT]*[OH] + k092*[ALD]*[OH] + k093*[KET]*[OH] +
!                k095*[OP1]*[OH] + k099*[ONIT]*[OH] + k194*[ClO]*[OH] +
!                k195*[OH]*[HCl] + k199*[OH]*[HOCl] + k211*[OH]*[CH3CCl3] +
!                k212*[OH]*[CH3Cl] + k214*[OH]*[HCFC22] + k225*[OH]*[HBr] +
!                k230*[OH]*[CH3Br] + k066*[HO2]*[NO2] + k068*[HO2]*[NO3] +
!                0.897400*k090*[AROMATIC]*[OH] + 0.520370*k094*[MACR]*[OH] +
!                0.692190*k098*[PAN]*[OH] + k112*[PHO]*[HO2] + k126*[MO2]*[HO2]
!                k127*[HC8P]*[HO2] + k128*[OLIP]*[HO2] + k129*[ISOP]*[HO2] +
!                k130*[TOLP]*[HO2] + k131*[ACO3]*[HO2] + k132*[OLNN]*[HO2] +
!                k155*[XO2]*[HO2] + k191*[Cl]*[HO2] + k198*[ClO]*[HO2] +
!                k222*[Br]*[HO2] + k227*[BrO]*[HO2]
!
      xl1HOx = + rHOxHO2*( + k(i,66)*HM(i,6) + k(i,68)*HM(i,7) +            &
        k(i,112)*QM(i,52) + k(i,126)*QM(i,21) + k(i,127)*QM(i,49) +         &
        k(i,128)*QM(i,50) + k(i,129)*QM(i,51) + k(i,130)*QM(i,53) +         &
        k(i,131)*QM(i,54) + k(i,132)*QM(i,55) + k(i,155)*QM(i,56) +         &
        k(i,191)*HM(i,9) + k(i,198)*HM(i,8) + k(i,222)*HM(i,11) +           &
        k(i,227)*HM(i,10) ) + HM(i,13)*( + 0.897400*k(i,90)*QM(i,41) +      &
        0.520370*k(i,94)*QM(i,44) + 0.692190*k(i,98)*QM(i,46) ) +           &
        k(i,62)*HM(i,5) + k(i,63)*HM(i,6) + k(i,70)*QM(i,35) +              &
        k(i,71)*gHNO3(i) + k(i,72)*QM(i,16) + k(i,85)*QM(i,2) +             &
        k(i,86)*QM(i,37) + k(i,88)*QM(i,39) + k(i,89)*QM(i,40) +            &
        k(i,92)*QM(i,42) + k(i,93)*QM(i,43) + k(i,95)*QM(i,22) +            &
        k(i,99)*QM(i,45) + k(i,194)*HM(i,8) + k(i,195)*gHCl(i) +            &
        k(i,199)*QM(i,10) + k(i,211)*QM(i,27) + k(i,212)*QM(i,28) +         &
        k(i,214)*QM(i,29) + k(i,225)*gHBr(i) + k(i,230)*QM(i,30)
!
!  PRODUCTION FOR FAMILY HOx = H + OH + HO2
!
!  prod(HOx) = + 2.0*k012*[HCHO] + 2.0*k023*[H2O] + k029*[HCl] +
!                2.0*k163*[O1D]*[H2] + 2.0*k169*[H2]*[O3P] + k177*[O1D]*[CH4] +
!                k179*[CH3]*[O3P] + k190*[Cl]*[H2] + k004*[HONO] + k005*[HNO3] +
!                k007*[HNO4] + 2.0*k010*[H2O2] + k014*[OP1] + 2.0*k015*[OP2] +
!                k016*[OP2] + k025*[HOCl] + k034*[HOBr] + 2.0*k052*[O1D]*[H2O] +
!                0.02*k084*[ISOTOT]*[O3P] + 0.37222*k107*[ALKENEbis]*[O3] +
!                0.28*k108*[ISOTOT]*[O3] + 0.20864*k109*[MACR]*[O3] +
!                0.036*k110*[PAN]*[O3] + k116*[ADDT]*[O3] + k176*[O1D]*[CH4] +
!                k181*[HCHO]*[O3P] + k200*[O3P]*[HOCl] + k202*[Cl]*[HOCl] +
!                k223*[O1D]*[HBr] + k228*[HOBr]*[O3P] + k006*[HNO4] +
!                k013*[ALD] + 0.77528*k018*[MACR] + k019*[ONIT] + k020*[ONIT] +
!                k067*[HNO4] + 0.28*k084*[ISOTOT]*[O3P] +
!                0.31237*k087*[ALKANEbis]*[OH] + 0.03679*k097*[OP2]*[OH] +
!                k100*[HCHO]*[NO3] + 0.57975*k102*[MACR]*[NO3] +
!                0.24074*k107*[ALKENEbis]*[O3] + 0.3*k108*[ISOTOT]*[O3] +
!                0.28812*k109*[MACR]*[O3] + 0.08*k110*[PAN]*[O3] +
!                k115*[ADDT]*[O2] + 0.74716*k120*[HC8P]*[NO] + k121*[OLIP]*[NO]
!                0.847*k122*[ISOP]*[NO] + 0.95122*k123*[TOLP]*[NO] +
!                0.12175*k124*[ACO3]*[NO] + 0.16657*k125*[OLNN]*[NO] +
!                0.98436*k134*[HC8P]*[MO2] + k135*[OLIP]*[MO2] +
!                k136*[ISOP]*[MO2] + 1.02861*k137*[TOLP]*[MO2] +
!                0.87734*k138*[ACO3]*[MO2] + 0.68801*k139*[OLNN]*[MO2] +
!                0.48134*k140*[HC8P]*[ACO3] + 0.5008*k141*[OLIP]*[ACO3] +
!                0.506*k142*[ISOP]*[ACO3] + k143*[TOLP]*[ACO3] +
!                0.08506*k144*[ACO3]*[ACO3] + 0.18866*k145*[OLNN]*[ACO3] +
!                k146*[OLNN]*[OLNN] + k148*[MO2]*[NO3] +
!                0.80993*k149*[HC8P]*[NO3] + k150*[OLIP]*[NO3] +
!                k151*[ISOP]*[NO3] + k152*[TOLP]*[NO3] +
!                0.06756*k153*[ACO3]*[NO3] + 0.28775*k154*[OLNN]*[NO3] +
!                k156*[XO2]*[MO2] + k181*[HCHO]*[O3P] + k182*[HCHO]*[Cl] +
!                k183*[CH3O]*[O2] + k185*[HCHO]*[Br]
!
      xpHOx = + HM(i,3)*( + 0.37222*k(i,107)*QM(i,39) +                     &
        0.28*k(i,108)*QM(i,40) + 0.20864*k(i,109)*QM(i,44) +                &
        0.036*k(i,110)*QM(i,46) + k(i,116)*HM(i,17) +                       &
        0.24074*k(i,107)*QM(i,39) + 0.3*k(i,108)*QM(i,40) +                 &
        0.28812*k(i,109)*QM(i,44) + 0.08*k(i,110)*QM(i,46) ) +              &
        HM(i,7)*( + k(i,100)*QM(i,20) + 0.57975*k(i,102)*QM(i,44) +         &
        k(i,148)*QM(i,21) + 0.80993*k(i,149)*QM(i,49) +                     &
        k(i,150)*QM(i,50) + k(i,151)*QM(i,51) + k(i,152)*QM(i,53) +         &
        0.06756*k(i,153)*QM(i,54) + 0.28775*k(i,154)*QM(i,55) ) +           &
        QM(i,54)*( + 0.12175*k(i,124)*HM(i,5) +                             &
        0.87734*k(i,138)*QM(i,21) + 0.48134*k(i,140)*QM(i,49) +             &
        0.5008*k(i,141)*QM(i,50) + 0.506*k(i,142)*QM(i,51) +                &
        k(i,143)*QM(i,53) + 0.08506*k(i,144)*QM(i,54) +                     &
        0.18866*k(i,145)*QM(i,55) ) + 2.0*( + k(i,12)*QM(i,20) +            &
        k(i,23)*gH2O(i) + k(i,163)*HM(i,2)*5.e-7*hnm(i) +                   &
        k(i,169)*5.e-7*hnm(i)*HM(i,1) + k(i,10)*QM(i,12) +                  &
        k(i,15)*QM(i,47) + k(i,52)*HM(i,2)*gH2O(i) ) +                      &
        HM(i,1)*( + k(i,179)*HM(i,15) + 0.02*k(i,84)*QM(i,40) +             &
        k(i,181)*QM(i,20) + k(i,200)*QM(i,10) + k(i,228)*QM(i,18) +         &
        0.28*k(i,84)*QM(i,40) ) + QM(i,21)*( +                              &
        0.98436*k(i,134)*QM(i,49) + k(i,135)*QM(i,50) +                     &
        k(i,136)*QM(i,51) + 1.02861*k(i,137)*QM(i,53) +                     &
        0.68801*k(i,139)*QM(i,55) + k(i,156)*QM(i,56) ) + HM(i,5)*( +       &
        0.74716*k(i,120)*QM(i,49) + k(i,121)*QM(i,50) +                     &
        0.847*k(i,122)*QM(i,51) + 0.95122*k(i,123)*QM(i,53) +               &
        0.16657*k(i,125)*QM(i,55) ) + HM(i,2)*( + k(i,177)*QM(i,2) +        &
        k(i,176)*QM(i,2) + k(i,223)*gHBr(i) ) + QM(i,16)*( + k(i,7) +       &
        k(i,6) + k(i,67) ) + HM(i,9)*( + k(i,190)*5.e-7*hnm(i) +            &
        k(i,202)*QM(i,10) ) + QM(i,47)*( + k(i,16) +                        &
        0.03679*k(i,97)*HM(i,13) ) + QM(i,45)*( + k(i,19) + k(i,20) ) +     &
        k(i,29)*gHCl(i) + k(i,4)*QM(i,35) + k(i,5)*gHNO3(i) +               &
        k(i,14)*QM(i,22) + k(i,25)*QM(i,10) + k(i,34)*QM(i,18) +            &
        k(i,13)*QM(i,42) + 0.77528*k(i,18)*QM(i,44) +                       &
        0.31237*k(i,87)*QM(i,38)*HM(i,13) +                                 &
        k(i,115)*HM(i,17)*0.21*hnm(i) + k(i,146)*QM(i,55)*QM(i,55) +        &
        QM(i,20)*( + k(i,181)*HM(i,1) + k(i,182)*HM(i,9) +                  &
        k(i,185)*HM(i,11) ) + k(i,183)*HM(i,16)*0.21*hnm(i)
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY PIVOT OH
!
      argum = max( 0.0 , xl1HOx**2. + 4.0*xl2HOx*xpHOx )
!
      aHOxOH = 0.5
!
      HM(i,13) = aHOxOH*HM(i,13) + (1.-aHOxOH)*(sqrt(argum) - xl1HOx)       &
        / (2.0*xl2HOx)
!
    endif
!
  enddo
!$omp end do
!
else
!
!================================================================
!  EXPLICIT CALCULATION FOR HOx
!================================================================
!
!$omp do
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
!  d[H]/dt = 0.0
!
      HM(i,12) = max( 1.0e-10 , ( + 5.e-7*hnm(i)*( + k(i,81)*HM(i,13) +     &
        k(i,163)*HM(i,2) + k(i,169)*HM(i,1) + k(i,190)*HM(i,9) ) +          &
        HM(i,13)*( + k(i,83)*QM(i,6) + k(i,164)*HM(i,1) ) +                 &
        k(i,12)*QM(i,20) + k(i,23)*gH2O(i) + k(i,29)*gHCl(i) +              &
        k(i,177)*HM(i,2)*QM(i,2) + k(i,179)*HM(i,15)*HM(i,1) ) / ( +        &
        HM(i,14)*( + k(i,166) + k(i,167) + k(i,168) ) +                     &
        k(i,161)*0.21*hnm(i) + k(i,162)*HM(i,3) ) )
!
    endif
!
  enddo
!$omp end do
!
!$omp do
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
!  d[OH]/dt = 0.0
!
      HM(i,13) = max( 1.0e-10 , ( + HM(i,3)*( + k(i,54)*HM(i,14) +          &
        0.37222*k(i,107)*QM(i,39) + 0.28*k(i,108)*QM(i,40) +                &
        0.20864*k(i,109)*QM(i,44) + 0.036*k(i,110)*QM(i,46) +               &
        k(i,116)*HM(i,17) + k(i,162)*HM(i,12) ) + HM(i,1)*( +               &
        0.02*k(i,84)*QM(i,40) + k(i,165)*HM(i,14) +                         &
        k(i,169)*5.e-7*hnm(i) + k(i,181)*QM(i,20) + k(i,200)*QM(i,10) +     &
        k(i,228)*QM(i,18) ) + HM(i,2)*( + 2.0*k(i,52)*gH2O(i) +             &
        k(i,163)*5.e-7*hnm(i) + k(i,176)*QM(i,2) + k(i,223)*gHBr(i) ) +     &
        HM(i,14)*( + k(i,65)*HM(i,5) + k(i,69)*HM(i,7) +                    &
        2.0*k(i,166)*HM(i,12) + k(i,192)*HM(i,9) ) + QM(i,47)*( +           &
        k(i,15) + k(i,16) + 0.43859*k(i,97)*HM(i,13) ) + QM(i,22)*( +       &
        k(i,14) + k(i,96)*HM(i,13) ) + QM(i,10)*( + k(i,25) +               &
        k(i,202)*HM(i,9) ) + k(i,4)*QM(i,35) + k(i,5)*gHNO3(i) +            &
        k(i,7)*QM(i,16) + 2.0*k(i,10)*QM(i,12) + k(i,23)*gH2O(i) +          &
        k(i,34)*QM(i,18) + 0.02327*k(i,87)*QM(i,38)*HM(i,13) ) / ( +        &
        QM(i,22)*( + k(i,95) + k(i,96) ) + 2.0*( + k(i,170)*HM(i,13) +      &
        k(i,171)*HM(i,13) ) + HM(i,8)*( + k(i,193) + k(i,194) ) +           &
        k(i,53)*HM(i,3) + k(i,55)*HM(i,14) + k(i,56)*QM(i,12) +             &
        k(i,62)*HM(i,5) + k(i,63)*HM(i,6) + k(i,64)*HM(i,7) +               &
        k(i,70)*QM(i,35) + k(i,71)*gHNO3(i) + k(i,72)*QM(i,16) +            &
        k(i,81)*5.e-7*hnm(i) + k(i,82)*QM(i,36) + k(i,83)*QM(i,6) +         &
        k(i,85)*QM(i,2) + k(i,86)*QM(i,37) + k(i,87)*QM(i,38) +             &
        k(i,88)*QM(i,39) + k(i,89)*QM(i,40) + k(i,90)*QM(i,41) +            &
        k(i,91)*QM(i,20) + k(i,92)*QM(i,42) + k(i,93)*QM(i,43) +            &
        k(i,94)*QM(i,44) + k(i,97)*QM(i,47) + k(i,98)*QM(i,46) +            &
        k(i,99)*QM(i,45) + k(i,164)*HM(i,1) + k(i,195)*gHCl(i) +            &
        k(i,199)*QM(i,10) + k(i,211)*QM(i,27) + k(i,212)*QM(i,28) +         &
        k(i,214)*QM(i,29) + k(i,224)*HM(i,10) + k(i,225)*gHBr(i) +          &
        k(i,230)*QM(i,30) ) )
!
    endif
!
  enddo
!$omp end do
!
!$omp do
  do i = i1,i2
!
   if (sza(i) <= SZAlim) then
!
!  d[HO2]/dt = 0.0
!
      xprHO2 =  + HM(i,13)*( + k(i,53)*HM(i,3) + k(i,56)*QM(i,12) +         &
        k(i,64)*HM(i,7) + k(i,82)*QM(i,36) + 0.31237*k(i,87)*QM(i,38) +     &
        0.10260*k(i,90)*QM(i,41) + k(i,91)*QM(i,20) +                       &
        0.47963*k(i,94)*QM(i,44) + 0.03679*k(i,97)*QM(i,47) +               &
        0.30781*k(i,98)*QM(i,46) + k(i,193)*HM(i,8) +                       &
        k(i,224)*HM(i,10) ) + HM(i,7)*( + k(i,100)*QM(i,20) +               &
        0.57975*k(i,102)*QM(i,44) + k(i,148)*QM(i,21) +                     &
        0.80993*k(i,149)*QM(i,49) + k(i,150)*QM(i,50) +                     &
        k(i,151)*QM(i,51) + k(i,152)*QM(i,53) +                             &
        0.06756*k(i,153)*QM(i,54) + 0.28775*k(i,154)*QM(i,55) ) +           &
        QM(i,54)*( + 0.12175*k(i,124)*HM(i,5) +                             &
        0.87734*k(i,138)*QM(i,21) + 0.48134*k(i,140)*QM(i,49) +             &
        0.5008*k(i,141)*QM(i,50) + 0.506*k(i,142)*QM(i,51) +                &
        k(i,143)*QM(i,53) + 0.08506*k(i,144)*QM(i,54) +                     &
        0.18866*k(i,145)*QM(i,55) ) + QM(i,21)*( +                          &
        0.98436*k(i,134)*QM(i,49) + k(i,135)*QM(i,50) +                     &
        k(i,136)*QM(i,51) + 1.02861*k(i,137)*QM(i,53) +                     &
        0.68801*k(i,139)*QM(i,55) + k(i,156)*QM(i,56) ) + HM(i,5)*( +       &
        0.74716*k(i,120)*QM(i,49) + k(i,121)*QM(i,50) +                     &
        0.847*k(i,122)*QM(i,51) + 0.95122*k(i,123)*QM(i,53) +               &
        0.16657*k(i,125)*QM(i,55) ) + QM(i,20)*( + k(i,12) +                &
        k(i,181)*HM(i,1) + k(i,182)*HM(i,9) + k(i,185)*HM(i,11) ) +         &
        HM(i,3)*( + 0.24074*k(i,107)*QM(i,39) + 0.3*k(i,108)*QM(i,40) +     &
        0.28812*k(i,109)*QM(i,44) + 0.08*k(i,110)*QM(i,46) ) +              &
        0.21*hnm(i)*( + k(i,115)*HM(i,17) + k(i,161)*HM(i,12) +             &
        k(i,183)*HM(i,16) ) + QM(i,16)*( + k(i,6) + k(i,67) ) +             &
        QM(i,45)*( + k(i,19) + k(i,20) ) + k(i,13)*QM(i,42) +               &
        k(i,15)*QM(i,47) + 0.77528*k(i,18)*QM(i,44) +                       &
        0.28*k(i,84)*QM(i,40)*HM(i,1) + k(i,146)*QM(i,55)*QM(i,55)
!
      xpeHO2 =  + HM(i,12)*( + k(i,166) + k(i,167) + k(i,168) ) +           &
        2.0*( + k(i,57)*HM(i,14) + k(i,58)*HM(i,14)*gH2O(i) ) +             &
        HM(i,7)*( + k(i,68) + k(i,69) ) + HM(i,9)*( + k(i,191) +            &
        k(i,192) ) + k(i,54)*HM(i,3) + k(i,55)*HM(i,13) +                   &
        k(i,65)*HM(i,5) + k(i,66)*HM(i,6) + k(i,112)*QM(i,52) +             &
        k(i,126)*QM(i,21) + k(i,127)*QM(i,49) + k(i,128)*QM(i,50) +         &
        k(i,129)*QM(i,51) + k(i,130)*QM(i,53) + k(i,131)*QM(i,54) +         &
        k(i,132)*QM(i,55) + k(i,155)*QM(i,56) + k(i,165)*HM(i,1) +          &
        k(i,198)*HM(i,8) + k(i,222)*HM(i,11) + k(i,227)*HM(i,10)
!
      HM(i,14) = max( 1.0e-10 , xprHO2 / xpeHO2 )
!
    endif
!
  enddo
!
!$omp end do
endif
!
!================================================================
!  EXPLICIT CALCULATION FOR SHORT-LIVED SPECIES THAT ARE NOT
!  TRANSPORTED
!================================================================
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
!  d[CH3]/dt = 0.0
!
    HM(i,15) = max( 1.0e-10 , ( + QM(i,2)*( + k(i,85)*HM(i,13) +          &
      k(i,176)*HM(i,2) + k(i,189)*HM(i,9) ) ) / ( + k(i,179)*HM(i,1) +    &
      k(i,180)*0.21*hnm(i) ) )
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
!  d[CH3O]/dt = 0.0
!
    HM(i,16) = max( 1.0e-10 , ( + QM(i,21)*( + k(i,119)*HM(i,5) +        &
      0.66*k(i,133)*QM(i,21) + k(i,184)*HM(i,8) ) + k(i,14)*QM(i,22) +   &
      k(i,177)*HM(i,2)*QM(i,2) ) / ( + k(i,183)*0.21*hnm(i) ) )
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
!  d[ADDT]/dt = 0.0
!
    HM(i,17) = max( 1.0e-10 , ( +                                          &
      0.93480*k(i,90)*QM(i,41)*HM(i,13) ) / ( + 0.21*hnm(i)*( +            &
      k(i,114) + k(i,115) ) + k(i,113)*HM(i,6) + k(i,116)*HM(i,3) ) )
!
  endif
!
enddo
!$omp end do
!
!!!!!
!
!  2. NIGHT-TIME CHEMISTRY
!
!!!!!
!
!$omp do
do i = i1,i2
!
  if (sza(i) > SZAlim) then
!
!================================================================
!  AT NIGHT, FAMILY Ox = O3P + O1D + O3
!  IS REDUCED TO O3 SPECIE
!================================================================
!
!  O3P
!
    HM(i,1) = 0.0
!
!  O1D
!
    HM(i,2) = 0.0
!
!  O3
!
    HM(i,3) = max( 0.0 , QM(i,58) )
!
!================================================================
!  FOR FAMILY NOx = N + NO + NO2 + NO3
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT AT NIGHT :
!
!  k064 :  OH + NO3  -->  NO2 + HO2
!  k065 :  HO2 + NO  -->  NO2 + OH
!  k069 :  HO2 + NO3  --> NO2 + OH
!  k073 :  O3 + NO  -->  NO2 + O2
!  k074 :  O3 + NO2  -->  NO3 + O2
!  k075 :  NO + NO + O2  -->  NO2 + NO2
!  k076 :  NO3 + NO  -->  NO2 + NO2
!  k077 :  NO3 + NO2  -->  NO + NO2 + O2
!  k078 :  NO3 + NO2  -->  N2O5  (destruction only)
!  k080 :  NO3 + NO3  -->  NO2 + NO2 + O2
!  k102 : MACR + NO3 --> 0.90869*HNO3 + 0.42992*ACO3 +
!                0.10099*MACR + 0.05049*ALD + 0.00606*KET + 0.10099*NO2 +
!                0.10099*XO2 + 0.57975*HO2 + 1.28646*CO
!  k106 :  PAN + NO3 --> 0.6*ONIT + 0.6*NO3 + 0.4*PAN + 0.4*HCHO + 0.4*NO2 + XO2
!  k119 :  MO2 + NO  -->  CH3O + NO2
!  k120 : HC8P + NO --> 0.33628*ALD + 0.03065*HCHO +
!                0.54089*KET + 0.03609*MACR + 0.74716*HO2 + 0.09255*MO2 +
!                0.07839*HC8P + 0.12222*XO2 + 0.08116*ONIT + 0.91884*NO2
!  k121 : OLIP + NO --> 1.40361*HCHO + 0.41431*ALD + 0.04833*KET + HO2 + NO2
!  k122 : ISOP + NO --> 0.4489*MACR + 0.606*HCHO +
!                0.3682*ALKENEbis + 0.847*HO2 + 0.847*NO2 + 0.153*ONIT
!  k123 : TOLP + NO --> 0.95122*NO2 + 0.95122*HO2 + 2.08075*MACR + 0.04878*ONIT
!  k124 : ACO3 + NO --> 0.77298*MO2 + 0.10527*ACO3 +
!                0.06891*HCHO + 0.07273*ALD + 0.0855*MACR + 0.12175*HO2 +
!                0.02530*XO2 + NO2
!  k125 : OLNN + NO --> 0.16657*ONIT + 1.83342*NO2 +
!                0.16657*HO2 + 0.23919*HCHO + 1.03345*ALD + 0.38671*KET
!  k148 :  MO2 + NO3  -->  HCHO + HO2 + NO2
!  k149 : HC8P + NO3 --> 0.32478*ALD + 0.80993*HO2 +
!                0.03209*HCHO + 0.6412*KET + 0.03717*MACR + 0.09954*MO2 +
!                0.09067*HC8P + 0.16478*XO2 + NO2
!  k150 : OLIP + NO3 --> 1.45792*HCHO + 0.37131*ALD + 0.01531*KET + HO2 + NO2
!  k151 : ISOP + NO3 --> 0.60390*MACR + 0.41604*ALKENEbis +
!                0.686*HCHO + HO2 + NO2
!  k152 : TOLP + NO3 --> 2.83695*MACR + HO2 + NO2
!  k153 : ACO3 + NO3 --> 0.89016*MO2 + 0.04229*ACO3 +
!                0.04229*HCHO + 0.04744*MACR + 0.04036*ALD + 0.06756*HO2 +
!                0.01404*XO2 + NO2
!  k154 : OLNN + NO3 --> 0.28775*ONIT + 1.71225*NO2 +
!                0.28775*HO2 + 0.19943*HCHO + 0.88319*ALD + 0.33405*KET
!  k159 :  XO2 + NO  -->  NO2
!  k160 :  XO2 + NO3  -->  NO2
!  k188 :  ClO + NO  -->  NO2 + Cl
!  k217 :  BrO + NO  -->  NO2 + Br
!================================================================
!
!  N
!
    HM(i,4) = 0.0
!
!  NO3 / NO2
!
    rNOxNO3 = ( + k(i,74)*HM(i,3) ) / ( + HM(i,6)*( + k(i,77) +             &
      k(i,78) ) + k(i,64)*HM(i,13) + k(i,69)*HM(i,14) +                     &
      k(i,76)*HM(i,5) + 2.0*k(i,80)*HM(i,7) +                               &
      0.10099*k(i,102)*QM(i,44) + 0.4*k(i,106)*QM(i,46) +                   &
      k(i,148)*QM(i,21) + k(i,149)*QM(i,49) + k(i,150)*QM(i,50) +           &
      k(i,151)*QM(i,51) + k(i,152)*QM(i,53) + k(i,153)*QM(i,54) +           &
      k(i,154)*QM(i,55) + k(i,160)*QM(i,56) )
!
    rNOxNO3 = max( 0. , rNOxNO3 )
!
!  NO / NO2
!
    rNOxNO = ( + k(i,77)*rNOxNO3*HM(i,6) ) / ( + k(i,65)*HM(i,14) +         &
      k(i,73)*HM(i,3) + 2.0*k(i,75)*0.21*hnm(i)*HM(i,5) +                   &
      k(i,76)*HM(i,7) + k(i,119)*QM(i,21) +                                 &
      0.91884*k(i,120)*QM(i,49) + k(i,121)*QM(i,50) +                       &
      0.847*k(i,122)*QM(i,51) + 0.95122*k(i,123)*QM(i,53) +                 &
      k(i,124)*QM(i,54) + k(i,125)*QM(i,55) + k(i,159)*QM(i,56) +           &
      k(i,188)*HM(i,8) + k(i,217)*HM(i,10) )
!
    rNOxNO = max( 0. , rNOxNO )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY AT NIGHT :
!
    rNOx = 1. + rNOxNO + rNOxNO3
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS AT NIGHT :
!
    aNOxNO2 = 0.0
!
    HM(i,6) = aNOxNO2*HM(i,6) + (1.-aNOxNO2)*QM(i,59)/rNOx
!
    aNOxNO = 0.0
!
    HM(i,5) = aNOxNO*HM(i,5) + (1.-aNOxNO)*HM(i,6)*rNOxNO
!
    aNOxNO3 = 0.0
!
    HM(i,7) = aNOxNO3*HM(i,7) + (1.-aNOxNO3)*HM(i,6)*rNOxNO3
!
!================================================================
!  AT NIGHT, FAMILY ClOx = ClO + Cl
!  IS REDUCED TO ClO SPECIE
!================================================================
!
!  ClO
!
    HM(i,8) = max( 0.0 , QM(i,60) )
!
!  Cl
!
    HM(i,9) = 0.0
!
!================================================================
!  AT NIGHT, FAMILY BrOx = BrO + Br
!  IS REDUCED TO BrO SPECIE
!================================================================
!
!  BrO
!
    HM(i,10) = max( 0.0 , QM(i,61) )
!
!  Br
!
    HM(i,11) = 0.0
!
!  H
!
    HM(i,12) = 0.0
!
!  CH3
!
    HM(i,15) = 0.0
!
!  CH3O
!
    HM(i,16) = 0.0
!
!  ADDT
!
    HM(i,17) = 0.0
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!      
  if (sza(i) > SZAlim) then
!
!  d[OH]/dt = 0.0
!
    HM(i,13) = max( 1.0e-10 , ( + HM(i,3)*( + k(i,54)*HM(i,14) +          &
      0.37222*k(i,107)*QM(i,39) + 0.28*k(i,108)*QM(i,40) +                &
      0.20864*k(i,109)*QM(i,44) + 0.036*k(i,110)*QM(i,46) ) +             &
      HM(i,13)*( + 0.02327*k(i,87)*QM(i,38) + k(i,96)*QM(i,22) +          &
      0.43859*k(i,97)*QM(i,47) ) + HM(i,14)*( + k(i,65)*HM(i,5) +         &
      k(i,69)*HM(i,7) ) ) / ( + QM(i,22)*( + k(i,95) + k(i,96) ) +        &
      2.0*( + k(i,170)*HM(i,13) + k(i,171)*HM(i,13) ) + HM(i,8)*( +       &
      k(i,193) + k(i,194) ) + k(i,53)*HM(i,3) + k(i,55)*HM(i,14) +        &
      k(i,56)*QM(i,12) + k(i,62)*HM(i,5) + k(i,63)*HM(i,6) +              &
      k(i,64)*HM(i,7) + k(i,70)*QM(i,35) + k(i,71)*gHNO3(i) +             &
      k(i,72)*QM(i,16) + k(i,81)*5.e-7*hnm(i) + k(i,82)*QM(i,36) +        &
      k(i,83)*QM(i,6) + k(i,85)*QM(i,2) + k(i,86)*QM(i,37) +              &
      k(i,87)*QM(i,38) + k(i,88)*QM(i,39) + k(i,89)*QM(i,40) +            &
      k(i,90)*QM(i,41) + k(i,91)*QM(i,20) + k(i,92)*QM(i,42) +            &
      k(i,93)*QM(i,43) + k(i,94)*QM(i,44) + k(i,97)*QM(i,47) +            &
      k(i,98)*QM(i,46) + k(i,99)*QM(i,45) + k(i,195)*gHCl(i) +            &
      k(i,199)*QM(i,10) + k(i,211)*QM(i,27) + k(i,212)*QM(i,28) +         &
      k(i,214)*QM(i,29) + k(i,224)*HM(i,10) + k(i,225)*gHBr(i) +          &
      k(i,230)*QM(i,30) ) )
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
   if (sza(i) > SZAlim) then
!
!  d[HO2]/dt = 0.0
!
    HM(i,14) = max( 1.0e-10 , ( + HM(i,13)*( + k(i,53)*HM(i,3) +          &
      k(i,56)*QM(i,12) + k(i,64)*HM(i,7) + k(i,82)*QM(i,36) +             &
      0.31237*k(i,87)*QM(i,38) + 0.10260*k(i,90)*QM(i,41) +               &
      k(i,91)*QM(i,20) + 0.47963*k(i,94)*QM(i,44) +                       &
      0.03679*k(i,97)*QM(i,47) + 0.30781*k(i,98)*QM(i,46) +               &
      k(i,193)*HM(i,8) + k(i,224)*HM(i,10) ) + HM(i,7)*( +                &
      k(i,100)*QM(i,20) + 0.57975*k(i,102)*QM(i,44) +                     &
      k(i,148)*QM(i,21) + 0.80993*k(i,149)*QM(i,49) +                     &
      k(i,150)*QM(i,50) + k(i,151)*QM(i,51) + k(i,152)*QM(i,53) +         &
      0.06756*k(i,153)*QM(i,54) + 0.28775*k(i,154)*QM(i,55) ) +           &
      QM(i,54)*( + 0.12175*k(i,124)*HM(i,5) +                             &
      0.87734*k(i,138)*QM(i,21) + 0.48134*k(i,140)*QM(i,49) +             &
      0.5008*k(i,141)*QM(i,50) + 0.506*k(i,142)*QM(i,51) +                &
      k(i,143)*QM(i,53) + 0.08506*k(i,144)*QM(i,54) +                     &
      0.18866*k(i,145)*QM(i,55) ) + QM(i,21)*( +                          &
      0.98436*k(i,134)*QM(i,49) + k(i,135)*QM(i,50) +                     &
      k(i,136)*QM(i,51) + 1.02861*k(i,137)*QM(i,53) +                     &
      0.68801*k(i,139)*QM(i,55) + k(i,156)*QM(i,56) ) + HM(i,5)*( +       &
      0.74716*k(i,120)*QM(i,49) + k(i,121)*QM(i,50) +                     &
      0.847*k(i,122)*QM(i,51) + 0.95122*k(i,123)*QM(i,53) +               &
      0.16657*k(i,125)*QM(i,55) ) + HM(i,3)*( +                           &
      0.24074*k(i,107)*QM(i,39) + 0.3*k(i,108)*QM(i,40) +                 &
      0.28812*k(i,109)*QM(i,44) + 0.08*k(i,110)*QM(i,46) ) +              &
      k(i,67)*QM(i,16) + k(i,146)*QM(i,55)*QM(i,55) ) / ( + 2.0*( +       &
      k(i,57)*HM(i,14) + k(i,58)*HM(i,14)*gH2O(i) ) + HM(i,7)*( +         &
      k(i,68) + k(i,69) ) + k(i,54)*HM(i,3) + k(i,55)*HM(i,13) +          &
      k(i,65)*HM(i,5) + k(i,66)*HM(i,6) + k(i,112)*QM(i,52) +             &
      k(i,126)*QM(i,21) + k(i,127)*QM(i,49) + k(i,128)*QM(i,50) +         &
      k(i,129)*QM(i,51) + k(i,130)*QM(i,53) + k(i,131)*QM(i,54) +         &
      k(i,132)*QM(i,55) + k(i,155)*QM(i,56) + k(i,198)*HM(i,8) +          &
      k(i,227)*HM(i,10) ) )
!
  endif
!
enddo
!$omp end do
!
!$omp end parallel
!
!
RETURN
END SUBROUTINE CH_EQUIL
