SUBROUTINE CH_PTERMS (k,jco2,hnm,day,QM,HM,pr,pe,                        &
                      partHNO3,partH2O,partHCl,partHBr,imax,nk,ntra,nsls)
!
!!!!!
!
!   CALCUL DES TERMES DE PRODUCTION ET DE DESTRUCTION DES ESPECES A
!   LONGUE DUREE DE VIE ET DES FAMILLES TRANSPORTEES
!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!
!   Centre National de Recherches Meteorologiques
!   Groupe de Modelisation de Grande Echelle et du Climat
!   Equipe de Recherches sur l'Atmosphere Moyenne
!
!!!!!
!
!  Original : H. Teyssedre, V.-H. Peuch, 08/2000
!
!  Modified : H. Teyssedre, version Fortran-90, 11/2006
!
!!!!!
!
#ifdef MPI
      USE CSTES_MOD
#endif
!
      IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
      INTEGER , INTENT(IN) :: imax, nk, ntra, nsls
      REAL (KIND=8) , DIMENSION(imax,nk)   , INTENT(IN) :: k
      REAL (KIND=8) , DIMENSION(imax)      , INTENT(IN) :: jco2,hnm,day
      REAL (KIND=8) , DIMENSION(imax)      , INTENT(IN) :: partHNO3
      REAL (KIND=8) , DIMENSION(imax)      , INTENT(IN) :: partH2O
      REAL (KIND=8) , DIMENSION(imax)      , INTENT(IN) :: partHCl
      REAL (KIND=8) , DIMENSION(imax)      , INTENT(IN) :: partHBr
      REAL (KIND=8) , DIMENSION(imax,ntra) , INTENT(IN) :: QM
      REAL (KIND=8) , DIMENSION(imax,nsls) , INTENT(IN) :: HM
      REAL (KIND=8) , DIMENSION(imax,ntra) , INTENT(OUT) :: pr,pe
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
      INTEGER :: i,j,i1,i2
!
      REAL (KIND=8), PARAMETER :: rco2=360.e-06
!
!!!!!
!
!  TRANSPORTED LONG-LIVED SPECIES AND FAMILIES (ARRAY QM)
!
!!!!!
!
!       1 : N2O           2 : CH4           3 : H2O           4 : HNO3
!       5 : N2O5          6 : CO            7 : OClO          8 : HCl
!       9 : ClONO2       10 : HOCl         11 : Cl2          12 : H2O2
!      13 : ClNO2        14 : HBr          15 : BrONO2       16 : HNO4
!      17 : Cl2O2        18 : HOBr         19 : BrCl         20 : HCHO
!      21 : MO2          22 : OP1          23 : CFC11        24 : CFC12
!      25 : CFC113       26 : CCl4         27 : CH3CCl3      28 : CH3Cl
!      29 : HCFC22       30 : CH3Br        31 : H1211        32 : H1301
!      33 : H2SO4        34 : PSC          35 : HONO         36 : SO2
!      37 : ETH          38 : ALKANEbis    39 : ALKENEbis    40 : ISOTOT
!      41 : AROMATIC     42 : ALD          43 : KET          44 : MACR
!      45 : ONIT         46 : PAN          47 : OP2          48 : LINO3
!      49 : HC8P         50 : OLIP         51 : ISOP         52 : PHO
!      53 : TOLP         54 : ACO3         55 : OLNN         56 : XO2
!      57 : SULF         58 : Ox           59 : NOx          60 : ClOx
!      61 : BrOx         62 : NOy          63 : Cly          64 : Bry
!
!!!!!
!
!  SHORT-LIVED SPECIES AT INSTANTANEOUS EQUILIBRIUM (ARRAY HM)
!
!!!!!
!
!       1 : O3P           2 : O1D           3 : O3            4 : N
!       5 : NO            6 : NO2           7 : NO3           8 : ClO
!       9 : Cl           10 : BrO          11 : Br           12 : H
!      13 : OH           14 : HO2          15 : CH3          16 : CH3O
!      17 : ADDT
!
!
!  pr(N2O) = 0.0
!
!  pe(N2O) =  + k024 + k174*[O1D] + k175*[O1D]
!
!  pr(CH4) =  + 0.03912*k107*[ALKENEbis]*[O3]
!
!  pe(CH4) =  + k085*[OH] + k176*[O1D] + k177*[O1D] + k178*[O1D] + k189*[Cl]
!
!  pr(H2O) =  + k055*[OH]*[HO2] + k056*[H2O2]*[OH] + k070*[OH]*[HONO] +
!                k071*[OH]*[HNO3]*partHNO3 + k072*[OH]*[HNO4] + k081*[OH]*[H2] +
!                k085*[CH4]*[OH] + k086*[ETH]*[OH] + k087*[ALKANEbis]*[OH] +
!                k091*[HCHO]*[OH] + k092*[ALD]*[OH] + k093*[KET]*[OH] +
!                0.76844*k094*[MACR]*[OH] + k095*[OP1]*[OH] + k096*[OP1]*[OH] +
!                0.23046*k098*[PAN]*[OH] + k099*[ONIT]*[OH] + k168*[H]*[HO2] +
!                k170*[OH]*[OH] + k195*[OH]*[HCl]*partHCl + k199*[OH]*[HOCl] +
!                k211*[OH]*[CH3CCl3] + k212*[OH]*[CH3Cl] + k214*[OH]*[HCFC22] +
!                k225*[OH]*[HBr]*partHBr + k230*[OH]*[CH3Br] +
!                k238*[HOCl]*[HCl] + k240*[HOBr]*[HCl] + k241*[HOBr]*[HBr] +
!                k242*[HOCl]*[HBr]
!
!  pe(H2O) =  + k023 + k052*[O1D] + k234*[ClONO2] + k236*[N2O5] + k239*[BrONO2]
!
!  pr(HNO3) =  + k063*[OH]*[NO2] + k068*[HO2]*[NO3] +
!                k100*[HCHO]*[NO3] + k101*[ALD]*[NO3] +
!                0.90869*k102*[MACR]*[NO3] + k103*[AROMATIC]*[NO3] +
!                k234*[ClONO2]*[H2O] + k235*[ClONO2]*[HCl] +
!                2.0*k236*[N2O5]*[H2O] + k237*[N2O5]*[HCl] + k239*[BrONO2]*[H2O]
!
!  pe(HNO3) =  + k005 + k071*[OH]
!
!  pr(N2O5) =  + k078*[NO3]*[NO2]
!
!  pe(N2O5) =  + k022 + k079 + k236*[H2O] + k237*[HCl]
!
!  pr(CO) =  + k011*[HCHO] + k012*[HCHO] + k013*[ALD] +
!                0.9118*k018*[MACR] + 0.01*k084*[ISOTOT]*[O3P] +
!                0.02327*k087*[ALKANEbis]*[OH] + k091*[HCHO]*[OH] +
!                1.03417*k094*[MACR]*[OH] + k100*[HCHO]*[NO3] +
!                1.28646*k102*[MACR]*[NO3] + 0.36475*k107*[ALKENEbis]*[O3] +
!                0.36*k108*[ISOTOT]*[O3] + 0.65571*k109*[MACR]*[O3] +
!                0.13*k110*[PAN]*[O3] + k181*[HCHO]*[O3P] + k182*[HCHO]*[Cl] +
!                k185*[HCHO]*[Br]
!
!  pe(CO) =  + k083*[OH]
!
!  pr(OClO) =  + k218*[BrO]*[ClO]
!
!  pe(OClO) =  + k035
!
!  pr(HCl) =  + k182*[HCHO]*[Cl] + k189*[Cl]*[CH4] +
!                k190*[Cl]*[H2] + k191*[Cl]*[HO2] + k194*[ClO]*[OH]
!
!  pe(HCl) =  + k029 + k195*[OH] + k235*[ClONO2] + k237*[N2O5] +
!                k238*[HOCl] + k240*[HOBr]
!
!  pr(ClONO2) =  + k196*[ClO]*[NO2]
!
!  pe(ClONO2) =  + k026 + k027 + k197*[O3P] + k234*[H2O] + k235*[HCl]
!
!  pr(HOCl) =  + k198*[ClO]*[HO2] + k234*[ClONO2]*[H2O]
!
!  pe(HOCl) =  + k025 + k199*[OH] + k200*[O3P] + k202*[Cl] +
!                k238*[HCl] + k242*[HBr]
!
!  pr(Cl2) =  + k202*[Cl]*[HOCl] + k205*[ClO]*[ClO] +
!                k206*[ClO]*[ClO] + k235*[ClONO2]*[HCl] + k238*[HOCl]*[HCl]
!
!  pe(Cl2) =  + k030
!
!  pr(H2O2) =  + k057*[HO2]*[HO2] +
!                k058*[HO2]*[HO2]*[H2O]*partH2O + 0.01643*k107*[ALKENEbis]*[O3]
!                0.001*k108*[ISOTOT]*[O3] + k171*[OH]*[OH]
!
!  pe(H2O2) =  + k010 + k056*[OH]
!
!  pr(ClNO2) =  + k201*[Cl]*[NO2] + k237*[N2O5]*[HCl]
!
!  pe(ClNO2) =  + k031
!
!  pr(HBr) =  + k185*[HCHO]*[Br] + k222*[Br]*[HO2]
!
!  pe(HBr) =  + k223*[O1D] + k225*[OH] + k241*[HOBr] + k242*[HOCl]
!
!  pr(BrONO2) =  + k226*[BrO]*[NO2]
!
!  pe(BrONO2) =  + k033 + k239*[H2O]
!
!  pr(HNO4) =  + k066*[HO2]*[NO2]
!
!  pe(HNO4) =  + k006 + k007 + k067 + k072*[OH]
!
!  pr(Cl2O2) =  + k203*[ClO]*[ClO]
!
!  pe(Cl2O2) =  + k028 + k204
!
!  pr(HOBr) =  + k227*[BrO]*[HO2] + k239*[BrONO2]*[H2O]
!
!  pe(HOBr) =  + k034 + k228*[O3P] + k240*[HCl] + k241*[HBr]
!
!  pr(BrCl) =  + k220*[BrO]*[ClO] + k240*[HOBr]*[HCl] + k242*[HOCl]*[HBr]
!
!  pe(BrCl) =  + k036
!
!  pr(HCHO) =  + 0.0605*k018*[MACR] + 0.05*k084*[ISOTOT]*[O3P] +
!                0.00238*k087*[ALKANEbis]*[OH] + k096*[OP1]*[OH] +
!                0.03679*k097*[OP2]*[OH] + 0.53828*k098*[PAN]*[OH] +
!                0.4*k106*[PAN]*[NO3] + 0.54280*k107*[ALKENEbis]*[O3] +
!                0.9*k108*[ISOTOT]*[O3] + 0.7*k110*[PAN]*[O3] +
!                0.03065*k120*[HC8P]*[NO] + 1.40361*k121*[OLIP]*[NO] +
!                0.606*k122*[ISOP]*[NO] + 0.06891*k124*[ACO3]*[NO] +
!                0.23919*k125*[OLNN]*[NO] + 0.66*k133*[MO2]*[MO2] +
!                0.80572*k134*[HC8P]*[MO2] + 1.42733*k135*[OLIP]*[MO2] +
!                1.09*k136*[ISOP]*[MO2] + k137*[TOLP]*[MO2] +
!                0.96426*k138*[ACO3]*[MO2] + 0.88103*k139*[OLNN]*[MO2] +
!                0.07663*k140*[HC8P]*[ACO3] + 0.6812*k141*[OLIP]*[ACO3] +
!                0.34*k142*[ISOP]*[ACO3] + 0.04453*k144*[ACO3]*[ACO3] +
!                0.12889*k145*[OLNN]*[ACO3] + k148*[MO2]*[NO3] +
!                0.03209*k149*[HC8P]*[NO3] + 1.45792*k150*[OLIP]*[NO3] +
!                0.686*k151*[ISOP]*[NO3] + 0.04229*k153*[ACO3]*[NO3] +
!                0.19943*k154*[OLNN]*[NO3] + k156*[XO2]*[MO2] +
!                k178*[O1D]*[CH4] + k179*[CH3]*[O3P] + k183*[CH3O]*[O2]
!
!  pe(HCHO) =  + k011 + k012 + k091*[OH] + k100*[NO3] +
!                k181*[O3P] + k182*[Cl] + k185*[Br]
!
!  pr(MO2) =  + k013*[ALD] + k016*[OP2] + k095*[OP1]*[OH] +
!                0.12708*k107*[ALKENEbis]*[O3] + 0.03*k108*[ISOTOT]*[O3] +
!                0.09255*k120*[HC8P]*[NO] + 0.77298*k124*[ACO3]*[NO] +
!                0.01346*k134*[HC8P]*[MO2] + 0.58551*k138*[ACO3]*[MO2] +
!                0.51431*k140*[HC8P]*[ACO3] + 0.5008*k141*[OLIP]*[ACO3] +
!                0.506*k142*[ISOP]*[ACO3] + k143*[TOLP]*[ACO3] +
!                1.61972*k144*[ACO3]*[ACO3] + 0.50996*k145*[OLNN]*[ACO3] +
!                0.09954*k149*[HC8P]*[NO3] + 0.89016*k153*[ACO3]*[NO3] +
!                k157*[XO2]*[ACO3] + k180*[CH3]*[O2]
!
!  pe(MO2) =  + k119*[NO] + k126*[HO2] + 2.0*k133*[MO2] +
!                k134*[HC8P] + k135*[OLIP] + k136*[ISOP] + k137*[TOLP] +
!                k138*[ACO3] + k139*[OLNN] + k148*[NO3] + k156*[XO2] + k184*[ClO
!
!  pr(OP1) =  + k126*[MO2]*[HO2]
!
!  pe(OP1) =  + k014 + k095*[OH] + k096*[OH]
!
!  pr(CFC11) = 0.0
!
!  pe(CFC11) =  + k037 + k207*[O1D]
!
!  pr(CFC12) = 0.0
!
!  pe(CFC12) =  + k038 + k208*[O1D]
!
!  pr(CFC113) = 0.0
!
!  pe(CFC113) =  + k039 + k209*[O1D]
!
!  pr(CCl4) = 0.0
!
!  pe(CCl4) =  + k040 + k210*[O1D]
!
!  pr(CH3CCl3) = 0.0
!
!  pe(CH3CCl3) =  + k041 + k211*[OH]
!
!  pr(CH3Cl) = 0.0
!
!  pe(CH3Cl) =  + k042 + k212*[OH]
!
!  pr(HCFC22) = 0.0
!
!  pe(HCFC22) =  + k043 + k213*[O1D] + k214*[OH]
!
!  pr(CH3Br) = 0.0
!
!  pe(CH3Br) =  + k044 + k229*[O1D] + k230*[OH]
!
!  pr(H1211) = 0.0
!
!  pe(H1211) =  + k045 + k231*[O1D]
!
!  pr(H1301) = 0.0
!
!  pe(H1301) =  + k046 + k232*[O1D]
!
!  pr(H2SO4) = 0.0
!
!  pe(H2SO4) = 0.0
!
!  pr(PSC) = 0.0
!
!  pe(PSC) = 0.0
!
!  pr(HONO) =  + k062*[OH]*[NO] + k113*[ADDT]*[NO2]
!
!  pe(HONO) =  + k004 + k070*[OH]
!
!  pr(SO2) = 0.0
!
!  pe(SO2) =  + k082*[OH]
!
!  pr(ETH) =  + 0.02918*k107*[ALKENEbis]*[O3]
!
!  pe(ETH) =  + k086*[OH]
!
!  pr(ALKANEbis) = 0.0
!
!  pe(ALKANEbis) =  + k087*[OH]
!
!  pr(ALKENEbis) =  + 0.89449*k084*[ISOTOT]*[O3P] +
!                0.36404*k108*[ISOTOT]*[O3] + 0.3682*k122*[ISOP]*[NO] +
!                0.46806*k136*[ISOP]*[MO2] + 0.23819*k142*[ISOP]*[ACO3] +
!                0.41604*k151*[ISOP]*[NO3]
!
!  pe(ALKENEbis) =  + k088*[OH] + k104*[NO3] + k107*[O3]
!
!  pr(ISOTOT) = 0.0
!
!  pe(ISOTOT) =  + k084*[O3P] + k089*[OH] + k105*[NO3] + k108*[O3]
!
!  pr(AROMATIC) =  + 0.10391*k111*[PHO]*[NO2] +
!                1.03908*k112*[PHO]*[HO2] + k113*[ADDT]*[NO2] +
!                k115*[ADDT]*[O2] + k116*[ADDT]*[O3]
!
!  pe(AROMATIC) =  + k090*[OH] + k103*[NO3]
!
!  pr(ALD) =  + k015*[OP2] + k020*[ONIT] +
!                0.21989*k087*[ALKANEbis]*[OH] + 0.05649*k094*[MACR]*[OH] +
!                0.07161*k097*[OP2]*[OH] + 0.05049*k102*[MACR]*[NO3] +
!                0.47021*k107*[ALKENEbis]*[O3] + 0.15896*k109*[MACR]*[O3] +
!                0.33628*k120*[HC8P]*[NO] + 0.41431*k121*[OLIP]*[NO] +
!                0.07273*k124*[ACO3]*[NO] + 1.03345*k125*[OLNN]*[NO] +
!                0.56293*k134*[HC8P]*[MO2] + 0.46518*k135*[OLIP]*[MO2] +
!                0.08696*k138*[ACO3]*[MO2] + 0.39935*k139*[OLNN]*[MO2] +
!                0.71704*k140*[HC8P]*[ACO3] + 0.68371*k141*[OLIP]*[ACO3] +
!                0.07834*k144*[ACO3]*[ACO3] + 0.40474*k145*[OLNN]*[ACO3] +
!                0.32478*k149*[HC8P]*[NO3] + 0.37131*k150*[OLIP]*[NO3] +
!                0.04036*k153*[ACO3]*[NO3] + 0.88319*k154*[OLNN]*[NO3]
!
!  pe(ALD) =  + k013 + k092*[OH] + k101*[NO3]
!
!  pr(KET) =  + k019*[ONIT] + 0.05953*k087*[ALKANEbis]*[OH] +
!                0.0077*k094*[MACR]*[OH] + 0.36698*k097*[OP2]*[OH] +
!                0.00606*k102*[MACR]*[NO3] + 0.06764*k107*[ALKENEbis]*[O3] +
!                0.54089*k120*[HC8P]*[NO] + 0.04833*k121*[OLIP]*[NO] +
!                0.38671*k125*[OLNN]*[NO] + 0.09095*k134*[HC8P]*[MO2] +
!                0.03882*k135*[OLIP]*[MO2] + 0.09297*k139*[OLNN]*[MO2] +
!                0.18281*k140*[HC8P]*[ACO3] + 0.06648*k141*[OLIP]*[ACO3] +
!                0.02462*k144*[ACO3]*[ACO3] + 0.10399*k145*[OLNN]*[ACO3] +
!                0.6412*k149*[HC8P]*[NO3] + 0.01531*k150*[OLIP]*[NO3] +
!                0.33405*k154*[OLNN]*[NO3]
!
!  pe(KET) =  + k017 + k093*[OH]
!
!  pr(MACR) =  + 0.13085*k084*[ISOTOT]*[O3P] +
!                0.02466*k087*[ALKANEbis]*[OH] + 0.14812*k094*[MACR]*[OH] +
!                0.27962*k098*[PAN]*[OH] + 0.10099*k102*[MACR]*[NO3] +
!                0.90585*k105*[ISOTOT]*[NO3] + 0.39253*k108*[ISOTOT]*[O3] +
!                1.1017*k109*[MACR]*[O3] + 0.03609*k120*[HC8P]*[NO] +
!                0.4489*k122*[ISOP]*[NO] + 2.08075*k123*[TOLP]*[NO] +
!                0.0855*k124*[ACO3]*[NO] + 0.08396*k134*[HC8P]*[MO2] +
!                0.55357*k136*[ISOP]*[MO2] + 1.99456*k137*[TOLP]*[MO2] +
!                0.16878*k138*[ACO3]*[MO2] + 0.0729*k140*[HC8P]*[ACO3] +
!                0.77601*k142*[ISOP]*[ACO3] + 1.99453*k143*[TOLP]*[ACO3] +
!                0.12104*k144*[ACO3]*[ACO3] + 0.03717*k149*[HC8P]*[NO3] +
!                0.60390*k151*[ISOP]*[NO3] + 2.83695*k152*[TOLP]*[NO3] +
!                0.04744*k153*[ACO3]*[NO3]
!
!  pe(MACR) =  + k018 + k094*[OH] + k102*[NO3] + k109*[O3]
!
!  pr(ONIT) =  + 0.6*k106*[PAN]*[NO3] + k111*[PHO]*[NO2] +
!                0.08116*k120*[HC8P]*[NO] + 0.153*k122*[ISOP]*[NO] +
!                0.04878*k123*[TOLP]*[NO] + 0.16657*k125*[OLNN]*[NO] +
!                k132*[OLNN]*[HO2] + 0.68801*k139*[OLNN]*[MO2] +
!                0.6787*k145*[OLNN]*[ACO3] + 2.0*k146*[OLNN]*[OLNN] +
!                0.28775*k154*[OLNN]*[NO3]
!
!  pe(ONIT) =  + k019 + k020 + k099*[OH]
!
!  pr(PAN) =  + 0.30781*k098*[PAN]*[OH] + 0.4*k106*[PAN]*[NO3] +
!                0.3*k110*[PAN]*[O3] + k117*[ACO3]*[NO2]
!
!  pe(PAN) =  + k098*[OH] + k106*[NO3] + k110*[O3] + k118
!
!  pr(OP2) =  + 0.10423*k109*[MACR]*[O3] +
!                1.00522*k127*[HC8P]*[HO2] + 1.00522*k128*[OLIP]*[HO2] +
!                1.00522*k129*[ISOP]*[HO2] + 1.00522*k130*[TOLP]*[HO2] +
!                0.85175*k131*[ACO3]*[HO2] + 1.00522*k155*[XO2]*[HO2]
!
!  pe(OP2) =  + k015 + k016 + k097*[OH]
!
!  pr(HC8P) =  + k017*[KET] + k086*[ETH]*[OH] +
!                0.68285*k087*[ALKANEbis]*[OH] + 0.39383*k097*[OP2]*[OH] +
!                k099*[ONIT]*[OH] + 0.08956*k107*[ALKENEbis]*[O3] +
!                0.07839*k120*[HC8P]*[NO] + 0.00359*k134*[HC8P]*[MO2] +
!                0.00809*k140*[HC8P]*[ACO3] + 0.09067*k149*[HC8P]*[NO3]
!
!  pe(HC8P) =  + k120*[NO] + k127*[HO2] + k134*[MO2] + k140*[ACO3] + k149*[NO3]
!
!  pr(OLIP) =  + 1.01963*k088*[ALKENEbis]*[OH]
!
!  pe(OLIP) =  + k121*[NO] + k128*[HO2] + k135*[MO2] + k141*[ACO3] + k150*[NO3]
!
!  pr(ISOP) =  + k089*[ISOTOT]*[OH]
!
!  pe(ISOP) =  + k122*[NO] + k129*[HO2] + k136*[MO2] + k142*[ACO3] + k151*[NO3]
!
!  pr(PHO) =  + 0.00285*k090*[AROMATIC]*[OH] + k103*[AROMATIC]*[NO3]
!
!  pe(PHO) =  + k111*[NO2] + k112*[HO2]
!
!  pr(TOLP) =  + k114*[ADDT]*[O2]
!
!  pe(TOLP) =  + k123*[NO] + k130*[HO2] + k137*[MO2] + k143*[ACO3] + k152*[NO3]
!
!  pr(ACO3) =  + k017*[KET] + 0.71757*k018*[MACR] +
!                k092*[ALD]*[OH] + k093*[KET]*[OH] + 0.53660*k094*[MACR]*[OH] +
!                0.06833*k097*[OP2]*[OH] + k101*[ALD]*[NO3] +
!                0.42992*k102*[MACR]*[NO3] + 0.05225*k107*[ALKENEbis]*[O3] +
!                0.17*k108*[ISOTOT]*[O3] + 0.27818*k109*[MACR]*[O3] +
!                0.7*k110*[PAN]*[O3] + k118*[PAN] + 0.10527*k124*[ACO3]*[NO] +
!                0.07152*k138*[ACO3]*[MO2] + 0.07139*k144*[ACO3]*[ACO3] +
!                0.04229*k153*[ACO3]*[NO3]
!
!  pe(ACO3) =  + k117*[NO2] + k124*[NO] + k131*[HO2] +
!                k138*[MO2] + k140*[HC8P] + k141*[OLIP] + k142*[ISOP] +
!                k143*[TOLP] + 2.0*k144*[ACO3] + k145*[OLNN] + k153*[NO3] +
!                k157*[XO2]
!
!  pr(OLNN) =  + 0.96421*k104*[ALKENEbis]*[NO3] + k105*[ISOTOT]*[NO3]
!
!  pe(OLNN) =  + k125*[NO] + k132*[HO2] + k139*[MO2] +
!                k145*[ACO3] + 2.0*k146*[OLNN] + 2.0*k147*[OLNN] + k154*[NO3]
!
!  pr(XO2) =  + 0.15*k084*[ISOTOT]*[O3P] +
!                0.10260*k090*[AROMATIC]*[OH] + 0.09179*k094*[MACR]*[OH] +
!                0.09945*k097*[OP2]*[OH] + k098*[PAN]*[OH] +
!                0.10099*k102*[MACR]*[NO3] + k106*[PAN]*[NO3] +
!                0.13*k108*[ISOTOT]*[O3] + 0.12222*k120*[HC8P]*[NO] +
!                0.02530*k124*[ACO3]*[NO] + 0.12985*k134*[HC8P]*[MO2] +
!                0.02319*k138*[ACO3]*[MO2] + 0.10953*k140*[HC8P]*[ACO3] +
!                0.01791*k144*[ACO3]*[ACO3] + 0.16478*k149*[HC8P]*[NO3] +
!                0.01404*k153*[ACO3]*[NO3]
!
!  pe(XO2) =  + k155*[HO2] + k156*[MO2] + k157*[ACO3] +
!                2.0*k158*[XO2] + k159*[NO] + k160*[NO3]
!
!  pr(SULF) =  + k082*[OH]*[SO2]
!
!  pe(SULF) = 0.0
!
!  pr(Ox) = 0.0
!              + k001*[NO2] + k009*[NO3] + 2.0*k021*[O2] +
!                k032*[BrO] + k035*[OClO] + k047*[NO] + k168*[H]*[HO2] +
!                k170*[OH]*[OH] + k172*[N]*[NO] + k173*[N]*[O2] + k024*[N2O] +
!                0.13345*k131*[ACO3]*[HO2]
!
!  pe(Ox) =  + k187*[ClO]*[O3P] +
!                2.0*k203*[ClO]*[ClO]*k028/(k028 + k204) +
!                min(k025*[HOCl],k198*[HO2]*[ClO]) +
!                2.0*min(k036*[BrCl],k220*[BrO]*[ClO]) +
!                2.0*k219*[BrO]*[ClO]*day + min(k034*[HOBr],k227*[HO2]*[BrO]) +
!                2.0*k221*[BrO]*[BrO]*day + 2.0*(k027 + k026)*[ClONO2]
!              + 2.0*k049*[O3P]*[O3] + k059*[O3P]*[NO] +
!                k060*[O3P]*[NO2] + k061*[O3P]*[NO2] + k084*[ISOTOT]*[O3P] +
!                0.910000*k108*[ISOTOT]*[O3] + k164*[O3P]*[OH] +
!                k165*[O3P]*[HO2] + k169*[H2]*[O3P] + k179*[CH3]*[O3P] +
!                k181*[HCHO]*[O3P] + k187*[ClO]*[O3P] + k197*[O3P]*[ClONO2] +
!                k200*[O3P]*[HOCl] + k216*[BrO]*[O3P] + k228*[HOBr]*[O3P] +
!                2.0*k233*[O3P]*[O3P] + k052*[O1D]*[H2O] + k163*[O1D]*[H2] +
!                k174*[O1D]*[N2O] + k175*[O1D]*[N2O] + k176*[O1D]*[CH4] +
!                k177*[O1D]*[CH4] + k178*[O1D]*[CH4] + k207*[O1D]*[CFC11] +
!                k208*[O1D]*[CFC12] + k209*[O1D]*[CFC113] + k210*[O1D]*[CCl4] +
!                k213*[O1D]*[HCFC22] + k223*[O1D]*[HBr] + k229*[O1D]*[CH3Br] +
!                k231*[O1D]*[H1211] + k232*[O1D]*[H1301] + k053*[O3]*[OH] +
!                k054*[O3]*[HO2] + k073*[O3]*[NO] + k074*[O3]*[NO2] +
!                k107*[ALKENEbis]*[O3] + k109*[MACR]*[O3] + k110*[PAN]*[O3] +
!                k116*[ADDT]*[O3] + k162*[O3]*[H] + k186*[Cl]*[O3] +
!                k215*[Br]*[O3]
!
!  pr(NOx) =  + 2.0*k022*[N2O5] + k197*[O3P]*[ClONO2]
!              + k004*[HONO] + 2.0*k175*[O1D]*[N2O] +
!                k005*[HNO3]*partHNO3 + k006*[HNO4] + k019*[ONIT] + k020*[ONIT]
!                k027*[ClONO2] + k031*[ClNO2] + k067*[HNO4] + k070*[OH]*[HONO] +
!                k072*[OH]*[HNO4] + 2.0*k079*[N2O5] + k099*[ONIT]*[OH] +
!                0.4*k106*[PAN]*[NO3] + 0.7*k110*[PAN]*[O3] + k118*[PAN] +
!                0.833420*k125*[OLNN]*[NO] + 0.31199*k139*[OLNN]*[MO2] +
!                0.712250*k154*[OLNN]*[NO3] + k007*[HNO4] + k026*[ClONO2] +
!                k033*[BrONO2] + k071*[OH]*[HNO3]*partHNO3 +
!                0.69128*k098*[PAN]*[OH]
!
!  pe(NOx) = 0.0
!              + 2.0*k172*[N]*[NO] + k062*[OH]*[NO] +
!                k063*[OH]*[NO2] + k066*[HO2]*[NO2] + 2.0*k078*[NO3]*[NO2] +
!                0.899010*k102*[MACR]*[NO3] + k111*[PHO]*[NO2] +
!                k113*[ADDT]*[NO2] + k117*[ACO3]*[NO2] +
!                0.081160*k120*[HC8P]*[NO] + 0.153000*k122*[ISOP]*[NO] +
!                0.048780*k123*[TOLP]*[NO] + k196*[ClO]*[NO2] + k201*[Cl]*[NO2]
!                k226*[BrO]*[NO2] + k068*[HO2]*[NO3] + k100*[HCHO]*[NO3] +
!                k101*[ALD]*[NO3] + k103*[AROMATIC]*[NO3] +
!                k104*[ALKENEbis]*[NO3] + k105*[ISOTOT]*[NO3]
!
!  pr(ClOx) =  + 3.0*k037*[CFC11] + 2.0*k038*[CFC12] +
!                3.0*k039*[CFC113] + 4.0*k040*[CCl4] + 3.0*k041*[CH3CCl3] +
!                k042*[CH3Cl] + k043*[HCFC22] + k197*[O3P]*[ClONO2] +
!                3.0*k207*[O1D]*[CFC11] + 2.0*k208*[O1D]*[CFC12] +
!                3.0*k209*[O1D]*[CFC113] + 4.0*k210*[O1D]*[CCl4] +
!                3.0*k211*[OH]*[CH3CCl3] + k212*[OH]*[CH3Cl] +
!                k213*[O1D]*[HCFC22] + k214*[OH]*[HCFC22]
!              + k035*[OClO] + k029*[HCl]*partHCl +
!                2.0*k030*[Cl2] + k031*[ClNO2] + k195*[OH]*[HCl]*partHCl +
!                k026*[ClONO2] + k027*[ClONO2] + k025*[HOCl] + k199*[OH]*[HOCl]
!                k200*[O3P]*[HOCl] + 2.0*k028*[Cl2O2] + 2.0*k204*[Cl2O2] +
!                k036*[BrCl]
!
!  pe(ClOx) = 0.0
!              + k194*[ClO]*[OH] + 2.0*k205*[ClO]*[ClO] +
!                2.0*k206*[ClO]*[ClO] + k218*[BrO]*[ClO] + k182*[HCHO]*[Cl] +
!                k189*[Cl]*[CH4] + k190*[Cl]*[H2] + k191*[Cl]*[HO2] +
!                k201*[Cl]*[NO2] + k202*[Cl]*[HOCl] + k196*[ClO]*[NO2] +
!                k198*[ClO]*[HO2] + 2.0*k203*[ClO]*[ClO] + k220*[BrO]*[ClO]
!
!  pr(BrOx) =  + k044*[CH3Br] + k045*[H1211] + k046*[H1301] +
!                k229*[O1D]*[CH3Br] + k231*[O1D]*[H1211] + k232*[O1D]*[H1301] +
!                2.0*k241*[HOBr]*[HBr]
!              + k223*[O1D]*[HBr]*partHBr +
!                k225*[OH]*[HBr]*partHBr + k033*[BrONO2] + k034*[HOBr] +
!                k228*[HOBr]*[O3P] + k036*[BrCl]
!
!  pe(BrOx) = 0.0
!              + k185*[HCHO]*[Br] + k222*[Br]*[HO2] +
!                k226*[BrO]*[NO2] + k227*[BrO]*[HO2] + k220*[BrO]*[ClO]
!
!
#ifdef MPI
i1 = i_deb(myid) ; i2 = i_fin(myid)
#else
i1 = 1           ; i2 = imax
#endif
!
!$omp parallel default(none) &
!$omp private (i,j) &
!$omp shared  (i1,i2,ntra,k,jco2,hnm,day,QM,HM,pr,pe) &
!$omp shared  (partHNO3,partH2O,partHCl,partHBr,imax)
!
!$omp do
do j=1,ntra
do i=1,imax
  pr(i,j) = 0.0
  pe(i,j) = 0.0
enddo
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  pe(i,1) =  + HM(i,2)*( + k(i,174) + k(i,175) ) + k(i,24)
!
  pr(i,2) =  + 0.03912*k(i,107)*QM(i,39)*HM(i,3)
!
  pe(i,2) =  + HM(i,2)*( + k(i,176) + k(i,177) + k(i,178) ) +                &
    k(i,85)*HM(i,13) + k(i,189)*HM(i,9)
!
  pe(i,4) = ( + k(i,5) + k(i,71)*HM(i,13) )*partHNO3(i)
!
  pr(i,5) =  + k(i,78)*HM(i,7)*HM(i,6)
!
  pe(i,5) =  + k(i,22) + k(i,79) + k(i,236)*QM(i,3) + k(i,237)*QM(i,8)
!
  pe(i,6) =  + k(i,83)*HM(i,13)
!
  pr(i,7) =  + k(i,218)*HM(i,10)*HM(i,8)
!
  pe(i,7) =  + k(i,35)
!
  pr(i,8) =  + HM(i,9)*( + k(i,182)*QM(i,20) + k(i,189)*QM(i,2) +            &
    k(i,190)*5.e-7*hnm(i) + k(i,191)*HM(i,14) ) + k(i,194)*HM(i,8)*HM(i,13)
!
  pr(i,9) =  + k(i,196)*HM(i,8)*HM(i,6)
!
  pe(i,9) =  + k(i,26) + k(i,27) + k(i,197)*HM(i,1) +                        &
    k(i,234)*QM(i,3) + k(i,235)*QM(i,8)
!
  pr(i,10) =  + k(i,198)*HM(i,8)*HM(i,14) + k(i,234)*QM(i,9)*QM(i,3)
!
  pr(i,11) =  + QM(i,10)*( + k(i,202)*HM(i,9) +                              &
    k(i,238)*QM(i,8) ) + HM(i,8)*( + k(i,205)*HM(i,8) +                      &
    k(i,206)*HM(i,8) ) + k(i,235)*QM(i,9)*QM(i,8)
!
  pe(i,11) =  + k(i,30)
!
  pr(i,12) =  + HM(i,14)*( + k(i,57)*HM(i,14) +                              &
    k(i,58)*HM(i,14)*QM(i,3)*partH2O(i) ) + HM(i,3)*( +                      &
    0.01643*k(i,107)*QM(i,39) + 0.001*k(i,108)*QM(i,40) ) +                  &
    k(i,171)*HM(i,13)*HM(i,13)
!
  pe(i,12) =  + k(i,10) + k(i,56)*HM(i,13)
!
  pr(i,13) =  + k(i,201)*HM(i,9)*HM(i,6) + k(i,237)*QM(i,5)*QM(i,8)
!
  pe(i,13) =  + k(i,31)
!
  pr(i,14) =  + HM(i,11)*( + k(i,185)*QM(i,20) + k(i,222)*HM(i,14) )
!
  pe(i,14) = ( + k(i,223)*HM(i,2) + k(i,225)*HM(i,13)                       &
    )*partHBr(i) + k(i,241)*QM(i,18) + k(i,242)*QM(i,10)
!
  pr(i,15) =  + k(i,226)*HM(i,10)*HM(i,6)
!
  pe(i,15) =  + k(i,33) + k(i,239)*QM(i,3)
!
  pr(i,16) =  + k(i,66)*HM(i,14)*HM(i,6)
!
  pe(i,16) =  + k(i,6) + k(i,7) + k(i,67) + k(i,72)*HM(i,13)
!
  pr(i,17) =  + k(i,203)*HM(i,8)*HM(i,8)
!
  pe(i,17) =  + k(i,28) + k(i,204)
!
  pr(i,18) =  + k(i,227)*HM(i,10)*HM(i,14) + k(i,239)*QM(i,15)*QM(i,3)
!
  pe(i,18) =  + k(i,34) + k(i,228)*HM(i,1) + k(i,240)*QM(i,8) +             &
    k(i,241)*QM(i,14)
!
  pr(i,19) =  + k(i,220)*HM(i,10)*HM(i,8) +                                 &
    k(i,240)*QM(i,18)*QM(i,8) + k(i,242)*QM(i,10)*QM(i,14)
!
  pe(i,19) =  + k(i,36)
!
  pr(i,22) =  + k(i,126)*QM(i,21)*HM(i,14)
!
  pe(i,22) =  + HM(i,13)*( + k(i,95) + k(i,96) ) + k(i,14)
!
  pe(i,23) =  + k(i,37) + k(i,207)*HM(i,2)
!
  pe(i,24) =  + k(i,38) + k(i,208)*HM(i,2)
!
  pe(i,25) =  + k(i,39) + k(i,209)*HM(i,2)
!
  pe(i,26) =  + k(i,40) + k(i,210)*HM(i,2)
!
  pe(i,27) =  + k(i,41) + k(i,211)*HM(i,13)
!
  pe(i,28) =  + k(i,42) + k(i,212)*HM(i,13)
!
  pe(i,29) =  + k(i,43) + k(i,213)*HM(i,2) + k(i,214)*HM(i,13)
!
  pe(i,30) =  + k(i,44) + k(i,229)*HM(i,2) + k(i,230)*HM(i,13)
!
  pe(i,31) =  + k(i,45) + k(i,231)*HM(i,2)
!
  pe(i,32) =  + k(i,46) + k(i,232)*HM(i,2)
!
  pr(i,35) =  + k(i,62)*HM(i,13)*HM(i,5) + k(i,113)*HM(i,17)*HM(i,6)
!
  pe(i,35) =  + k(i,4) + k(i,70)*HM(i,13)
!
  pe(i,36) =  + k(i,82)*HM(i,13)
!
  pr(i,37) =  + 0.02918*k(i,107)*QM(i,39)*HM(i,3)
!
  pe(i,37) =  + k(i,86)*HM(i,13)
!
  pe(i,38) =  + k(i,87)*HM(i,13)
!
  pe(i,39) =  + k(i,88)*HM(i,13) + k(i,104)*HM(i,7) + k(i,107)*HM(i,3)
!
  pe(i,40) =  + k(i,84)*HM(i,1) + k(i,89)*HM(i,13) +                        &
    k(i,105)*HM(i,7) + k(i,108)*HM(i,3)
!
  pr(i,41) =  + HM(i,17)*( + k(i,113)*HM(i,6) +                             &
    k(i,115)*0.21*hnm(i) + k(i,116)*HM(i,3) ) + QM(i,52)*( +                &
    0.10391*k(i,111)*HM(i,6) + 1.03908*k(i,112)*HM(i,14) )
!
  pe(i,41) =  + k(i,90)*HM(i,13) + k(i,103)*HM(i,7)
!
  pe(i,42) =  + k(i,13) + k(i,92)*HM(i,13) + k(i,101)*HM(i,7)
!
  pe(i,43) =  + k(i,17) + k(i,93)*HM(i,13)
!
  pe(i,44) =  + k(i,18) + k(i,94)*HM(i,13) + k(i,102)*HM(i,7) +             &
    k(i,109)*HM(i,3)
!
  pe(i,45) =  + k(i,19) + k(i,20) + k(i,99)*HM(i,13)
!
  pr(i,46) =  + QM(i,46)*( + 0.30781*k(i,98)*HM(i,13) +                     &
    0.4*k(i,106)*HM(i,7) + 0.3*k(i,110)*HM(i,3) ) +                         &
    k(i,117)*QM(i,54)*HM(i,6)
!
  pe(i,46) =  + k(i,98)*HM(i,13) + k(i,106)*HM(i,7) +                       &
    k(i,110)*HM(i,3) + k(i,118)
!
  pe(i,47) =  + k(i,15) + k(i,16) + k(i,97)*HM(i,13)
!
  pe(i,49) =  + k(i,120)*HM(i,5) + k(i,127)*HM(i,14) +                      &
    k(i,134)*QM(i,21) + k(i,140)*QM(i,54) + k(i,149)*HM(i,7)
!
  pr(i,50) =  + 1.01963*k(i,88)*QM(i,39)*HM(i,13)
!
  pe(i,50) =  + k(i,121)*HM(i,5) + k(i,128)*HM(i,14) +                      &
    k(i,135)*QM(i,21) + k(i,141)*QM(i,54) + k(i,150)*HM(i,7)
!
  pr(i,51) =  + k(i,89)*QM(i,40)*HM(i,13)
!
  pe(i,51) =  + k(i,122)*HM(i,5) + k(i,129)*HM(i,14) +                      &
    k(i,136)*QM(i,21) + k(i,142)*QM(i,54) + k(i,151)*HM(i,7)
!
  pr(i,52) =  + QM(i,41)*( + 0.00285*k(i,90)*HM(i,13) + k(i,103)*HM(i,7) )
!
  pe(i,52) =  + k(i,111)*HM(i,6) + k(i,112)*HM(i,14)
!
  pr(i,53) =  + k(i,114)*HM(i,17)*0.21*hnm(i)
!
  pe(i,53) =  + k(i,123)*HM(i,5) + k(i,130)*HM(i,14) +                      &
    k(i,137)*QM(i,21) + k(i,143)*QM(i,54) + k(i,152)*HM(i,7)
!
  pr(i,55) =  + HM(i,7)*( + 0.96421*k(i,104)*QM(i,39) + k(i,105)*QM(i,40) )
!
  pr(i,57) =  + k(i,82)*HM(i,13)*QM(i,36)
!
  pe(i,3) = ( + k(i,23) + k(i,52)*HM(i,2) )*partH2O(i) +                    &
    ( k(i,234)*QM(i,9) + k(i,236)*QM(i,5) + k(i,239)*QM(i,15) )*            &
    ( 1. - partH2O(i) )
!
  pe(i,8) = ( + k(i,29) + k(i,195)*HM(i,13)                                 &
    )*partHCl(i) + k(i,235)*QM(i,9) + k(i,237)*QM(i,5) +                    &
    k(i,238)*QM(i,10) + k(i,240)*QM(i,18)
!
  pe(i,10) =  + k(i,25) + k(i,199)*HM(i,13) + k(i,200)*HM(i,1) +            &
    k(i,202)*HM(i,9) + k(i,238)*QM(i,8) + k(i,242)*QM(i,14)
!
  pe(i,20) =  + k(i,11) + k(i,12) + k(i,91)*HM(i,13) +                      &
    k(i,100)*HM(i,7) + k(i,181)*HM(i,1) + k(i,182)*HM(i,9) +                &
    k(i,185)*HM(i,11)
!
  pr(i,39) =  + QM(i,51)*( + 0.3682*k(i,122)*HM(i,5) +                      &
    0.46806*k(i,136)*QM(i,21) + 0.23819*k(i,142)*QM(i,54) +                 &
    0.41604*k(i,151)*HM(i,7) ) + QM(i,40)*( +                               &
    0.89449*k(i,84)*HM(i,1) + 0.36404*k(i,108)*HM(i,3) )
!
  pr(i,47) =  + HM(i,14)*( + 1.00522*k(i,127)*QM(i,49) +                    &
    1.00522*k(i,128)*QM(i,50) + 1.00522*k(i,129)*QM(i,51) +                 &
    1.00522*k(i,130)*QM(i,53) + 0.85175*k(i,131)*QM(i,54) +                 &
    1.00522*k(i,155)*QM(i,56) ) + 0.10423*k(i,109)*QM(i,44)*HM(i,3)
!
  pr(i,49) =  + HM(i,13)*( + k(i,86)*QM(i,37) +                             &
    0.68285*k(i,87)*QM(i,38) + 0.39383*k(i,97)*QM(i,47) +                   &
    k(i,99)*QM(i,45) ) + QM(i,49)*( + 0.07839*k(i,120)*HM(i,5) +            &
    0.00359*k(i,134)*QM(i,21) + 0.00809*k(i,140)*QM(i,54) +                 &
    0.09067*k(i,149)*HM(i,7) ) + k(i,17)*QM(i,43) +                         &
    0.08956*k(i,107)*QM(i,39)*HM(i,3)
!
  pe(i,55) =  + 2.0*( + k(i,146)*QM(i,55) + k(i,147)*QM(i,55) ) +           &
    k(i,125)*HM(i,5) + k(i,132)*HM(i,14) + k(i,139)*QM(i,21) +              &
    k(i,145)*QM(i,54) + k(i,154)*HM(i,7)
!
  pe(i,56) =  + k(i,155)*HM(i,14) + k(i,156)*QM(i,21) +                     &
    k(i,157)*QM(i,54) + 2.0*k(i,158)*QM(i,56) + k(i,159)*HM(i,5) +          &
    k(i,160)*HM(i,7)
!
  pr(i,3) =  + HM(i,13)*( + k(i,55)*HM(i,14) + k(i,56)*QM(i,12) +           &
    k(i,70)*QM(i,35) + k(i,71)*QM(i,4)*partHNO3(i) +                        &
    k(i,72)*QM(i,16) + k(i,81)*5.e-7*hnm(i) + k(i,85)*QM(i,2) +             &
    k(i,86)*QM(i,37) + k(i,87)*QM(i,38) + k(i,91)*QM(i,20) +                &
    k(i,92)*QM(i,42) + k(i,93)*QM(i,43) + 0.76844*k(i,94)*QM(i,44) +        &
    k(i,95)*QM(i,22) + k(i,96)*QM(i,22) + 0.23046*k(i,98)*QM(i,46) +        &
    k(i,99)*QM(i,45) + k(i,170)*HM(i,13) +                                  &
    k(i,195)*QM(i,8)*partHCl(i) + k(i,199)*QM(i,10) +                       &
    k(i,211)*QM(i,27) + k(i,212)*QM(i,28) + k(i,214)*QM(i,29) +             &
    k(i,225)*QM(i,14)*partHBr(i) + k(i,230)*QM(i,30) ) +                    &
    k(i,168)*HM(i,12)*HM(i,14) + QM(i,10)*( + k(i,238)*QM(i,8) +            &
    k(i,242)*QM(i,14) ) + QM(i,18)*( + k(i,240)*QM(i,8) +                   &
    k(i,241)*QM(i,14) )
!
  pr(i,4) =  + HM(i,7)*( + k(i,68)*HM(i,14) + k(i,100)*QM(i,20) +    &
    k(i,101)*QM(i,42) + 0.90869*k(i,102)*QM(i,44) +    &
    k(i,103)*QM(i,41) ) + QM(i,3)*( + k(i,234)*QM(i,9) +    &
    2.0*k(i,236)*QM(i,5) + k(i,239)*QM(i,15) ) + QM(i,8)*( +    &
    k(i,235)*QM(i,9) + k(i,237)*QM(i,5) ) + k(i,63)*HM(i,13)*HM(i,6)
!
  pr(i,6) =  + QM(i,20)*( + k(i,11) + k(i,12) + k(i,91)*HM(i,13) +    &
    k(i,100)*HM(i,7) + k(i,181)*HM(i,1) + k(i,182)*HM(i,9) +    &
    k(i,185)*HM(i,11) ) + QM(i,44)*( + 0.9118*k(i,18) +    &
    1.03417*k(i,94)*HM(i,13) + 1.28646*k(i,102)*HM(i,7) +    &
    0.65571*k(i,109)*HM(i,3) ) + HM(i,3)*( +    &
    0.36475*k(i,107)*QM(i,39) + 0.36*k(i,108)*QM(i,40) +    &
    0.13*k(i,110)*QM(i,46) ) + k(i,13)*QM(i,42) +    &
    0.01*k(i,84)*QM(i,40)*HM(i,1) +    &
#ifdef NIV_60
    jco2(i)*rco2*hnm(i) + &
#endif
    0.02327*k(i,87)*QM(i,38)*HM(i,13)
!
  pr(i,20) =  + QM(i,21)*( + 0.66*k(i,133)*QM(i,21) +    &
    0.80572*k(i,134)*QM(i,49) + 1.42733*k(i,135)*QM(i,50) +    &
    1.09*k(i,136)*QM(i,51) + k(i,137)*QM(i,53) +    &
    0.96426*k(i,138)*QM(i,54) + 0.88103*k(i,139)*QM(i,55) +    &
    k(i,148)*HM(i,7) + k(i,156)*QM(i,56) ) + QM(i,54)*( +    &
    0.06891*k(i,124)*HM(i,5) + 0.07663*k(i,140)*QM(i,49) +    &
    0.6812*k(i,141)*QM(i,50) + 0.34*k(i,142)*QM(i,51) +    &
    0.04453*k(i,144)*QM(i,54) + 0.12889*k(i,145)*QM(i,55) +    &
    0.04229*k(i,153)*HM(i,7) ) + HM(i,7)*( + 0.4*k(i,106)*QM(i,46) +    &
    0.03209*k(i,149)*QM(i,49) + 1.45792*k(i,150)*QM(i,50) +    &
    0.686*k(i,151)*QM(i,51) + 0.19943*k(i,154)*QM(i,55) ) +    &
    HM(i,13)*( + 0.00238*k(i,87)*QM(i,38) + k(i,96)*QM(i,22) +    &
    0.03679*k(i,97)*QM(i,47) + 0.53828*k(i,98)*QM(i,46) ) +    &
    HM(i,5)*( + 0.03065*k(i,120)*QM(i,49) +    &
    1.40361*k(i,121)*QM(i,50) + 0.606*k(i,122)*QM(i,51) +    &
    0.23919*k(i,125)*QM(i,55) ) + HM(i,3)*( +    &
    0.54280*k(i,107)*QM(i,39) + 0.9*k(i,108)*QM(i,40) +    &
    0.7*k(i,110)*QM(i,46) ) + HM(i,1)*( + 0.05*k(i,84)*QM(i,40) +    &
    k(i,179)*HM(i,15) ) + 0.0605*k(i,18)*QM(i,44) +    &
    k(i,178)*HM(i,2)*QM(i,2) + k(i,183)*HM(i,16)*0.21*hnm(i)
!
  pr(i,21) =  + QM(i,54)*( + 0.77298*k(i,124)*HM(i,5) +    &
    0.58551*k(i,138)*QM(i,21) + 0.51431*k(i,140)*QM(i,49) +    &
    0.5008*k(i,141)*QM(i,50) + 0.506*k(i,142)*QM(i,51) +    &
    k(i,143)*QM(i,53) + 1.61972*k(i,144)*QM(i,54) +    &
    0.50996*k(i,145)*QM(i,55) + 0.89016*k(i,153)*HM(i,7) +    &
    k(i,157)*QM(i,56) ) + QM(i,49)*( + 0.09255*k(i,120)*HM(i,5) +    &
    0.01346*k(i,134)*QM(i,21) + 0.09954*k(i,149)*HM(i,7) ) +    &
    HM(i,3)*( + 0.12708*k(i,107)*QM(i,39) +    &
    0.03*k(i,108)*QM(i,40) ) + k(i,13)*QM(i,42) + k(i,16)*QM(i,47) +    &
    k(i,95)*QM(i,22)*HM(i,13) + k(i,180)*HM(i,15)*0.21*hnm(i)
!
  pe(i,21) =  + k(i,119)*HM(i,5) + k(i,126)*HM(i,14) +    &
    2.0*k(i,133)*QM(i,21) + k(i,134)*QM(i,49) + k(i,135)*QM(i,50) +    &
    k(i,136)*QM(i,51) + k(i,137)*QM(i,53) + k(i,138)*QM(i,54) +    &
    k(i,139)*QM(i,55) + k(i,148)*HM(i,7) + k(i,156)*QM(i,56) +    &
    k(i,184)*HM(i,8)
!
  pr(i,42) =  + QM(i,54)*( + 0.07273*k(i,124)*HM(i,5) +    &
    0.08696*k(i,138)*QM(i,21) + 0.71704*k(i,140)*QM(i,49) +    &
    0.68371*k(i,141)*QM(i,50) + 0.07834*k(i,144)*QM(i,54) +    &
    0.40474*k(i,145)*QM(i,55) + 0.04036*k(i,153)*HM(i,7) ) +    &
    HM(i,7)*( + 0.05049*k(i,102)*QM(i,44) +    &
    0.32478*k(i,149)*QM(i,49) + 0.37131*k(i,150)*QM(i,50) +    &
    0.88319*k(i,154)*QM(i,55) ) + HM(i,13)*( +    &
    0.21989*k(i,87)*QM(i,38) + 0.05649*k(i,94)*QM(i,44) +    &
    0.07161*k(i,97)*QM(i,47) ) + HM(i,5)*( +    &
    0.33628*k(i,120)*QM(i,49) + 0.41431*k(i,121)*QM(i,50) +    &
    1.03345*k(i,125)*QM(i,55) ) + QM(i,21)*( +    &
    0.56293*k(i,134)*QM(i,49) + 0.46518*k(i,135)*QM(i,50) +    &
    0.39935*k(i,139)*QM(i,55) ) + HM(i,3)*( +    &
    0.47021*k(i,107)*QM(i,39) + 0.15896*k(i,109)*QM(i,44) ) +    &
    k(i,15)*QM(i,47) + k(i,20)*QM(i,45)
!
  pr(i,43) =  + HM(i,7)*( + 0.00606*k(i,102)*QM(i,44) +    &
    0.6412*k(i,149)*QM(i,49) + 0.01531*k(i,150)*QM(i,50) +    &
    0.33405*k(i,154)*QM(i,55) ) + QM(i,54)*( +    &
    0.18281*k(i,140)*QM(i,49) + 0.06648*k(i,141)*QM(i,50) +    &
    0.02462*k(i,144)*QM(i,54) + 0.10399*k(i,145)*QM(i,55) ) +    &
    HM(i,13)*( + 0.05953*k(i,87)*QM(i,38) +    &
    0.0077*k(i,94)*QM(i,44) + 0.36698*k(i,97)*QM(i,47) ) +    &
    HM(i,5)*( + 0.54089*k(i,120)*QM(i,49) +    &
    0.04833*k(i,121)*QM(i,50) + 0.38671*k(i,125)*QM(i,55) ) +    &
    QM(i,21)*( + 0.09095*k(i,134)*QM(i,49) +    &
    0.03882*k(i,135)*QM(i,50) + 0.09297*k(i,139)*QM(i,55) ) +    &
    k(i,19)*QM(i,45) + 0.06764*k(i,107)*QM(i,39)*HM(i,3)
!
  pr(i,44) =  + QM(i,54)*( + 0.0855*k(i,124)*HM(i,5) +    &
    0.16878*k(i,138)*QM(i,21) + 0.0729*k(i,140)*QM(i,49) +    &
    0.77601*k(i,142)*QM(i,51) + 1.99453*k(i,143)*QM(i,53) +    &
    0.12104*k(i,144)*QM(i,54) + 0.04744*k(i,153)*HM(i,7) ) +    &
    HM(i,7)*( + 0.10099*k(i,102)*QM(i,44) +    &
    0.90585*k(i,105)*QM(i,40) + 0.03717*k(i,149)*QM(i,49) +    &
    0.60390*k(i,151)*QM(i,51) + 2.83695*k(i,152)*QM(i,53) ) +    &
    HM(i,13)*( + 0.02466*k(i,87)*QM(i,38) +    &
    0.14812*k(i,94)*QM(i,44) + 0.27962*k(i,98)*QM(i,46) ) +    &
    HM(i,5)*( + 0.03609*k(i,120)*QM(i,49) +    &
    0.4489*k(i,122)*QM(i,51) + 2.08075*k(i,123)*QM(i,53) ) +    &
    QM(i,21)*( + 0.08396*k(i,134)*QM(i,49) +    &
    0.55357*k(i,136)*QM(i,51) + 1.99456*k(i,137)*QM(i,53) ) +    &
    QM(i,40)*( + 0.13085*k(i,84)*HM(i,1) +    &
    0.39253*k(i,108)*HM(i,3) ) + 1.1017*k(i,109)*QM(i,44)*HM(i,3)
!
  pr(i,45) =  + QM(i,55)*( + 0.16657*k(i,125)*HM(i,5) +    &
    k(i,132)*HM(i,14) + 0.68801*k(i,139)*QM(i,21) +    &
    0.6787*k(i,145)*QM(i,54) + 2.0*k(i,146)*QM(i,55) +    &
    0.28775*k(i,154)*HM(i,7) ) + HM(i,5)*( +    &
    0.08116*k(i,120)*QM(i,49) + 0.153*k(i,122)*QM(i,51) +    &
    0.04878*k(i,123)*QM(i,53) ) + 0.6*k(i,106)*QM(i,46)*HM(i,7) +    &
    k(i,111)*QM(i,52)*HM(i,6)
!
  pr(i,54) =  + QM(i,44)*( + 0.71757*k(i,18) +    &
    0.53660*k(i,94)*HM(i,13) + 0.42992*k(i,102)*HM(i,7) +    &
    0.27818*k(i,109)*HM(i,3) ) + QM(i,54)*( +    &
    0.10527*k(i,124)*HM(i,5) + 0.07152*k(i,138)*QM(i,21) +    &
    0.07139*k(i,144)*QM(i,54) + 0.04229*k(i,153)*HM(i,7) ) +    &
    HM(i,13)*( + k(i,92)*QM(i,42) + k(i,93)*QM(i,43) +    &
    0.06833*k(i,97)*QM(i,47) ) + HM(i,3)*( +    &
    0.05225*k(i,107)*QM(i,39) + 0.17*k(i,108)*QM(i,40) +    &
    0.7*k(i,110)*QM(i,46) ) + k(i,17)*QM(i,43) +    &
    k(i,101)*QM(i,42)*HM(i,7) + k(i,118)*QM(i,46)
!
  pe(i,54) =  + k(i,117)*HM(i,6) + k(i,124)*HM(i,5) +    &
    k(i,131)*HM(i,14) + k(i,138)*QM(i,21) + k(i,140)*QM(i,49) +    &
    k(i,141)*QM(i,50) + k(i,142)*QM(i,51) + k(i,143)*QM(i,53) +    &
    2.0*k(i,144)*QM(i,54) + k(i,145)*QM(i,55) + k(i,153)*HM(i,7) +    &
    k(i,157)*QM(i,56)
!
  pr(i,56) =  + QM(i,54)*( + 0.02530*k(i,124)*HM(i,5) +    &
    0.02319*k(i,138)*QM(i,21) + 0.10953*k(i,140)*QM(i,49) +    &
    0.01791*k(i,144)*QM(i,54) + 0.01404*k(i,153)*HM(i,7) ) +    &
    HM(i,13)*( + 0.10260*k(i,90)*QM(i,41) +    &
    0.09179*k(i,94)*QM(i,44) + 0.09945*k(i,97)*QM(i,47) +    &
    k(i,98)*QM(i,46) ) + HM(i,7)*( + 0.10099*k(i,102)*QM(i,44) +    &
    k(i,106)*QM(i,46) + 0.16478*k(i,149)*QM(i,49) ) + QM(i,40)*( +    &
    0.15*k(i,84)*HM(i,1) + 0.13*k(i,108)*HM(i,3) ) + QM(i,49)*( +    &
    0.12222*k(i,120)*HM(i,5) + 0.12985*k(i,134)*QM(i,21) )
!
  pr(i,58) = + 0.21*hnm(i)*( + 2.0*k(i,21) + k(i,173)*HM(i,4) ) +    &
    HM(i,5)*( + k(i,47) + k(i,172)*HM(i,4) ) + HM(i,14)*( +    &
    k(i,168)*HM(i,12) + 0.13345*k(i,131)*QM(i,54) ) +    &
    k(i,1)*HM(i,6) + k(i,9)*HM(i,7) + k(i,32)*HM(i,10) +    &
    k(i,35)*QM(i,7) + k(i,170)*HM(i,13)*HM(i,13) + k(i,24)*QM(i,1)
!
  pe(i,58) =  + k(i,187)*HM(i,8)*HM(i,1) +    &
    2.0*k(i,203)*HM(i,8)*HM(i,8)*k(i,28)/(k(i,28) + k(i,204)) +    &
    MIN(k(i,25)*QM(i,10),k(i,198)*HM(i,14)*HM(i,8)) +    &
    2.0*MIN(k(i,36)*QM(i,19),k(i,220)*HM(i,10)*HM(i,8)) +    &
    2.0*k(i,219)*HM(i,10)*HM(i,8)*day(i) +    &
    MIN(k(i,34)*QM(i,18),k(i,227)*HM(i,14)*HM(i,10)) +    &
    2.0*k(i,221)*HM(i,10)*HM(i,10)*day(i) + 2.0*(k(i,27) +    &
    k(i,26))*QM(i,9)
  pe(i,58) = pe(i,58) + HM(i,1)*( + 2.0*k(i,49)*HM(i,3) +    &
    k(i,59)*HM(i,5) + k(i,60)*HM(i,6) + k(i,61)*HM(i,6) +    &
    k(i,84)*QM(i,40) + k(i,164)*HM(i,13) + k(i,165)*HM(i,14) +    &
    k(i,169)*5.e-7*hnm(i) + k(i,179)*HM(i,15) + k(i,181)*QM(i,20) +    &
    k(i,187)*HM(i,8) + k(i,197)*QM(i,9) + k(i,200)*QM(i,10) +    &
    k(i,216)*HM(i,10) + k(i,228)*QM(i,18) + 2.0*k(i,233)*HM(i,1) ) +    &
    HM(i,2)*( + k(i,52)*QM(i,3) + k(i,163)*5.e-7*hnm(i) +    &
    k(i,174)*QM(i,1) + k(i,175)*QM(i,1) + k(i,176)*QM(i,2) +    &
    k(i,177)*QM(i,2) + k(i,178)*QM(i,2) + k(i,207)*QM(i,23) +    &
    k(i,208)*QM(i,24) + k(i,209)*QM(i,25) + k(i,210)*QM(i,26) +    &
    k(i,213)*QM(i,29) + k(i,223)*QM(i,14) + k(i,229)*QM(i,30) +    &
    k(i,231)*QM(i,31) + k(i,232)*QM(i,32) ) + HM(i,3)*( +    &
    0.910000*k(i,108)*QM(i,40) + k(i,53)*HM(i,13) +    &
    k(i,54)*HM(i,14) + k(i,73)*HM(i,5) + k(i,74)*HM(i,6) +    &
    k(i,107)*QM(i,39) + k(i,109)*QM(i,44) + k(i,110)*QM(i,46) +    &
    k(i,116)*HM(i,17) + k(i,162)*HM(i,12) + k(i,186)*HM(i,9) +    &
    k(i,215)*HM(i,11) )
!
  pe(i,58) = pe(i,58) / QM(i,58)
!
  pr(i,59) =  + 2.0*k(i,22)*QM(i,5) + k(i,197)*HM(i,1)*QM(i,9)
  pr(i,59) = pr(i,59) + HM(i,13)*( + k(i,70)*QM(i,35) +    &
    k(i,72)*QM(i,16) + k(i,99)*QM(i,45) +    &
    k(i,71)*QM(i,4)*partHNO3(i) + 0.69128*k(i,98)*QM(i,46) ) +    &
    QM(i,16)*( + k(i,6) + k(i,67) + k(i,7) ) + QM(i,46)*( +    &
    0.7*k(i,110)*HM(i,3) + k(i,118) ) +    &
    QM(i,55)*( + 0.833420*k(i,125)*HM(i,5) +    &
    0.31199*k(i,139)*QM(i,21) + 0.712250*k(i,154)*HM(i,7) ) +    &
    2.0*( + k(i,175)*HM(i,2)*QM(i,1) + k(i,79)*QM(i,5) ) +    &
    QM(i,45)*( + k(i,19) + k(i,20) ) + QM(i,9)*( + k(i,27) +    &
    k(i,26) ) + k(i,4)*QM(i,35) + k(i,5)*QM(i,4)*partHNO3(i) +    &
    k(i,31)*QM(i,13) + k(i,33)*QM(i,15)
!
  pe(i,59) = + HM(i,6)*( + k(i,63)*HM(i,13) + k(i,66)*HM(i,14) +    &
    2.0*k(i,78)*HM(i,7) + k(i,111)*QM(i,52) + k(i,113)*HM(i,17) +    &
    k(i,117)*QM(i,54) + k(i,196)*HM(i,8) + k(i,201)*HM(i,9) +    &
    k(i,226)*HM(i,10) ) + HM(i,7)*( + 0.899010*k(i,102)*QM(i,44) +    &
    k(i,68)*HM(i,14) + k(i,100)*QM(i,20) + k(i,101)*QM(i,42) +    &
    k(i,103)*QM(i,41) + k(i,104)*QM(i,39) + k(i,105)*QM(i,40) ) +    &
    HM(i,5)*( + 2.0*k(i,172)*HM(i,4) + k(i,62)*HM(i,13) +    &
    0.081160*k(i,120)*QM(i,49) + 0.153000*k(i,122)*QM(i,51) +    &
    0.048780*k(i,123)*QM(i,53) )
!
  pe(i,59) = pe(i,59) / QM(i,59)
!
  pr(i,60) =  + 3.0*( + k(i,37)*QM(i,23) + k(i,39)*QM(i,25) +    &
    k(i,41)*QM(i,27) + k(i,207)*HM(i,2)*QM(i,23) +    &
    k(i,209)*HM(i,2)*QM(i,25) + k(i,211)*HM(i,13)*QM(i,27) ) +    &
    QM(i,29)*( + k(i,43) + k(i,213)*HM(i,2) + k(i,214)*HM(i,13) ) +    &
    2.0*( + k(i,38)*QM(i,24) + k(i,208)*HM(i,2)*QM(i,24) ) + 4.0*( +    &
    k(i,40)*QM(i,26) + k(i,210)*HM(i,2)*QM(i,26) ) + QM(i,28)*( +    &
    k(i,42) + k(i,212)*HM(i,13) ) + k(i,197)*HM(i,1)*QM(i,9)
  pr(i,60) = pr(i,60) + 2.0*( + k(i,30)*QM(i,11) +    &
    k(i,28)*QM(i,17) + k(i,204)*QM(i,17) ) + QM(i,10)*( + k(i,25) +    &
    k(i,199)*HM(i,13) + k(i,200)*HM(i,1) ) + QM(i,8)*( +    &
    k(i,29)*partHCl(i) + k(i,195)*HM(i,13)*partHCl(i) ) +    &
    QM(i,9)*( + k(i,26) + k(i,27) ) + k(i,35)*QM(i,7) +    &
    k(i,31)*QM(i,13) + k(i,36)*QM(i,19)
!
  pe(i,60) = + HM(i,8)*( + k(i,194)*HM(i,13) +    &
    2.0*k(i,205)*HM(i,8) + 2.0*k(i,206)*HM(i,8) +    &
    k(i,218)*HM(i,10) + k(i,196)*HM(i,6) + k(i,198)*HM(i,14) +    &
    2.0*k(i,203)*HM(i,8) + k(i,220)*HM(i,10) ) + HM(i,9)*( +    &
    k(i,182)*QM(i,20) + k(i,189)*QM(i,2) + k(i,190)*5.e-7*hnm(i) +    &
    k(i,191)*HM(i,14) + k(i,201)*HM(i,6) + k(i,202)*QM(i,10) )
!
  pe(i,60) = pe(i,60) / QM(i,60)
!
  pr(i,61) =  + HM(i,2)*( + k(i,229)*QM(i,30) + k(i,231)*QM(i,31) +    &
    k(i,232)*QM(i,32) ) + k(i,44)*QM(i,30) + k(i,45)*QM(i,31) +    &
    k(i,46)*QM(i,32) + 2.0*k(i,241)*QM(i,18)*QM(i,14)
  pr(i,61) = pr(i,61) + QM(i,14)*( + k(i,223)*HM(i,2)*partHBr(i) +    &
    k(i,225)*HM(i,13)*partHBr(i) ) + QM(i,18)*( + k(i,34) +    &
    k(i,228)*HM(i,1) ) + k(i,33)*QM(i,15) + k(i,36)*QM(i,19)
!
  pe(i,61) = + HM(i,10)*( + k(i,226)*HM(i,6) + k(i,227)*HM(i,14) +    &
    k(i,220)*HM(i,8) ) + HM(i,11)*( + k(i,185)*QM(i,20) +    &
    k(i,222)*HM(i,14) )
!
  pe(i,61) = pe(i,61) / QM(i,61)
#ifdef MASSy
!
  pr(i,62) = + 2.0*k(i,175)*HM(i,2)*QM(i,1)
!
  pe(i,62) = + 2.0*k(i,172)*HM(i,4)*HM(i,5)
!
  pe(i,62) = pe(i,62) / QM(i,62)
!
  pr(i,63) =    &
    3.0*( + k(i,37) + k(i,207)*HM(i,2) )*QM(i,23) +    &
    2.0*( + k(i,38) + k(i,208)*HM(i,2) )*QM(i,24) +    &
    3.0*( + k(i,39) + k(i,209)*HM(i,2) )*QM(i,25) +    &
    4.0*( + k(i,40) + k(i,210)*HM(i,2) )*QM(i,26) +    &
    3.0*( + k(i,41) + k(i,211)*HM(i,13) )*QM(i,27) +    &
        ( + k(i,42) + k(i,212)*HM(i,13) )*QM(i,28) +    &
        ( + k(i,43) + k(i,213)*HM(i,2) +    &
            k(i,214)*HM(i,13) )*QM(i,29)
!
  pr(i,64) =                                                       &
    HM(i,2)*( + k(i,229)*QM(i,30) + k(i,231)*QM(i,31) +            &
    k(i,232)*QM(i,32) ) + k(i,44)*QM(i,30) + k(i,45)*QM(i,31) +    &
    k(i,46)*QM(i,32)
#endif
!
enddo
!$omp end do
!
!$omp end parallel
!
RETURN
END SUBROUTINE CH_PTERMS
