SUBROUTINE KRATES(pm,t,hnm,tj,het,k,imax,nphot,nhet,nk)
!
!!!!!
!
!   CALCUL DES CONTANTES DE VITESSE DES REACTIONS CHIMIQUES
!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!
!   Centre National de Recherches Meteorologiques
!   Groupe de Modelisation de Grande Echelle et du Climat
!   Equipe de Recherches sur l'Atmosphere Moyenne
!
!!!!!
!
!  Original : H. Teyssedre, V.-H. Peuch, 08/2000
!
!  Modified : updated according to JPL-2000 for
!             stratospheric reactions, (Teyssedre, 12/2004)
!             H. Teyssedre, version Fortran-90, 11/2006
!
!!!!!
!
IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
INTEGER :: imax, nphot, nhet, nk
REAL (KIND=8) , DIMENSION (imax) :: pm,t,hnm
REAL (KIND=8) , DIMENSION (imax,nphot):: tj
REAL (KIND=8) , DIMENSION (imax,nhet):: het
REAL (KIND=8) , DIMENSION (imax,nk) :: k
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
INTEGER :: I
REAL (KIND=8) :: XK0,XE0,XN0,XKINF,XEINF,XMINF,XF,ZK0M
REAL (KIND=8) :: ZKINF,ZFACT,XAEQ,XBEQ
!
!$omp parallel default(none) &
!$omp private(i,xaeq,xbeq,xk0,xE0,xn0,xkinf,xEinf,xminf,xF,zk0m,zkinf,zfact) &
!$omp shared(imax,k,pm,tj,t,het,hnm)
!
!$omp do
do i=1,imax
!
!  NO2  -->  O3P + NO
!
  k(i,1) = tj(i,001)
!
!  O3  -->  O1D + O2
!
  k(i,2) = tj(i,002)
!
!  O3  -->  O3P + O2
!
  k(i,3) = tj(i,003)
!
!  HONO  -->  OH + NO
!
  k(i,4) = tj(i,004)
!
!  HNO3  -->  OH + NO2
!
  k(i,5) = tj(i,005)
!
!  HNO4  -->  HO2 + NO2
!
  k(i,6) = 0.65*tj(i,006)
!
!  HNO4  -->  OH + NO3
!
  k(i,7) = 0.35*tj(i,006)
!
!  NO3  -->  NO + O2
!
  k(i,8) = tj(i,007)
!
!  NO3  -->  NO2 + O3P
!
  k(i,9) = tj(i,008)
!
!  H2O2  -->  OH + OH
!
  k(i,10) = tj(i,009)
!
!  HCHO  -->  H2 + CO
!
  k(i,11) = tj(i,010)
!
!  HCHO  -->  H + HO2 + CO
!
  k(i,12) = tj(i,011)
!
!  ALD  -->  MO2 + HO2 + CO
!
  k(i,13) = tj(i,012)
!
!  OP1  -->  CH3O + OH
!
  k(i,14) = tj(i,013)
!
!  OP2  -->  ALD + HO2 + OH
!
  k(i,15) = 0.95105*tj(i,014)
!
!  OP2  -->  MO2 + OH
!
  k(i,16) = 0.04895*tj(i,014)
!
!  KET  -->  ACO3 + HC8P
!
  k(i,17) = tj(i,015)
!
!  MACR  -->  0.0605*HCHO + 0.71757*ACO3 + 0.77528*HO2 + 0.9118*CO + 0.19378*H2
!
  k(i,18) = tj(i,016)
!
!  ONIT  --> KET + HO2 + NO2
!
  k(i,19) = 0.8*tj(i,017)
!
!  ONIT  --> ALD + HO2 + NO2
!
  k(i,20) = 0.2*tj(i,017)
!
!  O2  -->  O3P + O3P
!
  k(i,21) = tj(i,018)
!
!  N2O5  -->  NOx + NOx + XX
!
  k(i,22) = tj(i,019)
!
!  H2O  -->  H + OH
!
  k(i,23) = tj(i,020)
!
!  N2O  -->  N2 + O1D
!
  k(i,24) = tj(i,021)
!
!  HOCl  -->  Cl + OH
!
  k(i,25) = tj(i,022)
!
!  ClONO2  -->  Cl + NO3
!
  k(i,26) = tj(i,023)
!
!  ClONO2  -->  ClO + NO2
!
  k(i,27) = tj(i,024)
!
!  Cl2O2  -->  Cl + Cl + O2
!
  k(i,28) = tj(i,025)
!
!  HCl  -->  H + Cl
!
  k(i,29) = tj(i,026)
!
!  Cl2  -->  Cl + Cl
!
  k(i,30) = tj(i,027)
!
!  ClNO2  -->  Cl + NO2
!
  k(i,31) = tj(i,028)
!
!  BrO  -->  Br + O3P
!
  k(i,32) = tj(i,029)
!
!  BrONO2  -->  Br + NO3
!
  k(i,33) = tj(i,030)
!
!  HOBr  -->  Br + OH
!
  k(i,34) = tj(i,031)
!
!  OClO  -->  O3P + ClO
!
  k(i,35) = tj(i,032)
!
!  BrCl  -->  Br + Cl
!
  k(i,36) = tj(i,033)
!
!  CFC11  -->  ClOx + ClOx + ClOx + XX
!
  k(i,37) = tj(i,034)
!
!  CFC12  -->  ClOx + ClOx + XX
!
  k(i,38) = tj(i,035)
!
!  CFC113  -->  ClOx + ClOx + ClOx + XX
!
  k(i,39) = tj(i,036)
!
!  CCl4  -->  ClOx + ClOx + ClOx + ClOx + XX
!
  k(i,40) = tj(i,037)
!
!  CH3CCl3  -->  ClOx + ClOx + ClOx + XX
!
  k(i,41) = tj(i,038)
!
!  CH3Cl  -->  ClOx + XX
!
  k(i,42) = tj(i,039)
!
!  HCFC22  -->  ClOx + XX
!
  k(i,43) = tj(i,040)
!
!  CH3Br  -->  BrOx + XX
!
  k(i,44) = tj(i,041)
!
!  H1211  -->  BrOx + XX
!
  k(i,45) = tj(i,042)
!
!  H1301  -->  BrOx + XX
!
  k(i,46) = tj(i,043)
!
!  NO  -->  N + O3P
!
  k(i,47) = tj(i,044)
!
!  O3P + O2  -->  O3
!
  k(i,48) = hnm(i)*6.00E-34*(t(i)/300)**(-2.4)
!
!  O3P + O3  -->  O2 + O2
!
  k(i,49) = 8.00E-12*exp(-(2060.0/t(i)))
!
!  O1D + N2  -->  O3P + N2
!
  k(i,50) = 1.80E-11*exp(-(-110.0/t(i)))
!
enddo
!$omp end do
!
!$omp do
do i=1,imax
!
!  O1D + O2  -->  O3P + O2
!
  k(i,51) = 3.20E-11*exp(-(-70.0/t(i)))
!
!  O1D + H2O  -->  OH + OH
!
  k(i,52) = 2.20E-10
!
!  O3 + OH  -->  HO2 + O2
!
  k(i,53) = 1.70E-12*exp(-(940.0/t(i)))
!
!  O3 + HO2  -->  OH + O2 + O2
!
  k(i,54) = 1.00E-14*exp(-(490.0/t(i)))
!
!  OH + HO2  -->  H2O + O2
!
  k(i,55) = 4.80E-11*exp(-(-250.0/t(i)))
!
!  H2O2 + OH  -->  HO2 + H2O
!
  k(i,56) = 2.90E-12*exp(-(160.0/t(i)))
!
!  HO2 + HO2  -->  H2O2 + O2
!
  k(i,57) = 2.3E-13*EXP(600./t(i))+1.7E-33*hnm(i)*exp(1000./t(i))
!
! HO2 + HO2 + H2O --> H2O2 + H2O + O2
!
  k(i,58) = 3.22E-34*EXP(2800./t(i))+2.38E-30*1.E-24*hnm(i)*EXP(3200./t(i))
!
!  O3P + NO  -->  NO2
!
  xk0   = 9.00E-31
  xE0   = 0.0
  xn0   = 1.5
  xkinf = 3.00E-11
  xEinf = 0.0
  xminf = 0.0
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,59) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  O3P + NO2  -->  NO + O2
!
  k(i,60) = 5.60E-12*exp(-(-180.0/t(i)))
!
!  O3P + NO2  -->  NO3
!
  xk0   = 2.50E-31
  xE0   = 0.0
  xn0   = 1.8
  xkinf = 2.20E-11
  xEinf = 0.0
  xminf = 0.7
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,61) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  OH + NO  -->  HONO
!
  xk0   = 7.00E-31
  xE0   = 0.0
  xn0   = 2.6
  xkinf = 3.60E-11
  xEinf = 0.0
  xminf = 0.1
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,62) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  OH + NO2  -->  HNO3
!
  xk0   = 2.00E-30
  xE0   = 0.0
  xn0   = 3.0
  xkinf = 2.50E-11
  xEinf = 0.0
  xminf = 0.0
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,63) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  OH + NO3  -->  NO2 + HO2
!
  k(i,64) = 2.20E-11
!
!  HO2 + NO  -->  NO2 + OH
!
  k(i,65) = 3.50E-12*exp(-(-250.0/t(i)))
!
!  HO2 + NO2  -->  HNO4
!
  xk0   = 1.80E-31
  xE0   = 0.0
  xn0   = 3.2
  xkinf = 4.70E-12
  xEinf = 0.0
  xminf = 1.4
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,66) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  HNO4  -->  HO2 + NO2
!
  xk0   = 1.80E-31
  xn0   = 3.2
  xkinf = 4.70E-12
  xminf = 1.4
  xaeq  = 2.10E-27
  xbeq  = 10900.0
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
  zkinf = xkinf*((t(i)/300.)**(-xminf))
  zfact = 0.6**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,67) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact
!
!  HO2 + NO3  --> HNO3
!
  k(i,68) = 0.3*3.50E-12
!
!  HO2 + NO3  --> NO2 + OH
!
  k(i,69) = 0.7*3.50E-12
!
!  OH + HONO  -->  H2O + NO2
!
  k(i,70) = 1.80E-11*exp(-(390.0/t(i)))
!
!  OH + HNO3  -->  NO3 + H2O
!
  k(i,71) = (2.4E-14*EXP(460/t(i)))+(6.5E-34*EXP(1335/t(i))*hnm(i))   &
       /(1+(6.5E-34*EXP(1335/t(i))*hnm(i))/(2.7E-17*EXP(2199/t(i))))
!
!  OH + HNO4  -->  NO2 + H2O + O2
!
  k(i,72) = 1.30E-12*exp(-(-380.0/t(i)))
!
!  O3 + NO  -->  NO2 + O2
!
  k(i,73) = 3.00E-12*exp(-(1500.0/t(i)))
!
!  O3 + NO2  -->  NO3 + O2
!
  k(i,74) = 1.20E-13*exp(-(2450.0/t(i)))
!
!  NO + NO + O2  -->  NO2 + NO2
!
  k(i,75) = 3.30E-39*exp(-(-530.0/t(i)))
!
!  NO3 + NO  -->  NO2 + NO2
!
  k(i,76) = 1.50E-11*exp(-(-170.0/t(i)))
!
!  NO3 + NO2  -->  NO + NO2 + O2
!
  k(i,77) = 4.50E-14*exp(-(1260.0/t(i)))
!
!  NO3 + NO2  -->  N2O5
!
  xk0   = 2.00E-30
  xE0   = 0.0
  xn0   = 4.4
  xkinf = 1.40E-12
  xEinf = 0.0
  xminf = 0.7
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,78) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  N2O5  -->  NO2 + NO3
!
  xk0   = 2.00E-30
  xn0   = 4.4
  xkinf = 1.40E-12
  xminf = 0.7
  xaeq  = 3.00E-27
  xbeq  = 10990.0
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
  zkinf = xkinf*((t(i)/300.)**(-xminf))
  zfact = 0.6**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,79) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact
!
!  NO3 + NO3  -->  NO2 + NO2 + O2
!
  k(i,80) = 8.50E-13*exp(-(2450.0/t(i)))
!
!  OH + H2  -->  H2O + H
!
  k(i,81) = 5.50E-12*exp(-(2000.0/t(i)))
!
!  OH + SO2  -->  SULF + HO2
!
  xk0   = 3.00E-31
  xE0   = 0.0
  xn0   = 3.3
  xkinf = 1.50E-12
  xEinf = 0.0
  xminf = 0.0
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,82) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  CO + OH  -->  H + CO2
!
  k(i,83) = 1.5E-13*(1.+0.6*pm(i)/101325.)
!
!  ISOTOT + O3P  -->  0.89449*ALKENEbis + 0.05*HCHO + 0.02*OH +
!                0.01*CO + 0.13085*MACR + 0.28*HO2 + 0.15*XO2
!
  k(i,84) = 6.00E-11
!
! CH4 + OH  -->  CH3 + H2O
!
  k(i,85) = 2.80E-14*(t(i)**0.667)*exp(-1575./t(i))
!
!  ETH + OH  -->  HC8P + H2O
!
  k(i,86) = t(i)*t(i)*1.51E-17*exp(-(492./t(i)))
!
! ALKANEbis + OH --> 0.68285*HC8P + 0.31237*HO2 + 0.21989*ALD +
!                0.05953*KET + 0.02466*MACR + 0.00238*HCHO + 0.02327*ORA1 +
!                0.02327*CO + 0.02327*OH + H2O
!
  k(i,87) = 3.90E-12*exp(-260/t(i))+1.58E-12*exp(-155/t(i)) &
      +1.01E-12*exp(-125/t(i))
!
!  ALKENEbis + OH  --> 1.01963*OLIP
!
  k(i,88) = 1.79E-12*exp(-(-438./t(i)))+5.75E-13*exp(-(-500/t(i)))
!
!  ISOTOT + OH  --> ISOP
!
  k(i,89) = 2.54E-11*exp(-(-410.0/t(i)))
!
!  AROMATIC + OH --> 0.93480*ADDT + 0.10260*XO2 + 0.10260*HO2 + 0.00285*PHO
!
  k(i,90) = 3.22E-12*exp(-(-355.0/t(i)))+3.16E-13
!
!  HCHO + OH  -->  HO2 + CO + H2O
!
  k(i,91) = 1.00E-11
!
!  ALD + OH  -->  ACO3 + H2O
!
  k(i,92) = 5.55E-12*exp(-(-331.0/t(i)))
!
!  KET + OH  -->  ACO3 + H2O
!
  k(i,93) = t(i)*t(i)*5.68E-18*exp(-(-92.0/t(i)))
!
! MACR + OH --> 0.53660*ACO3 + 0.14812*MACR + 1.03417*CO +
!                0.47963*HO2 + 0.05649*ALD + 0.0077*KET + 0.09179*XO2 +
!                0.76844*H2O
!
  k(i,94) = 1.34E-11+1.7E-12*exp(-(-175./t(i)))
!
!  OP1 + OH  --> MO2 + H2O
!
  k(i,95) = 0.65*2.93e-12*exp(190./t(i))
!
!  OP1 + OH  --> HCHO + OH + H2O
!
  k(i,96) = 0.35*2.93e-12*exp(190./t(i))
!
!  OP2 + OH  -->  0.39383*HC8P + 0.06833*ACO3 + 0.07161*ALD +
!                0.36698*KET + 0.09945*XO2 + 0.03679*HO2 + 0.03679*HCHO +
!                0.43859*OH
!
  k(i,97) = 3.35E-12*exp(-(-190.0/t(i)))
!
!  PAN + OH  -->  0.53828*HCHO + 0.27962*MACR + 0.69128*NO3 +
!                0.30781*PAN + 0.30781*HO2 + 0.23046*H2O + XO2
!
  k(i,98) = 3.71E-14+2.38E-14*exp(-(-500./t(i)))
!
!  ONIT + OH  -->  HC8P + NO2 + H2O
!
  k(i,99) = 5.31E-12*exp(-(260.0/t(i)))
!
!  HCHO + NO3  -->  HO2 + HNO3 + CO
!
  k(i,100) = 3.40E-13*exp(-(1900.0/t(i)))
!
enddo
!$omp end do
!
!$omp do
do i=1,imax
!
!  ALD + NO3  -->  ACO3 + HNO3
!
  k(i,101) = 1.40E-12*exp(-(1900.0/t(i)))
!
! MACR + NO3 --> 0.90869*HNO3 + 0.42992*ACO3 + 0.10099*MACR +
!                0.05049*ALD + 0.00606*KET + 0.10099*NO2 + 0.10099*XO2 +
!                0.57975*HO2 + 1.28646*CO
!
  k(i,102) = 1.60E-12*exp(-1900/t(i))+2.09E-14*exp(-1000./t(i))
!
!  AROMATIC + NO3  -->  HNO3 + PHO
!
  k(i,103) = 3.65E-16
!
! ALKENEbis + NO3 --> 0.96421*OLNN
!
  k(i,104) = 4.52E-18*t(i)*t(i)*exp(-2282/t(i))                         &
      +1.29E-14*exp(-450./t(i))+7.35E-16*exp(-(-450./t(i)))
!
! ISOTOT + NO3 --> 0.90585*MACR + OLNN
!
  k(i,105) = 4.0E-12*exp(-446./t(i))
!
! PAN + NO3 --> 0.6*ONIT + 0.6*NO3 + 0.4*PAN + 0.4*HCHO + 0.4*NO2 + XO2
!
  k(i,106) = 5.1E-16*exp(-500/t(i))
!
! ALKENEbis + O3 --> 0.54280*HCHO + 0.47021*ALD + 0.06764*KET +
!                0.36475*CO + 0.17831*ORA1 + 0.07417*ORA2 + 0.24074*HO2 +
!                0.37222*OH + 0.05225*ACO3 + 0.02918*ETH + 0.03912*CH4 +
!                0.12708*MO2 + 0.08956*HC8P + 0.01643*H2O2 + 0.06282*H2
!
  k(i,107) = 8.35E-15*exp(-2580./t(i))+3.46E-16*exp(-1800./t(i))       &
      +2.80E-17*exp(-845/t(i))
!
! ISOTOT + O3 --> 0.9*HCHO + 0.39253*MACR + 0.36*CO +
!                0.36404*ALKENEbis + 0.17*ACO3 + 0.03*MO2 + 0.15*ORA1 + 0.28*OH
!                0.3*HO2 + 0.001*H2O2 + 0.05*H2 + 0.13*XO2 + 0.09*O3P
!
  k(i,108) = 7.86E-15*exp(-1913./t(i))
!
! MACR + O3 --> 1.1017*MACR + 0.15896*ALD + 0.10929*ORA1 +
!                0.20864*ORA2 + 0.27818*ACO3 + 0.10423*OP2 + 0.65571*CO +
!                0.28812*HO2 + 0.20864*OH
!
  k(i,109) = 1.36E-19
!
! PAN + O3 --> 0.7*HCHO + 0.3*PAN + 0.7*NO2 + 0.13*CO + 0.04*H2 +
!                0.11*ORA1 + 0.08*HO2 + 0.036*OH + 0.7*ACO3
!
  k(i,110) = 1.31E-16*exp(-1700/t(i))
!
!  PHO + NO2  -->  0.10391*AROMATIC + ONIT
!
  k(i,111) = 2.00E-11
!
!  PHO + HO2  -->  1.03908*AROMATIC
!
  k(i,112) = 1.00E-11
!
!  ADDT + NO2  -->  AROMATIC + HONO
!
  k(i,113) = 3.60E-11
!
!  ADDT + O2  --> TOLP
!
  k(i,114) = 0.98*1.66E-17*exp(-(-1044.0/t(i)))
!
!  ADDT + O2  --> AROMATIC + HO2
!
  k(i,115) = 0.02*1.66E-17*exp(-(-1044.0/t(i)))
!
!  ADDT + O3  -->  AROMATIC + OH
!
  k(i,116) = 2.77E-11
!
!  ACO3 + NO2  -->  PAN
!
  xk0   = 9.70E-29
  xE0   = 0.0
  xn0   = 5.6
  xkinf = 9.30E-12
  xEinf = 0.0
  xminf = 1.5
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,117) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  PAN  -->  ACO3 + NO2
!
  xk0   = 9.70E-29
  xn0   = 5.6
  xkinf = 9.30E-12
  xminf = 1.5
  xaeq  = 8.60E-29
  xbeq  = 13954.0
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
  zkinf = xkinf*((t(i)/300.)**(-xminf))
  zfact = 0.6**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,118) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact
!
!  MO2 + NO  -->  CH3O + NO2
!
  k(i,119) = 4.2e-12*exp(180./t(i))
!
! HC8P + NO --> 0.33628*ALD + 0.03065*HCHO + 0.54089*KET +
!                0.03609*MACR + 0.74716*HO2 + 0.09255*MO2 + 0.07839*HC8P +
!                0.12222*XO2 + 0.08116*ONIT + 0.91884*NO2
!
  k(i,120) = 4.37E-12
!
! OLIP + NO --> 1.40361*HCHO + 0.41431*ALD + 0.04833*KET + HO2 + NO2
!
  k(i,121) = 7.01E-12
!
! ISOP + NO --> 0.4489*MACR + 0.606*HCHO + 0.3682*ALKENEbis +
!                0.847*HO2 + 0.847*NO2 + 0.153*ONIT
!
  k(i,122) = 4.00E-12
!
! TOLP + NO --> 0.95122*NO2 + 0.95122*HO2 + 2.08075*MACR + 0.04878*ONIT
!
  k(i,123) = 4.0E-12
!
! ACO3 + NO --> 0.77298*MO2 + 0.10527*ACO3 + 0.06891*HCHO +
!                0.07273*ALD + 0.0855*MACR + 0.12175*HO2 + 0.02530*XO2 + NO2
!
  k(i,124) = 1.23E-11
!
! OLNN + NO --> 0.16657*ONIT + 1.83342*NO2 + 0.16657*HO2 +
!                0.23919*HCHO + 1.03345*ALD + 0.38671*KET
!
  k(i,125) = 4.0E-12
!
!  MO2 + HO2  -->  OP1
!
  k(i,126) = 3.80E-13*exp(-(-800.0/t(i)))
!
! HC8P + HO2 --> 1.00522*OP2
!
  k(i,127) = 6.24E-14*exp(700/t(i))+1.52E-13*exp(1300/t(i))
!
! OLIP + HO2 --> 1.00522*OP2
!
  k(i,128) = 1.81E-13*exp(1300/t(i))
!
! ISOP + HO2 --> 1.00522*OP2
!
  k(i,129) = 1.28E-13*exp(1300/t(i))
!
! TOLP + HO2 --> 1.00522*OP2
!
  k(i,130) = 3.75E-13*exp(980/t(i))
!
! ACO3 + HO2 --> 0.85175*OP2 + 0.13345*ORA2 + 0.13345*O3
!
  k(i,131) = 5.83E-13*exp(550/t(i))+1.96E-16*exp(2640./t(i))        &
      +5.67E-14*exp(1300/t(i))
!
!  OLNN + HO2  -->  ONIT
!
  k(i,132) = 1.66E-13*exp(-(-1300.0/t(i)))
!
!  MO2 + MO2  -->  0.66*HCHO + 0.66*CH3O
!
  k(i,133) = 9.1E-14*exp(416./t(i))
!
! HC8P + MO2 --> 0.80572*HCHO + 0.98436*HO2 + 0.56293*ALD +
!                0.09095*KET + 0.01346*MO2 + 0.08396*MACR + 0.12985*XO2 +
!                0.00359*HC8P
!
  k(i,134) = 1.01E-14*exp(158./t(i))+6.39E-14*exp(431./t(i))        &
      +1.44E-14*exp(467/t(i))+4.1E-15*exp(633./t(i))
!
! OLIP + MO2 --> 1.42733*HCHO + 0.46518*ALD + 0.03882*KET + HO2
!
  k(i,135) = 1.56E-13*exp(708/t(i))
!
! ISOP + MO2 --> 0.55357*MACR + 0.46806*ALKENEbis + HO2 + 1.09*HCHO
!
  k(i,136) = 1.36E-13*exp(708/t(i))
!
! TOLP + MO2 --> HCHO + 1.02861*HO2 + 1.99456*MACR
!
  k(i,137) = 3.56E-14*exp(708/t(i))
!
! ACO3 + MO2 --> 0.96426*HCHO + 0.87734*HO2 + 0.58551*MO2 +
!                0.08788*ORA2 + 0.07152*ACO3 + 0.16878*MACR + 0.08696*ALD +
!                0.02319*XO2
!
  k(i,138) = 1.65E-11*exp(-440./t(i))+1.38E-16*exp(2510/t(i))             &
      +3.36E-13*exp(508/t(i))
!
!  OLNN + MO2  -->  0.88103*HCHO + 0.68801*HO2 + 0.68801*ONIT +
!                0.39935*ALD + 0.09297*KET + 0.31199*NO2
!
  k(i,139) = 1.14E-13*exp(-(-708.0/t(i)))
!
! HC8P + ACO3 --> 0.71704*ALD + 0.48134*HO2 + 0.51431*MO2 +
!                0.49817*ORA2 + 0.18281*KET + 0.07663*HCHO + 0.00809*HC8P +
!                0.10953*XO2 + 0.0729*MACR
!
  k(i,140) = 4.18E-14*exp(211/t(i))+2.19E-13*exp(460./t(i))               &
      +3.71E-14*exp(522/t(i))+1.07E-14*exp(683/t(i))
!
! OLIP + ACO3 --> 0.6812*HCHO + 0.68371*ALD + 0.5008*HO2 +
!                0.5008*MO2 + 0.4992*ORA2 + 0.06648*KET
!
  k(i,141) = 4.16E-13*exp(765/t(i))
!
! ISOP + ACO3 --> 0.77601*MACR + 0.23819*ALKENEbis + 0.506*HO2 +
!                0.494*ORA2 + 0.34*HCHO + 0.506*MO2
!
  k(i,142) = 7.6E-13*exp(765/t(i))
!
! TOLP + ACO3 --> MO2 + HO2 + 1.99453*MACR
!
  k(i,143) = 3.47E-13*exp(765/t(i))
!
! ACO3 + ACO3 --> 1.61972*MO2 + 0.07139*ACO3 + 0.04453*HCHO +
!                0.12104*MACR + 0.07834*ALD + 0.02462*KET + 0.08506*HO2 +
!                0.01791*XO2 + 0.11192*ORA2
!
  k(i,144) = 6.72E-13*exp(530/t(i))+1.73E-13*exp(565/t(i))
!
! OLNN + ACO3 --> 0.6787*ONIT + 0.50996*MO2 + 0.49004*ORA2 +
!                0.18866*HO2 + 0.12889*HCHO + 0.40474*ALD + 0.10399*KET
!
  k(i,145) = 4.51E-13*exp(765/t(i))
!
! OLNN + OLNN --> ONIT + ONIT + HO2
!
  k(i,146) = 5.15E-15*exp(1000/t(i))
!
! OLNN + OLNN --> O2
!
  k(i,147) = 2.42E-14*exp(1000./t(i))
!
!  MO2 + NO3  -->  HCHO + HO2 + NO2
!
  k(i,148) = 1.20E-12
!
! HC8P + NO3 --> 0.32478*ALD + 0.80993*HO2 + 0.03209*HCHO +
!                0.6412*KET + 0.03717*MACR + 0.09954*MO2 + 0.09067*HC8P +
!                0.16478*XO2 + NO2
!
  k(i,149) = 1.2E-12
!
! OLIP + NO3 --> 1.45792*HCHO + 0.37131*ALD + 0.01531*KET + HO2 + NO2
!
  k(i,150) = 1.2E-12
!
enddo
!$omp end do
!
!$omp do
do i=1,imax
!
! ISOP + NO3 --> 0.60390*MACR + 0.41604*ALKENEbis + 0.686*HCHO + HO2 + NO2
!
  k(i,151) = 1.2E-12
!
! TOLP + NO3 --> 2.83695*MACR + HO2 + NO2
!
  k(i,152) = 1.2E-12
!
! ACO3 + NO3 --> 0.89016*MO2 + 0.04229*ACO3 + 0.04229*HCHO +
!                0.04744*MACR + 0.04036*ALD + 0.06756*HO2 + 0.01404*XO2 + NO2
!
  k(i,153) = 3.32E-12
!
! OLNN + NO3 --> 0.28775*ONIT + 1.71225*NO2 + 0.28775*HO2 +
!                0.19943*HCHO + 0.88319*ALD + 0.33405*KET
!
  k(i,154) = 1.2E-12
!
!  XO2 + HO2  -->  1.00522*OP2
!
  k(i,155) = 1.66E-13*exp(-(-1300.0/t(i)))
!
!  XO2 + MO2  -->  HCHO + HO2
!
  k(i,156) = 5.99E-15*exp(-(-1510.0/t(i)))
!
!  XO2 + ACO3  -->  MO2
!
  k(i,157) = 3.40E-14*exp(-(-1560.0/t(i)))
!
!  XO2 + XO2  -->  O2
!
  k(i,158) = 7.13E-17*exp(-(-2950.0/t(i)))
!
!  XO2 + NO  -->  NO2
!
  k(i,159) = 4.00E-12
!
!  XO2 + NO3  -->  NO2
!
  k(i,160) = 1.20E-12
!
!  H + O2  -->  HO2
!
  xk0   = 5.70E-32
  xE0   = 0.0
  xn0   = 1.6
  xkinf = 7.50E-11
  xEinf = 0.0
  xminf = 0.0
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,161) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  O3 + H  -->  OH + O2
!
  k(i,162) = 1.4e-10*exp(-470./t(i))
!
!  O1D + H2  -->  OH + H
!
  k(i,163) = 1.1e-10
!
!  O3P + OH  -->  O2 + H
!
  k(i,164) = 2.2e-11*exp(120./t(i))
!
!  O3P + HO2  -->  OH + O2
!
  k(i,165) = 3.0e-11*exp(200./t(i))
!
!  H + HO2  -->  OH + OH
!
  k(i,166) = 0.9*8.1e-11
!
!  H + HO2  -->  H2 + O2
!
  k(i,167) = 0.08*8.1e-11
!
!  H + HO2  -->  H2O + O3P
!
  k(i,168) = 0.02*8.1e-11
!
!  H2 + O3P  -->  OH + H
!
  k(i,169) = 8.8e-12*exp(-4200./t(i))
!
!  OH + OH  -->  H2O + O3P
!
  k(i,170) = 4.2e-12*exp(-240./t(i))
!
!   OH + OH   -->   H2O2
!
  xk0   = 6.90E-31
  xE0   = 0.
  xn0   = 1.0
  xkinf = 2.60E-11
  xEinf = 0.
  xminf = 0.0
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,171) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  N + NO  -->  N2 + O3P
!
  k(i,172) = 2.1e-11*exp(100./t(i))
!
!  N + O2  -->  NO + O3P
!
  k(i,173) = 1.5e-11*exp(-3600./t(i))
!
!  O1D + N2O  -->  N2 + O2
!
  k(i,174) = 4.9e-11
!
!  O1D + N2O  -->  NO + NO
!
  k(i,175) = 6.7e-11
!
!  O1D + CH4  -->  OH + CH3
!
  k(i,176) = 0.75*1.5e-10
!
!  O1D + CH4  -->  CH3O + H
!
  k(i,177) = 0.20*1.5e-10
!
!  O1D + CH4  -->  HCHO + H2
!
  k(i,178) = 0.05*1.5e-10
!
!  CH3 + O3P  -->  HCHO + H
!
  k(i,179) = 1.1e-10
!
!  CH3 + O2  -->  MO2
!
  xk0   = 4.50E-31
  xE0   = 0.0
  xn0   = 3.0
  xkinf = 1.80E-12
  xEinf = 0.0
  xminf = 1.7
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,180) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  HCHO + O3P  -->  HO2 + CO + OH
!
  k(i,181) = 3.4e-11*exp(-1600./t(i))
!
!  HCHO + Cl  -->  HO2 + CO + HCl
!
  k(i,182) = 8.1e-11*exp(-30./t(i))
!
!  CH3O + O2  -->  HCHO + HO2
!
  k(i,183) = 3.9e-14*exp(-900./t(i))
!
!  MO2 + ClO  -->  CH3O + Cl + O2
!
  k(i,184) = 3.3e-12*exp(-115./t(i))
!
!  HCHO + Br  -->  HO2 + CO + HBr
!
  k(i,185) = 1.7e-11*exp(-800./t(i))
!
!  Cl + O3  -->  ClO + O2
!
  k(i,186) = 2.3e-11*exp(-200./t(i))
!
!  ClO + O3P  -->  Cl + O2
!
  k(i,187) = 3.0e-11*exp(70./t(i))
!
!  ClO + NO  -->  NO2 + Cl
!
  k(i,188) = 6.4e-12*exp(290./t(i))
!
!  Cl + CH4  -->  HCl + CH3
!
  k(i,189) = 9.6e-12*exp(-1360./t(i))
!
!  Cl + H2  -->  HCl + H
!
  k(i,190) = 3.7e-11*exp(-2300./t(i))
!
!  Cl + HO2  -->  HCl + O2
!
  k(i,191) = 1.8e-11*exp(170./t(i))
!
!  Cl + HO2  -->  OH + ClO
!
  k(i,192) = 4.1e-11*exp(-450./t(i))
!
!  ClO + OH  -->  Cl + HO2
!
  k(i,193) = 7.4e-12*exp(270./t(i))
!
!  ClO + OH  -->  HCl + O2
!
  k(i,194) = 6.0e-13*exp(230./t(i))
!
!  OH + HCl  -->  H2O + Cl
!
  k(i,195) = 2.6e-12*exp(-350./t(i))
!
!  ClO + NO2  -->  ClONO2
!
  xk0   = 1.80E-31
  xE0   = 0.0
  xn0   = 3.4
  xkinf = 1.50E-11
  xEinf = 0.0
  xminf = 1.9
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,196) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  O3P + ClONO2  -->  ClOx + NOx + XX
!
  k(i,197) = 2.9e-12*exp(-800./t(i))
!
!  ClO + HO2  -->  HOCl + O2
!
  k(i,198) = 2.7e-12*exp(220./t(i))
!
!  OH + HOCl  -->  H2O + ClO
!
  k(i,199) = 3.0e-12*exp(-500./t(i))
!
!  O3P + HOCl  -->  OH + ClO
!
  k(i,200) = 1.7e-13
!
enddo
!$omp end do
!
!$omp do
do i=1,imax
!
!  Cl + NO2  -->  ClNO2
!
  xk0   = 1.80E-31
  xE0   = 0.0
  xn0   = 2.0
  xkinf = 1.00E-10
  xEinf = 0.0
  xminf = 1.0
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,201) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  Cl + HOCl  -->  OH + Cl2
!
  k(i,202) = 2.5e-12*exp(-130./t(i))
!
!  ClO + ClO  -->  Cl2O2
!
  xk0   = 1.60E-32
  xE0   = 0.0
  xn0   = 4.5
  xkinf = 2.00E-12
  xEinf = 0.0
  xminf = 2.4
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,203) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  Cl2O2  -->  ClO + ClO
!
  xk0   = 1.60E-32
  xn0   = 4.5
  xkinf = 2.00E-12
  xminf = 2.4
  xaeq  = 1.27E-27
  xbeq  = 8744.0
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
  zkinf = xkinf*((t(i)/300.)**(-xminf))
  zfact = 0.6**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,204) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact
!
!  ClO + ClO  -->  Cl2 + O2
!
  k(i,205) = 1.0e-12*exp(-1590./t(i))
!
!  ClO + ClO  -->  Cl2 + O2
!
  k(i,206) = 3.0e-11*exp(-2450./t(i))
!
!  O1D + CFC11  -->  ClOx + ClOx + ClOx + XX
!
  k(i,207) = 2.3e-10
!
!  O1D + CFC12  -->  ClOx + ClOx + XX
!
  k(i,208) = 1.4e-10
!
!  O1D + CFC113  -->  ClOx + ClOx + ClOx + XX
!
  k(i,209) = 2.0e-10
!
!  O1D + CCl4  -->  ClOx + ClOx + ClOx + ClOx + XX
!
  k(i,210) = 3.3e-10
!
!  OH + CH3CCl3  -->  ClOx + ClOx + ClOx + H2O + XX
!
  k(i,211) = 1.6e-12*exp(-1520./t(i))
!
!  OH + CH3Cl  -->  ClOx + H2O + XX
!
  k(i,212) = 2.4e-12*exp(-1250./t(i))
!
!  O1D + HCFC22  -->  ClOx + XX
!
  k(i,213) = 1.0e-10
!
!  OH + HCFC22  -->  ClOx + H2O + XX
!
  k(i,214) = 1.05e-12*exp(-1600./t(i))
!
!  Br + O3  -->  BrO + O2
!
  k(i,215) = 1.7e-11*exp(-800./t(i))
!
!  BrO + O3P  -->  Br + O2
!
  k(i,216) = 1.9e-11*exp(230./t(i))
!
!  BrO + NO  -->  NO2 + Br
!
  k(i,217) = 8.8e-12*exp(260./t(i))
!
!  BrO + ClO  -->  OClO + Br
!
  k(i,218) = 9.5e-13*exp(550./t(i))
!
!  BrO + ClO  -->  Br + Cl + O2
!
  k(i,219) = 2.3e-12*exp(260./t(i))
!
!  BrO + ClO  -->  BrCl + O2
!
  k(i,220) = 4.1e-13*exp(290./t(i))
!
!  BrO + BrO  -->  Br + Br + O2
!
  k(i,221) = 1.5e-12*exp(230./t(i))
!
!  Br + HO2  -->  HBr + O2
!
  k(i,222) = 1.5e-11*exp(-600./t(i))
!
!  O1D + HBr  -->  OH + Br
!
  k(i,223) = 1.5e-10
!
!  BrO + OH  -->  Br + HO2
!
  k(i,224) = 1.0e-11
!
!  OH + HBr  -->  H2O + Br
!
  k(i,225) = 1.1e-11
!
!  BrO + NO2  -->  BrONO2
!
  xk0   = 5.20E-31
  xE0   = 0.0
  xn0   = 3.2
  xkinf = 6.90E-12
  xEinf = 0.0
  xminf = 2.9
  xF    = 0.6
  zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
  zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
  zfact = xF**(1./(1.+(alog10(zk0m/zkinf))**2.))
  k(i,226) = (zk0m/(1.+zk0m/zkinf))*zfact
!
!  BrO + HO2  -->  HOBr + O2
!
  k(i,227) = 3.4e-12*exp(540./t(i))
!
!  HOBr + O3P  -->  OH + BrO
!
  k(i,228) = 1.2e-10*exp(-430./t(i))
!
!  O1D + CH3Br  -->  BrOx + XX
!
  k(i,229) = 1.8e-10
!
!  OH + CH3Br  -->  CH2Br + H2O
!
  k(i,230) = 2.85e-12*exp(-1300./t(i))
!
!  O1D + H1211  -->  BrOx + XX
!
  k(i,231) = 1.5e-10
!
!  O1D + H1301  -->  BrOx + XX
!
  k(i,232) = 1.0e-10
!
!  O3P + O3P  -->  O2
!
  k(i,233) = 4.23e-28*hnm(i)/(t(i)*t(i))
!
!  ClONO2 + H2O  -->  HOCl + HNO3
!
  k(i,234) = het(i,1)
!
!  ClONO2 + HCl  -->  Cl2 + HNO3
!
  k(i,235) = het(i,2)
!
!  N2O5 + H2O  -->  HNO3 + HNO3
!
  k(i,236) = het(i,3)
!
!  N2O5 + HCl  -->  ClNO2 + HNO3
!
  k(i,237) = het(i,4)
!
!  HOCl + HCl  -->  Cl2 + H2O
!
  k(i,238) = het(i,5)
!
!  BrONO2 + H2O  -->  HOBr + HNO3
!
  k(i,239) = het(i,6)
!
!  HOBr + HCl  -->  BrCl + H2O
!
  k(i,240) = het(i,7)
!
!  HOBr + HBr  -->  BrOx + BrOx + H2O
!
  k(i,241) = het(i,8)
!
!  HOCl + HBr  -->  BrCl + H2O
!
  k(i,242) = het(i,9)
!
enddo
!$omp end do
!$omp end parallel

RETURN
END SUBROUTINE KRATES
