SUBROUTINE CH_CONSERV (QQQM,HHHM,lon,lat,niv,nivbas,ntra,nsls)
!
!!!!!
!
!   CONSERVATION DES ATOMES DE CHLORE, DE BROME ET D'AZOTE
!   SUR UN PAS DE TEMPS CHIMIQUE
!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!
!   Centre National de Recherches Meteorologiques
!   Groupe de Modelisation de Grande Echelle et du Climat
!   Equipe de Recherches sur l'Atmosphere Moyenne
!
!!!!!
!
!  Original : H. Teyssedre, 09/2003
!
!  Modified : H. Teyssedre, version Fortran-90, 11/2006
!
!!!!!
!
#ifdef MPI
USE CSTES_MOD
#endif
!
IMPLICIT NONE
!
!  subroutine arguments
!
INTEGER , INTENT(IN) :: lon,lat,niv,nivbas,ntra,nsls
!
REAL (KIND=8), INTENT(INOUT) :: QQQM(lon,lat,niv,ntra)
REAL (KIND=8), INTENT(INOUT) :: HHHM(lon,lat,niv,nsls)
!
!  local variables
!
INTEGER :: ilon,iniv,i1,i2
REAL (KIND=8) :: totN, totCl, totBr
REAL (KIND=8) :: rapN, rapCl, rapBr
!
#ifdef MPI
i1 = i_deb(myid) ; i2 = i_fin(myid)
#else
i1 = 1           ; i2 = lon*lat
#endif
!
!!!!!
!
!  TRANSPORTED LONG-LIVED SPECIES AND FAMILIES (ARRAY QM)
!
!!!!!
!
!       1 : N2O         2 : CH4         3 : H2O         4 : HNO3        5 : N2O5
!       6 : CO          7 : OClO        8 : HCl         9 : ClONO2     10 : HOCl
!      11 : Cl2        12 : H2O2       13 : ClNO2      14 : HBr        15 : BrON
!      16 : HNO4       17 : Cl2O2      18 : HOBr       19 : BrCl       20 : HCHO
!      21 : MO2        22 : OP1        23 : CFC11      24 : CFC12      25 : CFC1
!      26 : CCl4       27 : CH3CCl3    28 : CH3Cl      29 : HCFC22     30 : CH3B
!      31 : H1211      32 : H1301      33 : H2SO4      34 : PSC        35 : Ox
!      36 : NOx        37 : ClOx       38 : BrOx       39 : NOy        40 : Cly
!      41 : Bry
!
!!!!!
!
!  SHORT-LIVED SPECIES AT INSTANTANEOUS EQUILIBRIUM (ARRAY HM)
!
!!!!!
!
!       1 : O3P         2 : O1D         3 : O3          4 : N           5 : NO
!       6 : NO2         7 : NO3         8 : ClO         9 : Cl         10 : BrO
!      11 : Br         12 : H          13 : OH         14 : HO2        15 : CH3
!      16 : CH3O
!
!$omp parallel do default(none) &
!$omp shared(QQQM,HHHM,i1,i2,niv,nivbas,ntra,nsls) &
!$omp private(ilon,iniv,TOTCL,RAPCL,TOTBR,RAPBR,TOTN,RAPN)
do iniv=1,nivbas
  do ilon=i1,i2
!
!================================================================
!  CHECK CONSERVATION FOR Cl
!================================================================
!
    totCl = + QQQM(ilon,1,iniv,7) + QQQM(ilon,1,iniv,8) +          &
              QQQM(ilon,1,iniv,9) + QQQM(ilon,1,iniv,10) +         &
          2.0*QQQM(ilon,1,iniv,11) + QQQM(ilon,1,iniv,13) +        &
          2.0*QQQM(ilon,1,iniv,17) + QQQM(ilon,1,iniv,37)
!
! adjust to keep mass constant :
!
    rapCl = MAX( 0. , QQQM(ilon,1,iniv,40) -                       &
                      QQQM(ilon,1,iniv,19) ) / totCl
!
    QQQM(ilon,1,iniv,7) = QQQM(ilon,1,iniv,7) * rapCl
    QQQM(ilon,1,iniv,8) = QQQM(ilon,1,iniv,8) * rapCl
    QQQM(ilon,1,iniv,9) = QQQM(ilon,1,iniv,9) * rapCl
    QQQM(ilon,1,iniv,10) = QQQM(ilon,1,iniv,10) * rapCl
    QQQM(ilon,1,iniv,11) = QQQM(ilon,1,iniv,11) * rapCl
    QQQM(ilon,1,iniv,13) = QQQM(ilon,1,iniv,13) * rapCl
    QQQM(ilon,1,iniv,17) = QQQM(ilon,1,iniv,17) * rapCl
    QQQM(ilon,1,iniv,37) = QQQM(ilon,1,iniv,37) * rapCl
    HHHM(ilon,1,iniv,8) = HHHM(ilon,1,iniv,8) * rapCl
    HHHM(ilon,1,iniv,9) = HHHM(ilon,1,iniv,9) * rapCl
!
!================================================================
!  CHECK CONSERVATION FOR N
!================================================================
!
    totN = + QQQM(ilon,1,iniv,4) + 2.0*QQQM(ilon,1,iniv,5) +       &
             QQQM(ilon,1,iniv,16) + QQQM(ilon,1,iniv,36)
!
! adjust to keep mass constant :
!
    rapN = MAX( 0.0 , QQQM(ilon,1,iniv,39) -                       &
                      QQQM(ilon,1,iniv,9) - QQQM(ilon,1,iniv,13) - &
                      QQQM(ilon,1,iniv,15) ) / totN
!
    QQQM(ilon,1,iniv,4) = QQQM(ilon,1,iniv,4) * rapN
    QQQM(ilon,1,iniv,5) = QQQM(ilon,1,iniv,5) * rapN
    QQQM(ilon,1,iniv,16) = QQQM(ilon,1,iniv,16) * rapN
    QQQM(ilon,1,iniv,36) = QQQM(ilon,1,iniv,36) * rapN
    HHHM(ilon,1,iniv,4) = HHHM(ilon,1,iniv,4) * rapN
    HHHM(ilon,1,iniv,5) = HHHM(ilon,1,iniv,5) * rapN
    HHHM(ilon,1,iniv,6) = HHHM(ilon,1,iniv,6) * rapN
    HHHM(ilon,1,iniv,7) = HHHM(ilon,1,iniv,7) * rapN
!
!================================================================
!  CHECK CONSERVATION FOR Br
!================================================================
!
    totBr = + QQQM(ilon,1,iniv,14) + QQQM(ilon,1,iniv,15) +        &
              QQQM(ilon,1,iniv,18) + QQQM(ilon,1,iniv,19) +        &
              QQQM(ilon,1,iniv,38)
!
! adjust to keep mass constant :
!
    rapBr = MAX( 0.0 , QQQM(ilon,1,iniv,41) ) /totBr
!
    QQQM(ilon,1,iniv,14) = QQQM(ilon,1,iniv,14) * rapBr
    QQQM(ilon,1,iniv,15) = QQQM(ilon,1,iniv,15) * rapBr
    QQQM(ilon,1,iniv,18) = QQQM(ilon,1,iniv,18) * rapBr
    QQQM(ilon,1,iniv,19) = QQQM(ilon,1,iniv,19) * rapBr
    QQQM(ilon,1,iniv,38) = QQQM(ilon,1,iniv,38) * rapBr
    HHHM(ilon,1,iniv,10) = HHHM(ilon,1,iniv,10) * rapBr
    HHHM(ilon,1,iniv,11) = HHHM(ilon,1,iniv,11) * rapBr
!
  enddo
enddo
!$omp end parallel do
!
RETURN
END SUBROUTINE CH_CONSERV
