SUBROUTINE CH_EQUIL(sza,k,hnm,QM,HM,partHNO3,partH2O,partHCl,partHBr,   &
                    imax,nk,ntra,nsls,iniv,NIVHOx,SZAlim,LSOL)
!
!!!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_0 
!
!!! DATE : 15/05/2005
!
!
!   CALCUL DE L'EQUILIBRE PHOTOCHIMIQUE POUR LES ESPECES A
!   COURTE DUREE DE VIE
!
!!!!!
!
!  Original : H. Teyssedre, V.-H. Peuch, 08/2000
!
!  Modified : H. Teyssedre, version Fortran-90, 11/2006
!
!!!!!
!
#ifdef MPI
      USE CSTES_MOD
#endif
!
      IMPLICIT NONE
!	
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
      INTEGER , INTENT(IN) :: imax, nk, ntra, nsls, iniv, NIVHOx
      REAL (KIND=8) , INTENT(IN) :: SZAlim
      REAL (KIND=8) , DIMENSION (imax) , INTENT(IN) :: sza,hnm
      REAL (KIND=8) , DIMENSION (imax) , INTENT(IN) :: partHNO3,partH2O
      REAL (KIND=8) , DIMENSION (imax) , INTENT(IN) :: partHCl,partHBr
      REAL (KIND=8) , DIMENSION (imax,nk) , INTENT(IN) :: k
      REAL (KIND=8) , DIMENSION (imax,ntra) , INTENT(INOUT) :: QM
      REAL (KIND=8) , DIMENSION (imax,nsls) , INTENT(INOUT) :: HM
      LOGICAL , INTENT(IN) :: LSOL
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
      INTEGER :: I, i1, i2
!
      REAL (KIND=8) :: roxo3p,roxo1d,rox,aoxo3,aoxo3p,aoxo1d
      REAL (KIND=8) :: rnoxn,rnoxno,rnoxno3,rnox,anoxno2
      REAL (KIND=8) :: anoxn,anoxno,anoxno3,rcloxcl,rcloxclono2
      REAL (KIND=8) :: rcloxhocl,rcloxcl2o2,rcloxbrcl,rclox
      REAL (KIND=8) :: acloxclo,acloxcl,rbroxbr,rbroxbrono2
      REAL (KIND=8) :: rbroxhobr,rbroxbrcl,rbrox,abroxbro,abroxbr
      REAL (KIND=8) :: xproh,xpeoh,xprho2,xpeho2,argum
      REAL (KIND=8) :: rhoxh,rhoxho2,ahoxh,ahoxho2,ahoxoh
      REAL (KIND=8) :: xphox,xl1hox,xl2hox
      REAL (KIND=8) , DIMENSION (imax) :: gHNO3,gH2O
      REAL (KIND=8) , DIMENSION (imax) :: gHCl,gHBr
!
!!!!!
!
!  TRANSPORTED LONG-LIVED SPECIES AND FAMILIES (ARRAY QM)
!
!!!!!
!
!       1 : N2O         2 : CH4         3 : H2O         4 : HNO3        5 : N2O5
!       6 : CO          7 : OClO        8 : HCl         9 : ClONO2     10 : HOCl
!      11 : Cl2        12 : H2O2       13 : ClNO2      14 : HBr        15 : BrONO2
!      16 : HNO4       17 : Cl2O2      18 : HOBr       19 : BrCl       20 : HCHO
!      21 : MO2        22 : OP1        23 : CFC11      24 : CFC12      25 : CFC113
!      26 : CCl4       27 : CH3CCl3    28 : CH3Cl      29 : HCFC22     30 : CH3Br
!      31 : H1211      32 : H1301      33 : H2SO4      34 : PSC        35 : Ox
!      36 : NOx        37 : ClOx       38 : BrOx
!
!!!!!
!
!  SHORT-LIVED SPECIES AT INSTANTANEOUS EQUILIBRIUM (ARRAY HM)
!
!!!!!
!
!       1 : O3P         2 : O1D         3 : O3          4 : N           5 : NO
!       6 : NO2         7 : NO3         8 : ClO         9 : Cl         10 : BrO
!      11 : Br         12 : H          13 : OH         14 : HO2        15 : CH3
!      16 : CH3O  
!
!!!!!
!
!  1. DAY-TIME CHEMISTRY (CHEMICAL STEADY STATE)
!
!!!!!
!
if (LSOL) then
  do i=1,imax
    gHNO3(i)=partHNO3(i)*QM(i,4)
    gH2O (i)=partH2O (i)*QM(i,3)
    gHCl (i)=partHCl (i)*QM(i,8)
    gHBr (i)=partHBr (i)*QM(i,14)
  enddo
else
  do i=1,imax
    gHNO3(i)=QM(i,4)
    gH2O (i)=QM(i,3)
    gHCl (i)=QM(i,8)
    gHBr (i)=QM(i,14)
  enddo
endif
!
#ifdef MPI
i1 = i_deb(myid) ; i2 = i_fin(myid)
#else
i1 = 1           ; i2 = imax
#endif
!
!$omp parallel default(none) &
!$omp private (i,rOxO3P,rOxO1D,rOx,aOxO3,aOxO3P,aOxO1D) &
!$omp private (rNOxN,rNOxNO,rNOxNO3,rNOx) &
!$omp private (aNOxNO2,aNOxN,aNOxNO,aNOxNO3) &
!$omp private (rClOxCl,rClOxClONO2,rClOxHOCl,rClOxCl2O2) &
!$omp private (rClOxBrCl,rClOx,aClOxClO,aClOxCl) &
!$omp private (rBrOxBr,rBrOxBrONO2,rBrOxHOBr,rBrOxBrCl,rBrOx) &
!$omp private (aBrOxBrO,aBrOxBr) &
!$omp private (rHOxH,rHOxHO2,argum) &
!$omp private (aHOxH,aHOxHO2,aHOxOH) &
!$omp private (xpHOx,xl1HOx,xl2HOx) &
!$omp private (xprOH,xpeOH,xprHO2,xpeHO2) &
!$omp shared  (i1,i2,sza,SZAlim,hnm,k,QM,HM,imax,iniv,LSOL) &
!$omp shared  (gHNO3,gH2O,gHCl,gHBr,NIVHOx)
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
    rOxO3P  = HM(i,1) / HM(i,3)
    rOxO1D  = HM(i,2) / HM(i,3)
!
!================================================================
!  FOR FAMILY Ox = O3P + O1D + O3
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k002 :  O3  -->  O1D + O2
!  k003 :  O3  -->  O3P + O2
!  k138 :  O3P + O2  -->  O3
!  k140 :  O1D + N2  -->  O3P + N2
!  k141 :  O1D + O2  -->  O3P + O2
!================================================================
!
!  O3P / O3
!
    rOxO3P = ( + rOxO1D*( + k(i,140)*0.78*hnm(i) +                    &
      k(i,141)*0.21*hnm(i) ) + k(i,3) ) / ( + k(i,138)*0.21*hnm(i) )
!
    rOxO3P = max( 0. , rOxO3P )
!
!  O1D / O3
!
    rOxO1D = ( + k(i,2) ) / ( + k(i,140)*0.78*hnm(i) + k(i,141)*0.21*hnm(i) )
!
    rOxO1D = max( 0. , rOxO1D )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rOx = 1. + rOxO3P + rOxO1D
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aOxO3 = 0.0
!
    HM(i,3) = aOxO3*HM(i,3) + (1.-aOxO3)*QM(i,35)/rOx
!
    aOxO3P = 0.0
!
    HM(i,1) = aOxO3P*HM(i,1) + (1.-aOxO3P)*HM(i,3)*rOxO3P
!
    aOxO1D = 0.0
!
    HM(i,2) = aOxO1D*HM(i,2) + (1.-aOxO1D)*HM(i,3)*rOxO1D
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
    rNOxN   = HM(i,4) / HM(i,6)
    rNOxNO  = HM(i,5) / HM(i,6)
    rNOxNO3 = HM(i,7) / HM(i,6)
!
!================================================================
!  FOR FAMILY NOx = N + NO + NO2 + NO3
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k001 :  NO2  -->  O3P + NO
!  k006 :  NO3  -->  NO + O2
!  k007 :  NO3  -->  NO2 + O3P
!  k038 :  NO  -->  N + O3P
!  k053 :  NO + HO2  -->  NO2 + OH
!  k059 :  O3P + NO2 --> NO + O2
!  k060 :  O3 + NO  -->  NO2 + O2
!  k061 :  N + NO  -->  N2 + O3P  (destruction for N)
!  k062 :  N + O2  -->  NO + O3P
!  k063 :  O3 + NO2  -->  NO3 + O2
!  k064 :  NO3 + NO2  -->  N2O5  (destruction for NO3)
!  k079 :  MO2 + NO  -->  CH3O + NO2
!  k092 :  ClO + NO  -->  NO2 + Cl
!  k121 :  BrO + NO  -->  NO2 + Br
!================================================================
!
!  N / NO2
!
    rNOxN = ( + k(i,38)*rNOxNO ) / ( + k(i,61)*HM(i,5) + k(i,62)*0.21*hnm(i) )
!
    rNOxN = max( 0. , rNOxN )
!
!  NO / NO2
!
    rNOxNO = ( + k(i,1) + k(i,6)*rNOxNO3 + k(i,59)*HM(i,1) +                 &
      k(i,62)*rNOxN*0.21*hnm(i) ) / ( + k(i,38) + k(i,53)*HM(i,14) +         &
      k(i,60)*HM(i,3) + k(i,79)*QM(i,21) + k(i,92)*HM(i,8) +                 &
      k(i,121)*HM(i,10) )
!
    rNOxNO = max( 0. , rNOxNO )
!
!  NO3 / NO2
!
    rNOxNO3 = ( + k(i,63)*HM(i,3) ) / ( + k(i,6) + k(i,7) + k(i,64)*HM(i,6) )
!
    rNOxNO3 = max( 0. , rNOxNO3 )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rNOx = 1. + rNOxN + rNOxNO + rNOxNO3
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aNOxNO2 = 0.0
!
    HM(i,6) = aNOxNO2*HM(i,6) + (1.-aNOxNO2)*QM(i,36)/rNOx
!
    aNOxN = 0.0
!
    HM(i,4) = aNOxN*HM(i,4) + (1.-aNOxN)*HM(i,6)*rNOxN
!
    aNOxNO = 0.0
!
    HM(i,5) = aNOxNO*HM(i,5) + (1.-aNOxNO)*HM(i,6)*rNOxNO
!
    aNOxNO3 = 0.0
!
    HM(i,7) = aNOxNO3*HM(i,7) + (1.-aNOxNO3)*HM(i,6)*rNOxNO3
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
    rClOxCl = HM(i,9) / HM(i,8)
!
!================================================================
!  FOR FAMILY ClOx = ClO + Cl
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k016 :  HOCl  -->  Cl + OH
!  k017 :  ClONO2  -->  Cl + NO3
!  k018 :  ClONO2  -->  ClO + NO2
!  k019 :  Cl2O2  -->  Cl + Cl + O2
!  k027 :  BrCl  -->  Br + Cl
!  k088 :  MO2 + ClO  -->  CH3O + Cl + O2
!  k090 :  Cl + O3  -->  ClO + O2
!  k091 :  ClO + O3P  -->  Cl + O2
!  k092 :  ClO + NO  -->  NO2 + Cl
!  k093 :  Cl + CH4  -->  HCl + CH3  (destruction for Cl)
!  k095 :  Cl + HO2  -->  HCl + O2  (destruction for Cl)
!  k096 :   Cl + HO2   -->   OH + ClO  (destruction for Cl)
!  k097 :  ClO + OH  -->  Cl + HO2
!  k100 :  ClO + NO2  -->  ClONO2
!  k101 :  O3P + ClONO2  -->  ClO + NO3
!  k102 :  ClO + HO2  -->  HOCl + O2
!  k103 :  OH + HOCl  -->  H2O + ClO
!  k104 :  O3P + HOCl  -->  OH + ClO
!  k106 :  Cl + HOCl  -->  OH + Cl2  (destruction for Cl)
!  k107 :  ClO + ClO  -->  Cl2O2
!  k108 :  Cl2O2  -->  ClO + ClO
!  k123 :  BrO + ClO  -->  Br + Cl + O2
!  k124 :  BrO + ClO  -->  BrCl + O2
!  k142 :  ClONO2 + H2O  -->  HOCl + HNO3  (production for HOCl)
!  k143 :  ClONO2 + HCl  -->  Cl2 + HNO3  (destruction for ClONO2)
!  k146 :  HOCl + HCl  -->  Cl2 + H2O  (destruction for HOCl)
!================================================================
!
!  ClONO2 / ClO
!
    rClOxClONO2 = ( + k(i,100)*HM(i,6) ) / ( + k(i,17) + k(i,18) +        &
      k(i,101)*HM(i,1) + k(i,142)*QM(i,3) + k(i,143)*QM(i,8) )
!
!  HOCl / ClO
!
    rClOxHOCl = ( + k(i,102)*HM(i,14) +                                   &
      k(i,142)*rClOxClONO2*QM(i,3) ) / ( + k(i,16) +                      &
      k(i,103)*HM(i,13) + k(i,104)*HM(i,1) + k(i,146)*QM(i,8) )
!
!  Cl2O2 / ClO
!
    rClOxCl2O2 = ( + k(i,107)*HM(i,8) ) / ( + k(i,19) + k(i,108) )
!
!  BrCl / ClO
!
    rClOxBrCl = ( + k(i,124)*HM(i,10) ) / ( + k(i,27) )
!
!  Cl / ClO
!
    rClOxCl = ( + k(i,16)*rClOxHOCl + k(i,17)*rClOxClONO2 +               &
      2.0*k(i,19)*rClOxCl2O2 + k(i,27)*rClOxBrCl + k(i,88)*QM(i,21) +     &
      k(i,91)*HM(i,1) + k(i,92)*HM(i,5) + k(i,97)*HM(i,13) +              &
      k(i,123)*HM(i,10) ) / ( + HM(i,14)*( + k(i,95) + 2.0*k(i,96) ) +    &
      k(i,90)*HM(i,3) + k(i,93)*QM(i,2) + k(i,106)*QM(i,10) )
!
    rClOxCl = max( 0. , rClOxCl )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rClOx = 1. + rClOxCl
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aClOxClO = 0.5
!
    HM(i,8) = aClOxClO*HM(i,8) + (1.-aClOxClO)*QM(i,37)/rClOx
!
    aClOxCl = 0.0
!
    HM(i,9) = aClOxCl*HM(i,9) + (1.-aClOxCl)*HM(i,8)*rClOxCl
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
    rBrOxBr = HM(i,11) / HM(i,10)
!
!================================================================
!  FOR FAMILY BrOx = BrO + Br
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k023 :  BrO  -->  Br + O3P
!  k024 :  BrONO2  -->  Br + NO3
!  k025 :  HOBr  -->  Br + OH
!  k027 :  BrCl  -->  Br + Cl
!  k089 :  HCHO + Br  -->  HO2 + CO + HBr  (destruction for Br)
!  k119 :  Br + O3  -->  BrO + O2
!  k120 :  BrO + O3P  -->  Br + O2
!  k121 :  BrO + NO  -->  NO2 + Br
!  k122 :  BrO + ClO  -->  OClO + Br
!  k123 :  BrO + ClO  -->  Br + Cl + O2
!  k124 :  BrO + ClO  -->  BrCl + O2
!  k125 :  BrO + BrO  -->  Br + Br + O2
!  k126 :  Br + HO2  -->  HBr + O2  (destruction for Br)
!  k128 :  BrO + OH  -->  Br + HO2
!  k130 :  BrO + NO2  -->  BrONO2
!  k131 :  BrO + HO2  -->  HOBr + O2
!  k132 :  HOBr + O3P  -->  OH + BrO
!================================================================
!
!  BrONO2 / BrO
!
    rBrOxBrONO2 = ( + k(i,130)*HM(i,6) ) / ( + k(i,24) )
!
!  HOBr / BrO
!
    rBrOxHOBr = ( + k(i,131)*HM(i,14) ) / ( + k(i,25) + k(i,132)*HM(i,1) )
!
!  BrCl / BrO
!
    rBrOxBrCl = ( + k(i,124)*HM(i,8) ) / ( + k(i,27) )
!
!  Br / BrO
!
    rBrOxBr = ( + HM(i,8)*( + k(i,122) + k(i,123) ) + k(i,23) +           &
      k(i,24)*rBrOxBrONO2 + k(i,25)*rBrOxHOBr + k(i,27)*rBrOxBrCl +       &
      k(i,120)*HM(i,1) + k(i,121)*HM(i,5) + 2.0*k(i,125)*HM(i,10) +       &
      k(i,128)*HM(i,13) ) / ( + k(i,89)*QM(i,20) + k(i,119)*HM(i,3) +     &
      k(i,126)*HM(i,14) )
!
    rBrOxBr = max( 0. , rBrOxBr )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY :
!
    rBrOx = 1. + rBrOxBr
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS :
!
    aBrOxBrO = 0.5
!
    HM(i,10) = aBrOxBrO*HM(i,10) + (1.-aBrOxBrO)*QM(i,38)/rBrOx
!
    aBrOxBr = 0.0
!
    HM(i,11) = aBrOxBr*HM(i,11) + (1.-aBrOxBr)*HM(i,10)*rBrOxBr
!
  endif
!
enddo
!$omp end do
!
if (iniv <= NIVHOX) then
!
!$omp do
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
      if (HM(i,13) < 1.0e-10) then
        rHOxH   = 0.0
        rHOxHO2 = 0.0
      else
        rHOxH   = HM(i,12) / HM(i,13)
        rHOxHO2 = HM(i,14) / HM(i,13)
      endif
!
!================================================================
!  FOR FAMILY HOx = H + OH + HO2
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT :
!
!  k040 :  H + O2  -->  HO2
!  k041 :  O3 + H  -->  OH + O2
!  k043 :  O3P + OH  -->  O2 + H
!  k044 :  O3 + OH  -->  HO2 + O2
!  k045 :  O3 + HO2  -->  OH + O2 + O2
!  k046 :  O3P + HO2  -->  OH + O2
!  k047 :  OH + HO2  -->  H2O + O2  (destruction for HO2)
!  k048 :  OH + H2  -->  H2O + H
!  k049 :  H + HO2  -->  OH + OH
!  k053 :  NO + HO2  -->  NO2 + OH
!  k054 :  HO2 + HO2  -->  H2O2 + O2  (destruction for HO2)
!  k055 :  H2O2 + OH  -->  HO2 + H2O
!  k058 :  OH + CO  -->  CO2 + H
!  k081 :  HCHO + OH  -->  HO2 + CO + H2O
!  k095 :  Cl + HO2  -->  HCl + O2  (destruction for HO2)
!  k096 :   Cl + HO2   -->   OH + ClO  (destruction for HO2)
!  k097 :  ClO + OH  -->  Cl + HO2
!  k102 :  ClO + HO2  -->  HOCl + O2  (destruction for HO2)
!  k126 :  Br + HO2  -->  HBr + O2  (destruction for HO2)
!  k128 :  BrO + OH  -->  Br + HO2
!  k131 :  BrO + HO2  -->  HOBr + O2  (destruction for HO2)
!================================================================
!
!  H / OH
!
      rHOxH = ( + k(i,43)*HM(i,1) + k(i,48)*5.e-7*hnm(i) +                   &
        k(i,58)*QM(i,6) ) / ( + k(i,40)*0.21*hnm(i) + k(i,41)*HM(i,3) +      &
        k(i,49)*HM(i,14) )
!
      aHOxH = 0.0
!
      HM(i,12) = aHOxH*HM(i,12) + (1.-aHOxH)*HM(i,13)*rHOxH
!
!  HO2 / OH
!
      rHOxHO2 = ( + k(i,40)*rHOxH*0.21*hnm(i) + k(i,44)*HM(i,3) +            &
        k(i,55)*QM(i,12) + k(i,81)*QM(i,20) + k(i,97)*HM(i,8) +              &
        k(i,128)*HM(i,10) ) / ( + 2.0*( + k(i,54)*HM(i,14) +                 &
        k(i,96)*HM(i,9) ) + k(i,45)*HM(i,3) + k(i,46)*HM(i,1) +              &
        k(i,47)*HM(i,13) + k(i,49)*HM(i,12) + k(i,53)*HM(i,5) +              &
        k(i,95)*HM(i,9) + k(i,102)*HM(i,8) + k(i,126)*HM(i,11) +             &
        k(i,131)*HM(i,10) )
!
      aHOxHO2 = 0.5
!
      HM(i,14) = aHOxHO2*HM(i,14) + (1.-aHOxHO2)*HM(i,13)*rHOxHO2
!
!  QUADRATIC LOSS FOR FAMILY HOx = H + OH + HO2
!
!  loss(HOx)**2 = + 2.0*k050*[H]*[HO2] + 2.0*k051*[H]*[HO2] +
!                2.0*k047*[OH]*[HO2] + 2.0*k056*[OH]*[OH] + 2.0*k057*[OH]*[OH] +
!                2.0*k054*[HO2]*[HO2]
!
      xl2HOx = + 2.0*( + k(i,50)*rHOxH*rHOxHO2 +                             &
        k(i,51)*rHOxH*rHOxHO2 + k(i,47)*rHOxHO2 + k(i,56) + k(i,57) +        &
        k(i,54)*rHOxHO2*rHOxHO2 )
!
!  LINEAR LOSS FOR FAMILY HOx = H + OH + HO2
!
!  loss(HOx) = + k065*[OH]*[NO2] + k068*[OH]*[HNO3] +
!                k069*[OH]*[HNO4] + k076*[CH4]*[OH] + k086*[OP1]*[OH] +
!                k098*[ClO]*[OH] + k099*[OH]*[HCl] + k103*[OH]*[HOCl] +
!                k115*[OH]*[CH3CCl3] + k116*[OH]*[CH3Cl] + k118*[OH]*[HCFC22] +
!                k129*[OH]*[HBr] + k134*[OH]*[CH3Br] + k066*[HO2]*[NO2] +
!                k080*[MO2]*[HO2] + k095*[Cl]*[HO2] + k102*[ClO]*[HO2] +
!                k126*[Br]*[HO2] + k131*[BrO]*[HO2]
!
      xl1HOx = + rHOxHO2*( + k(i,66)*HM(i,6) + k(i,80)*QM(i,21) +            &
        k(i,95)*HM(i,9) + k(i,102)*HM(i,8) + k(i,126)*HM(i,11) +             &
        k(i,131)*HM(i,10) ) + k(i,65)*HM(i,6) + k(i,68)*gHNO3(i) +           &
        k(i,69)*QM(i,16) + k(i,76)*QM(i,2) + k(i,86)*QM(i,22) +              &
        k(i,98)*HM(i,8) + k(i,99)*gHCl(i) + k(i,103)*QM(i,10) +              &
        k(i,115)*QM(i,27) + k(i,116)*QM(i,28) + k(i,118)*QM(i,29) +          &
        k(i,129)*gHBr(i) + k(i,134)*QM(i,30)
!
!  PRODUCTION FOR FAMILY HOx = H + OH + HO2
!
!  prod(HOx) = + 2.0*k010*[HCHO] + 2.0*k014*[H2O] + k020*[HCl] +
!                2.0*k042*[O1D]*[H2] + 2.0*k052*[H2]*[O3P] + k074*[O1D]*[CH4] +
!                k077*[CH3]*[O3P] + k094*[Cl]*[H2] + k004*[HNO3] +
!                2.0*k008*[H2O2] + k011*[OP1] + k016*[HOCl] + k025*[HOBr] +
!                2.0*k039*[O1D]*[H2O] + k073*[O1D]*[CH4] +
!                2.0*k082*[HCHO]*[O3P] + k104*[O3P]*[HOCl] + k106*[Cl]*[HOCl] +
!                k127*[O1D]*[HBr] + k132*[HOBr]*[O3P] + k005*[HNO4] +
!                k067*[HNO4] + k083*[HCHO]*[Cl] + k085*[CH3O]*[O2] +
!                k089*[HCHO]*[Br]
!
      xpHOx = + 2.0*( + k(i,10)*QM(i,20) + k(i,14)*gH2O(i) +                 &
        k(i,42)*HM(i,2)*5.e-7*hnm(i) + k(i,52)*5.e-7*hnm(i)*HM(i,1) +        &
        k(i,8)*QM(i,12) + k(i,39)*HM(i,2)*gH2O(i) +                          &
        k(i,82)*QM(i,20)*HM(i,1) ) + HM(i,2)*( + k(i,74)*QM(i,2) +           &
        k(i,73)*QM(i,2) + k(i,127)*gHBr(i) ) + HM(i,1)*( +                   &
        k(i,77)*HM(i,15) + k(i,104)*QM(i,10) + k(i,132)*QM(i,18) ) +         &
        HM(i,9)*( + k(i,94)*5.e-7*hnm(i) + k(i,106)*QM(i,10) +               &
        k(i,83)*QM(i,20) ) + QM(i,16)*( + k(i,5) + k(i,67) ) +               &
        k(i,20)*gHCl(i) + k(i,4)*gHNO3(i) + k(i,11)*QM(i,22) +               &
        k(i,16)*QM(i,10) + k(i,25)*QM(i,18) + k(i,85)*HM(i,16)*0.21*hnm(i) + &
        k(i,89)*QM(i,20)*HM(i,11)
!
!  CALCULATE NEW CONCENTRATION FOR FAMILY PIVOT OH
!
      argum = max( 0.0 , xl1HOx**2. + 4.0*xl2HOx*xpHOx )
!
      aHOxOH = 0.5
!
      HM(i,13) = aHOxOH*HM(i,13) + (1.-aHOxOH)*(sqrt(argum) - xl1HOx) / (2.0*xl2HOx)
!
    endif
!
  enddo
!$OMP END DO
!
else
!
!================================================================
!  EXPLICIT CALCULATION FOR SHORT-LIVED SPECIES THAT ARE NOT
!  TRANSPORTED
!================================================================
!
!$omp do
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
!  d[H]/dt = 0.0
!
      HM(i,12) = max( 1.0e-10 , ( + 5.e-7*hnm(i)*( + k(i,42)*HM(i,2) +       &
        k(i,48)*HM(i,13) + k(i,52)*HM(i,1) + k(i,94)*HM(i,9) ) +             &
        HM(i,1)*( + k(i,43)*HM(i,13) + k(i,77)*HM(i,15) ) +                  &
        k(i,10)*QM(i,20) + k(i,14)*gH2O(i) + k(i,20)*gHCl(i) +               &
        k(i,58)*HM(i,13)*QM(i,6) + k(i,74)*HM(i,2)*QM(i,2) ) / ( +           &
        HM(i,14)*( + k(i,49) + k(i,50) + k(i,51) ) +                         &
        k(i,40)*0.21*hnm(i) + k(i,41)*HM(i,3) ) )
!
    endif
!
  enddo
!$omp end do
!
!$omp do
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
!  d[OH]/dt = 0.0
!
      HM(i,13) = max( 1.0e-10 , ( + HM(i,14)*( + k(i,45)*HM(i,3) +          &
        k(i,46)*HM(i,1) + 2.0*k(i,49)*HM(i,12) + k(i,53)*HM(i,5) +          &
        k(i,96)*HM(i,9) ) + HM(i,2)*( + 2.0*k(i,39)*gH2O(i) +               &
        k(i,42)*5.e-7*hnm(i) + k(i,73)*QM(i,2) + k(i,127)*gHBr(i) ) +       &
        HM(i,1)*( + k(i,52)*5.e-7*hnm(i) + k(i,82)*QM(i,20) +               &
        k(i,104)*QM(i,10) + k(i,132)*QM(i,18) ) + QM(i,22)*( + k(i,11) +    &
        k(i,87)*HM(i,13) ) + QM(i,10)*( + k(i,16) + k(i,106)*HM(i,9) ) +    &
        k(i,4)*gHNO3(i) + 2.0*k(i,8)*QM(i,12) + k(i,14)*gH2O(i) +           &
        k(i,25)*QM(i,18) + k(i,41)*HM(i,3)*HM(i,12) ) / ( + 2.0*( +         &
        k(i,56)*HM(i,13) + k(i,57)*HM(i,13) ) + QM(i,22)*( + k(i,86) +      &
        k(i,87) ) + HM(i,8)*( + k(i,97) + k(i,98) ) + k(i,43)*HM(i,1) +     &
        k(i,44)*HM(i,3) + k(i,47)*HM(i,14) + k(i,48)*5.e-7*hnm(i) +         &
        k(i,55)*QM(i,12) + k(i,58)*QM(i,6) + k(i,65)*HM(i,6) +              &
        k(i,68)*gHNO3(i) + k(i,69)*QM(i,16) + k(i,76)*QM(i,2) +             &
        k(i,81)*QM(i,20) + k(i,99)*gHCl(i) + k(i,103)*QM(i,10) +            &
        k(i,115)*QM(i,27) + k(i,116)*QM(i,28) + k(i,118)*QM(i,29) +         &
        k(i,128)*HM(i,10) + k(i,129)*gHBr(i) + k(i,134)*QM(i,30) ) )
!
    endif
!
  enddo
!$omp end do
!
!$omp do
  do i = i1,i2
!
    if (sza(i) <= SZAlim) then
!
!  d[HO2]/dt = 0.0
!
      HM(i,14) = max( 1.0e-10 , ( + QM(i,20)*( + k(i,10) +                  &
        k(i,81)*HM(i,13) + k(i,82)*HM(i,1) + k(i,83)*HM(i,9) +              &
        k(i,89)*HM(i,11) ) + HM(i,13)*( + k(i,44)*HM(i,3) +                 &
        k(i,55)*QM(i,12) + k(i,97)*HM(i,8) + k(i,128)*HM(i,10) ) +          &
        QM(i,16)*( + k(i,5) + k(i,67) ) + 0.21*hnm(i)*( +                   &
        k(i,40)*HM(i,12) + k(i,85)*HM(i,16) ) ) / ( + HM(i,12)*( +          &
        k(i,49) + k(i,50) + k(i,51) ) + HM(i,9)*( + k(i,95) +               &
        k(i,96) ) + k(i,45)*HM(i,3) + k(i,46)*HM(i,1) +                     &
        k(i,47)*HM(i,13) + k(i,53)*HM(i,5) + 2.0*k(i,54)*HM(i,14) +         &
        k(i,66)*HM(i,6) + k(i,80)*QM(i,21) + k(i,102)*HM(i,8) +             &
        k(i,126)*HM(i,11) + k(i,131)*HM(i,10) ) )
!
    endif
!
  enddo
!$omp end do
!
endif
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
!  d[CH3]/dt = 0.0
!
    HM(i,15) = max( 1.0e-10 , ( + QM(i,2)*( + k(i,73)*HM(i,2) +            &
      k(i,76)*HM(i,13) + k(i,93)*HM(i,9) ) ) / ( + k(i,77)*HM(i,1) +       &
      k(i,78)*0.21*hnm(i) ) )
!
  endif
!
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  if (sza(i) <= SZAlim) then
!
!  d[CH3O]/dt = 0.0
!
    HM(i,16) = max( 1.0e-10 , ( + QM(i,21)*( + k(i,79)*HM(i,5) +           &
      2.0*k(i,84)*QM(i,21) + k(i,88)*HM(i,8) ) + k(i,11)*QM(i,22) +        &
      k(i,74)*HM(i,2)*QM(i,2) ) / ( + k(i,85)*0.21*hnm(i) ) )
!
  endif
!
enddo
!$omp end do
!
!!!!!
!
!  2. NIGHT-TIME CHEMISTRY
!
!!!!!
!
!$omp do
do i = i1,i2
!
  if (sza(i) > SZAlim) then
!
!================================================================
!  AT NIGHT, FAMILY Ox = O3P + O1D + O3
!  IS REDUCED TO O3 SPECIE
!================================================================
!
!  O3P
!
    HM(i,1) = 0.0
!
!  O1D
!
    HM(i,2) = 0.0
!
!  O3
!
    HM(i,3) = max( 0.0 , QM(i,35) )
!
!================================================================
!  FOR FAMILY NOx = N + NO + NO2 + NO3
!  FOLLOWING REACTIONS ARE TAKEN INTO ACCOUNT AT NIGHT :
!
!  k063 :  O3 + NO2  -->  NO3 + O2
!  k064 :  NO3 + NO2  -->  N2O5  (destruction only)
!================================================================
!
!  N
!
    HM(i,4) = 0.0
!
!  NO
!
    HM(i,5) = 0.0
!
!  NO3 / NO2
!
    rNOxNO3 = ( + k(i,63)*HM(i,3) ) / ( + k(i,64)*HM(i,6) )
!
    rNOxNO3 = max( 0. , rNOxNO3 )
!
!  TOTAL SUM OF RATIOS WITHIN FAMILY AT NIGHT :
!
    rNOx = 1. + rNOxNO3
!
!  CALCULATE NEW CONCENTRATIONS FOR FAMILY MEMBERS AT NIGHT :
!
    aNOxNO2 = 0.0
!
    HM(i,6) = aNOxNO2*HM(i,6) + (1.-aNOxNO2)*QM(i,36)/rNOx
!
    aNOxNO3 = 0.0
!
    HM(i,7) = aNOxNO3*HM(i,7) + (1.-aNOxNO3)*HM(i,6)*rNOxNO3
!
!================================================================
!  AT NIGHT, FAMILY ClOx = ClO + Cl
!  IS REDUCED TO ClO SPECIE
!================================================================
!
!  ClO
!
    HM(i,8) = max( 0.0 , QM(i,37) )
!
!  Cl
!
    HM(i,9) = 0.0
!
!================================================================
!  AT NIGHT, FAMILY BrOx = BrO + Br
!  IS REDUCED TO BrO SPECIE
!================================================================
!
!  BrO
!
    HM(i,10) = max( 0.0 , QM(i,38) )
!
!  Br
!
    HM(i,11) = 0.0
!
!  H
!
    HM(i,12) = 0.0
!
!  OH
!
    HM(i,13) = 0.0
!
!  HO2
!
    HM(i,14) = 0.0
!
!  CH3
!
    HM(i,15) = 0.0
!
!  CH3O
!
    HM(i,16) = 0.0
!
  endif
!
enddo
!$omp end do
!
!$omp end parallel
!
RETURN
END SUBROUTINE CH_EQUIL
