SUBROUTINE CH_PTERMS(k,jco2,hnm,day,QM,HM,pr,pe,                            &
                     partHNO3,partH2O,partHCl,partHBr,imax,nk,ntra,nsls)
!
!!!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_0 
!
!!! DATE : 15/05/2005
!
!
!   CALCUL DES TERMES DE PRODUCTION ET DE DESTRUCTION DES ESPECES A
!   LONGUE DUREE DE VIE ET DES FAMILLES TRANSPORTEES
!
!!!!!
!
!  Original : H. Teyssedre, V.-H. Peuch, 08/2000
!
!  Modified : H. Teyssedre, version Fortran-90, 11/2006
!
!!!!!
!
#ifdef MPI
      USE CSTES_MOD
#endif
!
      IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
      INTEGER , INTENT(IN) :: imax, nk, ntra, nsls
      REAL (KIND=8) , DIMENSION(imax,nk)   , INTENT(IN) :: k
      REAL (KIND=8) , DIMENSION(imax)      , INTENT(IN) :: jco2,hnm,day
      REAL (KIND=8) , DIMENSION(imax)      , INTENT(IN) :: partHNO3,partH2O
      REAL (KIND=8) , DIMENSION(imax)      , INTENT(IN) :: partHCl,partHBr
      REAL (KIND=8) , DIMENSION(imax,ntra) , INTENT(IN) :: QM
      REAL (KIND=8) , DIMENSION(imax,nsls) , INTENT(IN) :: HM
      REAL (KIND=8) , DIMENSION(imax,ntra) , INTENT(INOUT) :: pr,pe
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
      INTEGER :: I,J,i1,i2
!
      REAL (KIND=8) , PARAMETER :: rco2=360.e-06
!
!!!!!
!
!  TRANSPORTED LONG-LIVED SPECIES AND FAMILIES (ARRAY QM)
!
!!!!!
!
!       1 : N2O         2 : CH4         3 : H2O         4 : HNO3        5 : N2O5
!       6 : CO          7 : OClO        8 : HCl         9 : ClONO2     10 : HOCl
!      11 : Cl2        12 : H2O2       13 : ClNO2      14 : HBr        15 : BrON
!      16 : HNO4       17 : Cl2O2      18 : HOBr       19 : BrCl       20 : HCHO
!      21 : MO2        22 : OP1        23 : CFC11      24 : CFC12      25 : CFC1
!      26 : CCl4       27 : CH3CCl3    28 : CH3Cl      29 : HCFC22     30 : CH3B
!      31 : H1211      32 : H1301      33 : H2SO4      34 : PSC        35 : Ox
!      36 : NOx        37 : ClOx       38 : BrOx
!
!!!!!
!
!  SHORT-LIVED SPECIES AT INSTANTANEOUS EQUILIBRIUM (ARRAY HM)
!
!!!!!
!
!       1 : O3P         2 : O1D         3 : O3          4 : N           5 : NO
!       6 : NO2         7 : NO3         8 : ClO         9 : Cl         10 : BrO
!      11 : Br         12 : H          13 : OH         14 : HO2        15 : CH3
!      16 : CH3O
!
!
!  pr(N2O) = 0.0
!
!  pe(N2O) =  + k015 + k071*[O1D] + k072*[O1D]
!
!  pr(CH4) = 0.0
!
!  pe(CH4) =  + k073*[O1D] + k074*[O1D] + k075*[O1D] + k076*[OH] + k093*[Cl]
!
!  pr(H2O) =  + k047*[OH]*[HO2] + k048*[OH]*[H2] +
!                k051*[H]*[HO2] + k055*[H2O2]*[OH] + k056*[OH]*[OH] +
!                k068*[OH]*[HNO3]*partHNO3 + k069*[OH]*[HNO4] + k076*[CH4]*[OH]
!                k081*[HCHO]*[OH] + k086*[OP1]*[OH] + k087*[OP1]*[OH] +
!                k099*[OH]*[HCl]*partHCl + k103*[OH]*[HOCl] +
!                k115*[OH]*[CH3CCl3] + k116*[OH]*[CH3Cl] + k118*[OH]*[HCFC22] +
!                k129*[OH]*[HBr]*partHBr + k134*[OH]*[CH3Br] +
!                k146*[HOCl]*[HCl] + k148*[HOBr]*[HCl] + k149*[HOBr]*[HBr] +
!                k150*[HOCl]*[HBr]
!
!  pe(H2O) =  + k014 + k039*[O1D] + k142*[ClONO2] + k144*[N2O5] + k147*[BrONO2]
!
!  pr(HNO3) =  + k065*[OH]*[NO2] + k142*[ClONO2]*[H2O] +
!                k143*[ClONO2]*[HCl] + 2.0*k144*[N2O5]*[H2O] +
!                k145*[N2O5]*[HCl] + k147*[BrONO2]*[H2O]
!
!  pe(HNO3) =  + k004 + k068*[OH]
!
!  pr(N2O5) =  + k064*[NO3]*[NO2]
!
!  pe(N2O5) =  + k013 + k070 + k144*[H2O] + k145*[HCl]
!
!  pr(CO) =  + k009*[HCHO] + k010*[HCHO] + k081*[HCHO]*[OH] +
!                k082*[HCHO]*[O3P] + k083*[HCHO]*[Cl] + k089*[HCHO]*[Br]
!
!  pe(CO) =  + k058*[OH]
!
!  pr(OClO) =  + k122*[BrO]*[ClO]
!
!  pe(OClO) =  + k026
!
!  pr(HCl) =  + k083*[HCHO]*[Cl] + k093*[Cl]*[CH4] +
!                k094*[Cl]*[H2] + k095*[Cl]*[HO2] + k098*[ClO]*[OH]
!
!  pe(HCl) =  + k020 + k099*[OH] + k143*[ClONO2] + k145*[N2O5] +
!                k146*[HOCl] + k148*[HOBr]
!
!  pr(ClONO2) =  + k100*[ClO]*[NO2]
!
!  pe(ClONO2) =  + k017 + k018 + k101*[O3P] + k142*[H2O] + k143*[HCl]
!
!  pr(HOCl) =  + k102*[ClO]*[HO2] + k142*[ClONO2]*[H2O]
!
!  pe(HOCl) =  + k016 + k103*[OH] + k104*[O3P] + k106*[Cl] +
!                k146*[HCl] + k150*[HBr]
!
!  pr(Cl2) =  + k106*[Cl]*[HOCl] + k109*[ClO]*[ClO] +
!                k110*[ClO]*[ClO] + k143*[ClONO2]*[HCl] + k146*[HOCl]*[HCl]
!
!  pe(Cl2) =  + k021
!
!  pr(H2O2) =  + k054*[HO2]*[HO2] + k057*[OH]*[OH]
!
!  pe(H2O2) =  + k008 + k055*[OH]
!
!  pr(ClNO2) =  + k105*[Cl]*[NO2] + k145*[N2O5]*[HCl]
!
!  pe(ClNO2) =  + k022
!
!  pr(HBr) =  + k089*[HCHO]*[Br] + k126*[Br]*[HO2]
!
!  pe(HBr) =  + k127*[O1D] + k129*[OH] + k149*[HOBr] + k150*[HOCl]
!
!  pr(BrONO2) =  + k130*[BrO]*[NO2]
!
!  pe(BrONO2) =  + k024 + k147*[H2O]
!
!  pr(HNO4) =  + k066*[HO2]*[NO2]
!
!  pe(HNO4) =  + k005 + k067 + k069*[OH]
!
!  pr(Cl2O2) =  + k107*[ClO]*[ClO]
!
!  pe(Cl2O2) =  + k019 + k108
!
!  pr(HOBr) =  + k131*[BrO]*[HO2] + k147*[BrONO2]*[H2O]
!
!  pe(HOBr) =  + k025 + k132*[O3P] + k148*[HCl] + k149*[HBr]
!
!  pr(BrCl) =  + k124*[BrO]*[ClO] + k148*[HOBr]*[HCl] + k150*[HOCl]*[HBr]
!
!  pe(BrCl) =  + k027
!
!  pr(HCHO) =  + k075*[O1D]*[CH4] + k077*[CH3]*[O3P] +
!                k085*[CH3O]*[O2] + k087*[OP1]*[OH]
!
!  pe(HCHO) =  + k009 + k010 + k081*[OH] + k082*[O3P] + k083*[Cl] + k089*[Br]
!
!  pr(MO2) =  + k078*[CH3]*[O2] + k086*[OP1]*[OH]
!
!  pe(MO2) =  + k079*[NO] + k080*[HO2] + 2.0*k084*[MO2] + k088*[ClO]
!
!  pr(OP1) =  + k080*[MO2]*[HO2]
!
!  pe(OP1) =  + k011 + k086*[OH] + k087*[OH]
!
!  pr(CFC11) = 0.0
!
!  pe(CFC11) =  + k028 + k111*[O1D]
!
!  pr(CFC12) = 0.0
!
!  pe(CFC12) =  + k029 + k112*[O1D]
!
!  pr(CFC113) = 0.0
!
!  pe(CFC113) =  + k030 + k113*[O1D]
!
!  pr(CCl4) = 0.0
!
!  pe(CCl4) =  + k031 + k114*[O1D]
!
!  pr(CH3CCl3) = 0.0
!
!  pe(CH3CCl3) =  + k032 + k115*[OH]
!
!  pr(CH3Cl) = 0.0
!
!  pe(CH3Cl) =  + k033 + k116*[OH]
!
!  pr(HCFC22) = 0.0
!
!  pe(HCFC22) =  + k034 + k117*[O1D] + k118*[OH]
!
!  pr(CH3Br) = 0.0
!
!  pe(CH3Br) =  + k035 + k133*[O1D] + k134*[OH]
!
!  pr(H1211) = 0.0
!
!  pe(H1211) =  + k036 + k135*[O1D]
!
!  pr(H1301) = 0.0
!
!  pe(H1301) =  + k037 + k136*[O1D]
!
!  pr(H2SO4) = 0.0
!
!  pe(H2SO4) = 0.0
!
!  pr(PSC) = 0.0
!
!  pe(PSC) = 0.0
!
!  pr(Ox) =
!              + k001*[NO2] + k007*[NO3] + 2.0*k012*[O2] +
!                k023*[BrO] + k026*[OClO] + k038*[NO] + k051*[H]*[HO2] +
!                k056*[OH]*[OH] + k061*[N]*[NO] + k062*[N]*[O2] + k015*[N2O]
!
!  pe(Ox) =  + k091*[ClO]*[O3P] +
!                2.0*k107*[ClO]*[ClO]*k019/(k019 + k108) +
!                min(k016*[HOCl],k102*[HO2]*[ClO]) +
!                2.0*min(k027*[BrCl],k124*[BrO]*[ClO]) +
!                2.0*k123*[BrO]*[ClO]*day + min(k025*[HOBr],k131*[HO2]*[BrO]) +
!                2.0*k125*[BrO]*[BrO]*day + 2.0*(k018 + k017)*[ClONO2]
!              + k043*[O3P]*[OH] + k046*[O3P]*[HO2] +
!                k052*[H2]*[O3P] + k059*[O3P]*[NO2] + k077*[CH3]*[O3P] +
!                k082*[HCHO]*[O3P] + k091*[ClO]*[O3P] + k101*[O3P]*[ClONO2] +
!                k104*[O3P]*[HOCl] + k120*[BrO]*[O3P] + k132*[HOBr]*[O3P] +
!                2.0*k137*[O3P]*[O3P] + 2.0*k139*[O3P]*[O3] + k039*[O1D]*[H2O] +
!                k042*[O1D]*[H2] + k071*[O1D]*[N2O] + k072*[O1D]*[N2O] +
!                k073*[O1D]*[CH4] + k074*[O1D]*[CH4] + k075*[O1D]*[CH4] +
!                k111*[O1D]*[CFC11] + k112*[O1D]*[CFC12] + k113*[O1D]*[CFC113] +
!                k114*[O1D]*[CCl4] + k117*[O1D]*[HCFC22] + k127*[O1D]*[HBr] +
!                k133*[O1D]*[CH3Br] + k135*[O1D]*[H1211] + k136*[O1D]*[H1301] +
!                k041*[O3]*[H] + k044*[O3]*[OH] + k045*[O3]*[HO2] +
!                k060*[O3]*[NO] + k063*[O3]*[NO2] + k090*[Cl]*[O3] +
!                k119*[Br]*[O3]
!
!  pr(NOx) =  + 2.0*k013*[N2O5]
!              + 2.0*k072*[O1D]*[N2O] + k004*[HNO3]*partHNO3 +
!                k005*[HNO4] + k018*[ClONO2] + k022*[ClNO2] + k067*[HNO4] +
!                k069*[OH]*[HNO4] + 2.0*k070*[N2O5] + k017*[ClONO2] +
!                k024*[BrONO2] + k068*[OH]*[HNO3]*partHNO3 + k101*[O3P]*[ClONO2]
!
!  pe(NOx) =
!              + 2.0*k061*[N]*[NO] + 2.0*k064*[NO3]*[NO2] +
!                k065*[OH]*[NO2] + k066*[HO2]*[NO2] + k100*[ClO]*[NO2] +
!                k105*[Cl]*[NO2] + k130*[BrO]*[NO2]
!
!  pr(ClOx) =  + 3.0*k028*[CFC11] + 2.0*k029*[CFC12] +
!                3.0*k030*[CFC113] + 4.0*k031*[CCl4] + 3.0*k032*[CH3CCl3] +
!                k033*[CH3Cl] + k034*[HCFC22] + 3.0*k111*[O1D]*[CFC11] +
!                2.0*k112*[O1D]*[CFC12] + 3.0*k113*[O1D]*[CFC113] +
!                4.0*k114*[O1D]*[CCl4] + 3.0*k115*[OH]*[CH3CCl3] +
!                k116*[OH]*[CH3Cl] + k117*[O1D]*[HCFC22] + k118*[OH]*[HCFC22]
!              + k026*[OClO] + k020*[HCl]*partHCl +
!                2.0*k021*[Cl2] + k022*[ClNO2] + k099*[OH]*[HCl]*partHCl +
!                k017*[ClONO2] + k018*[ClONO2] + k101*[O3P]*[ClONO2] +
!                k016*[HOCl] + k103*[OH]*[HOCl] + k104*[O3P]*[HOCl] +
!                2.0*k019*[Cl2O2] + 2.0*k108*[Cl2O2] + k027*[BrCl]
!
!  pe(ClOx) =
!              + k098*[ClO]*[OH] + 2.0*k109*[ClO]*[ClO] +
!                2.0*k110*[ClO]*[ClO] + k122*[BrO]*[ClO] + k083*[HCHO]*[Cl] +
!                k093*[Cl]*[CH4] + k094*[Cl]*[H2] + k095*[Cl]*[HO2] +
!                k105*[Cl]*[NO2] + k106*[Cl]*[HOCl] + k100*[ClO]*[NO2] +
!                k102*[ClO]*[HO2] + 2.0*k107*[ClO]*[ClO] + k124*[BrO]*[ClO]
!
!  pr(BrOx) =  + k035*[CH3Br] + k036*[H1211] + k037*[H1301] +
!                k133*[O1D]*[CH3Br] + k135*[O1D]*[H1211] + k136*[O1D]*[H1301] +
!                2.0*k149*[HOBr]*[HBr]
!              + k127*[O1D]*[HBr]*partHBr +
!                k129*[OH]*[HBr]*partHBr + k024*[BrONO2] + k025*[HOBr] +
!                k132*[HOBr]*[O3P] + k027*[BrCl]
!
!  pe(BrOx) =
!              + k089*[HCHO]*[Br] + k126*[Br]*[HO2] +
!                k130*[BrO]*[NO2] + k131*[BrO]*[HO2] + k124*[BrO]*[ClO]
!
!
#ifdef MPI
i1 = i_deb(myid) ; i2 = i_fin(myid)
#else
i1 = 1           ; i2 = imax
#endif
!
!$omp parallel default(none) &
!$omp private (i,j) &
!$omp shared  (i1,i2,k,jco2,hnm,day,QM,HM,pr,pe) &
!$omp shared  (partHNO3,partH2O,partHCl,partHBr,imax,ntra)
!
!$omp do
do j=1,ntra
do i = i1,i2
  pr(i,j) = 0.0
  pe(i,j) = 0.0
enddo
enddo
!$omp end do
!
!$omp do
do i = i1,i2
!
  pe(i,1) =  + HM(i,2)*( + k(i,71) + k(i,72) ) + k(i,15)
!
  pe(i,2) =  + HM(i,2)*( + k(i,73) + k(i,74) + k(i,75) ) +                 &
    k(i,76)*HM(i,13) + k(i,93)*HM(i,9)
!
  pe(i,3) = ( + k(i,14) + k(i,39)*HM(i,2) )*partH2O(i) +                   &
    k(i,142)*QM(i,9) + k(i,144)*QM(i,5) + k(i,147)*QM(i,15)
!
  pe(i,4) = ( + k(i,4) + k(i,68)*HM(i,13) )*partHNO3(i)
!
  pr(i,5) =  + k(i,64)*HM(i,7)*HM(i,6)
!
  pe(i,5) =  + k(i,13) + k(i,70) + k(i,144)*QM(i,3) + k(i,145)*QM(i,8)
!
  pe(i,6) =  + k(i,58)*HM(i,13)
!
  pr(i,7) =  + k(i,122)*HM(i,10)*HM(i,8)
!
  pe(i,7) =  + k(i,26)
!
  pr(i,8) =  + HM(i,9)*( + k(i,83)*QM(i,20) + k(i,93)*QM(i,2) +            &
    k(i,94)*5.e-7*hnm(i) + k(i,95)*HM(i,14) ) + k(i,98)*HM(i,8)*HM(i,13)
!
  pr(i,9) =  + k(i,100)*HM(i,8)*HM(i,6)
!
  pe(i,9) =  + k(i,17) + k(i,18) + k(i,101)*HM(i,1) +                      &
    k(i,142)*QM(i,3) + k(i,143)*QM(i,8)
!
  pr(i,10) =  + k(i,102)*HM(i,8)*HM(i,14) + k(i,142)*QM(i,9)*QM(i,3)
!
  pr(i,11) =  + QM(i,10)*( + k(i,106)*HM(i,9) +                            &
    k(i,146)*QM(i,8) ) + HM(i,8)*( + k(i,109)*HM(i,8) +                    &
    k(i,110)*HM(i,8) ) + k(i,143)*QM(i,9)*QM(i,8)
!
  pe(i,11) =  + k(i,21)
!
  pr(i,12) =  + k(i,54)*HM(i,14)*HM(i,14) + k(i,57)*HM(i,13)*HM(i,13)
!
  pe(i,12) =  + k(i,8) + k(i,55)*HM(i,13)
!
  pr(i,13) =  + k(i,105)*HM(i,9)*HM(i,6) + k(i,145)*QM(i,5)*QM(i,8)
!
  pe(i,13) =  + k(i,22)
!
  pr(i,14) =  + HM(i,11)*( + k(i,89)*QM(i,20) + k(i,126)*HM(i,14) )
!
  pe(i,14) = ( + k(i,127)*HM(i,2) + k(i,129)*HM(i,13)                     &
    )*partHBr(i) + k(i,149)*QM(i,18) + k(i,150)*QM(i,10)
!
  pr(i,15) =  + k(i,130)*HM(i,10)*HM(i,6)
!
  pe(i,15) =  + k(i,24) + k(i,147)*QM(i,3)
!
  pr(i,16) =  + k(i,66)*HM(i,14)*HM(i,6)
!
  pe(i,16) =  + k(i,5) + k(i,67) + k(i,69)*HM(i,13)
!
  pr(i,17) =  + k(i,107)*HM(i,8)*HM(i,8)
!
  pe(i,17) =  + k(i,19) + k(i,108)
!
  pr(i,18) =  + k(i,131)*HM(i,10)*HM(i,14) + k(i,147)*QM(i,15)*QM(i,3)
!
  pe(i,18) =  + k(i,25) + k(i,132)*HM(i,1) + k(i,148)*QM(i,8) +           &
    k(i,149)*QM(i,14)
!
  pr(i,19) =  + k(i,124)*HM(i,10)*HM(i,8) + k(i,148)*QM(i,18)*QM(i,8) +   &
    k(i,150)*QM(i,10)*QM(i,14)
!
  pe(i,19) =  + k(i,27)
!
  pr(i,20) =  + k(i,75)*HM(i,2)*QM(i,2) + k(i,77)*HM(i,15)*HM(i,1) +      &
    k(i,85)*HM(i,16)*0.21*hnm(i) + k(i,87)*QM(i,22)*HM(i,13)
!
  pr(i,21) =  + k(i,78)*HM(i,15)*0.21*hnm(i) + k(i,86)*QM(i,22)*HM(i,13)
!
  pe(i,21) =  + k(i,79)*HM(i,5) + k(i,80)*HM(i,14) +                      &
    2.0*k(i,84)*QM(i,21) + k(i,88)*HM(i,8)
!
  pr(i,22) =  + k(i,80)*QM(i,21)*HM(i,14)
!
  pe(i,22) =  + HM(i,13)*( + k(i,86) + k(i,87) ) + k(i,11)
!
  pe(i,23) =  + k(i,28) + k(i,111)*HM(i,2)
!
  pe(i,24) =  + k(i,29) + k(i,112)*HM(i,2)
!
  pe(i,25) =  + k(i,30) + k(i,113)*HM(i,2)
!
  pe(i,26) =  + k(i,31) + k(i,114)*HM(i,2)
!
  pe(i,27) =  + k(i,32) + k(i,115)*HM(i,13)
!
  pe(i,28) =  + k(i,33) + k(i,116)*HM(i,13)
!
  pe(i,29) =  + k(i,34) + k(i,117)*HM(i,2) + k(i,118)*HM(i,13)
!
  pe(i,30) =  + k(i,35) + k(i,133)*HM(i,2) + k(i,134)*HM(i,13)
!
  pe(i,31) =  + k(i,36) + k(i,135)*HM(i,2)
!
  pe(i,32) =  + k(i,37) + k(i,136)*HM(i,2)
!
  pr(i,4) =  + QM(i,3)*( + k(i,142)*QM(i,9) +                              &
    2.0*k(i,144)*QM(i,5) + k(i,147)*QM(i,15) ) + QM(i,8)*( +               &
    k(i,143)*QM(i,9) + k(i,145)*QM(i,5) ) + k(i,65)*HM(i,13)*HM(i,6)
!
  pr(i,6) =  + QM(i,20)*( + k(i,9) + k(i,10) + k(i,81)*HM(i,13) +          &
    k(i,82)*HM(i,1) + k(i,83)*HM(i,9) + k(i,89)*HM(i,11) )
#ifdef NIV_60
  pr(i,6) = pr(i,6) + jco2(i)*rco2*hnm(i)
#endif
!
  pe(i,8) = ( + k(i,20) + k(i,99)*HM(i,13) )*partHCl(i) +                  &
    k(i,143)*QM(i,9) + k(i,145)*QM(i,5) + k(i,146)*QM(i,10) +              &
    k(i,148)*QM(i,18)
!
  pe(i,10) =  + k(i,16) + k(i,103)*HM(i,13) + k(i,104)*HM(i,1) +           &
    k(i,106)*HM(i,9) + k(i,146)*QM(i,8) + k(i,150)*QM(i,14)
!
  pe(i,20) =  + k(i,9) + k(i,10) + k(i,81)*HM(i,13) +                      &
    k(i,82)*HM(i,1) + k(i,83)*HM(i,9) + k(i,89)*HM(i,11)
!
  pr(i,3) =  + HM(i,13)*( + k(i,47)*HM(i,14) + k(i,48)*5.e-7*hnm(i) +      &
    k(i,55)*QM(i,12) + k(i,56)*HM(i,13) +                                  &
    k(i,68)*QM(i,4)*partHNO3(i) + k(i,69)*QM(i,16) +                       &
    k(i,76)*QM(i,2) + k(i,81)*QM(i,20) + k(i,86)*QM(i,22) +                &
    k(i,87)*QM(i,22) + k(i,99)*QM(i,8)*partHCl(i) +                        &
    k(i,103)*QM(i,10) + k(i,115)*QM(i,27) + k(i,116)*QM(i,28) +            &
    k(i,118)*QM(i,29) + k(i,129)*QM(i,14)*partHBr(i) +                     &
    k(i,134)*QM(i,30) ) + QM(i,10)*( + k(i,146)*QM(i,8) +                  &
    k(i,150)*QM(i,14) ) + QM(i,18)*( + k(i,148)*QM(i,8) +                  &
    k(i,149)*QM(i,14) ) + k(i,51)*HM(i,12)*HM(i,14)
!
  pr(i,35) = + 0.21*hnm(i)*( + 2.0*k(i,12) + k(i,62)*HM(i,4) ) +           &
    HM(i,5)*( + k(i,38) + k(i,61)*HM(i,4) ) + k(i,1)*HM(i,6) +             &
    k(i,7)*HM(i,7) + k(i,23)*HM(i,10) + k(i,26)*QM(i,7) +                  &
    k(i,51)*HM(i,12)*HM(i,14) + k(i,56)*HM(i,13)*HM(i,13) +                &
    k(i,15)*QM(i,1)
!
  pe(i,35) =  + k(i,91)*HM(i,8)*HM(i,1) +                                  &
    2.0*k(i,107)*HM(i,8)*HM(i,8)*k(i,19)/(k(i,19) + k(i,108)) +            &
    MIN(k(i,16)*QM(i,10),k(i,102)*HM(i,14)*HM(i,8)) +                      &
    2.0*MIN(k(i,27)*QM(i,19),k(i,124)*HM(i,10)*HM(i,8)) +                  &
    2.0*k(i,123)*HM(i,10)*HM(i,8)*day(i) +                                 &
    MIN(k(i,25)*QM(i,18),k(i,131)*HM(i,14)*HM(i,10)) +                     &
    2.0*k(i,125)*HM(i,10)*HM(i,10)*day(i) + 2.0*(k(i,18) +                 &
    k(i,17))*QM(i,9)
  pe(i,35) = pe(i,35) + HM(i,2)*( + k(i,39)*QM(i,3) +                      &
    k(i,42)*5.e-7*hnm(i) + k(i,71)*QM(i,1) + k(i,72)*QM(i,1) +             &
    k(i,73)*QM(i,2) + k(i,74)*QM(i,2) + k(i,75)*QM(i,2) +                  &
    k(i,111)*QM(i,23) + k(i,112)*QM(i,24) + k(i,113)*QM(i,25) +            &
    k(i,114)*QM(i,26) + k(i,117)*QM(i,29) + k(i,127)*QM(i,14) +            &
    k(i,133)*QM(i,30) + k(i,135)*QM(i,31) + k(i,136)*QM(i,32) ) +          &
    HM(i,1)*( + k(i,43)*HM(i,13) + k(i,46)*HM(i,14) +                      &
    k(i,52)*5.e-7*hnm(i) + k(i,59)*HM(i,6) + k(i,77)*HM(i,15) +            &
    k(i,82)*QM(i,20) + k(i,91)*HM(i,8) + k(i,101)*QM(i,9) +                &
    k(i,104)*QM(i,10) + k(i,120)*HM(i,10) + k(i,132)*QM(i,18) +            &
    2.0*k(i,137)*HM(i,1) + 2.0*k(i,139)*HM(i,3) ) + HM(i,3)*( +            &
    k(i,41)*HM(i,12) + k(i,44)*HM(i,13) + k(i,45)*HM(i,14) +               &
    k(i,60)*HM(i,5) + k(i,63)*HM(i,6) + k(i,90)*HM(i,9) +                  &
    k(i,119)*HM(i,11) )
!
  pe(i,35) = pe(i,35) / QM(i,35)
!
  pr(i,36) =  + 2.0*k(i,13)*QM(i,5)
  pr(i,36) = pr(i,36) + QM(i,16)*( + k(i,5) + k(i,67) +                    &
    k(i,69)*HM(i,13) ) + QM(i,9)*( + k(i,18) + k(i,17) +                   &
    k(i,101)*HM(i,1) ) + 2.0*( + k(i,72)*HM(i,2)*QM(i,1) +                 &
    k(i,70)*QM(i,5) ) + QM(i,4)*( + k(i,4)*partHNO3(i) +                   &
    k(i,68)*HM(i,13)*partHNO3(i) ) + k(i,22)*QM(i,13) +                    &
    k(i,24)*QM(i,15)
!
  pe(i,36) = + HM(i,6)*( + 2.0*k(i,64)*HM(i,7) + k(i,65)*HM(i,13) +        &
    k(i,66)*HM(i,14) + k(i,100)*HM(i,8) + k(i,105)*HM(i,9) +               &
    k(i,130)*HM(i,10) ) + 2.0*k(i,61)*HM(i,4)*HM(i,5)
!
  pe(i,36) = pe(i,36) / QM(i,36)
!
  pr(i,37) =  + 3.0*( + k(i,28)*QM(i,23) + k(i,30)*QM(i,25) +              &
    k(i,32)*QM(i,27) + k(i,111)*HM(i,2)*QM(i,23) +                         &
    k(i,113)*HM(i,2)*QM(i,25) + k(i,115)*HM(i,13)*QM(i,27) ) +             &
    QM(i,29)*( + k(i,34) + k(i,117)*HM(i,2) + k(i,118)*HM(i,13) ) +        &
    2.0*( + k(i,29)*QM(i,24) + k(i,112)*HM(i,2)*QM(i,24) ) + 4.0*( +       &
    k(i,31)*QM(i,26) + k(i,114)*HM(i,2)*QM(i,26) ) + QM(i,28)*( +          &
    k(i,33) + k(i,116)*HM(i,13) )
  pr(i,37) = pr(i,37) + 2.0*( + k(i,21)*QM(i,11) +                         &
    k(i,19)*QM(i,17) + k(i,108)*QM(i,17) ) + QM(i,9)*( + k(i,17) +         &
    k(i,18) + k(i,101)*HM(i,1) ) + QM(i,10)*( + k(i,16) +                  &
    k(i,103)*HM(i,13) + k(i,104)*HM(i,1) ) + QM(i,8)*( +                   &
    k(i,20)*partHCl(i) + k(i,99)*HM(i,13)*partHCl(i) ) +                   &
    k(i,26)*QM(i,7) + k(i,22)*QM(i,13) + k(i,27)*QM(i,19)
!
  pe(i,37) = + HM(i,8)*( + k(i,98)*HM(i,13) +                              &
    2.0*k(i,109)*HM(i,8) + 2.0*k(i,110)*HM(i,8) +                          &
    k(i,122)*HM(i,10) + k(i,100)*HM(i,6) + k(i,102)*HM(i,14) +             &
    2.0*k(i,107)*HM(i,8) + k(i,124)*HM(i,10) ) + HM(i,9)*( +               &
    k(i,83)*QM(i,20) + k(i,93)*QM(i,2) + k(i,94)*5.e-7*hnm(i) +            &
    k(i,95)*HM(i,14) + k(i,105)*HM(i,6) + k(i,106)*QM(i,10) )
!
  pe(i,37) = pe(i,37) / QM(i,37)
!
  pr(i,38) =  + HM(i,2)*( + k(i,133)*QM(i,30) + k(i,135)*QM(i,31) +        &
    k(i,136)*QM(i,32) ) + k(i,35)*QM(i,30) + k(i,36)*QM(i,31) +            &
    k(i,37)*QM(i,32) + 2.0*k(i,149)*QM(i,18)*QM(i,14)
  pr(i,38) = pr(i,38) + QM(i,14)*( + k(i,127)*HM(i,2)*partHBr(i) +         &
    k(i,129)*HM(i,13)*partHBr(i) ) + QM(i,18)*( + k(i,25) +                &
    k(i,132)*HM(i,1) ) + k(i,24)*QM(i,15) + k(i,27)*QM(i,19)
!
  pe(i,38) = + HM(i,10)*( + k(i,130)*HM(i,6) + k(i,131)*HM(i,14) +         &
    k(i,124)*HM(i,8) ) + HM(i,11)*( + k(i,89)*QM(i,20) +                   &
    k(i,126)*HM(i,14) )
!
  pe(i,38) = pe(i,38) / QM(i,38)
#ifdef MASSy
!
  pr(i,39) = + 2.0*k(i,72)*HM(i,2)*QM(i,1)
!
  pe(i,39) = + 2.0*k(i,61)*HM(i,4)*HM(i,5)
!
  pe(i,39) = pe(i,39) / QM(i,39)
!
  pr(i,40) =                                                               &
    3.0*( + k(i,28) + k(i,111)*HM(i,2) )*QM(i,23) +                        &
    2.0*( + k(i,29) + k(i,112)*HM(i,2) )*QM(i,24) +                        &
    3.0*( + k(i,30) + k(i,113)*HM(i,2) )*QM(i,25) +                        &
    4.0*( + k(i,31) + k(i,114)*HM(i,2) )*QM(i,26) +                        &
    3.0*( + k(i,32) + k(i,115)*HM(i,13) )*QM(i,27) +                       &
        ( + k(i,33) + k(i,116)*HM(i,13) )*QM(i,28) +                       &
        ( + k(i,34) + k(i,117)*HM(i,2) +                                   &
            k(i,118)*HM(i,13) )*QM(i,29)
!
  pr(i,41) =                                                               &
    HM(i,2)*( + k(i,133)*QM(i,30) + k(i,135)*QM(i,31) +                    &
    k(i,136)*QM(i,32) ) + k(i,35)*QM(i,30) + k(i,36)*QM(i,31) +            &
    k(i,37)*QM(i,32)
#endif
!
enddo
!$omp end do
!
!$omp end parallel
!
!
RETURN
END SUBROUTINE CH_PTERMS
