subroutine KRATES(pm,t,hnm,tj,het,k,imax,nphot,nhet,nk)


!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_0 
!
!CC DATE : 15/05/2005
!
!
!   CALCUL DES CONTANTES DE VITESSE DES REACTIONS CHIMIQUES
!
!CCCC
!
!  Original : H. Teyssedre, V.-H. Peuch, 08/2000
!
!  Modified :


 
	IMPLICIT NONE
 
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
 	
      INTEGER :: imax, nphot, nhet, nk
      REAL (KIND=8) , DIMENSION(imax) :: pm,t,hnm
      REAL (KIND=8) , DIMENSION(imax,nphot) :: tj
      REAL (KIND=8) , DIMENSION(imax,nhet) :: het
      REAL (KIND=8) , DIMENSION(imax,nk) :: k
 
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
 
	INTEGER :: I
	REAL (KIND=8) :: XK0,XE0,XN0,XKINF,XEINF,XMINF,XF
	REAL (KIND=8) :: ZK0M,ZKINF,ZFACT,XAEQ,XBEQ
 
!$omp parallel do &
!$omp default(none) &
!$omp shared(imax,t,tj,k,hnm) private(i,xk0,xE0,xn0,xkinf,xEinf,xminf,xF,zk0m,zkinf,zfact)
do i=1,imax
 
      !  NO2  -->  O3P + NO
 
      k(i,1) = tj(i,001)
 
      !  O3  -->  O1D + O2
 
      k(i,2) = tj(i,002)
 
      !  O3  -->  O3P + O2
 
      k(i,3) = tj(i,003)
 
      !  HNO3  -->  OH + NO2
 
      k(i,4) = tj(i,004)
 
      !  HNO4  -->  HO2 + NO2
 
      k(i,5) = tj(i,005)
 
      !  NO3  -->  NO + O2
 
      k(i,6) = tj(i,006)
 
      !  NO3  -->  NO2 + O3P
 
      k(i,7) = tj(i,007)
 
      !  H2O2  -->  OH + OH
 
      k(i,8) = tj(i,008)
 
      !  HCHO  -->  CO + H2
 
      k(i,9) = tj(i,009)
 
      !  HCHO  -->  H + HO2 + CO
 
      k(i,10) = tj(i,010)
 
      !  OP1  -->  CH3O + OH
 
      k(i,11) = tj(i,011)
 
      !  O2  -->  O3P + O3P
 
      k(i,12) = tj(i,012)
 
      !  N2O5  -->  NOx + NOx + XX
 
      k(i,13) = tj(i,013)
 
      !  H2O  -->  H + OH
 
      k(i,14) = tj(i,014)
 
      !  N2O  -->  N2 + O1D
 
      k(i,15) = tj(i,015)
 
      !  HOCl  -->  Cl + OH
 
      k(i,16) = tj(i,016)
 
      !  ClONO2  -->  Cl + NO3
 
      k(i,17) = tj(i,017)
 
      !  ClONO2  -->  ClO + NO2
 
      k(i,18) = tj(i,018)
 
      !  Cl2O2  -->  Cl + Cl + O2
 
      k(i,19) = tj(i,019)
 
      !  HCl  -->  H + Cl
 
      k(i,20) = tj(i,020)
 
      !  Cl2  -->  Cl + Cl
 
      k(i,21) = tj(i,021)
 
      !  ClNO2  -->  Cl + NO2
 
      k(i,22) = tj(i,022)
 
      !  BrO  -->  Br + O3P
 
      k(i,23) = tj(i,023)
 
      !  BrONO2  -->  Br + NO3
 
      k(i,24) = tj(i,024)
 
      !  HOBr  -->  Br + OH
 
      k(i,25) = tj(i,025)
 
      !  OClO  -->  O3P + ClO
 
      k(i,26) = tj(i,026)
 
      !  BrCl  -->  Br + Cl
 
      k(i,27) = tj(i,027)
 
      !  CFC11  -->  ClOx + ClOx + ClOx + XX
 
      k(i,28) = tj(i,028)
 
      !  CFC12  -->  ClOx + ClOx + XX
 
      k(i,29) = tj(i,029)
 
      !  CFC113  -->  ClOx + ClOx + ClOx + XX
 
      k(i,30) = tj(i,030)
 
      !  CCl4  -->  ClOx + ClOx + ClOx + ClOx + XX
 
      k(i,31) = tj(i,031)
 
      !  CH3CCl3  -->  ClOx + ClOx + ClOx + XX
 
      k(i,32) = tj(i,032)
 
      !  CH3Cl  -->  ClOx + XX
 
      k(i,33) = tj(i,033)
 
      !  HCFC22  -->  ClOx + XX
 
      k(i,34) = tj(i,034)
 
      !  CH3Br  -->  BrOx + XX
 
      k(i,35) = tj(i,035)
 
      !  H1211  -->  BrOx + XX
 
      k(i,36) = tj(i,036)
 
      !  H1301  -->  BrOx + XX
 
      k(i,37) = tj(i,037)
 
      !  NO  -->  N + O3P
 
      k(i,38) = tj(i,038)
 
      !  O1D + H2O  -->  OH + OH
 
      k(i,39) = 2.20E-10
 
      !  H + O2  -->  HO2
 
      xk0   = 5.70E-32
      xE0   = 0.0
      xn0   = 1.6
      xkinf = 7.50E-11
      xEinf = 0.0
      xminf = 0.0
      xF    = 0.6
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
      zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
      zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,40) = (zk0m/(1.+zk0m/zkinf))*zfact

      !  O3 + H  -->  OH + O2

      k(i,41) = 1.4e-10*exp(-470./t(i))

      !  O1D + H2  -->  OH + H

      k(i,42) = 1.1e-10

      !  O3P + OH  -->  O2 + H

      k(i,43) = 2.2e-11*exp(120./t(i))

      !  O3 + OH  -->  HO2 + O2

      k(i,44) = 1.60E-12*exp(-(940.0/t(i)))

      !  O3 + HO2  -->  OH + O2 + O2

      k(i,45) = 1.10E-14*exp(-(500.0/t(i)))

      !  O3P + HO2  -->  OH + O2

      k(i,46) = 3.0e-11*exp(200./t(i))

      !  OH + HO2  -->  H2O + O2

      k(i,47) = 4.80E-11*exp(-(-250.0/t(i)))

      !  OH + H2  -->  H2O + H

      k(i,48) = 5.50E-12*exp(-(2000.0/t(i)))

      !  H + HO2  -->  OH + OH

      k(i,49) = 0.9*8.1e-11

      !  H + HO2  -->  H2 + O2

      k(i,50) = 0.08*8.1e-11

enddo
!$omp end parallel do

!$omp parallel do &
!$omp default(none) &
!$omp shared(imax,t,k,hnm,pm) &
!$omp private(i,xk0,xE0,xn0,xkinf,xEinf,xminf,xF,zk0m,zkinf,zfact,xaeq,xbeq)
do i=1,imax

      !  H + HO2  -->  H2O + O3P

      k(i,51) = 0.02*8.1e-11

      !  H2 + O3P  -->  OH + H

      k(i,52) = 8.8e-12*exp(-4200./t(i))

      !  NO + HO2  -->  NO2 + OH

      k(i,53) = 3.5e-12*exp(250./t(i))

      !  HO2 + HO2  -->  H2O2 + O2

      k(i,54) = 2.3E-13*exp(600./t(i))+1.7e-33*exp(1000./t(i))*hnm(i)

      !  H2O2 + OH  -->  HO2 + H2O

      k(i,55) = 2.90E-12*exp(-(160.0/t(i)))

      !  OH + OH  -->  H2O + O3P

      k(i,56) = 4.2e-12*exp(-240./t(i))

      !   OH + OH   -->   H2O2

      xk0   = 6.20E-31
      xE0   = 0.
      xn0   = 1.0
      xkinf = 2.60E-11
      xEinf = 0.
      xminf = 0.0
      xF    = 0.6
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
      zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
      zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,57) = (zk0m/(1.+zk0m/zkinf))*zfact

      !  OH + CO  -->  CO2 + H

      k(i,58) = 1.5e-13*(1.+0.6*pm(i)/101325.)

      !  O3P + NO2 --> NO + O2

      k(i,59) = 5.26e-12*exp(210./t(i))

      !  O3 + NO  -->  NO2 + O2

      k(i,60) = 2.00E-12*exp(-(1400.0/t(i)))

      !  N + NO  -->  N2 + O3P

      k(i,61) = 2.1e-11*exp(100./t(i))

      !  N + O2  -->  NO + O3P

      k(i,62) = 1.5e-11*exp(-3600./t(i))

      !  O3 + NO2  -->  NO3 + O2

      k(i,63) = 1.20E-13*exp(-(2450.0/t(i)))

      !  NO3 + NO2  -->  N2O5

      xk0   = 2.20E-30
      xE0   = 0.0
      xn0   = 3.9
      xkinf = 1.50E-12
      xEinf = 0.0
      xminf = 0.7
      xF    = 0.6
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
      zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
      zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,64) = (zk0m/(1.+zk0m/zkinf))*zfact

      !  OH + NO2 --> HNO3

      xk0   = 2.47E-30
      xE0   = 0.0
      xn0   = 2.97
      xkinf = 1.45E-11
      xEinf = 0.0
      xminf = 2.77
      xF    = 0.6
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
      zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
      zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,65) = (zk0m/(1.+zk0m/zkinf))*zfact

      !  HO2 + NO2  -->  HNO4

      xk0   = 1.80E-31
      xE0   = 0.0
      xn0   = 3.2
      xkinf = 4.70E-12
      xEinf = 0.0
      xminf = 1.4
      xF    = 0.6
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
      zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
      zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,66) = (zk0m/(1.+zk0m/zkinf))*zfact

      !  HNO4  -->  HO2 + NO2

      xk0   = 1.80E-31
      xn0   = 3.2
      xkinf = 4.70E-12
      xminf = 1.4
      xaeq  = 2.10E-27
      xbeq  = 10900.0
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
      zkinf = xkinf*((t(i)/300.)**(-xminf))
      zfact = 0.6**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,67) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact

      !  OH + HNO3  -->  NO3 + H2O

      k(i,68) = (2.4E-14*EXP(460/t(i)))+(6.5E-34*EXP(1335/t(i))* &
      hnm(i))/(1+(6.5E-34*EXP(1335/t(i))*hnm(i))/(2.7E-17*EXP(2199/t(i)))) 

      !  OH + HNO4  -->  NO2 + H2O + O2

      k(i,69) = 1.30E-12*exp(-(-380.0/t(i)))

      !  N2O5  -->  NO2 + NO3

      xk0   = 2.20E-30
      xn0   = 3.9
      xkinf = 1.50E-12
      xminf = 0.7
      xaeq  = 2.70E-27
      xbeq  = 11000.0
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
      zkinf = xkinf*((t(i)/300.)**(-xminf))
      zfact = 0.6**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,70) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact

      !  O1D + N2O  -->  N2 + O2

      k(i,71) = 4.9e-11

      !  O1D + N2O  -->  NO + NO

      k(i,72) = 6.7e-11

      !  O1D + CH4  -->  OH + CH3

      k(i,73) = 0.75*1.5e-10

      !  O1D + CH4  -->  CH3O + H

      k(i,74) = 0.20*1.5e-10

      !  O1D + CH4  -->  HCHO + H2

      k(i,75) = 0.05*1.5e-10

      !  CH4 + OH  -->  CH3 + H2O

      k(i,76) = 2.45e-12*exp(-1775./t(i))

      !  CH3 + O3P  -->  HCHO + H

      k(i,77) = 1.1e-10

      !  CH3 + O2  -->  MO2

      xk0   = 4.50E-31
      xE0   = 0.0
      xn0   = 3.0
      xkinf = 1.80E-12
      xEinf = 0.0
      xminf = 1.7
      xF    = 0.6
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
      zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
      zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,78) = (zk0m/(1.+zk0m/zkinf))*zfact

      !  MO2 + NO  -->  CH3O + NO2

      k(i,79) = 4.2e-12*exp(180./t(i))

      !  MO2 + HO2  -->  OP1 + O2

      k(i,80) = 3.8e-13*exp(800./t(i))

      !  HCHO + OH  -->  HO2 + CO + H2O

      k(i,81) = 1.1e-11

      !  HCHO + O3P  -->  HO2 + CO + OH

      k(i,82) = 3.4e-11*exp(-1600./t(i))

      !  HCHO + Cl  -->  HO2 + CO + HCl

      k(i,83) = 8.1e-11*exp(-30./t(i))

      !  MO2 + MO2  -->  CH3O + CH3O + O2

      k(i,84) = 2.5e-13*exp(190./t(i))

      !  CH3O + O2  -->  HCHO + HO2

      k(i,85) = 3.9e-14*exp(-900./t(i))

      !  OP1 + OH  --> MO2 + H2O

      k(i,86) = 0.7*3.8e-12*exp(200./t(i))

      !  OP1 + OH  --> HCHO + OH + H2O

      k(i,87) = 0.3*3.8e-12*exp(200./t(i))

      !  MO2 + ClO  -->  CH3O + Cl + O2

      k(i,88) = 3.3e-12*exp(-115./t(i))

      !  HCHO + Br  -->  HO2 + CO + HBr

      k(i,89) = 1.7e-11*exp(-800./t(i))

      !  Cl + O3  -->  ClO + O2

      k(i,90) = 2.9e-11*exp(-260./t(i))

      !  ClO + O3P  -->  Cl + O2

      k(i,91) = 3.0e-11*exp(70./t(i))

      !  ClO + NO  -->  NO2 + Cl

      k(i,92) = 6.4e-12*exp(290./t(i))

      !  Cl + CH4  -->  HCl + CH3

      k(i,93) = 1.1e-11*exp(-1400./t(i))

      !  Cl + H2  -->  HCl + H

      k(i,94) = 3.7e-11*exp(-2300./t(i))

      !  Cl + HO2  -->  HCl + O2

      k(i,95) = 1.8e-11*exp(170./t(i))

      !   Cl + HO2   -->   OH + ClO

      k(i,96) = 4.1e-11*exp(-450./t(i))

      !  ClO + OH  -->  Cl + HO2

      k(i,97) = 0.92*1.1e-11*exp(120./t(i))

      !  ClO + OH  -->  HCl + O2

      k(i,98) = 0.08*1.1e-11*exp(120./t(i))

      !  OH + HCl  -->  H2O + Cl

      k(i,99) = 2.6e-12*exp(-350./t(i))

      !  ClO + NO2  -->  ClONO2

      xk0   = 1.80E-31
      xE0   = 0.0
      xn0   = 3.4
      xkinf = 1.50E-11
      xEinf = 0.0
      xminf = 1.9
      xF    = 0.6
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
      zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
      zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,100) = (zk0m/(1.+zk0m/zkinf))*zfact

enddo
!$omp end parallel do

!$omp parallel do &
!$omp default(none) &
!$omp shared(imax,t,k,hnm,het) &
!$omp private(i,xk0,xE0,xn0,xkinf,xEinf,xminf,xF,zk0m,zkinf,zfact,xaeq,xbeq)
do i=1,imax

      !  O3P + ClONO2  -->  ClO + NO3

      k(i,101) = 2.9e-12*exp(-800./t(i))

      !  ClO + HO2  -->  HOCl + O2

      k(i,102) = 4.8e-13*exp(700./t(i))

      !  OH + HOCl  -->  H2O + ClO

      k(i,103) = 3.0e-12*exp(-500./t(i))

      !  O3P + HOCl  -->  OH + ClO

      k(i,104) = 1.7e-13

      !  Cl + NO2  -->  ClNO2

      xk0   = 1.80E-31
      xE0   = 0.0
      xn0   = 2.0
      xkinf = 1.00E-10
      xEinf = 0.0
      xminf = 1.0
      xF    = 0.6
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
      zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
      zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,105) = (zk0m/(1.+zk0m/zkinf))*zfact

      !  Cl + HOCl  -->  OH + Cl2

      k(i,106) = 2.5e-12*exp(-130./t(i))

      !  ClO + ClO  -->  Cl2O2

      xk0   = 2.20E-32
      xE0   = 0.0
      xn0   = 3.1
      xkinf = 3.50E-12
      xEinf = 0.0
      xminf = 1.0
      xF    = 0.6
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
      zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
      zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,107) = (zk0m/(1.+zk0m/zkinf))*zfact

      !  Cl2O2  -->  ClO + ClO

      xk0   = 2.20E-32
      xn0   = 3.1
      xkinf = 3.50E-12
      xminf = 1.0
      xaeq  = 1.30E-27
      xbeq  = 8744.0
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))
      zkinf = xkinf*((t(i)/300.)**(-xminf))
      zfact = 0.6**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,108) = (1./(xaeq))*exp(-xbeq/t(i))*(zk0m/(1.+zk0m/zkinf))*zfact

      !  ClO + ClO  -->  Cl2 + O2

      k(i,109) = 1.0e-12*exp(-1590./t(i))

      !  ClO + ClO  -->  Cl2 + O2

      k(i,110) = 3.0e-11*exp(-2450./t(i))

      !  O1D + CFC11  -->  ClOx + ClOx + ClOx + XX

      k(i,111) = 2.3e-10

      !  O1D + CFC12  -->  ClOx + ClOx + XX

      k(i,112) = 1.4e-10

      !  O1D + CFC113  -->  ClOx + ClOx + ClOx + XX

      k(i,113) = 2.0e-10

      !  O1D + CCl4  -->  ClOx + ClOx + ClOx + ClOx + XX

      k(i,114) = 3.3e-10

      !  OH + CH3CCl3  -->  ClOx + ClOx + ClOx + H2O + XX

      k(i,115) = 1.8e-12*exp(-1550./t(i))

      !  OH + CH3Cl  -->  ClOx + H2O + XX

      k(i,116) = 4.0e-12*exp(-1400./t(i))

      !  O1D + HCFC22  -->  ClOx + XX

      k(i,117) = 1.0e-10

      !  OH + HCFC22  -->  ClOx + H2O + XX

      k(i,118) = 1.0e-12*exp(-1600./t(i))

      !  Br + O3  -->  BrO + O2

      k(i,119) = 1.7e-11*exp(-800./t(i))

      !  BrO + O3P  -->  Br + O2

      k(i,120) = 1.9e-11*exp(230./t(i))

      !  BrO + NO  -->  NO2 + Br

      k(i,121) = 8.8e-12*exp(260./t(i))

      !  BrO + ClO  -->  OClO + Br

      k(i,122) = 1.6e-12*exp(430./t(i))

      !  BrO + ClO  -->  Br + Cl + O2

      k(i,123) = 2.9e-12*exp(220./t(i))

      !  BrO + ClO  -->  BrCl + O2

      k(i,124) = 5.8e-13*exp(170./t(i))

      !  BrO + BrO  -->  Br + Br + O2

      k(i,125) = 1.5e-12*exp(230./t(i))

      !  Br + HO2  -->  HBr + O2

      k(i,126) = 1.5e-11*exp(-600./t(i))

      !  O1D + HBr  -->  OH + Br

      k(i,127) = 1.5e-10

      !  BrO + OH  -->  Br + HO2

      k(i,128) = 1.0e-11

      !  OH + HBr  -->  H2O + Br

      k(i,129) = 1.1e-11

      !  BrO + NO2  -->  BrONO2

      xk0   = 5.20E-31
      xE0   = 0.0
      xn0   = 3.2
      xkinf = 6.90E-12
      xEinf = 0.0
      xminf = 2.9
      xF    = 0.6
      zk0m  = hnm(i)*xk0*((t(i)/300.)**(-xn0))*exp(-xE0/t(i))
      zkinf = xkinf*((t(i)/300.)**(-xminf))*exp(-xEinf/t(i))
      zfact = xF**(1./(1.+(log10(zk0m/zkinf))**2.))
      k(i,130) = (zk0m/(1.+zk0m/zkinf))*zfact

      !  BrO + HO2  -->  HOBr + O2

      k(i,131) = 3.4e-12*exp(540./t(i))

      !  HOBr + O3P  -->  OH + BrO

      k(i,132) = 1.2e-10*exp(-430./t(i))

      !  O1D + CH3Br  -->  BrOx + XX

      k(i,133) = 1.8e-10

      !  OH + CH3Br  -->  CH2Br + H2O

      k(i,134) = 4.0e-10*exp(-1470./t(i))

      !  O1D + H1211  -->  BrOx + XX

      k(i,135) = 1.5e-10

      !  O1D + H1301  -->  BrOx + XX

      k(i,136) = 1.0e-10

      !  O3P + O3P  -->  O2

      k(i,137) = 4.23e-28*hnm(i)/(t(i)*t(i))

      !  O3P + O2  -->  O3

      k(i,138) = hnm(i)*6.00E-34*(t(i)/300)**(-2.3)

      !  O3P + O3  -->  O2 + O2

      k(i,139) = 8.00E-12*exp(-(2060.0/t(i)))

      !  O1D + N2  -->  O3P + N2

      k(i,140) = 1.80E-11*exp(-(-110.0/t(i)))

      !  O1D + O2  -->  O3P + O2

      k(i,141) = 3.20E-11*exp(-(-70.0/t(i)))

      !  ClONO2 + H2O  -->  HOCl + HNO3

      k(i,142) = het(i,1)

      !  ClONO2 + HCl  -->  Cl2 + HNO3

      k(i,143) = het(i,2)

      !  N2O5 + H2O  -->  HNO3 + HNO3

      k(i,144) = het(i,3)

      !  N2O5 + HCl  -->  ClNO2 + HNO3

      k(i,145) = het(i,4)

      !  HOCl + HCl  -->  Cl2 + H2O

      k(i,146) = het(i,5)

      !  BrONO2 + H2O  -->  HOBr + HNO3

      k(i,147) = het(i,6)

      !  HOBr + HCl  -->  BrCl + H2O

      k(i,148) = het(i,7)

      !  HOBr + HBr  -->  BrOx + BrOx + H2O

      k(i,149) = het(i,8)

      !  HOCl + HBr  -->  BrCl + H2O

      k(i,150) = het(i,9)

enddo
!$omp end parallel do

RETURN
END subroutine KRATES
