      SUBROUTINE ADVECSL                                                
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : 3D semi-lagrangian transport scheme
C   Purpose    : Advection of tracers after [Rasch and Williamson, 1987]
C   Called by  : MAIN
C
CCC
C
C  Original : P. Simon, 03/1999
C
C  Modified : V.-H. Peuch              (inclusion dans MOCAGE), 04/1999
C             P. Simon (modif dans la formulation multigrille), 08/1999
C
CCC
C
      USE PARADI_MOD
      USE EXPE_MOD
      USE MOCF_MOD
      USE GRIDS_MOD
      USE POSI_MOD
      USE MRAT_MOD
      USE FORMT_MOD
      USE FORM_MOD
      USE TOPCH_MOD
      USE GAUSS_MOD
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C		
      INTEGER :: NDOM,IDOM,NTRA,NPTR,N,NPTR1,NPTR2,NPTR3,NPTR4
      INTEGER :: NDEB1,NDEB2,NDEB3,NDEB4,JC
      INTEGER :: ILON,ILAT
      REAL (KIND=8) :: CINTA,CINTB
      REAL (KIND=8) , POINTER :: XO(:),YO(:),ZO(:),PO(:)
      REAL (KIND=8) , ALLOCATABLE :: RMAS0(:)
      REAL (KIND=8) , POINTER :: QQQM(:,:,:)
C
      INTEGER , ALLOCATABLE :: INDX(:,:),INDY(:,:),INDZ(:,:),IGRI(:)
      REAL (KIND=8) , ALLOCATABLE :: ALFX1(:,:),ALFX2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: ALFX3(:,:),ALFX4(:,:)
      REAL (KIND=8) , ALLOCATABLE :: ALFY1(:,:),ALFY2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: ALFY3(:,:),ALFY4(:,:)
      REAL (KIND=8) , ALLOCATABLE :: ALFZ1(:,:),ALFZ2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: ALFZ3(:,:),ALFZ4(:,:) 
C
      LOGICAL MONOTRA
      LOGICAL LBEG
C
      DATA LBEG/.TRUE./
      SAVE LBEG
C
      ALLOCATE (RMAS0(NTRA1)) ; RMAS0(:) = 0.0
C	
      MONOTRA=.TRUE.
C
C COMPUTE INITIAL TRACER MASS (GLOBE)
C
      IF (LBEG) THEN
        LBEG=.FALSE.
        IF (IFLAG1) THEN
          CINTB=REAL(NITER-1)/REAL(NITERT)
          CINTA=1.-CINTB
          XPHFM1 = CINTA*XPHFM1A + CINTB*XPHFM1B

          CALL MASSE (LONM1,LATM1,NIVM1,NTRA1
     +               ,DRLONM1,DLATM1,CLAT1,TRANAM1
     +               ,XPHFM1,QQQM1,RMAS0)
        ENDIF
      ELSE
        IF (IFLAG1) THEN
          CALL MASSE (LONM1,LATM1,NIVM1,NTRA1
     +               ,DRLONM1,DLATM1,CLAT1,TRANAM1
     +               ,XPHFM1,QQQM1,RMAS0)
        ENDIF
      ENDIF
C
C COMPUTE BACKTRAJECTORIES
C
      CALL BACKTRA
C
C LOOP ON ACTIVE DOMAINS ONLY
C
      IF(IFLAG1) NDOM=1
      IF(IFLAG2) NDOM=2
      IF(IFLAG3) NDOM=3
      IF(IFLAG4) NDOM=4
C
      ALLOCATE (INDX(NPTS,NDOM)) ; INDX(:,:) = 0
      ALLOCATE (INDY(NPTS,NDOM)) ; INDY(:,:) = 0
      ALLOCATE (INDZ(NPTS,NDOM)) ; INDZ(:,:) = 0
C
      ALLOCATE (IGRI(NPTS)) ; IGRI(:) = 0
C	
      ALLOCATE (ALFX1(NPTS,NDOM)) ; ALFX1(:,:) = 0.0
      ALLOCATE (ALFX2(NPTS,NDOM)) ; ALFX2(:,:) = 0.0
      ALLOCATE (ALFX3(NPTS,NDOM)) ; ALFX3(:,:) = 0.0
      ALLOCATE (ALFX4(NPTS,NDOM)) ; ALFX4(:,:) = 0.0
C	
      ALLOCATE (ALFY1(NPTS,NDOM)) ; ALFY1(:,:) = 0.0
      ALLOCATE (ALFY2(NPTS,NDOM)) ; ALFY2(:,:) = 0.0
      ALLOCATE (ALFY3(NPTS,NDOM)) ; ALFY3(:,:) = 0.0
      ALLOCATE (ALFY4(NPTS,NDOM)) ; ALFY4(:,:) = 0.0
C	
      ALLOCATE (ALFZ1(NPTS,NDOM)) ; ALFZ1(:,:) = 0.0
      ALLOCATE (ALFZ2(NPTS,NDOM)) ; ALFZ2(:,:) = 0.0
      ALLOCATE (ALFZ3(NPTS,NDOM)) ; ALFZ3(:,:) = 0.0
      ALLOCATE (ALFZ4(NPTS,NDOM)) ; ALFZ4(:,:) = 0.0
C
C  Allocate working array
C
      IF (NDOM .ge. 2) THEN
        ALLOCATE (QQQM(NPTS,1,1)) ; QQQM(:,:,:) = 0.0
C
        ALLOCATE (XO(NPTS)) ; XO(:) = 0.0
        ALLOCATE (YO(NPTS)) ; YO(:) = 0.0
        ALLOCATE (ZO(NPTS)) ; ZO(:) = 0.0
        ALLOCATE (PO(NPTS)) ; PO(:) = 0.0
      ENDIF
C
      DO 1 IDOM=NDOM,1,-1
C
C  0. initialisations pour le domaine IDOM
C
        IF (IDOM.EQ.1) THEN
C
          NTRA=NTRA1
          NPTR=NPTS1
          IF (NDOM .ge. 2) THEN
            XO(1:NPTR) = X1O(1:NPTR)
            YO(1:NPTR) = Y1O(1:NPTR)
            ZO(1:NPTR) = Z1O(1:NPTR)
            PO(1:NPTR) = P1O(1:NPTR)
          ELSE
            XO => X1O
            YO => Y1O
            ZO => Z1O
            PO => P1O
          ENDIF
C
        ELSEIF(IDOM.EQ.2) THEN
C
          NTRA=NTRA2
          NPTR=NPTS2
          XO(1:NPTR) = X2O(1:NPTR)
          YO(1:NPTR) = Y2O(1:NPTR)
          ZO(1:NPTR) = Z2O(1:NPTR)
          PO(1:NPTR) = P2O(1:NPTR)
C
        ELSEIF(IDOM.EQ.3) THEN
C
          NTRA=NTRA3
          NPTR=NPTS3
          XO(1:NPTR) = X3O(1:NPTR)
          YO(1:NPTR) = Y3O(1:NPTR)
          ZO(1:NPTR) = Z3O(1:NPTR)
          PO(1:NPTR) = P3O(1:NPTR)
C
        ELSEIF(IDOM.EQ.4) THEN
C
          NTRA=NTRA4
          NPTR=NPTS4
          XO(1:NPTR) = X4O(1:NPTR)
          YO(1:NPTR) = Y4O(1:NPTR)
          ZO(1:NPTR) = Z4O(1:NPTR)
          PO(1:NPTR) = P4O(1:NPTR)
C
        ENDIF
C
C  positionnement par rapport aux differents domaines
C
        CALL POSIREG(XO,YO,IDOM
     +            ,RLONM2(1),RLONM2(LONM2),RLATM2(1),RLATM2(LATM2)
     +            ,RLONM3(1),RLONM3(LONM3),RLATM3(1),RLATM3(LATM3)
     +            ,RLONM4(1),RLONM4(LONM4),RLATM4(1),RLATM4(LATM4)
     +            ,IGRI,NPTR)
C
C  reordonnancement des points selon IGRI
C
        IF (NDOM .EQ. 1) THEN
          NPTR1 = NPTR
          NPTR2 = 0
          NPTR3 = 0
          NPTR4 = 0
        ELSE
          CALL REORPOS(XO,YO,ZO,PO,IGRI,IDOM
     +                ,NPTR,NPTR1,NPTR2,NPTR3,NPTR4)
        ENDIF
C
        IF(NPTR1.NE.0) THEN
C
C  calcul des poids pour l'interpolation
C
C
            CALL POIDREG(XO,RLONM1(1)-DRLONM1,DRLONM1
     +          ,INDX(1,1),ALFX1(1,1),ALFX2(1,1),ALFX3(1,1),ALFX4(1,1)
     +          ,NPTR1)
#ifndef GAUSS
            CALL POIDREG(YO,RLATM1(1)-DRLATM1,DRLATM1
     +          ,INDY(1,1),ALFY1(1,1),ALFY2(1,1),ALFY3(1,1),ALFY4(1,1)
     +          ,NPTR1)
#else
            CALL POIDGAU(YO,XLAT,YCOEF,LATF1
     +          ,INDY(1,1),ALFY1(1,1),ALFY2(1,1),ALFY3(1,1)
     +          ,ALFY4(1,1),NPTR1)
#endif
            CALL POIDZ  (ZO,PO,ALM1,BLM1,NIVM1
     +          ,INDZ(1,1),ALFZ1(1,1),ALFZ2(1,1),ALFZ3(1,1),ALFZ4(1,1)
     +          ,NPTR1)
        ENDIF
C
        IF(NPTR2.NE.0) THEN
C
            NDEB2=1+NPTR1
C
C  calcul des poids pour l'interpolation
C
            CALL POIDREG(XO(NDEB2),RLONM2(1)-DRLONM2,DRLONM2
     +          ,INDX(1,2),ALFX1(1,2),ALFX2(1,2),ALFX3(1,2),ALFX4(1,2)
     +          ,NPTR2)
            CALL POIDREG(YO(NDEB2),RLATM2(1)-DRLATM2,DRLATM2
     +          ,INDY(1,2),ALFY1(1,2),ALFY2(1,2),ALFY3(1,2),ALFY4(1,2)
     +          ,NPTR2)
            CALL POIDZ  (ZO(NDEB2),PO(NDEB2),ALM2,BLM2,NIVM2
     +          ,INDZ(1,2),ALFZ1(1,2),ALFZ2(1,2),ALFZ3(1,2),ALFZ4(1,2)
     +          ,NPTR2)
        ENDIF
C
        IF(NPTR3.NE.0) THEN
C
            NDEB3=1+NPTR1+NPTR2
C
C  calcul des poids pour l'interpolation
C
            CALL POIDREG(XO(NDEB3),RLONM3(1)-DRLONM3,DRLONM3
     +          ,INDX(1,3),ALFX1(1,3),ALFX2(1,3),ALFX3(1,3),ALFX4(1,3)
     +          ,NPTR3)
            CALL POIDREG(YO(NDEB3),RLATM3(1)-DRLATM3,DRLATM3
     +          ,INDY(1,3),ALFY1(1,3),ALFY2(1,3),ALFY3(1,3),ALFY4(1,3)
     +          ,NPTR3)
            CALL POIDZ  (ZO(NDEB3),PO(NDEB3),ALM3,BLM3,NIVM3
     +          ,INDZ(1,3),ALFZ1(1,3),ALFZ2(1,3),ALFZ3(1,3),ALFZ4(1,3)
     +          ,NPTR3)
        ENDIF
C
        IF(NPTR4.NE.0) THEN
C
            NDEB4=1+NPTR1+NPTR2+NPTR3
C
C  calcul des poids pour l'interpolation
C
            CALL POIDREG(XO(NDEB4),RLONM4(1)-DRLONM4,DRLONM4
     +          ,INDX(1,4),ALFX1(1,4),ALFX2(1,4),ALFX3(1,4),ALFX4(1,4)
     +          ,NPTR4)
            CALL POIDREG(YO(NDEB4),RLATM4(1)-DRLATM4,DRLATM4
     +          ,INDY(1,4),ALFY1(1,4),ALFY2(1,4),ALFY3(1,4),ALFY4(1,4)
     +          ,NPTR4)
            CALL POIDZ  (ZO(NDEB4),PO(NDEB4),ALM4,BLM4,NIVM4
     +          ,INDZ(1,4),ALFZ1(1,4),ALFZ2(1,4),ALFZ3(1,4),ALFZ4(1,4)
     +          ,NPTR4)
        ENDIF
C
C  1. traitement des NPTR1 points appartenant au domaine 1
C
        DO JC=1,NTRA1

         IF ((TRANAM1(JC).NE.'H_2SO_4').AND.
     +       (TRANAM1(JC).NE.'PSC')) THEN

          IF (NDOM.eq.1) THEN
              QQQM => QQQM1(:,:,:,JC)
          ELSE
              QQQM = 0.d0
          ENDIF          
          

          IF(NPTR1.NE.0) THEN
C
              NDEB1=1
C
C  interpolation proprement dite (traceur par traceur)
C
#ifdef MIDATM
              CALL INTER3D(QQQM1(1,1,1,JC),QQQM1(1,1,1,JC),
#else
              CALL INTER3D(QQQM1(1,1,1,JC),QQMCHF1(1,1,JC),
#endif
     +                LONM1,LATM1,NIVM1,
     +                INDX(1,1),INDY(1,1),INDZ(1,1),
     +                ALFX1(1,1),ALFX2(1,1),ALFX3(1,1),ALFX4(1,1),
     +                ALFY1(1,1),ALFY2(1,1),ALFY3(1,1),ALFY4(1,1),
     +                ALFZ1(1,1),ALFZ2(1,1),ALFZ3(1,1),ALFZ4(1,1),
     +                QQQM(NDEB1,1,1),NPTR1,+1.,MONOTRA,1,.TRUE.)
C      
          ENDIF
C
C  2. traitement des NPTR2 points appartenant au domaine 2
C
          IF(NPTR2.NE.0) THEN
C
              NDEB2=1+NPTR1
C
C  interpolation proprement dite (traceur par traceur)
C
#ifdef MIDATM
              CALL INTER3D(QQQM2(1,1,1,JC),QQQM2(1,1,1,JC),
#else
              CALL INTER3D(QQQM2(1,1,1,JC),QQMCHF2(1,1,JC),
#endif
     +                LONM2,LATM2,NIVM2,
     +                INDX(1,2),INDY(1,2),INDZ(1,2),
     +                ALFX1(1,2),ALFX2(1,2),ALFX3(1,2),ALFX4(1,2),
     +                ALFY1(1,2),ALFY2(1,2),ALFY3(1,2),ALFY4(1,2),
     +                ALFZ1(1,2),ALFZ2(1,2),ALFZ3(1,2),ALFZ4(1,2),
     +                QQQM(NDEB2,1,1),NPTR2,+1.,MONOTRA,2,.TRUE.)
C
          ENDIF
C
C  3. traitement des NPTR3 points appartenant au domaine 3
C
          IF(NPTR3.NE.0) THEN
C
              NDEB3=1+NPTR1+NPTR2
C
C  interpolation proprement dite (traceur par traceur)
C
#ifdef MIDATM
              CALL INTER3D(QQQM3(1,1,1,JC),QQQM3(1,1,1,JC),
#else
              CALL INTER3D(QQQM3(1,1,1,JC),QQMCHF3(1,1,JC),
#endif
     +                LONM3,LATM3,NIVM3,
     +                INDX(1,3),INDY(1,3),INDZ(1,3),
     +                ALFX1(1,3),ALFX2(1,3),ALFX3(1,3),ALFX4(1,3),
     +                ALFY1(1,3),ALFY2(1,3),ALFY3(1,3),ALFY4(1,3),
     +                ALFZ1(1,3),ALFZ2(1,3),ALFZ3(1,3),ALFZ4(1,3),
     +                QQQM(NDEB3,1,1),NPTR3,+1.,MONOTRA,3,.TRUE.)
C
          ENDIF
C
C  4. traitement des NPTR4 points appartenant au domaine 4
C
          IF(NPTR4.NE.0) THEN
C
              NDEB4=1+NPTR1+NPTR2+NPTR3
C
C  interpolation proprement dite (traceur par traceur)
C
#ifdef MIDATM
              CALL INTER3D(QQQM4(1,1,1,JC),QQQM4(1,1,1,JC),
#else
              CALL INTER3D(QQQM4(1,1,1,JC),QQMCHF4(1,1,JC),
#endif
     +                LONM4,LATM4,NIVM4,
     +                INDX(1,4),INDY(1,4),INDZ(1,4),
     +                ALFX1(1,4),ALFX2(1,4),ALFX3(1,4),ALFX4(1,4),
     +                ALFY1(1,4),ALFY2(1,4),ALFY3(1,4),ALFY4(1,4),
     +                ALFZ1(1,4),ALFZ2(1,4),ALFZ3(1,4),ALFZ4(1,4),
     +                QQQM(NDEB4,1,1),NPTR4,+1.,MONOTRA,4,.TRUE.)
C
          ENDIF
C
C  5. reordonnancement du tableau des traceurs
C
          IF (NDOM .GE. 2) THEN
              IF (IDOM .GE. 2) 
     +            CALL REORCHA(QQQM,IGRI,IDOM
     +            ,NPTR,NPTR1,NPTR2,NPTR3,NPTR4)
              IF(IDOM.EQ.1) THEN
                  DO N=1,LOLANIM1
                    QQQM1(N,1,1,JC)=QQQM(N,1,1)
                  ENDDO
              ELSEIF(IDOM.EQ.2) THEN
                  DO N=1,LOLANIM2
                    QQQM2(N,1,1,JC)=QQQM(N,1,1)
                  ENDDO
              ELSEIF(IDOM.EQ.3) THEN
                  DO N=1,LOLANIM3
                    QQQM3(N,1,1,JC)=QQQM(N,1,1)
                  ENDDO
              ELSEIF(IDOM.EQ.4) THEN
                  DO N=1,LOLANIM4
                    QQQM4(N,1,1,JC)=QQQM(N,1,1)
                  ENDDO
              ENDIF
          ENDIF
C
         ENDIF
C
        ENDDO
C	
 1    CONTINUE
C
C  6. Correction des rapports de melange negatifs
C     (MONOTRA=.FALSE.)
C
      CALL CORREC(RMAS0)
#ifndef MIDATM
C
      IF (IFLAG1) THEN
        DO JC=1,NTRA1
          DO ILON=1,LONM1
            DO ILAT=1,LATM1
              QQQM1(ILON,ILAT,1,JC)=QQMCHF1(ILON,ILAT,JC)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
      IF (IFLAG2) THEN
        DO JC=1,NTRA2
          DO ILON=1,LONM2
            DO ILAT=1,LATM2
              QQQM2(ILON,ILAT,1,JC)=QQMCHF2(ILON,ILAT,JC)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
      IF (IFLAG3) THEN
        DO JC=1,NTRA3
          DO ILON=1,LONM3
            DO ILAT=1,LATM3
              QQQM3(ILON,ILAT,1,JC)=QQMCHF3(ILON,ILAT,JC)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
      IF (IFLAG4) THEN
        DO JC=1,NTRA4
          DO ILON=1,LONM4
            DO ILAT=1,LATM4
              QQQM4(ILON,ILAT,1,JC)=QQMCHF4(ILON,ILAT,JC)
            ENDDO
          ENDDO
        ENDDO
      ENDIF
#endif
C
      IF (NDOM .ge. 2) THEN
        DEALLOCATE (XO)
        DEALLOCATE (YO)
        DEALLOCATE (ZO)
        DEALLOCATE (PO)
      ELSE
        NULLIFY(XO)
        NULLIFY(YO)
        NULLIFY(ZO)
        NULLIFY(PO)
      ENDIF
C
      DEALLOCATE (RMAS0)
      DEALLOCATE (INDX)
      DEALLOCATE (INDY)
      DEALLOCATE (INDZ)
      DEALLOCATE (IGRI)
C	
      DEALLOCATE (ALFX1)
      DEALLOCATE (ALFX2)
      DEALLOCATE (ALFX3)
      DEALLOCATE (ALFX4)
C	
      DEALLOCATE (ALFY1)
      DEALLOCATE (ALFY2)
      DEALLOCATE (ALFY3)
      DEALLOCATE (ALFY4)
C	
      DEALLOCATE (ALFZ1)
      DEALLOCATE (ALFZ2)
      DEALLOCATE (ALFZ3)
      DEALLOCATE (ALFZ4)
C
      IF (NDOM .ge. 2) THEN
        DEALLOCATE (QQQM)
      ELSE
        NULLIFY (QQQM) 
      ENDIF
C
      RETURN
      END
C
CCC
C
