      SUBROUTINE ANAWAT(QQQM,QGRIM,TGRIM,XPAM,LON,LAT,NIV,NTRA,
     +                  iH2O)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Physical parameterizations
C   Purpose    : Update tropospheric water vapour distribution
C                according to specific humidity analyses
C   Called by  :
C
CCC
C
C  Original : H. Teyssedre, V.-H. Peuch, 02/2001
C
C  Modified : H. Le Berre 09/2006 : Correction pour eviter une division par 0 (DELTANIV)
C
CCC
C
      USE CSTES_MOD
C
      IMPLICIT NONE
C	
#ifdef MIDATM
#ifdef NIV_91
#define CPP_NIVBAS 65
#define CPP_NIVHAU 56
#endif
#ifdef NIV_60
#define CPP_NIVBAS 40
#define CPP_NIVHAU 23
#endif
#else
#define CPP_NIVBAS 30
#define CPP_NIVHAU 15
#endif
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: LON,LAT,NIV,NTRA,iH2O
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV,NTRA) :: QQQM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: QGRIM,TGRIM,XPAM
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C	
      INTEGER :: NIVHAU,NIVBAS
      INTEGER :: ILON,ILAT,INIV
      REAL (KIND=8) :: XMCONV,DELTANIV,X,Z1H2O,Z2H2O
      INTEGER , ALLOCATABLE :: NIVTOP(:),NIVBOT(:)
      REAL (KIND=8) , ALLOCATABLE :: XHUM(:,:,:),ZHUM(:,:)
      REAL (KIND=8) , ALLOCATABLE :: XTHETA(:,:,:)
      PARAMETER (NIVHAU=CPP_NIVHAU, NIVBAS=CPP_NIVBAS)
      REAL (KIND=8) :: THTROPO,PVTROPO
      PARAMETER (THTROPO=380., PVTROPO=2.0E-06)
      LOGICAL, ALLOCATABLE :: LTROP(:,:,:)
C
      ALLOCATE (NIVTOP(LAT))
      ALLOCATE (NIVBOT(LAT))
      ALLOCATE (XHUM(LON,LAT,NIV))
      ALLOCATE (ZHUM(LAT,NIV)) ; ZHUM(:,:) = 0.0
#ifdef MIDATM
      ALLOCATE (LTROP (LON,LAT,NIV))
      ALLOCATE (XTHETA(LON,LAT,NIV))
#endif
C
C CONVERT SPECIFIC HUMIDITY TO MIXING RATIO
C
      XMCONV=29./18.
      DO ilon=1,LON*LAT*NIV
        X = MAX(QGRIM(ilon,1,1),1.e-10)
        XHUM(ilon,1,1) = XMCONV*X/(1.-X)
      ENDDO
C
#ifdef MIDATM      
C
C POTENTIAL TEMPERATURE
C
      DO ilon=1,LON*LAT*NIV
        XTHETA(ilon,1,1)=TGRIM(ilon,1,1)*
     +                   (1.E+05/XPAM(ilon,1,1))**0.288
      ENDDO
C
C TROPOSPHERE DETECTION
C
      DO ilon=1,LON*LAT*NIV
        LTROP(ilon,1,1)=.FALSE.
      ENDDO

      DO ilon=1,LON*LAT*NIV
        IF (XTHETA(ilon,1,1).LE.THTROPO) THEN
          LTROP(ilon,1,1)=.TRUE.
        ENDIF
      ENDDO
      DO iniv=1,NIV
        DO ilon=1,LON*LAT
          IF (LTROP(ilon,1,iniv)) THEN
C
C TROPOSPHERE : PRESCRIBE ANALYSES
C
            QQQM(ilon,1,iniv,iH2O) = XHUM(ilon,1,iniv)
          ENDIF
        ENDDO
      ENDDO
#else
C
C COMPUTE LONGITUDINAL MEAN
C
      DO iniv=1,NIV
      DO ilat=1,LAT
!       ZHUM(ilat,iniv)=0.
        DO ilon=1,LON
          ZHUM(ilat,iniv) = ZHUM(ilat,iniv) +
     +              XHUM(ilon,ilat,iniv)/REAL(LON)
        ENDDO
      ENDDO
      ENDDO
C
C LOOK FOR 10 ppmv LEVEL FOR SPECIFIC HUMIDITY
C
      DO ilat=1,LAT
C
        NIVTOP(ilat)=NIVHAU-1
        NIVBOT(ilat)=NIVHAU
C
        DO iniv=NIVBAS,NIVHAU+1,-1
          IF (ZHUM(ilat,iniv).LT.1.E-5) THEN
            NIVBOT(ilat) = MIN(iniv  ,NIVBAS)
            NIVTOP(ilat) = MAX(iniv-1,NIVHAU)
            GOTO 10
          ENDIF
        ENDDO
C
 10     CONTINUE
C
      ENDDO
C
      DO ilat=1,LAT
C
        DELTANIV=REAL(NIVBOT(ilat)-NIVTOP(ilat))
C
        DO iniv=1,NIV
C
          IF (iniv.lt.NIVTOP(ilat)) THEN
            Z1H2O=1.0
            Z2H2O=0.0
          ELSEIF (iniv.gt.NIVBOT(ilat)) THEN
            Z1H2O=0.0
            Z2H2O=1.0
          ELSE
            Z1H2O=REAL(NIVBOT(ilat)-iniv)/DELTANIV
            Z2H2O=1.-Z1H2O
          ENDIF
C
          DO ilon=1,LON
            QQQM(ilon,ilat,iniv,iH2O) =
     +             Z1H2O*QQQM(ilon,ilat,iniv,iH2O)
     +           + Z2H2O*XHUM(ilon,ilat,iniv)
          ENDDO
C
        ENDDO
C
      ENDDO
#endif
C
      DEALLOCATE (NIVTOP)
      DEALLOCATE (NIVBOT)
      DEALLOCATE (XHUM)
      DEALLOCATE (ZHUM)
#ifdef MIDATM
      DEALLOCATE (XTHETA)
      DEALLOCATE (LTROP)
#endif
C      
      RETURN
      END
