      SUBROUTINE ASSIMELT(NTHETA,NEL,XELTIN,XELTOUT)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Data assimilation
C   Purpose    : Perform data assimilation in (EL,theta) coordinates
C                [Cathala, Pailleux and Peuch, Tellus, 2003]
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 01/2002
C
C Modified  :
C
CCC
C	
	USE PARADI_MOD
	USE DATOBS_MOD
	USE THELOBS_MOD
C  
	IMPLICIT NONE	
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: NTHETA,NEL
	REAL (KIND=8) , DIMENSION (NEL,NTHETA) :: XELTIN,XELTOUT
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: NSUPOBSMAX
	INTEGER :: ITH,IEL,I,L,INI,NTEST,IELOBS,NSOBS
	REAL (KIND=8)  :: DELREF1,DELREF2
	REAL (KIND=8)  :: THETAMAX,DELTAEL
C
	PARAMETER(nsupobsmax=3000)
      PARAMETER(delREF1=10.,delREF2=5.)	
C      
	INTEGER , ALLOCATABLE :: NBobsELT(:,:)
      INTEGER , ALLOCATABLE :: NBsobs(:)
	INTEGER , ALLOCATABLE :: iTHinf(:),iELinf(:)
	REAL (KIND=8) , ALLOCATABLE :: OZEQ(:)
      REAL (KIND=8) , ALLOCATABLE :: NTH(:)      
      REAL (KIND=8) , ALLOCATABLE :: sobsELT(:,:),THsobsELT(:,:)
      REAL (KIND=8) , ALLOCATABLE :: ELsobsELT(:,:)
      REAL (KIND=8) , ALLOCATABLE :: sobs(:),THsobs(:),ELsobs(:)      
      REAL (KIND=8) , ALLOCATABLE :: xth(:),xel(:)
      REAL (KIND=8) , ALLOCATABLE :: xdd(:),xdg(:),somg(:)
C
CCC 0. INITIALIZATIONS
C
	ALLOCATE (OZEQ(NEL))
	ALLOCATE (NTH(NTHETA))
	ALLOCATE (NBobsELT(NEL,NTHETA))
	ALLOCATE (NBsobs(nsupobsmax)) ; NBsobs(:) = 0
	ALLOCATE (sobsELT(NEL,NTHETA))
	ALLOCATE (THsobsELT(NEL,NTHETA))
	ALLOCATE (ELsobsELT(NEL,NTHETA))
	ALLOCATE (sobs(nsupobsmax)) ; sobs(:) = 0.0
	ALLOCATE (THsobs(nsupobsmax)) ; THsobs(:) = 0.0
	ALLOCATE (ELsobs(nsupobsmax)) ; ELsobs(:) = 0.0
	ALLOCATE (iTHinf(nsupobsmax)) ; iTHinf(:) = 0
	ALLOCATE (iELinf(nsupobsmax)) ; iELinf(:) = 0
	ALLOCATE (xth(nsupobsmax)) ; xth(:) = 0.0
	ALLOCATE (xel(nsupobsmax)) ; xel(:) = 0.0
	ALLOCATE (xdd(nsupobsmax)) ; xdd(:) = 0.0
	ALLOCATE (xdg(nsupobsmax)) ; xdg(:) = 0.0
	ALLOCATE (somg(nsupobsmax)) ; somg(:) = 0.0
C
      DO ith=1,NTHETA
        NTH(ith)=THETAMIN+REAL(ith-1)*DELTATH
      ENDDO
      THETAMAX=NTH(NTHETA)
C
      DELTAEL=REAL(180./NEL)
      DO iEL=1,NEL
        OZEQ(iEL)=(iEL-0.5)*DELTAEL-90.
      ENDDO
C
CCC 1. BUILD SUPEROBSERVATIONS
C
      DO i=1,NEL*NTHETA
         NBobsELT(i,1)=0
         sobsELT(i,1)=0.
         THsobsELT(i,1)=0.
         ELsobsELT(i,1)=0.
      ENDDO
C
      DO l=1,nobs
      IF (typemesure(l).EQ.'FLIGHT') THEN
      DO ini=1,nbnivpres(l)
        ntest=0
        iELobs=MIN(1+INT((ELobs(l,ini)+90.)*REAL(NEL)/180.),NEL)
        DO ith=NTHETA,2,-1
          IF ((ntest.EQ.0).AND.
     +        (THobs(l,ini).LE.(THETAMAX+DELTATH/2.)).AND.
     +        (THobs(l,ini).GT.(NTH(ith-1)+NTH(ith))/2.).AND.
     +        (val(l,ini,1).NE.-999.)) THEN
            NBobsELT(iELobs,ith)=NBobsELT(iELobs,ith)+1
            sobsELT(iELobs,ith)=sobsELT(iELobs,ith)+val(l,ini,1)
            THsobsELT(iELobs,ith)=THsobsELT(iELobs,ith)+THobs(l,ini)
            ELsobsELT(iELobs,ith)=ELsobsELT(iELobs,ith)+ELobs(l,ini)
            ntest=1
            IF ((ith.EQ.2).AND.(ntest.EQ.0).AND.
     +          (THobs(l,ini).GE.(THETAMIN-DELTATH/2.))) THEN
              NBobsELT(iELobs,1)=NBobsELT(iELobs,1)+1
              sobsELT(iELobs,1)=sobsELT(iELobs,1)+val(l,ini,1)
              THsobsELT(iELobs,1)=THsobsELT(iELobs,1)+THobs(l,ini)
              ELsobsELT(iELobs,1)=ELsobsELT(iELobs,1)+ELobs(l,ini)
            ENDIF
          ENDIF
        ENDDO
      ENDDO
      ENDIF
      ENDDO
C
      Nsobs=0
      DO i=1,NEL*NTHETA
        IF (NBobsELT(i,1).NE.0) THEN
          Nsobs=Nsobs+1
          sobs(Nsobs)=sobsELT(i,1)/NBobsELT(i,1)
          THsobs(Nsobs)=THsobsELT(i,1)/NBobsELT(i,1)
          ELsobs(Nsobs)=ELsobsELT(i,1)/NBobsELT(i,1)
          NBsobs(Nsobs)=NBobsELT(i,1)
        ENDIF
      ENDDO
C
CCC 2. COMPUTE GUESS ON SUPEROBSERVATIONS LOCATIONS
C
      DO l=1,Nsobs
C
        DO i=1,NTHETA
          IF (THsobs(l).GE.NTH(i)) iTHinf(l)=i
        ENDDO
        IF (iTHinf(l).LT.NTHETA) THEN
          xth(l)=(NTH(iTHinf(l))-THsobs(l))/
     +           (NTH(iTHinf(l))-NTH(iTHinf(l)+1))
        ENDIF
C
        iELinf(l)=1
        DO i=1,NEL
          IF (ELsobs(l).GE.OZEQ(i)) iELinf(l)=i
        ENDDO
        IF (iELinf(l).LT.NEL) THEN
          xel(l)=(OZEQ(iELinf(l))-ELsobs(l))/
     +           (OZEQ(iELinf(l))-OZEQ(iELinf(l)+1))
        ENDIF
C
        CALL ELSOBSGUESS(NTHETA,NEL,sobs(l),XELTIN,
     +                   iTHinf(l),iELinf(l),
     +                   xel(l),xth(l),somg(l))
C
        CALL ELSOBSADAPT(NTHETA,NEL,XELTIN,iTHinf(l),iELinf(l),
     +                 xel(l),xth(l),xdd(l),xdg(l))
C
        IF (xdg(l).LT.(-delREF1)) xdg(l)=-delREF1
        IF (xdd(l).GT.delREF1) xdd(l)=delREF1
      ENDDO
C
CCC 3. PERFORM A FIRST ASSIMILATION OF SUPEROBSERVATIONS (LONG RANGE)
C
      CALL ELSOBSASSIM(Nsobs,nsupobsmax,NTHETA,NEL,delREF1,
     +           THsobs,ELsobs,somg,xdg,xdd,
     +           XELTIN,XELTOUT)
C
CCC 4. COMPUTE NEW GUESS ON SUPEROBSERVATIONS LOCATIONS
C
      DO l=1,Nsobs
        CALL ELSOBSGUESS(NTHETA,NEL,sobs(l),XELTOUT,
     +                   iTHinf(l),iELinf(l),
     +                   xel(l),xth(l),somg(l))
      ENDDO
C
CCC 5. PERFORM A SECOND ASSIMILATION OF SUPEROBSERVATIONS (REDUCED RANGE)
C
      DO l=1,Nsobs
        IF (xdg(l).LT.(-delREF2)) xdg(l)=-delREF2
        IF (xdd(l).GT.delREF2) xdd(l)=delREF2
      ENDDO
      DO i=1,NEL*NTHETA
        XELTIN(i,1)=XELTOUT(i,1)
      ENDDO
C
      CALL ELSOBSASSIM(Nsobs,nsupobsmax,NTHETA,NEL,delREF2,
     +                 THsobs,ELsobs,somg,xdg,xdd,
     +                 XELTIN,XELTOUT)
C
	DEALLOCATE (OZEQ)
	DEALLOCATE (NTH)
	DEALLOCATE (NBobsELT)
	DEALLOCATE (NBsobs)
	DEALLOCATE (sobsELT)
	DEALLOCATE (THsobsELT)
	DEALLOCATE (ELsobsELT)
	DEALLOCATE (sobs)
	DEALLOCATE (THsobs)
	DEALLOCATE (ELsobs)
	DEALLOCATE (iTHinf)
	DEALLOCATE (iELinf)
	DEALLOCATE (xth)
	DEALLOCATE (xel)
	DEALLOCATE (xdd)
	DEALLOCATE (xdg)
	DEALLOCATE (somg)
C
C
      RETURN
      END
