SUBROUTINE BUDGET(ZCUMUL,XPHFM,QQQM,RPLMN,LON,LAT,NIV,NTRA)
!
!
!!!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!!! DATE : 15/03/2006 / TEMPO, VERSION ACCIDENT BASEE SUR 0_1_0
!
!   Type      : Diagnostics
!   Purpose   : Compute 2D Budgets
!   Called by : IOPC3D 
!
!
!
!  Original : V.-H. Peuch, 03/2006
!
!  Modified : 
!
!
!
IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!!! INPUT
!
INTEGER :: LON,LAT,NIV,NTRA
REAL (KIND=8) :: RPLMN
REAL (KIND=8), DIMENSION (LON,LAT,NIV,NTRA) :: QQQM
REAL (KIND=8), DIMENSION (LON,LAT,NIV)      :: XPHFM
!
! LON,LAT,NIV,NTRA : dimensions for QQQM and XPHFM
! RPLMN            : sign for accumulation (+1./-1.)
! QQQM             : tracers mixing ratios
! XPHFM            : inter-level pressure
!
!!! INPUT/OUTPUT
!
REAL (KIND=8), DIMENSION(LON,LAT,NTRA) :: ZCUMUL
!
! ZCUMUL : field for accumulation
!
!     Local Variables
!     ~~~~~~~~~~~~~~~
!
INTEGER :: I,L,LM1,itra
REAL (KIND=8) , ALLOCATABLE :: VAL(:,:,:),DELTAP(:,:,:)
!
! 
ALLOCATE (VAL(LON,LAT,NTRA))
ALLOCATE (DELTAP(LON,LAT,NIV))
!
VAL(:,:,:)=0.
!
DO L=1,NIV
  LM1=MAX(L-1,1)
  DO I=1,LON*LAT
    DELTAP(I,1,L)=XPHFM(I,1,L)-XPHFM(I,1,LM1)
  ENDDO
ENDDO   
! 
!$omp parallel do default(none), &
!$omp private (ITRA,I,L), &
!$omp shared (LON,LAT,NIV,NTRA,RPLMN,VAL,DELTAP,QQQM,ZCUMUL)
DO ITRA=1,NTRA
!
  DO L=1,NIV
    DO I=1,LON*LAT    
      VAL(I,1,ITRA)=VAL(I,1,ITRA)+DELTAP(I,1,L)*QQQM(I,1,L,ITRA)
    ENDDO
  ENDDO
!
  DO I=1,LON*LAT
    ZCUMUL(I,1,ITRA)=ZCUMUL(I,1,ITRA)+RPLMN*VAL(I,1,ITRA)
  ENDDO
!
ENDDO

!$omp end parallel do
!
!
ZCUMUL(:,:,:)=MAX( 0.,ZCUMUL(:,:,:) )
DEALLOCATE (VAL)
DEALLOCATE (DELTAP)
!
!
RETURN
END
