      SUBROUTINE CGUESS(G1,G2,G3,G4,ILOBS,nlobs)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Data Assimilation
C   Purpose    : Read observational data for model diagnostic
C                and data assimilation purposes
C   Called by  :
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 03/1999
C
C Modified  : V.-H. Peuch, CNRM/GMGEC/ERAM, 01/2001
C             A. Dufour & V.-H. Peuch, CNRM/GMGEC/ERAM, 07/2001
C
CCC
C
      USE PARADI_MOD
      USE EXPE_MOD
      USE MOCF_MOD
      USE DATOBS_MOD
      USE FORMT_MOD
      USE MRAT_MOD
      USE PHOTO_MOD
      USE SVPHOT_MOD
C
      IMPLICIT NONE
C
CAP juin 2005
C
C Pour le moment on compare O_x (transporte) avec O_3 observe
C Si necessaire, prochainement on ajoutera une unite de conversion
C O_x -> O_3 (son LT et son ADJ) et l'on controlera O_x tout en 
C observant O_3
C
#ifdef MIDATM
#ifdef PALM_RESEARCH
#ifdef CARIOLLE
#define CPP_O3 'O_3'
#else
#define CPP_O3 'O_x'
#endif
#else
#define CPP_O3 'O_3'
#endif
#else
#define CPP_O3 'O_x'
#endif
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
      INTEGER :: nlobs
      INTEGER , DIMENSION(nlobs) :: ILOBS
      REAL (KIND=8) , INTENT(OUT), 
     & DIMENSION (nobstspmax,nbnivpresmax,nbconstmax) :: G1,G2,G3,G4
C   
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
C NB
      INTEGER :: I,IH2O,ICO,ICO3,INO,INO2,INO3,IN2O5,IHCHO,IKET,IETH
      INTEGER :: IO3,IHNO3,IISO,IH2O2,IOBS,LOBS,ILON,ICST,NBCONSTOBS
C NB
      INTEGER :: INOX,IHONO,IHNO4,IONIT,IOLNN,ICLNO2,ICLONO2,IBRONO2
      INTEGER :: IPAN,ITPAN,ISO2,IINDUV2,IINDUVCS2
      INTEGER :: IUVA2,IUVB2,ITRA,KO3,IS02,ISLS
      INTEGER :: ICOVA,ICOVB,IOH,INOY
      REAL (KIND=8) :: XH2O,XO3,XCO,XNO,XNO2,XNO3,XHNO3,XN2O5
C NB  
      REAL (KIND=8) :: XHCHO,XKET,XETH
      REAL (KIND=8) :: XNOX,XHONO,XHNO4,XONIT,XOLNN,XCLNO2,XH2O2,XISO
C NB
      REAL (KIND=8) :: XCLONO2,XBRONO2
      REAL (KIND=8) :: XPAN,XTPAN,XSO2,XINDUV,XINDUVCS,XUVA,XUVB
      REAL (KIND=8) :: XCOVA,XCOVB,XOH,XNOY
C
      REAL (KIND=8) , ALLOCATABLE :: GRID1(:,:,:,:),GRID2(:,:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: GRID3(:,:,:,:),GRID4(:,:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: P0(:)
      REAL (KIND=8) , ALLOCATABLE :: XINTER1(:,:),XINTER2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: XINTER3(:,:),XINTER4(:,:) 
      CHARACTER(len=5), ALLOCATABLE :: cstname(:)
      LOGICAL :: ll_select
C
      ALLOCATE (GRID1(LONM1,LATM1,NIVM1,nbconstmax)) ;
     $    GRID1(:,:,:,:)=0.0
      ALLOCATE (cstname(nbconstmax)) ; cstname(:) = 'xxxxx'
      ALLOCATE (P0(nbnivpresmax)) ; P0(:) = 0.0
      ALLOCATE (XINTER1(nbnivpresmax,nbconstmax)) ; XINTER1(:,:) = 0.0
      IF (IFLAG2) THEN
          ALLOCATE (GRID2(LONM2,LATM2,NIVM2,nbconstmax)) ;
     $        GRID2(:,:,:,:)=0.0
        ALLOCATE (XINTER2(nbnivpresmax,nbconstmax)) ; XINTER2(:,:) = 0.0
      ENDIF
      IF (IFLAG3) THEN
      ALLOCATE (GRID3(LONM3,LATM3,NIVM3,nbconstmax)) ;
     $        GRID3(:,:,:,:)=0.0
        ALLOCATE (XINTER3(nbnivpresmax,nbconstmax)) ; XINTER2(:,:) = 0.0
      ENDIF
      IF (IFLAG4) THEN
          ALLOCATE (GRID4(LONM4,LATM4,NIVM4,nbconstmax)) ;
     $        GRID4(:,:,:,:)=0.0
        ALLOCATE (XINTER4(nbnivpresmax,nbconstmax)) ; XINTER2(:,:) = 0.0
      ENDIF
C
C RESET GUESS FIELDS FOR ALL DOMAINS
C
      DO i=1,nobstspmax*nbnivpresmax*nbconstmax
        G1(i,1,1)=0.
        IF (IFLAG2) G2(i,1,1)=0.
        IF (IFLAG3) G3(i,1,1)=0.
        IF (IFLAG4) G4(i,1,1)=0.
      ENDDO
C
      iH2O  = 0
      iCO   = 0
      iO3   = 0
      iNOx  = 0
      iNO   = 0
      iNO2  = 0
      iNO3  = 0
      iHONO = 0
      iHNO3 = 0
      iHNO4 = 0
      iN2O5 = 0
      iPAN  = 0
      iTPAN = 0
      iONIT = 0
      iOLNN = 0
      iSO2  = 0
      iClNO2  = 0
      iClONO2 = 0
      iBrONO2 = 0
      iINDUV2 = iINDUV
      iINDUVCS2 = iINDUVCS
      iUVA2 = iUVA
      iUVB2 = iUVB
      iCOVA = 0
      iCOVB = 0
      iOH   = 0
      iNOy  = 0
C  NB 
      iHCHO = 0
      iH2O2 = 0
      iETH= 0
      iISO= 0
      iKET= 0
C NB 
C
      DO itra=1,NTRA1
        IF (TRANAM1(itra).EQ.'H_2O')   iH2O=itra
        IF (TRANAM1(itra).EQ.CPP_O3) THEN
            iO3 =itra
            kO3 = 1
        END IF
        IF (TRANAM1(itra).EQ.'NO_x')    iNOx    =itra
        IF (TRANAM1(itra).EQ.'CO')      iCO     =itra
        IF (TRANAM1(itra).EQ.'HONO')    iHONO   =itra
        IF (TRANAM1(itra).EQ.'HNO_3')   iHNO3   =itra
        IF (TRANAM1(itra).EQ.'HNO_4')   iHNO4   =itra
        IF (TRANAM1(itra).EQ.'N_2O_5')  iN2O5   =itra
        IF (TRANAM1(itra).EQ.'PAN')     iPAN    =itra
        IF (TRANAM1(itra).EQ.'TPAN')    iTPAN   =itra
        IF (TRANAM1(itra).EQ.'ONIT')    iONIT   =itra
        IF (TRANAM1(itra).EQ.'OLNN')    iOLNN   =itra
        IF (TRANAM1(itra).EQ.'SO_2')    iS02    =itra
        IF (TRANAM1(itra).EQ.'ClNO_2')  iClNO2  =itra
        IF (TRANAM1(itra).EQ.'ClONO_2') iClONO2 =itra
        IF (TRANAM1(itra).EQ.'BrONO_2') iBrONO2 =itra
C NB 
        IF (TRANAM1(itra).EQ.'HCHO')   iHCHO=itra
        IF (TRANAM1(itra).EQ.'ETH')   iETH =itra
        IF (TRANAM1(itra).EQ.'ISO')   iISO =itra
        IF (TRANAM1(itra).EQ.'H_2O_2')   iH2O2 =itra
        IF (TRANAM1(itra).EQ.'KET')   iKET =itra
C NB

      ENDDO
C
      DO isls=1,NSLS1
        IF (SLSNAM1(isls).EQ.CPP_O3) THEN
            iO3 =isls
            kO3 = 2
        END IF
        IF (SLSNAM1(isls).EQ.'NO')     iNO  =isls
        IF (SLSNAM1(isls).EQ.'NO_2')   iNO2 =isls
        IF (SLSNAM1(isls).EQ.'NO_3')   iNO3 =isls
        IF (SLSNAM1(isls).EQ.'OH')     iOH  =isls
        IF (SLSNAM1(isls).EQ.'NOy')    iNOy =isls
        IF (SLSNAM1(isls).EQ.'COVpA')  iCOVA=isls
        IF (SLSNAM1(isls).EQ.'COVpB')  iCOVB=isls
      ENDDO
C
      IF (iH2O.EQ.0) THEN
        iH2O=1
        xH2O=0.
      ELSE
        xH2O=1.
      ENDIF
C
      IF (iO3.EQ.0) THEN
        iO3=1
        kO3=1
        xO3=0.
      ELSE
        xO3=1.
      ENDIF
C
      IF (iCO.EQ.0) THEN
        iCO=1
        xCO=0.
      ELSE
        xCO=1.
      ENDIF
C
      IF (iNOx.EQ.0) THEN
        iNOx=1
        xNOx=0.
      ELSE
        xNOx=1.
      ENDIF
C
      IF (iNO.EQ.0) THEN
        iNO=1
        xNO=0.
      ELSE
        xNO=1.
      ENDIF
C
      IF (iNO2.EQ.0) THEN
        iNO2=1
        xNO2=0.
      ELSE
        xNO2=1.
      ENDIF
C
      IF (iNO3.EQ.0) THEN
        iNO3=1
        xNO3=0.
      ELSE
        xNO3=1.
      ENDIF
C
      IF (iHONO.EQ.0) THEN
        iHONO=1
        xHONO=0.
      ELSE
        xHONO=1.
      ENDIF
C
      IF (iHNO3.EQ.0) THEN
        iHNO3=1
        xHNO3=0.
      ELSE
        xHNO3=1.
      ENDIF
C
      IF (iHNO4.EQ.0) THEN
        iHNO4=1
        xHNO4=0.
      ELSE
        xHNO4=1.
      ENDIF
C
      IF (iN2O5.EQ.0) THEN
        iN2O5=1
        xN2O5=0.
      ELSE
        xN2O5=1.
      ENDIF
C
      IF (iPAN.EQ.0) THEN
        iPAN=1
        xPAN=0.
      ELSE
        xPAN=1.
      ENDIF
C
      IF (iTPAN.EQ.0) THEN
        iTPAN=1
        xTPAN=0.
      ELSE
        xTPAN=1.
      ENDIF
C
      IF (iONIT.EQ.0) THEN
        iONIT=1
        xONIT=0.
      ELSE
        xONIT=1.
      ENDIF
C
      IF (iOLNN.EQ.0) THEN
        iOLNN=1
        xOLNN=0.
      ELSE
        xOLNN=1.
      ENDIF
C
      IF (iClNO2.EQ.0) THEN
        iClNO2=1
        xClNO2=0.
      ELSE
        xClNO2=1.
      ENDIF
C
      IF (iClONO2.EQ.0) THEN
        iClONO2=1
        xClONO2=0.
      ELSE
        xClONO2=1.
      ENDIF
C
      IF (iBrONO2.EQ.0) THEN
        iBrONO2=1
        xBrONO2=0.
      ELSE
        xBrONO2=1.
      ENDIF
C
      IF (iSO2.EQ.0) THEN
        iSO2=1
        xSO2=0.
      ELSE
        xSO2=1.
      ENDIF
C
      IF (iINDUV.EQ.0) THEN
        iINDUV2=1
        xINDUV=0.
      ELSE
        xINDUV=1.
      ENDIF
C
      IF (iINDUVCS.EQ.0) THEN
        iINDUVCS2=1
        xINDUVCS=0.
      ELSE
        xINDUVCS=1.
      ENDIF
C
      IF (iUVA.EQ.0) THEN
        iUVA2=1
        xUVA=0.
      ELSE
        xUVA=1.
      ENDIF
C
      IF (iUVB.EQ.0) THEN
        iUVB2=1
        xUVB=0.
      ELSE
        xUVB=1.
      ENDIF
C
      IF (iNOy.EQ.0) THEN
        iNOy=1
        xNOy=0.
      ELSE
        xNOy=1.
      ENDIF
C
      IF (iOH.EQ.0) THEN
        iOH=1
        xOH=0.
      ELSE
        xOH=1.
      ENDIF
C
      IF (iCOVA.EQ.0) THEN
        iCOVA=1
        xCOVA=0.
      ELSE
        xCOVA=1.
      ENDIF
C
      IF (iCOVB.EQ.0) THEN
        iCOVB=1
        xCOVB=0.
      ELSE
        xCOVB=1.
      ENDIF
C NB
      IF (iHCHO.EQ.0) THEN
        iHCHO=1
        xHCHO=0.
      ELSE
        xHCHO=1.
      ENDIF
C
      IF (iETH.EQ.0) THEN
        iETH=1
        xETH=0.
      ELSE
        xETH=1.
      ENDIF
C
      IF (iISO.EQ.0) THEN
        iISO=1
        xISO=0.
      ELSE
        xISO=1.
      ENDIF
C
      IF (iH2O2.EQ.0) THEN
        iH2O2=1
        xH2O2=0.
      ELSE
        xH2O2=1.
      ENDIF
C
      IF (iKET.EQ.0) THEN
        iKET=1
        xKET=0.
      ELSE
        xKET=1.
      ENDIF

C NB 
C
CCC
C
      DO iobs=1,nlobs
        lobs=ILOBS(iobs)
C
        IF (iobs.EQ.1) THEN
            ll_select = .true.
        ELSE
            IF (typemesure(ILOBS(iobs)).EQ.
     +          typemesure(ILOBS(iobs-1))) THEN
                ll_select = .false.
            ELSE
                ll_select = .true.
            ENDIF
        ENDIF
C
        IF (ll_select) THEN
C
C SELECT OBSERVING SYSTEM
C
#ifndef RELACS                
        IF ((typemesure(lobs).EQ.'FLIGHT').OR.
     +      (typemesure(lobs).EQ.'GROUND').OR.
     +      (typemesure(lobs).EQ.'MOZART')) THEN
#else
        IF ((typemesure(lobs).EQ.'GROUND').OR.
     +      (typemesure(lobs).EQ.'MOZART')) THEN
#endif
C
C OS = FLIGHT (In-flight measurements : MOZAIC, campaigns,...)
C OS = GROUND (Ground based measurement site)
C OS = MOZART (TGV,aircraft,altitude measurements)
C
C      (1) O_3
C      (2) CO
C      (3) H_2O
C      (4) TEMPERATURE
C      (5) ZONAL WIND
C      (6) MERIDIONAL WIND
C
          IF (IFLAG1) THEN
            DO ilon=1,LOLANIM1
              IF (kO3 .eq. 1) THEN
                  GRID1(ilon,1,1,1)=xO3*QQQM1(ilon,1,1,iO3)
              ELSE
                  GRID1(ilon,1,1,1)=xO3*HHHM1(ilon,1,1,iO3)
              END IF
c              GRID1(ilon,1,1,1)=xO3*QQQM1(ilon,1,1,iO3)
              GRID1(ilon,1,1,2)=xCO*QQQM1(ilon,1,1,iCO)
              GRID1(ilon,1,1,3)=xH2O*QQQM1(ilon,1,1,iH2O)
              GRID1(ilon,1,1,4)=TGRIM1(ilon,1,1)
              GRID1(ilon,1,1,5)=UGRIM1(ilon,1,1)
              GRID1(ilon,1,1,6)=VGRIM1(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG2) THEN
            DO ilon=1,LOLANIM2
              IF (kO3 .eq. 1) THEN
                  GRID2(ilon,1,1,1)=xO3*QQQM2(ilon,1,1,iO3)
              ELSE
                  GRID2(ilon,1,1,1)=xO3*HHHM2(ilon,1,1,iO3)
              END IF
              GRID2(ilon,1,1,2)=xCO*QQQM2(ilon,1,1,iCO)
              GRID2(ilon,1,1,3)=xH2O*QQQM2(ilon,1,1,iH2O)
              GRID2(ilon,1,1,4)=TGRIM2(ilon,1,1)
              GRID2(ilon,1,1,5)=UGRIM2(ilon,1,1)
              GRID2(ilon,1,1,6)=VGRIM2(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG3) THEN
            DO ilon=1,LOLANIM3
              IF (kO3 .eq. 1) THEN
                  GRID3(ilon,1,1,1)=xO3*QQQM3(ilon,1,1,iO3)
              ELSE
                  GRID3(ilon,1,1,1)=xO3*HHHM3(ilon,1,1,iO3)
              END IF
              GRID3(ilon,1,1,2)=xCO*QQQM3(ilon,1,1,iCO)
              GRID3(ilon,1,1,3)=xH2O*QQQM3(ilon,1,1,iH2O)
              GRID3(ilon,1,1,4)=TGRIM3(ilon,1,1)
              GRID3(ilon,1,1,5)=UGRIM3(ilon,1,1)
              GRID3(ilon,1,1,6)=VGRIM3(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG4) THEN
            DO ilon=1,LOLANIM4
              IF (kO3 .eq. 1) THEN
                  GRID4(ilon,1,1,1)=xO3*QQQM4(ilon,1,1,iO3)
              ELSE
                  GRID4(ilon,1,1,1)=xO3*HHHM4(ilon,1,1,iO3)
              END IF
              GRID4(ilon,1,1,2)=xCO*QQQM4(ilon,1,1,iCO)
              GRID4(ilon,1,1,3)=xH2O*QQQM4(ilon,1,1,iH2O)
              GRID4(ilon,1,1,4)=TGRIM4(ilon,1,1)
              GRID4(ilon,1,1,5)=UGRIM4(ilon,1,1)
              GRID4(ilon,1,1,6)=VGRIM4(ilon,1,1)
            ENDDO
          ENDIF
          cstname(1)='[ O3]'
          cstname(2)='[ CO]'
          cstname(3)='[H2O]'
          cstname(4)='[  T]'
          cstname(5)='[  U]'
          cstname(6)='[  V]'
          nbconstobs=6
        ENDIF
        IF (typemesure(lobs).EQ.'FLIGHT') THEN
C MODIFIED 29/11 by N.B.
C MOZAIC : only for climate studies with RELACS (global domain)
c          check NOy definition for other chemical schemes
C
C      (1) CO
C      (2) O_3
C      (3) NO
C
          IF (IFLAG1) THEN
            DO ilon=1,LOLANIM1
              GRID1(ilon,1,1,1)=xCO*QQQM1(ilon,1,1,iCO)
              IF (kO3 .eq. 1) THEN
                  GRID1(ilon,1,1,2)=xO3*QQQM1(ilon,1,1,iO3)
              ELSE
                  GRID1(ilon,1,1,2)=xO3*HHHM1(ilon,1,1,iO3)
              END IF
              GRID1(ilon,1,1,3)  = xNO*HHHM1(ilon,1,1,iNO)
            ENDDO
          ENDIF
          cstname(1)='[ CO]'
          cstname(2)='[ O3]'
          cstname(3)='[ NO]'
          nbconstobs=3
        ENDIF
C
        IF (typemesure(lobs).EQ.'SURFUV') THEN
C
C OS = SURFUV (Ground UV measurement site)
C
C      (1) UV Index (WMO/WHO definition)
C      (2) UV Index for clear sky conditions
C      (3) UVA
C      (4) UVB
C      (5) H_2O
C      (6) TEMPERATURE
C      (7) ZONAL WIND
C      (8) MERIDIONAL WIND
C
          IF (IFLAG1) THEN
           DO ilon=1,LOLANIM1
           GRID1(ilon,1,1,1)=xINDUV*TJM1(ilon,1,1,iINDUV2)
           GRID1(ilon,1,1,2)=xINDUVCS*TJM1(ilon,1,1,iINDUVCS2)
           GRID1(ilon,1,1,3)=xUVA*TJM1(ilon,1,1,iUVA2)
           GRID1(ilon,1,1,4)=xUVB*TJM1(ilon,1,1,iUVB2)
           GRID1(ilon,1,1,5)=xH2O*QQQM1(ilon,1,1,iH2O)
           GRID1(ilon,1,1,6)=TGRIM1(ilon,1,1)
           GRID1(ilon,1,1,7)=UGRIM1(ilon,1,1)
           GRID1(ilon,1,1,8)=VGRIM1(ilon,1,1)
           ENDDO
          ENDIF
          IF (IFLAG2) THEN
           DO ilon=1,LOLANIM2
           GRID2(ilon,1,1,1)=xINDUV*TJM2(ilon,1,1,iINDUV2)
           GRID2(ilon,1,1,2)=xINDUVCS*TJM2(ilon,1,1,iINDUVCS2)
           GRID2(ilon,1,1,3)=xUVA*TJM2(ilon,1,1,iUVA2)
           GRID2(ilon,1,1,4)=xUVB*TJM2(ilon,1,1,iUVB2)
           GRID2(ilon,1,1,5)=xH2O*QQQM2(ilon,1,1,iH2O)
           GRID2(ilon,1,1,6)=TGRIM2(ilon,1,1)
           GRID2(ilon,1,1,7)=UGRIM2(ilon,1,1)
           GRID2(ilon,1,1,8)=VGRIM2(ilon,1,1)
           ENDDO
          ENDIF
          IF (IFLAG3) THEN
           DO ilon=1,LOLANIM3
           GRID3(ilon,1,1,1)=xINDUV*TJM3(ilon,1,1,iINDUV2)
           GRID3(ilon,1,1,2)=xINDUVCS*TJM3(ilon,1,1,iINDUVCS2)
           GRID3(ilon,1,1,3)=xUVA*TJM3(ilon,1,1,iUVA2)
           GRID3(ilon,1,1,4)=xUVB*TJM3(ilon,1,1,iUVB2)
           GRID3(ilon,1,1,5)=xH2O*QQQM3(ilon,1,1,iH2O)
           GRID3(ilon,1,1,6)=TGRIM3(ilon,1,1)
           GRID3(ilon,1,1,7)=UGRIM3(ilon,1,1)
           GRID3(ilon,1,1,8)=VGRIM3(ilon,1,1)
           ENDDO
          ENDIF
          IF (IFLAG4) THEN
           DO ilon=1,LOLANIM4
           GRID4(ilon,1,1,1)=xINDUV*TJM4(ilon,1,1,iINDUV2)
           GRID4(ilon,1,1,2)=xINDUVCS*TJM4(ilon,1,1,iINDUVCS2)
           GRID4(ilon,1,1,3)=xUVA*TJM4(ilon,1,1,iUVA2)
           GRID4(ilon,1,1,4)=xUVB*TJM4(ilon,1,1,iUVB2)
           GRID4(ilon,1,1,5)=xH2O*QQQM4(ilon,1,1,iH2O)
           GRID4(ilon,1,1,6)=TGRIM4(ilon,1,1)
           GRID4(ilon,1,1,7)=UGRIM4(ilon,1,1)
           GRID4(ilon,1,1,8)=VGRIM4(ilon,1,1)
           ENDDO
          ENDIF
          cstname(1)='[UVI]'
          cstname(2)='[NEB]'
          cstname(3)='[UVA]'
          cstname(4)='[UVB]'
          cstname(5)='[H2O]'
          cstname(6)='[  T]'
          cstname(7)='[  U]'
          cstname(8)='[  V]'
          nbconstobs=8
        ENDIF
C
        IF (typemesure(lobs).EQ.'OSONDE') THEN
C
C OS = OSONDE (Ozone radio-sounding)
C
C      (1) O_3
C      (2) H_2O
C      (3) TEMPERATURE
C      (4) ZONAL WIND
C      (5) MERIDIONAL WIND
C
          IF (IFLAG1) THEN
            DO ilon=1,LOLANIM1
              IF (kO3 .eq. 1) THEN
                  GRID1(ilon,1,1,1)=xO3*QQQM1(ilon,1,1,iO3)
              ELSE
                  GRID1(ilon,1,1,1)=xO3*HHHM1(ilon,1,1,iO3)
              END IF
              GRID1(ilon,1,1,2)=xH2O*QQQM1(ilon,1,1,iH2O)
              GRID1(ilon,1,1,3)=TGRIM1(ilon,1,1)
              GRID1(ilon,1,1,4)=UGRIM1(ilon,1,1)
              GRID1(ilon,1,1,5)=VGRIM1(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG2) THEN
            DO ilon=1,LOLANIM2
              IF (kO3 .eq. 1) THEN
                  GRID2(ilon,1,1,1)=xO3*QQQM2(ilon,1,1,iO3)
              ELSE
                  GRID2(ilon,1,1,1)=xO3*HHHM2(ilon,1,1,iO3)
              END IF
              GRID2(ilon,1,1,2)=xH2O*QQQM2(ilon,1,1,iH2O)
              GRID2(ilon,1,1,3)=TGRIM2(ilon,1,1)
              GRID2(ilon,1,1,4)=UGRIM2(ilon,1,1)
              GRID2(ilon,1,1,5)=VGRIM2(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG3) THEN
            DO ilon=1,LOLANIM3
              IF (kO3 .eq. 1) THEN
                  GRID3(ilon,1,1,1)=xO3*QQQM3(ilon,1,1,iO3)
              ELSE
                  GRID3(ilon,1,1,1)=xO3*HHHM3(ilon,1,1,iO3)
              END IF
              GRID3(ilon,1,1,2)=xH2O*QQQM3(ilon,1,1,iH2O)
              GRID3(ilon,1,1,3)=TGRIM3(ilon,1,1)
              GRID3(ilon,1,1,4)=UGRIM3(ilon,1,1)
              GRID3(ilon,1,1,5)=VGRIM3(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG4) THEN
            DO ilon=1,LOLANIM4
              IF (kO3 .eq. 1) THEN
                  GRID4(ilon,1,1,1)=xO3*QQQM4(ilon,1,1,iO3)
              ELSE
                  GRID4(ilon,1,1,1)=xO3*HHHM4(ilon,1,1,iO3)
              END IF
              GRID4(ilon,1,1,2)=xH2O*QQQM4(ilon,1,1,iH2O)
              GRID4(ilon,1,1,3)=TGRIM4(ilon,1,1)
              GRID4(ilon,1,1,4)=UGRIM4(ilon,1,1)
              GRID4(ilon,1,1,5)=VGRIM4(ilon,1,1)
            ENDDO
          ENDIF
          cstname(1)='[ O3]'
          cstname(2)='[H2O]'
          cstname(3)='[  T]'
          cstname(4)='[  U]'
          cstname(5)='[  V]'
          nbconstobs=5
        ENDIF
C
        IF (typemesure(lobs).EQ.'SOUNDG') THEN
C
C OS = SOUNDG (Radio-sounding)
C
C      (1) H_2O
C      (2) TEMPERATURE
C      (3) ZONAL WIND
C      (4) MERIDIONAL WIND
C
          IF (IFLAG1) THEN
            DO ilon=1,LOLANIM1
              GRID1(ilon,1,1,1)=xH2O*QQQM1(ilon,1,1,iH2O)
              GRID1(ilon,1,1,2)=TGRIM1(ilon,1,1)
              GRID1(ilon,1,1,3)=UGRIM1(ilon,1,1)
              GRID1(ilon,1,1,4)=VGRIM1(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG2) THEN
            DO ilon=1,LOLANIM2
              GRID2(ilon,1,1,1)=xH2O*QQQM2(ilon,1,1,iH2O)
              GRID2(ilon,1,1,2)=TGRIM2(ilon,1,1)
              GRID2(ilon,1,1,3)=UGRIM2(ilon,1,1)
              GRID2(ilon,1,1,4)=VGRIM2(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG3) THEN
            DO ilon=1,LOLANIM3
              GRID3(ilon,1,1,1)=xH2O*QQQM3(ilon,1,1,iH2O)
              GRID3(ilon,1,1,2)=TGRIM3(ilon,1,1)
              GRID3(ilon,1,1,3)=UGRIM3(ilon,1,1)
              GRID3(ilon,1,1,4)=VGRIM3(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG4) THEN
            DO ilon=1,LOLANIM4
              GRID4(ilon,1,1,1)=xH2O*QQQM4(ilon,1,1,iH2O)
              GRID4(ilon,1,1,2)=TGRIM4(ilon,1,1)
              GRID4(ilon,1,1,3)=UGRIM4(ilon,1,1)
              GRID4(ilon,1,1,4)=VGRIM4(ilon,1,1)
            ENDDO
          ENDIF
          cstname(1)='[H2O]'
          cstname(2)='[  T]'
          cstname(3)='[  U]'
          cstname(4)='[  V]'
          nbconstobs=4
        ENDIF
C
        IF ((typemesure(lobs).EQ.' MIPAS') .OR.
     +      (typemesure(lobs).EQ.'  GOME') .OR.
     +      (typemesure(lobs).EQ.'OD_SMR') .OR.
     +      (typemesure(lobs).EQ.'PROFIL')) THEN
C
C OS = MIPAS/ENVISAT 
C OS = GOME/ERS2 (KNMI Processing)
C OS = ODIN/SMR
C OS = Generic 1 species profile
C
          IF (IFLAG1) THEN
            DO ilon=1,LOLANIM1
              IF (kO3 .eq. 1) THEN
                  GRID1(ilon,1,1,1)=xO3*QQQM1(ilon,1,1,iO3)
              ELSE
                  GRID1(ilon,1,1,1)=xO3*HHHM1(ilon,1,1,iO3)
              END IF
            ENDDO
          ENDIF
          IF (IFLAG2) THEN
            DO ilon=1,LOLANIM2
              IF (kO3 .eq. 1) THEN
                  GRID2(ilon,1,1,1)=xO3*QQQM2(ilon,1,1,iO3)
              ELSE
                  GRID2(ilon,1,1,1)=xO3*HHHM2(ilon,1,1,iO3)
              END IF
            ENDDO
          ENDIF
          IF (IFLAG3) THEN
            DO ilon=1,LOLANIM3
              IF (kO3 .eq. 1) THEN
                  GRID3(ilon,1,1,1)=xO3*QQQM3(ilon,1,1,iO3)
              ELSE
                  GRID3(ilon,1,1,1)=xO3*HHHM3(ilon,1,1,iO3)
              END IF
            ENDDO
          ENDIF
          IF (IFLAG4) THEN
            DO ilon=1,LOLANIM4
              IF (kO3 .eq. 1) THEN
                  GRID4(ilon,1,1,1)=xO3*QQQM4(ilon,1,1,iO3)
              ELSE
                  GRID4(ilon,1,1,1)=xO3*HHHM4(ilon,1,1,iO3)
              END IF
            ENDDO
          ENDIF
          cstname(1)='[ O3]'
          nbconstobs=1
        ENDIF
C
        IF (typemesure(lobs).EQ.'MOZAIC') THEN
C
C OS = MOPITT CO profiles 
C
          IF (IFLAG1) THEN
            DO ilon=1,LOLANIM1
                  GRID1(ilon,1,1,1)=xCO*QQQM1(ilon,1,1,iCO)
            ENDDO
          ENDIF
          IF (IFLAG2) THEN
            DO ilon=1,LOLANIM2
                  GRID2(ilon,1,1,1)=xCO*QQQM2(ilon,1,1,iCO)
            ENDDO
          ENDIF
          IF (IFLAG3) THEN
            DO ilon=1,LOLANIM3
                  GRID3(ilon,1,1,1)=xCO*QQQM3(ilon,1,1,iCO)
            ENDDO
          ENDIF
          IF (IFLAG4) THEN
            DO ilon=1,LOLANIM4
                  GRID4(ilon,1,1,1)=xCO*QQQM4(ilon,1,1,iCO)
            ENDDO
          ENDIF
          cstname(1)='[ CO]'
          nbconstobs=1
        ENDIF
C
        IF ((typemesure(lobs).EQ.'RSCHFL').OR.
     +      (typemesure(lobs).EQ.'RSCHGR')) THEN
C
C OS = RSCHFL (Chemical measurements on board equiped aircrafts)
C OS = RSCHGR (Ground based chemical measurements)
C
C      (1)  O_3
C      (2)  CO
C      (3)  H_2O
C      (4)  NO
C      (5)  NO2
C      (6)  NO3
C      (7)  N2O5
C      (8)  HNO3
C      (9)  PAN
C      (10) TPAN
C      (11) SO_2
C      (12) TEMPERATURE
C      (13) ZONAL WIND
C      (14) MERIDIONAL WIND
C
          IF (IFLAG1) THEN
            DO ilon=1,LOLANIM1
              IF (kO3 .eq. 1) THEN
                  GRID1(ilon,1,1,1)=xO3*QQQM1(ilon,1,1,iO3)
              ELSE
                  GRID1(ilon,1,1,1)=xO3*HHHM1(ilon,1,1,iO3)
              END IF
              GRID1(ilon,1,1,2)  = xCO*QQQM1(ilon,1,1,iCO)
              GRID1(ilon,1,1,3)  = xH2O*QQQM1(ilon,1,1,iH2O)
              GRID1(ilon,1,1,4)  = xNO*HHHM1(ilon,1,1,iNO)
              GRID1(ilon,1,1,5)  = xNO2*HHHM1(ilon,1,1,iNO2)
              GRID1(ilon,1,1,6)  = xNO3*HHHM1(ilon,1,1,iNO3)
              GRID1(ilon,1,1,7)  = xN2O5*QQQM1(ilon,1,1,iN2O5)
              GRID1(ilon,1,1,8)  = xHNO3*QQQM1(ilon,1,1,iHNO3)
              GRID1(ilon,1,1,9)  = xPAN*QQQM1(ilon,1,1,iPAN)
              GRID1(ilon,1,1,10) = xTPAN*QQQM1(ilon,1,1,iTPAN)
              GRID1(ilon,1,1,11) = xSO2*QQQM1(ilon,1,1,iSO2)
              GRID1(ilon,1,1,12) = TGRIM1(ilon,1,1)
              GRID1(ilon,1,1,13) = UGRIM1(ilon,1,1)
              GRID1(ilon,1,1,14) = VGRIM1(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG2) THEN
            DO ilon=1,LOLANIM2
              IF (kO3 .eq. 1) THEN
                  GRID2(ilon,1,1,1)=xO3*QQQM2(ilon,1,1,iO3)
              ELSE
                  GRID2(ilon,1,1,1)=xO3*HHHM2(ilon,1,1,iO3)
              END IF
              GRID2(ilon,1,1,2)  = xCO*QQQM2(ilon,1,1,iCO)
              GRID2(ilon,1,1,3)  = xH2O*QQQM2(ilon,1,1,iH2O)
              GRID2(ilon,1,1,4)  = xNO*HHHM2(ilon,1,1,iNO)
              GRID2(ilon,1,1,5)  = xNO2*HHHM2(ilon,1,1,iNO2)
              GRID2(ilon,1,1,6)  = xNO3*HHHM2(ilon,1,1,iNO3)
              GRID2(ilon,1,1,7)  = xN2O5*QQQM2(ilon,1,1,iN2O5)
              GRID2(ilon,1,1,8)  = xHNO3*QQQM2(ilon,1,1,iHNO3)
              GRID2(ilon,1,1,9)  = xPAN*QQQM2(ilon,1,1,iPAN)
              GRID2(ilon,1,1,10) = xTPAN*QQQM2(ilon,1,1,iTPAN)
              GRID2(ilon,1,1,11) = xSO2*QQQM2(ilon,1,1,iSO2)
              GRID2(ilon,1,1,12) = TGRIM2(ilon,1,1)
              GRID2(ilon,1,1,13) = UGRIM2(ilon,1,1)
              GRID2(ilon,1,1,14) = VGRIM2(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG3) THEN
            DO ilon=1,LOLANIM3
              IF (kO3 .eq. 1) THEN
                  GRID3(ilon,1,1,1)=xO3*QQQM3(ilon,1,1,iO3)
              ELSE
                  GRID3(ilon,1,1,1)=xO3*HHHM3(ilon,1,1,iO3)
              END IF
              GRID3(ilon,1,1,2)  = xCO*QQQM3(ilon,1,1,iCO)
              GRID3(ilon,1,1,3)  = xH2O*QQQM3(ilon,1,1,iH2O)
              GRID3(ilon,1,1,4)  = xNO*HHHM3(ilon,1,1,iNO)
              GRID3(ilon,1,1,5)  = xNO2*HHHM3(ilon,1,1,iNO2)
              GRID3(ilon,1,1,6)  = xNO3*HHHM3(ilon,1,1,iNO3)
              GRID3(ilon,1,1,7)  = xN2O5*QQQM3(ilon,1,1,iN2O5)
              GRID3(ilon,1,1,8)  = xHNO3*QQQM3(ilon,1,1,iHNO3)
              GRID3(ilon,1,1,9)  = xPAN*QQQM3(ilon,1,1,iPAN)
              GRID3(ilon,1,1,10) = xTPAN*QQQM3(ilon,1,1,iTPAN)
              GRID3(ilon,1,1,11) = xSO2*QQQM3(ilon,1,1,iSO2)
              GRID3(ilon,1,1,12) = TGRIM3(ilon,1,1)
              GRID3(ilon,1,1,13) = UGRIM3(ilon,1,1)
              GRID3(ilon,1,1,14) = VGRIM3(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG4) THEN
            DO ilon=1,LOLANIM4
              IF (kO3 .eq. 1) THEN
                  GRID4(ilon,1,1,1)=xO3*QQQM4(ilon,1,1,iO3)
              ELSE
                  GRID4(ilon,1,1,1)=xO3*HHHM4(ilon,1,1,iO3)
              END IF
              GRID4(ilon,1,1,2)  = xCO*QQQM4(ilon,1,1,iCO)
              GRID4(ilon,1,1,3)  = xH2O*QQQM4(ilon,1,1,iH2O)
              GRID4(ilon,1,1,4)  = xNO*HHHM4(ilon,1,1,iNO)
              GRID4(ilon,1,1,5)  = xNO2*HHHM4(ilon,1,1,iNO2)
              GRID4(ilon,1,1,6)  = xNO3*HHHM4(ilon,1,1,iNO3)
              GRID4(ilon,1,1,7)  = xN2O5*QQQM4(ilon,1,1,iN2O5)
              GRID4(ilon,1,1,8)  = xHNO3*QQQM4(ilon,1,1,iHNO3)
              GRID4(ilon,1,1,9)  = xPAN*QQQM4(ilon,1,1,iPAN)
              GRID4(ilon,1,1,10) = xTPAN*QQQM4(ilon,1,1,iTPAN)
              GRID4(ilon,1,1,11) = xSO2*QQQM4(ilon,1,1,iSO2)
              GRID4(ilon,1,1,12) = TGRIM4(ilon,1,1)
              GRID4(ilon,1,1,13) = UGRIM4(ilon,1,1)
              GRID4(ilon,1,1,14) = VGRIM4(ilon,1,1)
            ENDDO
          ENDIF
          cstname(1)='[ O3]'
          cstname(2)='[ CO]'
          cstname(3)='[H2O]'
          cstname(4)='[ NO]'
          cstname(5)='[NO2]'
          cstname(6)='[NO3]'
          cstname(7)='[NO5]'
          cstname(8)='[HN3]'
          cstname(9)='[PAN]'
          cstname(10)='[TPA]'
          cstname(11)='[SO2]'
          cstname(12)='[  T]'
          cstname(13)='[  U]'
          cstname(14)='[  V]'
          nbconstobs=14
        ENDIF
C
        IF ((typemesure(lobs).EQ.'BDQAFR').OR.
     +      (typemesure(lobs).EQ.'BDQAEU')) THEN
C
C OS = RSCHFL (Chemical measurements on board equiped aircrafts)
C OS = RSCHGR (Ground based chemical measurements)
C
C      (1)  O_3
C      (2)  CO
C      (3)  H_2O
C      (4)  NO
C      (5)  NO2
C      (6)  SO_2
C      (7)  COVA
C      (8)  COVB
C      (9)  TEMPERATURE
C      (10) ZONAL WIND
C      (11) MERIDIONAL WIND
C
          IF (IFLAG1) THEN
            DO ilon=1,LOLANIM1
              GRID1(ilon,1,1,1)  = xO3*QQQM1(ilon,1,1,iO3)
              GRID1(ilon,1,1,2)  = xCO*QQQM1(ilon,1,1,iCO)
              GRID1(ilon,1,1,3)  = xH2O*QQQM1(ilon,1,1,iH2O)
              GRID1(ilon,1,1,4)  = xNO*HHHM1(ilon,1,1,iNO)
              GRID1(ilon,1,1,5)  = xNO2*HHHM1(ilon,1,1,iNO2)
              GRID1(ilon,1,1,6)  = xSO2*QQQM1(ilon,1,1,iSO2)
              GRID1(ilon,1,1,7)  = xCOVA*HHHM1(ilon,1,1,iCOVA)
              GRID1(ilon,1,1,8)  = xCOVB*HHHM1(ilon,1,1,iCOVB)
              GRID1(ilon,1,1,9)  = TGRIM1(ilon,1,1)
              GRID1(ilon,1,1,10) = UGRIM1(ilon,1,1)
              GRID1(ilon,1,1,11) = VGRIM1(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG2) THEN
            DO ilon=1,LOLANIM2
              GRID2(ilon,1,1,1)  = xO3*QQQM2(ilon,1,1,iO3)
              GRID2(ilon,1,1,2)  = xCO*QQQM2(ilon,1,1,iCO)
              GRID2(ilon,1,1,3)  = xH2O*QQQM2(ilon,1,1,iH2O)
              GRID2(ilon,1,1,4)  = xNO*HHHM2(ilon,1,1,iNO)
              GRID2(ilon,1,1,5)  = xNO2*HHHM2(ilon,1,1,iNO2)
              GRID2(ilon,1,1,6)  = xSO2*QQQM2(ilon,1,1,iSO2)
              GRID2(ilon,1,1,7)  = xCOVA*HHHM2(ilon,1,1,iCOVA)
              GRID2(ilon,1,1,8)  = xCOVB*HHHM2(ilon,1,1,iCOVB)
              GRID2(ilon,1,1,9)  = TGRIM2(ilon,1,1)
              GRID2(ilon,1,1,10) = UGRIM2(ilon,1,1)
              GRID2(ilon,1,1,11) = VGRIM2(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG3) THEN
            DO ilon=1,LOLANIM3
              GRID3(ilon,1,1,1)  = xO3*QQQM3(ilon,1,1,iO3)
              GRID3(ilon,1,1,2)  = xCO*QQQM3(ilon,1,1,iCO)
              GRID3(ilon,1,1,3)  = xH2O*QQQM3(ilon,1,1,iH2O)
              GRID3(ilon,1,1,4)  = xNO*HHHM3(ilon,1,1,iNO)
              GRID3(ilon,1,1,5)  = xNO2*HHHM3(ilon,1,1,iNO2)
              GRID3(ilon,1,1,6)  = xSO2*QQQM3(ilon,1,1,iSO2)
              GRID3(ilon,1,1,7)  = xCOVA*HHHM3(ilon,1,1,iCOVA)
              GRID3(ilon,1,1,8)  = xCOVB*HHHM3(ilon,1,1,iCOVB)
              GRID3(ilon,1,1,9)  = TGRIM3(ilon,1,1)
              GRID3(ilon,1,1,10) = UGRIM3(ilon,1,1)
              GRID3(ilon,1,1,11) = VGRIM3(ilon,1,1)
            ENDDO
          ENDIF
          IF (IFLAG4) THEN
            DO ilon=1,LOLANIM4
              GRID4(ilon,1,1,1)  = xO3*QQQM4(ilon,1,1,iO3)
              GRID4(ilon,1,1,2)  = xCO*QQQM4(ilon,1,1,iCO)
              GRID4(ilon,1,1,3)  = xH2O*QQQM4(ilon,1,1,iH2O)
              GRID4(ilon,1,1,4)  = xNO*HHHM4(ilon,1,1,iNO)
              GRID4(ilon,1,1,5)  = xNO2*HHHM4(ilon,1,1,iNO2)
              GRID4(ilon,1,1,6)  = xSO2*QQQM4(ilon,1,1,iSO2)
              GRID4(ilon,1,1,7)  = xCOVA*HHHM4(ilon,1,1,iCOVA)
              GRID4(ilon,1,1,8)  = xCOVB*HHHM4(ilon,1,1,iCOVB)
              GRID4(ilon,1,1,9)  = TGRIM4(ilon,1,1)
              GRID4(ilon,1,1,10) = UGRIM4(ilon,1,1)
              GRID4(ilon,1,1,11) = VGRIM4(ilon,1,1)
            ENDDO
          ENDIF
          cstname(1)='[ O3]'
          cstname(2)='[ CO]'
          cstname(3)='[H2O]'
          cstname(4)='[ NO]'
          cstname(5)='[NO2]'
          cstname(6)='[SO2]'
          cstname(7)='[OVA]'
          cstname(8)='[OVB]'
          cstname(9)='[  T]'
          cstname(10)='[  U]'
          cstname(11)='[  V]'
          nbconstobs=11
        ENDIF
C

        IF ((typemesure(lobs).NE.'FLIGHT').AND.
     +      (typemesure(lobs).NE.'GROUND').AND.
     +      (typemesure(lobs).NE.'MOZAIC').AND.
     +      (typemesure(lobs).NE.'MOZART').AND.
     +      (typemesure(lobs).NE.'SURFUV').AND.
     +      (typemesure(lobs).NE.'OSONDE').AND.
     +      (typemesure(lobs).NE.'SOUNDG').AND.
     +      (typemesure(lobs).NE.'  GOME').AND.
     +      (typemesure(lobs).NE.' MIPAS').AND.
     +      (typemesure(lobs).NE.'PROFIL').AND.
     +      (typemesure(lobs).NE.'OD_SMR').AND.
     +      (typemesure(lobs).NE.'BDQAFR').AND.
     +      (typemesure(lobs).NE.'BDQEUR').AND.
     +      (typemesure(lobs).NE.'RSCHFL').AND.
     +      (typemesure(lobs).NE.'RSCHGR')) THEN
C
C OS is not recognized 
C
          WRITE(*,*) ' > OS FOR OBS #',lobs,' WAS NOT RECOGNIZED!'
          nbconstobs=0
        ENDIF
C
      ENDIF
C
CCC COMPUTE GUESS FOR ALL DOMAINS
C
      IF (nbconstobs.GT.0) THEN
C
        DO i=1,nbnivpres(lobs)
          P0(i)=nivpres(lobs,i)
        ENDDO
C
        IF (IFLAG1) THEN
          CALL HOBS3D(GRID1,LONM1,LATM1,NIVM1,1,
     +                xobslon(lobs),xobslat(lobs),P0,
     +                nbnivpres(lobs),
     +                nbconstobs,
     +                XINTER1)
          DO icst=1,nbconstobs
            DO i=1,nbnivpres(lobs)
              G1(iobs,i,icst)=XINTER1(i,icst)
            ENDDO
          ENDDO
        ENDIF
C
        IF (IFLAG2) THEN
          CALL HOBS3D(GRID2,LONM2,LATM2,NIVM2,2,
     +                xobslon(lobs),xobslat(lobs),P0,
     +                nbnivpres(lobs),
     +                nbconstobs,
     +                XINTER2)
          DO icst=1,nbconstobs
            DO i=1,nbnivpres(lobs)
              G2(iobs,i,icst)=XINTER2(i,icst)
            ENDDO
          ENDDO
        ENDIF
C
        IF (IFLAG3) THEN
          CALL HOBS3D(GRID3,LONM3,LATM3,NIVM3,3,
     +                xobslon(lobs),xobslat(lobs),P0,
     +                nbnivpres(lobs),
     +                nbconstobs,
     +                XINTER3)
          DO icst=1,nbconstobs
            DO i=1,nbnivpres(lobs)
              G3(iobs,i,icst)=XINTER3(i,icst)
            ENDDO
          ENDDO
        ENDIF
C
        IF (IFLAG4) THEN
          CALL HOBS3D(GRID4,LONM4,LATM4,NIVM4,4,
     +                xobslon(lobs),xobslat(lobs),P0,
     +                nbnivpres(lobs),
     +                nbconstobs,
     +                XINTER4)
          DO icst=1,nbconstobs
            DO i=1,nbnivpres(lobs)
              G4(iobs,i,icst)=XINTER4(i,icst)
            ENDDO
          ENDDO
        ENDIF
C
      ENDIF
C
      ENDDO
C
      DEALLOCATE (GRID1)
      IF (IFLAG2) DEALLOCATE (GRID2)
      IF (IFLAG3) DEALLOCATE (GRID3)
      IF (IFLAG4) DEALLOCATE (GRID4)
      DEALLOCATE (cstname)
      DEALLOCATE (P0)
      DEALLOCATE (XINTER1)
      IF (IFLAG2) DEALLOCATE (XINTER2)
      IF (IFLAG3) DEALLOCATE (XINTER3)
      IF (IFLAG4) DEALLOCATE (XINTER4)
C
      RETURN
      END
