      SUBROUTINE CORREC(RMAS0)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C
C   Type       : 3D semi-lagrangian transport scheme
C   Purpose    : Correct negative mixing ratios and force mass conservation 
C   Called by  :
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 12/1999
C
C  Modified :
C
CCC
C
      USE PARADI_MOD
      USE MRAT_MOD
      USE FORMT_MOD
      USE GRIDS_MOD
      USE EXPE_MOD
      USE MOCF_MOD
      USE CSTES_MOD
      USE GAUSS_MOD
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      REAL (KIND=8) , DIMENSION (NTRA1) :: RMAS0
C	
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: I,JC
      REAL (KIND=8) :: COEF
      REAL (KIND=8) , ALLOCATABLE :: RMAS1A(:),RMAS2A(:)
      REAL (KIND=8) , ALLOCATABLE :: RMAS3A(:),RMAS4A(:)
      REAL (KIND=8) , ALLOCATABLE :: RMAS1B(:),RMAS2B(:)
      REAL (KIND=8) , ALLOCATABLE :: RMAS3B(:),RMAS4B(:)
C
C masse initiale et correction
C    
      IF (IFLAG1) THEN
        ALLOCATE (RMAS1A(NTRA1)) ; RMAS1A(:) = 0.0
        ALLOCATE (RMAS1B(NTRA1)) ; RMAS1B(:) = 0.0
        CALL MASSE (LONM1,LATM1,NIVM1,NTRA1
     +             ,DRLONM1,DLATM1,CLAT1,TRANAM1
     +             ,XPHFM1,QQQM1,RMAS1A)
        DO I=1,LOLANIM1*NTRA1
          QQQM1(I,1,1,1)=MAX(0.,QQQM1(I,1,1,1))
        ENDDO
      ENDIF 
C
      IF (IFLAG2) THEN
        ALLOCATE (RMAS2A(NTRA2)) ; RMAS2A(:) = 0.0
        ALLOCATE (RMAS2B(NTRA2)) ; RMAS2B(:) = 0.0
        CALL MASSE (LONM2,LATM2,NIVM2,NTRA2
     +             ,DRLONM2,DLATM2,CLAT2,TRANAM2
     +             ,XPHFM2,QQQM2,RMAS2A)
        DO I=1,LOLANIM2*NTRA2
          QQQM2(I,1,1,1)=MAX(0.,QQQM2(I,1,1,1))
        ENDDO
      ENDIF
C
      IF (IFLAG3) THEN
        ALLOCATE (RMAS3A(NTRA3)) ; RMAS3A(:) = 0.0
        ALLOCATE (RMAS3B(NTRA3)) ; RMAS3B(:) = 0.0
        CALL MASSE (LONM3,LATM3,NIVM3,NTRA3
     +             ,DRLONM3,DLATM3,CLAT3,TRANAM3
     +             ,XPHFM3,QQQM3,RMAS3A)
        DO I=1,LOLANIM3*NTRA3
          QQQM3(I,1,1,1)=MAX(0.,QQQM3(I,1,1,1))
        ENDDO
      ENDIF
C
      IF (IFLAG4) THEN
        ALLOCATE (RMAS4A(NTRA4)) ; RMAS4A(:) = 0.0
        ALLOCATE (RMAS4B(NTRA4)) ; RMAS4A(:) = 0.0
        CALL MASSE (LONM4,LATM4,NIVM4,NTRA4
     +             ,DRLONM4,DLATM4,CLAT4,TRANAM4
     +             ,XPHFM4,QQQM4,RMAS4A)
        DO I=1,LOLANIM4*NTRA4
          QQQM4(I,1,1,1)=MAX(0.,QQQM4(I,1,1,1))
        ENDDO
      ENDIF
C
C Masse apres correction et renormalisation
C  
      IF (IFLAG1) THEN
        CALL MASSE (LONM1,LATM1,NIVM1,NTRA1
     +             ,DRLONM1,DLATM1,CLAT1,TRANAM1
     +             ,XPHFM1,QQQM1,RMAS1B)
C
        DO JC=1,NTRA1 
          IF ( (TRANAM1(JC).NE.'H_2SO_4')     .AND.     
     +         (TRANAM1(JC).NE.'H_2O')        .AND.   
     +         (TRANAM1(JC).NE.'PSC')         .AND.   
     +         (TRANAM1(JC).NE.'LINO_3')      .AND.
     +         (TRANAM1(JC).NE.'TRACEUR_FROID')  ) THEN
C  
            IF (RMAS1B(JC).NE.0.) THEN
              COEF=RMAS1A(JC)/RMAS1B(JC)
            ELSE
              COEF=1.
            ENDIF
            DO I=1,LOLANIM1
              QQQM1(I,1,1,JC)=QQQM1(I,1,1,JC)*COEF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
C
      IF (IFLAG2) THEN
        CALL MASSE (LONM2,LATM2,NIVM2,NTRA2
     +             ,DRLONM2,DLATM2,CLAT2,TRANAM2
     +             ,XPHFM2,QQQM2,RMAS2B)
C
        DO JC=1,NTRA2 
          IF ( (TRANAM2(JC).NE.'H_2SO_4')     .AND.     
     +         (TRANAM2(JC).NE.'H_2O')        .AND.   
     +         (TRANAM2(JC).NE.'PSC')         .AND.   
     +         (TRANAM2(JC).NE.'LINO_3')      .AND.
     +         (TRANAM2(JC).NE.'TRACEUR_FROID')  ) THEN
C  
            IF (RMAS2B(JC).NE.0.) THEN
              COEF=RMAS2A(JC)/RMAS2B(JC)
            ELSE
              COEF=1.
            ENDIF
            DO I=1,LOLANIM2
              QQQM2(I,1,1,JC)=QQQM2(I,1,1,JC)*COEF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
C
      IF (IFLAG3) THEN
        CALL MASSE (LONM3,LATM3,NIVM3,NTRA3
     +             ,DRLONM3,DLATM3,CLAT3,TRANAM3
     +             ,XPHFM3,QQQM3,RMAS3B)
C
        DO JC=1,NTRA3 
          IF ( (TRANAM3(JC).NE.'H_2SO_4')     .AND.     
     +         (TRANAM3(JC).NE.'H_2O')        .AND.   
     +         (TRANAM3(JC).NE.'PSC')         .AND.   
     +         (TRANAM3(JC).NE.'LINO_3')      .AND.
     +         (TRANAM3(JC).NE.'TRACEUR_FROID')  ) THEN
C  
            IF (RMAS3B(JC).NE.0.) THEN
              COEF=RMAS3A(JC)/RMAS3B(JC)
            ELSE
              COEF=1.
            ENDIF
            DO I=1,LOLANIM3
              QQQM3(I,1,1,JC)=QQQM3(I,1,1,JC)*COEF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
C
      IF (IFLAG4) THEN
        CALL MASSE (LONM4,LATM4,NIVM4,NTRA4
     +             ,DRLONM4,DLATM4,CLAT4,TRANAM4
     +             ,XPHFM4,QQQM4,RMAS4B)
C
        DO JC=1,NTRA4 
          IF ( (TRANAM4(JC).NE.'H_2SO_4')     .AND.     
     +         (TRANAM4(JC).NE.'H_2O')        .AND.   
     +         (TRANAM4(JC).NE.'PSC')         .AND.   
     +         (TRANAM4(JC).NE.'LINO_3')      .AND.
     +         (TRANAM4(JC).NE.'TRACEUR_FROID')  ) THEN
C  
            IF (RMAS4B(JC).NE.0.) THEN
              COEF=RMAS4A(JC)/RMAS4B(JC)
            ELSE
              COEF=1.
            ENDIF
            DO I=1,LOLANIM4
              QQQM4(I,1,1,JC)=QQQM4(I,1,1,JC)*COEF
            ENDDO
          ENDIF
        ENDDO
      ENDIF
C
C Correction pour assurer la conservation de la masse
C sur le domaine globe
C    
      DO JC=1,NTRA1
        IF ( (TRANAM1(JC).NE.'H_2SO_4')     .AND.     
     +       (TRANAM1(JC).NE.'H_2O')        .AND.   
     +       (TRANAM1(JC).NE.'PSC')         .AND.   
     +       (TRANAM1(JC).NE.'LINO_3')      .AND.
     +       (TRANAM1(JC).NE.'TRACEUR_FROID')  ) THEN
C
          COEF=1.
C
          IF (RMAS1A(JC).NE.0.) THEN
            COEF=RMAS0(JC)/RMAS1A(JC)
            IF (IFLAG1) THEN
              DO I=1,LOLANIM1
                QQQM1(I,1,1,JC)=QQQM1(I,1,1,JC)*COEF
              ENDDO
            ENDIF
            IF (IFLAG2) THEN
              DO I=1,LOLANIM2
                QQQM2(I,1,1,JC)=QQQM2(I,1,1,JC)*COEF
              ENDDO
            ENDIF
            IF (IFLAG3) THEN
              DO I=1,LOLANIM3
                QQQM3(I,1,1,JC)=QQQM3(I,1,1,JC)*COEF
              ENDDO
            ENDIF
            IF (IFLAG4) THEN
              DO I=1,LOLANIM4
                QQQM4(I,1,1,JC)=QQQM4(I,1,1,JC)*COEF
              ENDDO
            ENDIF
          ENDIF
C
       ENDIF
C
      ENDDO
C
      IF (IFLAG1) THEN
        DEALLOCATE (RMAS1A)
        DEALLOCATE (RMAS1B)
      ENDIF
      IF (IFLAG2) THEN
        DEALLOCATE (RMAS2A)
        DEALLOCATE (RMAS2B)
      ENDIF
      IF (IFLAG3) THEN
        DEALLOCATE (RMAS3A)
        DEALLOCATE (RMAS3B)
      ENDIF
      IF (IFLAG4) THEN
        DEALLOCATE (RMAS4A)
        DEALLOCATE (RMAS4B)
      ENDIF
C
      RETURN 
      END
