	MODULE DATOBS_MOD
C
C------------------------------------------------------------------------
C 	DATA ASSIMILATION AND ON-LINE DIAGNOSTICS
C
C	AUTHOR	: HERVE LE BERRE	- CERFACS -
C	DATE		: 03/08/2005
C	VERSION	: 1_0_0
C------------------------------------------------------------------------
C
CAP
	USE MOCF_MOD
CAP
	IMPLICIT NONE
C
C
C nobs       : total number of observations for the current day
C nos        : number of different OS for the current day
C defos      : description of the OS for the current day
C
C FOR EACH OBSERVATION :
C
C idatobs    : date
C nsecobs    : time from start of the model (s)
C ninst      : instrument or flight number
C typemesure : type of observational system
C xobslat    : latitude (degrees)
C xobslon    : longitude (degrees)
C nbnivpres  : number of pressure levels
C nivpres    : pressure levels (Pa)
C nbconst    : number of constituents for each pressure level
C constname  : constituents measured for each pressure level
C VAL        : measured value
C GUESS1     : model equivalent to observed mixing ratio (pp) for DOMAIN 1
C GUESS2     : model equivalent to observed mixing ratio (pp) for DOMAIN 2
C GUESS3     : model equivalent to observed mixing ratio (pp) for DOMAIN 3
C GUESS4     : model equivalent to observed mixing ratio (pp) for DOMAIN 4
C
C
	CHARACTER(LEN=68) , ALLOCATABLE , SAVE :: defos(:)
	CHARACTER(LEN=6) , ALLOCATABLE , SAVE :: typemesure(:)
	CHARACTER(LEN=5) , ALLOCATABLE , SAVE :: constname(:,:,:)
	INTEGER , SAVE :: NOBS,NOS
	INTEGER , ALLOCATABLE , SAVE :: nsecobs(:),idatobs(:,:),ninst(:)
	INTEGER , ALLOCATABLE , SAVE :: nbnivpres(:),nbconst(:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: xobslat(:),xobslon(:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: nivpres(:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: val(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS1A(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS1B(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS2A(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS2B(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS3A(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS3B(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS4A(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS4B(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS1(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS2(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS3(:,:,:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: GUESS4(:,:,:)
C
	CONTAINS
C
	SUBROUTINE ALLOC_DATOBS
C
	USE PARADI_MOD
        integer i, k
C
	ALLOCATE (defos(ninstmax))
        do i = 1,ninstmax
        do k = 1,68
          defos(i)(k:k) = 'x'
        end do
        end do
	ALLOCATE (typemesure(nobsmax))	; typemesure(:) = '123456'
	ALLOCATE (constname(nobsmax,nbnivpresmax,nbconstmax))	; constname(:,:,:) = '12345'
	ALLOCATE (nsecobs(nobsmax))	; nsecobs(:) = 0
	ALLOCATE (idatobs(nobsmax,5))	; idatobs(:,:) = 0
	ALLOCATE (ninst(nobsmax))	; ninst(:) = 0
	ALLOCATE (nbnivpres(nobsmax))	; nbnivpres(:) = 0
	ALLOCATE (nbconst(nobsmax,nbnivpresmax))	; nbconst(:,:) = 0
	ALLOCATE (xobslat(nobsmax))	; xobslat(:) = 0.0
	ALLOCATE (xobslon(nobsmax))	; xobslon(:) = 0.0
	ALLOCATE (nivpres(nobsmax,nbnivpresmax))	; nivpres(:,:) = 0
	ALLOCATE (val(nobsmax,nbnivpresmax,nbconstmax))	; val(:,:,:) = 0.0
	ALLOCATE (GUESS1A(nobstspmax,nbnivpresmax,nbconstmax))	; GUESS1A(:,:,:) = 0.0
	ALLOCATE (GUESS1B(nobstspmax,nbnivpresmax,nbconstmax))	; GUESS1B(:,:,:) = 0.0
	ALLOCATE (GUESS1(nobsmax,nbnivpresmax,nbconstmax))	; GUESS1(:,:,:) = 0.0
	IF (IFLAG2) THEN
	ALLOCATE (GUESS2A(nobstspmax,nbnivpresmax,nbconstmax))
	ALLOCATE (GUESS2B(nobstspmax,nbnivpresmax,nbconstmax))
	ALLOCATE (GUESS2(nobsmax,nbnivpresmax,nbconstmax))
	ENDIF
	IF (IFLAG3) THEN
	ALLOCATE (GUESS3A(nobstspmax,nbnivpresmax,nbconstmax))
	ALLOCATE (GUESS3B(nobstspmax,nbnivpresmax,nbconstmax))
	ALLOCATE (GUESS3(nobsmax,nbnivpresmax,nbconstmax))
	ENDIF
	IF (IFLAG4) THEN
	ALLOCATE (GUESS4A(nobstspmax,nbnivpresmax,nbconstmax))
	ALLOCATE (GUESS4B(nobstspmax,nbnivpresmax,nbconstmax))
	ALLOCATE (GUESS4(nobsmax,nbnivpresmax,nbconstmax))
	ENDIF
C
	END SUBROUTINE ALLOC_DATOBS
C
	SUBROUTINE DEALLOC_DATOBS
C
	DEALLOCATE (defos)
	DEALLOCATE (typemesure)
	DEALLOCATE (constname)
	DEALLOCATE (nsecobs)
	DEALLOCATE (idatobs)
	DEALLOCATE (ninst)
	DEALLOCATE (nbnivpres)
	DEALLOCATE (nbconst)
	DEALLOCATE (xobslat)
	DEALLOCATE (xobslon)
	DEALLOCATE (nivpres)
	DEALLOCATE (val)
	DEALLOCATE (GUESS1A)
	DEALLOCATE (GUESS1B)
	DEALLOCATE (GUESS1)
	IF (IFLAG2) THEN
	DEALLOCATE (GUESS2A)
	DEALLOCATE (GUESS2B)
	DEALLOCATE (GUESS2)
	ENDIF
	IF (IFLAG3) THEN
	DEALLOCATE (GUESS3A)
	DEALLOCATE (GUESS3B)
	DEALLOCATE (GUESS3)
	ENDIF
	IF (IFLAG4) THEN
	DEALLOCATE (GUESS4A)
	DEALLOCATE (GUESS4B)
	DEALLOCATE (GUESS4)
	ENDIF
C
	END SUBROUTINE DEALLOC_DATOBS
C
	END MODULE DATOBS_MOD
