      SUBROUTINE DENSITY(WS,WN,TEMP,DENS,IMAX)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Chemistry
C   Purpose   : Compute density of ternary solution in g.cm-3
C   Called by :
C
C     WS ,WN ARE WT FRACTION,
C     FITTED TO 0.05<WS+WN<0.70 WT FRACTION, BUT EXTRAPOLATES WELL
C     185 < T (K)
C
	IMPLICIT NONE
C	
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C      
	INTEGER :: IMAX
	REAL (KIND=8) , DIMENSION (IMAX) :: WS,WN,TEMP,DENS
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: I
	REAL (KIND=8) :: T,W,WH,V1,VS,VN,VMCAL
      REAL (KIND=8) , DIMENSION (22) :: X
      SAVE X
C
      X=(/
     $     2.393284E-02,-4.359335E-05,7.961181E-08,0.0,-0.198716351,
     $     1.39564574E-03,-2.020633E-06,0.51684706,-3.0539E-03,
     $     4.505475E-06,-0.30119511,1.840408E-03,-2.7221253742E-06,
     $     -0.11331674116,8.47763E-04,-1.22336185E-06,0.3455282,
     $     -2.2111E-03,3.503768245E-06,-0.2315332,1.60074E-03,
     $     -2.5827835E-06/)
C
      DO I=1,IMAX
         T = TEMP(I)
         W = WS(I) + WN(I)
         WH= 1.0 - W
         V1=X(1)+X(2)*T+X(3)*T**2+X(4)*T**3
         VS=X(5)+X(6)*T+X(7)*T**2+(X(8)+X(9)*T+X(10)*T**2)*W
     $      +(X(11)+X(12)*T+X(13)*T**2)*W*W
         VN=X(14)+X(15)*T+X(16)*T**2+(X(17)+X(18)*T+X(19)*T**2)*W
     $      +(X(20)+X(21)*T+X(22)*T**2)*W*W
         VMCAL=WH/18.0160*V1 + VS*WS(I)/98.080
     $         + VN*WN(I)/63.0160
         DENS(I) = 1.0/VMCAL*0.001
      ENDDO
C
      RETURN
      END
C
CCC
C
