      SUBROUTINE DRYDEP (LON,LAT,NIV,NTRA,NSLS,NDEP,TRANAM,SLSNAM,
     +                   DT,XPAM,XPHFM,TGRIM,
     +                   IQMDEP,IHMDEP,VDEP,QQQM,HHHM)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Interface
C   Purpose   : 2D dry deposition at surface
C   Called by : IOPC3D
C
CCC
C
C  Original : V.-H. Peuch, 05/1999
C
C  Modified : H. Teyssedre (passage en subroutine) 06/2006
C
CCC
C
      USE CSTES_MOD
C
      IMPLICIT NONE
C	
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER , INTENT(IN) :: LON,LAT,NIV,NTRA,NSLS,NDEP
      INTEGER, DIMENSION(NTRA) , INTENT(IN) :: IQMDEP
      INTEGER, DIMENSION(NSLS) , INTENT(IN) :: IHMDEP
C
      REAL (KIND=8) , INTENT(IN) :: DT
      REAL (KIND=8) , DIMENSION(LON,LAT,NIV) , INTENT(IN) ::  XPAM
      REAL (KIND=8) , DIMENSION(LON,LAT,NIV) , INTENT(IN) ::  XPHFM
      REAL (KIND=8) , DIMENSION(LON,LAT,NIV) , INTENT(IN) :: TGRIM
      REAL (KIND=8) , DIMENSION(LON,LAT,NDEP) , INTENT(IN)::  VDEP
      REAL (KIND=8) , DIMENSION(LON,LAT,NIV,NTRA),INTENT(INOUT) :: QQQM
      REAL (KIND=8) , DIMENSION(LON,LAT,NIV,NSLS),INTENT(INOUT) :: HHHM
C
      CHARACTER (LEN=15) , DIMENSION(NTRA) , INTENT(IN) :: TRANAM
      CHARACTER (LEN=15) , DIMENSION(NSLS) , INTENT(IN) :: SLSNAM
C      
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      LOGICAL :: LDEPNOX
C
      INTEGER :: iHNO3,iN2O5,iHNO4,iHONO,iONIT,iPAN,iTPAN
      INTEGER :: iNOx,iNOy,iNO,iNO2,iNO3,itra
C
      REAL (KIND=8) , ALLOCATABLE :: ZFACT(:,:)
#ifdef MASSy
      REAL (KIND=8) , ALLOCATABLE :: NOx  (:,:),HNO3  (:,:)
      REAL (KIND=8) , ALLOCATABLE :: N2O5 (:,:),HNO4  (:,:)
      REAL (KIND=8) , ALLOCATABLE :: CLONO2(:,:),ClNO2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: BrONO2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: HONO (:,:),ONIT  (:,:)
      REAL (KIND=8) , ALLOCATABLE :: PAN  (:,:),OLNN (:,:)
#ifdef RACMOBUS
      REAL (KIND=8) , ALLOCATABLE :: TPAN (:,:),OLND (:,:)
#endif
      REAL (KIND=8) , ALLOCATABLE :: ClOx (:,:),OCLO  (:,:)
      REAL (KIND=8) , ALLOCATABLE :: HCl  (:,:),HOCl  (:,:)
      REAL (KIND=8) , ALLOCATABLE :: Cl2  (:,:),BrCl  (:,:)
      REAL (KIND=8) , ALLOCATABLE :: BrOx (:,:),HBr   (:,:)
      REAL (KIND=8) , ALLOCATABLE :: HOBr (:,:)
#endif
C	
      ALLOCATE (ZFACT (LON,LAT))
#ifdef MASSy
      ALLOCATE (NOx   (LON,LAT))
      ALLOCATE (HNO3  (LON,LAT))
      ALLOCATE (N2O5  (LON,LAT))
      ALLOCATE (HNO4  (LON,LAT))
      ALLOCATE (HONO  (LON,LAT))
      ALLOCATE (ONIT  (LON,LAT))
      ALLOCATE (PAN   (LON,LAT))
      ALLOCATE (OLNN  (LON,LAT))
#ifdef RACMOBUS
      ALLOCATE (TPAN  (LON,LAT))
      ALLOCATE (OLND  (LON,LAT))
#endif
      ALLOCATE (CLONO2(LON,LAT))
      ALLOCATE (CLNO2 (LON,LAT))
      ALLOCATE (BRONO2(LON,LAT))
      ALLOCATE (CLOx  (LON,LAT))
      ALLOCATE (OCLO  (LON,LAT))
      ALLOCATE (HCL   (LON,LAT))
      ALLOCATE (HOCL  (LON,LAT))
      ALLOCATE (CL2   (LON,LAT))
      ALLOCATE (BRCL  (LON,LAT))
      ALLOCATE (BROx  (LON,LAT))
      ALLOCATE (HBR   (LON,LAT))
      ALLOCATE (HOBR  (LON,LAT))
#endif
C
CCCCC
C     _________________
C    |                 |
C    | DRY DEPOSITION  |
C    |_________________|
C
C C(t+DT)+Vd*[...]*C(t+DT)=C(t)
C
CCCCC
C
      iHNO3= 0
      iN2O5= 0
      iHNO4= 0
      iHONO= 0
      iONIT= 0
      iPAN = 0
      iTPAN= 0
      iNOx = 0
      iNOy = 0
      iNO  = 0
      iNO2 = 0
      iNO3 = 0
      DO itra=1,NTRA
        IF (TRANAM(itra).EQ.'HNO_3' ) iHNO3=itra
        IF (TRANAM(itra).EQ.'N_2O_5') iN2O5=itra
        IF (TRANAM(itra).EQ.'HNO_4' ) iHNO4=itra
        IF (TRANAM(itra).EQ.'HONO'  ) iHONO=itra
        IF (TRANAM(itra).EQ.'ONIT'  ) iONIT=itra
        IF (TRANAM(itra).EQ.'PAN'   ) iPAN =itra
        IF (TRANAM(itra).EQ.'TPAN'  ) iTPAN=itra
        IF (TRANAM(itra).EQ.'NO_x'  ) iNOx =itra
#ifdef MASSy
        IF (TRANAM(itra).EQ.'NO_y'  ) iNOy =itra
#endif
      ENDDO
      DO itra=1,NSLS
        IF (SLSNAM(itra).EQ.'NO')   iNO=itra
        IF (SLSNAM(itra).EQ.'NO_2') iNO2=itra
        IF (SLSNAM(itra).EQ.'NO_3') iNO3=itra
      ENDDO
      IF ((iNOx.NE.0).AND.(iNO.NE.0).AND.(iNO2.NE.0).AND.
     +    (iNO3.NE.0)) THEN
        LDEPNOX=.TRUE.
      ELSE
        LDEPNOX=.FALSE.
      ENDIF
C
#ifdef MASSy
      IF (iHNO3.NE.0.) THEN
       HNO3(:,:)=QQQM(:,:,NIV,iHNO3)
      ELSE
       HNO3(:,:)=0.
      ENDIF
      IF (iN2O5.NE.0.) THEN
       N2O5(:,:)=QQQM(:,:,NIV,iN2O5)
      ELSE
       N2O5(:,:)=0.
      ENDIF
      IF (iHNO4.NE.0.) THEN
       HNO4(:,:)=QQQM(:,:,NIV,iHNO4)
      ELSE
       HNO4(:,:)=0.
      ENDIF
      IF (iHONO.NE.0.) THEN
       HONO(:,:)=QQQM(:,:,NIV,iHONO)
      ELSE
       HONO(:,:)=0.
      ENDIF
      IF (iONIT.NE.0.) THEN
       ONIT(:,:)=QQQM(:,:,NIV,iONIT)
      ELSE
       ONIT(:,:)=0.
      ENDIF
      IF (iPAN.NE.0.) THEN
       PAN(:,:)=QQQM(:,:,NIV,iPAN)
      ELSE
       PAN(:,:)=0.
      ENDIF
#ifdef RACMOBUS
      TPAN(:,:)=QQQM(:,:,NIV,iTPAN)
#endif
#endif
      DO itra=1,NTRA
        IF (IQMDEP(itra).NE.0) THEN
          QQQM(:,:,NIV,itra)=MAX(1.e-30,QQQM(:,:,NIV,itra) /
     +     (1. + MAIR*GG*DT*VDEP(:,:,IQMDEP(itra))*XPAM(:,:,NIV)/
     +     ((XPHFM(:,:,NIV)-XPHFM(:,:,NIV-1))*
     +       TGRIM(:,:,NIV)*8.314517)))
        ENDIF
      ENDDO
#ifdef MASSy
      QQQM(:,:,NIV,iNOy)=MAX(1.E-30, QQQM(:,:,NIV,iNOy)-
     +                  MAX(0.,HNO3(:,:)-QQQM(:,:,NIV,iHNO3))-
     +                  MAX(0.,N2O5(:,:)-QQQM(:,:,NIV,iN2O5))-
     +                  MAX(0.,HNO4(:,:)-QQQM(:,:,NIV,iHNO4))-
     +                  MAX(0.,HONO(:,:)-QQQM(:,:,NIV,iHONO))-
     +                  MAX(0.,ONIT(:,:)-QQQM(:,:,NIV,iONIT))-
#ifdef RACMOBUS
     +                  MAX(0.,TPAN(:,:)-QQQM(:,:,NIV,iTPAN))-
#endif
     +                  MAX(0.,PAN (:,:)-QQQM(:,:,NIV,iPAN )))
#endif
C
C DEPOSITION OF NOx
C
      IF (LDEPNOX) THEN
C
C SUBSTRACT SPECIES BEFORE DEPOSITION 
C
        QQQM(:,:,NIV,iNOx)=QQQM(:,:,NIV,iNOx)
     +      -HHHM(:,:,NIV,iNO)-HHHM(:,:,NIV,iNO2)
     +      -HHHM(:,:,NIV,iNO3)
C
#ifdef MASSy
        QQQM(:,:,NIV,iNOy)=QQQM(:,:,NIV,iNOy)
     +      -HHHM(:,:,NIV,iNO)-HHHM(:,:,NIV,iNO2)
     +      -HHHM(:,:,NIV,iNO3)
#endif              
        ZFACT(:,:)=MAIR*GG*DT*XPAM(:,:,NIV)/
     +            ((XPHFM(:,:,NIV)-XPHFM(:,:,NIV-1))*
     +              TGRIM(:,:,NIV)*8.314517)
    
C
C APPLY DEPOSITION FOR NO
C
        HHHM(:,:,NIV,iNO)=MAX(1.e-30,HHHM(:,:,NIV,iNO) /
     +      (1. + ZFACT(:,:)*VDEP(:,:,IHMDEP(iNO))))
C
C APPLY DEPOSITION FOR NO2
C
        HHHM(:,:,NIV,iNO2)=MAX(1.e-30,HHHM(:,:,NIV,iNO2) /
     +      (1. + ZFACT(:,:)*VDEP(:,:,IHMDEP(iNO2))))
C
C APPLY DEPOSITION FOR NO3
C
        HHHM(:,:,NIV,iNO3)=MAX(1.e-30,HHHM(:,:,NIV,iNO3) /
     +      (1. + ZFACT(:,:)*VDEP(:,:,IHMDEP(iNO3))))
C
C ADD SPECIES AFTER DEPOSITION
C
        QQQM(:,:,NIV,iNOx)=QQQM(:,:,NIV,iNOx)
     +      +HHHM(:,:,NIV,iNO)+HHHM(:,:,NIV,iNO2)
     +      +HHHM(:,:,NIV,iNO3) 
#ifdef MASSy
C
        QQQM(:,:,NIV,iNOy)=QQQM(:,:,NIV,iNOy)
     +      +HHHM(:,:,NIV,iNO)+HHHM(:,:,NIV,iNO2)
     +      +HHHM(:,:,NIV,iNO3) 
#endif
      ENDIF
C
      DEALLOCATE(ZFACT )
#ifdef MASSy
      DEALLOCATE(NOx   )
      DEALLOCATE(HNO3  )
      DEALLOCATE(N2O5  )
      DEALLOCATE(HNO4  )
      DEALLOCATE(HONO  )
      DEALLOCATE(ONIT  )
      DEALLOCATE(PAN   )
      DEALLOCATE(OLNN  )
#ifdef RACMOBUS
      DEALLOCATE(OLND  )
      DEALLOCATE(TPAN  )
#endif
      DEALLOCATE(CLONO2)
      DEALLOCATE(CLNO2 )
      DEALLOCATE(BRONO2)
      DEALLOCATE(CLOx  )
      DEALLOCATE(OCLO  )
      DEALLOCATE(HCL   )
      DEALLOCATE(HOCL  )
      DEALLOCATE(CL2   )
      DEALLOCATE(BRCL  )
      DEALLOCATE(BROx  )
      DEALLOCATE(HBR   )
      DEALLOCATE(HOBR  )
#endif
C
      RETURN
      END
