      SUBROUTINE ELASSIM
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Data assimilation
C   Purpose    : Equivalent latitude assimilation of ozone
C   Called by  :
C
CCC
C
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 01/2002
C
C Modified  :
C
CCC
C
	USE PARADI_MOD
	USE FORMT_MOD
	USE MRAT_MOD
	USE MOCF_MOD
	USE DATOBS_MOD
	USE THELOBS_MOD
C
	IMPLICIT NONE	
C	
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: NTHETA,NEL,IO3,ITRA,ILON,I,LOBS,ITHOBS,IELOBS,K,ITEST2
	REAL (KIND=8) :: XA,XB,X,D1,D2
	PARAMETER (NTHETA=15,NEL=360)
C XGRIM (AXGRIM) : VARIABLE IN (lon,lat,niv) COORDINATES FOR GUESS (ANALYSIS)
C XGRIT (AXGRIT) : VARIABLE IN (lon,lat,theta) COORDINATES FOR GUESS (ANALYSIS)
C XELT  (AXELT)  : VARIABLE IN (el,theta) COORDINATES FOR GUESS (ANALYSIS)
	REAL (KIND=8) , ALLOCATABLE :: XGRIM1(:,:,:),AXGRIM1 (:,:,:)
	REAL (KIND=8) , ALLOCATABLE :: XGRIT1(:,:,:)
	REAL (KIND=8) , ALLOCATABLE :: ELGRIT1(:,:,:),AXGRIT1(:,:,:)
	REAL (KIND=8) , ALLOCATABLE :: XELT1(:,:),AXELT1(:,:)
	REAL (KIND=8) , ALLOCATABLE :: class(:),moymod(:),nmoy(:)
	REAL (KIND=8) , ALLOCATABLE :: model(:),modobs(:)
C
#ifdef MIDATM
#define CPP_O3 'O_3'
#else
#define CPP_O3 'O_x'
#endif
C
      ALLOCATE (XGRIM1(LONM1,LATM1,NIVM1))     ; XGRIM1(:,:,:) = 0.0
      ALLOCATE (AXGRIM1(LONM1,LATM1,NIVM1))    ; AXGRIM1(:,:,:) = 0.0
      ALLOCATE (XGRIT1(LONM1,LATM1,NTHETA))    ; XGRIT1(:,:,:) = 0.0
      ALLOCATE (ELGRIT1(LONM1,LATM1,NTHETA))    ; ELGRIT1(:,:,:) = 0.0
      ALLOCATE (AXGRIT1(LONM1,LATM1,NTHETA))    ; AXGRIT1(:,:,:) = 0.0
      ALLOCATE (XELT1(NEL,NTHETA))    ; XELT1(:,:) = 0.0
      ALLOCATE (AXELT1(NEL,NTHETA))    ; AXELT1(:,:) = 0.0
	ALLOCATE (class(10))    ; class(:) = 0.0
	ALLOCATE (moymod(10))	    ; moymod(:) = 0
	ALLOCATE (nmoy(10))    ; nmoy(:) = 0
	ALLOCATE (model(10))    ; model(:) = 0
	ALLOCATE (modobs(10))	; modobs(:) = 0
C
	ALLOCATE (THOBS(NOBSMAX,NBNIVPRESMAX)) ; THOBS(:,:) = 0.0
	ALLOCATE (ELOBS(NOBSMAX,NBNIVPRESMAX)) ; ELOBS(:,:) = 0.0
C
CCC 0. SELECT OZONE FIELD
C
      iO3=0
      DO itra=1,NTRA1
        IF (TRANAM1(itra).EQ.CPP_O3) iO3=itra
      ENDDO
      IF (iO3.GT.0) THEN
        DO ilon=1,LOLANIM1
           XGRIM1(ilon,1,1)=QQQM1(ilon,1,1,iO3)
          AXGRIM1(ilon,1,1)=QQQM1(ilon,1,1,iO3)
        ENDDO
      ELSE
        WRITE(*,*) '> O3 IS MISSING : ABORT E.L. ASSIMILATION!'
        CALL MOCAGE_STOP
      ENDIF
C
CCC 1. PROJECT OZONE FROM (lon,lat,niv) TO (lon,lat,theta) COORDINATES
C
      IF (IFLAG1) THEN
        CALL GRIM2T(LONM1,LATM1,NIVM1,NTHETA,
     +              TGRIM1,XPAM1,
     +              XGRIT1,XGRIM1,1)
      ENDIF
C
CCC 2. COMPUTE OZONE EQUIVALENT LATITUDE FOR (lon,lat,theta) GRID AND
C      AT OBSERVATIONS LOCATIONS
C
      IF (IFLAG1) THEN
        CALL O3ELCALC(LONM1,LATM1,NIVM1,NTHETA,
     +                TGRIM1,XPAM1,XGRIT1,
     +                ELGRIT1)
      ENDIF
C
CCC 3. PROJECT OZONE FROM (lon,lat,theta) TO (EL,theta) COORDINATES
C
      IF (IFLAG1) THEN
        CALL GRIT2EL(LONM1,LATM1,NIVM1,NTHETA,NEL,
     +              XGRIT1,ELGRIT1,XELT1,1)
      ENDIF
C
CCC 4. PERFORM ASSIMILATION IN (EL,theta) COORDINATES
C
CCC 4.1 PREPARE INCREMENTAL ASSIMILATION
C
      IF (IFLAG1) THEN
C
        CALL GRIT2EL(LONM1,LATM1,NIVM1,NTHETA,NEL,
     +               XGRIT1,ELGRIT1,XELT1,-1)
C
        CALL GRIM2T(LONM1,LATM1,NIVM1,NTHETA,
     +              TGRIM1,XPAM1,
     +              XGRIT1,XGRIM1,-1)
C
      ENDIF
C
CCC 4.2 REMOVE BIAS
C
      DO i=1,10
        class(i)=real(i-1)*25.+25.*i
        moymod(i)=0.
        nmoy(i)=0.
      ENDDO
C
      DO lobs=1,nobs
      IF ((THobs(lobs,1).GE.310.).AND.
     +    (VAL(lobs,1,1).GT.0.).AND.
     +    (typemesure(lobs).EQ.'FLIGHT')) THEN
        iTHobs=int(((THobs(lobs,1)-THETAMIN)/DELTATH)-0.5)+1
        iELobs=MIN(1+INT((ELobs(lobs,1)+90.)*REAL(NEL)/180.),NEL)
        DO i=1,10
          IF ((((class(i)-25.)*1.e-9).LE.VAL(lobs,1,1)).AND.
     +        (((class(i)+25.)*1.e-9).GT.VAL(lobs,1,1))) THEN
            moymod(i)=moymod(i)+XELT1(iELobs,iTHobs)
            nmoy(i)=nmoy(i)+1.
          ENDIF
        ENDDO
      ENDIF
      ENDDO
C     
      DO i=1,10
        IF (nmoy(i).NE.0.) THEN
          model(i)=moymod(i)/nmoy(i)
          modobs(i)=(class(i)*1.e-9)-model(i)
        ELSE 
          xa=(model(i-2)-model(i-1))/
     +       ((class(i-2)-class(i-1))*1.e-9)
          xb=model(i-1)-xa*class(i-1)*1.e-9
          model(i)=xa*class(i)*1.e-9+xb
          modobs(i)=(class(i)*1.e-9)-model(i)
        ENDIF
      ENDDO
C
      DO i=1,NEL*NTHETA
        itest2=0
        DO k=1,9
          IF ((XELT1(i,1).GE.model(k)).AND.
     +        (XELT1(i,1).LE.model(k+1)).AND.
     +        (itest2.EQ.0)) THEN
            d1=model(k+1)-XELT1(i,1)
            d2=model(k+1)-model(k)
            x=d1/d2
            XELT1(i,1)=XELT1(i,1)+modobs(k+1)*(1.-x)+
     +                            modobs(k)*x
             itest2=1
          ENDIF
        ENDDO
      ENDDO
C
CCC 4.2 PERFORM ASSIMILATION
C
      IF (IFLAG1) THEN
        CALL ASSIMELT(NTHETA,NEL,XELT1,AXELT1)
      ENDIF
C
CCC 5. PROJECT ASSIMILATED AND GUESS OZONE FROM (EL,theta) TO
C      (lon,lat,theta) COORDINATES
C
      IF (IFLAG1) THEN
        CALL GRIT2EL(LONM1,LATM1,NIVM1,NTHETA,NEL,
     +               AXGRIT1,ELGRIT1,AXELT1,-1)
      ENDIF                
C
CCC 6. PROJECT ASSIMILATED AND GUESS OZONE FROM (lon,lat,theta) TO
C      (lon,lat,niv) COORDINATES
C
      IF (IFLAG1) THEN
        CALL GRIM2T(LONM1,LATM1,NIVM1,NTHETA,
     +              TGRIM1,XPAM1,
     +              AXGRIT1,AXGRIM1,-1)
      ENDIF
C
CCC 7. UPTADE MODEL OZONE ACCORDING TO INCREMENTS COMPUTED IN
C      MODEL (lon,lat,niv) COORDINATES
C
      IF (IFLAG1) THEN
        DO ilon=1,LOLANIM1
          QQQM1(ilon,1,1,iO3)=   QQQM1(ilon,1,1,iO3)
     +                        + (AXGRIM1(ilon,1,1)-XGRIM1(ilon,1,1))
        ENDDO
      ENDIF
C
C
      DEALLOCATE (XGRIM1)    
      DEALLOCATE (AXGRIM1)
      DEALLOCATE (XGRIT1)
      DEALLOCATE (ELGRIT1)
      DEALLOCATE (AXGRIT1)
      DEALLOCATE (XELT1)
      DEALLOCATE (AXELT1)
	DEALLOCATE (class)
	DEALLOCATE (moymod)	
	DEALLOCATE (nmoy)
	DEALLOCATE (model)
	DEALLOCATE (modobs)	
C HLB
C
	DEALLOCATE (THOBS)
	DEALLOCATE (ELOBS)
C
C HLB
C

      RETURN
      END
