      SUBROUTINE ELSOBSADAPT(NTHETA,NEL,XELT,iTH,iEL,
     +                       del,dth,dd,dg)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Data assimilation
C   Purpose    : (EL,theta) assimilation 
C   Called by  :
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 02/2002
C
C Modified  :
C
CCC
C
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: NTHETA,NEL,ITH,IEL
	REAL (KIND=8) :: DEL,DTH,DD,DG
	REAL (KIND=8) , DIMENSION (NEL,NTHETA) :: XELT
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C		
	INTEGER :: I,J,NTEST,IDG,IDD,ITESTD,ITESTG
	REAL (KIND=8) :: DELTAEL,ozhmax1,ozhmax2,ozhmin1,ozhmin2
	REAL (KIND=8) :: ozmax,ozmin
      REAL (KIND=8) , ALLOCATABLE :: OZEQ(:),ozinterp(:)
C
CCC 0. INITIALIZATIONS
C
	ALLOCATE (OZEQ(NEL))
	ALLOCATE (ozinterp(0:NEL))
C	
      DELTAEL=REAL(180./NEL)
      DO i=1,NEL
        OZEQ(i)=(i-0.5)*DELTAEL-90.
      ENDDO
      ozinterp(0)=999.
C
      IF (iTH.LT.(NTHETA-1)) THEN
        ozhmax1=   del*XELT(iEL+1,iTH+2)+
     $         (1.-del)*XELT(iEL,iTH+2)
        ozhmax2=   del*XELT(iEL+1,iTH+1)+
     $         (1.-del)*XELT(iEL,iTH+1)
        ozmax=dth*ozhmax1+(1.-dth)*ozhmax2
      ELSE
        ozmax=0.
      ENDIF
c
      IF (iTH.GT.1) THEN
        ozhmin1=   del*XELT(iEL+1,iTH)+
     $         (1.-del)*XELT(iEL,iTH)
        ozhmin2=   del*XELT(iEL+1,iTH-1)+
     $         (1.-del)*XELT(iEL,iTH-1)
        ozmin=dth*ozhmin1+(1.-dth)*ozhmin2
      ELSE
        ozmin=0.
      ENDIF
C
      IF (XELT(iEL,iTH).GE.XELT(iEL-1,iTH)) then
        ntest=0
      ELSE
        ntest=1
      ENDIF
C
CCC 1. INTERPOLATION BETWEEN ithinf and ithinf+1
C
      idg=0
      idd=0
      itestd=0
      itestg=0
C
      DO i=1,NEL
C
        IF (iTH.NE.NTHETA) THEN
          ozinterp(i)= dth*XELT(i,iTH+1)+
     $             (1.-dth)*XELT(i,iTH)
        ELSE
          ozinterp(i)= XELT(i,iTH)
        ENDIF
C
        IF (ntest.EQ.0) THEN       
          IF ((ozinterp(i).LE.ozmin).AND.
     $        (XELT(i,iTH+1).NE.-999.).AND.
     $        (XELT(i,iTH).NE.-999.)) then 
            idg=i 
          ENDIF
          IF ((ozinterp(i).LE.ozmax).AND.
     $        (XELT(i,iTH+1).NE.-999.).AND.
     $        (XELT(i,iTH).NE.-999.)) then 
            idd=i
          ENDIF
        ELSE
          IF (ozinterp(i).GT.ozinterp(i-1)) then
            itestg=1
            itestd=1
          ENDIF
          IF ((ozinterp(i).GE.ozmin).AND.
     $        (ozinterp(i).LE.ozinterp(i-1)).AND.
     $        (iTH.GT.1).AND.(itestg.EQ.0).AND.
     $        (XELT(i,iTH+1).NE.-999.).AND.
     $        (XELT(i,iTH).NE.-999.)) then 
            idg=i
          ENDIF
          IF ((ozinterp(i).GE.ozmax).AND.
     $        (ozinterp(i).LE.ozinterp(i-1)).AND.
     $        (iTH.LT.(NTHETA-1)).AND.
     $        (itestd.EQ.0).AND.
     $        (XELT(i,iTH+1).NE.-999.).AND.
     $        (XELT(i,iTH).NE.-999.)) then 
            idd=i
          ENDIF
        ENDIF               
C
      ENDDO
C
      IF (idg.GT.idd) THEN
          j=idg
          idg=idd
          idd=j
      ENDIF
C
      dd= OZEQ(idd)-OZEQ(iEL)
      IF ((iTH.GT.1).AND.(iTH.LT.NTHETA)) THEN
        dg= OZEQ(idg)-OZEQ(iEL)
      ELSE
        dg=-dd
      ENDIF
C
	DEALLOCATE (OZEQ)
	DEALLOCATE (ozinterp)
C
C
      RETURN
      END
