      SUBROUTINE ELSOBSASSIM(Nsobs,nsupobsmax,NTHETA,NEL,
     +                       delREF,THsobs,ELsobs,somg,dg,dd,
     +                       XELTIN,XELTOUT)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Data assimilation
C   Purpose    : Assimilation of superobservations in (EL,theta)
C                coordinates
C   Called by  :
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 02/2002
C
C Modified  :
C
CCC
C
	USE PARADI_MOD
C
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: NSUPOBSMAX,NTHETA,NEL,NSOBS
	REAL (KIND=8) :: delREF
	REAL (KIND=8) , DIMENSION (nsupobsmax) :: THsobs,ELsobs,somg
      REAL (KIND=8) , DIMENSION (nsupobsmax) :: dd,dg
      REAL (KIND=8) , DIMENSION (NEL,NTHETA) :: XELTIN,XELTOUT	
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: ITH,IEL,I,L
	REAL (KIND=8) :: XDIST,DELTAEL,delTH,delEL,polTH,polEL
      PARAMETER (Xdist=0.01)
C
	REAL (KIND=8) , ALLOCATABLE :: xnum(:,:),xden(:,:)
	REAL (KIND=8) , ALLOCATABLE :: NTH(:)
      REAL (KIND=8) , ALLOCATABLE :: OZEQ(:)
C
C CRESSMAN COEFFICIENTS (5th ORDER)
C
      REAL (KIND=8), parameter, dimension(6) :: cofEL =
     $      (/ 0.713367555, -0.343354805,    0.12246838,
     $        -0.0251689401, 0.00241957423, -8.553542e-05/)
      REAL (KIND=8), parameter, dimension(6) :: cofTH =
     $      (/ 0.62597478,  -0.293449249,   0.0948295561,
     $        -0.0170297118, 0.0014529468, -4.6740599e-05/)
C	
CCC 0. INITIALIZATIONS
C
	ALLOCATE (xnum(NEL,NTHETA))
	ALLOCATE (xden(NEL,NTHETA))
	ALLOCATE (NTH(NTHETA))
	ALLOCATE (OZEQ(NEL))
C
	DO ith=1,NTHETA
        NTH(ith)=THETAMIN+REAL(ith-1)*DELTATH
      ENDDO
C
      DELTAEL=REAL(180./NEL)
      DO iEL=1,NEL
        OZEQ(iEL)=(iEL-0.5)*DELTAEL-90.
      ENDDO
C
      DO i=1,NEL*NTHETA
        xnum(i,1)=0.
        xden(i,1)=0.
      ENDDO
C
CCC 1. FOR EACH GRIDPOINT, COMPUTE CONTRIBUTION TO INCREMENT
C      FROM EACH SUPEROBSERVATION
C
      DO l=1,Nsobs
        DO ith=1,NTHETA
        DO iEL=1,NEL
          delTH=abs(NTH(ith)-THsobs(l))
          delEL= OZEQ(iEL)-ELsobs(l)
          IF (delTH.LE.delREF) THEN
            polTH=cofTH(1)+cofTH(2)*delTH+
     +            cofTH(3)*(delTH**2.)+
     +            cofTH(4)*(delTH**3.)+
     +            cofTH(5)*(delTH**4.)+
     +            cofTH(6)*(delTH**5.)
            IF ((delEL.LT.0.).AND.
     +          (delEL.GE.dg(l))) THEN
              polEL=cofEL(1)+cofEL(2)*abs(delEL)+
     +              cofEL(3)*(delEL**2.)+
     +              cofEL(4)*(abs(delEL)**3.)+
     +              cofEL(5)*(delEL**4.)+
     +              cofEL(6)*(abs(delEL)**5.)
              IF ((polEL.GE.Xdist).AND.(polTH.GE.Xdist)) then
                xnum(iEL,ith)=xnum(iEL,ith)+
     +                        polEL*polTH*somg(l)
                xden(iEL,ith)=xden(iEL,ith)+polEL*polTH
              ENDIF
            ENDIF
            IF ((delEL.GE.0.).AND.
     +          (delEL.LE.dd(l))) THEN
              polEL=cofEL(1)+cofEL(2)*delEL+
     +              cofEL(3)*(delEL**2.)+
     +              cofEL(4)*(delEL**3.)+
     +              cofEL(5)*(delEL**4.)+
     +              cofEL(6)*(delEL**5.)
              IF ((polEL.GE.Xdist).AND.(polTH.GE.Xdist)) THEN
                xnum(iEL,ith)=xnum(iEL,ith)+
     +                        polEL*polTH*somg(l)
                xden(iEL,ith)=xden(iEL,ith)+polEL*polTH
              ENDIF
            ENDIF
          ENDIF
        ENDDO
        ENDDO
      ENDDO
C
CCC 2. ADD INCREMENT TO INITIAL FIELD
C
      DO i=1,NEL*NTHETA
        xden(i,1)=xden(i,1)+cofEL(1)*cofTH(1)
        XELTOUT(i,1)=MAX(1.e-30,
     +                   XELTIN(i,1)+xnum(i,1)/xden(i,1))
      ENDDO
C
	DEALLOCATE (xnum)
	DEALLOCATE (xden)
	DEALLOCATE (NTH)
	DEALLOCATE (OZEQ)
C	
	RETURN
      END
