      SUBROUTINE EXT2D(X2D,NDIM1,NDIM2,RSYMY,IGR)                           
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : 3D semi-lagrangian transport scheme
C   Purpose   : Initialize X2D on extended grid for interpolations
C   Called by : 
C
CCC
C
C  Original : P. Simon, 03/1999
C
C  Modified : V.-H. Peuch (inclusion dans MOCAGE), 04/1999
C             P. Simon    (distinction selon IGR), 08/1999
C
CCC
C                                                                       
	IMPLICIT NONE
C	
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: IGR,NDIM1,NDIM2
	REAL (KIND=8) :: RSYMY
	REAL (KIND=8) , DIMENSION (-1:NDIM1+2,-1:NDIM2+2) :: X2D
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: ISYM,J,K
C
C
C  si IGR=1: la grille est globale
C
      IF(IGR.EQ.1) THEN
C
C  direction meridienne: symetrie par rapport aux poles
C
      DO 11 J=1,NDIM1
         ISYM=MOD(J+NDIM1/2-1,NDIM1)+1
         X2D(J,     -1)=RSYMY*X2D(ISYM,      2)
         X2D(J,      0)=RSYMY*X2D(ISYM,      1)
         X2D(J,NDIM2+1)=RSYMY*X2D(ISYM,  NDIM2)
         X2D(J,NDIM2+2)=RSYMY*X2D(ISYM,NDIM2-1)
 11   CONTINUE
C
C  direction zonale: periodicite
C
      DO 12 K=-1,NDIM2+2
         X2D(     -1,K)=X2D(NDIM1-1,K)
         X2D(      0,K)=X2D(  NDIM1,K)
         X2D(NDIM1+1,K)=X2D(      1,K)
         X2D(NDIM1+2,K)=X2D(      2,K)
 12   CONTINUE
C
C  sinon: la grille est locale
C
      ELSE
C
C  direction meridienne: gradient nul
C
      DO 21 J=1,NDIM1
         X2D(J,     -1)=X2D(J,    1)
         X2D(J,      0)=X2D(J,    1)
         X2D(J,NDIM2+1)=X2D(J,NDIM2)
         X2D(J,NDIM2+2)=X2D(J,NDIM2)
 21   CONTINUE
C
C  direction zonale: gradient nul
C
      DO 22 K=-1,NDIM2+2
         X2D(     -1,K)=X2D(    1,K)
         X2D(      0,K)=X2D(    1,K)
         X2D(NDIM1+1,K)=X2D(NDIM1,K)
         X2D(NDIM1+2,K)=X2D(NDIM1,K)
 22   CONTINUE
C
      ENDIF
C
      RETURN
      END
C
