      SUBROUTINE EXT3D(X3D,NDIM1,NDIM2,NDIM3,RSYMY,IGR)                     
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : 3D semi-lagrangian transport scheme 
C   Purpose   : Initialize X3D on the extended grid for interpolations
C   Called by : 
C
CCC
C
C  Original : P. Simon, 03/1999
C
C  Modified : V.-H. Peuch (inclusion dans MOCAGE), 04/1999
C             P. Simon    (distinction selon IGR), 08/1999
C
CCC
C
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: IGR,NDIM1,NDIM2,NDIM3
      REAL (KIND=8) :: RSYMY
	REAL (KIND=8) , DIMENSION (-1:NDIM1+2,-1:NDIM2+2,0:NDIM3) :: X3D  
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: J,ISYM,L,K
C
C
C  si IGR=1: la grille est globale
C
      IF(IGR.EQ.1) THEN
C
C  direction meridienne: symetrie par rapport aux poles
C
      DO 11 J=1,NDIM1
         ISYM=MOD(J+NDIM1/2-1,NDIM1)+1
         DO 110 L=0,NDIM3
            X3D(J,     -1,L)=RSYMY*X3D(ISYM,      2,L)
            X3D(J,      0,L)=RSYMY*X3D(ISYM,      1,L)
            X3D(J,NDIM2+1,L)=RSYMY*X3D(ISYM,  NDIM2,L)
            X3D(J,NDIM2+2,L)=RSYMY*X3D(ISYM,NDIM2-1,L)
 110     CONTINUE
 11   CONTINUE
C
C  direction zonale: periodicite
C
      DO 12 L=0,NDIM3
      DO 12 K=-1,NDIM2+2
         X3D(     -1,K,L)=X3D(NDIM1-1,K,L)
         X3D(      0,K,L)=X3D(  NDIM1,K,L)
         X3D(NDIM1+1,K,L)=X3D(      1,K,L)
         X3D(NDIM1+2,K,L)=X3D(      2,K,L)
 12   CONTINUE
C
C  sinon: la grille est locale
C
      ELSE
C
C  direction meridienne: gradient nul
C
      DO 21 L=0,NDIM3
      DO 21 J=1,NDIM1
         X3D(J,     -1,L)=X3D(J,    1,L)
         X3D(J,      0,L)=X3D(J,    1,L)
         X3D(J,NDIM2+1,L)=X3D(J,NDIM2,L)
         X3D(J,NDIM2+2,L)=X3D(J,NDIM2,L)
 21   CONTINUE
C
C  direction zonale: gradient nul
C
      DO 22 L=0,NDIM3
      DO 22 K=-1,NDIM2+2
         X3D(     -1,K,L)=X3D(    1,K,L)
         X3D(      0,K,L)=X3D(    1,K,L)
         X3D(NDIM1+1,K,L)=X3D(NDIM1,K,L)
         X3D(NDIM1+2,K,L)=X3D(NDIM1,K,L)
 22   CONTINUE
C
      ENDIF
C
      RETURN
      END
C
