       SUBROUTINE FLPRECSTRA(LON,LAT,NIV
     +                      ,XPHFM,XPAM
     +                      ,QGRIM,TGRIM
     +                      ,LNEIGE
     +                      ,PWAL,PSNL
     +                      ,LSTPFLX,SSTPFLX)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Physical parameterizations
C   Purpose    : Compute liquid and solid stratiform precipitation
C                flux (based upon ARPEGE/ACPLUIE routine)
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 02/2003
C
C  Modified :
C
CCC
C
	USE CSTES_MOD
	USE EXPE_MOD
C
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: LON,LAT,NIV
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: XPHFM,XPAM
	REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: QGRIM,TGRIM
	REAL (KIND=8) , DIMENSION (LON,LAT) :: PWAL,PSNL
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV):: LSTPFLX
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV):: SSTPFLX
      LOGICAL :: LNEIGE
C     WARNING: PWAL and PSNL are in m3/m2/s
C     (liquid stratiform prec. flux (kg water/m2/s))
C     (solid  stratiform prec. flux (kg water/m2/s))     
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: L,K,I,LM1,LMAX
      INTEGER , ALLOCATABLE :: ITES(:)
      REAL (KIND=8) :: DELTAP,EAUSOL,QTES,RPLUIE,ZGDTI,EVAP,FONT
      REAL (KIND=8) :: REVGSL,ZSIGMA,ZEVA,ZEPS1,ZEPS2	
      REAL (KIND=8) :: ZFON,ZRPTH,ZRPTB,ZDELTA,ZPLS
      REAL (KIND=8) :: ZRMIE,ZRMIT,FLSOL1,FLSOL2,COEF,DIFFLUX
      REAL (KIND=8) , ALLOCATABLE :: QWGRIM(:,:,:),QECH(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: ZPOID(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: ZDQ(:,:),ZFPTB(:,:),ZFPTH(:,:)
      REAL (KIND=8) , ALLOCATABLE :: ZIPB(:,:),ZIPH(:,:),ZRME(:,:) 	
      REAL (KIND=8) , ALLOCATABLE :: ZRMI(:,:),PWALLOC(:,:),PSNLLOC(:,:)	
      REAL (KIND=8) , ALLOCATABLE :: EPSILON(:)
C
C
      ALLOCATE (QWGRIM(LON,LAT,NIV))	 ;QWGRIM(:,:,:) = 0.0
      ALLOCATE (QECH  (LON,LAT,NIV))
      ALLOCATE (ZPOID (LON,LAT,NIV))
      ALLOCATE (ZDQ    (LON,LAT))
      ALLOCATE (ZFPTB  (LON,LAT))
      ALLOCATE (ZFPTH  (LON,LAT))
      ALLOCATE (ZIPB   (LON,LAT))
      ALLOCATE (ZIPH   (LON,LAT))	 ;ZIPH   (:,:) = 0.0
      ALLOCATE (ZRME   (LON,LAT))
      ALLOCATE (ZRMI   (LON,LAT))
      ALLOCATE (PWALLOC(LON,LAT))
      ALLOCATE (PSNLLOC(LON,LAT))
      ALLOCATE (ITES(NIV))
      ALLOCATE (EPSILON(NIV))
C
CCC
C  
      DO L=1,NIV
        EPSILON(L)=0.03
      ENDDO      
      DO K=1,LAT
      DO I=1,LON
        PWALLOC(I,K)=1.E3*PWAL(I,K)
        PSNLLOC(I,K)=1.E3*PSNL(I,K)
      ENDDO
      ENDDO      
C
CCC 0. COMPUTE WET BULB SPECIF. HUMIDITY
C
      CALL WETBULB(LON,LAT,NIV,
     +             XPAM,QGRIM,TGRIM,
     +             LNEIGE,
     +             QWGRIM)
C         
      DO NITER=1,2
C
       DO K=1,LAT
       DO I=1,LON      
       DO L=1,NIV
        QECH(I,K,L)=QGRIM(I,K,L) 
       ENDDO
       ENDDO
       ENDDO
C      
       DO K=1,LAT
       DO I=1,LON
        DELTAP=0.
        EAUSOL=PWALLOC(I,K)+PSNLLOC(I,K)  
        DO L=1,NIV
         LM1=MAX(1,L-1)
         ITES(L)=0
         QTES=ABS(QWGRIM(I,K,L)-QGRIM(I,K,L))
	 IF (QGRIM(I,K,L).GT.1.E-30) THEN
          QTES=QTES/ABS(QGRIM(I,K,L))
          IF(QTES.LE.EPSILON(L)) THEN
           ITES(L)=1
           DELTAP=DELTAP+XPHFM(I,K,L)-XPHFM(I,K,LM1)       
          ENDIF
	 ENDIF
        ENDDO  
        IF (DELTAP.GT.0.) THEN  
         RPLUIE=EAUSOL*GG*DT0/DELTAP  
        ENDIF
        DO L=1,NIV
         IF (ITES(L).EQ.1) THEN
          QECH(I,K,L)=QWGRIM(I,K,L)+RPLUIE
         ENDIF
        ENDDO      
       ENDDO
       ENDDO  
C
C 1. INITIALIZATIONS
C
      ZGDTI = 1./ (GG * DT0)
      EVAP=4.8E+06
      FONT=2.4E+04
      REVGSL=80.
      ZEPS1=1.E-10
      ZEPS2=1.E-05
C      
      DO K=1,LAT
      DO I=1,LON
       DO L=1,NIV
        ZPOID  (I,K,L)=0.
	LSTPFLX(I,K,L)=0.
	SSTPFLX(I,K,L)=0.
       ENDDO
       ZDQ  (I,K)=0.
       ZFPTB(I,K)=0.
       ZFPTH(I,K)=0.
       ZRME (I,K)=0.
       ZRMI (I,K)=0.
       ZIPH (I,K)= 1/XPHFM(I,K,1)
      ENDDO
      ENDDO
C      
      ZSIGMA=0.      
C
C 2. Preliminary computations of the layer's thickness
C divided by g*dt
C      
      DO K=1,LAT
      DO I=1,LON
       ZPOID(I,K,1)=XPHFM(I,K,1)*ZGDTI
      ENDDO
      ENDDO
C      
      DO L=2,NIV
      LM1=L-1
       DO K=1,LAT
       DO I=1,LON
        ZPOID(I,K,L)=(XPHFM(I,K,L)-XPHFM(I,K,LM1))*ZGDTI
       ENDDO
       ENDDO 
      ENDDO
C
C 3. Vertical loop, effective computations
C  
      DO L=1,NIV  
C      
C 3.1 Computation of provisional flux at the base
C This flux will be maintained in case of sursaturation
C (that is zdq < 0)
C
       DO K=1,LAT
       DO I=1,LON
        ZIPB (I,K) = 1./XPHFM(I,K,L)
	ZDQ  (I,K) = QWGRIM (I,K,L) - QECH (I,K,L)
	ZFPTB(I,K) = MAX ( 0.,  
     +                     ZFPTH (I,K) - ZPOID(I,K,L)* ZDQ (I,K) )
       ENDDO
       ENDDO 
C
C 3.2 ZSIGMA is used to make sure that at a given level,
C precipitations occur. If not, computations of melting and 
C evaporation are useless.
C
C 3.2.1 If Zsigma > 0 , evaporation and melting are computed
C
       IF (ZSIGMA . GT . 0. ) THEN
       DO K=1,LAT
       DO I=1,LON
        ZEVA=EVAP * (1. - ZRME(I,K)*(1.-REVGSL) )
	ZFON=FONT * (1. - ZRME(I,K)*(1.-REVGSL) )
C
C 3.2.2 Correction of the flux including evaporation
C       
        ZRPTH=SQRT( MAX( ZFPTH(I,K),ZFPTB(I,K) ) )
	ZRPTB=MAX ( 0. ,
     +              ZRPTH - ZEVA 
     +                    * MAX( 0.,ZDQ(I,K) ) 
     +                    * ( ZIPH(I,K) - ZIPB (I,K) ) )
        ZFPTB(I,K) = MAX ( ZFPTB (I,K) , ZRPTB**2 )
C
C 3.2.3 New snow proportions due to
C the generation of preciptations and melting,
C in the 'NEIGE' case
C       	
        IF (LNEIGE) THEN
	 ZDELTA = MAX ( 0. , SIGN( 1., RTT-TGRIM(I,K,L) ) )
C    (if T (C) <0 ZDELTA=1, else ZDELTA =0)
         ZPLS = ZDELTA * 
     +          (1. - EXP( - (RTT-TGRIM(I,K,L))**2 * ZGAUSS) )
C     
         ZRMIE = ZPLS   +  
     +         ( ZRME(I,K) -ZPLS   ) * ZFPTH(I,K) 
     +         / MAX ( ZEPS1+ZFPTH(I,K) , ZFPTB(I,K) )
C     
         ZRMIT = ZDELTA +  
     +         ( ZRME(I,K) -ZDELTA ) * ZFPTH(I,K) 
     +         / MAX ( ZEPS1+ZFPTH(I,K) , ZFPTB(I,K) ) 
C     
         ZRME(I,K) = MAX ( 0., MIN ( 1. , 
     +   ZRMIE + ZFON * ( RTT - TGRIM(I,K,L) )
     +                * ( ZIPH(I,K) - ZIPB(I,K) )
     +                /MAX ( ZEPS2, 0.5*(ZRPTH+ZRPTB) ) ) )
C
         ZRMI(I,K) = MAX ( 0., MIN ( 1. , 
     +   ZRMIT + ZFON * ( RTT - TGRIM(I,K,L) )
     +                * ( ZIPH(I,K) - ZIPB(I,K) )
     +                /MAX ( ZEPS2, 0.5*(ZRPTH+ZRPTB) ) ) )       
C
C 3.2.4 LNEIGE false: snow proportions = zero
C        
        ELSE
	 ZRME(I,K)=0.
	 ZRMI(I,K)=0.
	ENDIF
       ENDDO
       ENDDO       
C
C 3.3 ZSIGMA =0.
C       
       ELSE
C
C 3.3.1 Computation of ZSIGMA
C       
       ZSIGMA=0.
       DO K=1,LAT
       DO I=1,LON
        ZSIGMA=ZSIGMA+ZFPTB(I,K)
       ENDDO
       ENDDO
C
C 3.3.2 in the 'NEIGE' case, initialization
C of the snow proportions if ZSIGMA>0
C (this will be used on the next level)
C
       DO K=1,LAT
       DO I=1,LON
        IF ( LNEIGE . AND . (ZSIGMA.GT.0.) ) THEN	  
 	 ZDELTA = MAX ( 0. , SIGN( 1., RTT-TGRIM(I,K,L) ) )
C    (if T (C) <0 ZDELTA=1, else ZDELTA =0)
         ZPLS = ZDELTA * 
     +          (1. - EXP( - (RTT-TGRIM(I,K,L))**2 * ZGAUSS) )
C     
         ZRME(I,K) = ZPLS  
         ZRMI(I,K) = ZDELTA   
	ELSE	        
	 ZRME(I,K)=0.
	 ZRMI(I,K)=0.
        ENDIF
       ENDDO
       ENDDO     
C
C END of loop on ZSIGMA
C
       ENDIF
C
C 4. Swap on the next level
C          
       DO K=1,LAT
       DO I=1,LON
        ZIPH   (I,K  )= ZIPB (I,K)
	ZFPTH  (I,K  )= ZFPTB(I,K)
	LSTPFLX(I,K,L)= ZFPTB(I,K) * ( 1. - ZRMI(I,K) )
	SSTPFLX(I,K,L)= ZFPTB(I,K) *        ZRMI(I,K)   
       ENDDO
       ENDDO
C
C End of loop on levels
C 
      ENDDO
C
C Additional computation in order to
c take  reevaporation into account
C 
 
       DO K=1,LAT
       DO I=1,LON
        FLSOL1=PWALLOC(I,K)+PSNLLOC(I,K)
        FLSOL2=LSTPFLX(I,K,NIV)+SSTPFLX(I,K,NIV)      
        IF ((FLSOL1.GT.1.E-8).AND.(FLSOL2.GT.1.E-8)) THEN
         COEF=FLSOL1/FLSOL2
	 IF ((COEF.GE.0.5).AND.(COEF.LE.2.)) THEN
	  PWALLOC(I,K)=COEF*PWALLOC(I,K)
	  PSNLLOC(I,K)=COEF*PSNLLOC(I,K)
	 ENDIF
        ENDIF
	IF ((FLSOL1.GT.1.E-8).AND.(FLSOL2.LE.1.E-8)) THEN
         LMAX=NIV
	 DO L=2,NIV
	  DIFFLUX=LSTPFLX(I,K,L  )+SSTPFLX(I,K,L  )
     +           -LSTPFLX(I,K,L-1)-SSTPFLX(I,K,L-1)	  
          IF (DIFFLUX.GT.0.) THEN
	   LMAX=L
	  ENDIF
	 ENDDO
	 LMAX=MIN(LMAX,NIV-1)
	 DO L=LMAX+1,NIV
	  COEF=(XPAM(I,K,L  )-XPAM(I,K,LMAX))
     +        /(XPAM(I,K,NIV)-XPAM(I,K,LMAX))
          COEF=MIN(1.,COEF)
          COEF=MAX(0.,COEF)
	  LSTPFLX(I,K,L)=(1.-COEF)*LSTPFLX(I,K,LMAX)+COEF*PWALLOC(I,K)
	  SSTPFLX(I,K,L)=(1.-COEF)*SSTPFLX(I,K,LMAX)+COEF*PSNLLOC(I,K)
	 ENDDO
	ENDIF
       ENDDO
       ENDDO
      ENDDO
C      
      DEALLOCATE (QWGRIM)
      DEALLOCATE (QECH)
      DEALLOCATE (ZPOID)
      DEALLOCATE (ZDQ)
      DEALLOCATE (ZFPTB)
      DEALLOCATE (ZFPTH)
      DEALLOCATE (ZIPB)
      DEALLOCATE (ZIPH)
      DEALLOCATE (ZRME)
      DEALLOCATE (ZRMI) 
      DEALLOCATE (PWALLOC)
      DEALLOCATE (PSNLLOC)
      DEALLOCATE (ITES)    
      DEALLOCATE (EPSILON)
C
C
      RETURN
      END
