      SUBROUTINE GRIM2T(LON,LAT,NIV,NTHETA,
     +                  TGRIM,XPAM,
     +                  XGRIT,XGRIM,MODE)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Data assimilation
C   Purpose    : Project X from (lon,lat,niv) coordinates to
C                (lon,lat,theta) or vice-versa
C   Called by  :
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 01/2002
C
C Modified  :
C
CCC
C
	USE PARADI_MOD
C
	IMPLICIT NONE
C
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: LON,LAT,NIV,NTHETA,MODE
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV):: TGRIM,XPAM
	REAL (KIND=8) , DIMENSION (LON,LAT,NTHETA) :: XGRIT
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: XGRIM
C 	MODE =  1  : IN (XGRIM) OUT (XGRIT)
C 	MODE = -1  : IN (XGRIT) OUT (XGRIM)
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: LOLANI,LOLA,ITHETA,ILON,I,INIV
	REAL (KIND=8) :: A
      REAL (KIND=8) , ALLOCATABLE :: THGRIM(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: NTH(:)
C
C
CCC 0. INITIALIZATIONS
C
	ALLOCATE (THGRIM(LON,LAT,NIV))
	ALLOCATE (NTH(NTHETA))
C	
      LOLANI = LON*LAT*NIV
      LOLA   = LON*LAT
      DO itheta=1,NTHETA
        NTH(itheta)=THETAMIN+REAL(itheta-1)*DELTATH
      ENDDO
C
CCC 1. COMPUTE POTENTIAL TEMPERATURE
C
      DO ilon = 1,LOLANI
        THGRIM(ilon,1,1)= TGRIM(ilon,1,1)*
     +                    (100000./XPAM(ilon,1,1))**(2./7.)
      ENDDO
C
      DO i=NIV-1,2,-1
        DO ilon=1,LOLA
          IF ((THGRIM(ilon,1,i).LT.THGRIM(ilon,1,i+1)).AND.
     +        (THGRIM(ilon,1,i).LT.THGRIM(ilon,1,i-1))) THEN
             a=(XPAM(ilon,1,i+1)-XPAM(ilon,1,i))/
     +         (XPAM(ilon,1,i+1)-XPAM(ilon,1,i-1))
             THGRIM(ilon,1,i)=     a*THGRIM(ilon,1,i-1)+
     +                       (1. -a)*THGRIM(ilon,1,i+1)
          ENDIF
        ENDDO
      ENDDO

C
CCC 2. MODE = 1 : (lon,lat,niv) TO (lon,lat,theta)
C
      IF (MODE.EQ.1) THEN
        DO itheta=1,NTHETA
          DO ilon=1,LOLA
            iniv=NIV
            DO i=NIV-1,1,-1
              IF (THGRIM(ilon,1,i).LT.NTH(itheta)) iniv=i
            ENDDO
            IF (iniv.LT.NIV) THEN
              a   = (NTH(itheta)-THGRIM(ilon,1,iniv))
     +            / (THGRIM(ilon,1,iniv-1)-THGRIM(ilon,1,iniv))
              XGRIT(ilon,1,itheta) =    a * XGRIM(ilon,1,iniv-1)
     +                             + (1-a)* XGRIM(ilon,1,iniv)
            ELSE
              XGRIT(ilon,1,itheta) = XGRIM(ilon,1,NIV)
            ENDIF
          ENDDO
        ENDDO
C
C
CCC 3. MODE = -1 : (lon,lat,theta) TO (lon,lat,niv)
C
      ELSEIF (MODE.EQ.-1) THEN
        DO iniv=1,NIV
          DO ilon=1,LOLA
            itheta=NTHETA
            DO i=1,NTHETA
              if (THGRIM(ilon,1,iniv).GT.NTH(i)) itheta=i
            ENDDO
            IF (itheta.LT.NTHETA) THEN
              a= (NTH(itheta)-THGRIM(ilon,1,iniv))
     +          /(NTH(itheta)-NTH(itheta+1))
              XGRIM(ilon,1,iniv)=     a*XGRIT(ilon,1,itheta+1)+
     +                           (1.-a)*XGRIT(ilon,1,itheta)
            ENDIF
          ENDDO
        ENDDO
C
CCC
C
      ELSE
        PRINT*,'>> ERROR IN CALL TO GRIM2T : ABORT!'
        CALL MOCAGE_STOP
      ENDIF
C
CCC
C
	DEALLOCATE (THGRIM)
	DEALLOCATE (NTH)
C
C
      RETURN
      END
