      SUBROUTINE GRIT2EL(LON,LAT,NIV,NTHETA,NEL,
     +                  XGRIT,ELGRIT,XELT,MODE)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Data assimilation
C   Purpose    : Project X from (lon,lat,theta) coordinates to
C                (EL,theta) or vice-versa
C   Called by  :
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 01/2002
C
C Modified  :
C
CCC
C
	IMPLICIT NONE
C
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: LON,LAT,NIV,NTHETA,NEL,MODE
      REAL (KIND=8) , DIMENSION (LON,LAT,NTHETA) :: ELGRIT
      REAL (KIND=8) , DIMENSION (LON,LAT,NTHETA) :: XGRIT
      REAL (KIND=8) , DIMENSION (NEL,NTHETA) :: XELT
C     MODE =  1  : IN (XGRIT) OUT (XELT)
C     MODE = -1  : IN (XELT)  OUT (XGRIT)
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
	INTEGER :: LOLA,I,ITHETA,ILON,IEL,ITEST,K		
      REAL (KIND=8) , ALLOCATABLE :: XNBEL(:,:)
C
C
CCC 0. INITIALIZATIONS
C
	ALLOCATE (XNBEL(NEL,NTHETA))
C
      LOLA=LON*LAT
C
CCC 1. MODE = 1 : (lon,lat,theta) TO (EL,theta)
C
      IF (MODE.EQ.1) THEN
C
        DO i=1,NEL*NTHETA
          XNBEL(i,1)=0.
          XELT(i,1)=0.
        ENDDO
C
C COMPUTE MEAN FOR EACH (THETA,EQUIVALENT LATITUDE) CELL
C
        DO itheta=1,NTHETA
          DO ilon=1,LOLA
            iEL=MIN(1+INT((ELGRIT(ilon,1,itheta)+90.)
     +                     *REAL(NEL)/180.)
     +             ,NEL)
            iEL=MAX(1,iEL)
            XELT(iEL,itheta)= XELT(iEL,itheta)
     +                       +XGRIT(ilon,1,itheta)
            XNBEL(iEL,itheta)=XNBEL(iEL,itheta)+1.
          ENDDO
        ENDDO
C
        DO i=1,NEL*NTHETA
          IF (XNBEL(i,1).NE.0.) THEN
            XELT(i,1)=XELT(i,1)/XNBEL(i,1)
          ELSE
            XELT(i,1)=-999.
          ENDIF
        ENDDO
C
        DO i=1,NEL*NTHETA
          IF (XELT(i,1).EQ.-999.) THEN
            IF (MOD(i-1,NEL).EQ.0) THEN
              itest=0
              DO k=1,NEL-1
                IF ((XELT(i+k,1).NE.-999.).AND.
     +              (itest.EQ.0)) THEN
                  XELT(i,1)=XELT(i+k,1)
                  itest=1
                ENDIF
              ENDDO
            ELSE
              XELT(i,1)=XELT(i-1,1)
            ENDIF
          ENDIF
        ENDDO
C
CCC 2. MODE = -1 : (EL,theta) TO (lon,lat,theta)
C
      ELSEIF (MODE.EQ.-1) THEN
C
        DO itheta=1,NTHETA
          DO ilon=1,LOLA
            iEL=MIN(1+INT((ELGRIT(ilon,1,itheta)+90.)
     +                     *REAL(NEL)/180.)
     +             ,NEL)
            iEL=MAX(1,iEL)
            XGRIT(ilon,1,itheta)=XELT(iEL,itheta)
          ENDDO
        ENDDO
C
CCC
C
      ELSE
        PRINT*,'>> ERROR IN CALL TO GRIT2EL : ABORT!'
        CALL MOCAGE_STOP
      ENDIF
C
CCC 
C
	DEALLOCATE (XNBEL)
C	
      RETURN
      END
