subroutine hetero(aw,temp,ptot,hnm,ck,ws,             &
                  hhcl,hhocl,hhobr,hclono2,hhbr,      &
                  rice,nice,rnats,rnatl,nnats,nnatl,  &
                  aliq,rmean,condliq,lnat,lice,       &
                  k1,k2,k3,k4,k5,k6,k7,k8,k9,wt,      &
                  ih2o,ihno3,ihcl,ihbr,ihocl,ihobr,   &
                  iclono2,in2o5,                      &
                  lola,nbcon)

! ROUTINE TO CALCULATE UPTAKE COEFFICIENTS (GAMMA VALUES).
! GAMMA VALUES ARE INDICATED BY VARIABLES WITH PREFIX 'G', FOR
! EXAMPLE GHOCLHCL IS THE GAMMA VALUE OF HOCL DUE TO REACTION WITH
! HCL IN THE DROPLETS.
! FROM THE GAMMA VALUES, SECOND ORDER RATE CONSTANTS ARE CALCULATED.
! THESE HAVE THE PREFIX 'R' AND HAVE UNITS CM3 MOLECULE-1 S-1. FOR
! EXAMPLE, THE LOSS OF CLNO3 AND HCL DUE TO THE HETEROGENEOUS REACTION
! CLNO3+HCL -> CL2+HNO3 IS D(CLNO3)/DT (UNITS MOLECULE CM-3 S-1) =
! -RCLNO3HCL.[CLNO3].[HCL], WHERE [CLNO3] AND [HCL] ARE THE
! ****TOTAL**** AMOUNTS OF THESE SPECIES IN UNITS MOLECULE CM-3.

      	USE CSTES_MOD
	IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!	
  integer :: ih2o,ihno3,ihcl,ihbr,ihocl,ihobr,iclono2,in2o5
  integer :: LOLA,NBCON
  real (kind=8) :: RICE,RNATS,RNATL
  integer , DIMENSION (LOLA) :: LNAT,LICE,CONDLIQ
  real (kind=8) , DIMENSION (LOLA) :: TEMP,PTOT,HNM,WS,AW,NICE
  real (kind=8) , DIMENSION (LOLA) :: HHCL,HHOCL,HHOBR,HCLONO2,HHBR
  real (kind=8) , DIMENSION (LOLA) :: NNATS,NNATL,ALIQ,RMEAN,WT
  real (kind=8) , DIMENSION (LOLA) :: K1,K2,K3,K4,K5,K6,K7,K8,K9
  real (kind=8) , DIMENSION (LOLA,NBCON) :: CK
!
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
  integer :: ilon
  real (kind=8) :: t,t0,zh2o,zhcl,zclono2,zhocl,zhobr,zhbr
  real (kind=8) :: biga,eta,ah,dclono2,cclono2,gbh2o,grxn,gbhcl
  real (kind=8) :: gs,fhcl,gps,gbhclp,gb,gclono2,gclono2hcl
  real (kind=8) :: gclono2h2o
  real (kind=8) :: rclono2h2o,rclono2hcl,chocl,dhocl,fhocl,ghoclhcl
  real (kind=8) :: rhoclhcl,ratio,dhobr,fhobr,chobr,ghobrhcl
  real (kind=8) :: rhobrhcl,dhcl,chcl,gn2o5h2o,cbar,rn2o5h2o,gxrn
  real (kind=8) :: gbrno3h2o,rbrno3h2o,gclno3hclnat,gclno3h2onat
  real (kind=8) :: gn2o5h2onat,gn2o5hclnat,ghoclhclnat,gbrno3h2onat
  real (kind=8) :: ghobrhclnat,ghobrhbrnat,ghoclhbrnat,rclno3h2onat
  real (kind=8) :: rclno3hclnat,rn2o5h2onat,rbrno3h2onat,rn2o5hclnat
  real (kind=8) :: rhoclhclnat,rhobrhclnat,gclno3hclice,gclno3h2oice
  real (kind=8) :: gn2o5h2oice,gn2o5hclice,ghoclhclice,gbrno3h2oice
  real (kind=8) :: ghobrhclice,ghobrhbrice,ghoclhbrice,rclno3hclice
  real (kind=8) :: rclno3h2oice,rn2o5h2oice,rn2o5hclice,rhoclhclice
  real (kind=8) :: rbrno3h2oice,rhobrhclice,rhobrhbrice
  real (kind=8) , allocatable :: K1L(:), K2L(:), K3L(:)
  real (kind=8) , allocatable :: K4L(:), K5L(:), K6L(:)
  real (kind=8) , allocatable :: K7L(:), K8L(:), K9L(:)

  real (kind=8) , allocatable :: K1S(:), K2S(:), K3S(:)
  real (kind=8) , allocatable :: K4S(:), K5S(:), K6S(:)
  real (kind=8) , allocatable :: K7S(:), K8S(:), K9S(:)
!
!     additions pour jpl 2000
!
  real (kind=8) :: kh, kh2o, khydr
  real (kind=8) :: phcl, mhcl, khcl
  real (kind=8) :: pclono2, lclono2, fclono2
  real (kind=8) :: phobr, mhobr, k1hobrhcl, k2hobrhcl
  real (kind=8) :: phbr, mhbr, lhbr, k1hoclhbr, k2hoclhbr
  real (kind=8) :: khoclhcl, lhocl, phocl, mhocl
  real (kind=8) :: lhobr, lhcl
!
  ALLOCATE (K1L(LOLA))
  ALLOCATE (K2L(LOLA))
  ALLOCATE (K3L(LOLA))
  ALLOCATE (K4L(LOLA))
  ALLOCATE (K5L(LOLA))
  ALLOCATE (K6L(LOLA))
  ALLOCATE (K7L(LOLA))
  ALLOCATE (K8L(LOLA))
  ALLOCATE (K9L(LOLA))
  ALLOCATE (K1S(LOLA))
  ALLOCATE (K2S(LOLA))
  ALLOCATE (K3S(LOLA))
  ALLOCATE (K4S(LOLA))
  ALLOCATE (K5S(LOLA))
  ALLOCATE (K6S(LOLA))
  ALLOCATE (K7S(LOLA))
  ALLOCATE (K8S(LOLA))
  ALLOCATE (K9S(LOLA))


! ******************************************************
! THE FOLLOWING PARAMETERS ARE NEEDED IN THIS ROUTINE!!
! ******************************************************
!
! RNAT=MEAN RADIUS OF NAT PARTICLES (CM)
! NNAT=NUMBER OF NAT PARTICLES PER CM3 AIR
! RICE=MEAN RADIUS OF ICE PARTICLES (CM)
! NICE=NUMBER OF ICE PARTICLES PER CM3 AIR
! RMEAN=MEAN RADIUS OF LIQUID AEROSOL DISTRIBUTION (CM)
! ALIQ=TOTAL AREA OF LIQUID AEROSOLS (CM2 CM-3 AIR)


!$omp parallel &
!$omp default(none) &
!$omp private(AH,BIGA,CBAR,CCLONO2,CHCL,CHOBR,CHOCL,DCLONO2,DHCL,DHOBR,DHOCL) &
!$omp private(ETA,FCLONO2,FHCL,FHOBR,FHOCL,GB,GBH2O,GBHCL,GBHCLP,GBRNO3H2O) &
!$omp private(GBRNO3H2OICE,GBRNO3H2ONAT,GCLNO3H2OICE,GCLNO3H2ONAT,GCLNO3HCLICE,GCLNO3HCLNAT) &
!$omp private(GCLONO2,GCLONO2H2O,GCLONO2HCL,GHOBRHBRICE,GHOBRHBRNAT,GHOBRHCL) &
!$omp private(GHOBRHCLICE,GHOBRHCLNAT,GHOCLHBRICE,GHOCLHBRNAT,GHOCLHCL,GHOCLHCLICE) &
!$omp private(GHOCLHCLNAT,GN2O5H2O,GN2O5H2OICE,GN2O5H2ONAT,GN2O5HCLICE,GN2O5HCLNAT) &
!$omp private(GPS,GRXN,GS,GXRN,ILON,K1HOBRHCL,K2HOBRHCL,KH,KH2O,KHCL,KHOCLHCL,KHYDR) &
!$omp private(LCLONO2,LHCL,LHOBR,LHOCL,MHCL,MHOBR,PCLONO2,PHBR,PHCL,PHOBR,PHOCL,RATIO) &
!$omp private(RBRNO3H2O,RBRNO3H2OICE,RBRNO3H2ONAT,RCLNO3H2OICE,RCLNO3H2ONAT,RCLNO3HCLICE) &
!$omp private(RCLNO3HCLNAT,RCLONO2H2O,RCLONO2HCL,RHOBRHBRICE,RHOBRHCL,RHOBRHCLICE,RHOBRHCLNAT) &
!$omp private(RHOCLHCL,RHOCLHCLICE,RHOCLHCLNAT,RN2O5H2O,RN2O5H2OICE,RN2O5H2ONAT,RN2O5HCLICE) &
!$omp private(RN2O5HCLNAT,T,T0,ZCLONO2,ZH2O,ZHBR,ZHCL,ZHOBR,ZHOCL) &
!$omp shared(ALIQ,AW,CK,CONDLIQ,HCLONO2,HHCL,HHOBR,HHOCL,HNM) &
!$omp shared(ICLONO2,IH2O,IHBR,IHCL,IHOBR,IHOCL) &
!$omp shared(K1,K1L,K1S,K2,K2L,K2S,K3,K3L,K3S,K4,K4L,K4S,K5,K5L,K5S,K6,K6L,K6S) &
!$omp shared(K7,K7L,K7S,K8,K8L,K8S,K9,K9L,K9S,LICE,LNAT,LOLA,NICE,NNATL,NNATS,PREF,PTOT,RICE) &
!$omp shared(RMEAN,RNATL,RNATS,TEMP,WS,WT)

!$omp do
DO ILON = 1,LOLA
      K1L(ILON) = 0.0 ; K1S(ILON) = 0.0
      K2L(ILON) = 0.0 ; K2S(ILON) = 0.0
      K3L(ILON) = 0.0 ; K3S(ILON) = 0.0
      K4L(ILON) = 0.0 ; K4S(ILON) = 0.0
      K5L(ILON) = 0.0 ; K5S(ILON) = 0.0
      K6L(ILON) = 0.0 ; K6S(ILON) = 0.0
      K7L(ILON) = 0.0 ; K7S(ILON) = 0.0
      K8L(ILON) = 0.0 ; K8S(ILON) = 0.0
      K9L(ILON) = 0.0 ; K9S(ILON) = 0.0
END DO

!$omp do
DO ILON = 1,LOLA
   if (condliq(ILON) == 1) then

      T = TEMP(ILON)

      ! Z VARIABLES ARE SET TO AVOID DIVISION BY ZERO

      ZH2O    = MAX(CK(ILON,ih2o   ),1.E-08*HNM(ILON))
      ZHCL    = MAX(CK(ILON,ihcl   ),1.E-13*HNM(ILON))
      ZCLONO2 = MAX(CK(ILON,iclono2),1.E-13*HNM(ILON))
      ZHOCL   = MAX(CK(ILON,ihocl  ),1.E-13*HNM(ILON))
      ZHOBR   = MAX(CK(ILON,ihobr  ),1.E-15*HNM(ILON))
      ZHBR    = MAX(CK(ILON,ihbr   ),1.E-15*HNM(ILON))

      ! pressions de hcl, clono2, hobr, et hbr (atm)

      phcl    = (zhcl   /hnm(ILON))*ptot(ILON)/PREF
      pclono2 = (zclono2/hnm(ILON))*ptot(ILON)/PREF
      phocl   = (zhocl  /hnm(ILON))*ptot(ILON)/PREF
      phobr   = (zhobr  /hnm(ILON))*ptot(ILON)/PREF
      phbr    = (zhbr   /hnm(ILON))*ptot(ILON)/PREF

      ! =====================================================================
      ! ====================== CLONO2+HCL -> CL2+HNO3 =======================
      ! ====================== CLONO2+H2O -> HOCL+HNO3 ======================
      ! ========================= ON LIQUID AEROSOL =========================
      ! =====================================================================

      !     d'apres jpl 2000. On ne tient pas compte de la deposition
      !     de hno3 (solutions ternaires) sur les probabilites de reaction.

      t0   = 144.11 + 0.166*wt(ILON) - 0.0150*wt(ILON)**2 + 2.18e-4*wt(ILON)**3
      biga = 169.5  + 5.180*wt(ILON) - 0.0825*wt(ILON)**2 + 3.27e-3*wt(ILON)**3

      !     eta : viscosity of h2so4 solution, cp

      eta = biga*t**(-1.43)*exp(448./(t - t0))

      !     ah : acid activity in molarity

      ah = exp(60.51 - 0.095*wt(ILON) + 0.0077*wt(ILON)**2   &
               -1.61e-5*wt(ILON)**3 - (1.76 +                &
                2.52e-4*wt(ILON)**2)*sqrt(t)                 &
               + (-805.89 + 253.05*wt(ILON)**0.076)/sqrt(t))

      !     dclono2 : clono2 diffusivity, klassen et al., cm2 cm-1

      dclono2 = 5.e-8*t/eta

      !     cbar

      cclono2 = 1474.*sqrt(t)

      !     kh2o : shi et al., s-1

      kh2o = 1.95e10*exp(-2800./t)

      !     kh : shi et al., s-1

      kh = 1.22e12*exp(-6200./t)

      !     khydr : shi et al., s-1

      khydr = kh2o*aw(ILON) + kh*ah*aw(ILON)

      gbh2o = 4*hclono2(ILON)*0.082*t*sqrt(dclono2*khydr) / cclono2

      mhcl = hhcl(ILON)*phcl

      !     khcl : shi et al., s-1

      khcl = 7.9e11*ah*dclono2*mhcl

      !     lclono2 : reacto-diffusive length, cm

      lclono2 = sqrt(dclono2/(khydr + khcl))

      !     fclono2

      fclono2 = 1./tanh(rmean(ILON)/lclono2) - lclono2/rmean(ILON)

      grxn  = fclono2*gbh2o*sqrt(1. + khcl/khydr)
      gbhcl = grxn*khcl/(khcl + khydr)
      gs    = 66.12*hclono2(ILON)*mhcl*exp(-1374./t)

      fhcl  = 1./(1. + 0.612*(gs + gbhcl)*pclono2/phcl)

      gps    = fhcl*gs
      gbhclp = fhcl*gbhcl
      gb     = gbhclp + grxn*khydr/(khcl + khydr)

      gclono2 = 1./(1. + 1./(gps + gb))

      gclono2hcl = gclono2*(gps + gbhclp)/(gps + gb)
      gclono2h2o = gclono2 - gclono2hcl

      rclono2h2o = 0.25*gclono2h2o*cclono2*aliq(ILON)/zh2o
      k1l(ILON) = rclono2h2o

      rclono2hcl = 0.25*gclono2hcl*cclono2*aliq(ILON)/zhcl
      k2l(ILON) = rclono2hcl

      ! =====================================================================
      ! ========================== HOCL + HCL ===============================
      ! ======================= ON LIQUID AEROSOL ===========================
      ! =====================================================================

      !     d'apres jpl 2000

      !     cbar

      chocl = 2009.*sqrt(t)

      !     dhocl : hocl diffusivity, klassen et al., cm2 cm-1

      dhocl = 6.4e-8*t/eta

      !     khoclhcl : shi et al., s-1

      khoclhcl = 1.25e9*ah*dhocl*mhcl

      !     lhocl : reacto-diffusive length, cm

      lhocl = sqrt(dhocl/khoclhcl)

      !     fhocl

      fhocl = 1./tanh(rmean(ILON)/lhocl) - lhocl/rmean(ILON)

      grxn = 4.*hhocl(ILON)*0.082*t*sqrt(dhocl*khoclhcl)/chocl

      ghoclhcl = 1./(1. + 1./(fhocl*grxn*fhcl))

      rhoclhcl = 0.25*ghoclhcl*chocl*aliq(ILON)/zhcl
      k5l(ILON) = rhoclhcl

      ! ====================================================================
      ! =========================== HOBR + HCL =============================
      ! ======================= ON LIQUID AEROSOL ==========================
      ! ====================================================================

      !     d'apres waschewsky and abbatt, j. phys. chem. a,
      !     5312-5320, 1999.

      !     mhobr : concentration de hobr en phase liquide

      mhobr = hhobr(ILON)*phobr

      k2hobrhcl = exp(0.542*wt(ILON) - 6.44e3/t + 10.3)

      ratio = mhobr/mhcl

      if (ratio < 1.) then

         !        hcl est en exces dans l'aerosol.
         !        hobr est le reactif limitant.

         k1hobrhcl = k2hobrhcl*mhcl

         !        dhobr : hobr diffusivity, klassen et al., cm2 cm-1

         dhobr = 6.2e-8*t/eta

         !        lhobr : reacto-diffusive length, cm

         lhobr = sqrt(dhobr/k1hobrhcl)

         fhobr = 1./tanh(rmean(ILON)/lhobr) - lhobr/rmean(ILON)

         chobr = 1477.*sqrt(t)

         ghobrhcl = 4.*hhobr(ILON)*0.082*t*sqrt(dhobr*k1hobrhcl)*fhobr/chobr

         rhobrhcl = 0.25*ghobrhcl*chobr*aliq(ILON)/zhcl
      else

         !        hobr est en exces dans l'aerosol.
         !        hcl est le reactif limitant.

         k1hobrhcl = k2hobrhcl*mhobr

         !        dhcl : hcl diffusivity, klassen et al., cm2 cm-1

         dhcl = 7.8e-8*t/eta

         !        lhcl : reacto-diffusive length, cm

         lhcl = sqrt(dhcl/k1hobrhcl)

         fhcl = 1./tanh(rmean(ILON)/lhcl) - lhcl/rmean(ILON)

         chcl  = 2408.*sqrt(t)

         ghobrhcl = 4.*hhcl(ILON)*0.082*t * sqrt(dhcl*k1hobrhcl)*fhcl/chcl

         rhobrhcl = 0.25*ghobrhcl*chcl*aliq(ILON)/zhobr
      endif

      k7l(ILON) = rhobrhcl

      ! ====================================================================
      ! =========================== HOBR + HBR =============================
      ! ======================= ON LIQUID AEROSOL ==========================
      ! ====================================================================

      k8l(ILON) = 0.

      ! ====================================================================
      ! =========================== HOCL + HBR =============================
      ! ======================= ON LIQUID AEROSOL ==========================
      ! ====================================================================

      !     mhbr  : concentration de hbr  en phase liquide
      !     mhocl : concentration de hocl en phase liquide

      !     mhbr  = hhbr(ILON)*phbr
      !     mhocl = hhocl(ILON)*phocl

      !     k2hoclhbr: abbatt and nowak, j. phys. chem. a, 2131, 1997.
      !     valeur a 228k et 69.3% h2so4

      !     k2hoclhbr = 2.e6

      !     ratio = mhbr/mhocl 

      !     if (ratio .lt. 1.) then

      !        hocl est en exces dans l'aerosol.
      !        hbr est le reactif limitant.

      !        k1hoclhbr = k2hoclhbr*mhocl

      !        dhbr : hbr diffusivity, klassen et al., cm2 cm-1

      !        dhbr = 7.9e-8*t/eta
      !        chbr = 1616.*sqrt(t)

      !        lhbr : reacto-diffusive length, cm

      !        lhbr = sqrt(dhbr/k1hoclhbr)

      !        fhbr = 1./tanh(rmean(ILON)/lhbr)
      !    $         - lhbr/rmean(ILON)

      !        ghoclhbr = 4.*hhbr(ILON)*0.082*t
      !    $              *sqrt(dhbr*k1hoclhbr)*fhbr/chbr

      !     else

      !        hbr est en exces dans l'aerosol.
      !        hocl est le reactif limitant.

      !        k1hoclhbr = k2hoclhbr*mhbr
      !
      !        lhocl : reacto-diffusive length, cm

      !        lhocl = sqrt(dhocl/k1hoclhbr)

      !        fhocl = 1./tanh(rmean(ILON)/lhocl)
      !    $         - lhocl/rmean(ILON)

      !        ghoclhbr = 4.*hhocl(ILON)*0.082*t
      !    $              *sqrt(dhocl*k1hoclhbr)*fhocl/chocl
      !     endif

      k9l(ILON) = 0.

      ! =====================================================================
      ! ======================== N2O5 + H2O =================================
      ! ====================== ON LIQUID AEROSOL ============================
      ! =====================================================================

      GN2O5H2O=0.1
      CBAR=1400.1*SQRT(T)

      RN2O5H2O=0.25*GN2O5H2O*CBAR*ALIQ(ILON)/ZH2O
      K3L(ILON) = RN2O5H2O

      ! =====================================================================
      ! ======================== N2O5 + HCl =================================
      ! ====================== ON LIQUID AEROSOL ============================
      ! =====================================================================

      K4L(ILON) = 0.

      ! =====================================================================
      ! ======================== BrONO2 + H2O ===============================
      ! ====================== ON LIQUID AEROSOL ============================
      ! =====================================================================

      !     parametrisation jpl 2000 (hanson, private comm.)

      gxrn = exp(29.24 - 0.396*100.*ws(ILON)) + 0.114
      gbrno3h2o = 1./(1./0.805 + 1./gxrn)

      CBAR=1221.4*SQRT(T)

      RBRNO3H2O=0.25*GBRNO3H2O*CBAR*ALIQ(ILON)/ZH2O
      K6L(ILON) = RBRNO3H2O

   else
         k1l(ILON) = 0.
         k2l(ILON) = 0.
         k3l(ILON) = 0.
         k4l(ILON) = 0.
         k5l(ILON) = 0.
         k6l(ILON) = 0.
         k7l(ILON) = 0.
         k8l(ILON) = 0.
         k9l(ILON) = 0.
   end if
END DO
!$omp end do

!$omp do
DO ILON = 1,LOLA

   IF (LNAT(ILON) == 1) THEN

      T = TEMP(ILON)

      ! Z VARIABLES ARE SET TO AVOID DIVISION BY ZERO

      ZH2O  = MAX(CK(ILON,ih2o ),1.E-08*HNM(ILON))
      ZHCL  = MAX(CK(ILON,ihcl ),1.E-13*HNM(ILON))
      ZHOCL = MAX(CK(ILON,ihocl),1.E-13*HNM(ILON))
      ZHOBR = MAX(CK(ILON,ihbr ),1.E-13*HNM(ILON))

      ! =====================================================================
      ! ====================== CLNO3 + HCL ==================================
      ! ====================== CLNO3 + H2O ==================================
      ! ====================== N2O5  + H2O ==================================
      ! ====================== N2O5  + HCL ==================================
      ! ====================== HOCL  + HCL ==================================
      ! ====================== BRNO3 + H2O  =================================
      ! ====================== HOBR  + HCL  =================================
      ! ====================== HOBR  + HBR  =================================
      ! ====================== HOCL  + HBR  =================================
      ! ======================= ON NAT ======================================
      ! =====================================================================

      ! GAMMA VALUES

      GCLNO3HCLNAT=0.2
      GCLNO3H2ONAT=0.004
      GN2O5H2ONAT=4.0E-4
      GN2O5HCLNAT=3.0E-3
      GHOCLHCLNAT=0.1
      GBRNO3H2ONAT=0.
      GHOBRHCLNAT=0.
      GHOBRHBRNAT=0.
      GHOCLHBRNAT=0.

      ! BECAUSE NAT PARTICLES CAN BE VERY LARGE, GAS DIFFUSION LIMITATION
      ! IS TAKEN INTO ACCOUNT. THE SECOND ORDER RATE CONSTANTS ARE
      ! CALCULATED FROM THE EQUATION R=G*PI*R**2*CBAR*N/(1+3G.R/(4.L)), WHERE
      ! G=GAMMA, R=PARTICLE RADIUS, CBAR=MEAN MOLECULAR SPEED, L=MEAN FREE
      ! PATH (TURCO ET AL, JGR, 94, 16493, 1989).

      RCLNO3H2ONAT =                                        &
           GCLNO3H2ONAT*4.56E4*SQRT(T/97.45)*RNATS**2       &
           *NNATS(ILON)/                                    &
           (1.0+3.3E4*GCLNO3H2ONAT*RNATS*PTOT(ILON)/100./T) &
           /ZH2O                                            &
         + GCLNO3H2ONAT*4.56E4*SQRT(T/97.45)*RNATL**2       &
           *NNATL(ILON)/                                    &
           (1.0+3.3E4*GCLNO3H2ONAT*RNATL*PTOT(ILON)/100./T) &
           /ZH2O
      K1S(ILON) = RCLNO3H2ONAT

      RCLNO3HCLNAT=                                         &
           GCLNO3HCLNAT*4.56E4*SQRT(T/97.45)*RNATS**2       &
           *NNATS(ILON)/                                    &
           (1.0+3.3E4*GCLNO3HCLNAT*RNATS*PTOT(ILON)/100./T) &
           /ZHCL                                            &
         + GCLNO3HCLNAT*4.56E4*SQRT(T/97.45)*RNATL**2       &
           *NNATL(ILON)/                                    &
           (1.0+3.3E4*GCLNO3HCLNAT*RNATL*PTOT(ILON)/100./T) &
           /ZHCL
      K2S(ILON) = RCLNO3HCLNAT

      RN2O5H2ONAT  =                                        &
           GN2O5H2ONAT*4.56E4*SQRT(T/108.0)*RNATS**2        &
           *NNATS(ILON)/                                    &
           (1.0+3.3E4*GN2O5H2ONAT*RNATS*PTOT(ILON)/100./T)  &
           /ZH2O                                            &
         + GN2O5H2ONAT*4.56E4*SQRT(T/108.0)*RNATL**2        &
           *NNATL(ILON)/                                    &
           (1.0+3.3E4*GN2O5H2ONAT*RNATL*PTOT(ILON)/100./T)  &
           /ZH2O
      K3S(ILON) = RN2O5H2ONAT

      RBRNO3H2ONAT =                                        &
           GBRNO3H2ONAT*4.56E4*SQRT(T/142.0)*RNATS**2       &
           *NNATS(ILON)/                                    &
           (1.0+3.3E4*GBRNO3H2ONAT*RNATS*PTOT(ILON)/100./T) &
           /ZH2O                                            &
         + GBRNO3H2ONAT*4.56E4*SQRT(T/142.0)*RNATL**2       &
           *NNATL(ILON)/                                    &
           (1.0+3.3E4*GBRNO3H2ONAT*RNATL*PTOT(ILON)/100./T) &
           /ZH2O
      K6S(ILON) = RBRNO3H2ONAT

      RN2O5HCLNAT=                                          &
           GN2O5HCLNAT*4.56E4*SQRT(T/108.0)*RNATS**2        &
           *NNATS(ILON)/                                    &
           (1.0+3.3E4*GN2O5HCLNAT*RNATS*PTOT(ILON)/100./T)  &
           /ZHCL                                            &
         + GN2O5HCLNAT*4.56E4*SQRT(T/108.0)*RNATL**2        &
           *NNATL(ILON)/                                    &
           (1.0+3.3E4*GN2O5HCLNAT*RNATL*PTOT(ILON)/100./T ) &
           /ZHCL
      K4S(ILON) = RN2O5HCLNAT

      RHOCLHCLNAT=                                          &
           GHOCLHCLNAT*4.56E4*SQRT(T/52.5)*RNATS**2         &
           *NNATS(ILON)/                                    &
           (1.0+3.3E4*GHOCLHCLNAT*RNATS*PTOT(ILON)/100./T)  &
           /ZHCL                                            &
         + GHOCLHCLNAT*4.56E4*SQRT(T/52.5)*RNATL**2         &
           *NNATL(ILON)/                                    &
           (1.0+3.3E4*GHOCLHCLNAT*RNATL*PTOT(ILON)/100./T)  &
           /ZHCL
      K5S(ILON) = RHOCLHCLNAT

      RHOBRHCLNAT=                                          &
           GHOBRHCLNAT*4.56E4*SQRT(T/96.9)*RNATS**2         &
           *NNATS(ILON)/                                    &
           (1.0+3.3E4*GHOBRHCLNAT*RNATS*PTOT(ILON)/100./T)  &
           /ZHCL                                            &
         + GHOBRHCLNAT*4.56E4*SQRT(T/96.9)*RNATL**2         &
           *NNATL(ILON)/                                    &
           (1.0+3.3E4*GHOBRHCLNAT*RNATL*PTOT(ILON)/100./T)  &
           /ZHCL
      K7S(ILON) = RHOBRHCLNAT

      K8S(ILON) = 0.
      K9S(ILON) = 0.

   ENDIF

END DO
!$omp end do

!$omp do
DO ILON = 1,LOLA

   IF (LICE(ILON) == 1) THEN

      T = TEMP(ILON)

      ! Z VARIABLES ARE SET TO AVOID DIVISION BY ZERO

      ZH2O  = MAX(CK(ILON,ih2o ),1.E-08*HNM(ILON))
      ZHCL  = MAX(CK(ILON,ihcl ),1.E-13*HNM(ILON))
      ZHOCL = MAX(CK(ILON,ihocl),1.E-13*HNM(ILON))
      ZHOBR = MAX(CK(ILON,ihobr),1.E-13*HNM(ILON))

      ! =====================================================================
      ! ====================== CLNO3 + HCL ==================================
      ! ====================== CLNO3 + H2O ==================================
      ! ====================== N2O5  + H2O ==================================
      ! ====================== N2O5  + HCL ==================================
      ! ====================== HOCL  + HCL ==================================
      ! ====================== BRNO3 + H2O  =================================
      ! ====================== HOBR  + HCL  =================================
      ! ====================== HOBR  + HBR  =================================
      ! ====================== HOCL  + HBR  =================================
      ! ======================= ON ICE ======================================
      ! =====================================================================
      ! GAMMA VALUES

      GCLNO3HCLICE=0.3
      GCLNO3H2OICE=0.3
      GN2O5H2OICE=0.02
      GN2O5HCLICE=0.03
      GHOCLHCLICE=0.2
      GBRNO3H2OICE=0.3
      GHOBRHCLICE=0.3
      GHOBRHBRICE=0.1
      GHOCLHBRICE=0.

      ! BECAUSE ICE PARTICLES CAN BE VERY LARGE, GAS DIFFUSION LIMITATION
      ! IS TAKEN INTO ACCOUNT. THE SECOND ORDER RATE CONSTANTS ARE
      ! CALCULATED FROM THE EQUATION R=G*PI*R**2*CBAR*N/(1+3G.R/(4.L)), WHERE
      ! G=GAMMA, R=PARTICLE RADIUS, CBAR=MEAN MOLECULAR SPEED, L=MEAN FREE
      ! PATH (TURCO ET AL, JGR, 94, 16493, 1989).

      RCLNO3HCLICE=                                         &
           GCLNO3HCLICE*4.56E4*SQRT(T/97.45)*RICE**2        &
           *NICE(ILON)/                                     &
           (1.0+3.3E4*GCLNO3HCLICE*RICE*PTOT(ILON)/100./T)  &
           /ZHCL 
      K2S(ILON) = RCLNO3HCLICE

      RCLNO3H2OICE=                                         &
           GCLNO3H2OICE*4.56E4*SQRT(T/97.45)*RICE**2        &
           *NICE(ILON)/                                     &
           (1.0+3.3E4*GCLNO3H2OICE*RICE*PTOT(ILON)/100./T)  &
           /ZH2O
      K1S(ILON) = RCLNO3H2OICE

      RN2O5H2OICE=                                          &
           GN2O5H2OICE*4.56E4*SQRT(T/108.0)*RICE**2         &
           *NICE(ILON)/                                     &
           (1.0+3.3E4*GN2O5H2OICE*RICE*PTOT(ILON)/100./T)   &
           /ZH2O
      K3S(ILON) = RN2O5H2OICE

      RN2O5HCLICE=                                          &
           GN2O5HCLICE*4.56E4*SQRT(T/108.0)*RICE**2         &
           *NICE(ILON)/                                     &
           (1.0+3.3E4*GN2O5HCLICE*RICE*PTOT(ILON)/100./T)   &
           /ZHCL
      K4S(ILON) = RN2O5HCLICE

      RHOCLHCLICE=                                          &
           GHOCLHCLICE*4.56E4*SQRT(T/52.5)*RICE**2          &
           *NICE(ILON)/                                     &
           (1.0+3.3E4*GHOCLHCLICE*RICE*PTOT(ILON)/100./T)   &
           /ZHCL
      K5S(ILON) = RHOCLHCLICE
 
      RBRNO3H2OICE=                                         &
           GBRNO3H2OICE*4.56E4*SQRT(T/142.0)*RICE**2        &
           *NICE(ILON)/                                     &
           (1.0+3.3E4*GBRNO3H2OICE*RICE*PTOT(ILON)/100./T)  &
           /ZH2O
      K6S(ILON) = RBRNO3H2OICE

      RHOBRHCLICE=                                          &
           GHOBRHCLICE*4.56E4*SQRT(T/96.9)*RICE**2          &
           *NICE(ILON)/                                     &
           (1.0+3.3E4*GHOBRHCLICE*RICE*PTOT(ILON)/100./T)   &
           /ZHCL
      K7S(ILON) = RHOBRHCLICE

      RHOBRHBRICE=                                          &
           GHOBRHBRICE*4.56E4*SQRT(T/96.9)*RICE**2          &
           *NICE(ILON)/                                     &
           (1.0+3.3E4*GHOBRHBRICE*RICE*PTOT(ILON)/100./T)   &
           /ZHOBR
      K8S(ILON) = RHOBRHBRICE

   END IF

END DO
!$omp end do

!$omp do
DO ILON = 1,LOLA
   K1(ILON) = K1L(ILON) + (LNAT(ILON) + LICE(ILON))*K1S(ILON)
   K2(ILON) = K2L(ILON) + (LNAT(ILON) + LICE(ILON))*K2S(ILON)
   K3(ILON) = K3L(ILON) + (LNAT(ILON) + LICE(ILON))*K3S(ILON)
   K4(ILON) = K4L(ILON) + (LNAT(ILON) + LICE(ILON))*K4S(ILON)
   K5(ILON) = K5L(ILON) + (LNAT(ILON) + LICE(ILON))*K5S(ILON)
   K6(ILON) = K6L(ILON) + (LNAT(ILON) + LICE(ILON))*K6S(ILON)
   K7(ILON) = K7L(ILON) + (LNAT(ILON) + LICE(ILON))*K7S(ILON)
   K8(ILON) = K8L(ILON) + (LNAT(ILON) + LICE(ILON))*K8S(ILON)
   K9(ILON) = 0.
END DO
!$omp end do

!$omp end parallel

  DEALLOCATE (K1L)
  DEALLOCATE (K2L)
  DEALLOCATE (K3L)
  DEALLOCATE (K4L)
  DEALLOCATE (K5L)
  DEALLOCATE (K6L)
  DEALLOCATE (K7L)
  DEALLOCATE (K8L)
  DEALLOCATE (K9L)
  DEALLOCATE (K1S)
  DEALLOCATE (K2S)
  DEALLOCATE (K3S)
  DEALLOCATE (K4S)
  DEALLOCATE (K5S)
  DEALLOCATE (K6S)
  DEALLOCATE (K7S)
  DEALLOCATE (K8S)
  DEALLOCATE (K9S)

RETURN
END subroutine hetero
