SUBROUTINE HITLORES(LONML,LATML,LONMH,LATMH          &
                   ,NIVM,NTRA,DT                     &
                   ,TRELAX,IX,IY,IGRID               &
                   ,DRLONML,DRLATML,DRLONMH,DRLATMH  &
                   ,SURFM,QQQML,QQQMH)

!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!CC DATE : 15/05/2005
!
!   Type      : Parameterizations and processes 
!   Purpose   : Two-ways nesting
!   Called by : 
!
!CC
!
!  Original : M. Amodei & V.-H. Peuch, 04/2001
!
!  Modified :

	IMPLICIT NONE

!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~

	INTEGER :: LONML,LATML,LONMH,LATMH,NIVM,NTRA
	REAL (KIND=8) :: DT,TRELAX,DRLONML,DRLATML,DRLONMH,DRLATMH
	INTEGER , DIMENSION (LONML) :: IX
	INTEGER , DIMENSION (LATML) :: IY
	INTEGER , DIMENSION (LONML,LATML) :: IGRID
	REAL (KIND=8), DIMENSION (LONMH,LATMH) :: SURFM
        REAL (KIND=8) , DIMENSION (LONML,LATML,NIVM,NTRA) :: QQQML
        REAL (KIND=8) , DIMENSION (LONMH,LATMH,NIVM,NTRA) :: QQQMH

!     Variables locales.
!     ~~~~~~~~~~~~~~~~~

	INTEGER :: NPX,NPY,NBP,ITRAC,INIV,ILAT,ILON,I,J,IDEB,JDEB
        INTEGER :: NPXS2,NPYS2,NPXS,NPYS
        REAL (KIND=8) :: SURFBP
	REAL (KIND=8) :: SOMQQQMH
        REAL (KIND=8) :: CRELAX

!AP
        INTEGER, DIMENSION(:,:,:,:), ALLOCATABLE :: INDXMIN,INDXMAX,INDYMIN,INDYMAX
        INTEGER :: IXPI, IYPJ, IYPJ0, IXPI0, INDXMIN_, INDXMAX_, INDYMIN_, INDYMAX_
!APEND

! Nombre de petites mailles dans une grande

NPX=NINT(DRLONML/DRLONMH)
NPY=NINT(DRLATML/DRLATMH)

NPXS2=NPX/2
NPYS2=NPY/2

! NPXS est introduit pour les cas ou NPX est impair
! Dans ce cas, on rduira le nombre de mailles
! prises en compte 

NPXS=2*NPXS2
NPYS=2*NPYS2
      
IF (NPX == NPXS) THEN
   IDEB=1
ELSE
   IDEB=0
ENDIF
IF (NPY == NPYS) THEN
   JDEB=1
ELSE
   JDEB=0
ENDIF

! Constante de relaxation

CRELAX=EXP(-DT/TRELAX)

ALLOCATE(INDXMIN(IDEB:NPXS,JDEB:NPYS,LONML,LATML))
ALLOCATE(INDYMIN(IDEB:NPXS,JDEB:NPYS,LONML,LATML))
ALLOCATE(INDXMAX(IDEB:NPXS,JDEB:NPYS,LONML,LATML))
ALLOCATE(INDYMAX(IDEB:NPXS,JDEB:NPYS,LONML,LATML))

!$omp parallel default(none) &
!$omp private(IXPI,IYPJ,SOMQQQMH,SURFBP,i,ilat,ilon,iniv,itrac,j) &
!$omp shared(CRELAX,IDEB,IGRID,INDXMAX,INDXMIN,INDYMAX,INDYMIN,IX,IXPI0,IY,IYPJ0,JDEB) &
!$omp shared(LATML,LONML,NIVM,NPXS,NPXS2,NPYS,NPYS2,NTRA,QQQMH,QQQML,SURFM)

!$omp do schedule(dynamic)
DO ilat=1,LATML
   IYPJ0  = IY(ilat)-NPYS2
   DO ilon=1,LONML

      IF(IGRID(ilon,ilat) == 1)THEN
         IXPI0  = IX(ilon)-NPXS2

         SURFBP=0.

         DO j=JDEB,NPYS
         DO i=IDEB,NPXS

            IXPI   = IXPI0+i
            IYPJ   = IYPJ0+j
            SURFBP = SURFBP + SURFM(IXPI,IYPJ)
            !AP
            INDXMIN(i,j,ilon,ilat)=IXPI
            INDXMAX(i,j,ilon,ilat)=IXPI
            INDYMIN(i,j,ilon,ilat)=IYPJ
            INDYMAX(i,j,ilon,ilat)=IYPJ
            !APEND
         ENDDO
         ENDDO

         DO itrac=1,NTRA
         DO iniv=1,NIVM

            SOMQQQMH=0.

            DO j=JDEB,NPYS
            DO i=IDEB,NPXS
           
               IXPI   = IXPI0+i
               IYPJ   = IYPJ0+j
               SOMQQQMH = SOMQQQMH + QQQMH(IXPI,IYPJ,iniv,itrac) &
                                   * SURFM(IXPI,IYPJ)
            
            ENDDO
            ENDDO

            QQQML(ilon,ilat,iniv,itrac) = SOMQQQMH/SURFBP +( &
            QQQML(ilon,ilat,iniv,itrac) - SOMQQQMH/SURFBP ) * CRELAX

         ENDDO
         ENDDO

      ENDIF

   ENDDO
ENDDO
!$omp end do

!$omp end parallel

!AP

INDXMIN_ = 200 ; INDXMAX_ = 0
INDYMIN_ = 200 ; INDYMAX_ = 0
DO ilat=1,LATML
DO ilon=1,LONML
   IF(IGRID(ilon,ilat) == 1)THEN
      DO j=JDEB,NPYS
      DO i=IDEB,NPXS
         INDXMIN_ = MIN ( INDXMIN_, INDXMIN(i,j,ilon,ilat) )
         INDYMIN_ = MIN ( INDYMIN_, INDYMIN(i,j,ilon,ilat) )
         INDXMAX_ = MAX ( INDXMAX_, INDXMAX(i,j,ilon,ilat) )
         INDYMAX_ = MAX ( INDYMAX_, INDYMAX(i,j,ilon,ilat) )
      ENDDO
      ENDDO
  ENDIF
ENDDO
ENDDO

IF (INDXMIN_ < 1     .OR.  &
    INDYMIN_ < 1     .OR.  &
    INDXMAX_ > LONMH .OR.  &
    INDYMAX_ > LATMH) THEN
          
    PRINT*,'INDXMIN=',INDXMIN_
    PRINT*,'INDXMAX=',INDXMAX_
    PRINT*,'INDYMIN=',INDYMIN_
    PRINT*,'INDYMAX=',INDYMAX_
    CALL MOCAGE_STOP
ENDIF
!APEND

DEALLOCATE(INDXMIN,INDYMIN,INDXMAX,INDYMAX)

RETURN
END SUBROUTINE HITLORES
