      SUBROUTINE HOBS3D(XCH,LON,LAT,NIV,IGRCH,
     +                  X0,Y0,P0,NDIM,
     +                  NCONST,XINTER)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Observations
C   Purpose    : 3D observation operator (tri-cubic)
C   Called by  : MADONA
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 04/2000
C
C  Modified : A. Dufour, F. Couvreux & V.-H. Peuch, 07/2001
C
CCC
C	
	USE PARADI_MOD
	USE GRIDS_MOD
	USE FORMT_MOD
	USE FORS_MOD
	USE CSTES_MOD
      USE GAUSS_MOD
C  
	IMPLICIT NONE
C	
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C      
	INTEGER :: LON,LAT,NIV,NDIM,NCONST,IGRCH
	REAL (KIND=8) :: X0,Y0
	REAL (KIND=8) , DIMENSION (LON,LAT,NIV,nbconstmax) :: XCH
	REAL (KIND=8) , DIMENSION (nbnivpresmax) :: P0
	REAL (KIND=8) , DIMENSION(nbnivpresmax,nbconstmax) :: XINTER  
C        XCH(LON,LAT,NIV,nbconstmax) : 3D MOCAGE FIELD
C        IGRCH                       : MOCAGE DOMAIN
C        X0,Y0,P0(nbnivpresmax)      : LOCALIZATION OF OBSERVATION
C        XINTER(nbnivpresmax,nbconstmax)     : MODEL EQUIVALENT TO OBSERVATION
C        (IF OBS. IS NOT WITHIN DOMAIN, XINTER=-777.)
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: I,ICST,ILON,M,N
	INTEGER :: DIM_TEMP
C HLB DIM_TEMP pour dimensionner des tableaux de dimension 1 non correctement
	REAL (KIND=8) :: ZALTREL,ZTOT,XP1,ZLAYER,A1
	LOGICAL :: LOGICH
	INTEGER , ALLOCATABLE :: INDX(:),INDY(:),INDZ(:)
	INTEGER , ALLOCATABLE :: INDXbis(:),INDYbis(:)
	INTEGER , ALLOCATABLE :: IGRI(:)
	REAL (KIND=8) , ALLOCATABLE :: XX0(:),YY0(:),PP0(:),XPSOL(:)
	REAL (KIND=8) , ALLOCATABLE :: XPSOL1(:),ZMOD(:),XTM(:),XP2(:)
	REAL (KIND=8) , ALLOCATABLE :: ALFX1(:),ALFX2(:)
	REAL (KIND=8) , ALLOCATABLE :: ALFX3(:),ALFX4(:)
	REAL (KIND=8) , ALLOCATABLE :: ALFY1(:),ALFY2(:)
	REAL (KIND=8) , ALLOCATABLE :: ALFY3(:),ALFY4(:)
	REAL (KIND=8) , ALLOCATABLE :: ALFZ1(:),ALFZ2(:)
	REAL (KIND=8) , ALLOCATABLE :: ALFZ3(:),ALFZ4(:)
	REAL (KIND=8) , ALLOCATABLE :: ALFX1bis(:),ALFX2bis(:)
	REAL (KIND=8) , ALLOCATABLE :: ALFX3bis(:),ALFX4bis(:)
	REAL (KIND=8) , ALLOCATABLE :: ALFY1bis(:),ALFY2bis(:)
	REAL (KIND=8) , ALLOCATABLE :: ALFY3bis(:),ALFY4bis(:)
	REAL (KIND=8) , ALLOCATABLE :: XCHRED(:,:,:)
	REAL (KIND=8) , ALLOCATABLE :: XCHRED_TEMP(:,:)
	REAL (KIND=8) , ALLOCATABLE :: XINTERRED(:)
	REAL (KIND=8) , ALLOCATABLE :: TGRIM12D(:,:),XPHFM12D(:,:)
	REAL (KIND=8) , ALLOCATABLE :: TGRIM22D(:,:),XPHFM22D(:,:)
	REAL (KIND=8) , ALLOCATABLE :: TGRIM32D(:,:),XPHFM32D(:,:)
	REAL (KIND=8) , ALLOCATABLE :: TGRIM42D(:,:),XPHFM42D(:,:)
C
C 0. INITIALIZATIONS
C      
	DIM_TEMP=1
        ALLOCATE (XX0(DIM_TEMP))	  ; XX0(:) = 0.0
        ALLOCATE (YY0(DIM_TEMP))	  ; YY0(:) = 0.0
        ALLOCATE (PP0(NDIM))	  ; PP0(:) = 0.0
        ALLOCATE (XPSOL(NDIM))	  ; XPSOL(:) = 0.0
        ALLOCATE (XPSOL1(DIM_TEMP))	  ; XPSOL1(:) = 0.0
        ALLOCATE (ZMOD(NDIM))	  ; ZMOD(:) = 0.0
        ALLOCATE (XTM(NDIM))	  ; XTM(:) = 0.0
        ALLOCATE(XP2(NDIM))	  ; XP2(:) = 0.0
        ALLOCATE (INDX(DIM_TEMP))       	  ; INDX(:) = 0       
        ALLOCATE (INDY(DIM_TEMP))	  ; INDY(:) = 0
        ALLOCATE (INDZ(NDIM))	  ; INDZ(:) = 0
        ALLOCATE (INDXbis(NDIM))	  ; INDXbis(:) = 0
        ALLOCATE (INDYbis(NDIM))	  ; INDYbis(:) = 0
        ALLOCATE (IGRI(DIM_TEMP))	  ; IGRI(:) = 0
        ALLOCATE (ALFX1(DIM_TEMP))	  ; ALFX1(:) = 0.0
        ALLOCATE (ALFX2(DIM_TEMP))	  ; ALFX2(:) = 0.0
        ALLOCATE (ALFX3(DIM_TEMP))	  ; ALFX3(:) = 0.0
        ALLOCATE (ALFX4(DIM_TEMP))	  ; ALFX4(:) = 0.0
        ALLOCATE (ALFY1(DIM_TEMP))	  ; ALFY1(:) = 0.0
        ALLOCATE (ALFY2(DIM_TEMP))	  ; ALFY2(:) = 0.0
        ALLOCATE (ALFY3(DIM_TEMP))	  ; ALFY3(:) = 0.0
        ALLOCATE (ALFY4(DIM_TEMP))	  ; ALFY4(:) = 0.0
        ALLOCATE (ALFZ1(NDIM))	  ; ALFZ1(:) = 0.0
        ALLOCATE (ALFZ2(NDIM))	  ; ALFZ2(:) = 0.0
        ALLOCATE (ALFZ3(NDIM))	  ; ALFZ3(:) = 0.0
        ALLOCATE (ALFZ4(NDIM))	  ; ALFZ4(:) = 0.0
        ALLOCATE (ALFX1bis(NDIM))	  ; ALFX1bis(:) = 0.0
        ALLOCATE (ALFX2bis(NDIM))	  ; ALFX2bis(:) = 0.0
        ALLOCATE (ALFX3bis(NDIM))	  ; ALFX3bis(:) = 0.0
        ALLOCATE (ALFX4bis(NDIM))	  ; ALFX4bis(:) = 0.0
        ALLOCATE (ALFY1bis(NDIM))	  ; ALFY1bis(:) = 0.0
        ALLOCATE (ALFY2bis(NDIM))	  ; ALFY2bis(:) = 0.0
        ALLOCATE (ALFY3bis(NDIM))	  ; ALFY3bis(:) = 0.0
        ALLOCATE (ALFY4bis(NDIM))	  ; ALFY4bis(:) = 0.0
        ALLOCATE (XCHRED(LON,LAT,NIV))	  ; XCHRED(:,:,:) = 0.0
        ALLOCATE (XCHRED_TEMP(LON,LAT))	  ; XCHRED_TEMP(:,:) = 0.0
        ALLOCATE (XINTERRED(NDIM))	  ; XINTERRED(:) = 0.0
        ALLOCATE (TGRIM12D(LONM1,LATM1))	  ; TGRIM12D(:,:) = 0.0
        ALLOCATE (XPHFM12D(LONM1,LATM1))	  ; XPHFM12D(:,:) = 0.0
        ALLOCATE (TGRIM22D(LONM2,LATM2))	  ; TGRIM22D(:,:) = 0.0
        ALLOCATE (XPHFM22D(LONM2,LATM2))	  ; XPHFM22D(:,:) = 0.0
        ALLOCATE (TGRIM32D(LONM3,LATM3))	  ; TGRIM32D(:,:) = 0.0
        ALLOCATE (XPHFM32D(LONM3,LATM3))	  ; XPHFM32D(:,:) = 0.0
        ALLOCATE (TGRIM42D(LONM4,LATM4))	  ; TGRIM42D(:,:) = 0.0
        ALLOCATE (XPHFM42D(LONM4,LATM4))	  ; XPHFM42D(:,:) = 0.0
C
      XX0(:)=X0/CONV
      YY0(:)=Y0/CONV
C
      DO N=1,NDIM
        PP0(N)=P0(N)
        IF (PP0(N).EQ.0.) PP0(N)=110000.
      ENDDO
C
C 1. GLOBAL DOMAIN
C 
      IF (IGRCH.EQ.1) THEN
C
C 1.1 WEIGHTS FOR HORIZONTAL INTERPOLATION
C      
        CALL POIDREG(XX0,RLONM1(1)-DRLONM1,DRLONM1
     +              ,INDX,ALFX1,ALFX2,ALFX3,ALFX4,1)
#ifndef GAUSS
        CALL POIDREG(YY0,RLATM1(1)-DRLATM1,DRLATM1
     +              ,INDY,ALFY1,ALFY2,ALFY3,ALFY4,1)
#else
        CALL POIDGAU(YY0,XLAT,YCOEF,LATM1
     +              ,INDY,ALFY1,ALFY2,ALFY3,ALFY4,1)
#endif
C
C 1.2 COMPUTE SURFACE PRESSION FOR (X0,Y0)
C     
        CALL INTER2D(PSOLM1,LONM1,LATM1,
     +               INDX,INDY,
     +               ALFX1,ALFX2,ALFX3,ALFX4,
     +               ALFY1,ALFY2,ALFY3,ALFY4,
     +               XPSOL1,1,+1.,.FALSE.,1)     
C
        DO N=1,NDIM
          XPSOL(N)=XPSOL1(1)
        ENDDO
C
C CHECK IF VERTICAL POSITION IS GIVEN IN METERS (PP0<0)
C
        DO N=1,NDIM
          IF (PP0(N).LT.0) THEN
C
C COMPUTE MODEL OROGRAPHY (m)
C 
            CALL INTER2D(GZ0M1,LONM1,LATM1,
     +                   INDX,INDY,
     +                   ALFX1,ALFX2,ALFX3,ALFX4,
     +                   ALFY1,ALFY2,ALFY3,ALFY4,
     +                   ZMOD,1,+1.,.FALSE.,1)
            ZMOD=ZMOD/GG
C
C COMPUTE RELATIVE ALTITUDE RELATIVE TO MODEL SURFACE (m)
C
            ZALTREL=MAX(0.,-PP0(N)-ZMOD(1))
C
C COMPUTE EQUIVALENT PRESSURE
C
            LOGICH=.TRUE.
            ZTOT = 0.
            XP1=XPSOL1(1)
C
            DO M=NIVM1,2,-1
              IF (LOGICH) THEN
                DO I=1,LONM1*LATM1
                  TGRIM12D(I,1)=TGRIM1(I,1,M)
                  XPHFM12D(I,1)=XPHFM1(I,1,M-1)
                ENDDO
                CALL INTER2D(TGRIM12D,LONM1,LATM1,
     +                       INDX,INDY,
     +                       ALFX1,ALFX2,ALFX3,ALFX4,
     +                       ALFY1,ALFY2,ALFY3,ALFY4,
     +                       XTM,1,+1.,.FALSE.,1)
                CALL INTER2D(XPHFM12D,LONM1,LATM1,
     +                       INDX,INDY,
     +                       ALFX1,ALFX2,ALFX3,ALFX4,
     +                       ALFY1,ALFY2,ALFY3,ALFY4,
     +                       XP2,1,+1.,.FALSE.,1)
                ZLAYER=RA*XTM(1)/GG*
     +                 MAX(0.,ALOG(XP1/XP2(1)))
                ZTOT=ZTOT+ZLAYER
                IF (ZALTREL.LT.ZTOT) THEN
                  a1=(ZTOT-ZALTREL)/ZLAYER
                  PP0(N)=a1*XP1+(1.-a1)*XP2(1)
                  LOGICH=.FALSE.
                ELSE
                  XP1=XP2(1)
                ENDIF
              ENDIF
            ENDDO
          ENDIF
        ENDDO
C
C 1.3 WEIGHTS FOR VERTICAL INTERPOLATION
C          
        CALL POIDZ  (PP0,XPSOL,ALM1,BLM1,NIVM1
     +              ,INDZ,ALFZ1,ALFZ2,ALFZ3,ALFZ4,NDIM)
C
C 1.4 3D-INTERPOLATION
C     
        DO N=1,NDIM
          ALFX1bis(N)=ALFX1(1)
          ALFX2bis(N)=ALFX2(1)
          ALFX3bis(N)=ALFX3(1)
          ALFX4bis(N)=ALFX4(1)
          ALFY1bis(N)=ALFY1(1)
          ALFY2bis(N)=ALFY2(1)
          ALFY3bis(N)=ALFY3(1)
          ALFY4bis(N)=ALFY4(1)
          INDXbis(N)=INDX(1)
          INDYbis(N)=INDY(1)
        ENDDO
C
        DO icst=1,NCONST
          DO ilon=1,LON*LAT*NIV
            XCHRED(ilon,1,1)=XCH(ilon,1,1,icst)
          ENDDO
          DO ILON=1,LON*LAT
	      XCHRED_TEMP(ILON,1)=XCHRED(ILON,1,1)
	    ENDDO
	    CALL INTER3D(XCHRED,XCHRED_TEMP,LONM1,LATM1,NIVM1
     +                ,INDXbis,INDYbis,INDZ
     +                ,ALFX1bis,ALFX2bis,ALFX3bis,ALFX4bis
     +                ,ALFY1bis,ALFY2bis,ALFY3bis,ALFY4bis
     +                ,ALFZ1,ALFZ2,ALFZ3,ALFZ4
     +                ,XINTERRED,NDIM,+1.,.FALSE.,1,.TRUE.)
          DO N=1,NDIM
            XINTER(N,icst)=XINTERRED(N)
          ENDDO
        ENDDO
C
C 2. CONTINENTAL DOMAIN
C     
      ELSEIF(IGRCH.EQ.2) THEN    
C
C 2.1 OBSERVATIONS WITHIN CONTINENTAL DOMAIN
C     
        CALL POSIREG(XX0,YY0,2
     +              ,RLONM2(1),RLONM2(LONM2),RLATM2(1),RLATM2(LATM2)
     +              ,RLONM3(1),RLONM3(LONM3),RLATM3(1),RLATM3(LATM3)
     +              ,RLONM4(1),RLONM4(LONM4),RLATM4(1),RLATM4(LATM4)
     +              ,IGRI,1)
C
C 2.2 OBS. OUTSIDE CONTINENTAL DOMAIN
C      
        IF(IGRI(1).NE.2) THEN
          DO N=1,nbnivpresmax*nbconstmax
            XINTER(N,1)=-777.
          ENDDO 
        ELSE
C
C 2.3 OBS. WITHIN CONTINENTAL DOMAIN
C
C 2.3.1 WEIGHTS FOR HORIZONTAL INTERPOLATION
C            
          CALL POIDREG(XX0,RLONM2(1)-DRLONM2,DRLONM2
     +                ,INDX,ALFX1,ALFX2,ALFX3,ALFX4,1)
          CALL POIDREG(YY0,RLATM2(1)-DRLATM2,DRLATM2
     +                ,INDY,ALFY1,ALFY2,ALFY3,ALFY4,1)
C
C 2.3.2 COMPUTE SURFACE PRESSION FOR (X0,Y0)
C         
          CALL INTER2D(PSOLM2,LONM2,LATM2,
     +                 INDX,INDY,
     +                 ALFX1,ALFX2,ALFX3,ALFX4,
     +                 ALFY1,ALFY2,ALFY3,ALFY4,
     +                 XPSOL1,1,+1.,.FALSE.,2) 
          DO N=1,NDIM
            XPSOL(N)=XPSOL1(1)
          ENDDO
C
C CHECK IF VERTICAL POSITION IS GIVEN IN METERS (PP0<0)
C
          DO N=1,NDIM
            IF (PP0(N).LT.0) THEN
C
C COMPUTE MODEL OROGRAPHY (m)
C
              CALL INTER2D(GZ0M2,LONM2,LATM2,
     +                     INDX,INDY,
     +                     ALFX1,ALFX2,ALFX3,ALFX4,
     +                     ALFY1,ALFY2,ALFY3,ALFY4,
     +                     ZMOD,1,+1.,.FALSE.,2)
              ZMOD=ZMOD/GG
C
C COMPUTE RELATIVE ALTITUDE RELATIVE TO MODEL SURFACE (m)
C
              ZALTREL=MAX(0.,-PP0(N)-ZMOD(1))
C
C COMPUTE EQUIVALENT PRESSURE
C
              LOGICH=.TRUE.
              ZTOT = 0.
              XP1=XPSOL1(1)
C
              DO M=NIVM2,2,-1
                IF (LOGICH) THEN
                  DO I=1,LONM2*LATM2
                    TGRIM22D(I,1)=TGRIM2(I,1,M)
                    XPHFM22D(I,1)=XPHFM2(I,1,M-1)
                  ENDDO
                  CALL INTER2D(TGRIM22D,LONM2,LATM2,
     +                         INDX,INDY,
     +                         ALFX1,ALFX2,ALFX3,ALFX4,
     +                         ALFY1,ALFY2,ALFY3,ALFY4,
     +                         XTM,1,+1.,.FALSE.,2)
                  CALL INTER2D(XPHFM22D,LONM2,LATM2,
     +                         INDX,INDY,
     +                         ALFX1,ALFX2,ALFX3,ALFX4,
     +                         ALFY1,ALFY2,ALFY3,ALFY4,
     +                         XP2,1,+1.,.FALSE.,2)
                  ZLAYER=RA*XTM(1)/GG*
     +                   MAX(0.,ALOG(XP1/XP2(1)))
                  ZTOT=ZTOT+ZLAYER
                  IF (ZALTREL.LT.ZTOT) THEN
                    a1=(ZTOT-ZALTREL)/ZLAYER
                    PP0(N)=a1*XP1+(1.-a1)*XP2(1)
                    LOGICH=.FALSE.
                  ELSE
                    XP1=XP2(1)
                  ENDIF
                ENDIF
              ENDDO
            ENDIF
          ENDDO
C
C 2.3.3 WEIGHTS FOR VERTICAL INTERPOLATION
C               
          CALL POIDZ  (PP0,XPSOL,ALM2,BLM2,NIVM2
     +                ,INDZ,ALFZ1,ALFZ2,ALFZ3,ALFZ4,NDIM)
C
C 2.3.4 3D-INTERPOLATION
C          
          DO N=1,NDIM
            ALFX1bis(N)=ALFX1(1)
            ALFX2bis(N)=ALFX2(1)
            ALFX3bis(N)=ALFX3(1)
            ALFX4bis(N)=ALFX4(1)
            ALFY1bis(N)=ALFY1(1)
            ALFY2bis(N)=ALFY2(1)
            ALFY3bis(N)=ALFY3(1)
            ALFY4bis(N)=ALFY4(1)
            INDXbis(N)=INDX(1)
            INDYbis(N)=INDY(1)
          ENDDO
C
          DO icst=1,NCONST
            DO ilon=1,LON*LAT*NIV
              XCHRED(ilon,1,1)=XCH(ilon,1,1,icst)
            ENDDO 
            DO ILON=1,LON*LAT
	      XCHRED_TEMP(ILON,1)=XCHRED(ILON,1,1)
	    ENDDO
           CALL INTER3D(XCHRED,XCHRED_TEMP,LONM2,LATM2,NIVM2
     +                  ,INDXbis,INDYbis,INDZ
     +                  ,ALFX1bis,ALFX2bis,ALFX3bis,ALFX4bis
     +                  ,ALFY1bis,ALFY2bis,ALFY3bis,ALFY4bis
     +                  ,ALFZ1,ALFZ2,ALFZ3,ALFZ4
     +                  ,XINTERRED,NDIM,+1.,.FALSE.,2,.TRUE.)
            DO N=1,NDIM
              XINTER(N,icst)=XINTERRED(N)
            END DO
C
          ENDDO
        ENDIF     
C
C 3. REGIONAL DOMAIN
C    
      ELSEIF(IGRCH.EQ.3) THEN
C
C 3.1 OBSERVATIONS WITHIN REGIONAL DOMAIN
C           
        CALL POSIREG(XX0,YY0,3
     +              ,RLONM2(1),RLONM2(LONM2),RLATM2(1),RLATM2(LATM2)
     +              ,RLONM3(1),RLONM3(LONM3),RLATM3(1),RLATM3(LATM3)
     +              ,RLONM4(1),RLONM4(LONM4),RLATM4(1),RLATM4(LATM4)
     +              ,IGRI,1)
C
C 3.2 OBS. OUTSIDE REGIONAL DOMAIN
C          
        IF(IGRI(1).NE.3) THEN
          DO N=1,nbnivpresmax*nbconstmax
            XINTER(N,1)=-777.
          ENDDO 
        ELSE
C      
C 3.3 OBS. WITHIN REGIONAL DOMAIN
C      
C 3.3.1 WEIGHTS FOR HORIZONTAL INTERPOLATION
C              
          CALL POIDREG(XX0,RLONM3(1)-DRLONM3,DRLONM3
     +                ,INDX,ALFX1,ALFX2,ALFX3,ALFX4,1)
          CALL POIDREG(YY0,RLATM3(1)-DRLATM3,DRLATM3
     +                ,INDY,ALFY1,ALFY2,ALFY3,ALFY4,1)
C
C 3.3.2 COMPUTE SURFACE PRESSION FOR (X0,Y0)
C         
          CALL INTER2D(PSOLM3,LONM3,LATM3,
     +                 INDX,INDY,
     +                 ALFX1,ALFX2,ALFX3,ALFX4,
     +                 ALFY1,ALFY2,ALFY3,ALFY4,
     +                 XPSOL1,1,+1.,.FALSE.,3) 
C
          DO N=1,NDIM
            XPSOL(N)=XPSOL1(1)
          ENDDO
C
C CHECK IF VERTICAL POSITION IS GIVEN IN METERS (PP0<0)
C
          DO N=1,NDIM
            IF (PP0(N).LT.0) THEN
C
C COMPUTE MODEL OROGRAPHY (m)
C
              CALL INTER2D(GZ0M3,LONM3,LATM3,
     +                     INDX,INDY,
     +                     ALFX1,ALFX2,ALFX3,ALFX4,
     +                     ALFY1,ALFY2,ALFY3,ALFY4,
     +                     ZMOD,1,+1.,.FALSE.,3)
              ZMOD=ZMOD/GG
C
C COMPUTE RELATIVE ALTITUDE RELATIVE TO MODEL SURFACE (m)
C
              ZALTREL=MAX(0.,-PP0(N)-ZMOD(1))
C
C COMPUTE EQUIVALENT PRESSURE
C
              LOGICH=.TRUE.
              ZTOT = 0.
              XP1=XPSOL1(1)
C
              DO M=NIVM3,2,-1
                IF (LOGICH) THEN
                  DO I=1,LONM3*LATM3
                    TGRIM32D(I,1)=TGRIM3(I,1,M)
                    XPHFM32D(I,1)=XPHFM3(I,1,M-1)
                  ENDDO
                  CALL INTER2D(TGRIM32D,LONM3,LATM3,
     +                         INDX,INDY,
     +                         ALFX1,ALFX2,ALFX3,ALFX4,
     +                         ALFY1,ALFY2,ALFY3,ALFY4,
     +                         XTM,1,+1.,.FALSE.,3)
                  CALL INTER2D(XPHFM32D,LONM3,LATM3,
     +                         INDX,INDY,
     +                         ALFX1,ALFX2,ALFX3,ALFX4,
     +                         ALFY1,ALFY2,ALFY3,ALFY4,
     +                         XP2,1,+1.,.FALSE.,3)
                  ZLAYER=RA*XTM(1)/GG*
     +                   MAX(0.,ALOG(XP1/XP2(1)))
                  ZTOT=ZTOT+ZLAYER
                  IF (ZALTREL.LT.ZTOT) THEN
                    a1=(ZTOT-ZALTREL)/ZLAYER
                    PP0(N)=a1*XP1+(1.-a1)*XP2(1)
                    LOGICH=.FALSE.
                  ELSE
                    XP1=XP2(1)
                  ENDIF
                ENDIF
              ENDDO
            ENDIF
          ENDDO
C
C 3.3.3 WEIGHTS FOR VERTICAL INTERPOLATION
C               
          CALL POIDZ  (PP0,XPSOL,ALM3,BLM3,NIVM3
     +                ,INDZ,ALFZ1,ALFZ2,ALFZ3,ALFZ4,NDIM)
C
C 3.3.4 3D-INTERPOLATION
C          
          DO N=1,NDIM
            ALFX1bis(N)=ALFX1(1)
            ALFX2bis(N)=ALFX2(1)
            ALFX3bis(N)=ALFX3(1)
            ALFX4bis(N)=ALFX4(1)
            ALFY1bis(N)=ALFY1(1)
            ALFY2bis(N)=ALFY2(1)
            ALFY3bis(N)=ALFY3(1)
            ALFY4bis(N)=ALFY4(1)
            INDXbis(N)=INDX(1)
            INDYbis(N)=INDY(1)
          ENDDO
C
          DO icst=1,NCONST
            DO ilon=1,LON*LAT*NIV
              XCHRED(ilon,1,1)=XCH(ilon,1,1,icst)
            ENDDO
          DO ILON=1,LON*LAT
	      XCHRED_TEMP(ILON,1)=XCHRED(ILON,1,1)
	    ENDDO
          CALL INTER3D(XCHRED,XCHRED_TEMP,LONM3,LATM3,NIVM3
     +                ,INDXbis,INDYbis,INDZ
     +                ,ALFX1bis,ALFX2bis,ALFX3bis,ALFX4bis
     +                ,ALFY1bis,ALFY2bis,ALFY3bis,ALFY4bis
     +                ,ALFZ1,ALFZ2,ALFZ3,ALFZ4
     +                ,XINTERRED,NDIM,+1.,.FALSE.,3,.TRUE.)
            DO N=1,NDIM
              XINTER(N,icst)=XINTERRED(N)
            ENDDO
          ENDDO
        ENDIF
C      
C 4. ZOOM DOMAIN
C    
      ELSEIF(IGRCH.EQ.4) THEN
C
C 4.1 OBSERVATIONS WITHIN ZOOM DOMAIN
C           
        CALL POSIREG(XX0,YY0,4
     +              ,RLONM2(1),RLONM2(LONM2),RLATM2(1),RLATM2(LATM2)
     +              ,RLONM3(1),RLONM3(LONM3),RLATM3(1),RLATM3(LATM3)
     +              ,RLONM4(1),RLONM4(LONM4),RLATM4(1),RLATM4(LATM4)
     +              ,IGRI,1)
C
C 4.2 OBS. OUTSIDE ZOOM DOMAIN
C          
        IF(IGRI(1).NE.4) THEN
          DO N=1,nbnivpresmax*nbconstmax
            XINTER(N,1)=-777.
          ENDDO 
        ELSE
C      
C 4.3 OBS. WITHIN ZOOM DOMAIN
C      
C 4.3.1 WEIGHTS FOR HORIZONTAL INTERPOLATION
C              
          CALL POIDREG(XX0,RLONM4(1)-DRLONM4,DRLONM4
     +                ,INDX,ALFX1,ALFX2,ALFX3,ALFX4,1)
          CALL POIDREG(YY0,RLATM4(1)-DRLATM4,DRLATM4
     +                ,INDY,ALFY1,ALFY2,ALFY3,ALFY4,1)
C
C 4.3.2 COMPUTE SURFACE PRESSION FOR (X0,Y0)
C         
          CALL INTER2D(PSOLM4,LONM4,LATM4,
     +                 INDX,INDY,
     +                 ALFX1,ALFX2,ALFX3,ALFX4,
     +                 ALFY1,ALFY2,ALFY3,ALFY4,
     +                 XPSOL1,1,+1.,.FALSE.,4) 
C
          DO N=1,NDIM
            XPSOL(N)=XPSOL1(1)
          ENDDO
C
C CHECK IF VERTICAL POSITION IS GIVEN IN METERS (PP0<0)
C
          DO N=1,NDIM
            IF (PP0(N).LT.0) THEN
C
C COMPUTE MODEL OROGRAPHY (m)
C
              CALL INTER2D(GZ0M4,LONM4,LATM4,
     +                     INDX,INDY,
     +                     ALFX1,ALFX2,ALFX3,ALFX4,
     +                     ALFY1,ALFY2,ALFY3,ALFY4,
     +                     ZMOD,1,+1.,.FALSE.,4)
              ZMOD=ZMOD/GG
C
C COMPUTE RELATIVE ALTITUDE RELATIVE TO MODEL SURFACE (m)
C
              ZALTREL=MAX(0.,-PP0(N)-ZMOD(1))
C
C COMPUTE EQUIVALENT PRESSURE
C
              LOGICH=.TRUE.
              ZTOT = 0.
              XP1=XPSOL1(1)
C
              DO M=NIVM4,2,-1
                IF (LOGICH) THEN
                  DO I=1,LONM4*LATM4
                    TGRIM42D(I,1)=TGRIM4(I,1,M)
                    XPHFM42D(I,1)=XPHFM4(I,1,M-1)
                  ENDDO
                  CALL INTER2D(TGRIM42D,LONM4,LATM4,
     +                         INDX,INDY,
     +                         ALFX1,ALFX2,ALFX3,ALFX4,
     +                         ALFY1,ALFY2,ALFY3,ALFY4,
     +                         XTM,1,+1.,.FALSE.,4)
                  CALL INTER2D(XPHFM42D,LONM4,LATM4,
     +                         INDX,INDY,
     +                         ALFX1,ALFX2,ALFX3,ALFX4,
     +                         ALFY1,ALFY2,ALFY3,ALFY4,
     +                         XP2,1,+1.,.FALSE.,4)
                  ZLAYER=RA*XTM(1)/GG*
     +                   MAX(0.,ALOG(XP1/XP2(1)))
                  ZTOT=ZTOT+ZLAYER
                  IF (ZALTREL.LT.ZTOT) THEN
                    a1=(ZTOT-ZALTREL)/ZLAYER
                    PP0(N)=a1*XP1+(1.-a1)*XP2(1)
                    LOGICH=.FALSE.
                  ELSE
                    XP1=XP2(1)			
                  ENDIF
                ENDIF
              ENDDO
            ENDIF
          ENDDO
C
C 4.3.3 WEIGHTS FOR VERTICAL INTERPOLATION
C               
          CALL POIDZ  (PP0,XPSOL,ALM4,BLM4,NIVM4
     +                ,INDZ,ALFZ1,ALFZ2,ALFZ3,ALFZ4,NDIM)
C
C 4.3.4 3D-INTERPOLATION
C          
          DO N=1,NDIM
            ALFX1bis(N)=ALFX1(1)
            ALFX2bis(N)=ALFX2(1)
            ALFX3bis(N)=ALFX3(1)
            ALFX4bis(N)=ALFX4(1)
            ALFY1bis(N)=ALFY1(1)
            ALFY2bis(N)=ALFY2(1)
            ALFY3bis(N)=ALFY3(1)
            ALFY4bis(N)=ALFY4(1)
            INDXbis(N)=INDX(1)
            INDYbis(N)=INDY(1)
          ENDDO
C
          DO icst=1,NCONST
            DO ilon=1,LON*LAT*NIV
              XCHRED(ilon,1,1)=XCH(ilon,1,1,icst)
            ENDDO
          DO ILON=1,LON*LAT
	      XCHRED_TEMP(ILON,1)=XCHRED(ILON,1,1)
	    ENDDO
          CALL INTER3D(XCHRED,XCHRED_TEMP,LONM4,LATM4,NIVM4
     +                ,INDXbis,INDYbis,INDZ
     +                ,ALFX1bis,ALFX2bis,ALFX3bis,ALFX4bis
     +                ,ALFY1bis,ALFY2bis,ALFY3bis,ALFY4bis
     +                ,ALFZ1,ALFZ2,ALFZ3,ALFZ4
     +                ,XINTERRED,NDIM,+1.,.FALSE.,4,.TRUE.)
            DO N=1,NDIM
              XINTER(N,icst)=XINTERRED(N)
            ENDDO
          ENDDO
        ENDIF
C      
      ENDIF
C
	DEALLOCATE (XX0)
	DEALLOCATE (YY0)
	DEALLOCATE (PP0)
	DEALLOCATE (XPSOL)
	DEALLOCATE (XPSOL1)
        DEALLOCATE (ZMOD)
        DEALLOCATE (XTM)
        DEALLOCATE (XP2)
	DEALLOCATE (INDX)
	DEALLOCATE (INDY)
	DEALLOCATE (INDZ)
	DEALLOCATE (INDXbis)
	DEALLOCATE (INDYbis)
	DEALLOCATE (IGRI)
	DEALLOCATE (ALFX1)
	DEALLOCATE (ALFX2)
	DEALLOCATE (ALFX3)
	DEALLOCATE (ALFX4)
	DEALLOCATE (ALFY1)
	DEALLOCATE (ALFY2)
	DEALLOCATE (ALFY3)
	DEALLOCATE (ALFY4)
	DEALLOCATE (ALFZ1)
	DEALLOCATE (ALFZ2)
	DEALLOCATE (ALFZ3)
	DEALLOCATE (ALFZ4)
	DEALLOCATE (ALFX1bis)
	DEALLOCATE (ALFX2bis)
	DEALLOCATE (ALFX3bis)
	DEALLOCATE (ALFX4bis)
	DEALLOCATE (ALFY1bis)
	DEALLOCATE (ALFY2bis)
	DEALLOCATE (ALFY3bis)
	DEALLOCATE (ALFY4bis)
	DEALLOCATE (XCHRED)
	DEALLOCATE (XCHRED_TEMP)
	DEALLOCATE (XINTERRED)
	DEALLOCATE (TGRIM12D)
	DEALLOCATE (XPHFM12D)
	DEALLOCATE (TGRIM22D)
	DEALLOCATE (XPHFM22D)
	DEALLOCATE (TGRIM32D)
	DEALLOCATE (XPHFM32D)
	DEALLOCATE (TGRIM42D)
	DEALLOCATE (XPHFM42D)
C
C
      RETURN
      END
C
CCC
C
