SUBROUTINE INICT3D(idate0)
!
!!!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!!! DATE : 15/05/2005
!
!   Type      : Initializations 
!   Purpose   : Chemical tracers 3D distributions initialization
!   Called by : 
!
!!!
!
!  Original : V.-H. Peuch, 04/1999
!
!  Modified : V.-H. Peuch, 02/2000
!             V.-H. Peuch, 07/2002
!             H. Teyssedre, version Fortran-90, 11/2006
!
!!!
!
USE PARADI_MOD
USE MRAT_MOD
USE MOCF_MOD
USE GRIDS_MOD
USE EXPE_MOD
!
IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
      INTEGER ,DIMENSION(5) :: idate0
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
! VARIABLE NAMES FOR CURRENT CHEMICAL SCHEME
!
#ifdef CARIOLLE
!
!!!
!
!  VARIABLES NAMES FOR CARIOLLE 
!
      CHARACTER(len=15) , DIMENSION(NTRA1) :: NAMES
!
#ifdef NIV_60
#define CPP_ESP 'O_3            '
#else
#define CPP_ESP 'O_x            '
#endif
#endif
!
#ifdef REPROBUS
#ifdef MASSy
#include "REPROBUSMASSY_NAMES"
#else
#include "REPROBUS_NAMES"
#endif
#endif
!
#ifdef RELACS
#ifdef MASSy
#include "RELACSMASSY_NAMES"
#else
#include "RELACS_NAMES"
#endif
#endif
!
#ifdef RACMOBUS
#ifdef MASSy
#include "RACMOBUSMASSY_NAMES"
#else
#include "RACMOBUS_NAMES"
#endif
#endif
#ifdef ACCIDENT
#include "ACCIDENT_NAMES"
#endif

!
! OTHER LOCAL VARIABLES
!
INTEGER :: ITRA,ICTDEP,ICTEMI,I,IEMI,IDEP,ISLS,IB
REAL (KIND=8) :: XKHENRY1,XKHENRY2,XKHENRY3,XKHENRY4
REAL (KIND=8) , ALLOCATABLE :: XKHENRY(:,:)
CHARACTER(LEN=15) :: SURFNAM
CHARACTER(LEN=1)  :: CDUM
!
#ifdef CARIOLLE
ib=0
#ifdef LINO_3
      ib=ib+1
      NAMES(ib)=CPP_ESP
#endif
#ifdef TFROID
      ib=ib+1
      NAMES(ib)='TRACEUR_FROID  '
#endif
#ifdef LINCO
      ib=ib+1
      NAMES(ib)='CO             '
#endif
#ifdef LINHNO_3
      ib=ib+1
      NAMES(ib)='HNO_3          '
#endif
#endif
!
WRITE(*,*) '>> BUILD INITIAL CONDITIONS...'
WRITE(*,*) ' > CHEMICAL SCHEME : ',CHEMSCHEME
!
!!! PREPARE NAMES FOR EMISSIONS/DEPOSITION
!
! 1. GLOBAL DOMAIN
!
IF (IFLAG1) THEN
    DO itra=1,NTRA1
      TRANAM1(itra)=NAMES(itra)
      PRINT*,"QM(",itra,") : ",TRANAM1(itra)
    ENDDO          
    DO itra=1,NSLS1
      SLSNAM1(itra)=NAMES(NTRA1+itra)
      PRINT*,"HM(",itra,") : ",SLSNAM1(itra)
    END DO
!
! PREPARE NAMES FOR EMISSIONS/DEPOSITION
!
#ifndef ACCIDENT
    IF (SURF) THEN
      ALLOCATE (XKHENRY(NDEP1,4))
!	
      OPEN(UNIT=100,file='CHEMSCHEME_SURF',FORM='FORMATTED')
      ICTDEP=0
      ICTEMI=0
!
      DO I=1,NDEP1+NEMI1
        READ(100,101,end=102) SURFNAM,CDUM,XKHENRY1,      &
                        CDUM,XKHENRY2,CDUM,XKHENRY3,      &
                        CDUM,XKHENRY4,IEMI
!
        IF (IEMI /= 0) THEN
          ICTEMI=ICTEMI+1
          EMINAM1(ICTEMI)=SURFNAM
        ENDIF
        IF (XKHENRY1 /= -9.999) THEN
          ICTDEP=ICTDEP+1
          DEPNAM1(ICTDEP)=SURFNAM
          XKHENRY(ICTDEP,1)=XKHENRY1
          XKHENRY(ICTDEP,2)=XKHENRY2
          XKHENRY(ICTDEP,3)=XKHENRY3
          XKHENRY(ICTDEP,4)=XKHENRY4
        ENDIF
      ENDDO
      CLOSE(100)
!
 101  FORMAT(A15,4(A1,E12.4),1X,I1)
 102 &
      IF ((ICTEMI /= NEMI1).OR.(ICTDEP /= NDEP1)) THEN
        WRITE(*,*) '>> EMIS./DEP. MISMATCH : ',ICTEMI,'/',NEMI1,' ; ',ICTDEP,'/',NDEP1
    ENDIF
!
      DO itra=1,NTRA1
        IQMDEP1(itra)=0
        IQMEMI1(itra)=0
        DO idep=1,NDEP1
          IF(TRANAM1(itra) == DEPNAM1(idep)) IQMDEP1(itra)=idep
        ENDDO
        DO iemi=1,NEMI1
          IF(TRANAM1(itra) == EMINAM1(iemi)) IQMEMI1(itra)=iemi
        ENDDO
      ENDDO
      DO isls=1,NSLS1
        IHMDEP1(isls)=0
        DO idep=1,NDEP1
          IF(SLSNAM1(isls) == DEPNAM1(idep)) IHMDEP1(isls)=idep
        ENDDO
      ENDDO
      DEALLOCATE (XKHENRY)
    ENDIF
#endif
ENDIF
!
! 2. CONTINENTAL DOMAIN
!
IF (IFLAG2) THEN
    DO itra=1,NTRA2
      TRANAM2(itra)=NAMES(itra)
    ENDDO
    DO itra=1,NSLS2
      SLSNAM2(itra)=NAMES(NTRA1+itra)
    ENDDO
    IF (SURF) THEN
      DO iemi=1,NEMI2
        EMINAM2(iemi)=EMINAM1(iemi)
      ENDDO
      DO idep=1,NDEP2
        DEPNAM2(idep)=DEPNAM1(idep)
      ENDDO
      DO itra=1,NTRA2
        IQMDEP2(itra)=IQMDEP1(itra)
        IQMEMI2(itra)=IQMEMI1(itra)
      ENDDO
      DO isls=1,NSLS2
        IHMDEP2(isls)=IHMDEP1(isls)
      ENDDO
    ENDIF
ENDIF
!
! 3. REGIONAL DOMAIN
!
IF (IFLAG3) THEN
    DO itra=1,NTRA3
      TRANAM3(itra)=NAMES(itra)
    ENDDO
    DO itra=1,NSLS3
      SLSNAM3(itra)=NAMES(NTRA1+itra)
    ENDDO
    IF (SURF) THEN
      DO iemi=1,NEMI3
        EMINAM3(iemi)=EMINAM1(iemi)
      ENDDO
      DO idep=1,NDEP3
        DEPNAM3(idep)=DEPNAM1(idep)
      ENDDO
      DO itra=1,NTRA3
        IQMDEP3(itra)=IQMDEP1(itra)
        IQMEMI3(itra)=IQMEMI1(itra)
      ENDDO
      DO isls=1,NSLS3
        IHMDEP3(isls)=IHMDEP1(isls)
      ENDDO
    ENDIF
ENDIF
!
! 4. ZOOM DOMAIN
!
IF (IFLAG4) THEN
    DO itra=1,NTRA4
      TRANAM4(itra)=NAMES(itra)
    ENDDO
    DO itra=1,NSLS4
      SLSNAM4(itra)=NAMES(NTRA1+itra)
    ENDDO
    IF (SURF) THEN
      DO iemi=1,NEMI4
        EMINAM4(iemi)=EMINAM1(iemi)
      ENDDO
      DO idep=1,NDEP4
        DEPNAM4(idep)=DEPNAM1(idep)
      ENDDO
      DO itra=1,NTRA4
        IQMDEP4(itra)=IQMDEP1(itra)
        IQMEMI4(itra)=IQMEMI1(itra)
      ENDDO
      DO isls=1,NSLS4
        IHMDEP4(isls)=IHMDEP1(isls)
      ENDDO
    ENDIF
ENDIF
!
CALL INHISTO(idate0)
!	
!
RETURN
END SUBROUTINE INICT3D
