       SUBROUTINE INIHENRY
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Physical parameterizations 
C   Purpose   : Initialize effective Henry's law constants
C   Called by : 
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 07/2002
C
C  Modified : H. Le Berre 12/2006 Bug fix on XKENRY1 sign
C
CCC
C
	USE PARADI_MOD
	USE MRAT_MOD
	USE HENRY_MOD
C
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~		
C
CCC HENRY's LAW CONSTANTS
C
C  KHenry(T)=KHenry(T0) * exp{-DeltaH0(T0) /R * ( 1/T - 1/T0)}
C
C  WHERE : T0=298K
C
C  XKHenry(J,1) =  KHenry (T0)
C  XKHenry(J,2) =  DeltaH0(T0) /R
C
	INTEGER :: JHENRY,ITRA,ITES,IO3,NTAB
	REAL (KIND=8) , ALLOCATABLE :: XKHENRY1D(:,:)	
C
        PARAMETER(NTAB=142)        
C	
C
CCC MASTER TABLE FOR HENRY's LAW CONSTANTS
C
C N.B. : FOR NOx, constants correspond to NO2
C
C
	CHARACTER(LEN=15) , dimension(NTAB), parameter :: NAMHENRY = (/
     +        'O_3            ','H_2O_2         ',
     +        'NO             ','NO_2           ',
     +        'NO_3           ','N_2O_5         ',
     +        'HONO           ','HNO_3          ',
     +        'HNO_4          ','SO_2           ',
     +        'SULF           ','CO             ',
     +        'HO_2           ','CH_4           ',
     +        'ETH            ','HC3            ',
     +        'HC5            ','HC8            ',
     +        'ETE            ','OLT            ',
     +        'OLI            ','DIEN           ',
     +        'ISO            ','TOL            ',
     +        'XYL            ','CSL            ',
     +        'HCHO           ','ALD            ',
     +        'KET            ','GLY            ',
     +        'MGLY           ','DCB            ',
     +        'MACR           ','UDD            ',
     +        'HKET           ','ONIT           ',
     +        'PAN            ','TPAN           ',
     +        'OP1            ','OP2            ',
     +        'PAA            ','ORA1           ',
     +        'ORA2           ','MO2            ',
     +        'ETHP           ','HC3P           ',
     +        'HC5P           ','HC8P           ',
     +        'ETEP           ','OLTP           ',
     +        'OLIP           ','ISOP           ',
     +        'C3H6bis        ','OLE1           ',
     +        'OLE2           ','PHEN           ',
     +        'NPHE           ','BALD           ',
     +        'RCHO           ','ACET           ',
     +        'MEK            ','MVK            ',
     +        'PROD2          ','DCB2           ',
     +        'DCB3           ','PBZN           ',
     +        'C2H2           ','CH3CHO         ',
     +        'CH3CHObis      ','PCHO           ',
     +        'C2H5OH         ','ROH            ',
     +        'C3H6OHOOH      ','C2H5OOH        ',
     +        'C3H7OOH        ','CH3COCH2OOH    ',
     +        'MACROOH        ','XOOH           ',
     +        'MEKOOH         ','ALKENEOOH      ',
     +        'AROMATICOOH    ','APINPAN        ',
     +        'PCO3PAN        ','ONITR          ',
     +        'CH3COOH        ','CH2OOHCH2OH    ',
     +        'CH3CHOOHCH2OH  ','OXYO2H         ',
     +        'MALO2H         ','ISRO2H         ',
     +        'MVKO2H         ','CH2CO2HCH3     ',
     +        'ISNIRH         ','ISNO3H         ',
     +        'ISNI           ','ISOPROD        ',
     +        'GLYC           ','HAC            ',
     +        'IALD           ','PPN            ',
     +        'GPAN           ','ACTA           ',
     +        'HCOOH          ','RP             ',
     +        'CH3OH          ','CH3COY         ',
     +        'obioH          ','CARNIT         ',
     +        'OFURAN         ','CH3O2C         ',
     +        'MEMALH         ','PEROX          ',
     +        'CH4COZ         ','APIO2H         ',
     +        'MEMAH2         ','MACO2H         ',
     +        'MEMPAN         ','ETNO3          ',
     +        'CH3NO3         ','ORNIT2         ',
     +        'CH3NO4         ','PB_210         ',
     +        'NO_x           ','CCl_4          ',
     +        'CH_3CCl_3      ','HCFC22         ',
     +        'CFC12          ','CFC11          ',
     +        'CFC113         ','CH_3Br         ',
     +        'HCl            ','HOCl           ',
     +        'ClONO_2        ','Cl_2           ',
     +        'ClO_x          ','BrONO_2        ',
     +        'OClO           ','HBr            ',
     +        'HOBr           ','BrCl           ',
     +        'BrO_x          ','ClNO_2         '/)
      
	REAL (KIND=8) , dimension(NTAB), parameter :: XHENRY1 =
     +        (/ 0.1000E-01,  0.1000E+06,  0.2000E-03,
     +              0.1000E-01,  0.2000E+01,  0.2100E+01,
     +              0.1000E+06,  0.1000E+15,  0.2000E+05,
     +              0.1000E+06,  0.2600E+07,  0.9500E-03,
     +              0.8300E+05,  0.1400E-02,  0.1900E-02,
     +              0.1500E-02,  0.8120E-03,  0.3380E-03,
     +              0.4660E-02,  0.4756E-02,  0.1350E-02,
     +              0.1430E-01,  0.1300E-01,  0.1800E+00,
     +              0.2500E+00,  0.8200E+03,  0.6000E+04,
     +              0.1500E+02,  0.2000E+02,  0.3600E+06,
     +              0.3710E+06,  0.7400E+02,  0.7400E+01,
     +              0.7400E+02,  0.7400E+02,  0.1000E+02,
     +              0.3600E+01,  0.3600E+01,  0.2200E+03,
     +              0.2200E+01,  0.2200E+01,  0.4000E+07,
     +              0.4000E+07,  0.3000E+03,  0.3400E+03,
     +              0.2660E+03,  0.1450E+03,  0.6000E+02,
     +              0.8300E+03,  0.8450E+03,  0.2400E+03,
     +              0.2311E+04,  0.4756E-02,  0.4756E-02,
     +              0.1350E-02,  0.2500E+04,  0.2500E+06,
     +              0.4200E+02,  0.1500E+02,  0.2000E+02,
     +              0.2000E+02,  0.5800E+02,  0.2000E+02,
     +              0.7400E+02,  0.7400E+02,  0.3600E+01,
     +              0.4100E-01,  0.1500E+02,  0.1500E+02,
     +              0.5400E+05,  0.1400E+03,  0.1400E+03,
     +              0.2200E+01,  0.2200E+01,  0.2200E+01,
     +              0.2200E+01,  0.2200E+01,  0.2200E+01,
     +              0.2200E+01,  0.2200E+01,  0.2200E+01,
     +              0.3600E+01,  0.3600E+01,  0.1090E+05,
     +              0.2200E+01,  0.2200E+01,  0.2200E+01,
     +              0.2200E+01,  0.2200E+01,  0.2200E+01,
     +              0.2200E+01,  0.2200E+01,  0.2200E+01,
     +              0.2200E+01,  0.3000E+11,  0.4250E+02,
     +              0.8600E+06,  0.7400E+02,  0.1800E+10,
     +              0.3600E+01,  0.3600E+01,  0.3080E+07,
     +              0.3080E+07,  0.2200E+01,  0.2200E+03,
     +              0.8600E+06,  0.2200E+01,  0.1090E+05,
     +              0.2000E+02,  0.2200E+01,  0.2200E+01,
     +              0.2200E+01,  0.2200E+01,  0.2200E+01,
     +              0.2200E+01,  0.2200E+01,  0.3600E+01,
     +              0.1000E+02,  0.1000E+02,  0.1090E+05,
     +              0.2000E+05,  0.1000E+15,  0.1000E-01,
     +              0.3400E-01,  0.5900E-01,  0.3400E-01,
     +              0.2300E-02,  0.1000E-01,  0.2000E-02,
     +              0.1900E-00,  0.1538E+01,  0.6600E+01,
     +              0.1000E+15,  0.9500E-01,  0.2000E+00,
     +              0.1000E+15,  0.1500E+01,  0.1300E+01,
     +              0.1800E+01,  0.9400E+00,  0.1200E+01,
     +              0.2400E-01/)

	REAL (KIND=8) , dimension(NTAB), parameter :: XHENRY2 =
     +           (/ -0.2300E+04, -0.6800E-04, -0.1480E+04,
     +              -0.2500E+04, -0.2000E+04, -0.3400E+04,
     +              -0.4800E+04, -0.8650E+04, -0.1500E+04,
     +              -0.3020E+04, -0.8700E+04, -0.1300E+04,
     +              -0.7400E+04, -0.1600E+04, -0.2300E+04,
     +              -0.2700E+04, -0.3171E+04, -0.4977E+04,
     +              -0.1800E+04, -0.2197E+04, -0.2535E+04,
     +              -0.4500E+04, -0.2444E+04, -0.4500E+04,
     +              -0.4200E+04, -0.5463E+04, -0.6500E+04,
     +              -0.6500E+04, -0.5000E+04, -0.7545E+04,
     +              -0.7545E+04, -0.5700E+04, -0.4885E+04,
     +              -0.5700E+04, -0.5700E+04, -0.5910E+04,
     +              -0.5910E+04, -0.5910E+04, -0.5600E+04,
     +              -0.5600E+04, -0.5600E+04, -0.5740E+04,
     +              -0.5740E+04, -0.5300E+04, -0.6000E+04,
     +              -0.6000E+04, -0.6000E+04, -0.6000E+04,
     +              -0.6000E+04, -0.6000E+04, -0.6000E+04,
     +              -0.6000E+04,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00,  0.0000E+00,  0.0000E+00,
     +               0.0000E+00, -0.8650E+04, -0.2500E+04,
     +              -0.4200E+04, -0.3900E+04, -0.3400E+04,
     +               0.0000E+00, -0.3100E+04,  0.0000E+00,
     +               0.0000E+00, -0.9000E+04, -0.5900E+04,
     +               0.0000E+00, -0.2100E+04,  0.0000E+00,
     +               0.0000E+00, -0.3300E+04, -0.1000E+05,
     +               0.0000E+00, -0.5600E+04,  0.0000E+00,
     +               0.0000E+00/)
C
CCC 0. INITIALIZATIONS
C
      DO JHENRY=1,2
        DO ITRA=1,NTRA1
          XKHENRY1(ITRA,JHENRY)=0.
        ENDDO
      ENDDO
      DO JHENRY=1,2
        DO ITRA=1,NTRA2
          XKHENRY2(ITRA,JHENRY)=0.
        ENDDO
      ENDDO
      DO JHENRY=1,2
        DO ITRA=1,NTRA3
          XKHENRY3(ITRA,JHENRY)=0.
        ENDDO
      ENDDO
      DO JHENRY=1,2
        DO ITRA=1,NTRA4
          XKHENRY4(ITRA,JHENRY)=0.
        ENDDO
      ENDDO
      ALLOCATE (XKHENRY1D(NTRA1D,2))
      DO JHENRY=1,2
        DO ITRA=1,NTRA1D
          XKHENRY1D(ITRA,JHENRY)=0.
        ENDDO
      ENDDO
C
CCC 1. FILL-IN TABLES ACCORDING TO GLOBAL DOMAIN
C
      DO ITES=1,NTAB
        IF(NAMHENRY(ITES).EQ.'O_3            ') THEN
          IO3=ITES
        ENDIF
      ENDDO
      DO ITRA=1,NTRA1
        DO ITES=1,NTAB
          IF(TRANAM1(ITRA).EQ.NAMHENRY(ITES)) THEN
            XKHENRY1(ITRA,1)= XHENRY1(ITES)
            XKHENRY1(ITRA,2)= XHENRY2(ITES)
          ENDIF
        ENDDO
        IF(TRIM(TRANAM1(ITRA)).EQ.'O_x') THEN
          XKHENRY1(ITRA,1)= XHENRY1(IO3)
          XKHENRY1(ITRA,2)= XHENRY2(IO3)
        ENDIF
      ENDDO
C
CCC 2. FILL-IN TABLES FOR THE OTHER DOMAINS
C
      DO JHENRY=1,2
        DO ITRA=1,NTRA2
          XKHENRY2(ITRA,JHENRY)=XKHENRY1(ITRA,JHENRY)
        ENDDO
      ENDDO
      DO JHENRY=1,2
        DO ITRA=1,NTRA3
          XKHENRY3(ITRA,JHENRY)=XKHENRY1(ITRA,JHENRY)
        ENDDO
      ENDDO
      DO JHENRY=1,2
        DO ITRA=1,NTRA4
          XKHENRY4(ITRA,JHENRY)=XKHENRY1(ITRA,JHENRY)
        ENDDO
      ENDDO
      DO JHENRY=1,2
        DO ITRA=1,NTRA1D
          XKHENRY1D(ITRA,JHENRY)=XKHENRY1(ITRA,JHENRY)
        ENDDO
      ENDDO
C
	DEALLOCATE (XKHENRY1D)
C
C
      RETURN
      END
