SUBROUTINE INIPHOT
!
!CC
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!CC DATE : 15/05/2005
!
!   Type       : Chemistry
!   Purpose    : Initialize chemical scheme and constants 
!   Called by  :
!
!CC
!
!  Original : F. Lefevre (REPROBUS, LIB37)
!
!  Modified : VH. Peuch    08/1999
!             (inclusion dans MOCAGE)
!             H. Teyssedre 04/2000
!             (table avec especes tropospheriques
!              et stratospheriques, modification
!              du format de lecture)
!             F. Brocheton, VH. Peuch 10/2000
!             (nouveau master, format de lecture
!              binaire) 
!             H. Teyssedre, 11/2006
!             (ajout photolyse du CO2 et version
!              Fortran-90)
!
!!!
!
!!! MASTER LOOK-UP TABLE FOR PHOTOLYSES
!
!!! VERSION : 02/2002, 115 PHOTOLYSES IN TABLE
!
!  1  : NO2    --> O3P + NO
!  2  : O3     --> O1D + O2
!  3  : O3     --> O3P + O2
!  4  : HONO   --> OH  + NO
!  5  : HNO3   --> OH  + NO2
!  6  : HNO4   --> x*HO2 + x*NO2 + (1-x)*OH + (1-x)*NO3
!  7  : NO3    --> NO + O2
!  8  : NO3    --> NO2 + O3P
!  9  : H2O2   --> OH + OH
! 10  : HCHO   --> H2 + CO
! 11  : HCHO   --> H + CO + HO2
! 12  : ALD    --> CH3 + HO2 + CO
! 13  : OP1    --> CH3O + OH
! 14  : OP2    --> ALD + HO2 + OH
! 15  : PAA    --> CH3 + OH + CO2
! 16  : KET    --> ACO3 + ETHP
! 17  : GLY    --> 0.13*HCHO + 1.87*CO + 0.87*H2
! 18  : GLY    --> 0.45*HCHO + 1.55*CO + 0.80*HO2 + 0.15*H2
! 19  : MGLY   --> ACO3 + HO2 + CO
! 20  : DCB    --> TCO3 + HO2
! 21  : ONIT   --> HO2 + NO2 + 0.20*ALD + 0.80*KET
! 22  : MACR   --> HCHO + ACO3 + CO + HO2
! 23  : HKET   --> ACO3 + HCHO + HO2
! 24  : O2     --> O3P + O3P
! 25  : N2O5   --> NOx + NOx + XX
! 26  : H2O    --> H + OH
! 27  : N2O    --> N2 + O1D
! 28  : HOCl   --> Cl + OH
! 29  : ClONO2 --> Cl + NO3
! 30  : Cl2O2  --> Cl + Cl + O2
! 31  : HCl    --> H + Cl
! 32  : Cl2    --> Cl + Cl
! 33  : ClNO2  --> Cl + NO2
! 34  : BrO    --> Br + O3P
! 35  : BrONO2 --> Br + NO3
! 36  : HOBr   --> Br + OH
! 37  : OClO   --> O3P + ClO
! 38  : BrCl   --> Br + Cl
! 39  : CFC11  --> ClOx + ClOx + ClOx + XX
! 40  : CFC12  --> ClOx + ClOx + XX
! 41  : CFC113 --> ClOx + ClOx + ClOx + XX
! 42  : CCl4   --> ClOx + ClOx + ClOx + ClOx + XX
! 43  : CH3CCl3--> ClOx + ClOx + ClOx + XX
! 44  : CH3Cl  --> ClOx + XX
! 45  : HCFC22 --> ClOx + XX
! 46  : CH3Br  --> BrOx + XX
! 47  : H1211  --> BrOx + XX
! 48  : H1301  --> BrOx + XX
! 49  : NO     --> N + O3P
! 50  : HONO   --> H + NO2
! 51  : CH3CHO --> CH3 + CO + HO2
! 52  : RCHO   --> CH3CHO + RO2R + CO + HO2
! 53  : ACET   --> CCOO2 + CH3
! 54  : MEK    --> CCOO2 + CH3CHO + RO2R
! 55  : GLY    --> HO2 + HO2 + CO + CO
! 56  : GLY    --> HCHO + CO
! 57  : BACL   --> CCOO2 + CCOO2
! 58  : BALD   --> XC1 + XC1 + XC1 + XC1 + XC1 + XC1 + XC1
! 59  : MVK    --> 0.3*MO2 + 0.7*CO + 0.7*PROD2 + 0.3*TCO3 + 2.4*XC2
! 60  : ISOPROD--> HO2,CCOO2,RCO3,CO,HCHO,CH3CHO,MEK,XC2 [Carter et al., 1999]
! 61  : PROD2  --> RO2R,RO2N,R2O2,CCOO2,RCO3,HCHO,CH3CHO,RCHO,XC1 [ibidem]
! 62  : ONIT   --> NO2,HO2,RO2R,RO2N,R2O2,HCHO,CH3CHO,RCHO,ACET,MEK,PROD2,XC1 [ibidem]
! 63  : DCB2   --> RO2R + 0.5*CCOO2 + 0.5*HO2 + CO + R2O2 + 0.5*GLY + 0.5*MGLY + 0.5*XC1
! 64  : DCB3   --> RO2R + 0.5*CCOO2 + 0.5*HO2 + CO + R2O2 + 0.5*GLY + 0.5*MGLY + 0.5*XC1
! 65  : CH3CHObis --> CH3 + HO2 + CO
! 66  : C2H5OOH--> OH + HO2 + CH3CHObis
! 67  : MEKOOH --> CH3CHObis + CH3COO2 + OH
! 68  : CH2OOHCH2OH --> HO2 + OH + 1.56*HCHO + 0.22*CH3CHObis
! 69  : CH3CHOOHCH2OH --> CH3CHObis + HCHO + HO2 + OH
! 70  : OXYO2H --> OH + MGLY + MAL + HO2
! 71  : MALO2H --> OH + HO2 + MGLY + GLY
! 72  : PCHO   --> HO2 + CO + XO2 + C3H6OHO2
! 73  : PAN    --> CH3CO3 + NO2
! 74  : TPAN   --> TCO3 + NO2
! 75  : APINPAN--> APINO3 + NO2
! 76  : PCO3PAN--> PCO3 + NO2
! 77  : ONITR  --> HO2 + CO + NO2 + HCHO
! 78  : C3H6OHOOH --> CH3CHObis + HCHO + HO2 + OH
! 79  : C3H7OOH--> 0.218*CH3CHObis + 0.782*ACET + OH + HO2
! 80  : CH3COCH2OOH --> HCHO + CH3CO3 + OH
! 81  : XOOH   --> OH
! 82  : MACROOH--> OH + 0.84*CO + 0.16*HCHO + HO2 + 0.16*MGLY + 0.84*MGLY
! 83  : MACROOH--> CO + 1.16*HO2 + 0.84*OH + 0.84*MGLY + 0.16*CH3COCH2OOH
! 84  : ALKENEOOH --> OH + HO2 + 0.165*HCHO + 0.68*CH3CHObis + 0.155*ACET
! 85  : AROMATICOOH --> OH + 0.423*MGLY + 1.658*HO2 + 0.658*CO + 0.658*CH3CO3
! 86  : N2O5   --> NO3 + NO2
! 87  : N2O5   --> NO + NO3 + O3P
! 88  : CH3CHO --> CH4 + CO
! 89  : CH3NO3 --> HCHO + H2O + NO2
! 90  : GLYC   --> HCHO + HO2 + HO2 + CO
! 91  : GLY    --> TOT+0.333*H2+0.666*CO+0.666*CO+0.666*HO2+0.333*HCHO+0.333*CO
! 92  : HAC    --> MCO3 + HCHO + HO2
! 93  : RP     --> OH + HO2 + CH3CHO
! 94  : obioH  --> OH + HO2     
! 95  : MACO2H --> OH + HO2        
! 96  : CH3COY --> CH3COO + CH3COO
! 97  : ETNO3  --> CH3CHObis + HO2 + NO2
! 98  : CARNIT --> CH3COO + HCHO + NO2
! 99  : CH3NO4 --> MO2 + NO2
! 100 : CH3O2C --> HCHO + HCHO + OH + HO2
! 101 : MEMALH --> GLY + MGLY + OH + HO2
! 102 : MEMAH2 --> MALEIC + OH + MO2
! 103 : PEROX  --> 0.14*RO2IP1 + 0.488*MEK + 0.372*MACR + 0.86*HCHO + 0.9*HO2 + OH
! 104 : MVKO2H --> 0.28*HCHO + 0.28*MGLY + 0.72*CH3CHObis + 0.72*CH3COO
! 105 : CH4COZ --> MEK + OH + HO2
! 106 : APIO2H --> CH3CHObis + MEK + OH + HO2
! 107 : ORNIT2 --> 0.95*(CH3CHObis+NO2) + 0.475*(MEK+MGLY) + 0.05*(ISNI+HO2) + OH
! 108 : ClONO2 --> ClO + NO2
! 109 : C2H5CHO --> C2H5O2 + HO2 + CO 
! 110 : OXEPIN --> PHENOL 
! 111 : ALD3 --> HCHO + XO2 + HO2 + HO2 + CO
! 112 : UV index [Diffey-Mc Kinlay] WMO/WHO
! 113 : UV-B (280 - 315 nm)
! 114 : UV-A (315 - 400 nm)
! 115 : CO2 --> CO + O(3P)
!
!!!
!
USE PARADI_MOD
USE PHOTO_MOD
USE CSTES_MOD
!
      IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!	
INTEGER :: I,J
REAL (KIND=8) , ALLOCATABLE :: ajt(:,:,:,:,:)
CHARACTER(LEN=160) , ALLOCATABLE :: cname(:)
CHARACTER(LEN=80) :: header
CHARACTER(LEN=3) :: cphotmaster
!
#ifdef REPROBUS
#include "REPROBUS_IND"
#endif
!
#ifdef RELACS
#include "RELACS_IND"
#endif
!
#ifdef RACMOBUS
#include "RACMOBUS_IND"
#endif
#ifdef ACCIDENT
#include "ACCIDENT_IND"
#endif

!
ALLOCATE (ajt(nphotmaster,nz,nalb,nsza,nozo))
ALLOCATE (cname(nphotmaster))
!
WRITE(cphotmaster,'(I3.3)') nphotmaster
!
WRITE(*,*) '>> INITIALIZING CHEMISTRY'
!
!!! READ-IN MASTER LOOK-UP TABLE
!
OPEN( unit = 99, file = 'jdata'//cphotmaster, form = 'unformatted')
WRITE(*,*) '> READING MASTER LOOK-UP TABLE FOR PHOTOLYSES'
READ(99)   header
WRITE(*,*) header
READ(99)   cname
READ(99)   ajt
CLOSE (99)
!
!!!
!
WRITE(*,*) '> PHOTODISSOCIATIONS TAKEN INTO ACCOUNT :'
!  
iJO1D   =0
iJNO2   =0
iINDUV  =0
iINDUVCS=0
iUVB    =0
iUVA    =0
!
DO i=1,nphot
!
  IF(MYID == 0) WRITE(*,*) i,cname(ind(i))
!
  IF (cname(ind(i)) == 'O3 --> O1D + O2') iJO1D=i
  IF (cname(ind(i)) == 'NO2 --> O3P + NO') iJNO2=i
  IF ((cname(ind(i)) == 'INDUV').AND.(iINDUV >  0)) iINDUVCS=i
  IF ((cname(ind(i)) == 'INDUV').AND.(iINDUV == 0)) iINDUV=i
  IF (cname(ind(i)) == 'UV-B')  iUVB =i
  IF (cname(ind(i)) == 'UV-A')  iUVA =i
!
  DO j = 1,nz*nalb*nsza*nozo
    ajl(i,j,1,1,1) = ajt(ind(i),j,1,1,1)
  ENDDO
ENDDO
!
! NO AND CO2 PHOTOLYSIS
!
iphotNO =0
iphotCO2=0
DO i=1,nphot
  if (ind(i) ==  49) iphotNO =i
  if (ind(i) == 115) iphotCO2=i
ENDDO
IF (iphotNO == 0) THEN
  WRITE(*,*) 'NITROGEN MONOXIDE (NO) PHOTOLYSIS IS NOT IN CURRENT SCHEME : ABORT!'
  CALL MOCAGE_STOP
ENDIF
!
DEALLOCATE (ajt)
DEALLOCATE (cname)
!
RETURN
END SUBROUTINE INIPHOT
