      SUBROUTINE INITOP
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Chemistry
C   Purpose    : Initialize top boundaries for chemistry
C   Called by  :
C
CCC
C
C  Original : H. Teyssedre, V.-H. Peuch 03/2001
C
C  Modified :
C
CCC
C
C  PARAMETERS FOR CHEMICAL TOP-BOUNDARY FILES
C
      
C
      USE PARADI_MOD
      USE MOCF_MOD
      USE EXPE_MOD
      USE GRIDS_MOD
      USE MRAT_MOD
      USE TOPCH_MOD
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C	
      INTEGER :: NLATTOPMAX,NTRATOPMAX,JC
      INTEGER :: I,LATCHF,NTRACHF,ITRA,ILAT,ISP,ITRACHF,ILATCHF,ILON
      REAL (KIND=8) :: YFACT,BrLim
      REAL (KIND=8) , ALLOCATABLE :: RLATCHF(:)
      REAL (KIND=8) , ALLOCATABLE :: QMCHF(:,:)
      CHARACTER(LEN=15) , ALLOCATABLE :: TRANAMCHF(:)
C
      PARAMETER(nlattopmax=34,ntratopmax=55)
C
      ALLOCATE (RLATCHF(nlattopmax)) ; RLATCHF(:) = 0.0
      ALLOCATE (QMCHF(nlattopmax,ntratopmax)) ; QMCHF(:,:) = 0.0
      ALLOCATE (TRANAMCHF(ntratopmax)) ; TRANAMCHF(:) ='123456789 12345'
C
      WRITE(*,*) '>> INITIALIZING TOP BOUNDARY CONDITIONS'
C
      IF ((CHEMSCHEME.EQ.'TRACERS0').OR.
     +    (CHEMSCHEME.EQ.'TRACERCO').OR.
     +    (CHEMSCHEME.EQ.'ACCIDENT').OR.
     +    (CHEMSCHEME.EQ.'AEROSOLS')) THEN
C
        IF (IFLAG1) THEN
          DO i=1,LONM1*LATM1*NTRA1
            QQMCHF1(i,1,1)=0.
          ENDDO
        ENDIF
        IF (IFLAG2) THEN
          DO i=1,LONM2*LATM2*NTRA2
            QQMCHF2(i,1,1)=0.
          ENDDO
        ENDIF
        IF (IFLAG3) THEN
          DO i=1,LONM3*LATM3*NTRA3
            QQMCHF3(i,1,1)=0.
          ENDDO
        ENDIF
        IF (IFLAG4) THEN
          DO i=1,LONM4*LATM4*NTRA4
            QQMCHF4(i,1,1)=0.
          ENDDO
        ENDIF
C
      ELSE
C
CCC READ-IN TOP-BOUNDARY FILE
C
      OPEN( unit = 98, file = 'topbd', form = 'unformatted')
      READ (98) LATCHF,NTRACHF
      IF ((LATCHF.GT.nlattopmax).OR.
     +    (NTRACHF.GT.ntratopmax)) THEN
        WRITE (*,*) "ABORT IN INITOP : WRONG TOP BOUNDARIES!"
        CALL MOCAGE_STOP
      ENDIF
C
      READ (98) (TRANAMCHF(itra),itra=1,NTRACHF)
      READ (98) (RLATCHF(ilat),ilat=1,LATCHF)
     +    ,((QMCHF(ilat,itra),ilat=1,LATCHF),itra=1,NTRACHF)
      CLOSE (98)
C
C GLOBAL DOMAIN
C
      IF (IFLAG1) THEN
        DO itra=1,NTRA1
          isp=0
          DO itrachf=1,NTRACHF
            IF (TRANAMCHF(itrachf).EQ.TRANAM1(itra)) isp=itrachf
          ENDDO
          IF (isp.NE.0) THEN
            DO ilat=1,LATM1
              QQMCHF1(1,ilat,itra) = 0.
              DO ilatchf=1,LATCHF-1
                IF ((RLATM1(ilat).GE.RLATCHF(ilatchf)).AND.
     +              (RLATM1(ilat).LE.RLATCHF(ilatchf+1))) THEN
                  YFACT=(RLATCHF(ilatchf+1)-RLATM1 (ilat   ))/
     +                  (RLATCHF(ilatchf+1)-RLATCHF(ilatchf))
                  QQMCHF1(1,ilat,itra) =
     +                      YFACT *QMCHF(ilatchf  ,isp) +
     +                  (1.-YFACT)*QMCHF(ilatchf+1,isp)
                ENDIF
              ENDDO
              DO ilon=1,LONM1
                QQMCHF1(ilon,ilat,itra)=MAX(1.E-30,
     +                                  QQMCHF1(1,ilat,itra))
              ENDDO
            ENDDO 
          ELSE
            DO ilon=1,LONM1*LATM1
              QQMCHF1(ilon,1,itra)=1.E-30
            ENDDO
          ENDIF
        ENDDO
#ifdef MASSy
#ifdef REPROBUS
          print*,' TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM1
            QQMCHF1(ilon,1,39) =
     +         QQMCHF1(ilon,1,4) + 2.0*QQMCHF1(ilon,1,5) +
     +         QQMCHF1(ilon,1,9) + QQMCHF1(ilon,1,13) +
     +         QQMCHF1(ilon,1,15) + QQMCHF1(ilon,1,16) +
     +         QQMCHF1(ilon,1,36)
          enddo
          print*,' TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM1
            QQMCHF1(ilon,1,40) =
     +         QQMCHF1(ilon,1,7) + QQMCHF1(ilon,1,8) +
     +         QQMCHF1(ilon,1,9) + QQMCHF1(ilon,1,10) +
     +     2.0*QQMCHF1(ilon,1,11) + QQMCHF1(ilon,1,13) +
     +     2.0*QQMCHF1(ilon,1,17) + QQMCHF1(ilon,1,19) +
     +         QQMCHF1(ilon,1,37)
          enddo
C
          BrLIM=20.9
          print*,'Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM1
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF1(ilon,1,41) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF1(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF1(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#ifdef RELACS
          print*,' TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM1
            QQMCHF1(ilon,1,62) =
     +         QQMCHF1(ilon,1,4) + 2.0*QQMCHF1(ilon,1,5) +
     +         QQMCHF1(ilon,1,9) + QQMCHF1(ilon,1,13) +
     +         QQMCHF1(ilon,1,15) + QQMCHF1(ilon,1,16) +
     +         QQMCHF1(ilon,1,35) + QQMCHF1(ilon,1,55) +
     +         QQMCHF1(ilon,1,59)
          enddo
          print*,' TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM1
            QQMCHF1(ilon,1,63) =
     +         QQMCHF1(ilon,1,7) + QQMCHF1(ilon,1,8) +
     +         QQMCHF1(ilon,1,9) + QQMCHF1(ilon,1,10) +
     +     2.0*QQMCHF1(ilon,1,11) + QQMCHF1(ilon,1,13) +
     +     2.0*QQMCHF1(ilon,1,17) + QQMCHF1(ilon,1,19) +
     +         QQMCHF1(ilon,1,60)
          enddo
C
          BrLIM=20.9
          print*,'TOP Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM1
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF1(ilon,1,64) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF1(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF1(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#ifdef RACMOBUS
          print*,'TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM1
            QQMCHF1(ilon,1,90) =
     +         QQMCHF1(ilon,1,4) + 2.0*QQMCHF1(ilon,1,5) +
     +         QQMCHF1(ilon,1,9) + QQMCHF1(ilon,1,13) +
     +         QQMCHF1(ilon,1,15) + QQMCHF1(ilon,1,16) +
     +         QQMCHF1(ilon,1,35) + QQMCHF1(ilon,1,60) +
     +         QQMCHF1(ilon,1,61) + QQMCHF1(ilon,1,62) +
     +         QQMCHF1(ilon,1,82) + QQMCHF1(ilon,1,83) +
     +         QQMCHF1(ilon,1,87)
          enddo
          print*,'TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM1
            QQMCHF1(ilon,1,91) =
     +         QQMCHF1(ilon,1,7) + QQMCHF1(ilon,1,8) +
     +         QQMCHF1(ilon,1,9) + QQMCHF1(ilon,1,10) +
     +     2.0*QQMCHF1(ilon,1,11) + QQMCHF1(ilon,1,13) +
     +     2.0*QQMCHF1(ilon,1,17) + QQMCHF1(ilon,1,19) +
     +         QQMCHF1(ilon,1,88)
          enddo
C
          BrLIM=20.9
          print*,'TOP Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM1
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF1(ilon,1,92) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF1(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF1(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#endif
      ENDIF
C
C CONTINENTAL DOMAIN
C
      IF (IFLAG2) THEN
        DO itra=1,NTRA2
          isp=0
          DO itrachf=1,NTRACHF
            IF (TRANAMCHF(itrachf).EQ.TRANAM2(itra)) isp=itrachf
          ENDDO
          IF (isp.NE.0) THEN
            DO ilat=1,LATM2
              QQMCHF2(1,ilat,itra) = 0.
              DO ilatchf=1,LATCHF-1
                IF ((RLATM2(ilat).GE.RLATCHF(ilatchf)).AND.
     +              (RLATM2(ilat).LE.RLATCHF(ilatchf+1))) THEN
                  YFACT=(RLATCHF(ilatchf+1)-RLATM2 (ilat   ))/
     +                  (RLATCHF(ilatchf+1)-RLATCHF(ilatchf))
                  QQMCHF2(1,ilat,itra) =
     +                      YFACT *QMCHF(ilatchf  ,isp) +
     +                  (1.-YFACT)*QMCHF(ilatchf+1,isp)
                ENDIF
              ENDDO
              DO ilon=1,LONM2
                QQMCHF2(ilon,ilat,itra)=MAX(1.E-30,
     +                                 QQMCHF2(1,ilat,itra))
              ENDDO
            ENDDO
          ELSE
            DO ilon=1,LONM2*LATM2
              QQMCHF2(ilon,1,itra)=1.E-30
            ENDDO
          ENDIF
        ENDDO
#ifdef MASSy
#ifdef REPROBUS
          print*,' TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM2
            QQMCHF2(ilon,1,39) =
     +         QQMCHF2(ilon,1,4) + 2.0*QQMCHF2(ilon,1,5) +
     +         QQMCHF2(ilon,1,9) + QQMCHF2(ilon,1,13) +
     +         QQMCHF2(ilon,1,15) + QQMCHF2(ilon,1,16) +
     +         QQMCHF2(ilon,1,36)
          enddo
          print*,' TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM2
            QQMCHF2(ilon,1,40) =
     +         QQMCHF2(ilon,1,7) + QQMCHF2(ilon,1,8) +
     +         QQMCHF2(ilon,1,9) + QQMCHF2(ilon,1,10) +
     +     2.0*QQMCHF2(ilon,1,11) + QQMCHF2(ilon,1,13) +
     +     2.0*QQMCHF2(ilon,1,17) + QQMCHF2(ilon,1,19) +
     +         QQMCHF2(ilon,1,37)
          enddo
C
          BrLIM=20.9
          print*,'Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM2
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF2(ilon,1,41) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF2(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF2(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#ifdef RELACS
          print*,' TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM2
            QQMCHF2(ilon,1,62) =
     +         QQMCHF2(ilon,1,4) + 2.0*QQMCHF2(ilon,1,5) +
     +         QQMCHF2(ilon,1,9) + QQMCHF2(ilon,1,13) +
     +         QQMCHF2(ilon,1,15) + QQMCHF2(ilon,1,16) +
     +         QQMCHF2(ilon,1,35) + QQMCHF2(ilon,1,55) +
     +         QQMCHF2(ilon,1,59)
          enddo
          print*,' TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM2
            QQMCHF2(ilon,1,63) =
     +         QQMCHF2(ilon,1,7) + QQMCHF2(ilon,1,8) +
     +         QQMCHF2(ilon,1,9) + QQMCHF2(ilon,1,10) +
     +     2.0*QQMCHF2(ilon,1,11) + QQMCHF2(ilon,1,13) +
     +     2.0*QQMCHF2(ilon,1,17) + QQMCHF2(ilon,1,19) +
     +         QQMCHF2(ilon,1,60)
          enddo
C
          BrLIM=20.9
          print*,'TOP Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM2
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF2(ilon,1,64) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF2(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF2(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#ifdef RACMOBUS
          print*,'TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM2
            QQMCHF2(ilon,1,90) =
     +         QQMCHF2(ilon,1,4) + 2.0*QQMCHF2(ilon,1,5) +
     +         QQMCHF2(ilon,1,9) + QQMCHF2(ilon,1,13) +
     +         QQMCHF2(ilon,1,15) + QQMCHF2(ilon,1,16) +
     +         QQMCHF2(ilon,1,35) + QQMCHF2(ilon,1,60) +
     +         QQMCHF2(ilon,1,61) + QQMCHF2(ilon,1,62) +
     +         QQMCHF2(ilon,1,82) + QQMCHF2(ilon,1,83) +
     +         QQMCHF2(ilon,1,87)
          enddo
          print*,'TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM2
            QQMCHF2(ilon,1,91) =
     +         QQMCHF2(ilon,1,7) + QQMCHF2(ilon,1,8) +
     +         QQMCHF2(ilon,1,9) + QQMCHF2(ilon,1,10) +
     +     2.0*QQMCHF2(ilon,1,11) + QQMCHF2(ilon,1,13) +
     +     2.0*QQMCHF2(ilon,1,17) + QQMCHF2(ilon,1,19) +
     +         QQMCHF2(ilon,1,88)
          enddo
C
          BrLIM=20.9
          print*,'TOP Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM2
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF2(ilon,1,92) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF2(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF2(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#endif
      ENDIF
C
C REGIONAL DOMAIN
C
      IF (IFLAG3) THEN
        DO itra=1,NTRA3
          isp=0
          DO itrachf=1,NTRACHF
            IF (TRANAMCHF(itrachf).EQ.TRANAM3(itra)) isp=itrachf
          ENDDO
          IF (isp.NE.0) THEN
            DO ilat=1,LATM3
              QQMCHF3(1,ilat,itra) = 0.
              DO ilatchf=1,LATCHF-1
                IF ((RLATM3(ilat).GE.RLATCHF(ilatchf)).AND.
     +              (RLATM3(ilat).LE.RLATCHF(ilatchf+1))) THEN
                  YFACT=(RLATCHF(ilatchf+1)-RLATM3 (ilat   ))/
     +                  (RLATCHF(ilatchf+1)-RLATCHF(ilatchf))
                  QQMCHF3(1,ilat,itra) =
     +                      YFACT *QMCHF(ilatchf  ,isp) +
     +                  (1.-YFACT)*QMCHF(ilatchf+1,isp)
                ENDIF
              ENDDO
              DO ilon=1,LONM3
                QQMCHF3(ilon,ilat,itra)=MAX(1.E-30,
     +                                 QQMCHF3(1,ilat,itra))
              ENDDO
            ENDDO
          ELSE
            DO ilon=1,LONM3*LATM3
              QQMCHF3(ilon,1,itra)=1.E-30
            ENDDO
          ENDIF
        ENDDO
#ifdef MASSy
#ifdef REPROBUS
          print*,' TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM3
            QQMCHF3(ilon,1,39) =
     +         QQMCHF3(ilon,1,4) + 2.0*QQMCHF3(ilon,1,5) +
     +         QQMCHF3(ilon,1,9) + QQMCHF3(ilon,1,13) +
     +         QQMCHF3(ilon,1,15) + QQMCHF3(ilon,1,16) +
     +         QQMCHF3(ilon,1,36)
          enddo
          print*,' TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM3
            QQMCHF3(ilon,1,40) =
     +         QQMCHF3(ilon,1,7) + QQMCHF3(ilon,1,8) +
     +         QQMCHF3(ilon,1,9) + QQMCHF3(ilon,1,10) +
     +     2.0*QQMCHF3(ilon,1,11) + QQMCHF3(ilon,1,13) +
     +     2.0*QQMCHF3(ilon,1,17) + QQMCHF3(ilon,1,19) +
     +         QQMCHF3(ilon,1,37)
          enddo
C
          BrLIM=20.9
          print*,'Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM3
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF3(ilon,1,41) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF3(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF3(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#ifdef RELACS
          print*,' TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM3
            QQMCHF3(ilon,1,62) =
     +         QQMCHF3(ilon,1,4) + 2.0*QQMCHF3(ilon,1,5) +
     +         QQMCHF3(ilon,1,9) + QQMCHF3(ilon,1,13) +
     +         QQMCHF3(ilon,1,15) + QQMCHF3(ilon,1,16) +
     +         QQMCHF3(ilon,1,35) + QQMCHF3(ilon,1,55) +
     +         QQMCHF3(ilon,1,59)
          enddo
          print*,' TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM3
            QQMCHF3(ilon,1,63) =
     +         QQMCHF3(ilon,1,7) + QQMCHF3(ilon,1,8) +
     +         QQMCHF3(ilon,1,9) + QQMCHF3(ilon,1,10) +
     +     2.0*QQMCHF3(ilon,1,11) + QQMCHF3(ilon,1,13) +
     +     2.0*QQMCHF3(ilon,1,17) + QQMCHF3(ilon,1,19) +
     +         QQMCHF3(ilon,1,60)
          enddo
C
          BrLIM=20.9
          print*,'TOP Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM3
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF3(ilon,1,64) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF3(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF3(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#ifdef RACMOBUS
          print*,'TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM3
            QQMCHF3(ilon,1,90) =
     +         QQMCHF3(ilon,1,4) + 2.0*QQMCHF3(ilon,1,5) +
     +         QQMCHF3(ilon,1,9) + QQMCHF3(ilon,1,13) +
     +         QQMCHF3(ilon,1,15) + QQMCHF3(ilon,1,16) +
     +         QQMCHF3(ilon,1,35) + QQMCHF3(ilon,1,60) +
     +         QQMCHF3(ilon,1,61) + QQMCHF3(ilon,1,62) +
     +         QQMCHF3(ilon,1,82) + QQMCHF3(ilon,1,83) +
     +         QQMCHF3(ilon,1,87)
          enddo
          print*,'TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM3
            QQMCHF3(ilon,1,91) =
     +         QQMCHF3(ilon,1,7) + QQMCHF3(ilon,1,8) +
     +         QQMCHF3(ilon,1,9) + QQMCHF3(ilon,1,10) +
     +     2.0*QQMCHF3(ilon,1,11) + QQMCHF3(ilon,1,13) +
     +     2.0*QQMCHF3(ilon,1,17) + QQMCHF3(ilon,1,19) +
     +         QQMCHF3(ilon,1,88)
          enddo
C
          BrLIM=20.9
          print*,'TOP Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM3
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF3(ilon,1,92) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF3(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF3(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#endif


        
      ENDIF
C
C ZOOM DOMAIN
C
      IF (IFLAG4) THEN
        DO itra=1,NTRA4
          isp=0
          DO itrachf=1,NTRACHF
            IF (TRANAMCHF(itrachf).EQ.TRANAM4(itra)) isp=itrachf
          ENDDO
          IF (isp.NE.0) THEN
            DO ilat=1,LATM4
              QQMCHF4(1,ilat,itra) = 0.
              DO ilatchf=1,LATCHF-1
                IF ((RLATM4(ilat).GE.RLATCHF(ilatchf)).AND.
     +              (RLATM4(ilat).LE.RLATCHF(ilatchf+1))) THEN
                  YFACT=(RLATCHF(ilatchf+1)-RLATM4 (ilat   ))/
     +                  (RLATCHF(ilatchf+1)-RLATCHF(ilatchf))
                  QQMCHF4(1,ilat,itra) =
     +                      YFACT *QMCHF(ilatchf  ,isp) +
     +                  (1.-YFACT)*QMCHF(ilatchf+1,isp)
                ENDIF
              ENDDO
              DO ilon=1,LONM4
                QQMCHF4(ilon,ilat,itra)=MAX(1.E-30,
     +                                 QQMCHF4(1,ilat,itra))
              ENDDO
            ENDDO
          ELSE
            DO ilon=1,LONM4*LATM4
              QQMCHF4(ilon,1,itra)=1.E-30
            ENDDO
          ENDIF
        ENDDO
#ifdef MASSy
#ifdef REPROBUS
          print*,' TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM4
            QQMCHF4(ilon,1,39) =
     +         QQMCHF4(ilon,1,4) + 2.0*QQMCHF4(ilon,1,5) +
     +         QQMCHF4(ilon,1,9) + QQMCHF4(ilon,1,13) +
     +         QQMCHF4(ilon,1,15) + QQMCHF4(ilon,1,16) +
     +         QQMCHF4(ilon,1,36)
          enddo
          print*,' TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM4
            QQMCHF4(ilon,1,40) =
     +         QQMCHF4(ilon,1,7) + QQMCHF4(ilon,1,8) +
     +         QQMCHF4(ilon,1,9) + QQMCHF4(ilon,1,10) +
     +     2.0*QQMCHF4(ilon,1,11) + QQMCHF4(ilon,1,13) +
     +     2.0*QQMCHF4(ilon,1,17) + QQMCHF4(ilon,1,19) +
     +         QQMCHF4(ilon,1,37)
          enddo
C
          BrLIM=20.9
          print*,'Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM4
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF4(ilon,1,41) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF4(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF4(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#ifdef RELACS
          print*,' TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM4
            QQMCHF4(ilon,1,62) =
     +         QQMCHF4(ilon,1,4) + 2.0*QQMCHF4(ilon,1,5) +
     +         QQMCHF4(ilon,1,9) + QQMCHF4(ilon,1,13) +
     +         QQMCHF4(ilon,1,15) + QQMCHF4(ilon,1,16) +
     +         QQMCHF4(ilon,1,35) + QQMCHF4(ilon,1,55) +
     +         QQMCHF4(ilon,1,59)
          enddo
          print*,' TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM4
            QQMCHF4(ilon,1,63) =
     +         QQMCHF4(ilon,1,7) + QQMCHF4(ilon,1,8) +
     +         QQMCHF4(ilon,1,9) + QQMCHF4(ilon,1,10) +
     +     2.0*QQMCHF4(ilon,1,11) + QQMCHF4(ilon,1,13) +
     +     2.0*QQMCHF4(ilon,1,17) + QQMCHF4(ilon,1,19) +
     +         QQMCHF4(ilon,1,60)
          enddo
C
          BrLIM=20.9
          print*,'TOP Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM4
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF4(ilon,1,64) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF4(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF4(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#ifdef RACMOBUS
          print*,'TOP NOy initialised by sum of N species'
          do ilon=1,LOLAM4
            QQMCHF4(ilon,1,90) =
     +         QQMCHF4(ilon,1,4) + 2.0*QQMCHF4(ilon,1,5) +
     +         QQMCHF4(ilon,1,9) + QQMCHF4(ilon,1,13) +
     +         QQMCHF4(ilon,1,15) + QQMCHF4(ilon,1,16) +
     +         QQMCHF4(ilon,1,35) + QQMCHF4(ilon,1,60) +
     +         QQMCHF4(ilon,1,61) + QQMCHF4(ilon,1,62) +
     +         QQMCHF4(ilon,1,82) + QQMCHF4(ilon,1,83) +
     +         QQMCHF4(ilon,1,87)
          enddo
          print*,'TOP Cly initialised by sum of Cl species'
          do ilon=1,LOLAM4
            QQMCHF4(ilon,1,91) =
     +         QQMCHF4(ilon,1,7) + QQMCHF4(ilon,1,8) +
     +         QQMCHF4(ilon,1,9) + QQMCHF4(ilon,1,10) +
     +     2.0*QQMCHF4(ilon,1,11) + QQMCHF4(ilon,1,13) +
     +     2.0*QQMCHF4(ilon,1,17) + QQMCHF4(ilon,1,19) +
     +         QQMCHF4(ilon,1,88)
          enddo
C
          BrLIM=20.9
          print*,'TOP Bry initialised using CFC11 profile'
C
          do ilon=1,LOLAM4
C
C Bry is calculated using empirical formula based upon CFC11 profile
C
            QQMCHF4(ilon,1,92) =
     +      ( BrLIM - ( 4.36 E-02*QQMCHF4(ilon,1,23)*1. E+12 ) -
     +    1.1 E-04*(QQMCHF4(ilon,1,23)*1. E+12)**2.0 )*1. E-12
          enddo
#endif
#endif



      
      ENDIF
C
      ENDIF
      IF (IFLAG1) THEN
       DO JC=1,NTRA1
        DO ILON=1,LONM1
         DO ILAT=1,LATM1
          QQQM1(ILON,ILAT,1,JC)=QQMCHF1(ILON,ILAT,JC)
         ENDDO
        ENDDO
       ENDDO
      ENDIF
      IF (IFLAG2) THEN
       DO JC=1,NTRA2
        DO ILON=1,LONM2
         DO ILAT=1,LATM2
          QQQM2(ILON,ILAT,1,JC)=QQMCHF2(ILON,ILAT,JC)
         ENDDO
        ENDDO
       ENDDO
      ENDIF
      IF (IFLAG3) THEN
       DO JC=1,NTRA3
        DO ILON=1,LONM3
         DO ILAT=1,LATM3
          QQQM3(ILON,ILAT,1,JC)=QQMCHF3(ILON,ILAT,JC)
         ENDDO
        ENDDO
       ENDDO
      ENDIF
      IF (IFLAG4) THEN
       DO JC=1,NTRA4
        DO ILON=1,LONM4
         DO ILAT=1,LATM4
          QQQM4(ILON,ILAT,1,JC)=QQMCHF4(ILON,ILAT,JC)
         ENDDO
        ENDDO
       ENDDO
      ENDIF
C
      DEALLOCATE (RLATCHF)
      DEALLOCATE (QMCHF)
      DEALLOCATE (TRANAMCHF)
C
      RETURN
      END
C
CCC
C
