      SUBROUTINE INIVIT
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1
C
CCC DATE : 15/05/2005
C
C   Type      : 3D semi-lagrangian transport scheme
C   Purpose   : Initialize velocities in auxialiary grid
C   Called by :
C
CCC
C
C  Original : P. Simon, 03/1999
C
C  Modified : V.-H. Peuch (inclusion dans MOCAGE), 04/1999
C  Modified : H. LE BERRE 07/2006  Securisation pour eviter la division par 0
C             Calcul de UMODF et VMODF uniquement si abs(latitude) > latitude min du repre auxilliaire
C             Sinon pour la latitude 0 on prend la moyenne des points de latitude en dessous et au dessus
C             Pour l'optimisation on verra plus tard , idem pour les domaines imbriqus.
C
CCC
C
	USE PARADI_MOD
	USE EXPE_MOD
	USE MOCF_MOD
	USE GRIDS_MOD
	USE FORFT_MOD
	USE CSTES_MOD
C
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C	
C
	INTEGER :: L,K,J		
C
C  composantes horizontales de la vitesse dans le repere auxiliaire
C  NB: la composante verticale est la meme dans les deux reperes
C
C
C  1. Global domain
C
      IF(IFLAG1) THEN
C
        DO L=1,NIVF1
        DO K=1,LATF1
          IF (ABS(RLATF1(K)).GE.1.E-2) THEN
            DO J=1,LONF1
             VMODF1(J,K,L)=(UGRIF1(J,K,L)*SLON1(J)
     +                     +VGRIF1(J,K,L)*CLON1(J)*SLAT1(K))
     +                    /CLAT1R(J,K)
             UMODF1(J,K,L)=(UGRIF1(J,K,L)*CLON1(J)
     +                     -VGRIF1(J,K,L)*SLON1(J)*SLAT1(K)
     +                     +VMODF1(J,K,L)*SLON1R(J,K)*SLAT1R(J,K))
     +                    /CLON1R(J,K)
            ENDDO
          ELSE
            DO J=1,LONF1
             VMODF1(J,K,L)=0.5*(
     +                     (UGRIF1(J,K-1,L)*SLON1(J)
     +                     +VGRIF1(J,K-1,L)*CLON1(J)*SLAT1(K-1))
     +                     /CLAT1R(J,K-1)
     +                    +(UGRIF1(J,K+1,L)*SLON1(J)
     +                     +VGRIF1(J,K+1,L)*CLON1(J)*SLAT1(K+1))
     +                     /CLAT1R(J,K+1) )
             UMODF1(J,K,L)=0.5*(
     +                     (UGRIF1(J,K-1,L)*CLON1(J)
     +                     -VGRIF1(J,K-1,L)*SLON1(J)*SLAT1(K-1)
     +                     +VMODF1(J,K-1,L)*SLON1R(J,K-1)*SLAT1R(J,K-1))
     +                     /CLON1R(J,K-1)
     +                    +(UGRIF1(J,K+1,L)*CLON1(J)
     +                     -VGRIF1(J,K+1,L)*SLON1(J)*SLAT1(K+1)
     +                     +VMODF1(J,K+1,L)*SLON1R(J,K+1)*SLAT1R(J,K+1))
     +                     /CLON1R(J,K+1) )
            ENDDO
          ENDIF
        ENDDO
        ENDDO
C
      ENDIF
C
C  2. Continental domain
C
      IF(IFLAG2) THEN
C
        DO L=1,NIVF2
        DO K=1,LATF2
            DO J=1,LONF2
              VMODF2(J,K,L)=(UGRIF2(J,K,L)*SLON2(J)
     +                      +VGRIF2(J,K,L)*CLON2(J)*SLAT2(K))
     +                     /CLAT2R(J,K)
              UMODF2(J,K,L)=(UGRIF2(J,K,L)*CLON2(J)
     +                      -VGRIF2(J,K,L)*SLON2(J)*SLAT2(K)
     +                      +VMODF2(J,K,L)*SLON2R(J,K)*SLAT2R(J,K))
     +                     /CLON2R(J,K)
           ENDDO
        ENDDO
        ENDDO
C
      ENDIF
C
C  3. Regional domain
C
      IF(IFLAG3) THEN
C
        DO L=1,NIVF3
        DO K=1,LATF3
            DO J=1,LONF3
              VMODF3(J,K,L)=(UGRIF3(J,K,L)*SLON3(J)
     +                      +VGRIF3(J,K,L)*CLON3(J)*SLAT3(K))
     +                  /CLAT3R(J,K)
              UMODF3(J,K,L)=(UGRIF3(J,K,L)*CLON3(J)
     +                      -VGRIF3(J,K,L)*SLON3(J)*SLAT3(K)
     +                      +VMODF3(J,K,L)*SLON3R(J,K)*SLAT3R(J,K))
     +                     /CLON3R(J,K)
            ENDDO
        ENDDO
        ENDDO
C
      ENDIF
C
C  4. Zoom domain
C
      IF(IFLAG4) THEN
C
        DO L=1,NIVF4
        DO K=1,LATF4
            DO J=1,LONF4
              VMODF4(J,K,L)=(UGRIF4(J,K,L)*SLON4(J)
     +                      +VGRIF4(J,K,L)*CLON4(J)*SLAT4(K))
     +                     /CLAT4R(J,K)
              UMODF4(J,K,L)=(UGRIF4(J,K,L)*CLON4(J)
     +                      -VGRIF4(J,K,L)*SLON4(J)*SLAT4(K)
     +                      +VMODF4(J,K,L)*SLON4R(J,K)*SLAT4R(J,K))
     +                     /CLON4R(J,K)
            ENDDO
        ENDDO
        ENDDO
C
      ENDIF
C
      RETURN
      END
