      SUBROUTINE INSOURCE
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 23/02/2006
C
C   Type       : Point source release model version
C   Purpose    : Read-in namelist for point source release
C   Called by  : MAIN
C
CCC
C
C  Original : V.-H. Peuch, D. Lacroix (02/2006)
C
C  Last Modified :
C
CCC
C
      USE PARADI_MOD
      USE MOCF_MOD
      USE SRC_MOD
      USE SRCFLX_MOD
      USE HENRY_MOD
C
C LOCAL VARIABLES
C
      INTEGER isrc
      INTEGER ilon,ilat,iniv,itra
      INTEGER CHARAC1,CHARAC2
C
C 0. INITIALIZE LOGICAL UNITS
C
      IFSORACC1=61
      IFSORACC2=62
      IFSORACC3=63
      IFSORACC4=64
      IFINACC=65
C
C 1. READ-IN SPECIFIC NAMELIST
C
      OPEN(IFINACC,FILE='psrc.dat',FORM='FORMATTED')
      READ (IFINACC,*) NSRC
      NSRC=MIN(NSRC,NTRA1)
C
C MAX. NUMBER OF SOURCES IS NTRA1=NTRA2=NTRA3=NTRA4
C
      DO isrc = 1,NSRC
        READ (IFINACC,*) POLNAME(isrc),
     +              PSOURCE(isrc,1),
     +              PSOURCE(isrc,2),
     +              PSOURCE(isrc,5),
     +              PSOURCE(isrc,4),
     +              PSOURCE(isrc,3),
     +              EMIUNIT(isrc),
     +              CHARAC1,
     +              CHARAC2
        READ (CHARAC1,'(I4,3I2)')(IDATEMI1(isrc,ii),ii=1,4)
        READ (CHARAC2,'(I4,3I2)')(IDATEMI2(isrc,ii),ii=1,4)
        IDATEMI1(isrc,5)=0
        IDATEMI2(isrc,5)=0
      ENDDO
      CLOSE(IFINACC)
C
      PRINT*,">> MOCAGE-ACCIDENT : EXPERIMENT DEFINITION"
      DO isrc = 1,NSRC
        WRITE (*,*) isrc,POLNAME(isrc),
     +              PSOURCE(isrc,1),
     +              PSOURCE(isrc,2),
     +              PSOURCE(isrc,5),
     +              PSOURCE(isrc,4),
     +              PSOURCE(isrc,3),
     +              EMIUNIT(isrc),
     +              CHARAC1,
     +              CHARAC2
      ENDDO 
C
C 2. INITIALIZE TRACER HALF-LIFE, DEP. VELOCITY, HENRY LAW CONSTANTS
C
      PRINT*,">> MOCAGE-ACCIDENT : INITIALIZATION"
      DO isrc=1,NSRC
C
C    a)  CESIUM 137
C
        IF (POLNAME(isrc).EQ.'CESIUM') THEN
          PRINT*," > TRACER #",isrc
          PRINT*,"   (Cs) HAS A HALF-LIFE OF 33 YEARS" 
          HALFLIFE(isrc) = 33.*365.25*24.*3600.
          PRINT*,"   (Cs) HAS A DEP. VELOCITY OF 1 mm/s" 
          VDEPOSIT(isrc) = 0.001
          PRINT*,"   (Cs) HAS A MOLECULAR WEIGHT OF 132.9g"          
          MOLWEIGHT(isrc)  = 132.9045
          PRINT*,"   (Cs) IS NOT SOLUBLE"
C
C VHP : A VERIFIER!
C
          IF (IFLAG1) THEN
            XKHENRY1(isrc,1) = 0.
            XKHENRY1(isrc,2) = 0.
          ENDIF
          IF (IFLAG2) THEN
            XKHENRY2(isrc,1) = 0.
            XKHENRY2(isrc,2) = 0.
          ENDIF
          IF (IFLAG3) THEN
            XKHENRY3(isrc,1) = 0.
            XKHENRY3(isrc,2) = 0.
          ENDIF
          IF (IFLAG4) THEN
            XKHENRY4(isrc,1) = 0.
            XKHENRY4(isrc,2) = 0.
          ENDIF
C
C    b)  IODINE 131
C 
        ELSEIF (POLNAME(isrc).EQ.'IODINE') THEN
          PRINT*," > TRACER #",isrc
          PRINT*,"   (I) HAS A HALF-LIFE OF 8 DAYS"
          HALFLIFE(isrc) = 8.*24.*3600.
          PRINT*,"   (I) HAS A DEP. VELOCITY OF 10 mm/s" 
          VDEPOSIT(isrc) = 0.01          
          PRINT*,"   (I) HAS A MOLECULAR WEIGHT OF 253.81g"          
          MOLWEIGHT(isrc)  = 253.81
          PRINT*,"   (I) IS SOLUBLE"
C
C HENRY CONSTANTS FROM http://www.mpch-mainz.mpg.de/~sander/res/henry.html
C
          IF (IFLAG1) THEN
            XKHENRY1(isrc,1) = 3.3
            XKHENRY1(isrc,2) = -4800.
          ENDIF
          IF (IFLAG2) THEN
            XKHENRY2(isrc,1) = 3.3
            XKHENRY2(isrc,2) = -4800.
          ENDIF
          IF (IFLAG3) THEN
            XKHENRY3(isrc,1) = 3.3
            XKHENRY3(isrc,2) = -4800.
          ENDIF
          IF (IFLAG4) THEN
            XKHENRY4(isrc,1) = 3.3
            XKHENRY4(isrc,2) = -4800.
          ENDIF          
C
C    c) PMCH (PERFLUORO-METHYL-CYCLO-HEXANE)
C
        ELSEIF (POLNAME(isrc).EQ.'PMCH') THEN
          PRINT*," > TRACER #",isrc
          PRINT*,"   (PMCH) IS PASSIVE"        
          HALFLIFE(isrc)   = 100.*365.25*24.*3600.
          PRINT*,"   (PMCH) IS NOT DEPOSITED" 
          VDEPOSIT(isrc)   = 0.
          PRINT*,"   (PMCH) HAS A MOLECULAR WEIGHT OF 350g"          
          MOLWEIGHT(isrc)  = 350.
          PRINT*,"   (PMCH) IS NOT SOLUBLE"
          IF (IFLAG1) THEN
            XKHENRY1(isrc,1) = 0.
            XKHENRY1(isrc,2) = 0.
          ENDIF
          IF (IFLAG2) THEN
            XKHENRY2(isrc,1) = 0.
            XKHENRY2(isrc,2) = 0.
          ENDIF
          IF (IFLAG3) THEN
            XKHENRY3(isrc,1) = 0.
            XKHENRY3(isrc,2) = 0.
          ENDIF
          IF (IFLAG4) THEN
            XKHENRY4(isrc,1) = 0.
            XKHENRY4(isrc,2) = 0.
          ENDIF
C
C    d) PASSIVE TRACER ; NO DEPOSITION ; NO KNOWN WEIGHT ; NOT SOLUBLE
C
        ELSEIF (POLNAME(isrc).EQ.'PASSIVE') THEN
          PRINT*," > TRACER #",isrc,
     +      " IS PASSIVE AND NOT SOLUBLE"
          HALFLIFE(isrc)  = 100.*365.25*24.*3600.
          VDEPOSIT(isrc)  = 0.
          MOLWEIGHT(isrc) = 0.
          IF (IFLAG1) THEN
            XKHENRY1(isrc,1) = 0.
            XKHENRY1(isrc,2) = 0.
          ENDIF
          IF (IFLAG2) THEN
            XKHENRY2(isrc,1) = 0.
            XKHENRY2(isrc,2) = 0.
          ENDIF
          IF (IFLAG3) THEN
            XKHENRY3(isrc,1) = 0.
            XKHENRY3(isrc,2) = 0.
          ENDIF
          IF (IFLAG4) THEN
            XKHENRY4(isrc,1) = 0.
            XKHENRY4(isrc,2) = 0.
          ENDIF          
        ELSE
C
C    e) UKNOWN TRACER ; NO EMISSION ; NO DEPOSITION ; NO KNOWN WEIGHT
C
          WRITE(*,*) '>> WARNING : ',POLNAME(isrc),
     +               ' IS NOT A KNOWN POLLUTANT !'
          WRITE(*,*) '>  NOTHING WILL BE DONE...'
          HALFLIFE(isrc)  = 100.*365.25*24.*3600.
          PSOURCE(isrc,3) = 0.
          VDEPOSIT(isrc)  = 0.
          MOLWEIGHT(isrc) = 0.     
          IF (IFLAG1) THEN
            XKHENRY1(isrc,1) = 0.
            XKHENRY1(isrc,2) = 0.
          ENDIF
          IF (IFLAG2) THEN
            XKHENRY2(isrc,1) = 0.
            XKHENRY2(isrc,2) = 0.
          ENDIF
          IF (IFLAG3) THEN
            XKHENRY3(isrc,1) = 0.
            XKHENRY3(isrc,2) = 0.
          ENDIF
          IF (IFLAG4) THEN
            XKHENRY4(isrc,1) = 0.
            XKHENRY4(isrc,2) = 0.
          ENDIF
        ENDIF
      ENDDO
C
C 3. OPEN OUTPUT FILE(s)
C
      IF (IFLAG1) THEN
        OPEN(IFSORACC1,file='RESGLOB22',form='UNFORMATTED')
        WRITE(IFSORACC1) NSRC
      ENDIF
      IF (IFLAG2) THEN
        OPEN(IFSORACC2,file='RESEURAT5',form='UNFORMATTED')
        WRITE(IFSORACC2) NSRC
      ENDIF
      IF (IFLAG3) THEN
        OPEN(IFSORACC3,file='RESFRA025',form='UNFORMATTED')
        WRITE(IFSORACC3) NSRC
      ENDIF
      IF (IFLAG4) THEN
        OPEN(IFSORACC4,file='RESDUMMY0',form='UNFORMATTED')
        WRITE(IFSORACC4) NSRC
      ENDIF
C
C 4. INITIALIZE FLUXES
C
      IF (IFLAG1) THEN
        DO iniv=1,NIVM1
          DO ilat=1,LATM1
	    DO ilon=1,LONM1
	      PCVRCUMUL1(ilon,ilat,iniv)=0.
	      PCVSCUMUL1(ilon,ilat,iniv)=0.
	      PSTRCUMUL1(ilon,ilat,iniv)=0.
	    ENDDO
	  ENDDO
	ENDDO
        DO itra=1,NTRA1
	  DO ilat=1,LATM1
	    DO ilon=1,LONM1
	      DFBCVCUMUL1(ilon,ilat,itra)=0.
	      DFICVCUMUL1(ilon,ilat,itra)=0.
	      DFSTRCUMUL1(ilon,ilat,itra)=0.
	      DFDRYCUMUL1(ilon,ilat,itra)=0.
	    ENDDO
	  ENDDO
	ENDDO
      ENDIF
C
      IF (IFLAG2) THEN
        DO iniv=1,NIVM2
          DO ilat=1,LATM2
	    DO ilon=1,LONM2
	      PCVRCUMUL2(ilon,ilat,iniv)=0.
	      PCVSCUMUL2(ilon,ilat,iniv)=0.
	      PSTRCUMUL2(ilon,ilat,iniv)=0.
	    ENDDO
	  ENDDO
	ENDDO
        DO itra=1,NTRA2
	  DO ilat=1,LATM2
	    DO ilon=1,LONM2
	      DFBCVCUMUL2(ilon,ilat,itra)=0.
	      DFICVCUMUL2(ilon,ilat,itra)=0.
	      DFSTRCUMUL2(ilon,ilat,itra)=0.
	      DFDRYCUMUL2(ilon,ilat,itra)=0.
	    ENDDO
	  ENDDO
	ENDDO
      ENDIF
C
      IF (IFLAG3) THEN
        DO iniv=1,NIVM3
          DO ilat=1,LATM3
	    DO ilon=1,LONM3
	      PCVRCUMUL3(ilon,ilat,iniv)=0.
	      PCVSCUMUL3(ilon,ilat,iniv)=0.
	      PSTRCUMUL3(ilon,ilat,iniv)=0.
	    ENDDO
	  ENDDO
	ENDDO
        DO itra=1,NTRA3
	  DO ilat=1,LATM3
	    DO ilon=1,LONM3
	      DFBCVCUMUL3(ilon,ilat,itra)=0.
	      DFICVCUMUL3(ilon,ilat,itra)=0.
	      DFSTRCUMUL3(ilon,ilat,itra)=0.
	      DFDRYCUMUL3(ilon,ilat,itra)=0.
	    ENDDO
	  ENDDO
	ENDDO
      ENDIF
C	      	 
      IF (IFLAG4) THEN
        DO iniv=1,NIVM4
          DO ilat=1,LATM4
	    DO ilon=1,LONM4
	      PCVRCUMUL4(ilon,ilat,iniv)=0.
	      PCVSCUMUL4(ilon,ilat,iniv)=0.
	      PSTRCUMUL4(ilon,ilat,iniv)=0.
	    ENDDO
	  ENDDO
	ENDDO
        DO itra=1,NTRA4
	  DO ilat=1,LATM4
	    DO ilon=1,LONM4
	      DFBCVCUMUL4(ilon,ilat,itra)=0.
	      DFICVCUMUL4(ilon,ilat,itra)=0.
	      DFSTRCUMUL4(ilon,ilat,itra)=0.
	      DFDRYCUMUL4(ilon,ilat,itra)=0.
	    ENDDO
	  ENDDO
	ENDDO
      ENDIF
C
      RETURN
      END
C
CCC
C
