      SUBROUTINE INTER2D(XINP,LON,LAT
     +                  ,INDX,INDY
     +                  ,ALFX1,ALFX2,ALFX3,ALFX4
     +                  ,ALFY1,ALFY2,ALFY3,ALFY4
     +                  ,XOUT,NDIM,RSYM,LIMIT,IGR)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : 3D semi-lagrangian transport scheme
C   Purpose   : Cubic interpolation in 2D
C   Called by : INITRANS
C
CCC
C
C  Original : P. Simon, 03/1999
C
C  Modified : V.-H. Peuch (inclusion dans MOCAGE), 04/1999
C             P. Simon    (suppression des tests), 08/1999
C
CCC
C
	IMPLICIT NONE
C
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: LON,LAT,NDIM,IGR
	REAL (KIND=8) :: RSYM
	LOGICAL :: LIMIT
	INTEGER , DIMENSION (NDIM) :: INDX,INDY
	REAL (KIND=8) , DIMENSION (NDIM)    :: ALFX1,ALFX2,ALFX3,ALFX4
	REAL (KIND=8) , DIMENSION (NDIM)    :: ALFY1,ALFY2,ALFY3,ALFY4
	REAL (KIND=8) , DIMENSION (NDIM) :: XOUT
	REAL (KIND=8) , DIMENSION (LON,LAT) :: XINP
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: J,J1,J2,J3,J4
	INTEGER :: K,K1,K2,K3,K4
	INTEGER :: L,L1,L2,L3,L4
	INTEGER :: N
	REAL (KIND=8) :: X11,X12,X13,X14
	REAL (KIND=8) :: X21,X22,X23,X24
	REAL (KIND=8) :: X31,X32,X33,X34
	REAL (KIND=8) :: X41,X42,X43,X44
	REAL (KIND=8) :: R2MX,R2MN,R3MX,R3MN
	REAL (KIND=8) , ALLOCATABLE :: X(:,:),Y(:,:)
	REAL (KIND=8) , ALLOCATABLE :: RMX(:),RMN(:)
C
C
	ALLOCATE (X(-1:LON+2,-1:LAT+2)) ; X(:,:) = 0.0
	ALLOCATE (Y(NDIM,4))
        IF(LIMIT) THEN
            ALLOCATE (RMX(NDIM))
            ALLOCATE (RMN(NDIM))
        ENDIF
C
C  1. extension de la grille pour le champ a interpoler
C
      DO 11 K=1,LAT
      DO 11 J=1,LON
         X(J,K)=XINP(J,K)
 11   CONTINUE
      CALL EXT2D(X,LON,LAT,RSYM,IGR)
C
C  2. interpolation selon x
C
      DO 2 N=1,NDIM
C	  
         J=INDX(N)
         K=INDY(N)
C
         J1=J-1
         J2=J
         J3=J+1
         J4=J+2
C
         K1=K-1
         K2=K
         K3=K+1
         K4=K+2
C
         X11=X(J1,K1)
         X21=X(J2,K1)
         X31=X(J3,K1)
         X41=X(J4,K1)
C
         X12=X(J1,K2)
         X22=X(J2,K2)
         X32=X(J3,K2)
         X42=X(J4,K2)
C
         X13=X(J1,K3)
         X23=X(J2,K3)
         X33=X(J3,K3)
         X43=X(J4,K3)
C
         X14=X(J1,K4)
         X24=X(J2,K4)
         X34=X(J3,K4)
         X44=X(J4,K4)
C
         IF(LIMIT) THEN
C
           R2MX=AMAX1(X22,X32)
           R2MN=AMIN1(X22,X32)
C
           R3MX=AMAX1(X23,X33)
           R3MN=AMIN1(X23,X33)
C
           RMX(N)=AMAX1(R2MX,R3MX)
           RMN(N)=AMIN1(R2MN,R3MN)
C
         ENDIF
C
C  interpolation proprement dite
C
         Y(N,1)=ALFX1(N)*X11+ALFX2(N)*X21+ALFX3(N)*X31+ALFX4(N)*X41
         Y(N,2)=ALFX1(N)*X12+ALFX2(N)*X22+ALFX3(N)*X32+ALFX4(N)*X42
         Y(N,3)=ALFX1(N)*X13+ALFX2(N)*X23+ALFX3(N)*X33+ALFX4(N)*X43
         Y(N,4)=ALFX1(N)*X14+ALFX2(N)*X24+ALFX3(N)*X34+ALFX4(N)*X44
C
 2    CONTINUE
C
C  interpolation selon y
C
      DO 3 N=1,NDIM
         XOUT(N)=ALFY1(N)*Y(N,1)+ALFY2(N)*Y(N,2)
     +          +ALFY3(N)*Y(N,3)+ALFY4(N)*Y(N,4)
 3    CONTINUE
C
C  limitation pour assurer la monotonie
C
      IF(LIMIT) THEN
        DO 4 N=1,NDIM
           XOUT(N)=AMAX1(AMIN1(XOUT(N),RMX(N)),RMN(N))
 4      CONTINUE
      ENDIF
C
	DEALLOCATE (X)
	DEALLOCATE (Y)
        IF(LIMIT) THEN
            DEALLOCATE (RMX)
            DEALLOCATE (RMN)
        ENDIF
C
C
      RETURN
      END
