      SUBROUTINE INTER3D(XINP,XUP,LON,LAT,NIV        &
                        ,INDX,INDY,INDZ              &
                        ,ALFX1,ALFX2,ALFX3,ALFX4     &
                        ,ALFY1,ALFY2,ALFY3,ALFY4     &
                        ,ALFZ1,ALFZ2,ALFZ3,ALFZ4     &
                        ,XOUT,NDIM,RSYM,LIMIT,IGR,LCLEUP)


!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!CC DATE : 15/05/2005
!
!   Type       : 3D semi-lagrangian transport scheme
!   Purpose    : 3D interpolations
!   Called by  :



!  Original : P. Simon, 03/1999

!  Modified : V.-H. Peuch (inclusion dans MOCAGE), 04/1999
!             P. Simon    (suppression des tests), 08/1999



	IMPLICIT NONE


!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~

	INTEGER :: LON,LAT,NIV,NDIM,IGR
	INTEGER , DIMENSION (NDIM) :: INDX,INDY,INDZ
	REAL (KIND=8) :: RSYM
	LOGICAL :: LIMIT,LCLEUP
	REAL (KIND=8) , DIMENSION (LON,LAT,NIV), INTENT(IN) :: XINP
	REAL (KIND=8) , DIMENSION (LON,LAT), INTENT(IN) :: XUP
	REAL (KIND=8) , DIMENSION (NDIM) :: ALFX1,ALFX2,ALFX3,ALFX4
	REAL (KIND=8) , DIMENSION (NDIM) :: ALFY1,ALFY2,ALFY3,ALFY4
	REAL (KIND=8) , DIMENSION (NDIM) :: ALFZ1,ALFZ2,ALFZ3,ALFZ4	
	REAL (KIND=8) , DIMENSION (NDIM), INTENT(OUT) :: XOUT

!     Variables locales.
!     ~~~~~~~~~~~~~~~~~

	INTEGER :: J,J1,J2,J3,J4
	INTEGER :: K,K1,K2,K3,K4
	INTEGER :: L,L1,L2,L3,L4
	INTEGER :: N
	REAL (KIND=8) :: X111,X211,X311,X411
	REAL (KIND=8) :: x121,X221,X321,X421
	REAL (KIND=8) :: X131,X231,X331,X431
	REAL (KIND=8) :: X141,X241,X341,X441	
	REAL (KIND=8) :: X112,X212,X312,X412
	REAL (KIND=8) :: x122,X222,X322,X422
	REAL (KIND=8) :: X132,X232,X332,X432
	REAL (KIND=8) :: X142,X242,X342,X442	
	REAL (KIND=8) :: X113,X213,X313,X413
	REAL (KIND=8) :: x123,X223,X323,X423
	REAL (KIND=8) :: X133,X233,X333,X433
	REAL (KIND=8) :: X143,X243,X343,X443	
	REAL (KIND=8) :: X114,X214,X314,X414
	REAL (KIND=8) :: x124,X224,X324,X424
	REAL (KIND=8) :: X134,X234,X334,X434
	REAL (KIND=8) :: X144,X244,X344,X444
	REAL (KIND=8) :: Y11,Y12,Y13,Y14,Z1
	REAL (KIND=8) :: Y21,Y22,Y23,Y24,Z2
	REAL (KIND=8) :: Y31,Y32,Y33,Y34,Z3
	REAL (KIND=8) :: Y41,Y42,Y43,Y44,Z4
	REAL (KIND=8) :: R22MX,R22MN,R32MX,R32MN
	REAL (KIND=8) :: R23MX,R23MN,R33MX,R33MN
	REAL (KIND=8) :: R2MX,R2MN,R3MX,R3MN,RMX,RMN
	REAL (KIND=8) , ALLOCATABLE :: X(:,:,:)


      ALLOCATE (X(-1:LON+2,-1:LAT+2,0:NIV)) !!! quid des bornes ??; X(:,:,:) = 0.0

      !  1. extension de la grille pour le champ a interpoler

!$omp parallel default(none) private(J,K,L) shared(LCLEUP,LON,LAT,NIV,XINP,XUP,X)

      IF (LCLEUP) THEN
!$omp do
         DO K=1,LAT
         DO J=1,LON
           X(J,K,0)=XUP(J,K)
           X(J,K,1)=XUP(J,K)
         ENDDO
         ENDDO
!$omp end do
      ELSE
!$omp do
         DO K=1,LAT
         DO J=1,LON
           X(J,K,0)=XUP(J,K)
           X(J,K,1)=XINP(J,K,1)
         ENDDO
         ENDDO
!$omp end do
      ENDIF
     
!$omp do
      DO 1 L=2,NIV
      DO 1 K=1,LAT
      DO 1 J=1,LON
         X(J,K,L)=XINP(J,K,L)
 1    CONTINUE
!$omp end do

!$omp end parallel
      
      CALL EXT3D(X,LON,LAT,NIV,RSYM,IGR)

      !  2. interpolation selon x

!$omp parallel do &
!$omp default(none) &
!$omp private(N,J,J1,J2,J3,J4,K,K1,K2,K3,K4,L,L1,L2,L3,L4) &
!$omp private(X111,X211,X311,X411,X121,X221,X321,X421,X131,X231,X331,X431,X141,X241,X341,X441) &
!$omp private(X112,X212,X312,X412,X122,X222,X322,X422,X132,X232,X332,X432,X142,X242,X342,X442) &
!$omp private(X113,X213,X313,X413,X123,X223,X323,X423,X133,X233,X333,X433,X143,X243,X343,X443) &
!$omp private(X114,X214,X314,X414,X124,X224,X324,X424,X134,X234,X334,X434,X144,X244,X344,X444) &
!$omp private(Y11,Y21,Y31,Y41,Y12,Y22,Y32,Y42,Y13,Y23,Y33,Y43,Y14,Y24,Y34,Y44,Z1,Z2,Z3,Z4) &
!$omp private(R22MX,R22MN,R32MX,R32MN,R23MX,R23MN,R33MX,R33MN,R2MX,R2MN,R3MX,R3MN,RMX,RMN) &
!$omp shared(LIMIT,NDIM,INDX,INDY,INDZ,NIV,X,XOUT) &
!$omp shared(ALFX1,ALFX2,ALFX3,ALFX4,ALFY1,ALFY2,ALFY3,ALFY4,ALFZ1,ALFZ2,ALFZ3,ALFZ4)
      DO N=1,NDIM

	 J=INDX(N)
         K=INDY(N)
         L=INDZ(N)

         J1=J-1
         J2=J
         J3=J+1
         J4=J+2

         K1=K-1
         K2=K
         K3=K+1
         K4=K+2

         L1=MAX(L-1,0)
         L2=MAX(L  ,0)
         L3=MIN(L+1,NIV)
         L4=MIN(L+2,NIV)

         X111=X(J1,K1,L1)
         X211=X(J2,K1,L1)
         X311=X(J3,K1,L1)
         X411=X(J4,K1,L1)

         X121=X(J1,K2,L1)
         X221=X(J2,K2,L1)
         X321=X(J3,K2,L1)
         X421=X(J4,K2,L1)

         X131=X(J1,K3,L1)
         X231=X(J2,K3,L1)
         X331=X(J3,K3,L1)
         X431=X(J4,K3,L1)

         X141=X(J1,K4,L1)
         X241=X(J2,K4,L1)
         X341=X(J3,K4,L1)
         X441=X(J4,K4,L1)

         X112=X(J1,K1,L2)
         X212=X(J2,K1,L2)
         X312=X(J3,K1,L2)
         X412=X(J4,K1,L2)

         X122=X(J1,K2,L2)
         X222=X(J2,K2,L2)
         X322=X(J3,K2,L2)
         X422=X(J4,K2,L2)

         X132=X(J1,K3,L2)
         X232=X(J2,K3,L2)
         X332=X(J3,K3,L2)
         X432=X(J4,K3,L2)

         X142=X(J1,K4,L2)
         X242=X(J2,K4,L2)
         X342=X(J3,K4,L2)
         X442=X(J4,K4,L2)

         X113=X(J1,K1,L3)
         X213=X(J2,K1,L3)
         X313=X(J3,K1,L3)
         X413=X(J4,K1,L3)

         X123=X(J1,K2,L3)
         X223=X(J2,K2,L3)
         X323=X(J3,K2,L3)
         X423=X(J4,K2,L3)

         X133=X(J1,K3,L3)
         X233=X(J2,K3,L3)
         X333=X(J3,K3,L3)
         X433=X(J4,K3,L3)

         X143=X(J1,K4,L3)
         X243=X(J2,K4,L3)
         X343=X(J3,K4,L3)
         X443=X(J4,K4,L3)

         X114=X(J1,K1,L4)
         X214=X(J2,K1,L4)
         X314=X(J3,K1,L4)
         X414=X(J4,K1,L4)

         X124=X(J1,K2,L4)
         X224=X(J2,K2,L4)
         X324=X(J3,K2,L4)
         X424=X(J4,K2,L4)

         X134=X(J1,K3,L4)
         X234=X(J2,K3,L4)
         X334=X(J3,K3,L4)
         X434=X(J4,K3,L4)

         X144=X(J1,K4,L4)
         X244=X(J2,K4,L4)
         X344=X(J3,K4,L4)
         X444=X(J4,K4,L4)

         Y11=ALFX1(N)*X111+ALFX2(N)*X211  &
            +ALFX3(N)*X311+ALFX4(N)*X411
         Y21=ALFX1(N)*X121+ALFX2(N)*X221  &
            +ALFX3(N)*X321+ALFX4(N)*X421
         Y31=ALFX1(N)*X131+ALFX2(N)*X231  &
            +ALFX3(N)*X331+ALFX4(N)*X431
         Y41=ALFX1(N)*X141+ALFX2(N)*X241  &
            +ALFX3(N)*X341+ALFX4(N)*X441

         Y12=ALFX1(N)*X112+ALFX2(N)*X212  &
            +ALFX3(N)*X312+ALFX4(N)*X412
         Y22=ALFX1(N)*X122+ALFX2(N)*X222  &
            +ALFX3(N)*X322+ALFX4(N)*X422
         Y32=ALFX1(N)*X132+ALFX2(N)*X232  &
            +ALFX3(N)*X332+ALFX4(N)*X432
         Y42=ALFX1(N)*X142+ALFX2(N)*X242  &
            +ALFX3(N)*X342+ALFX4(N)*X442

         Y13=ALFX1(N)*X113+ALFX2(N)*X213  &
            +ALFX3(N)*X313+ALFX4(N)*X413
         Y23=ALFX1(N)*X123+ALFX2(N)*X223  &
            +ALFX3(N)*X323+ALFX4(N)*X423
         Y33=ALFX1(N)*X133+ALFX2(N)*X233  &
            +ALFX3(N)*X333+ALFX4(N)*X433
         Y43=ALFX1(N)*X143+ALFX2(N)*X243  &
            +ALFX3(N)*X343+ALFX4(N)*X443

         Y14=ALFX1(N)*X114+ALFX2(N)*X214  &
            +ALFX3(N)*X314+ALFX4(N)*X414
         Y24=ALFX1(N)*X124+ALFX2(N)*X224  &
            +ALFX3(N)*X324+ALFX4(N)*X424
         Y34=ALFX1(N)*X134+ALFX2(N)*X234  &
            +ALFX3(N)*X334+ALFX4(N)*X434
         Y44=ALFX1(N)*X144+ALFX2(N)*X244  &
            +ALFX3(N)*X344+ALFX4(N)*X444

         !  interpolation selon y

         Z1=ALFY1(N)*Y11+ALFY2(N)*Y21  &
           +ALFY3(N)*Y31+ALFY4(N)*Y41

         Z2=ALFY1(N)*Y12+ALFY2(N)*Y22  &
           +ALFY3(N)*Y32+ALFY4(N)*Y42

         Z3=ALFY1(N)*Y13+ALFY2(N)*Y23  &
           +ALFY3(N)*Y33+ALFY4(N)*Y43

         Z4=ALFY1(N)*Y14+ALFY2(N)*Y24  &
           +ALFY3(N)*Y34+ALFY4(N)*Y44

         !   interpolation selon z

         XOUT(N)=ALFZ1(N)*Z1+ALFZ2(N)*Z2  &
                +ALFZ3(N)*Z3+ALFZ4(N)*Z4

         IF(LIMIT) THEN

           R22MX=AMAX1(X222,X232)
           R22MN=AMIN1(X222,X232)

           R32MX=AMAX1(X322,X332)
           R32MN=AMIN1(X322,X332)

           R23MX=AMAX1(X223,X233)
           R23MN=AMIN1(X223,X233)

           R33MX=AMAX1(X323,X333)
           R33MN=AMIN1(X323,X333)

           R2MX=AMAX1(R22MX,R32MX)
           R2MN=AMIN1(R22MN,R32MN)

           R3MX=AMAX1(R23MX,R33MX)
           R3MN=AMIN1(R23MN,R33MN)

           RMX=AMAX1(R2MX,R3MX)
           RMN=AMIN1(R2MN,R3MN)

           ! limitation pour assurer la monotonie
           XOUT(N)=AMAX1(AMIN1(XOUT(N),RMX),RMN)

         ENDIF

      END DO
!$omp end parallel do

      DEALLOCATE (X)

      RETURN
      END
