      SUBROUTINE INTERPOLFORCING( 
     $                            LONM,LATM,LONF,LATF,NIVM,NIVF,
     $                            LOLAM,LOLAF,
     $                            CPP_NIVM,CPP_PHUC,
     $                            TGRIF,UGRIF,VGRIF,WGRIF,QGRIF,PVGRIF,
     $                            PSOLF,
     $                            ALF,BLF,zvamh,zvbmh,
     $                            RLONFMIN,RLONFMAX,RLATFMIN,RLATFMAX,
     $                            PREF,HUCOE,NPCL01,NPCL02,HUTIL,
     $                            RTER,
     $                            TGRIM,UGRIM,VGRIM,WGRIM,QGRIM,PVGRIM,
     $                            PSOLM,DIVQM,DIVUM,
     $                            XPAM,XPHFM,
     $                            ALM,BLM,
     $                            RLONMMIN,RLONMMAX,RLATMMIN,RLATMMAX,
     $                            PHUC)

C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : I/O
C   Purpose    : Perform interpolations from F (forcing) to M (model)
C                grids
C   Called by  : FORCAGE
C
CCC
C
C  Original : P. Moinat, 21/10/2005
C
C  Last Modified : H. Le Berre 10/2006  Integration in CVS
C                : H. Le Berre 11/2006 Prise en compte seuil sur QGRIM 
C
C
#ifdef GAUSS
      USE GRIDS_MOD
#endif
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C INPUT PARAMETERS
      INTEGER :: NIVM,NIVF,LONM,LATM,LONF,LATF
      INTEGER :: LOLAM,LOLAF,CPP_NIVM,CPP_PHUC
      REAL (KIND=8), DIMENSION(LONF,LATF,NIVF) :: TGRIF,UGRIF,
     $                               VGRIF,WGRIF,QGRIF,PVGRIF
      REAL (KIND=8), DIMENSION(LONF,LATF) :: PSOLF
      REAL (KIND=8), DIMENSION(0:NIVF) :: ALF,BLF
      REAL (KIND=8), DIMENSION(0:NIVM) :: zvamh,zvbmh
      REAL (KIND=8) :: RLONFMIN,RLONFMAX,RLATFMIN,RLATFMAX
      REAL (KIND=8) :: PREF,HUCOE,HUTIL
      INTEGER  :: NPCL01,NPCL02
      REAL (KIND=8) :: RTER
C
C OUTPUT PARAMETERS
      REAL (KIND=8), DIMENSION(LONM,LATM,NIVM) :: TGRIM,UGRIM,
     $                               VGRIM,WGRIM,QGRIM,PVGRIM
      REAL (KIND=8), DIMENSION(LONM,LATM) :: PSOLM
      REAL (KIND=8), DIMENSION(LONM,LATM,NIVM) :: DIVQM,DIVUM
      REAL (KIND=8) :: RLONMMIN,RLONMMAX,RLATMMIN,RLATMMAX
      REAL (KIND=8), DIMENSION(0:NIVM) :: ALM,BLM
      REAL (KIND=8), DIMENSION(CPP_PHUC) :: PHUC
      REAL (KIND=8), DIMENSION(LONM,LATM,NIVM) :: XPAM,XPHFM

C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER iniv,ilat,ilon,i
      REAL (KIND=8) :: ETA
      REAL (KIND=8) :: XPAPLUS,XPAMOIN,XPM
      REAL (KIND=8), ALLOCATABLE :: XPAF(:,:,:)
      INTEGER iref
      REAL (KIND=8) :: xp
      REAL (KIND=8) :: DELTALON,DELTALAT
      INTEGER ilatp1,ilatm1,ilonp1,ilonm1
      REAL (KIND=8) :: XRLAT,COSDLAT,TANDLAT,DLATDIF
C
C
C Dynamic allocation of local arrays
      ALLOCATE (XPAF(LONF,LATF,NIVF))
C      
c Surface pressure
c
          do ilon=1,LOLAM
            PSOLM(ilon,1)=PSOLF(ilon,1)
          enddo
c
#ifdef MIDATM
          do ilon=1,LOLAM
            XPHFM(ilon,1,1)=ALM(1)+BLM(1)*PSOLM(ilon,1)
            XPAM(ilon,1,1)=0.5*XPHFM(ilon,1,1)
          enddo
c
          do iniv=2,NIVM
          do ilon=1,LOLAM
            XPAPLUS=ALM(iniv)+BLM(iniv)*PSOLM(ilon,1)
            XPHFM(ilon,1,iniv)=XPAPLUS
            XPAMOIN=ALM(iniv-1)+BLM(iniv-1)*PSOLM(ilon,1)
            XPAM(ilon,1,iniv)=0.5*(XPAPLUS+XPAMOIN)
          enddo
          enddo
c
          do ilon=1,LOLAF
            XPAF(ilon,1,1)=0.5*(ALF(1)+
     $                            BLF(1)*PSOLF(ilon,1))
          enddo
c
          do iniv=2,NIVF
          do ilon=1,LOLAF
            XPAPLUS=ALF(iniv)+BLF(iniv)*PSOLF(ilon,1)
            XPAMOIN=ALF(iniv-1)+BLF(iniv-1)*PSOLF(ilon,1)
            XPAF(ilon,1,iniv)=0.5*(XPAPLUS+XPAMOIN)
          enddo
          enddo
#else
#ifndef ACCIDENT
          
          do ilon=1,LOLAM
            XPAM(ilon,1,1)=
     $      EXP(ALOG(ALM(1)+BLM(1)*PSOLM(ilon,1))-1.)
            XPHFM(ilon,1,1)=ALM(1)+BLM(1)*PSOLM(ilon,1)
          enddo
c
          do iniv=2,NIVM
          do ilon=1,LOLAM
            XPAPLUS=ALM(iniv)+BLM(iniv)*PSOLM(ilon,1)
            XPHFM(ilon,1,iniv)=XPAPLUS
            XPAMOIN=ALM(iniv-1)+BLM(iniv-1)*PSOLM(ilon,1)
            XPAM(ilon,1,iniv)=EXP(
     $        (XPAPLUS*ALOG(XPAPLUS)-XPAMOIN*ALOG(XPAMOIN))
     $       /(XPAPLUS-XPAMOIN) - 1. )
          enddo
          enddo
c
          do ilon=1,LOLAF
            XPAF(ilon,1,1)=
     $      EXP(ALOG(ALF(1)+BLF(1)*PSOLF(ilon,1))-1.)
          enddo
c
          do iniv=2,NIVF
          do ilon=1,LOLAF
            XPAPLUS=ALF(iniv)+BLF(iniv)*PSOLF(ilon,1)
            XPAMOIN=ALF(iniv-1)+BLF(iniv-1)*PSOLF(ilon,1)
            XPAF(ilon,1,iniv)=EXP(
     $        (XPAPLUS*ALOG(XPAPLUS)-XPAMOIN*ALOG(XPAMOIN))
     $       /(XPAPLUS-XPAMOIN) - 1. )
          enddo
          enddo
#else
          do ilon=1,LOLAM
            XPHFM(ilon,1,1)=ALM(1)+BLM(1)*PSOLM(ilon,1)
            XPAM(ilon,1,1)=0.5*XPHFM(ilon,1,1)
          enddo
c
          do iniv=2,NIVM
          do ilon=1,LOLAM
            XPAPLUS=ALM(iniv)+BLM(iniv)*PSOLM(ilon,1)
            XPHFM(ilon,1,iniv)=XPAPLUS
            XPAMOIN=ALM(iniv-1)+BLM(iniv-1)*PSOLM(ilon,1)
            XPAM(ilon,1,iniv)=0.5*(XPAPLUS+XPAMOIN)
          enddo
          enddo
c
          do ilon=1,LOLAF
            XPAF(ilon,1,1)=0.5*(ALF(1)+
     $                            BLF(1)*PSOLF(ilon,1))
          enddo
c
          do iniv=2,NIVF
          do ilon=1,LOLAF
            XPAPLUS=ALF(iniv)+BLF(iniv)*PSOLF(ilon,1)
            XPAMOIN=ALF(iniv-1)+BLF(iniv-1)*PSOLF(ilon,1)
            XPAF(ilon,1,iniv)=0.5*(XPAPLUS+XPAMOIN)
          enddo
          enddo
#endif
#endif
c
          do iniv=1,NIVM                ! LOOP / LEVELS
          do ilon=1,LOLAM               ! LOOP / HORIZONTAL POINTS
c
            XPM=XPAM(ilon,1,iniv)
c
            if (XPM.LT.XPAF(ilon,1,1)) then
              iref=1
              xp=1.
              print*,' EXTRAPOLATION NEEDED FOR (M) : TOP'
            else
c
              if (XPM.GE.XPAF(ilon,1,NIVF)) then
                iref=NIVF-1
                xp=0.
c                print*,' EXTRAPOLATION NEEDED FOR (M) : BOTTOM'
              else
c
                do i=1,NIVF-1
                  if ((XPM.GE.XPAF(ilon,1,i)).AND.
     $                (XPM.LT.XPAF(ilon,1,i+1))) iref=i
                enddo
                xp= (XPAF(ilon,1,iref+1)-XPM)
     $             /(XPAF(ilon,1,iref+1)-XPAF(ilon,1,iref))
              endif
            endif
            xp=min(1.,xp)
            xp=max(0.,xp)

c
            TGRIM(ilon,1,iniv)=
     $             xp  * TGRIF(ilon,1,iref)
     $        +(1.-xp) * TGRIF(ilon,1,iref+1)
            UGRIM(ilon,1,iniv)=
     $             xp  * UGRIF(ilon,1,iref)
     $        +(1.-xp) * UGRIF(ilon,1,iref+1)
            VGRIM(ilon,1,iniv)=
     $             xp  * VGRIF(ilon,1,iref)
     $        +(1.-xp) * VGRIF(ilon,1,iref+1)
            WGRIM(ilon,1,iniv)=
     $             xp  * WGRIF(ilon,1,iref)
     $        +(1.-xp) * WGRIF(ilon,1,iref+1)
            QGRIM(ilon,1,iniv)=
     $             xp  * QGRIF(ilon,1,iref)
     $        +(1.-xp) * QGRIF(ilon,1,iref+1)
            QGRIM(ilon,1,iniv)=
     $      MAX (QGRIM(ilon,1,iniv),1.E-10)
            PVGRIM(ilon,1,iniv)=
     $             xp  * PVGRIF(ilon,1,iref)
     $        +(1.-xp) * PVGRIF(ilon,1,iref+1)

          enddo               ! LOOP / HORIZONTAL POINTS
c
c Humidity and mass convergence
c
          DELTALON=(RLONMMAX-RLONMMIN)/REAL(LONM)
#ifndef GAUSS
          DELTALAT=(RLATMMAX-RLATMMIN)/REAL(LATM)
#endif
c
          do ilat=1,LATM
          do ilon=1,LONM
c
            ilatp1=MIN(LATM,ilat+1)
            ilatm1=MAX(1    ,ilat-1)
#ifdef GAUSS
            DELTALAT=RLATM1(ilatp1)-RLATM1(ilatm1)
#endif	    
            ilonp1=MIN(LONM,ilon+1)
            ilonm1=MAX(1    ,ilon-1)
c
#ifndef GAUSS
            XRLAT=RLATMMIN+DELTALAT*(REAL(ilat)-0.5)
#else
            XRLAT=RLATM1(ILAT)
#endif
            COSDLAT=COS(XRLAT)*DELTALON*(ilonp1-ilonm1)*RTER
            TANDLAT=SIN(XRLAT)/(COS(XRLAT)*RTER)
#ifndef GAUSS
            DLATDIF=DELTALAT*(ilatp1-ilatm1)*RTER
#else
            DLATDIF=ABS(DELTALAT)*RTER
#endif
c
            DIVQM(ilon,ilat,iniv)=
     $        (UGRIM(ilonp1,ilat,iniv)*QGRIM(ilonp1,ilat,iniv)
     $        -UGRIM(ilonm1,ilat,iniv)*QGRIM(ilonm1,ilat,iniv))
     $        /(COSDLAT) +
     $        (VGRIM(ilon,ilatp1,iniv)*QGRIM(ilon,ilatp1,iniv)
     $        -VGRIM(ilon,ilatm1,iniv)*QGRIM(ilon,ilatm1,iniv))
     $        /(DLATDIF) -
     $        VGRIM(ilon,ilat,iniv)*QGRIM(ilon,ilat,iniv)
     $        *TANDLAT
c
            DIVUM(ilon,ilat,iniv)=
     $        (UGRIM(ilonp1,ilat,iniv)
     $        -UGRIM(ilonm1,ilat,iniv))/(COSDLAT) +
     $        (VGRIM(ilon,ilatp1,iniv)
     $        -VGRIM(ilon,ilatm1,iniv))/(DLATDIF) -
     $        VGRIM(ilon,ilat,iniv)*TANDLAT
c
          enddo
          enddo
c
          enddo                ! LOOP / LEVELS

c De-allocations
          DEALLOCATE (XPAF)
C
C
          return
          end
