      SUBROUTINE IOPC3D(daynum,idate0)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Interface
C   Purpose   : 3D interface with physics and chemistry
C   Called by : MAIN 
C
CCC
C
C  Original : V.-H. Peuch, 05/1999
C
C  Modified : V.-H. Peuch, 02/2000, 02/2003
C	      H. Le Berre, 09/2006, modification interface d'appel KFBCONV,
C                                   passage de variables en argument pour
C                                   eviter des tests et instructions
C                                   redondantes sur les 4 domaines
C             B. Josse; 01/2007     elimination du facteur de reduction 
C                                   des emissions des Nox
C
CCC
C
C LSUP : HIGHEST POSSIBLE MODEL LEVEL FOR WET SCAVENGING
C
      USE PARADI_MOD
      USE PHOTO_MOD
      USE GRIDS_MOD
      USE EXPE_MOD
      USE CSTES_MOD
      USE MOCF_MOD
      USE MRAT_MOD
      USE FORMT_MOD
      USE FORS_MOD
      USE SVPHOT_MOD
      USE HENRY_MOD
      USE CLOUDRAIN_MOD
#ifdef GAUSS
      USE GAUSS_MOD
#endif
#ifdef CARIOLLE
      USE CARIOP_MOD
#endif
      USE SRC_MOD
      USE SRCFLX_MOD
C
      IMPLICIT NONE
C	
C TIEDTKE CONVECTION 
C
C     COMMON/BIGMAT/BIGG1(LONM1,LATM1,NIVM1,NIVM1)
C    +             ,BIGG2(LONM2,LATM2,NIVM2,NIVM2)
C    +             ,BIGG3(LONM3,LATM3,NIVM3,NIVM3)
C    +             ,BIGG4(LONM4,LATM4,NIVM4,NIVM4)
C
C     COMMON/BIGMATIP/IBIGIP1(LONM1,LATM1,NIVM1)
C    +               ,IBIGIP2(LONM2,LATM2,NIVM2)
C    +               ,IBIGIP3(LONM3,LATM3,NIVM3)
C    +               ,IBIGIP4(LONM4,LATM4,NIVM4)
C
C KEYS
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C      
      INTEGER , DIMENSION (5) , INTENT(IN) :: idate0
      REAL (KIND=8) , INTENT(IN) :: daynum
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: IORDER
      INTEGER :: NBNIV,LSUP,IHOUR,NSEC,NSECTOT,NSECCY,LDEB
      INTEGER :: I,K,L,ILAT
      INTEGER :: IH2O,ICO,ITRA,ISRC,JJ,NSECDEB,NSECFIN
      INTEGER :: INDPOL,ILON,INOX,IEMI,IBIGIP1,IBIGIP2,IBIGIP3,IBIGIP4
      INTEGER :: IOPTLESSIV,INO,INO2,INO3,IORDEROLD
      INTEGER :: INIV,IF11,IF12,IF113,IMCL3,JNOX,JNOY,NIVDEB,INOY
      INTEGER :: IHNO3,IN2O5,IHNO4,IHONO,IONIT,IPAN     
      INTEGER :: IPSOURCE 
      REAL (KIND=8) :: BETA,XLONS,XLATS,XHOUR,XP,XSDAY,XDAY
      REAL (KIND=8) :: CONC,PTOP,PBOT
      REAL (KIND=8) :: BIGG1,BIGG2,BIGG3,BIGG4,TAUX,TXDEC_RAD
      REAL (KIND=8) :: DELTAQ,ZFACT,TRELAX,ZPAM
      REAL (KIND=8) :: XF11,XF12,XF113,XMCL3,DTH2O
      REAL (KIND=8) :: DEMIVIE,MOLWT,TXDEC,DXS
      LOGICAL :: LBDGT
      CHARACTER(LEN=6) :: UNITE
      CHARACTER(LEN=15) :: POLLUTNAME

      
C
#ifdef MIDATM
#ifdef NIV_91
      PARAMETER(BETA=0.5,NBNIV=9,LSUP=39)
#endif
#ifdef NIV_60
      PARAMETER(BETA=0.5,NBNIV=8,LSUP=26)
#endif
#else      
      PARAMETER(BETA=0.5,NBNIV=5,LSUP=20)
#endif
C
      LOGICAL :: ICALPHOT
      LOGICAL :: ICALCONV
      LOGICAL :: LDEPNOX
      LOGICAL :: LSCAVKFB
      LOGICAL :: LSCAVSTRATI
      LOGICAL :: LSCAVCONVBC
      LOGICAL :: LNEIGE
      INTEGER , ALLOCATABLE :: idate(:)
      INTEGER , ALLOCATABLE :: DATEMPO(:)      
C
      REAL (KIND=8) , ALLOCATABLE :: EV1(:,:),EV2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: EV3(:,:),EV4(:,:)
      REAL (KIND=8) , ALLOCATABLE :: XALBSM1(:,:),XALBSM2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: XALBSM3(:,:),XALBSM4(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PWAL1(:,:),PWAL2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PWAL3(:,:),PWAL4(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PWAC1(:,:),PWAC2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PWAC3(:,:),PWAC4(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PSNL1(:,:),PSNL2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PSNL3(:,:),PSNL4(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PSNC1(:,:),PSNC2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PSNC3(:,:),PSNC4(:,:)
      REAL (KIND=8) , ALLOCATABLE :: EM1(:,:,:),EM2(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: EM3(:,:,:),EM4(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: VD1(:,:,:),VD2(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: VD3(:,:,:),VD4(:,:,:)
#ifdef MASSy
      REAL (KIND=8) , ALLOCATABLE :: NOX1 (:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: NOX2 (:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: NOX3 (:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: NOX4 (:,:,:)
#endif
C	
      SAVE IORDER
      DATA IORDER / 1 /
C	
      ALLOCATE (IDATE(5))
      ALLOCATE (DATEMPO(5))
      ALLOCATE (EV1(LONM1,LATM1))
      ALLOCATE (EV2(LONM2,LATM2))
      ALLOCATE (EV3(LONM3,LATM3))
      ALLOCATE (EV4(LONM4,LATM4))
      ALLOCATE (XALBSM1(LONM1,LATM1))
      ALLOCATE (XALBSM2(LONM2,LATM2))
      ALLOCATE (XALBSM3(LONM3,LATM3))
      ALLOCATE (XALBSM4(LONM4,LATM4))
      ALLOCATE (PWAL1(LONM1,LATM1))
      ALLOCATE (PWAL2(LONM2,LATM2))
      ALLOCATE (PWAL3(LONM3,LATM3))
      ALLOCATE (PWAL4(LONM4,LATM4))
      ALLOCATE (PWAC1(LONM1,LATM1))
      ALLOCATE (PWAC2(LONM2,LATM2))
      ALLOCATE (PWAC3(LONM3,LATM3))
      ALLOCATE (PWAC4(LONM4,LATM4))
      ALLOCATE (PSNL1(LONM1,LATM1))
      ALLOCATE (PSNL2(LONM2,LATM2))
      ALLOCATE (PSNL3(LONM3,LATM3))
      ALLOCATE (PSNL4(LONM4,LATM4))
      ALLOCATE (PSNC1(LONM1,LATM1))
      ALLOCATE (PSNC2(LONM2,LATM2))
      ALLOCATE (PSNC3(LONM3,LATM3))
      ALLOCATE (PSNC4(LONM4,LATM4))
      ALLOCATE (EM1(LONM1,LATM1,NEMI1))
      ALLOCATE (EM2(LONM2,LATM2,NEMI2))
      ALLOCATE (EM3(LONM3,LATM3,NEMI3))
      ALLOCATE (EM4(LONM4,LATM4,NEMI4))
      ALLOCATE (VD1(LONM1,LATM1,NDEP1))
      ALLOCATE (VD2(LONM2,LATM2,NDEP2))
      ALLOCATE (VD3(LONM3,LATM3,NDEP3))
      ALLOCATE (VD4(LONM4,LATM4,NDEP4))
#ifdef MASSy
      ALLOCATE (NOX1(LONM1,LATM1,NIVM1))
      ALLOCATE (NOX2(LONM2,LATM2,NIVM2))
      ALLOCATE (NOX3(LONM3,LATM3,NIVM3))
      ALLOCATE (NOX4(LONM4,LATM4,NIVM4))
#endif
C
CCC CONFIGURATION FOR IOPC
C
C IORDER=1 : Emissions -> 2*Convection -> Chemistry -> Deposition
C IORDER=2 : Deposition -> Chemistry -> Emissions
C
CCCCC
C     _________________
C    |                 |
C    | 0. DATE / HOUR  |
C    |_________________|
C
CCCCC
C
      ihour=mod(idate0(4)+int(daynum)/3600,24)
      xhour=daynum/3600.-INT(daynum/3600.)
      xp=1.-xhour
      xp=min(1.,xp)
      xp=max(0.,xp)
C
      XSDAY=REAL(
     +       mod(idate0(4)*3600+idate0(5)*60+int(daynum),86400))
C
      idate(1)=idate0(1)
      idate(2)=1
      idate(3)=1
      idate(4)=0
      idate(5)=0     
      CALL TIMESEC(idate,idate0,NSEC)
      NSEC=NSEC+int(daynum)
      NSECTOT=365*86400
      IF (mod(idate0(1),4).EQ.0) NSECTOT=NSECTOT+86400
      XDAY=REAL(NSEC)/REAL(NSECTOT) 
C
      ICALCONV=.FALSE.
C
C COMPUTE PHOTOLYSES EVERY TIMESTEP 
C
C TO UPDATE PHOTOLYSES EVERY DT0 ONLY, CHANGE
C NEXT LINE TO :
C
C     ICALPHOT=.FALSE.
C
      ICALPHOT=.TRUE.
C
      NSECCY=mod(int(daynum),NSCY)
      IF (mod(NSECCY,NSCY/NITERT).EQ.0) THEN
        ICALCONV=.TRUE.
        ICALPHOT=.TRUE.
      ENDIF
C
C COMPUTE SCAVENGING (KFB AND/OR STRATIFORM)
C 
#ifdef CARIOLLE
      LSCAVKFB=.FALSE.
      LSCAVSTRATI=.FALSE.
      LSCAVCONVBC=.FALSE.
#else
      LSCAVKFB=.TRUE.
      LSCAVSTRATI=.TRUE.
      LSCAVCONVBC=.TRUE.
#endif
C
C COMPUTE BUDGETS AND FLUXES (YET : ACCIDENT ONLY)
C
      IF (CHEMSCHEME.EQ.'ACCIDENT') THEN
       IF (.NOT.TRANSINV) THEN
        LBDGT=.TRUE.
       ELSE
        LBDGT=.FALSE.
       ENDIF
      ELSE
        LBDGT=.FALSE.
      ENDIF          
C
C DETECT WATER VAPOUR AND CARBON MONOXIDE MIXING RATIOS
C
      iH2O = 0
      iCO  = 0
      DO itra=1,NTRA1
        IF (TRANAM1(itra).EQ.'H_2O') iH2O=itra
        IF (TRANAM1(itra).EQ.'CO'  ) iCO =itra
      ENDDO
C
C INDEX FOR KFB CONVECTION
C
#ifdef NIV_47
      LDEB=10
#else
      LDEB = 1
      DO iniv=1,NIVM1
        ZPAM=0.
        DO ilon=1,LONM1*LATM1
          ZPAM=ZPAM+XPAM1(ilon,1,iniv)/REAL(LONM1*LATM1)
        ENDDO
        IF (ZPAM.LE. 5000.) LDEB = iniv
      ENDDO
#endif
C
C POINT SOURCE RELEASE
C
      IF (SRC) THEN
C
        DO isrc=1,NSRC
        
         IF (.NOT.TRANSINV) THEN

          DO jj=1,5
            DATEMPO(jj) = IDATEMI1(isrc,jj)
          ENDDO
          CALL TIMESEC(idate0,DATEMPO,NSECDEB)
          DO jj=1,5
            DATEMPO(jj) = IDATEMI2(isrc,jj)
          ENDDO
          CALL TIMESEC(idate0,DATEMPO,NSECFIN)
              ELSE
          DO jj=1,5
            DATEMPO(jj) = IDATEMI1(isrc,jj)
          ENDDO
          CALL TIMESEC(DATEMPO,idate0,NSECDEB)
          DO jj=1,5
            DATEMPO(jj) = IDATEMI2(isrc,jj)
          ENDDO
          CALL TIMESEC(DATEMPO,idate0,NSECFIN)
         ENDIF     


C
          IF ((daynum.GE.NSECDEB).AND.
     +        (daynum.LT.NSECFIN)) THEN
C
             IPSOURCE=INT(PSOURCE(isrc,1))
             DXS=PSOURCE(isrc,1)-REAL(IPSOURCE)
             DXS=DXS*10./6.
             XLONS=(REAL(IPSOURCE)+DXS)/CONV

             IPSOURCE=INT(PSOURCE(isrc,2))
             DXS=PSOURCE(isrc,2)-REAL(IPSOURCE)
             DXS=DXS*10./6.
             XLATS=(REAL(IPSOURCE)+DXS)/CONV

            CONC=PSOURCE(isrc,3)
            PTOP=PSOURCE(isrc,4)
            PBOT=PSOURCE(isrc,5)
            INDPOL=isrc
C
C LATER, USE POLLUTANT NAME
C
            UNITE=EMIUNIT(isrc)
            POLLUTNAME=POLNAME(isrc)
            DEMIVIE=HALFLIFE(isrc)
            MOLWT=MOLWEIGHT(isrc)

C
C GLOBAL GRID
C
            IF (IFLAG1) THEN
              CALL SOURCE(XLONS,XLATS,CONC,
     +                    PTOP,PBOT,INDPOL,UNITE,
     +                    RLATM1,DRLATM1,RLONM1,DRLONM1,
     +                    TGRIM1,XPHFM1,XPAM1,QQQM1,
     +                    LONM1,LATM1,NIVM1,NTRA1,
     +                    POLLUTNAME,DEMIVIE,MOLWT,1) 
            ENDIF
C
C CONTINENTAL GRID
C
            IF (IFLAG2) THEN
              CALL SOURCE(XLONS,XLATS,CONC,
     +                    PTOP,PBOT,INDPOL,UNITE,
     +                    RLATM2,DRLATM2,RLONM2,DRLONM2,
     +                    TGRIM2,XPHFM2,XPAM2,QQQM2,
     +                    LONM2,LATM2,NIVM2,NTRA2,
     +                    POLLUTNAME,DEMIVIE,MOLWT,2)
            ENDIF
C
C REGIONAL GRID
C
            IF (IFLAG3) THEN
              CALL SOURCE(XLONS,XLATS,CONC,
     +                    PTOP,PBOT,INDPOL,UNITE,
     +                    RLATM3,DRLATM3,RLONM3,DRLONM3,
     +                    TGRIM3,XPHFM3,XPAM3,QQQM3,
     +                    LONM3,LATM3,NIVM3,NTRA3,
     +                    POLLUTNAME,DEMIVIE,MOLWT,3)
            ENDIF
C
C ZOOM GRID
C
            IF (IFLAG4) THEN
              CALL SOURCE(XLONS,XLATS,CONC,
     +                    PTOP,PBOT,INDPOL,UNITE,
     +                    RLATM4,DRLATM4,RLONM4,DRLONM4,
     +                    TGRIM4,XPHFM4,XPAM4,QQQM4,
     +                    LONM4,LATM4,NIVM4,NTRA4,
     +                    POLLUTNAME,DEMIVIE,MOLWT,4)
            ENDIF
          ENDIF
C
        ENDDO
      ENDIF
C
CCC
C     ___________________________
C    |                           |
C    | 1. ALBEDO, RAIN AND SNOW  |
C    |___________________________|
C
CCC
C
      IF (SURF) THEN
C
        IF (IFLAG1) THEN
          DO ilon=1,LOLAM1
            XALBSM1(ilon,1)=   xp*XALBSM1D(ilon,1,ihour)
     +                   +(1.-xp)*XALBSM1D(ilon,1,ihour+1)
            PWAL1(ilon,1)=     xp*PWAL1D(ilon,1,ihour)
     +                   +(1.-xp)*PWAL1D(ilon,1,ihour+1)
            PWAC1(ilon,1)=     xp*PWAC1D(ilon,1,ihour)
     +                   +(1.-xp)*PWAC1D(ilon,1,ihour+1)
            PSNL1(ilon,1)=     xp*PSNL1D(ilon,1,ihour)
     +                   +(1.-xp)*PSNL1D(ilon,1,ihour+1)
            PSNC1(ilon,1)=     xp*PSNC1D(ilon,1,ihour)
     +                   +(1.-xp)*PSNC1D(ilon,1,ihour+1)
          ENDDO
        ENDIF
        IF (IFLAG2)THEN
          DO ilon=1,LOLAM2
            XALBSM2(ilon,1)=   xp*XALBSM2D(ilon,1,ihour)
     +                   +(1.-xp)*XALBSM2D(ilon,1,ihour+1)
            PWAL2(ilon,1)=     xp*PWAL2D(ilon,1,ihour)
     +                   +(1.-xp)*PWAL2D(ilon,1,ihour+1)
            PWAC2(ilon,1)=     xp*PWAC2D(ilon,1,ihour)
     +                   +(1.-xp)*PWAC2D(ilon,1,ihour+1)
            PSNL2(ilon,1)=     xp*PSNL2D(ilon,1,ihour)
     +                   +(1.-xp)*PSNL2D(ilon,1,ihour+1)
            PSNC2(ilon,1)=     xp*PSNC2D(ilon,1,ihour)
     +                   +(1.-xp)*PSNC2D(ilon,1,ihour+1)
          ENDDO
        ENDIF
        IF (IFLAG3) THEN
          DO ilon=1,LOLAM3
            XALBSM3(ilon,1)=   xp*XALBSM3D(ilon,1,ihour)
     +                   +(1.-xp)*XALBSM3D(ilon,1,ihour+1)
            PWAL3(ilon,1)=     xp*PWAL3D(ilon,1,ihour)
     +                   +(1.-xp)*PWAL3D(ilon,1,ihour+1)
            PWAC3(ilon,1)=     xp*PWAC3D(ilon,1,ihour)
     +                   +(1.-xp)*PWAC3D(ilon,1,ihour+1)
            PSNL3(ilon,1)=     xp*PSNL3D(ilon,1,ihour)
     +                   +(1.-xp)*PSNL3D(ilon,1,ihour+1)
            PSNC3(ilon,1)=     xp*PSNC3D(ilon,1,ihour)
     +                   +(1.-xp)*PSNC3D(ilon,1,ihour+1)
          ENDDO
        ENDIF
        IF (IFLAG4) THEN
          DO ilon=1,LOLAM4
            XALBSM4(ilon,1)=   xp*XALBSM4D(ilon,1,ihour)
     +                   +(1.-xp)*XALBSM4D(ilon,1,ihour+1)
            PWAL4(ilon,1)=     xp*PWAL4D(ilon,1,ihour)
     +                   +(1.-xp)*PWAL4D(ilon,1,ihour+1)
            PWAC4(ilon,1)=     xp*PWAC4D(ilon,1,ihour)
     +                   +(1.-xp)*PWAC4D(ilon,1,ihour+1)
            PSNL4(ilon,1)=     xp*PSNL4D(ilon,1,ihour)
     +                   +(1.-xp)*PSNL4D(ilon,1,ihour+1)
            PSNC4(ilon,1)=     xp*PSNC4D(ilon,1,ihour)
     +                   +(1.-xp)*PSNC4D(ilon,1,ihour+1)
          ENDDO
        ENDIF
C
      ELSE
C
      IF (IFLAG1) THEN
        DO ilon=1,LOLAM1
          XALBSM1(ilon,1) = 0.2
          PWAL1(ilon,1)   = 0.
          PWAC1(ilon,1)   = 0.
          PSNL1(ilon,1)   = 0.
          PSNC1(ilon,1)   = 0.
        ENDDO
      ENDIF
      IF (IFLAG2) THEN
        DO ilon=1,LOLAM2
          XALBSM2(ilon,1) = 0.2
          PWAL2(ilon,1)   = 0.
          PWAC2(ilon,1)   = 0.
          PSNL2(ilon,1)   = 0.
          PSNC2(ilon,1)   = 0.
        ENDDO
      ENDIF
      IF (IFLAG3) THEN
        DO ilon=1,LOLAM3
          XALBSM3(ilon,1) = 0.2
          PWAL3(ilon,1)   = 0.
          PWAC3(ilon,1)   = 0.
          PSNL3(ilon,1)   = 0.
          PSNC3(ilon,1)   = 0.
        ENDDO
      ENDIF
      IF (IFLAG4) THEN
        DO ilon=1,LOLAM4
          XALBSM4(ilon,1) = 0.2
          PWAL4(ilon,1)   = 0.
          PWAC4(ilon,1)   = 0.
          PSNL4(ilon,1)   = 0.
          PSNC4(ilon,1)   = 0.
        ENDDO
      ENDIF
C
      ENDIF

C
C CROSSROADS
C
      GOTO (200,500),IORDER
C
CCCCC
C     _____________________________
C    |                             |
C    | 2. SURFACE EMISSIONS        |
C    |_____________________________|
C
CCCCC
C
 200  IF ( (SURF) .AND. (CHEMSCHEME.NE.'ACCIDENT') ) THEN
C
        WRITE(*,*) '> EMISSIONS'
C
        iNOx = 0
        DO iemi=1,NEMI1
          IF (EMINAM1(iemi).EQ.'NO_x') iNOx=iemi
        ENDDO
        
        iF11 = 0
        iF12 = 0
        iF113= 0
        iMCl3= 0
        jNOx = 0
        jNOy = 0
         DO itra=1,NEMI1
          IF (EMINAM1(itra).EQ.'CFC11'    ) iF11 =itra
          IF (EMINAM1(itra).EQ.'CFC12'    ) iF12 =itra
          IF (EMINAM1(itra).EQ.'CFC113'   ) iF113=itra
          IF (EMINAM1(itra).EQ.'CH_3CCl_3') iMCl3=itra
        ENDDO
        DO itra=1,NTRA1
          IF (TRANAM1(itra).EQ.'NO_x') jNOx=itra
          IF (TRANAM1(itra).EQ.'NO_y') jNOy=itra
        ENDDO
C
C correction factor for mid-80s emission values reduced for year 2000
C
        XF11 = 86./360.
        XF12 =122./460.
        XF113= 23./200.
        XMCl3=  5./600.
C
        IF (IFLAG1) THEN
C
          EM1(:,:,:)=     xp *EM1D(:,:,:,ihour)
     +               +(1.-xp)*EM1D(:,:,:,ihour+1)
C
          IF (iF11.NE.0) THEN 
            EM1(:,:,iF11 )=XF11 *EM1(:,:,iF11 )
          ENDIF
          IF (iF12.NE.0) THEN 
            EM1(:,:,iF12 )=XF12 *EM1(:,:,iF12 )
          ENDIF
          IF (iF113.NE.0) THEN 
            EM1(:,:,iF113)=XF113*EM1(:,:,iF113)
          ENDIF
          IF (iMCl3.NE.0) THEN 
            EM1(:,:,iMCl3)=XMCl3*EM1(:,:,iMCl3)
          ENDIF
#ifdef MASSy
C
          IF (jNOx.NE.0) THEN
            NOX1(:,:,:)=QQQM1(:,:,:,jNOx)
          ENDIF
#endif
C
          CALL EMIS(LONM1,LATM1,NIVM1,NIVM1,NBNIV,NEMI1
     +             ,NTRA1,BETA,DT,XPHFM1,EM1,QQQM1,IQMEMI1)
#ifdef MASSy
C
          IF (jNOx.NE.0) THEN
            QQQM1(:,:,:,jNOy)=QQQM1(:,:,:,jNOy) +
     +         MAX( 0., QQQM1(:,:,:,jNOx)- NOX1(:,:,:) )
          ENDIF
#endif
C
        ENDIF
C
        IF (IFLAG2) THEN
          EM2(:,:,:)=     xp *EM2D(:,:,:,ihour)
     +               +(1.-xp)*EM2D(:,:,:,ihour+1)
C         
          IF (iF11.NE.0) THEN 
            EM2(:,:,iF11 )=XF11 *EM2(:,:,iF11 )
          ENDIF
          IF (iF12.NE.0) THEN 
            EM2(:,:,iF12 )=XF12 *EM2(:,:,iF12 )
          ENDIF
          IF (iF113.NE.0) THEN 
            EM2(:,:,iF113)=XF113*EM2(:,:,iF113)
          ENDIF
          IF (iMCl3.NE.0) THEN 
            EM2(:,:,iMCl3)=XMCl3*EM2(:,:,iMCl3)
          ENDIF
#ifdef MASSy
C
          IF (jNOx.NE.0) THEN
            NOX2(:,:,:)=QQQM2(:,:,:,jNOx)
          ENDIF
#endif
C
          CALL EMIS(LONM2,LATM2,NIVM2,NIVM2,NBNIV,NEMI2
     +             ,NTRA2,BETA,DT,XPHFM2,EM2,QQQM2,IQMEMI2)
#ifdef MASSy
C
          IF (jNOx.NE.0) THEN
            QQQM2(:,:,:,jNOy)=QQQM2(:,:,:,jNOy) +
     +         MAX( 0., QQQM2(:,:,:,jNOx)- NOX2(:,:,:) )
          ENDIF
#endif
C
        ENDIF
C
        IF (IFLAG3) THEN
          EM3(:,:,:)=     xp *EM3D(:,:,:,ihour)
     +               +(1.-xp)*EM3D(:,:,:,ihour+1)
C
          IF (iF11.NE.0) THEN 
            EM3(:,:,iF11 )=XF11 *EM3(:,:,iF11 )
          ENDIF
          IF (iF12.NE.0) THEN 
            EM3(:,:,iF12 )=XF12 *EM3(:,:,iF12 )
          ENDIF
          IF (iF113.NE.0) THEN 
            EM3(:,:,iF113)=XF113*EM3(:,:,iF113)
          ENDIF
          IF (iMCl3.NE.0) THEN 
            EM3(:,:,iMCl3)=XMCl3*EM3(:,:,iMCl3)
          ENDIF
#ifdef MASSy
C
          IF (jNOx.NE.0) THEN
            NOX3(:,:,:)=QQQM3(:,:,:,jNOx)
          ENDIF
#endif
C
          CALL EMIS(LONM3,LATM3,NIVM3,NIVM3,NBNIV,NEMI3
     +             ,NTRA3,BETA,DT,XPHFM3,EM3,QQQM3,IQMEMI3)
#ifdef MASSy
C
          IF (jNOx.NE.0) THEN
            QQQM3(:,:,:,jNOy)=QQQM3(:,:,:,jNOy) +
     +         MAX( 0., QQQM3(:,:,:,jNOx)- NOX3(:,:,:) )
          ENDIF
#endif
C
        ENDIF
C
        IF (IFLAG4) THEN
          EM4(:,:,:)=     xp *EM4D(:,:,:,ihour)
     +               +(1.-xp)*EM4D(:,:,:,ihour+1)
C
          IF (iF11.NE.0) THEN 
            EM4(:,:,iF11 )=XF11 *EM4(:,:,iF11 )
          ENDIF
          IF (iF12.NE.0) THEN 
            EM4(:,:,iF12 )=XF12 *EM4(:,:,iF12 )
          ENDIF
          IF (iF113.NE.0) THEN 
            EM4(:,:,iF113)=XF113*EM4(:,:,iF113)
          ENDIF
          IF (iMCl3.NE.0) THEN 
            EM4(:,:,iMCl3)=XMCl3*EM4(:,:,iMCl3)
          ENDIF
#ifdef MASSy
C
          IF (jNOx.NE.0) THEN
            NOX4(:,:,:)=QQQM4(:,:,:,jNOx)
          ENDIF
#endif
C
          CALL EMIS(LONM4,LATM4,NIVM4,NIVM4,NBNIV,NEMI4
     +             ,NTRA4,BETA,DT,XPHFM4,EM4,QQQM4,IQMEMI4)
#ifdef MASSy
C
          IF (jNOx.NE.0) THEN
            QQQM4(:,:,:,jNOy)=QQQM4(:,:,:,jNOy) +
     +         MAX( 0., QQQM4(:,:,:,jNOx)- NOX4(:,:,:) )
          ENDIF
#endif
C
        ENDIF
C
      ENDIF
C
C CROSSROADS
C
      GOTO (300,600),IORDER
C
CCCCC
C     _____________________________________
C    |                                     |
C    | 3. CONVECTION & VERTICAL DIFFUSION  |
C    |_____________________________________|
C
CCCCC
C
 300  IF (SURF) THEN
C
        IF (IFLAG1) THEN
          DO ilon=1,LOLAM1
            EV1(ilon,1)=     xp*EV1D(ilon,1,ihour)
     +                 +(1.-xp)*EV1D(ilon,1,ihour+1)
          ENDDO
        ENDIF
        IF (IFLAG2) THEN
          DO ilon=1,LOLAM2
            EV2(ilon,1)=     xp*EV2D(ilon,1,ihour)
     +                 +(1.-xp)*EV2D(ilon,1,ihour+1)
          ENDDO
        ENDIF
        IF (IFLAG3) THEN
          DO ilon=1,LOLAM3
            EV3(ilon,1)=     xp*EV3D(ilon,1,ihour)
     +                 +(1.-xp)*EV3D(ilon,1,ihour+1)
          ENDDO
        ENDIF
        IF (IFLAG4) THEN
          DO ilon=1,LOLAM4
            EV4(ilon,1)=     xp*EV4D(ilon,1,ihour)
     +                 +(1.-xp)*EV4D(ilon,1,ihour+1)
          ENDDO
        ENDIF
C
        IF (PRMTRSP.EQ.'TDK') THEN
          WRITE(*,*) '> TIEDTKE CONVECTION + LOUIS DIFFUSION'
          IF (IFLAG1) THEN
            CALL TCVCONV(LONM1,LATM1,NIVM1,NTRA1,
     +                   SURFM1,
     +                   XPAM1,XPHFM1,PSOLM1,EV1,
     +                   UGRIM1,VGRIM1,TGRIM1,QGRIM1,
     +                   DIVQM1,DIVUM1,
     +                   QQQM1,LCVPFLX1,SCVPFLX1,
     +                   ICALCONV,BIGG1,IBIGIP1)
          ENDIF
          IF (IFLAG2) THEN
            CALL TCVCONV(LONM2,LATM2,NIVM2,NTRA2,
     +                   SURFM2,
     +                   XPAM2,XPHFM2,PSOLM2,EV2,
     +                   UGRIM2,VGRIM2,TGRIM2,QGRIM2,
     +                   DIVQM2,DIVUM2,
     +                   QQQM2,LCVPFLX2,SCVPFLX2,
     +                   ICALCONV,BIGG2,IBIGIP2)
          ENDIF
          IF (IFLAG3) THEN
            CALL TCVCONV(LONM3,LATM3,NIVM3,NTRA3,
     +                   SURFM3,
     +                   XPAM3,XPHFM3,PSOLM3,EV3,
     +                   UGRIM3,VGRIM3,TGRIM3,QGRIM3,
     +                   DIVQM3,DIVUM3,
     +                   QQQM3,LCVPFLX3,SCVPFLX3,
     +                   ICALCONV,BIGG3,IBIGIP3)
          ENDIF
          IF (IFLAG4) THEN
            CALL TCVCONV(LONM4,LATM4,NIVM4,NTRA4,
     +                   SURFM4,
     +                   XPAM4,XPHFM4,PSOLM4,EV4,
     +                   UGRIM4,VGRIM4,TGRIM4,QGRIM4,
     +                   DIVQM4,DIVUM4,
     +                   QQQM4,LCVPFLX4,SCVPFLX4,
     +                   ICALCONV,BIGG4,IBIGIP4)
          ENDIF
C
        ELSEIF (PRMTRSP.EQ.'KFB') THEN
          WRITE(*,*) '> KFB CONVECTION + LOUIS DIFFUSION'
          IF (IFLAG1) THEN

            IF (LBDGT) THEN
              CALL BUDGET(DFICVCUMUL1,XPHFM1,QQQM1,+1.,
     +                      LONM1,LATM1,NIVM1,NTRA1)
            ENDIF

            CALL KFBCONV(LONM1,LATM1,NIVM1,NTRA1,NSLS1,LDEB,
     +                   TRANAM1,LSCAVKFB,XKHENRY1,
     +                   XPAM1,TGRIM1,QGRIM1,UGRIM1,VGRIM1,
     +                   WGRIM1,SURFM1,XPHFM1,QQQM1,
     +                   LCVPFLX1,SCVPFLX1,KCVDEB1,KCVFIN1)

            IF (LBDGT) THEN
             CALL BUDGET(DFICVCUMUL1,XPHFM1,QQQM1,-1.,
     +                      LONM1,LATM1,NIVM1,NTRA1)
             DO L=1,NIVM1
             DO K=1,LATM1
             DO I=1,LONM1
              PCVRCUMUL1(I,K,L)=PCVRCUMUL1(I,K,L)
     +                         +DTCONV*LCVPFLX1(I,K,L)
              PCVSCUMUL1(I,K,L)=PCVSCUMUL1(I,K,L)
     +                         +DTCONV*SCVPFLX1(I,K,L)
             ENDDO
             ENDDO
             ENDDO    
       
            ENDIF

          ENDIF
          IF (IFLAG2) THEN

            IF (LBDGT) THEN
              CALL BUDGET(DFICVCUMUL2,XPHFM2,QQQM2,+1.,
     +                      LONM2,LATM2,NIVM2,NTRA2)
            ENDIF

                  
            CALL KFBCONV(LONM2,LATM2,NIVM2,NTRA2,NSLS2,LDEB,
     +                   TRANAM2,LSCAVKFB,XKHENRY2,
     +                   XPAM2,TGRIM2,QGRIM2,UGRIM2,VGRIM2,
     +                   WGRIM2,SURFM2,XPHFM2,QQQM2,
     +                   LCVPFLX2,SCVPFLX2,KCVDEB2,KCVFIN2)

            IF (LBDGT) THEN
             CALL BUDGET(DFICVCUMUL2,XPHFM2,QQQM2,-1.,
     +                   LONM2,LATM2,NIVM2,NTRA2)
C
             DO L=1,NIVM2
             DO K=1,LATM2
             DO I=1,LONM2
              PCVRCUMUL2(I,K,L)=PCVRCUMUL2(I,K,L)
     +                         +DTCONV*LCVPFLX2(I,K,L)
              PCVSCUMUL2(I,K,L)=PCVSCUMUL2(I,K,L)
     +                         +DTCONV*SCVPFLX2(I,K,L)
             ENDDO
             ENDDO
             ENDDO
            ENDIF


          ENDIF
          IF (IFLAG3) THEN

            IF (LBDGT) THEN
              CALL BUDGET(DFICVCUMUL3,XPHFM3,QQQM3,+1.,
     +                    LONM3,LATM3,NIVM3,NTRA3)
            ENDIF      
            

                  
            CALL KFBCONV(LONM3,LATM3,NIVM3,NTRA3,NSLS3,LDEB,
     +                   TRANAM3,LSCAVKFB,XKHENRY3,
     +                   XPAM3,TGRIM3,QGRIM3,UGRIM3,VGRIM3,
     +                   WGRIM3,SURFM3,XPHFM3,QQQM3,
     +                   LCVPFLX3,SCVPFLX3,KCVDEB3,KCVFIN3)
            
            IF (LBDGT) THEN
             CALL BUDGET(DFICVCUMUL3,XPHFM3,QQQM3,-1.,
     +                   LONM3,LATM3,NIVM3,NTRA3)
C
             DO L=1,NIVM3
             DO K=1,LATM3
             DO I=1,LONM3
              PCVRCUMUL3(I,K,L)=PCVRCUMUL3(I,K,L)
     +                         +DTCONV*LCVPFLX3(I,K,L)
              PCVSCUMUL3(I,K,L)=PCVSCUMUL3(I,K,L)
     +                         +DTCONV*SCVPFLX3(I,K,L)
             ENDDO
             ENDDO
             ENDDO
            ENDIF


            
          ENDIF
          IF (IFLAG4) THEN
                  
            IF (LBDGT) THEN
             CALL BUDGET(DFICVCUMUL4,XPHFM4,QQQM4,+1.,
     +                    LONM4,LATM4,NIVM4,NTRA4)
            ENDIF
            
            CALL KFBCONV(LONM4,LATM4,NIVM4,NTRA4,NSLS4,LDEB,
     +                   TRANAM4,LSCAVKFB,XKHENRY4,
     +                   XPAM4,TGRIM4,QGRIM4,UGRIM4,VGRIM4,
     +                   WGRIM4,SURFM4,XPHFM4,QQQM4,
     +                   LCVPFLX4,SCVPFLX4,KCVDEB4,KCVFIN4)

            IF (LBDGT) THEN
             CALL BUDGET(DFICVCUMUL4,XPHFM4,QQQM4,-1.,
     +                   LONM4,LATM4,NIVM4,NTRA4)
C
             DO L=1,NIVM4
             DO K=1,LATM4
             DO I=1,LONM4
              PCVRCUMUL4(I,K,L)=PCVRCUMUL4(I,K,L)
     +                      +DTCONV*LCVPFLX4(I,K,L)
              PCVSCUMUL4(I,K,L)=PCVSCUMUL4(I,K,L)
     +                      +DTCONV*SCVPFLX4(I,K,L)
             ENDDO
             ENDDO
             ENDDO
            ENDIF

            
          ENDIF
        ELSE
          print*,'LCVPFLXi not used !'
c         IF (IFLAG1) THEN
c           DO ilon=1,LONM1*LATM1*NIVM1
c             LCVPFLX1(ilon,1,1)=0.
c             SCVPFLX1(ilon,1,1)=0.
c           ENDDO
c         ENDIF
c         IF (IFLAG2) THEN
c           DO ilon=1,LONM2*LATM2*NIVM2
c             LCVPFLX2(ilon,1,1)=0.
c             SCVPFLX2(ilon,1,1)=0.
c           ENDDO
c         ENDIF
c         IF (IFLAG3) THEN
c           DO ilon=1,LONM3*LATM3*NIVM3
c             LCVPFLX3(ilon,1,1)=0.
c             SCVPFLX3(ilon,1,1)=0.
c           ENDDO
c         ENDIF
c         IF (IFLAG4) THEN
c           DO ilon=1,LONM4*LATM4*NIVM4
c             LCVPFLX4(ilon,1,1)=0.
c             SCVPFLX4(ilon,1,1)=0.
c           ENDDO
c         ENDIF
        ENDIF
C
C BELOW-CLOUD SCAVENGING BY CONVECTIVE RAIN/SNOW
C
        IF (LSCAVCONVBC) THEN
          IOPTLESSIV=0
          IF (IFLAG1) THEN
            IF (LBDGT) THEN
              CALL BUDGET(DFBCVCUMUL1,XPHFM1,QQQM1,+1.,
     +                      LONM1,LATM1,NIVM1,NTRA1)
            ENDIF  
                  
            CALL LESSIV_PREC
     +              (IOPTLESSIV,
     +               LONM1,LATM1,NIVM1,NTRA1,SURFM1,
     +               LCVPFLX1,SCVPFLX1,
     +               XPHFM1,XPAM1,TGRIM1,QLIQ1,QICE1,
     +               XKHENRY1,TRANAM1,QQQM1)
            IF (LBDGT) THEN
              CALL BUDGET(DFBCVCUMUL1,XPHFM1,QQQM1,-1.,
     +                    LONM1,LATM1,NIVM1,NTRA1)
            ENDIF
            
          ENDIF
          IF (IFLAG2) THEN
                  
            IF (LBDGT) THEN
              CALL BUDGET(DFBCVCUMUL2,XPHFM2,QQQM2,+1.,
     +                      LONM2,LATM2,NIVM2,NTRA2)
            ENDIF   
            
                  
            CALL LESSIV_PREC
     +              (IOPTLESSIV,
     +               LONM2,LATM2,NIVM2,NTRA2,SURFM2,
     +               LCVPFLX2,SCVPFLX2,
     +               XPHFM2,XPAM2,TGRIM2,QLIQ2,QICE2,
     +               XKHENRY2,TRANAM2,QQQM2)
            IF (LBDGT) THEN
              CALL BUDGET(DFBCVCUMUL2,XPHFM2,QQQM2,-1.,
     +                      LONM2,LATM2,NIVM2,NTRA2)
            ENDIF
            
          ENDIF
          IF (IFLAG3) THEN
                  
            IF (LBDGT) THEN
              CALL BUDGET(DFBCVCUMUL3,XPHFM3,QQQM3,+1.,
     +                      LONM3,LATM3,NIVM3,NTRA3)
            ENDIF
                  
            CALL LESSIV_PREC
     +                (IOPTLESSIV,
     +               LONM3,LATM3,NIVM3,NTRA3,SURFM3,
     +               LCVPFLX3,SCVPFLX3,
     +               XPHFM3,XPAM3,TGRIM3,QLIQ3,QICE3,
     +               XKHENRY3,TRANAM3,QQQM3)
            IF (LBDGT) THEN
              CALL BUDGET(DFBCVCUMUL3,XPHFM3,QQQM3,-1.,
     +                      LONM3,LATM3,NIVM3,NTRA3)
            ENDIF
            
          ENDIF
          IF (IFLAG4) THEN
            IF (LBDGT) THEN
              CALL BUDGET(DFBCVCUMUL4,XPHFM4,QQQM4,+1.,
     +                    LONM4,LATM4,NIVM4,NTRA4)
            ENDIF
                  
            CALL LESSIV_PREC
     +              (IOPTLESSIV,
     +               LONM4,LATM4,NIVM4,NTRA4,SURFM4,
     +               LCVPFLX4,SCVPFLX4,
     +               XPHFM4,XPAM4,TGRIM4,QLIQ4,QICE4,
     +               XKHENRY4,TRANAM4,QQQM4)       
           IF (LBDGT) THEN
              CALL BUDGET(DFBCVCUMUL4,XPHFM4,QQQM4,-1.,
     +                      LONM4,LATM4,NIVM4,NTRA4)
           ENDIF
            
          ENDIF
        ENDIF
C
      ENDIF
C
CCCCC
C     ___________________
C    |                   |
C    | 3bis. CLOUDINESS  |
C    |___________________|
C
CCCCC
C
      IF (CLOUD) THEN
C
C INDEX FOR CLOUDINESS
C
        NIVDEB = 1
        DO iniv=1,NIVM1
          ZPAM=0.
          DO ilon=1,LONM1*LATM1
            ZPAM=ZPAM+XPAM1(ilon,1,iniv)/REAL(LONM1*LATM1)
          ENDDO
          IF (ZPAM.LE.1500.) NIVDEB = iniv
        ENDDO
C
        WRITE(*,*) '> CLOUDINESS'
C
        IF (IFLAG1) THEN
          CALL NEBUL(TGRIM1,QGRIM1,XPAM1,XPHFM1,LCVPFLX1,SCVPFLX1,
     +               FCC1,FCCV1,QLIQ1,QICE1,
     +               DT,LONM1,LATM1,NIVDEB,NIVM1)
        ENDIF
        IF (IFLAG2) THEN
          CALL NEBUL(TGRIM2,QGRIM2,XPAM2,XPHFM2,LCVPFLX2,SCVPFLX2,
     +               FCC2,FCCV2,QLIQ2,QICE2,
     +               DT,LONM2,LATM2,NIVDEB,NIVM2)
        ENDIF
        IF (IFLAG3) THEN
          CALL NEBUL(TGRIM3,QGRIM3,XPAM3,XPHFM3,LCVPFLX3,SCVPFLX3,
     +               FCC3,FCCV3,QLIQ3,QICE3,
     +               DT,LONM3,LATM3,NIVDEB,NIVM3)
        ENDIF
        IF (IFLAG4) THEN
          CALL NEBUL(TGRIM4,QGRIM4,XPAM4,XPHFM4,LCVPFLX4,SCVPFLX4,
     +               FCC4,FCCV4,QLIQ4,QICE4,
     +               DT,LONM4,LATM4,NIVDEB,NIVM4)
        ENDIF
      ELSE
        IF (IFLAG1) THEN
          DO ilon=1,LONM1*LATM1*NIVM1
            FCC1 (ilon,1,1)=0.
            FCCV1(ilon,1,1)=0.
            QLIQ1(ilon,1,1)=0.
            QICE1(ilon,1,1)=0.
          ENDDO
        ENDIF
        IF (IFLAG2) THEN
          DO ilon=1,LONM2*LATM2*NIVM2
            FCC2 (ilon,1,1)=0.
            FCCV2(ilon,1,1)=0.
            QLIQ2(ilon,1,1)=0.
            QICE2(ilon,1,1)=0.
          ENDDO
        ENDIF
        IF (IFLAG3) THEN
          DO ilon=1,LONM3*LATM3*NIVM3
            FCC3 (ilon,1,1)=0.
            FCCV3(ilon,1,1)=0.
            QLIQ3(ilon,1,1)=0.
            QICE3(ilon,1,1)=0.
          ENDDO
        ENDIF
        IF (IFLAG4) THEN
          DO ilon=1,LONM4*LATM4*NIVM4
            FCC4 (ilon,1,1)=0.
            FCCV4(ilon,1,1)=0.
            QLIQ4(ilon,1,1)=0.
            QICE4(ilon,1,1)=0.
          ENDDO
        ENDIF
      ENDIF
C
CCCCC
C     _____________________
C    |                     |
C    | 3ter. WATER VAPOUR  |
C    |_____________________|
C
CCCCC
C
      IF (iH2O.NE.0) THEN
C
C UPDATE TROPOSPHERIC WATER VAPOUR MIXING RATIO ACCORDING TO
C METEOROLOGICAL ANALYSES
C
        WRITE(*,*) '> UPDATING TROPOSPHERIC WATER VAPOUR'
C
        IF (IFLAG1) CALL ANAWAT(QQQM1,QGRIM1,TGRIM1,XPAM1,
     +                          LONM1,LATM1,NIVM1,NTRA1,iH2O)
        IF (IFLAG2) CALL ANAWAT(QQQM2,QGRIM2,TGRIM2,XPAM2,
     +                          LONM2,LATM2,NIVM2,NTRA2,iH2O)
        IF (IFLAG3) CALL ANAWAT(QQQM3,QGRIM3,TGRIM3,XPAM3,
     +                          LONM3,LATM3,NIVM3,NTRA3,iH2O)
        IF (IFLAG4) CALL ANAWAT(QQQM4,QGRIM4,TGRIM4,XPAM4,
     +                          LONM4,LATM4,NIVM4,NTRA4,iH2O)
      ENDIF
      IF ((iH2O.NE.0).OR.(iCO.NE.0)) THEN
#ifdef MIDATM
         DTH2O=3600.
         IF (IFLAG1) THEN
           CALL TOPWAT (DTH2O,QQQM1,
     +                  LONM1,LATM1,NIVM1,NTRA1,iH2O,iCO)
         ENDIF
         IF (IFLAG2) THEN
           CALL TOPWAT (DTH2O,QQQM2,
     +                  LONM2,LATM2,NIVM2,NTRA2,iH2O,iCO)
         ENDIF
         IF (IFLAG3) THEN
           CALL TOPWAT (DTH2O,QQQM3,
     +                  LONM3,LATM3,NIVM3,NTRA3,iH2O,iCO)
         ENDIF
         IF (IFLAG4) THEN
           CALL TOPWAT (DTH2O,QQQM4,
     +                  LONM4,LATM4,NIVM4,NTRA4,iH2O,iCO)
         ENDIF
#endif
      ENDIF
C
CCCCC
C     ____________________________________________
C    |                                            |
C    | 3quat. SCAVENGING BY STRATIFORM RAIN/SNOW  |
C    |____________________________________________|
C
CCCCC
C
      IF (SURF) THEN
C
      IF ((mod(NSECCY,NSCY/NITERT).EQ.0) 
     +     .AND.(LSCAVSTRATI)) THEN
        LNEIGE=.TRUE.
C
        IF (IFLAG1) THEN
          CALL FLPRECSTRA(LONM1,LATM1,NIVM1
     +                   ,XPHFM1,XPAM1
     +                   ,QGRIM1,TGRIM1
     +                   ,LNEIGE
     +                   ,PWAL1,PSNL1
     +                   ,LSTPFLX1,SSTPFLX1)
        ENDIF
        IF (IFLAG2) THEN
          CALL FLPRECSTRA(LONM2,LATM2,NIVM2
     +                   ,XPHFM2,XPAM2
     +                   ,QGRIM2,TGRIM2
     +                   ,LNEIGE
     +                   ,PWAL2,PSNL2
     +                   ,LSTPFLX2,SSTPFLX2)
        ENDIF
        IF (IFLAG3) THEN
          CALL FLPRECSTRA(LONM3,LATM3,NIVM3
     +                   ,XPHFM3,XPAM3
     +                   ,QGRIM3,TGRIM3
     +                   ,LNEIGE
     +                   ,PWAL3,PSNL3
     +                   ,LSTPFLX3,SSTPFLX3)
        ENDIF
        IF (IFLAG4) THEN
          CALL FLPRECSTRA(LONM4,LATM4,NIVM4
     +                   ,XPHFM4,XPAM4
     +                   ,QGRIM4,TGRIM4
     +                   ,LNEIGE
     +                   ,PWAL4,PSNL4
     +                   ,LSTPFLX4,SSTPFLX4)
        ENDIF
      ENDIF
      
      IF (LSCAVSTRATI) THEN
C
        IOPTLESSIV=1
C
        IF (IFLAG1) THEN
         IF (LBDGT) THEN
          CALL BUDGET(DFSTRCUMUL1,XPHFM1,QQQM1,+1.,
     +                LONM1,LATM1,NIVM1,NTRA1)
         ENDIF
                
          CALL LESSIV_PREC
     +            (IOPTLESSIV,
     +             LONM1,LATM1,NIVM1,NTRA1,SURFM1,
     +             LSTPFLX1,SSTPFLX1,
     +             XPHFM1,XPAM1,TGRIM1,QLIQ1,QICE1,
     +             XKHENRY1,TRANAM1,QQQM1)
          IF (LBDGT) THEN
            CALL BUDGET(DFSTRCUMUL1,XPHFM1,QQQM1,-1.,
     +                 LONM1,LATM1,NIVM1,NTRA1)

          ENDIF

          
        ENDIF
        IF (IFLAG2) THEN
          IF (LBDGT) THEN
           CALL BUDGET(DFSTRCUMUL2,XPHFM2,QQQM2,+1.,
     +                     LONM2,LATM2,NIVM2,NTRA2)
          ENDIF
                
          CALL LESSIV_PREC
     +            (IOPTLESSIV,
     +             LONM2,LATM2,NIVM2,NTRA2,SURFM2,
     +             LSTPFLX2,SSTPFLX2,
     +             XPHFM2,XPAM2,TGRIM2,QLIQ2,QICE2,
     +             XKHENRY2,TRANAM2,QQQM2)
         IF (LBDGT) THEN
           CALL BUDGET(DFSTRCUMUL2,XPHFM2,QQQM2,-1.,
     +                 LONM2,LATM2,NIVM2,NTRA2)
         ENDIF
          
        ENDIF
        IF (IFLAG3) THEN
         IF (LBDGT) THEN
             CALL BUDGET(DFSTRCUMUL3,XPHFM3,QQQM3,+1.,
     +                    LONM3,LATM3,NIVM3,NTRA3)
          ENDIF    
          CALL LESSIV_PREC
     +            (IOPTLESSIV,
     +             LONM3,LATM3,NIVM3,NTRA3,SURFM3,
     +             LSTPFLX3,SSTPFLX3,
     +             XPHFM3,XPAM3,TGRIM3,QLIQ3,QICE3,
     +             XKHENRY3,TRANAM3,QQQM3)
          IF (LBDGT) THEN
              CALL BUDGET(DFSTRCUMUL3,XPHFM3,QQQM3,-1.,
     +                    LONM3,LATM3,NIVM3,NTRA3)
           ENDIF
          
          
        ENDIF
        IF (IFLAG4) THEN
          IF (LBDGT) THEN
            CALL BUDGET(DFSTRCUMUL4,XPHFM4,QQQM4,+1.,
     +                    LONM4,LATM4,NIVM4,NTRA4)
          ENDIF 

          CALL LESSIV_PREC
     +            (IOPTLESSIV,
     +             LONM4,LATM4,NIVM4,NTRA4,SURFM4,
     +             LSTPFLX4,SSTPFLX4,
     +             XPHFM4,XPAM4,TGRIM4,QLIQ4,QICE4,
     +             XKHENRY4,TRANAM4,QQQM4) 
          IF (LBDGT) THEN
           CALL BUDGET(DFSTRCUMUL4,XPHFM4,QQQM4,-1.,
     +                    LONM4,LATM4,NIVM4,NTRA4)
          ENDIF

          
          ENDIF
       ENDIF
C
       ENDIF
C
C CROSSROADS
C
      GOTO (400,400),IORDER
C
CCCCC
C     _______________
C    |               |
C    | 4. CHEMISTRY  |
C    |_______________|
C
CCCCC
C
 400  IF (CHEMSCHEME.EQ.'TRACERS0') THEN
C
        WRITE(*,*) '> RN-222 AND PB'
        TAUX=5.4*24.*3600.
        TXDEC_RAD =1.- EXP(-1.*DT/TAUX)
        IF (IFLAG1) THEN
          DO ilon=1,LOLANIM1
            DELTAQ= QQQM1(ilon,1,1,1)*TXDEC_RAD
            QQQM1(ilon,1,1,1)=QQQM1(ilon,1,1,1)
     +                       -DELTAQ
            QQQM1(ilon,1,1,2)=QQQM1(ilon,1,1,2)
     +                       +DELTAQ
          ENDDO
        ENDIF
        IF (IFLAG2) THEN
          DO ilon=1,LOLANIM2
            DELTAQ= QQQM2(ilon,1,1,1)*TXDEC_RAD
            QQQM2(ilon,1,1,1)=QQQM2(ilon,1,1,1)
     +                       -DELTAQ
            QQQM2(ilon,1,1,2)=QQQM2(ilon,1,1,2)
     +                       +DELTAQ
          ENDDO
        ENDIF
        IF (IFLAG3) THEN
          DO ilon=1,LOLANIM3
            DELTAQ= QQQM3(ilon,1,1,1)*TXDEC_RAD
            QQQM3(ilon,1,1,1)=QQQM3(ilon,1,1,1)
     +                       -DELTAQ
            QQQM3(ilon,1,1,2)=QQQM3(ilon,1,1,2)
     +                       +DELTAQ
          ENDDO
        ENDIF
        IF (IFLAG4) THEN
          DO ilon=1,LOLANIM4
           DELTAQ= QQQM4(ilon,1,1,1)*TXDEC_RAD
            QQQM4(ilon,1,1,1)=QQQM4(ilon,1,1,1)
     +                       -DELTAQ
            QQQM4(ilon,1,1,2)=QQQM4(ilon,1,1,2)
     +                       +DELTAQ
          ENDDO
        ENDIF
C
       ELSEIF (CHEMSCHEME.EQ.'ACCIDENT') THEN
        DO isrc=1,NSRC
          PRINT*,'HALFLIFE',isrc,')=',HALFLIFE(isrc)
          IF (HALFLIFE(isrc).GT.1.E+9) THEN
C
C IF HALF-LIFE IS OVER 1.E+9s, THE TRACER IS CONSIDERED
C PASSIVE.
C
            WRITE(*,*) '> ',POLNAME(isrc),
     +                 ' IS CONSIDERED PASSIVE'
          ELSE
            WRITE(*,*) '> ',POLNAME(isrc),
     +                 ' DECAYS WITH A HALF-LIFE OF ',
     +                 HALFLIFE(isrc),' (s)'
            TAUX=HALFLIFE(isrc)/LOG(2.)
            TXDEC =EXP(-1.*DT/TAUX)
            IF (IFLAG1) THEN
              QQQM1(:,:,:,isrc)=QQQM1(:,:,:,isrc)*TXDEC      
            ENDIF
            IF (IFLAG2) THEN
             QQQM2(:,:,:,isrc)=QQQM2(:,:,:,isrc)*TXDEC             
            ENDIF
            IF (IFLAG3) THEN
             QQQM3(:,:,:,isrc)=QQQM3(:,:,:,isrc)*TXDEC     
            ENDIF            
            IF (IFLAG4) THEN
             QQQM4(:,:,:,isrc)=QQQM4(:,:,:,isrc)*TXDEC          
            ENDIF
          ENDIF
       ENDDO     

      ELSEIF ((CHEMSCHEME.EQ.'TRACERCO').OR.
     +        (CHEMSCHEME.EQ.'AEROSOLS')) THEN
        WRITE(*,*) '> TRACERS ARE PASSIVE'
C
#ifdef CARIOLLE
      ELSEIF (CHEMSCHEME.EQ.'CARIOLLE') THEN
C
        WRITE(*,*) '> CARIOLLE CHEMISTRY (GLOBE ONLY)'
C
        IF (IFLAG1) THEN
#ifdef LINO_3
#ifdef MIDATM
          WRITE(*,*) '> NO TROPO. CHEMISTRY : RELAXATION INSTEAD!'
          CALL CARIORELAX(LONM1,LATM1,NIVM1,NTRA1,DT,
     +                    XPAM1,QQQM1,TRANAM1)
#endif
#endif
          CALL CARIOCHEM(LONM1,LATM1,NIVM1,NTRA1,SOLID,
#ifdef LINO_3
     +                   NCARIO3,CARO3PARM1,
#endif
#ifdef LINCO
     +                   NCARIOCO,CARCOPARM1,
#endif
#ifdef LINHNO_3
     +                   NCARIOHNO3,CARHNO3PARM1,
#endif
     +                   RLONM1,RLATM1,XSDAY,XDAY,DT,
     +                   TRANAM1,TGRIM1,XPAM1,XPHFM1,
     +                   QQQM1,TRANSINV)
        ENDIF
C
#else
      ELSE
#ifndef ACCIDENT              
C
#ifndef SBL
         WRITE(*,*) '> CHEMISTRY'
#else
         WRITE(*,*) '> CHEMISTRY BETWEEN LEVELS 1 AND ',NIVBL
#endif
C
C RELAXATION FOR STRATOSPHERIC ONLY CHEMICAL SCHEMES
C
        IF ((CHEMSCHEME.EQ.'REPROBUS').OR.
     +      (CHEMSCHEME.EQ.'REPROBU2')) THEN
          WRITE(*,*) '> NO TROPO. CHEMISTRY : RELAXATION INSTEAD!'
          IF (IFLAG1) THEN
            CALL RELAX(LONM1,LATM1,NIVM1,NTRA1,NSLS1,DT,
     +                 XPAM1,QQQM1,HHHM1,TRANAM1,SLSNAM1)
          ENDIF
          IF (IFLAG2) THEN
            CALL RELAX(LONM2,LATM2,NIVM2,NTRA2,NSLS2,DT,
     +                 XPAM2,QQQM2,HHHM2,TRANAM2,SLSNAM2)
          ENDIF
          IF (IFLAG3) THEN
            CALL RELAX(LONM3,LATM3,NIVM3,NTRA3,NSLS3,DT,
     +                 XPAM3,QQQM3,HHHM3,TRANAM3,SLSNAM3)
          ENDIF
          IF (IFLAG4) THEN
            CALL RELAX(LONM4,LATM4,NIVM4,NTRA4,NSLS4,DT,
     +                 XPAM4,QQQM4,HHHM4,TRANAM4,SLSNAM4)
          ENDIF
        ENDIF
C
C CHEMISTRY
C
        IF (IFLAG1) THEN
#ifdef MPI
          I_DEB(MYID)=MYID*(LONM1*LATM1/NPROCS)+1
          IF (MYID.NE.(NPROCS-1)) THEN
            I_FIN(MYID)=I_DEB(MYID)+(LONM1*LATM1/NPROCS)-1
          ELSE
            I_FIN(MYID)=LONM1*LATM1
          ENDIF
#endif
          
#ifdef MASSy 
#ifndef RACMOBUS 
C
C RELAXATION FOR NON-EMITTED STRATOSPHERIC SPECIES
C
          CALL RELAX (LONM1,LATM1,NIVM1,NTRA1,NSLS1,DT,
     +                XPAM1,QQQM1,HHHM1,TRANAM1,SLSNAM1)
#endif
#endif
          CALL CHEM (LONM1,LATM1,NIVM1,NIVBL,NTRA1,NSLS1,LONM1*LATM1,
     +               RLONM1,RLATM1,SOLID,CLOUD,XSDAY,XDAY,DT,
     +               TGRIM1,XPAM1,QQQM1,HHHM1,XALBSM1,TRANAM1,
     +               SLSNAM1,XPHFM1,FCC1,QLIQ1,TJM1,ICALPHOT)
#ifdef MASSy
          CALL CH_CONSERV (QQQM1,HHHM1,
     +                     LONM1,LATM1,NIVM1,NIVM1,NTRA1,NSLS1)
#endif
C
        ENDIF
        IF (IFLAG2) THEN
#ifdef MPI
          I_DEB(MYID)=MYID*(LONM2*LATM2/NPROCS)+1
          IF (MYID.NE.(NPROCS-1)) THEN
            I_FIN(MYID)=I_DEB(MYID)+(LONM2*LATM2/NPROCS)-1
          ELSE
            I_FIN(MYID)=LONM2*LATM2
          ENDIF
#endif
#ifdef MASSy
#ifndef RACMOBUS
          CALL RELAX (LONM2,LATM2,NIVM2,NTRA2,NSLS2,DT,
     +               XPAM2,QQQM2,HHHM2,TRANAM2,SLSNAM2)
#endif
#endif
          CALL CHEM (LONM2,LATM2,NIVM2,NIVBL,NTRA2,NSLS2,LONM2*LATM2,
     +               RLONM2,RLATM2,SOLID,CLOUD,XSDAY,XDAY,DT,
     +               TGRIM2,XPAM2,QQQM2,HHHM2,XALBSM2,TRANAM2,
     +               SLSNAM2,XPHFM2,FCC2,QLIQ2,TJM2,ICALPHOT)
#ifdef MASSy
          CALL CH_CONSERV (QQQM2,HHHM2,
     +                     LONM2,LATM2,NIVM2,NIVM2,NTRA2,NSLS2)
#endif
        ENDIF
        IF (IFLAG3) THEN
#ifdef MPI
          I_DEB(MYID)=MYID*(LONM3*LATM3/NPROCS)+1
          IF (MYID.NE.(NPROCS-1)) THEN
            I_FIN(MYID)=I_DEB(MYID)+(LONM3*LATM3/NPROCS)-1
          ELSE
            I_FIN(MYID)=LONM3*LATM3
          ENDIF
#endif
#ifdef MASSy
#ifndef RACMOBUS
          CALL RELAX (LONM3,LATM3,NIVM3,NTRA3,NSLS3,DT,
     +               XPAM3,QQQM3,HHHM3,TRANAM3,SLSNAM3)
#endif
#endif
          CALL CHEM (LONM3,LATM3,NIVM3,NIVBL,NTRA3,NSLS3,LONM3*LATM3,
     +               RLONM3,RLATM3,SOLID,CLOUD,XSDAY,XDAY,DT,
     +               TGRIM3,XPAM3,QQQM3,HHHM3,XALBSM3,TRANAM3,
     +               SLSNAM3,XPHFM3,FCC3,QLIQ3,TJM3,ICALPHOT)
#ifdef MASSy
          CALL CH_CONSERV (QQQM3,HHHM3,
     +                     LONM3,LATM3,NIVM3,NIVM3,NTRA3,NSLS3)
#endif
        ENDIF
        IF (IFLAG4) THEN
#ifdef MPI
          I_DEB(MYID)=MYID*(LONM4*LATM4/NPROCS)+1
          IF (MYID.NE.(NPROCS-1)) THEN
            I_FIN(MYID)=I_DEB(MYID)+(LONM4*LATM4/NPROCS)-1
          ELSE
            I_FIN(MYID)=LONM4*LATM4
          ENDIF
#endif	    
#ifdef MASSy
#ifndef RACMOBUS
          CALL RELAX (LONM4,LATM4,NIVM4,NTRA4,NSLS4,DT,
     +               XPAM4,QQQM4,HHHM4,TRANAM4,SLSNAM4)
#endif
#endif
          CALL CHEM (LONM4,LATM4,NIVM4,NIVBL,NTRA4,NSLS4,LONM4*LATM4,
     +               RLONM4,RLATM4,SOLID,CLOUD,XSDAY,XDAY,DT,
     +               TGRIM4,XPAM4,QQQM4,HHHM4,XALBSM4,TRANAM4,
     +               SLSNAM4,XPHFM4,FCC4,QLIQ4,TJM4,ICALPHOT)
#ifdef MASSy
          CALL CH_CONSERV (QQQM4,HHHM4,
     +                     LONM4,LATM4,NIVM4,NIVM4,NTRA4,NSLS4)
#endif
        ENDIF
C
#endif 
#endif       
      ENDIF
C
C CROSSROADS
C
      GOTO (500,200),IORDER
C
CCCCC
C     ____________________
C    |                    |
C    | 5. DRY DEPOSITION  |
C    |____________________|
C
C C(t+DT)+Vd*[...]*C(t+DT)=C(t)
C
CCCCC
C
  
 500    IF (SURF) THEN
C
        WRITE(*,*) '> DRY DEPOSITION'
C
CCC GLOBAL DOMAIN
C
        IF (IFLAG1) THEN
          
          IF (CHEMSCHEME.EQ.'ACCIDENT') THEN
                  
           IQMDEP1(:)=1
           DO ISRC=1,NSRC
            VD1(:,:, ISRC)=VDEPOSIT(ISRC)
           ENDDO   
           
          ELSE

           VD1(:,:,:)=     xp *VD1D(:,:,:,ihour  )
     +               +(1.-xp)*VD1D(:,:,:,ihour+1)
    
          ENDIF
          
          IF (LBDGT) THEN
            CALL BUDGET(DFDRYCUMUL1,XPHFM1,QQQM1,+1.,
     +                    LONM1,LATM1,NIVM1,NTRA1)
          ENDIF

          CALL DRYDEP(LONM1,LATM1,NIVM1,NTRA1,NSLS1,NDEP1,
     +                TRANAM1,SLSNAM1,
     +                DT,XPAM1,XPHFM1,TGRIM1,
     +                IQMDEP1,IHMDEP1,VD1,QQQM1,HHHM1)

         IF (LBDGT) THEN
          CALL BUDGET(DFDRYCUMUL1,XPHFM1,QQQM1,-1.,
     +                    LONM1,LATM1,NIVM1,NTRA1)
         ENDIF
        ENDIF
C
CCC CONTINENTAL DOMAIN
C
        IF (IFLAG2) THEN

         IF (CHEMSCHEME.EQ.'ACCIDENT') THEN
                 
           IQMDEP2(:)=1
           DO ISRC=1,NSRC
            VD2(:,:, ISRC)=VDEPOSIT(ISRC)
           ENDDO   

          ELSE

           VD2(:,:,:)=     xp *VD2D(:,:,:,ihour  )
     +               +(1.-xp)*VD2D(:,:,:,ihour+1)
       
           
          ENDIF
          
          IF (LBDGT) THEN
            CALL BUDGET(DFDRYCUMUL2,XPHFM2,QQQM2,+1.,
     +                    LONM2,LATM2,NIVM2,NTRA2)
          ENDIF


          CALL DRYDEP(LONM2,LATM2,NIVM2,NTRA2,NSLS2,NDEP2,
     +                TRANAM2,SLSNAM2,
     +                DT,XPAM2,XPHFM2,TGRIM2,
     +                IQMDEP2,IHMDEP2,VD2,QQQM2,HHHM2)

          IF (LBDGT) THEN
            CALL BUDGET(DFDRYCUMUL2,XPHFM2,QQQM2,-1.,
     +                    LONM2,LATM2,NIVM2,NTRA2)
          ENDIF

        ENDIF
C
CCC REGIONAL DOMAIN
C
        IF (IFLAG3) THEN

          IF (CHEMSCHEME.EQ.'ACCIDENT') THEN

           IQMDEP3(:)=1
           DO ISRC=1,NSRC
            VD3(:,:, ISRC)=VDEPOSIT(ISRC)
           ENDDO      
           
          ELSE

           VD3(:,:,:)=     xp *VD3D(:,:,:,ihour  )
     +               +(1.-xp)*VD3D(:,:,:,ihour+1)

          ENDIF

          IF (LBDGT) THEN
            CALL BUDGET(DFDRYCUMUL3,XPHFM3,QQQM3,+1.,
     +                    LONM3,LATM3,NIVM3,NTRA3)
          ENDIF

          
          CALL DRYDEP(LONM3,LATM3,NIVM3,NTRA3,NSLS3,NDEP3,
     +                TRANAM3,SLSNAM3,
     +                DT,XPAM3,XPHFM3,TGRIM3,
     +                IQMDEP3,IHMDEP3,VD3,QQQM3,HHHM3)

           IF (LBDGT) THEN
            CALL BUDGET(DFDRYCUMUL3,XPHFM3,QQQM3,-1.,
     +                    LONM3,LATM3,NIVM3,NTRA3)
          ENDIF

        ENDIF
C
CCC ZOOM DOMAIN
C
        IF (IFLAG4) THEN

         IF (CHEMSCHEME.EQ.'ACCIDENT') THEN
           IQMDEP4(:)=1
           DO ISRC=1,NSRC
            VD4(:,:, ISRC)=VDEPOSIT(ISRC)
           ENDDO  

         ELSE

          VD4(:,:,:)=     xp *VD4D(:,:,:,ihour  )
     +               +(1.-xp)*VD4D(:,:,:,ihour+1)

         ENDIF

          IF (LBDGT) THEN
            CALL BUDGET(DFDRYCUMUL4,XPHFM4,QQQM4,+1.,
     +                    LONM4,LATM4,NIVM4,NTRA4)
          ENDIF


          CALL DRYDEP(LONM4,LATM4,NIVM4,NTRA4,NSLS4,NDEP4,
     +                TRANAM4,SLSNAM4,
     +                DT,XPAM4,XPHFM4,TGRIM4,
     +                IQMDEP4,IHMDEP4,VD4,QQQM4,HHHM4)
          IF (LBDGT) THEN
            CALL BUDGET(DFDRYCUMUL4,XPHFM4,QQQM4,-1.,
     +                  LONM4,LATM4,NIVM4,NTRA4)
          ENDIF

          
        ENDIF
C        
      ENDIF
C
C CROSSROADS
C
      GOTO (600,400),IORDER
C
CCCCC
C
 600  IORDEROLD=IORDER
      IF (IORDEROLD.EQ.1) IORDER=2
      IF (IORDEROLD.EQ.2) IORDER=1
C
CCCC
C     ____________________
C    |                    |
C    | 6. UPSCALE NESTING |
C    |____________________|
C
CCCC
C
      LGRID12=.TRUE.
      LGRID23=.TRUE.
      LGRID34=.TRUE.
      TRELAX=REAL(6*3600)
C
C M4 -> M3
C
      IF (IFLAG4.AND.LGRID34)THEN
        CALL HITLORES(LONM3,LATM3,LONM4,LATM4
     +               ,NIVM3,NTRA3,DT
     +               ,TRELAX,IX34,IY34,IGRID34
     +               ,DRLONM3,DRLATM3,DRLONM4,DRLATM4
     +               ,SURFM4
     +               ,QQQM3,QQQM4)
      ENDIF
C
C M3 -> M2
C
      IF (IFLAG3.AND.LGRID23)THEN
        CALL HITLORES(LONM2,LATM2,LONM3,LATM3
     +               ,NIVM2,NTRA2,DT
     +               ,TRELAX,IX23,IY23,IGRID23
     +               ,DRLONM2,DRLATM2,DRLONM3,DRLATM3
     +               ,SURFM3
     +               ,QQQM2,QQQM3)
      ENDIF
C
C M2 -> M1
C
      IF (IFLAG2.AND.LGRID12)THEN
        CALL HITLORES(LONM1,LATM1,LONM2,LATM2
     +               ,NIVM1,NTRA1,DT
     +               ,TRELAX,IX12,IY12,IGRID12
     +               ,DRLONM1,DRLATM1,DRLONM2,DRLATM2
     +               ,SURFM2
     +               ,QQQM1,QQQM2)
      ENDIF
C
      DEALLOCATE (DATEMPO)
      DEALLOCATE (IDATE)
      DEALLOCATE (EV1)
      DEALLOCATE (EV2)
      DEALLOCATE (EV3)
      DEALLOCATE (EV4)
      DEALLOCATE (XALBSM1)
      DEALLOCATE (XALBSM2)
      DEALLOCATE (XALBSM3)
      DEALLOCATE (XALBSM4)
      DEALLOCATE (PWAL1)
      DEALLOCATE (PWAL2)
      DEALLOCATE (PWAL3)
      DEALLOCATE (PWAL4)
      DEALLOCATE (PWAC1)
      DEALLOCATE (PWAC2)
      DEALLOCATE (PWAC3)
      DEALLOCATE (PWAC4)
      DEALLOCATE (PSNL1)
      DEALLOCATE (PSNL2)
      DEALLOCATE (PSNL3)
      DEALLOCATE (PSNL4)
      DEALLOCATE (PSNC1)
      DEALLOCATE (PSNC2)
      DEALLOCATE (PSNC3)
      DEALLOCATE (PSNC4)
      DEALLOCATE (EM1)
      DEALLOCATE (EM2)
      DEALLOCATE (EM3)
      DEALLOCATE (EM4)
      DEALLOCATE (VD1)
      DEALLOCATE (VD2)
      DEALLOCATE (VD3)
      DEALLOCATE (VD4)
#ifdef MASSy
      DEALLOCATE (NOX1)
      DEALLOCATE (NOX2)
      DEALLOCATE (NOX3)
      DEALLOCATE (NOX4)
#endif
C
      RETURN
      END
C
CCC
C
