      SUBROUTINE KALMAN(daynum,idate0)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Data assimilation
C   Purpose    : Interface with NCAR Kalman filter
C   Called by  :
C
CCC
C
C  Original : J.-F. Lamarque (NCAR), P. Bechtold (LA)
C             and V.-H. Peuch, 12/2001
C
C  Modified :
C
CCC
C
	USE PARADI_MOD
	USE EXPE_MOD
	USE GRIDS_MOD
	USE MOCF_MOD
	USE MRAT_MOD
	USE FORMT_MOD
C
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	REAL (KIND=8) :: DAYNUM
	INTEGER , DIMENSION (5) ::IDATE0
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: NBDAY,NBHOUR,ICO,IERRCO,ITRA
      INTEGER , ALLOCATABLE :: nbjour(:),idate1(:)
C
	ALLOCATE (nbjour(12))
	ALLOCATE (idate1(5))
C
      nbjour =(/31,28,31,30,31,30,31,31,30,31,30,31/)

C	
C 1. DETERMINE CURRENT DATE
C
C idate1(1) = YEAR 
C idate1(2) = MONTH
C idate1(3) = DAY
C idate1(4) = HOUR
C idate1(5) = MINUTE (not used)
C
      NBDAY=int(daynum/86400.)
      NBHOUR=int((daynum-real(NBDAY)*86400.)/3600.)
      idate1(5)=0
      idate1(4)=idate0(4)+NBHOUR
      idate1(3)=idate0(3)+NBDAY
      idate1(2)=idate0(2)
      idate1(1)=idate0(1)
C
      IF (idate1(4).GE.24) THEN
        idate1(4)=idate1(4)-24
        idate1(3)=idate1(3)+1
      ENDIF
      IF (mod(idate0(1),4).EQ.0) THEN
        nbjour(2)=nbjour(2)+1
      ENDIF
      IF (idate1(3).GT.nbjour(idate1(2))) THEN
        idate1(3)=idate1(3)-nbjour(idate1(2))
        idate1(2)=idate1(2)+1
      ENDIF
      IF (idate1(2).GT.12) THEN
        idate1(1)=idate1(1)+1
        idate1(2)=1
      ENDIF
C
C 2. FIND INDEX FOR CO AND ITS ERROR
C
      iCO=0
      iERRCO=0
C
      DO itra=1,NTRA1
        IF (TRANAM1(itra).EQ.'CO') iCO=itra
        IF (TRANAM1(itra).EQ.'ERRCO') iERRCO=itra
      ENDDO
C
      IF ((iCO.EQ.0).OR.(iERRCO.EQ.0)) THEN
        print*,'> CANNOT FIND CO OR ERRCO IN CHEMICAL SCHEME : ABORT!'       
        CALL MOCAGE_STOP
      ENDIF
C
C 3. APPLY KALMAN FILTER (ASSIMILATION OF MOPITT CO) 
C
      IF (IFLAG1) THEN
        CALL assim_mopitt(LONM1,LATM1,NIVM1
     &                   ,idate1
     &                   ,RLONM1,RLATM1
     &                   ,PSOLM1,XPHFM1,XPAM1
     &                   ,QQQM1(1:LONM1,1:LATM1,1:NIVM1,iCO)
     &                   ,QQQM1(1:LONM1,1:LATM1,1:NIVM1,iERRCO))
      ENDIF
C
      IF (IFLAG2) THEN
        CALL assim_mopitt(LONM2,LATM2,NIVM2
     &                   ,idate1
     &                   ,RLONM2,RLATM2
     &                   ,PSOLM2,XPHFM2,XPAM2
     &                   ,QQQM2(1:LONM2,1:LATM2,1:NIVM2,iCO)
     &                   ,QQQM2(1:LONM2,1:LATM2,1:NIVM2,iERRCO))
      ENDIF
C
      IF (IFLAG3) THEN
        CALL assim_mopitt(LONM3,LATM3,NIVM3
     &                   ,idate1
     &                   ,RLONM3,RLATM3
     &                   ,PSOLM3,XPHFM3,XPAM3
     &                   ,QQQM3(1:LONM3,1:LATM3,1:NIVM3,iCO)
     &                   ,QQQM3(1:LONM3,1:LATM3,1:NIVM3,iERRCO))
      ENDIF
C
      IF (IFLAG4) THEN
        CALL assim_mopitt(LONM4,LATM4,NIVM4
     &                   ,idate1
     &                   ,RLONM4,RLATM4
     &                   ,PSOLM4,XPHFM4,XPAM4
     &                   ,QQQM4(1:LONM4,1:LATM4,1:NIVM4,iCO)
     &                   ,QQQM4(1:LONM4,1:LATM4,1:NIVM4,iERRCO))
      ENDIF
C
	DEALLOCATE (NBJOUR)
	DEALLOCATE (IDATE1)	
C
C
      RETURN
      END
