      SUBROUTINE KFBCALSUB(J,LON,LAT,NIV,NTRA,
     +                 SURFM,
     +                 PL,PLT,
     +                 U3D,V3D,T3D,
     +                 QQQM,
     +                 IDEB,IFIN,LDEB,LFIN)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Convection and Diffusion (Bechtold/Louis)
C   Purpose   : Compute vertical diffusion coefficients from model
C               fields, using hermite cubics for the vertical
C               interpolation. 
C   Called by : 
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 04/2000
C
C  Modified :
C
CCC
C
C     Externals
C     ---------
C     SUBROUTINES:   KFBCONVMA
C                    KFBLOUIS
C
	USE CSTES_MOD
C
	IMPLICIT NONE
C
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: J,LON,LAT,NIV,NTRA,IDEB,IFIN,LDEB,LFIN
      REAL (KIND=8) , DIMENSION (LON,LAT) :: SURFM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV)   :: PL
	REAL (KIND=8) , DIMENSION (LON,LAT,0:NIV) :: PLT
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV)   :: U3D,V3D,T3D
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV,NTRA) :: QQQM
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: L,I,NC,ITAU,N,M1,I1,J1P1,J1M1,I1P1,I1M1,K,J1,JL
	REAL (KIND=8) :: PLLZ,PLUZ,ALPHA,BETA,CZETA,CZETAK
      REAL (KIND=8) :: AMMIN,AMAXDIFF
      PARAMETER (AMMIN   =1.0E-10)
      PARAMETER (AMAXDIFF=1.0E5)
      REAL (KIND=8) , ALLOCATABLE :: DKGM(:,:),DKG(:,:)      
      REAL (KIND=8) , ALLOCATABLE :: ZDKG(:)
C
C "FULL" MODEL LEVELS
C
      REAL (KIND=8) , ALLOCATABLE :: PRESS(:,:),TTEMP(:,:)
      REAL (KIND=8) , ALLOCATABLE :: UTEMP(:,:),VTEMP(:,:) 
C
C "HALF" LEVELS
C
      REAL (KIND=8) , ALLOCATABLE :: WZ(:,:),WT(:,:)
      REAL (KIND=8) , ALLOCATABLE :: WU(:,:),WV(:,:)
C
      REAL (KIND=8) , ALLOCATABLE :: TMPRESS(:,:)
      REAL (KIND=8) , ALLOCATABLE :: AM(:),AK(:)
      REAL (KIND=8) , ALLOCATABLE :: G(:,:)
      REAL (KIND=8) , ALLOCATABLE :: SM(:,:,:)
C
C 1. INITIALIZATIONS
C
	ALLOCATE (DKGM(LDEB:NIV,IFIN)) ; DKGM(:,:) = 0.0
	ALLOCATE (DKG(LDEB:NIV,IFIN)) ; DKG(:,:) = 0.0
	ALLOCATE (ZDKG(LDEB:NIV)) ; ZDKG(:) = 0.0
	ALLOCATE (PRESS(NIV,IFIN)) ; PRESS(:,:) = 0.0
	ALLOCATE (TTEMP(NIV,IFIN)) ; TTEMP(:,:) = 0.0
	ALLOCATE (UTEMP(NIV,IFIN)) ; UTEMP(:,:) = 0.0
	ALLOCATE (VTEMP(NIV,IFIN)) ; VTEMP(:,:) = 0.0
	ALLOCATE (WZ(0:NIV,IFIN)) ; WZ(:,:) = 0.0
	ALLOCATE (WT(0:NIV,IFIN)) ; WT(:,:) = 0.0
	ALLOCATE (WU(0:NIV,IFIN)) ; WU(:,:) = 0.0
	ALLOCATE (WV(0:NIV,IFIN)) ; WV(:,:) = 0.0
	ALLOCATE (TMPRESS(0:NIV,IFIN)) ; TMPRESS(:,:) = 0.0
	ALLOCATE (AM(LDEB:NIV)) ; AM(:) = 0.0
	ALLOCATE (AK(LDEB-1:NIV)) ; AK(:) = 0.0
	ALLOCATE (G(LDEB:NIV,LDEB:NIV)) ; G(:,:) = 0.0
	ALLOCATE (SM(LON,LAT,NIV)) ; SM(:,:,:) = 0.0
C
      DO L=LDEB,NIV
      DO I=IDEB,IFIN
        DKGM(L,I)=0.
      ENDDO
      ENDDO
C
C 2. LOOP OVER ANALYSIS TIMES TO AVERAGE
C
      NC=0
      DO 10 ITAU=1,1
      DO 100 I=IDEB,IFIN
        N=IFIN*(J-1)+I
        M1=(N-1)/LON
        I1=N-LON*M1
        J1=M1+1
        J1P1=MIN(LAT,J1+1)
        J1M1=MAX(1  ,J1-1)
        I1P1=MIN(LON,I1+1)
        I1M1=MAX(1  ,I1-1)
C
C PRESSURE AT HALF LEVELS
C
        DO 1001 L=0,NIV
          TMPRESS(L,I)=PLT(I1,J1,L)
 1001   CONTINUE
C
        DO 1002 L=1,NIV
          PRESS(L,I)=PL (I1,J1,L)
          TTEMP(L,I)=T3D(I1,J1,L)
          UTEMP(L,I)=U3D(I1,J1,L)
          VTEMP(L,I)=V3D(I1,J1,L)
          SM   (I1,J1,L)=ABS(PLT(I1,J1,L)-PLT(I1,J1,L-1))
     +                  *SURFM(I1,J1)/GG
 1002   CONTINUE
C
C ALTITUDE AT HALF LEVELS
C
        WZ(NIV,I)=RA*TTEMP(NIV,I)/GG*
     +            MAX(0.,ALOG(PREF/TMPRESS(NIV,I)))
     
 100  CONTINUE
C
      DO 101 L=NIV-1,0,-1
      DO 1010 I=IDEB,IFIN
        K=MAX(L,1)
        PLLZ=TMPRESS(L+1,I)
        PLUZ=TMPRESS(K  ,I)
        WZ(L,I)=WZ(L+1,I) + RA*TTEMP(L+1,I)*ALOG(PLLZ/PLUZ)/GG
 1010 CONTINUE
 101  CONTINUE
C
C VARIABLES ONTO HALF LEVES 1 TO NIV-1
C
      DO 102 L=1,NIV-1
      DO 1020 I=IDEB,IFIN
        ALPHA=(TMPRESS(L,I)-PRESS(L,I))/(TMPRESS(L+1,I)-PRESS(L,I))
        BETA=1.-ALPHA
        WU(L,I) =ALPHA*UTEMP(L+1,I)+BETA*UTEMP(L,I)
        WV(L,I) =ALPHA*VTEMP(L+1,I)+BETA*VTEMP(L,I)
        WT(L,I) =ALPHA*TTEMP(L+1,I)+BETA*TTEMP(L,I)
 1020 CONTINUE
 102  CONTINUE
C
C EXTRAPOLATION FOR LEVELS 0 AND NIV
C
      DO 103 I=IDEB,IFIN
C
C a) HALF LEVEL NIV
C
        ALPHA=(TMPRESS(NIV,I)-TMPRESS(NIV-1,I))
     +       /(PRESS(NIV,I)-TMPRESS(NIV-1,I))
        BETA=1.-ALPHA
        WU(NIV,I) =ALPHA*UTEMP(NIV,I)+BETA* WU(NIV-1,I)
        WV(NIV,I) =ALPHA*VTEMP(NIV,I)+BETA* WV(NIV-1,I)
        WT(NIV,I) =ALPHA*TTEMP(NIV,I)+BETA* WT(NIV-1,I)
C         
C b) HALF LEVEL 0
C
        ALPHA=(TMPRESS(1,I)-TMPRESS(0,I))/(TMPRESS(1,I)-PRESS(1,I))
        BETA=1.-ALPHA
        WU(0,I) =ALPHA*UTEMP(1,I)+BETA* WU(1,I)
        WV(0,I) =ALPHA*VTEMP(1,I)+BETA* WV(1,I)
        WT(0,I) =ALPHA*TTEMP(1,I)+BETA* WT(1,I)
C
 103  CONTINUE
C
C COMPUTE DIFFUSION COEFFICIENTS
C
      DO 104 I=IDEB,IFIN     
C
C APPLY LOUIS PARAMETERIZATION
C
        CALL KFBLOUIS(NIV,LDEB,WZ(0,I),WT(0,I)
     +               ,WU(0,I),WV(0,I),ZDKG)
C
C PROJECTION ONTO MODEL LEVELS
C
      DO 4 JL=LDEB,NIV-1
        ZDKG(JL)=AMAX1(0.,ZDKG(JL)*2.0*
     +          (TMPRESS(JL+1,I)-TMPRESS(JL-1,I))
     +         /(GG*((WZ(JL-1,I)-WZ(JL+1,I))**2)))
 4    CONTINUE
      ZDKG(NIV)=0.0
C
C SAVE COEFFICIENTS
C
      DO 1040 L=LDEB,NIV
        DKGM (L,I)=DKGM (L,I) + ZDKG (L)
 1040 CONTINUE
 104  CONTINUE
      NC=NC+1
C
C END OF LOOP OVER ANALYSIS TIMES 
C
 10   CONTINUE
C
      DO 11 L=LDEB,NIV
      DO 110 I=IDEB,IFIN
        DKG (L,I)=DKGM (L,I)/REAL(NC)
 110  CONTINUE
 11   CONTINUE
C
C 3. COMPUTE DIFFUSION MATRIX
C
      CZETA =1.0
      CZETAK=1.0
C
      DO 12 I=IDEB,IFIN
      DO 120 L=LDEB,NIV
        N=IFIN*(J-1)+I
        M1=(N-1)/LON
        I1=N-LON*M1
        J1=M1+1
        AM  (L)=MAX(AMMIN,SM(I1,J1,L)/SURFM(I1,J1))
        AK  (L)=MAX(0.,DKG(L,I))*CZETAK
 120  CONTINUE
C
C COMPUTE [I-M*dt]-1 AND NEW TRACER MASSES
C
      CALL KFBCONVMA(LON,LAT,NTRA,
     +               NIV,LDEB,AM,AK,I1,J1,SM,QQQM)
C
 12   CONTINUE
C	
	DEALLOCATE (DKGM)
	DEALLOCATE (DKG)
	DEALLOCATE (ZDKG)
	DEALLOCATE (PRESS)
	DEALLOCATE (TTEMP)
	DEALLOCATE (UTEMP)
	DEALLOCATE (VTEMP)
	DEALLOCATE (WZ)
	DEALLOCATE (WT)
	DEALLOCATE (WU)
	DEALLOCATE (WV)
	DEALLOCATE (TMPRESS)
	DEALLOCATE (AM)
	DEALLOCATE (AK)
	DEALLOCATE (G)
	DEALLOCATE (SM)
C
C      
	RETURN
      END
