      SUBROUTINE KFBCONV(LON,LAT,NIV,NTRA,NSLS,LDEB,
     +                   TRANAM,LSCAVKFB,XKHENRY,
     +                   XPAM,TGRIM,QGRIM,UGRIM,VGRIM,WGRIM,
     +                   SURFM,XPHFM,QQQM,
     +                   LCVPFLX,SCVPFLX,KCVDEB,KCVFIN)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Convection and diffusion (Bechtold/Louis)
C   Purpose    : Kain-Fritsch-Bechtold convection and Louis diffusion
C   Called by  : IOPC3D
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 04/2000
C
C  Modified : B. Josse, 05/2002
C	      H. Le Berre, 09/2006, modification interface d'appel passage
C                                   de variables en argument pour eviter des
C                                   tests et instructions redondantes
C                                   sur les 4 domaines
C
CCC
C
      USE PARADI_MOD
      USE EXPE_MOD
      USE CSTES_MOD
      USE FORMT_MOD
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: LON,LAT,NIV,NTRA,NSLS,LDEB
      INTEGER :: KCVDEB,KCVFIN
      REAL (KIND=8) , DIMENSION (NTRA,2)   :: XKHENRY
      REAL (KIND=8) , DIMENSION (LON,LAT) :: SURFM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: XPAM,TGRIM,QGRIM,UGRIM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: VGRIM,WGRIM,XPHFM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: LCVPFLX,SCVPFLX
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV,NTRA) :: QQQM
      CHARACTER(LEN=15) , DIMENSION (NTRA) :: TRANAM
      LOGICAL :: LSCAVKFB
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: IDEB,IFIN,JDEB,JFIN,LLDEB,LFIN,J,IJ,K,L,JC,INIV,ILON
      INTEGER :: KIDIA,KFDIA,KBDIA,KTDIA,KCH,KENSM
      REAL (KIND=8) :: PLLZ,PLUZ,DENO,ZRHO,PDTCONV,XTADJD,XTADJS,ZPAM
C 
C INPUT FOR EXTERNAL CONVECTION
C
      REAL (KIND=8) , ALLOCATABLE :: PZ(:,:),PPRES(:,:),PHPRES(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PT(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PRV(:,:),PU(:,:),PV(:,:),PW(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PRC(:,:),PRI(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PCH1(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: PDXDY (:)
      INTEGER :: KICE,ITOPCONV
      LOGICAL :: LDEEP,LSHAL,LREFRESH
      LOGICAL :: LDOWN,LCH1CONV,LSETTADJ
C
C OUTPUT FOR EXTERNAL CONVECTION
C
      INTEGER , ALLOCATABLE :: KCOUNT(:),KCLTOP(:),KCLBAS(:)
      REAL (KIND=8) , ALLOCATABLE :: PTTEN(:,:),PRVTEN (:,:)
      REAL (KIND=8) , ALLOCATABLE :: PRCTEN (:,:)
      REAL (KIND=8) , ALLOCATABLE :: PRITEN(:,:),PPRTEN (:)
      REAL (KIND=8) , ALLOCATABLE :: PPRSTEN(:)
      REAL (KIND=8) , ALLOCATABLE :: PUMF(:,:),PDMF(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PPRLFLX(:,:),PPRSFLX(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PUW(:,:),PWSUB(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PCAPE(:)
      REAL (KIND=8) , ALLOCATABLE :: PCH1TEN(:,:,:)
C
#ifdef ACCIDENT
      REAL (KIND=8) SOMTES,VALSEUIL,EPSILON
#endif
C
      EXTERNAL CONVECTION 
C
      ALLOCATE (PZ    (LAT,NIV-LDEB+1)) ; PZ    (:,:) = 0.0
      ALLOCATE (PPRES (LAT,NIV-LDEB+1)) ; PPRES (:,:) = 0.0
      ALLOCATE (PHPRES(LAT,NIV-LDEB+1)) ; PHPRES(:,:) = 0.0
      ALLOCATE (PT    (LAT,NIV-LDEB+1)) ; PT    (:,:) = 0.0
      ALLOCATE (PRV   (LAT,NIV-LDEB+1)) ; PRV   (:,:) = 0.0
      ALLOCATE (PU    (LAT,NIV-LDEB+1)) ; PU    (:,:) = 0.0
      ALLOCATE (PV    (LAT,NIV-LDEB+1)) ; PV    (:,:) = 0.0
      ALLOCATE (PW    (LAT,NIV-LDEB+1)) ; PW    (:,:) = 0.0
      ALLOCATE (PRC   (LAT,NIV-LDEB+1)) ; PRC   (:,:) = 0.0
      ALLOCATE (PRI   (LAT,NIV-LDEB+1)) ; PRI   (:,:) = 0.0
      ALLOCATE (PCH1  (LAT,NIV-LDEB+1,NTRA)) ; PCH1  (:,:,:) = 0.0
      ALLOCATE (PDXDY (LAT)) ; PDXDY (:) = 0.0      
C	
C OUTPUT FOR EXTERNAL CONVECTION	
C	
      ALLOCATE (KCOUNT(LAT)) ; KCOUNT(:) = 0
      ALLOCATE (KCLTOP(LAT),KCLBAS(LAT)) ; KCLTOP(:) = 0 ; KCLBAS(:) = 0
      ALLOCATE (PTTEN  (LAT,NIV-LDEB+1)) ; PTTEN  (:,:) = 0.0
      ALLOCATE (PRVTEN (LAT,NIV-LDEB+1)) ; PRVTEN (:,:) = 0.0
      ALLOCATE (PRCTEN (LAT,NIV-LDEB+1)) ; PRCTEN (:,:) = 0.0
      ALLOCATE (PRITEN (LAT,NIV-LDEB+1)) ; PRITEN (:,:) = 0.0
      ALLOCATE (PPRTEN (LAT)) ; PPRTEN (:) = 0.0
      ALLOCATE (PPRSTEN(LAT)) ; PPRSTEN(:) = 0.0
      ALLOCATE (PUMF   (LAT,NIV-LDEB+1)) ; PUMF   (:,:) = 0.0
      ALLOCATE (PDMF   (LAT,NIV-LDEB+1)) ; PDMF   (:,:) = 0.0
      ALLOCATE (PPRLFLX(LAT,NIV-LDEB+1)) ; PPRLFLX(:,:) = 0.0
      ALLOCATE (PPRSFLX(LAT,NIV-LDEB+1)) ; PPRSFLX(:,:) = 0.0
      ALLOCATE (PUW(LAT,NIV-LDEB+1))   ; PUW(:,:) = 0.0
      ALLOCATE (PWSUB(LAT,NIV-LDEB+1)) ; PWSUB(:,:) = 0.0
      ALLOCATE (PCAPE  (LAT)) ; PCAPE  (:) = 0.0
      ALLOCATE (PCH1TEN(LAT,NIV-LDEB+1,NTRA)) ; PCH1TEN(:,:,:) = 0.0   
C
C 0. DIFFUSION : [Louis,1979]
C      
      IDEB=1
      IFIN=LON
      JDEB=1
      JFIN=LAT
      LFIN=NIV
C
C INDEX FOR LOUIS DIFFUSION
C
      LLDEB = 1
      DO iniv=1,NIVM1
        ZPAM=0.
        DO ilon=1,LONM1*LATM1
          ZPAM=ZPAM+XPAM1(ilon,1,iniv)/REAL(LONM1*LATM1)
        ENDDO
        IF (ZPAM.LE.10000.) LLDEB = iniv
      ENDDO
C     
      CALL LOUISDIF(LON,LAT,NIV,NTRA,IDEB,IFIN,JDEB,JFIN,LLDEB,
     +              XPAM,TGRIM,UGRIM,VGRIM,SURFM,XPHFM,QQQM)
C
C 1. INITIALIZE FLUXES
C
        DO J=1,LON*LAT*NIV
          LCVPFLX(J,1,1)=0.
          SCVPFLX(J,1,1)=0.
        ENDDO
#ifdef ACCIDENT
C
C Definitions de valeurs seuil au dessous desquelles
C on n'active pas la diffusion (valeurs nulles), 
C valable en particulier pour gagner du temps
C dans ACCIDENT
C
      EPSILON=1.E-28
      VALSEUIL=REAL(LAT)*REAL(NIV)*REAL(NTRA)*1.E-30
#endif        
C
C 2. LOOP ON LONGITUDES
C    
      DO IJ=1,LON
#ifdef ACCIDENT
C
C calcul de la valeur somme
C      
       SOMTES=0.
       DO JC=1,NTRA
       DO L=1,NIV
       DO J=1,LAT
        SOMTES=SOMTES+QQQM(IJ,J,L,JC)
       ENDDO
       ENDDO
       ENDDO
C 
C Appel seulement si on depasse le seuil
C      
       IF (SOMTES+EPSILON.GE.VALSEUIL) THEN
#endif
C
      ITOPCONV=NIV-LDEB+1
C
      DO K=1,LAT
        KCOUNT(K)=0
        KCLTOP(K)=0
        KCLBAS(K)=0
        PCAPE (K)=0.
      ENDDO
C
      DO J=1,LAT
      PPRTEN (J)=0.
      PPRSTEN(J)=0.
      ENDDO
      DO J=1,LAT*ITOPCONV
        PTTEN  (J,1)=0.
        PRVTEN (J,1)=0.
        PRCTEN (J,1)=0.
        PRITEN (J,1)=0.
        PUMF   (J,1)=0.
        PDMF   (J,1)=0.
        PPRLFLX(J,1)=0.
        PPRSFLX(J,1)=0.
        PWSUB  (J,1)=0.
        PUW    (J,1)=0.
      ENDDO
C
      DO J=1,LAT*ITOPCONV*NTRA
         PCH1TEN(J,1,1)=0.
         PCH1   (J,1,1)=0.
      ENDDO
C
C 3. CHANGE 3D-ARRAYS TO 2D
C
C WARNING : in MOCAGE levels go from fop to bottom
C           in KFB, they go from bottom to top
C
      DO K=1,LAT
        PDXDY(K)=SURFM(IJ,K)
      ENDDO     
C      
      DO L=1,ITOPCONV
        DO K=1,LAT
         PPRES (K,L)=XPAM (IJ,K,NIV-L+1)
         PHPRES(K,L)=XPHFM(IJ,K,NIV-L+1)
         PT    (K,L)=TGRIM(IJ,K,NIV-L+1)
         PRV   (K,L)=QGRIM(IJ,K,NIV-L+1)
     +              /(1.-QGRIM(IJ,K,NIV-L+1))
         PU    (K,L)=UGRIM(IJ,K,NIV-L+1)
         PV    (K,L)=VGRIM(IJ,K,NIV-L+1)
         PW    (K,L)=WGRIM(IJ,K,NIV-L+1)
         PRC   (K,L)=0.
         PRI   (K,L)=0.
        ENDDO
      ENDDO
C
      DO JC=1,NTRA
      DO L=1,ITOPCONV
        DO K=1,LAT  
          PCH1(K,L,JC)=QQQM(IJ,K,NIV-L+1,JC)
        ENDDO
      ENDDO
      ENDDO
C
C 4. COMPUTE HEIGHT
C
      DO K=1,LAT
        PZ(K,1)=RA*PT(K,1)/GG
     +         *MAX(0.,ALOG(PREF/PPRES(K,1)))
      ENDDO
C
      DO L=2,ITOPCONV
      DO K=1,LAT
        PLLZ=PPRES(K,L-1)
        PLUZ=PPRES(K,L  )
        PZ(K,L)=PZ(K,L-1) + RA*PT(K,L)*ALOG(PLLZ/PLUZ)/GG
      ENDDO
      ENDDO
C
C 5. CONVERT VERTICAL VELOCITIES (PW) FROM Pa/s TO m/s
C
      DO K=1,LAT*ITOPCONV
        DENO= RA*PT(K,1)
     +       *( 1.+0.608*PRV(K,1) )
        ZRHO= PPRES(K,1) / DENO
        IF (ZRHO.NE.0.) THEN
          PW(K,1)=-PW(K,1)/(GG*ZRHO)
        ENDIF
      ENDDO
C
C 7. PARAMETERS FOR EXTERNAL CONVECTION CALL
C
C LOOPS LENGTH
C
       KIDIA=1
       KFDIA=LAT
       KBDIA=1
       KTDIA=1
       KCH  =NTRA
C
C TIMESTEP FOR CONVECTION
C
       PDTCONV=DTCONV
C
C FLAGS FOR OPTIONS
C
       LDEEP   =.TRUE.
       LSHAL   =.TRUE.
       LCH1CONV=.TRUE.
C
#ifdef MIDATM
       LDOWN   =.FALSE.
#else
       LDOWN   =.TRUE.
#endif
       KICE    =1
C
       LREFRESH=.TRUE.
       LSETTADJ=.FALSE.
       XTADJD  =4.*PDTCONV
       XTADJS  =4.*PDTCONV
C
       KENSM=0
C
C 8. CALL EXTERNAL CONVECTION  [Kain-Fritsch-Bechtold]
C
      CALL CONVECTION( LAT, ITOPCONV, KIDIA, KFDIA, KBDIA, KTDIA,
     +         PDTCONV, LDEEP, LSHAL, LREFRESH, LDOWN, KICE,
     +         LSETTADJ, XTADJD, XTADJS,
     +         KENSM,
     +         PPRES, PZ, PDXDY,
     +         PT, PRV, PRC, PRI, PU, PV, PW,
     +         KCOUNT, PTTEN, PRVTEN, PRCTEN, PRITEN,
     +         PPRTEN, PPRSTEN,
     +         PUMF, PDMF, PPRLFLX, PPRSFLX, 
     +         PUW, PWSUB,
     +         PCAPE, KCLTOP, KCLBAS,
     +         LCH1CONV, NTRA, PCH1, PCH1TEN, 
     +         TRANAM, LSCAVKFB, XKHENRY)
C
      IF (.NOT.TRANSINV) THEN
      CALL KFBCORREC1D(LAT,ITOPCONV,NTRA,PCH1,PCH1TEN,
     +                 PHPRES,PPRES,PDTCONV,LSCAVKFB)
      ENDIF
      CALL KFBCORRECFLUX(LAT,ITOPCONV,PHPRES,PPRES,PT,
     +                   PPRLFLX,PPRSFLX,PUW,PWSUB)
C
C 9. CHANGE BACK 2D-ARRAYS TO 3D
C       
      DO JC=1,NTRA
#ifdef MIDATM      
       IF (TRANAM(JC).NE.'H_2O') THEN
#endif
#ifdef CARIOLLE
       IF (TRANAM(JC).EQ.'CO') THEN
#endif
        DO L=LDEB,NIV
          DO K=KCVDEB,KCVFIN
            QQQM(IJ,K,L,JC)=PCH1(K,NIV-L+1,JC)
          ENDDO
        ENDDO
#ifdef MIDATM      
       ENDIF
#endif
#ifdef CARIOLLE
       ENDIF
#endif
      ENDDO
      DO L=LDEB,NIV
        DO K=KCVDEB,KCVFIN
          SCVPFLX(IJ,K,L)=1000.*PPRSFLX(K,NIV-L+1)
          LCVPFLX(IJ,K,L)=1000.*PPRLFLX(K,NIV-L+1)
        ENDDO
      ENDDO
#ifdef ACCIDENT
C
C Fin du test de depassement de seuil
C
      ENDIF
#endif
CC
C END OF LOOP ON  LONGITUDES
C
      ENDDO
C
      DEALLOCATE (PZ)
      DEALLOCATE (PPRES)
      DEALLOCATE (PHPRES)
      DEALLOCATE (PT)
      DEALLOCATE (PRV)
      DEALLOCATE (PU)
      DEALLOCATE (PV)
      DEALLOCATE (PW)
      DEALLOCATE (PRC)
      DEALLOCATE (PRI)
      DEALLOCATE (PCH1)
      DEALLOCATE (PDXDY)      
      DEALLOCATE (KCOUNT)
      DEALLOCATE (KCLTOP)
      DEALLOCATE (KCLBAS)
      DEALLOCATE (PTTEN)
      DEALLOCATE (PRVTEN)
      DEALLOCATE (PRCTEN)
      DEALLOCATE (PRITEN)
      DEALLOCATE (PPRTEN)
      DEALLOCATE (PPRSTEN)
      DEALLOCATE (PUMF)
      DEALLOCATE (PDMF)
      DEALLOCATE (PPRLFLX)
      DEALLOCATE (PPRSFLX)
      DEALLOCATE (PUW)
      DEALLOCATE (PWSUB)
      DEALLOCATE (PCAPE)
      DEALLOCATE (PCH1TEN)
C
      RETURN
      END
