      SUBROUTINE KFBCONV(LON,LAT,NIV,NTRA,NSLS,LDEB,                 &
                         TRANAM,LSCAVKFB,XKHENRY,                    &
                         XPAM,TGRIM,QGRIM,UGRIM,VGRIM,WGRIM,         &
                         SURFM,XPHFM,QQQM,                           &
                         LCVPFLX,SCVPFLX,KCVDEB,KCVFIN)
!
!CCC
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!CC! DATE : 15/05/2005
!C
!   Type       : Convection and diffusion (Bechtold/Louis)
!   Purpose    : Kain-Fritsch-Bechtold convection and Louis diffusion
!   Called by  : IOPC3D
!C
!CCC
!C
!  Original : B. Josse & V.-H. Peuch, 04/2000
!C
!  Modified : B. Josse, 05/2002
!C	      H. Le Berre, 09/2006, modification interface d'appel passage
!                                   de variables en argument pour eviter des
!                                   tests et instructions redondantes
!                                   sur les 4 domaines
!C
!CCC
!C
      USE PARADI_MOD
      USE EXPE_MOD
      USE CSTES_MOD
      USE FORMT_MOD
!
      IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
      INTEGER :: LON,LAT,NIV,NTRA,NSLS,LDEB
      INTEGER :: KCVDEB,KCVFIN
      REAL (KIND=8) , DIMENSION (NTRA,2)   :: XKHENRY
      REAL (KIND=8) , DIMENSION (LON,LAT) :: SURFM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: XPAM,TGRIM,QGRIM,UGRIM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: VGRIM,WGRIM,XPHFM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: LCVPFLX,SCVPFLX
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV,NTRA) :: QQQM
      CHARACTER(LEN=15) , DIMENSION (NTRA) :: TRANAM
      LOGICAL :: LSCAVKFB
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
      INTEGER :: IDEB,IFIN,JDEB,JFIN,LLDEB,LFIN,J,IJ,K,L,JC,INIV,ILON
      INTEGER :: KIDIA,KFDIA,KBDIA,KTDIA,KCH,KENSM
      REAL (KIND=8) :: PLLZ,PLUZ,DENO,ZRHO,PDTCONV,XTADJD,XTADJS,ZPAM
! 
! INPUT FOR EXTERNAL CONVECTION
!
      REAL (KIND=8) , ALLOCATABLE :: PZ(:,:),PPRES(:,:),PHPRES(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PT(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PRV(:,:),PU(:,:),PV(:,:),PW(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PRC(:,:),PRI(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PCH1(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: PDXDY (:)
      INTEGER :: KICE,ITOPCONV
      LOGICAL :: LDEEP,LSHAL,LREFRESH
      LOGICAL :: LDOWN,LCH1CONV,LSETTADJ
!
! OUTPUT FOR EXTERNAL CONVECTION
!
      INTEGER , ALLOCATABLE :: KCOUNT(:),KCLTOP(:),KCLBAS(:)
      REAL (KIND=8) , ALLOCATABLE :: PTTEN(:,:),PRVTEN (:,:)
      REAL (KIND=8) , ALLOCATABLE :: PRCTEN (:,:)
      REAL (KIND=8) , ALLOCATABLE :: PRITEN(:,:),PPRTEN (:)
      REAL (KIND=8) , ALLOCATABLE :: PPRSTEN(:)
      REAL (KIND=8) , ALLOCATABLE :: PUMF(:,:),PDMF(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PPRLFLX(:,:),PPRSFLX(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PUW(:,:),PWSUB(:,:)
      REAL (KIND=8) , ALLOCATABLE :: PCAPE(:)
      REAL (KIND=8) , ALLOCATABLE :: PCH1TEN(:,:,:)
!
      EXTERNAL CONVECTION 
!
   
!
! 0. DIFFUSION : [Louis,1979]
!      
      IDEB=1
      IFIN=LON
      JDEB=1
      JFIN=LAT
      LFIN=NIV
!
! INDEX FOR LOUIS DIFFUSION
!
      LLDEB = 1
      DO iniv=1,NIVM1
        ZPAM=0.
        DO ilon=1,LONM1*LATM1
          ZPAM=ZPAM+XPAM1(ilon,1,iniv)/REAL(LONM1*LATM1)
        ENDDO
        IF (ZPAM.LE.10000.) LLDEB = iniv
      ENDDO
!     
      CALL LOUISDIF(LON,LAT,NIV,NTRA,IDEB,IFIN,JDEB,JFIN,LLDEB,    &
                    XPAM,TGRIM,UGRIM,VGRIM,SURFM,XPHFM,QQQM)
!
! 1. INITIALIZE FLUXES
!
        DO J=1,LON*LAT*NIV
          LCVPFLX(J,1,1)=0.
          SCVPFLX(J,1,1)=0.
        ENDDO
!
      ITOPCONV=NIV-LDEB+1
!      
! 2. LOOP ON LONGITUDES
!    
!$omp parallel default(none), & 
!$omp private(IJ,J,K,KCOUNT,KCLTOP,KCLBAS,PCAPE,PPRTEN,PPRSTEN), &
!$omp private(PTTEN,PRVTEN,PRCTEN,PRITEN,PUMF,PDMF,PPRLFLX,PPRSFLX), &
!$omp private(PWSUB,PUW,PCH1TEN,PCH1,PDXDY,PPRES,PHPRES,PT,PRV,PU,PV), &
!$omp private(PW,PRC,PRI,JC,L,PZ,PLLZ,PLUZ,DENO,ZRHO,KIDIA,KFDIA,KBDIA), &
!$omp private(KTDIA,KCH,LDEEP,LSHAL,LCH1CONV,LDOWN,KICE,LREFRESH), &
!$omp private(LSETTADJ,XTADJD,XTADJS,KENSM), &

!$omp shared(LAT,LON,NIV,ITOPCONV,LDEB,SURFM,XPAM,XPHFM,TGRIM,QGRIM,UGRIM), &
!$omp shared(VGRIM,WGRIM,QQQM,NTRA,RA,GG,PREF,DTCONV,PDTCONV,TRANAM,LSCAVKFB), &
!$omp shared(XKHENRY,KCVDEB,KCVFIN,SCVPFLX,LCVPFLX)
!
      ALLOCATE (PZ    (LAT,NIV-LDEB+1))
      ALLOCATE (PPRES (LAT,NIV-LDEB+1))
      ALLOCATE (PHPRES(LAT,NIV-LDEB+1))
      ALLOCATE (PT    (LAT,NIV-LDEB+1))
      ALLOCATE (PRV   (LAT,NIV-LDEB+1))
      ALLOCATE (PU    (LAT,NIV-LDEB+1))
      ALLOCATE (PV    (LAT,NIV-LDEB+1))
      ALLOCATE (PW    (LAT,NIV-LDEB+1))
      ALLOCATE (PRC   (LAT,NIV-LDEB+1))
      ALLOCATE (PRI   (LAT,NIV-LDEB+1))
      ALLOCATE (PCH1  (LAT,NIV-LDEB+1,NTRA))
      ALLOCATE (PDXDY (LAT))
!	
! OUTPUT FOR EXTERNAL CONVECTION	
!	
      ALLOCATE (KCOUNT(LAT))
      ALLOCATE (KCLTOP(LAT),KCLBAS(LAT))
      ALLOCATE (PTTEN  (LAT,NIV-LDEB+1))
      ALLOCATE (PRVTEN (LAT,NIV-LDEB+1))
      ALLOCATE (PRCTEN (LAT,NIV-LDEB+1))
      ALLOCATE (PRITEN (LAT,NIV-LDEB+1))
      ALLOCATE (PPRTEN (LAT))
      ALLOCATE (PPRSTEN(LAT))
      ALLOCATE (PUMF   (LAT,NIV-LDEB+1))
      ALLOCATE (PDMF   (LAT,NIV-LDEB+1))
      ALLOCATE (PPRLFLX(LAT,NIV-LDEB+1))
      ALLOCATE (PPRSFLX(LAT,NIV-LDEB+1))
      ALLOCATE (PUW(LAT,NIV-LDEB+1))
      ALLOCATE (PWSUB(LAT,NIV-LDEB+1))
      ALLOCATE (PCAPE  (LAT))
      ALLOCATE (PCH1TEN(LAT,NIV-LDEB+1,NTRA))

!$omp do 
      DO IJ=1,LON
!
!
      DO K=1,LAT
        KCOUNT(K)=0
        KCLTOP(K)=0
        KCLBAS(K)=0
        PCAPE (K)=0.
      ENDDO
!
      DO J=1,LAT
      PPRTEN (J)=0.
      PPRSTEN(J)=0.
      ENDDO
      DO J=1,LAT*ITOPCONV
        PTTEN  (J,1)=0.
        PRVTEN (J,1)=0.
        PRCTEN (J,1)=0.
        PRITEN (J,1)=0.
        PUMF   (J,1)=0.
        PDMF   (J,1)=0.
        PPRLFLX(J,1)=0.
        PPRSFLX(J,1)=0.
        PWSUB  (J,1)=0.
        PUW    (J,1)=0.
      ENDDO
!
      DO J=1,LAT*ITOPCONV*NTRA
         PCH1TEN(J,1,1)=0.
         PCH1   (J,1,1)=0.
      ENDDO
!
! 3. CHANGE 3D-ARRAYS TO 2D
!
! WARNING : in MOCAGE levels go from fop to bottom
!           in KFB, they go from bottom to top
!
      DO K=1,LAT
        PDXDY(K)=SURFM(IJ,K)
      ENDDO     
!      
      DO L=1,ITOPCONV
        DO K=1,LAT
         PPRES (K,L)=XPAM (IJ,K,NIV-L+1)
         PHPRES(K,L)=XPHFM(IJ,K,NIV-L+1)
         PT    (K,L)=TGRIM(IJ,K,NIV-L+1)
         PRV   (K,L)=QGRIM(IJ,K,NIV-L+1)        &
                    /(1.-QGRIM(IJ,K,NIV-L+1))
         PU    (K,L)=UGRIM(IJ,K,NIV-L+1)
         PV    (K,L)=VGRIM(IJ,K,NIV-L+1)
         PW    (K,L)=WGRIM(IJ,K,NIV-L+1)
         PRC   (K,L)=0.
         PRI   (K,L)=0.
        ENDDO
      ENDDO
!
      DO JC=1,NTRA
      DO L=1,ITOPCONV
        DO K=1,LAT  
          PCH1(K,L,JC)=QQQM(IJ,K,NIV-L+1,JC)
        ENDDO
      ENDDO
      ENDDO
!
! 4. COMPUTE HEIGHT
!
      DO K=1,LAT
        PZ(K,1)=RA*PT(K,1)/GG                     &
               *MAX(0.,ALOG(PREF/PPRES(K,1)))
      ENDDO
!
      DO L=2,ITOPCONV
      DO K=1,LAT
        PLLZ=PPRES(K,L-1)
        PLUZ=PPRES(K,L  )
        PZ(K,L)=PZ(K,L-1) + RA*PT(K,L)*ALOG(PLLZ/PLUZ)/GG
      ENDDO
      ENDDO
!
! 5. CONVERT VERTICAL VELOCITIES (PW) FROM Pa/s TO m/s
!
      DO K=1,LAT*ITOPCONV
        DENO= RA*PT(K,1)               &
             *( 1.+0.608*PRV(K,1) )
        ZRHO= PPRES(K,1) / DENO
        IF (ZRHO.NE.0.) THEN
          PW(K,1)=-PW(K,1)/(GG*ZRHO)
        ENDIF
      ENDDO
!
! 7. PARAMETERS FOR EXTERNAL CONVECTION CALL
!
! LOOPS LENGTH
!
       KIDIA=1
       KFDIA=LAT
       KBDIA=1
       KTDIA=1
       KCH  =NTRA
!
! TIMESTEP FOR CONVECTION
!
       PDTCONV=DTCONV
!
! FLAGS FOR OPTIONS
!
       LDEEP   =.TRUE.
       LSHAL   =.TRUE.
       LCH1CONV=.TRUE.
!
#ifdef NIV_60
       LDOWN   =.FALSE.
#else
       LDOWN   =.TRUE.
#endif
       KICE    =1
!
       LREFRESH=.TRUE.
       LSETTADJ=.FALSE.
       XTADJD  =4.*PDTCONV
       XTADJS  =4.*PDTCONV
!
       KENSM=0
!
! 8. CALL EXTERNAL CONVECTION  [Kain-Fritsch-Bechtold]
!
      CALL CONVECTION( LAT, ITOPCONV, KIDIA, KFDIA, KBDIA, KTDIA,  &
               PDTCONV, LDEEP, LSHAL, LREFRESH, LDOWN, KICE,       &
               LSETTADJ, XTADJD, XTADJS,                           &
               KENSM,                                              &
               PPRES, PZ, PDXDY,                                   &
               PT, PRV, PRC, PRI, PU, PV, PW,                      &
               KCOUNT, PTTEN, PRVTEN, PRCTEN, PRITEN,              &
               PPRTEN, PPRSTEN,                                    &
               PUMF, PDMF, PPRLFLX, PPRSFLX,                       &
               PUW, PWSUB,                                         &
               PCAPE, KCLTOP, KCLBAS,                              &
               LCH1CONV, NTRA, PCH1, PCH1TEN,                      &
               TRANAM, LSCAVKFB, XKHENRY)
!
      CALL KFBCORREC1D(LAT,ITOPCONV,NTRA,PCH1,PCH1TEN,             &
                       PHPRES,PPRES,PDTCONV,LSCAVKFB)
      CALL KFBCORRECFLUX(LAT,ITOPCONV,PHPRES,PPRES,PT,             &
                         PPRLFLX,PPRSFLX,PUW,PWSUB)
!
! 9. CHANGE BACK 2D-ARRAYS TO 3D
!       
      DO JC=1,NTRA
#ifdef NIV_60      
       IF (TRANAM(JC).NE.'H_2O') THEN
#endif
#ifdef CARIOLLE    
       IF (TRANAM(JC).EQ.'CO') THEN
#endif
        DO L=LDEB,NIV
          DO K=KCVDEB,KCVFIN
            QQQM(IJ,K,L,JC)=PCH1(K,NIV-L+1,JC)
          ENDDO
        ENDDO
#ifdef NIV_60      
       ENDIF
#endif
#ifdef CARIOLLE    
       ENDIF
#endif
      ENDDO
      DO L=LDEB,NIV
        DO K=KCVDEB,KCVFIN
          SCVPFLX(IJ,K,L)=1000.*PPRSFLX(K,NIV-L+1)
          LCVPFLX(IJ,K,L)=1000.*PPRLFLX(K,NIV-L+1)
        ENDDO
      ENDDO
!
! END OF LOOP ON  LONGITUDES
!      
      ENDDO
!$omp enddo
!
      DEALLOCATE (PZ)
      DEALLOCATE (PPRES)
      DEALLOCATE (PHPRES)
      DEALLOCATE (PT)
      DEALLOCATE (PRV)
      DEALLOCATE (PU)
      DEALLOCATE (PV)
      DEALLOCATE (PW)
      DEALLOCATE (PRC)
      DEALLOCATE (PRI)
      DEALLOCATE (PCH1)
      DEALLOCATE (PDXDY)      
      DEALLOCATE (KCOUNT)
      DEALLOCATE (KCLTOP)
      DEALLOCATE (KCLBAS)
      DEALLOCATE (PTTEN)
      DEALLOCATE (PRVTEN)
      DEALLOCATE (PRCTEN)
      DEALLOCATE (PRITEN)
      DEALLOCATE (PPRTEN)
      DEALLOCATE (PPRSTEN)
      DEALLOCATE (PUMF)
      DEALLOCATE (PDMF)
      DEALLOCATE (PPRLFLX)
      DEALLOCATE (PPRSFLX)
      DEALLOCATE (PUW)
      DEALLOCATE (PWSUB)
      DEALLOCATE (PCAPE)
      DEALLOCATE (PCH1TEN)
!$omp end parallel
!

!
      RETURN
      END
