      SUBROUTINE KFBCORREC1D(LOLA,NIV,NTRA,PCH1,PCH1TEN,
     +                       PHPRES,PPRES,PDTCONV,LSCAV)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Convection and diffusion (Bechtold/Louis)
C   Purpose    : Apply mass correction in each column after call
C                to Kain-Fritsch-Bechtold convection routine
C   Called by  :
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 04/2000
C
C  Modified : B. Josse, 07/2002
C
CCC
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: LOLA,NIV,NTRA
      LOGICAL :: LSCAV
      REAL (KIND=8) , DIMENSION (LOLA,NIV,NTRA) :: PCH1,PCH1TEN
      REAL (KIND=8) , DIMENSION (LOLA,NIV) :: PHPRES,PPRES
      REAL (KIND=8) :: PDTCONV      
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: J,L,JC
      REAL (KIND=8) :: RMINTEN
      REAL (KIND=8) , ALLOCATABLE :: RMAS1(:,:),RMAS2(:,:)
      REAL (KIND=8) , ALLOCATABLE :: RDPS (:,:)
      REAL (KIND=8) , ALLOCATABLE :: ZWORK(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: COEFCOR(:,:)
C
C 1. INITIALIZATIONS
C
      ALLOCATE (RMAS1(LOLA,NTRA))
      ALLOCATE (RMAS2(LOLA,NTRA))
      ALLOCATE (RDPS (LOLA,NIV))
      ALLOCATE (ZWORK(LOLA,NIV,NTRA))
      ALLOCATE (COEFCOR(LOLA,NTRA))
C	
      DO J=1,LOLA*NTRA
        RMAS1(J,1)=0.
        RMAS2(J,1)=0.
      ENDDO
C
C 2. COMPUTE INITIAL MASS IN EACH COLUMN
C
      DO L=1,NIV-1
      DO J=1,LOLA
        RDPS(J,L)=PHPRES(J,L)-PHPRES(J,L+1)
      ENDDO
      ENDDO
C
      DO J=1,LOLA
        RDPS(J,NIV)=2*(PHPRES(J,NIV)-PPRES(J,NIV))
      ENDDO
C
      DO JC=1,NTRA
      DO L=1,NIV
      DO J=1,LOLA
        RMAS1(J,JC)=RMAS1(J,JC)+RDPS(J,L)*PCH1(J,L,JC)
      ENDDO
      ENDDO
      ENDDO
C
C 3. COMPUTE TRACER MIXING RATIOS AFTER CONVECTION
C
C 3.1 CHECK THAT NO MORE TRACER IS REMOVED THAN
C     THE INITIAL CONTENT
C
      DO J=1,LOLA*NIV*NTRA
       RMINTEN=-1.*PCH1(J,1,1)/PDTCONV
       PCH1TEN(J,1,1)=MAX(PCH1TEN(J,1,1),RMINTEN)
      ENDDO
C
C 3.2 TRACER MIXING RATION AFTER CONVECTION
C
      DO J=1,LOLA*NIV*NTRA
        ZWORK(J,1,1)=PCH1(J,1,1)+PCH1TEN(J,1,1)*PDTCONV
      ENDDO
C
C 4. COMPUTE MASS IN EACH COLUMN AFTER CONVECTION
C
      DO JC=1,NTRA
      DO L=1,NIV
      DO J=1,LOLA
        RMAS2(J,JC)=RMAS2(J,JC)+RDPS(J,L)*ZWORK(J,L,JC)
      ENDDO
      ENDDO
      ENDDO
C
C 5. COMPUTE CORRECTION COEFFICIENT
C
      DO J=1,LOLA*NTRA
        IF (RMAS2(J,1).NE.0.) THEN
          IF (LSCAV) THEN
C
C SCAVENGING SHOULD DECREASE MASS
C
            COEFCOR(J,1)=MIN(RMAS1(J,1)/RMAS2(J,1),1.)
          ELSE
            COEFCOR(J,1)=RMAS1(J,1)/RMAS2(J,1)
          ENDIF
        ELSE
          COEFCOR(J,1)=1.
        ENDIF
      ENDDO
C
C 6. COMPUTE NEW MIXING RATIOS
C
      DO JC=1,NTRA
      DO L=1,NIV
      DO J=1,LOLA
        PCH1(J,L,JC)=ZWORK(J,L,JC)*COEFCOR(J,JC)
	PCH1(J,L,JC)=MAX(PCH1(J,L,JC),1.e-30)
      ENDDO
      ENDDO
      ENDDO
C
      DEALLOCATE (RMAS1)
      DEALLOCATE (RMAS2)
      DEALLOCATE (RDPS)
      DEALLOCATE (ZWORK)
      DEALLOCATE (COEFCOR)
C
C
      RETURN
      END
