      SUBROUTINE KFBCORRECFLUX(LAT,NIV,PHPRES,PPRES,PT,
     +                       LCVPFLX,SCVPFLX,PUW,PWSUB)      
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Convection and Diffusion (Bechtold/Louis)
C   Purpose    : Correct convective flux for Kain-Fritsch-Bechtold
C   Called by  :
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 02/2003
C
C  Modified :
C
CCC
C
	USE CSTES_MOD
C
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: LAT,NIV
      REAL (KIND=8) , DIMENSION (LAT,NIV) :: PHPRES,PPRES
      REAL (KIND=8) , DIMENSION (LAT,NIV) :: PT,LCVPFLX,SCVPFLX
      REAL (KIND=8) , DIMENSION (LAT,NIV) :: PUW,PWSUB
C     PUW   : updraft vertical velocity (m/s)
C     PWSUB : environmental compensatory subsidence (Pa/s)    
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: L,K,LM1
	REAL (KIND=8) :: SEUIL,CONVERT,SUMUPD,SUMSUB,DELTAP,RAPSURF
	REAL (KIND=8) , ALLOCATABLE :: PUOMEGA(:,:)
C
C
	ALLOCATE (PUOMEGA(LAT,NIV))
C	
      SEUIL=1.
      DO L=1,NIV
        DO K=1,LAT        
          CONVERT=-PPRES(K,L)*GG/RA/PT(K,L)
	  PUOMEGA(K,L)=CONVERT*PUW(K,L)       
        ENDDO
      ENDDO
C      
      SUMUPD=0.
      SUMSUB=0.
C     
      DO K=1,LAT
        SUMUPD=0.
        SUMSUB=0.
        DO L=1,NIV
          LM1=MAX(L-1,1)
          DELTAP= PHPRES(K,L)-PHPRES(K,LM1)
          SUMUPD=SUMUPD+PUOMEGA(K,L)*DELTAP
          SUMSUB=SUMSUB+PWSUB  (K,L)*DELTAP 
        ENDDO
        SUMSUB=ABS(SUMSUB)
        SUMUPD=ABS(SUMUPD)
        IF ((SUMSUB.GT.SEUIL).AND.(SUMUPD.GT.SEUIL)) THEN
          RAPSURF=SQRT(SUMSUB/SUMUPD)
          DO L=1,NIV
            LCVPFLX(K,L)= RAPSURF * LCVPFLX(K,L)
            SCVPFLX(K,L)= RAPSURF * SCVPFLX(K,L)
          ENDDO
        ENDIF
      ENDDO
C
	DEALLOCATE (PUOMEGA)
C
C
      RETURN      
      END
