      SUBROUTINE LOUISDIF(LON,LAT,NIV,NTRA,IDEB,IFIN,JDEB,JFIN,LDEB,
     +                    XPAM,TGRIM,UGRIM,VGRIM,SURFM,XPHFM,QQQM)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Convection and Diffusion (Bechtold/Louis)
C   Purpose    : Interface for Louis diffusion scheme
C   Called by  :
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 04/2000
C
C  Modified : B. Josse, 07/2002
C	      H. Le Berre, 09/2006 , modification interface d'appel , passage de variables 
C	      en argument pour eviter des tests et instructions redondantes sur les 4 domaines
C	      H. Le Berre, 07/2007, Amelioration de la vectorisation, traitement 3D
C
CCC
C
      USE CSTES_MOD
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: LON,LAT,NIV,NTRA,IDEB,IFIN,JDEB,JFIN,LDEB
      REAL (KIND=8) ,DIMENSION (LON,LAT) :: SURFM
      REAL (KIND=8) ,DIMENSION (LON,LAT,NIV) :: XPAM,TGRIM,UGRIM,VGRIM
      REAL (KIND=8) ,DIMENSION (LON,LAT,NIV) :: XPHFM
      REAL (KIND=8) ,DIMENSION (LON,LAT,NIV,NTRA) :: QQQM
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: I,J,L,NC,ITAU
      INTEGER :: K,JL 
      REAL (KIND=8) :: PLLZ,PLUZ,ALPHA,BETA,CZETA,CZETAK
      REAL (KIND=8) :: AMMIN,AMAXDIFF
      PARAMETER (AMMIN   =1.0E-10)
      PARAMETER (AMAXDIFF=1.0E5)
      REAL (KIND=8) , ALLOCATABLE :: DKGM(:,:,:),DKG(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: ZDKG(:,:,:)
C
C
C "HALF" LEVELS
C
      REAL (KIND=8) , ALLOCATABLE :: WZ(:,:,:),WT(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: WU(:,:,:),WV(:,:,:)
C
      REAL (KIND=8) , ALLOCATABLE :: TMPRESS(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: AM(:,:,:),AK(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: SM(:,:,:)
C
C
      ALLOCATE (DKGM (LON,LAT,LDEB:NIV))         ; DKGM (:,:,:) = 0.0
      ALLOCATE (DKG  (LON,LAT,LDEB:NIV))         ; DKG  (:,:,:) = 0.0
      ALLOCATE (ZDKG(LON,LAT,LDEB:NIV))         ; ZDKG(:,:,:) = 0.0
      ALLOCATE (WZ(LON,LAT,0:NIV))         ; WZ(:,:,:) = 0.0
      ALLOCATE (WT(LON,LAT,0:NIV))         ; WT(:,:,:) = 0.0
      ALLOCATE (WU(LON,LAT,0:NIV))         ; WU(:,:,:) = 0.0
      ALLOCATE (WV(LON,LAT,0:NIV))         ; WV(:,:,:) = 0.0
      ALLOCATE (TMPRESS(LON,LAT,0:NIV))         ; TMPRESS(:,:,:) = 0.0
      ALLOCATE (AM(LON,LAT,LDEB:NIV))         ; AM(:,:,:) = 0.0
      ALLOCATE (AK(LON,LAT,LDEB:NIV))         ; AK(:,:,:) = 0.0
      ALLOCATE (SM(LON,LAT,NIV))         ; SM(:,:,:) = 0.0
C        
C
C 1. INITIALIZATIONS
C
C
      DO J=1,LAT
        DO I=1,LON
          TMPRESS(I,J,0)=200.
        ENDDO
      ENDDO          
C             
       DO L=1,NIV
         DO J=1,LAT
           DO I=1,LON
             TMPRESS(I,J,L)=XPHFM(I,J,L)
           ENDDO
         ENDDO
       ENDDO
C       
       DO L=1,NIV
         DO J=1,LAT
           DO I=1,LON
             SM(I,J,L)=ABS(TMPRESS(I,J,L)-TMPRESS(I,J,L-1))
     +                 *SURFM(I,J)/GG
           ENDDO
         ENDDO
       ENDDO
C
C
C ALTITUDE AT HALF LEVELS
C
      DO J=1,LAT
      DO I=1,LON
        WZ(I,J,NIV)=RA*TGRIM(I,J,NIV)/GG*
     +              MAX(0.,ALOG(PREF/TMPRESS(I,J,NIV)))
      ENDDO
      ENDDO
C
      DO L=NIV-1,0,-1
      DO J=1,LAT
      DO I=1,LON
        K=MAX(L,1)
        PLLZ=TMPRESS(I,J,L+1)
        PLUZ=TMPRESS(I,J,K)
        WZ(I,J,L)=WZ(I,J,L+1) + RA*TGRIM(I,J,L+1)*ALOG(PLLZ/PLUZ)/GG
      ENDDO
      ENDDO
      ENDDO      
C
C VARIABLES ONTO HALF LEVES 1 TO NIV-1
C
      DO L=1,NIV-1
      DO J=1,LAT
      DO I=1,LON      
        ALPHA=(TMPRESS(I,J,L)-XPAM(I,J,L))/
     +        (TMPRESS(I,J,L+1)-XPAM(I,J,L))
        BETA=1.-ALPHA
        WU(I,J,L) =ALPHA*UGRIM(I,J,L+1)+BETA*UGRIM(I,J,L)
        WV(I,J,L) =ALPHA*VGRIM(I,J,L+1)+BETA*VGRIM(I,J,L)
        WT(I,J,L) =ALPHA*TGRIM(I,J,L+1)+BETA*TGRIM(I,J,L)
      ENDDO
      ENDDO
      ENDDO
C
C EXTRAPOLATION FOR LEVELS 0 AND NIV
C
      DO J=1,LAT
      DO I=1,LON
C
C a) HALF LEVEL NIV
C
        ALPHA=(TMPRESS(I,J,NIV)-TMPRESS(I,J,NIV-1))
     +       /(XPAM(I,J,NIV)-TMPRESS(I,J,NIV-1))
        BETA=1.-ALPHA
        WU(I,J,NIV) =ALPHA*UGRIM(I,J,NIV)+BETA* WU(I,J,NIV-1)
        WV(I,J,NIV) =ALPHA*VGRIM(I,J,NIV)+BETA* WV(I,J,NIV-1)
        WT(I,J,NIV) =ALPHA*TGRIM(I,J,NIV)+BETA* WT(I,J,NIV-1)
C
C b) HALF LEVEL 0
C
        ALPHA=(TMPRESS(I,J,1)-TMPRESS(I,J,0))/
     +        (TMPRESS(I,J,1)-XPAM(I,J,1))
        BETA=1.-ALPHA
        WU(I,J,0) =ALPHA*UGRIM(I,J,1)+BETA* WU(I,J,1)
        WV(I,J,0) =ALPHA*VGRIM(I,J,1)+BETA* WV(I,J,1)
        WT(I,J,0) =ALPHA*TGRIM(I,J,1)+BETA* WT(I,J,1)
C
      ENDDO
      ENDDO
C
C
C
C COMPUTE DIFFUSION COEFFICIENTS
C
      NC=0
      DO ITAU=1,1
C
C
C APPLY LOUIS PARAMETERIZATION
C
        CALL KFBLOUIS(LON,LAT,NIV,LDEB,WZ,WT,WU,WV,ZDKG)
C
C
C PROJECTION ONTO MODEL LEVELS
C        
        DO JL=LDEB,NIV-1
        DO J=1,LAT
        DO I=1,LON
          ZDKG(I,J,JL)=AMAX1(0.,ZDKG(I,J,JL)*2.0*
     +            (TMPRESS(I,J,JL+1)-TMPRESS(I,J,JL-1))
     +           /(GG*((WZ(I,J,JL-1)-WZ(I,J,JL+1))**2)))
        ENDDO      
        ENDDO
        ENDDO
        ZDKG(:,:,NIV)=0.0

C
C SAVE COEFFICIENTS
C        
        DO L=LDEB,NIV
        DO J=1,LAT
        DO I=1,LON
          DKGM (I,J,L)=DKGM (I,J,L) + ZDKG (I,J,L)
        ENDDO
        ENDDO
        ENDDO
C      
        NC=NC+1
C
C END OF LOOP OVER ANALYSIS TIMES
C
      ENDDO
C     
      DO L=LDEB,NIV
      DO J=1,LAT
      DO I=1,LON
        DKG (I,J,L)=DKGM (I,J,L)/REAL(NC)
      ENDDO
      ENDDO
      ENDDO
C
C 3. COMPUTE DIFFUSION MATRIX
C
      CZETA =1.0
      CZETAK=1.0
C
      DO L=LDEB,NIV
      DO J=1,LAT
      DO I=1,LON     
        AM(I,J,L)=MAX(AMMIN,SM(I,J,L)/SURFM(I,J))
        AK(I,J,L)=MAX(0.,DKG(I,J,L))*CZETAK
      ENDDO
      ENDDO
      ENDDO
C
C COMPUTE [I-M*dt]-1 AND NEW TRACER MASSES
C      
      CALL KFBCONVMA(LON,LAT,NIV,NTRA,IDEB,IFIN,JDEB,JFIN,
     +               XPHFM,SURFM,QQQM,LDEB,AM,AK)
C
C
      DEALLOCATE (DKGM)
      DEALLOCATE (DKG)
      DEALLOCATE (ZDKG)
      DEALLOCATE (WZ)
      DEALLOCATE (WT)
      DEALLOCATE (WU)
      DEALLOCATE (WV)
      DEALLOCATE (TMPRESS)
      DEALLOCATE (AM)
      DEALLOCATE (AK)
      DEALLOCATE (SM)
C
C
      RETURN
      END
