C
      SUBROUTINE LECTOBS(daynum,idate0)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Observations 
C   Purpose   : Read-in observational data file HOBS
C   Called by : 
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 03/1999
C
C Modified  : V.-H. Peuch, CNRM/GMGEC/ERAM, 01/2001
C
CCC
C
	USE PARADI_MOD
	USE EXPE_MOD
	USE MOCF_MOD
	USE DATOBS_MOD
C
	IMPLICIT NONE	
C
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      REAL (KIND=8) :: daynum
      INTEGER , DIMENSION(5) :: idate0
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: I,IOBS,J,K
	INTEGER :: NBDAY,NBHOUR
      CHARACTER(LEN=13) :: clf
	INTEGER , ALLOCATABLE :: idate1(:),nbjour(:)	
C
#ifdef PALM_RESEARCH
        LOGICAL, SAVE :: ll_obsfile
C
#endif
        ALLOCATE (idate1(5)) ; idate1(:) = 0
        ALLOCATE (nbjour(12)); nbjour(:) = 0
C      
	nbjour=(/31,28,31,30,31,30,31,31,30,31,30,31/)
C
CCC 0. READ A NEW OBSERVATIONAL FILE EVERY DAY
C
#ifdef PALM_RESEARCH
      IF (INT(daynum).EQ.0) THEN
#else
      IF ((INT(daynum).EQ.0).OR.
     +    (MOD(INT(daynum)+idate0(4)*3600,
     +     86400).EQ.0)) THEN
#endif
C
CCC 1. DETERMINE FILE NAME AND OPEN FILE
C
        NBDAY=int(daynum/86400.)
        NBHOUR=int((daynum-real(NBDAY)*86400.)/3600.)
C
        idate1(5)=0
        idate1(4)=idate0(4)+NBHOUR
        idate1(3)=idate0(3)+NBDAY
        idate1(2)=idate0(2)
        idate1(1)=idate0(1)
C
        IF (idate1(4).GE.24) THEN
          idate1(4)=idate1(4)-24
          idate1(3)=idate1(3)+1
        ENDIF
C
        IF (mod(idate0(1),4).EQ.0) THEN
          nbjour(2)=29
        ENDIF 
C
        IF (idate1(3).GT.nbjour(idate1(2))) THEN
          idate1(3)=idate1(3)-nbjour(idate1(2))
          idate1(2)=idate1(2)+1
        ENDIF
C
        IF (idate1(2).GT.12) THEN
          idate1(1)=idate1(1)+1
          idate1(2)=1
        ENDIF
C
            WRITE(clf,fmt='(''HOBS+'',I4.4,I2.2,I2.2)')
     +          idate1(1),idate1(2),idate1(3)
C
#ifdef PALM_RESEARCH
        INQUIRE(FILE=clf, EXIST=ll_obsfile)
        IF (ll_obsfile) THEN
#endif
        OPEN(unit=IFOBS,file=clf,form='formatted',err=99)
        GOTO 100
 99     WRITE(*,*) '>> ERROR OPENING OBS. FILE ',clf,' : ABORT!'
        CALL MOCAGE_STOP
C
CCC 2. READ-IN OBSERVATIONS
C
 100    READ(IFOBS,*) nobs,nos
C
        WRITE(*,*) '>> ',nobs,' OBSERVATIONS READ IN : ',clf
C
        WRITE(*,*) '>> ',nos,' DIFFERENT OS TODAY : '
        DO i=1,nos
          READ(IFOBS,101) defos(i)
          WRITE(*,*) ' > ',defos(i)
        ENDDO
C
        DO iobs=1,nobs
C
CCC 2.1 LOOP OVER OBSERVATIONS
C
          READ(IFOBS,102)
     +      idatobs(iobs,1),idatobs(iobs,2),
     +      idatobs(iobs,3),idatobs(iobs,4),idatobs(iobs,5),
     +      typemesure(iobs),ninst(iobs),
     +      xobslat(iobs),xobslon(iobs),
     +      nbnivpres(iobs)
#ifdef GAUSS
          if (xobslon(iobs).lt.0.) then
            xobslon(iobs)=xobslon(iobs)+360.
          endif
#endif
C
          DO j=1,nbnivpres(iobs)
C
CCC 2.2 LOOP OVER PRESSURE LEVELS
C
            READ(IFOBS,103)
     +        nivpres(iobs,j),nbconst(iobs,j)
C
            DO k=1,nbconst(iobs,j)
C
CCC 2.3 LOOP OVER CONSTITUENTS
C
              READ(IFOBS,104)
     +          constname(iobs,j,k),val(iobs,j,k)
C
            ENDDO
          END DO
        ENDDO
C
CCC 3. FORMATS
C
 101    FORMAT(A68)
 102    FORMAT(I4,I2.2,I2.2,I2.2,I2.2,1X,A6,1X,I5.5,1X,
     +         F6.2,1X,F7.2,1X,I2)
 103    FORMAT(F7.0,1X,I2)
 104    FORMAT(A5,1X,E12.5)
C
CCC
C
        CLOSE(IFOBS)
C
CCC 4. COMPUTE TIME FROM START (s) FOR ALL OBSERVATIONS
C
        DO iobs=1,nobs
          DO j=1,5
            idate1(j)=idatobs(iobs,j)
          ENDDO
          CALL timesec(idate0,idate1,nsecobs(iobs))
        END DO
C
CCC 5. INITIALIZE lobs (NEXT OBSERVATION TO PROCESS)
C
        DO i=1,nobsmax*nbnivpresmax*nbconstmax
          IF (IFLAG1) GUESS1(i,1,1)=-999.
          IF (IFLAG2) GUESS2(i,1,1)=-999.
          IF (IFLAG3) GUESS3(i,1,1)=-999.
        ENDDO
C
CCC 6. REJECTED OBS. ARE FLAGGED -888.
C
C       DO iobs=1,nobs
C         IF (nsecobs(iobs).EQ.-1) THEN
C           DO k=1,nbconst(iobs,j)
C           DO j=1,nbnivpres(iobs)
C             IF (IFLAG1) GUESS1(iobs,j,k)=-888.
C             IF (IFLAG2) GUESS2(iobs,j,k)=-888.
C             IF (IFLAG3) GUESS3(iobs,j,k)=-888.
C           ENDDO
C           ENDDO
C         ENDIF
C       ENDDO
C
C
#ifdef PALM_RESEARCH
       ELSE
           WRITE(*,*) '>> NO VALID OBSERVATION FILE ',trim(clf)
           nobs = 0
           nos = 0
       ENDIF
#endif
      ENDIF
C
	DEALLOCATE (idate1)
	DEALLOCATE (nbjour)
C
C
      RETURN
C
      END
C
CCC
C
