#ifdef PALM_RESEARCH
C
      SUBROUTINE MADONA(daynum,idate0,il_iter,FIRSTTIME,LASTTIME)
#else
C
C
      SUBROUTINE MADONA(daynum,idate0,LASTTIME)
#endif
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Data assimilation
C   Purpose    : Interface for data assimilation and misfit diagnostics
C   Called by  : MAIN
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 03/1999
C
C Modified  : V.-H. Peuch, CNRM/GMGEC/ERAM, 01/2001
C
CCC
C
	USE PARADI_MOD
	USE EXPE_MOD
	USE MOCF_MOD
	USE DATOBS_MOD
	USE FORMT_MOD
	USE MRAT_MOD
C
	IMPLICIT NONE
C          WRITE(6,*) 'From model ',daynum+DT0,idate0

C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	REAL (KIND=8) :: daynum
      LOGICAL :: LASTTIME
      INTEGER ,DIMENSION(5) :: idate0
C daynum   : time from start (s)
C idate0   : model initial date (1)=Year (2)=Month (3)=Day (4)=Hr (5)=Min
C LASTTIME : if current call is the last one for "madona" (LASTTIME=.T.),
C            do not call "LECTOBS" and prepare GUESS*A
C
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: ICOUNT,IOBS,I,LOBS,IPRES,ICST
	REAL (KIND=8) :: AWEIGHT,BWEIGHT
      LOGICAL :: FIRSTTIME
      INTEGER , ALLOCATABLE :: ILOBS(:)
#ifdef PALM_RESEARCH
      INTEGER :: il_iter
C
#else
      DATA FIRSTTIME/.TRUE./
      SAVE FIRSTTIME
#endif
C	
	ALLOCATE (ILOBS(nobstspmax)) ; ILOBS(:) = 0
C
#ifndef PALM_RESEARCH
#ifdef ELASSIM
C EL ASSIMILATION
C
      IF  (MOD(INT(daynum)+idate0(4)*3600,86400)
     +     .EQ.43200) THEN
        PRINT*,'>> EL ASSIMILATION IS PERFORMED'
        CALL ELASSIM
      ENDIF
#endif
#endif
C
C SELECT OBSERVATIONS IN CURRENT TIMESTEP
C
      icount=0
      DO iobs=1,nobs
        IF (icount.GT.nobstspmax) THEN
          WRITE(*,*) '>> TOO MANY OBS. FOR CURRENT TIMESTEP: ABORT!'
          CALL MOCAGE_STOP
        ENDIF
        IF (((daynum).LE.nsecobs(iobs)).AND.
     +      ((daynum+DT0).GT.nsecobs(iobs))) THEN
          icount=icount+1
          ILOBS(icount)=iobs
        ENDIF
      ENDDO
#ifdef PALM_RESEARCH
      IF (icount.EQ.0) THEN
        WRITE(*,*) '>> NO OBS. FOR CURRENT TIMESTEP'
      ENDIF
#endif
C
C SPATIAL INTERPOLATION : END OF CURRENT TIMESTEP / CURRENT OBSERVATIONS
C
      IF (FIRSTTIME) THEN
#ifndef PALM_RESEARCH
        FIRSTTIME=.FALSE.
#endif
        IF (icount.NE.0)
     $      CALL CGUESS(GUESS1A,GUESS2A,GUESS3A,GUESS4A,ILOBS,icount)
C
        RETURN
      END IF
C            
      IF (icount.NE.0) THEN
C
      CALL CGUESS(GUESS1B,GUESS2B,GUESS3B,GUESS4B,ILOBS,icount)
C
C TEMPORAL INTERPOLATION
C
      DO i=1,icount
C
        lobs=ILOBS(i)
C
        Bweight = (real(nsecobs(lobs)) - daynum)/DT0
        Aweight = 1. - Bweight
C
        IF (IFLAG1) THEN
          DO ipres=1,nbnivpres(lobs)
          DO icst=1,nbconst(lobs,ipres)
            GUESS1(lobs,ipres,icst) =
     +            Aweight*GUESS1A(i,ipres,icst) +
     +            Bweight*GUESS1B(i,ipres,icst)
C
          ENDDO
          ENDDO
C
        ENDIF
C
        IF (IFLAG2) THEN
          DO ipres=1,nbnivpres(lobs)
          DO icst=1,nbconst(lobs,ipres)
            GUESS2(lobs,ipres,icst) =
     +            Aweight*GUESS2A(i,ipres,icst) +
     +            Bweight*GUESS2B(i,ipres,icst)
C
          ENDDO
          ENDDO
C
        ENDIF
C
        IF (IFLAG3) THEN
          DO ipres=1,nbnivpres(lobs)
          DO icst=1,nbconst(lobs,ipres)
            GUESS3(lobs,ipres,icst) =
     +            Aweight*GUESS3A(i,ipres,icst) +
     +            Bweight*GUESS3B(i,ipres,icst)
C
          ENDDO
          ENDDO
C
        ENDIF
C
        IF (IFLAG4) THEN
          DO ipres=1,nbnivpres(lobs)
          DO icst=1,nbconst(lobs,ipres)
            GUESS4(lobs,ipres,icst) =
     +            Aweight*GUESS4A(i,ipres,icst) +
     +            Bweight*GUESS4B(i,ipres,icst)
C
C
          ENDDO
          ENDDO
C
        ENDIF
C
      ENDDO
C
      ENDIF
C
      IF (.NOT.LASTTIME) THEN
C
C CHECK IF DIAG FILES HAVE TO BE WRITTEN
C
#ifdef PALM_RESEARCH
      CALL WRTOBS(daynum+DT0,idate0,il_iter,.FALSE.)
#else
      CALL WRTOBS(daynum+DT0,idate0,.FALSE.)
#endif
C
C CHECK IF A NEW OBS. FILE IS NEEDED FOR NEXT TIMESTEP
C
      CALL LECTOBS(daynum+DT0,idate0)
C
C SELECT OBSERVATIONS IN NEXT TIMESTEP
C
      icount=0
      DO iobs=1,nobs
        IF (icount.GT.nobstspmax) THEN
          WRITE(*,*) '>> TOO MANY OBS. FOR NEXT TIMESTEP: ABORT!'
          CALL MOCAGE_STOP
        ENDIF
        IF (((daynum+DT0).LE.nsecobs(iobs)).AND.
     +      ((daynum+2.*DT0).GT.nsecobs(iobs))) THEN
          icount=icount+1
          ILOBS(icount)=iobs
        ENDIF
      ENDDO
C
C SPATIAL INTERPOLATION : END OF CURRENT TIMESTEP / CURRENT OBSERVATIONS
C
      IF (icount.NE.0) THEN
        CALL CGUESS(GUESS1A,GUESS2A,GUESS3A,GUESS4A,ILOBS,icount)
      ENDIF
C
      ENDIF
C
	DEALLOCATE (ILOBS)
C	
C
      RETURN
      END
C
CCC
C
