SUBROUTINE CHEM (nlon,nlat,niv,NIVBL,ntra,nsls,IVECMAX,    &
                 DLON,DLAT,SOLID,CLOUD,XSDAY,XDAY,DT,      &
                 TGRIM,XPAM,QQQM,HHHM,XALBSM,TRANAM,       &
                 SLSNAM,XPHFM,FCC,QLIQ,TJM,ICALPHOT)
!
!!!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!!! DATE : 15/05/2005
!
!   Type       : Chemistry
!   Purpose    : Compute chemistry according to current chemical scheme
!   Called by  : IOPC3D
!
!!!
!
!  Original : H. Teyssedre, V.-H. Peuch, 08/2000
!
!  Modified : D. Charpin, version fortran 90, 08/2006
!             H. Teyssedre, version avec couche limite simplifiee, 11/2006
!             B. Josse, differentiation 47/60 Niveaux pour appel chimie
!            
!!!
!
      USE PHOTO_MOD
      USE CSTES_MOD
!
      IMPLICIT NONE
#ifdef MPI
#include "mpif.h"
#endif
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
      INTEGER , INTENT(IN) :: nlon, nlat, niv, NIVBL, ntra, nsls
      INTEGER , INTENT(IN) :: IVECMAX
      REAL (KIND=8) , DIMENSION (nlon) , INTENT(IN) :: DLON
      REAL (KIND=8) , DIMENSION (nlat) , INTENT(IN) :: DLAT
      REAL (KIND=8) , INTENT(IN) :: XDAY, XSDAY, DT
      LOGICAL , INTENT(IN) :: SOLID,CLOUD,ICALPHOT
      REAL (KIND=8),DIMENSION (nlon,nlat,niv) , INTENT(INOUT) :: XPAM
      REAL (KIND=8),DIMENSION (nlon,nlat,niv) , INTENT(INOUT) :: TGRIM
      REAL (KIND=8),DIMENSION (nlon,nlat,niv)  , INTENT(INOUT) :: XPHFM
      REAL (KIND=8),DIMENSION (nlon,nlat)      , INTENT(IN) :: XALBSM
      REAL (KIND=8),DIMENSION (nlon,nlat,niv)  , INTENT(INOUT) :: FCC
      REAL (KIND=8),DIMENSION (nlon,nlat,niv)  , INTENT(INOUT) :: QLIQ
      REAL (KIND=8),DIMENSION (nlon,nlat,niv,nphot),INTENT(INOUT) :: TJM  
      REAL (KIND=8),DIMENSION (nlon,nlat,niv,ntra),INTENT(INOUT) :: QQQM  
      REAL (KIND=8),DIMENSION (nlon,nlat,niv,nsls),INTENT(INOUT) :: HHHM
      CHARACTER(LEN=15) , DIMENSION (ntra) , INTENT(IN) :: TRANAM
      CHARACTER(LEN=15) , DIMENSION (nsls) , INTENT(IN) :: SLSNAM
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!		
      INTEGER, PARAMETER :: nitermin=3
      INTEGER, PARAMETER :: nitermax=5
!
      INTEGER :: i,j,inivbl
      INTEGER :: NITER,NK,NHET,NIVCH
      INTEGER :: ITOPHET,IBOTHET,ITOPMAS
      INTEGER :: ilatpscmin,ilatpscmax,ilat,IVECPSCMIN,IVECPSCMAX
!
      REAL (KIND=8), PARAMETER :: DCHEMH2OMAX=0.05
      REAL (KIND=8), PARAMETER :: XLATPSCMIN=45.
      REAL (KIND=8), PARAMETER :: SZAlim=94.0
      REAL (KIND=8) :: XCHEM
!
#ifdef REPROBUS
#define CPP_NK 150
#define CPP_NHET 9
#endif
#ifdef RELACS
#define CPP_NK 242
#define CPP_NHET 9
#endif
#ifdef RACMOBUS
#define CPP_NK 372
#define CPP_NHET 9
#endif
      PARAMETER(nk=CPP_NK,nhet=CPP_NHET)
!
!!! INPUT VARIABLES
!
! TIMESTEPS VARYING WITH LEVEL
!
#ifdef REPROBUS
#include "REPROBUS_TIME"
#endif
#ifdef RELACS
#include "RELACS_TIME"
#endif
#ifdef RACMOBUS
#include "RACMOBUS_TIME"
#endif
!
!!! LOCAL VARIABLES
!
      INTEGER :: LOLANI,LOLANIBL,ITRA,ILON,IVEC,IPHOT,INIV,I1,I2
      INTEGER :: NCHEMSTEP,ISLS,IHET,IDTCHEM,ITER,iH2O,NIVHOx
      REAL (KIND=8) :: ZPAM,XCOEFF
      REAL (KIND=8) :: DTCHEM,DCHEMH2O
      REAL (KIND=8) , ALLOCATABLE :: o3t(:,:,:),sza(:,:),hnm(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: solH2O(:,:,:),solHNO3(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: vsed(:,:,:),photofact(:,:,:)
      LOGICAL , ALLOCATABLE :: day (:,:)
!
!!! SPLITTED ARRAYS
!
      REAL (KIND=8) , ALLOCATABLE :: SPLTtj (:,:), SPLTjco2(:)
      REAL (KIND=8) , ALLOCATABLE :: SPLTsza(:),SPLThnm(:),SPLTPAM(:)
      REAL (KIND=8) , ALLOCATABLE :: SPLTTGR(:),SPLTday(:),SPLTvsed(:)
!
      REAL (KIND=8) , ALLOCATABLE :: k(:,:),QM0(:,:),HM0(:,:)
      REAL (KIND=8) , ALLOCATABLE :: pr(:,:),pe(:,:)
      REAL (KIND=8) , ALLOCATABLE , TARGET :: QM(:,:),HM(:,:)
!
      REAL (KIND=8) , ALLOCATABLE :: het (:,:)
      REAL (KIND=8) , ALLOCATABLE :: partHNO3(:),partH2O(:)
      REAL (KIND=8) , ALLOCATABLE :: partHCl(:),partHBr(:)
      REAL (KIND=8) , ALLOCATABLE :: partHNO3s(:),partH2Os(:)
!
      REAL (KIND=8) , POINTER :: QMbis(:,:)
      REAL (KIND=8) , POINTER :: HMbis(:,:)
!
      CHARACTER(LEN=3) :: SOLVER
      DATA SOLVER / 'SIS' /
#ifdef SBL
!
      REAL (KIND=8) , ALLOCATABLE :: DELTABL(:),XPAMBL(:),XPHFMBL(:)
      REAL (KIND=8) , ALLOCATABLE :: TGRIMBL(:),hnmBL(:)
      REAL (KIND=8) , ALLOCATABLE :: QMBL(:,:),HMBL(:,:)
      REAL (KIND=8) , ALLOCATABLE :: TJMBL(:,:)
!
      REAL (KIND=8) , ALLOCATABLE :: XPAMNIVBL(:),XPHFMNIVBL(:)
      REAL (KIND=8) , ALLOCATABLE :: TGRIMNIVBL(:),QLIQNIVBL(:)
      REAL (KIND=8) , ALLOCATABLE :: FCCNIVBL(:),hnmNIVBL(:)
      REAL (KIND=8) , ALLOCATABLE :: QMNIVBL(:,:),HMNIVBL(:,:)
#endif
!
!!!
!
! 0. DYNAMICAL ALLOCATIONS AND INITIALIZATIONS
!
!!!
!
      ALLOCATE (o3t      (nlon,nlat,niv))       ; o3t      (:,:,:) = 0.0
      ALLOCATE (photofact(nlon,nlat,niv))       ; photofact(:,:,:) = 0.0
      ALLOCATE (hnm      (nlon,nlat,niv))
      ALLOCATE (solH2O   (nlon,nlat,niv))
      ALLOCATE (solHNO3  (nlon,nlat,niv))
      ALLOCATE (vsed     (nlon,nlat,niv))
      ALLOCATE (sza      (nlon,nlat))           ; sza        (:,:) = 0.0
      ALLOCATE (day      (nlon,nlat))           ; day        (:,:) = .false.
      ALLOCATE (SPLTtj  (IVECMAX,nphot))        ; SPLTtj     (:,:) = 0.0
      ALLOCATE (SPLTjco2(IVECMAX))              ; SPLTjco2     (:) = 0.0
      ALLOCATE (SPLTsza (IVECMAX))              ; SPLTsza      (:) = 0.0
      ALLOCATE (SPLThnm (IVECMAX))              ; SPLThnm      (:) = 0.0
      ALLOCATE (SPLTPAM (IVECMAX))              ; SPLTPAM      (:) = 0.0
      ALLOCATE (SPLTTGR (IVECMAX))              ; SPLTTGR      (:) = 0.0
      ALLOCATE (SPLTday (IVECMAX))              ; SPLTday      (:) = 0.0
      ALLOCATE (SPLTvsed(IVECMAX))              ; SPLTvsed     (:) = 0.0
      ALLOCATE (k   (IVECMAX,nk))               ; k          (:,:) = 0.0
      ALLOCATE (QM  (IVECMAX,ntra))             ; QM         (:,:) = 0.0
      ALLOCATE (QM0 (IVECMAX,ntra))             ; QM0        (:,:) = 0.0
      ALLOCATE (HM  (IVECMAX,nsls))             ; HM         (:,:) = 0.0
      ALLOCATE (HM0 (IVECMAX,nsls))             ; HM0        (:,:) = 0.0
      ALLOCATE (pr  (IVECMAX,ntra))             ; pr         (:,:) = 0.0
      ALLOCATE (pe  (IVECMAX,ntra))             ; pe         (:,:) = 0.0
      ALLOCATE (het (IVECMAX,nhet))             ; het        (:,:) = 0.0
      ALLOCATE (partHNO3 (IVECMAX))             ; partHNO3     (:) = 1.0
      ALLOCATE (partH2O  (IVECMAX))             ; partH2O      (:) = 1.0
      ALLOCATE (partHCl  (IVECMAX))             ; partHCl      (:) = 1.0
      ALLOCATE (partHBr  (IVECMAX))             ; partHBr      (:) = 1.0
      ALLOCATE (partHNO3s(IVECMAX))             ; partHNO3s    (:) = 0.0
      ALLOCATE (partH2Os (IVECMAX))             ; partH2Os     (:) = 0.0
#ifdef SBL
      ALLOCATE (DELTABL(IVECMAX))               ; DELTABL      (:) = 0.0
      ALLOCATE (XPAMBL (IVECMAX))               ; XPAMBL       (:) = 0.0
      ALLOCATE (XPHFMBL(IVECMAX))               ; XPHFMBL      (:) = 0.0
      ALLOCATE (TGRIMBL(IVECMAX))               ; TGRIMBL      (:) = 0.0
      ALLOCATE (hnmBL  (IVECMAX))               ; hnmBL        (:) = 0.0
      ALLOCATE (QMBL   (IVECMAX,ntra))          ; QMBL       (:,:) = 0.0
      ALLOCATE (HMBL   (IVECMAX,nsls))          ; HMBL       (:,:) = 0.0
      ALLOCATE (TJMBL  (IVECMAX,nphot))         ; TJMBL      (:,:) = 0.0
      ALLOCATE (XPAMNIVBL (IVECMAX))            ; XPAMNIVBL    (:) = 0.0
      ALLOCATE (XPHFMNIVBL(IVECMAX))            ; XPHFMNIVBL   (:) = 0.0
      ALLOCATE (TGRIMNIVBL(IVECMAX))            ; TGRIMNIVBL   (:) = 0.0
      ALLOCATE (QLIQNIVBL (IVECMAX))            ; QLIQNIVBL    (:) = 0.0
      ALLOCATE (FCCNIVBL  (IVECMAX))            ; FCCNIVBL     (:) = 0.0
      ALLOCATE (hnmNIVBL  (IVECMAX))            ; hnmNIVBL     (:) = 0.0
      ALLOCATE (QMNIVBL   (IVECMAX,ntra))       ; QMNIVBL    (:,:) = 0.0
      ALLOCATE (HMNIVBL   (IVECMAX,nsls))       ; HMNIVBL    (:,:) = 0.0
#endif
#ifdef MPI
      IF (NPROCS > 1) THEN
        ALLOCATE (QMbis(IVECMAX,ntra)) ; QMbis(:,:) = 0.0
        ALLOCATE (HMbis(IVECMAX,nsls)) ; HMbis(:,:) = 0.0
      ENDIF
!
      I1=i_deb(myid) ; I2 = i_fin(myid)
#else
      I1=1 ; I2 = IVECMAX
#endif
!
      lolani  =nlon*nlat*niv
      lolaniBL=nlon*nlat*NIVBL
!
! 0.1 DETECT INDEX CORRESPONDING TO WATER VAPOUR
!
      iH2O = 0
      DO itra=1,ntra
        IF (TRANAM(itra) == 'H_2O') iH2O=itra
      ENDDO
      IF (iH2O == 0) THEN
        WRITE(*,*) '>> WATER VAPOUR IS MISSING : ABORT!'
        CALL MOCAGE_STOP
      ENDIF
!
! 0.2 VARIOUS INDEXES FOR HETEROGENEOUS CHEMISTRY, SEMI-IMPLICIT 
!     CALCULATION FOR HOx ABOVE 1 hPa, MASS CONSERVATION DURING 
!     CHEMICAL TIMESTEP
!
      NIVHOx=0
      ITOPHET=0
      IBOTHET=0
#ifdef MASSy
      ITOPMAS =0
#endif
      DO iniv=1,niv
        ZPAM=0.
        DO ilon=1,NLON*NLAT
          ZPAM=ZPAM+XPAM(ilon,1,iniv)/REAL(nlon*nlat)
        ENDDO
        IF (ZPAM <=   100.) NIVHOx  = iniv
        IF (ZPAM <=  1500.) ITOPHET = iniv
        IF (ZPAM <= 30000.) IBOTHET = iniv
#ifdef MASSy
        IF (ZPAM <= 10000.) ITOPMAS = iniv
#endif
      ENDDO
!
      ITOPHET = MAX(ITOPHET,   1)
      IBOTHET = MIN(IBOTHET, niv)
#ifdef MASSy
      ITOPMAS = MAX(ITOPMAS,   1)
#endif
!
!!!
!
! 1. DENSITY (molec./cm3) OF AIR MOLECULES
!
!!!
!
!$omp parallel do default(none) private(iniv,ilon) &
!$omp shared(niv,nlon,nlat,hnm,XPAM,TGRIM,vsed,solH2O,solHNO3)
      DO iniv=1,niv
      DO ilon=1,nlon*nlat
        hnm    (ilon,1,iniv) = XPAM(ilon,1,iniv)/(TGRIM(ilon,1,iniv)*1.38e-17)
        vsed   (ilon,1,iniv) = 0.0
        solH2O (ilon,1,iniv) = 0.0
        solHNO3(ilon,1,iniv) = 0.0
      ENDDO
      ENDDO
!$omp end parallel do
!
!!!
!
! 2. COMPUTE SOLAR ZENITHAL ANGLE
!
!!!
!
      CALL ZENITH (DLON, DLAT, XSDAY, XDAY, sza, nlon, nlat)
!
      IF (ICALPHOT) THEN
!
!!!
!
! 3. COMPUTE TOTAL OZONE
!
!!!
!
        CALL TOTAL (HHHM, hnm, XPAM, o3t, SLSNAM, nlon, nlat, niv, nsls)
!
!!!
!
! 4. COMPUTE RATES FOR PHOTOLYSES
!
! 4.0 CLEAR SKY CONDITIONS
!
!!!
!
        CALL PHOT (day, sza, o3t, XALBSM, TJM, XPAM, nlon*nlat, niv, SZAlim)
!
! 4.1 DAY / NIGHT SWITCH FOR CHEMISTRY
!
!$omp parallel do default(none) private(IVEC) &
!$omp shared(IVECMAX,SPLTday,day)
        DO IVEC=1,IVECMAX
          SPLTday(IVEC) = 0.0 
          IF (day(IVEC,1)) SPLTday(IVEC) = 1.0
        ENDDO
!$omp end parallel do
!
!!!
!
! 4.2 MODIFY PHOTOLYSIS RATES ACCORDING TO CLOUDINESS
!
!!!
!
        IF (CLOUD) THEN
!
          CALL CLOUDPHOT(sza,QLIQ,XPHFM,FCC,photofact,nlon*nlat,niv)
!
!$omp parallel do default(none) private(iphot,ilon) &
!$omp shared(nlon,nlat,niv,iINDUVCS,TJM,photofact)
          DO iphot=1,nphot
            IF (iphot /= iINDUVCS) THEN
              DO ilon=1,nlon*nlat*niv
                TJM(ilon,1,1,iphot)= TJM(ilon,1,1,iphot)*photofact(ilon,1,1)
              ENDDO
            ENDIF
          ENDDO
!$omp end parallel do
!
        ENDIF
!
      ENDIF
#ifdef SBL
!
! SBL.0 KEEP VALUES AT LEVEL NIVBL
!
!$omp parallel do default(none) private(ivec) &
!$omp shared(IVECMAX,NIVBL,XPAM,XPAMNIVBL,XPHFM,XPHFMNIVBL,QLIQ,QLIQNIVBL) &
!$omp shared(FCC,FCCNIVBL,hnm,hnmNIVBL,TGRIM,TGRIMNIVBL)
      DO ivec = 1,IVECMAX
        XPAMNIVBL (ivec) = XPAM (ivec,1,NIVBL)
        XPHFMNIVBL(ivec) = XPHFM(ivec,1,NIVBL)
        TGRIMNIVBL(ivec) = TGRIM(ivec,1,NIVBL)
        QLIQNIVBL (ivec) = QLIQ (ivec,1,NIVBL)
        FCCNIVBL  (ivec) = FCC  (ivec,1,NIVBL)
        hnmNIVBL  (ivec) = hnm  (ivec,1,NIVBL)
      ENDDO
!$omp end parallel do
!
! SBL.1 THICKNESS OF BOUNDARY LAYER (BL)
!
!$omp parallel do default(none) private(ivec) &
!$omp shared(IVECMAX,NIV,NIVBL,XPHFM,XPHFMBL,DELTABL)
      DO ivec = 1,IVECMAX
        DELTABL(ivec)=XPHFM(ivec,1,NIV)-XPHFM(ivec,1,NIVBL-1)
        XPHFMBL(ivec)=XPHFM(ivec,1,NIV)
      ENDDO
!$omp end parallel do
!
! SBL.3 BOUNDARY LAYER AVERAGES FOR PRESSURE, TEMPERATURE AND DENSITY
!
! CONVENTION
!  varBL    : AVERAGED VALUE FOR THE WHOLE BL (BETWEEN NIVBL AND NIV)
!  varNIVBL : VALUE AT LEVEL NIVBL
!
      DO iniv=NIVBL,NIV
        DO ivec=1,IVECMAX
          XCOEFF = (XPHFM(ivec,1,iniv)-XPHFM(ivec,1,iniv-1))/DELTABL(ivec)
          XPAMBL (ivec) = XPAMBL (ivec) + XCOEFF*XPAM (ivec,1,iniv)
          TGRIMBL(ivec) = TGRIMBL(ivec) + XCOEFF*TGRIM(ivec,1,iniv)
          hnmBL  (ivec) = hnmBL  (ivec) + XCOEFF*hnm  (ivec,1,iniv)
        ENDDO
      ENDDO
!
! SBL.4 CONVERT MIXING RATIOS TO DENSITIES (molec/cm3)
!       AND KEEP MEAN CONCENTRATIONS OF BL
!
      DO iniv = NIVBL,niv
!
        DO itra=1,ntra
        DO ivec=1,IVECMAX
          XCOEFF = (XPHFM(ivec,1,iniv)-XPHFM(ivec,1,iniv-1))/DELTABL(ivec)
          QMBL(ivec,itra) = QMBL(ivec,itra) + XCOEFF*MAX(QQQM(ivec,1,iniv,itra), 1.e-30)
        ENDDO
        ENDDO
!
        DO isls=1,nsls
        DO ivec=1,IVECMAX
          XCOEFF = (XPHFM(ivec,1,iniv)-XPHFM(ivec,1,iniv-1))/DELTABL(ivec)
          HMBL(ivec,isls) = HMBL(ivec,isls) + XCOEFF*MAX(HHHM(ivec,1,iniv,isls), 1.e-30)
        ENDDO
        ENDDO
!
        DO iphot=1,nphot
        DO ivec=1,IVECMAX
          XCOEFF = (XPHFM(ivec,1,iniv)-XPHFM(ivec,1,iniv-1))/ DELTABL(ivec)
          TJMBL(ivec,iphot) = TJMBL(ivec,iphot) + XCOEFF*MAX(TJM(ivec,1,iniv,iphot), 1.e-30)
        ENDDO
        ENDDO
!
      ENDDO
!
!
! SBL.5 LEVEL NIVBL BECOMES TEMPORARY MAXIMUM SIZE OF ARRAYS
!       AFFECT AVERAGED BL VALUES TO LEVEL NIVBL
!
!$omp parallel default(none), &
!$omp private(ivec,itra,isls,iphot), &
!$omp shared(IVECMAX,ntra,nsls,NIVBL,XPAM,XPAMBL,XPHFM,XPHFMBL,TGRIM,TGRIMBL), &
!$omp shared(hnm,hnmBL,QQQM,QMBL,QMNIVBL,HHHM,HMBL,HMNIVBL,TJM,TJMBL)
!
!$omp do
      DO ivec = 1,IVECMAX
        XPAM (ivec,1,NIVBL) = XPAMBL (ivec)
        XPHFM(ivec,1,NIVBL) = XPHFMBL(ivec)
        TGRIM(ivec,1,NIVBL) = TGRIMBL(ivec)
        hnm  (ivec,1,NIVBL) = hnmBL  (ivec)
      ENDDO
!$omp end do
!
!$omp do
      DO itra = 1,ntra
      DO ivec = 1,IVECMAX
        QMNIVBL(ivec,itra) = QQQM(ivec,1,NIVBL,itra)
        QQQM(ivec,1,NIVBL,itra) = MAX(1.E-30,QMBL(ivec,itra))
      ENDDO
      ENDDO
!$omp enddo
!
!$omp do
      DO isls = 1,nsls
      DO ivec = 1,IVECMAX
        HMNIVBL(ivec,isls) = HHHM(ivec,1,NIVBL,isls)
        HHHM(ivec,1,NIVBL,isls) = MAX(1.E-30,HMBL(ivec,isls))
      ENDDO
      ENDDO
!$omp enddo
!
!$omp do
      DO iphot = 1,nphot
      DO ivec = 1,IVECMAX
        TJM(ivec,1,NIVBL,iphot) = TJMBL(ivec,iphot)
      ENDDO
      ENDDO
!$omp enddo
!
!$omp end parallel
#endif
!
!!!
!
! 5. SPLIT ARRAYS
!
!!!
!
      NIVCH=INT(lolaniBL/IVECMAX)
      IF (NIVCH /= NIVBL) THEN
       WRITE(*,*) 'Incoherence sur le nombre de niveaux chimiques'
       CALL MOCAGE_STOP
      ENDIF
!
#ifdef MIDATM 
      DO INIV = 1,NIVCH
#else
      DO INIV = 2,NIVCH
#endif
        NCHEMSTEP=ndtchem(INIV)
        IF (NCHEMSTEP > 0) THEN
!
!!!
!
          dtchem=DT/REAL(NCHEMSTEP)
!
          partH2O  =1.0
          partHNO3 =1.0
          partHCl  =1.0
          partHBr  =1.0
          partH2Os =0.0
          partHNO3s=0.0
          SPLTvsed =0.0
          het      =0.0
!
!$omp parallel do default(none) private(iphot,ivec) &
!$omp shared(IVECMAX,INIV,TJM,SPLTtj)
          DO iphot= 1,nphot
          DO IVEC = 1,IVECMAX
            SPLTtj(IVEC,iphot)=TJM(IVEC,1,INIV,iphot)
          ENDDO
          ENDDO
!$omp end parallel do
!
          IF (iphotCO2.GT.0) THEN
!$omp parallel do default(none) private(ivec) &
!$omp shared(IVECMAX,INIV,iphotCO2,TJM,SPLTjco2)
            DO IVEC = 1,IVECMAX
              SPLTjco2(IVEC)=TJM(IVEC,1,INIV,iphotCO2)
            ENDDO
!$omp end parallel do
          ENDIF
!
!$omp parallel do default(none) private(ivec) &
!$omp shared(IVECMAX,INIV,sza,SPLTsza,XPAM,SPLTPAM,TGRIM,SPLTTGR,hnm,SPLThnm)
          DO IVEC=1,IVECMAX
            SPLTsza(IVEC) = sza  (IVEC,1)
            SPLTPAM(IVEC) = XPAM (IVEC,1,INIV)
            SPLTTGR(IVEC) = TGRIM(IVEC,1,INIV)
            SPLThnm(IVEC) = hnm  (IVEC,1,INIV)
          ENDDO
!$omp end parallel do
!
! 6. CONVERT MIXING RATIOS TO DENSITIES (molec/cm3)
!
!$omp parallel do default(none) private(itra,ivec) &
!$omp shared(ntra,IVECMAX,INIV,QQQM,QM,QM0,SPLThnm)
          DO itra=1,ntra
          DO ivec=1,IVECMAX
            QM (ivec,itra) = MAX(QQQM(ivec,1,INIV,itra), 1.e-30)*SPLThnm(ivec)
            QM0(ivec,itra) = QM(ivec,itra)
          ENDDO
          ENDDO
!$omp end parallel do
!
!$omp parallel do default(none) private(isls,ivec) &
!$omp shared(nsls,IVECMAX,INIV,HHHM,HM,HM0,SPLThnm)
          DO isls=1,nsls
          DO ivec=1,IVECMAX
            HM (ivec,isls) = MAX(HHHM(ivec,1,INIV,isls), 1.e-30)*SPLThnm(ivec)
            HM0(ivec,isls) = HM(ivec,isls)
          ENDDO
          ENDDO
!$omp end parallel do
#ifdef SBL
!
          IF (INIV == NIVBL) THEN
!
            DO itra=1,ntra
            DO ivec=1,IVECMAX
              QMBL(ivec,itra) = QM(ivec,itra)
            ENDDO
            ENDDO
!
            DO isls=1,nsls
            DO ivec=1,IVECMAX
              HMBL(ivec,isls) = HM(ivec,isls)
            ENDDO
            ENDDO
!
          ENDIF
#endif
!
!!!
!
! 7. COMPUTE CHEMICAL REACTION RATES
!
!!!
!
! 7.1. PSC HETEROGENEOUS CHEMISTRY REACTION RATES
!
          IF ((INIV >= ITOPHET).AND.(INIV <= IBOTHET)) THEN
!
            CALL ANALYTIC (SOLID,SPLTTGR,SPLTPAM,SPLThnm,QM0,        &
                    SPLTvsed,het,partHNO3,partH2O,partHCl,partHBr,   &
                    partHNO3s,partH2Os,IVECMAX,ntra,nhet,TRANAM)
!
            ilatpscmin=1
            ilatpscmax=nlat
            DO ilat=1,nlat
              IF (DLAT(ilat) < (-XLATPSCMIN/CONV)) ilatpscmin=ilat
              IF (DLAT(ilat) < ( XLATPSCMIN/CONV)) ilatpscmax=ilat
            ENDDO
            IVECPSCMIN=(ilatpscmin-1)*nlon+1
            IVECPSCMAX=ilatpscmax*nlon
            DO ihet=1,nhet
            DO ivec=IVECPSCMIN,IVECPSCMAX
              het(ivec,ihet) = 0.0
            ENDDO
            ENDDO
            DO ivec=IVECPSCMIN,IVECPSCMAX
              partHNO3 (ivec) = 1.0
              partH2O  (ivec) = 1.0
              partHCl  (ivec) = 1.0
              partHBr  (ivec) = 1.0
              partHNO3s(ivec) = 0.0
              partH2Os (ivec) = 0.0
              SPLTvsed (ivec) = 0.0
            ENDDO
!
            DO ivec=1,IVECMAX
              vsed(ivec,1,INIV)=SPLTvsed (ivec)
            ENDDO
!
          ENDIF
!
! 7.2. CHEMICAL REACTION RATES
!
          CALL krates (SPLTPAM,SPLTTGR,SPLThnm,SPLTtj,het,k,IVECMAX,nphot,nhet,nk)
!
          if (SOLVER == 'SIS') then
!
#ifdef MPI
            IF (NPROCS > 1) THEN
              QMbis=0.
              HMbis=0.
              DO i=i_deb(myid),i_fin(myid) 
                DO j=1,ntra
                  QMbis(i,j)=QM(i,j)
                ENDDO
                DO j=1,nsls
                  HMbis(i,j)=HM(i,j)
                ENDDO
              END DO
            ELSE
              QMbis => QM
              HMbis => HM
            ENDIF
#else
            QMbis => QM
            HMbis => HM
#endif
!
!!!
!
! SEMI-IMPLICIT NUMERICAL SOLVER (SIS) :
!
! 8. COMPUTE CHEMISTRY AND TEMPORAL EVOLUTION FOR EACH SUB-TIMESTEP
!
!!!
!
            DO idtchem = 1,NCHEMSTEP
#ifndef SBL
              niter=nitermax
#else
!
! ITERATION NUMBER (BETWEEN nitermin AND nitermax)
!
              niter=MAX(nitermin,nitermax-NCHEMSTEP+1)
#endif
              DO iter = 1,niter
!
! 8.1. COMPUTE CHEMISTRY FOR SHORT-LIVED SPECIES
!
                CALL ch_equil (SPLTsza,k,SPLThnm,QMbis,HMbis,            &
                               partHNO3,partH2O,partHCl,partHBr,         &
                               IVECMAX,nk,ntra,nsls,INIV,NIVHOx,SZAlim,.FALSE.)
!
! 8.2. COMPUTE PRODUCTION AND DESTRUCTION TERMS FOR LONG-LIVED SPECIES
!      AND TRANSPORTED FAMILIES
!
                CALL ch_pterms (k,SPLTjco2,SPLThnm,SPLTday,              &
                                QMbis,HMbis,pr,pe,                       &
                                partHNO3,partH2O,partHCl,partHBr,        &
                                IVECMAX,nk,ntra,nsls)
!
! 8.3. TEMPORAL EVOLUTION FOR LONG-LIVED SPECIES AND FAMILIES UPON
!      ONE SUB-TIMESTEP
!
                CALL sis (IVECMAX,ntra,dtchem,QM0,QMbis,pr,pe)
#ifndef MASSy
                CALL ch_elem (QM0,QMbis,HMbis,IVECMAX,ntra,nsls)
#else
                IF (INIV >= ITOPMAS) THEN
                  CALL ch_elem (QM0,QMbis,HMbis,IVECMAX,ntra,nsls)
                ENDIF
#endif
!
              ENDDO
!
#ifndef MASSy
              CALL ch_elem (QM0,QMbis,HMbis,IVECMAX,ntra,nsls)
!
! PREVENT STRONG H2O CHEMICAL PRODUCTION OR DESTRUCTION (TEMPORARY?)
!
#ifdef MPI
              DO ivec=i_deb(myid),i_fin(myid)
#else
              DO IVEC=1,IVECMAX
#endif
                DCHEMH2O=ABS(QMbis(ivec,iH2O)-QM0(ivec,iH2O))/QM0(ivec,iH2O)
                IF (DCHEMH2O > DCHEMH2OMAX) QMbis(ivec,iH2O)=QM0(ivec,iH2O)
              ENDDO
#endif
!
              DO itra=1,ntra
#ifdef MPI
                DO ivec=i_deb(myid),i_fin(myid)
#else
                DO IVEC=1,IVECMAX
#endif
                  QM0(ivec,itra) = QMbis(ivec,itra)
                ENDDO
              ENDDO
!
              DO isls=1,nsls
#ifdef MPI
                DO ivec=i_deb(myid),i_fin(myid)
#else
                DO IVEC=1,IVECMAX
#endif
                  HM0(ivec,isls) = HMbis(ivec,isls)
                ENDDO
              ENDDO
!
! END OF LOOP UPON TIMESTEPS
!
            ENDDO
!
#ifdef MPI
            IF (NPROCS > 1) THEN
!
              call MPI_ALLREDUCE(QMbis,QM,ntra*IVECMAX,           &
                                 MPI_DOUBLE_PRECISION,MPI_SUM,    &
                                 MPI_COMM_MOCAGE,IERR)
!
              call MPI_ALLREDUCE(HMbis,HM,nsls*IVECMAX,           &
                                 MPI_DOUBLE_PRECISION,MPI_SUM,    &
                                 MPI_COMM_MOCAGE,IERR)   
!		
            ENDIF
!
#endif
          endif
!
!!!
!
! 9. CONVERT BACK TO MIXING RATIOS AND BUILD 3D ARRAYS
!
!!!
!
!$omp parallel do default(none) private(itra,ivec) &
!$omp shared(nsls,IVECMAX,ntra,INIV,QQQM,QM,SPLThnm)
          DO itra=1,ntra
          DO ivec=1,IVECMAX
            QQQM(ivec,1,INIV,itra)= MAX(QM(ivec,itra)/SPLThnm(ivec),1.e-30)
          ENDDO
          ENDDO
!$omp end parallel do
!
!$omp parallel do default(none) private(isls,ivec) &
!$omp shared(nsls,IVECMAX,INIV,HHHM,HM,SPLThnm)
          DO isls=1,nsls
          DO ivec=1,IVECMAX
            HHHM(ivec,1,INIV,isls)= MAX(HM(ivec,isls)/SPLThnm(ivec),1.e-30)
          ENDDO
          ENDDO
!$omp end parallel do
#ifdef SBL
!
          IF (INIV == NIVBL) THEN
!
!!!
!
! SBL.6 RETRIEVAL OF BL FROM SINGLE NIVBL CALCULATION
!       RESTORE BACK INITIAL BL VALUES AT LEVEL NIVBL
!
!!!
!
!$omp parallel default(none), &
!$omp private (ivec,itra,isls), &
!$omp shared (IVECMAX,ntra,nsls,NIVBL,XPAM,XPHFM,TGRIM,hnm), &
!$omp shared (XPAMNIVBL,XPHFMNIVBL,TGRIMNIVBL,hnmNIVBL), &
!$omp shared (QQQM,QMNIVBL,HHHM,HMNIVBL)
!
!$omp do
            DO ivec = 1,IVECMAX
              XPAM (ivec,1,NIVBL) = XPAMNIVBL (ivec)
              XPHFM(ivec,1,NIVBL) = XPHFMNIVBL(ivec)
              TGRIM(ivec,1,NIVBL) = TGRIMNIVBL(ivec)
              hnm  (ivec,1,NIVBL) = hnmNIVBL  (ivec)
            ENDDO
!$omp enddo
!
!$omp do
            DO itra = 1,ntra
            DO ivec = 1,IVECMAX
              QQQM(ivec,1,NIVBL,itra) = QMNIVBL(ivec,itra)
            ENDDO
            ENDDO
!$omp enddo
!
!$omp do
            DO isls = 1,nsls
            DO ivec = 1,IVECMAX
              HHHM(ivec,1,NIVBL,isls) = HMNIVBL(ivec,isls)
            ENDDO
            ENDDO
!$omp enddo
!
!$omp end parallel
!
! SBL.7 MODIFY INITIAL BL PROFILE
!
            DO inivbl = NIVBL,niv
!
              DO itra=1,ntra
              DO ivec=1,IVECMAX
                XCHEM=QM0(ivec,itra)/QMBL(ivec,itra)
                IF(XCHEM < 1.E+29) THEN
                  QQQM(ivec,1,inivbl,itra)=MAX(1.E-30,XCHEM*QQQM(ivec,1,inivbl,itra))
                ENDIF
              ENDDO
              ENDDO
!
              DO isls=1,nsls
              DO ivec=1,IVECMAX
                XCHEM=HM0(ivec,isls)/HMBL(ivec,isls)
                IF(XCHEM < 1.E+29) THEN
                  HHHM(ivec,1,inivbl,isls)=MAX(1.E-30,XCHEM*HHHM(ivec,1,inivbl,isls))
                ENDIF
              ENDDO
              ENDDO
!
            ENDDO
!
          ENDIF
#endif
!
          IF (SOLID) THEN
            DO ivec=1,IVECMAX
              solH2O (ivec,1,INIV) = partH2Os (ivec)
              solHNO3(ivec,1,INIV) = partHNO3s(ivec)
            ENDDO
          ENDIF
!
        ENDIF
!
! END OF LOOP UPON LEVELS
!
      ENDDO
!
!!!
!
! 10. POLAR STRATOSPHERIC CLOUDS SEDIMENTATION
!
!!!
!
      IF (SOLID) THEN
        CALL PSCSED (nlon,nlat,niv,ntra,DT,DLAT,              &
                     solH2O,solHNO3,vsed,                     &
                     XPAM,XPHFM,TGRIM,QQQM,TRANAM)
      ENDIF
!
!!!
!
! 11. APPLY CORRECTION FOR UV INDEX AND FLUXES (TEMPORARY) 
!
!!!
!
!$omp parallel do default(none) private(iphot,ilon,iniv) &
!$omp shared(iINDUV,iINDUVCS,iUVB,iUVA,nlon,nlat,niv,NIVBL,TJM)
      DO iphot=1,nphot
        IF ((iphot == iINDUV).OR.                            &
            (iphot == iINDUVCS).OR.                          &
            (iphot == iUVB).OR.                              &
            (iphot == iUVA)) THEN
          DO ilon=1,nlon*nlat*NIVBL
            TJM(ilon,1,1,iphot)=0.70*TJM(ilon,1,1,iphot)
          ENDDO
          IF (NIVBL <  niv) THEN
            DO iniv=NIVBL+1,niv
            DO ilon=1,nlon*nlat
              TJM(ilon,1,iniv,iphot)=0.70*TJM(ilon,1,NIVBL,iphot)
            ENDDO
            ENDDO
          ENDIF
        ENDIF
      ENDDO
!$omp end parallel do
!
!!!
!
      DEALLOCATE (o3t)
      DEALLOCATE (sza)
      DEALLOCATE (hnm)
      DEALLOCATE (solH2O)
      DEALLOCATE (solHNO3)
      DEALLOCATE (vsed)
      DEALLOCATE (photofact)
      DEALLOCATE (day)
      DEALLOCATE (SPLTtj)
      DEALLOCATE (SPLTjco2)
      DEALLOCATE (SPLTsza)
      DEALLOCATE (SPLThnm)
      DEALLOCATE (SPLTPAM)
      DEALLOCATE (SPLTTGR)
      DEALLOCATE (SPLTday)
      DEALLOCATE (SPLTvsed)
      DEALLOCATE (k)
      DEALLOCATE (QM)
      DEALLOCATE (QM0)
      DEALLOCATE (HM)
      DEALLOCATE (HM0)
      DEALLOCATE (pr)
      DEALLOCATE (pe)
      DEALLOCATE (het)
      DEALLOCATE (partHNO3)
      DEALLOCATE (partH2O)
      DEALLOCATE (partHCl)
      DEALLOCATE (partHBr)
      DEALLOCATE (partHNO3s)
      DEALLOCATE (partH2Os)
#ifdef SBL
      DEALLOCATE (DELTABL)
      DEALLOCATE (XPAMBL)
      DEALLOCATE (XPHFMBL)
      DEALLOCATE (TGRIMBL)
      DEALLOCATE (hnmBL)
      DEALLOCATE (TJMBL)
      DEALLOCATE (QMBL)
      DEALLOCATE (HMBL)
      DEALLOCATE (XPAMNIVBL)
      DEALLOCATE (XPHFMNIVBL)
      DEALLOCATE (TGRIMNIVBL)
      DEALLOCATE (QLIQNIVBL)
      DEALLOCATE (FCCNIVBL)
      DEALLOCATE (hnmNIVBL)
      DEALLOCATE (QMNIVBL)
      DEALLOCATE (HMNIVBL)
#endif
#ifdef MPI
      IF (NPROCS > 1) THEN
        DEALLOCATE (QMbis)
        DEALLOCATE (HMbis)
      ELSE
        NULLIFY (QMbis)
        NULLIFY (HMbis)
      ENDIF
#else
      NULLIFY (QMbis)
      NULLIFY (HMbis)
#endif
!
RETURN
END SUBROUTINE CHEM
