#ifdef PALM_RESEARCH
      SUBROUTINE INIT() 
#else
C
      PROGRAM MOCAGE
#endif
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : MAIN
C   Purpose    : MOCAGE MAIN
C
C
C    1A   2A                                         3A  4A  5A  6A  7A  8A
C   -----                                                               -----
C 1 | H |                                                               |He |
C   |---+----                                       --------------------+---|
C 2 |Li |Be |                                       | B | C | N | O | F |Ne |
C   |---+---|                                       |---+---+---+---+---+---|
C 3 |Na |Mg |3B  4B  5B  6B  7B |    8B     |1B  2B |Al |Si | P | S |Cl |Ar |
C   |---+---+---------------------------------------+---+---+---+---+---+---|
C 4 | K |Ca |Sc |Ti | V |Cr |Mn |Fe |Co |Ni |Cu |Zn |Ga |Ge |As |Se |Br |Kr |
C   |---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
C 5 |Rb |Sr | Y |Zr |Nb |Mo |Tc |Ru |Rh |Pd |Ag |Cd |In |Sn |Sb |Te | I |Xe |
C   |---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
C 6 |Cs |Ba |LAN|Hf |Ta | W |Re |Os |Ir |Pt |Au |Hg |Tl |Pb |Bi |Po |At |Rn |
C   |---+---+---+------------------------------------------------------------
C 7 |Fr |Ra |ACT|
C   -------------             __  __   _____   _____   _____   _____   _____
C                            |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C                            |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C                            |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C 
C               -------------------------------------------------------------
C    Lanthanide |La |Ce |Pr |Nd |Pm |Sm |Eu |Gd |Tb |Dy |Ho |Er |Tm |Yb |Lu |
C               |---+---+---+---+---+---+---+---+---+---+---+---+---+---+---|
C    Actinide   |Ac |Th |Pa | U |Np |Pu |Am |Cm |Bk |Cf |Es |Fm |Md |No |Lw |
C               -------------------------------------------------------------
C
C         METEO-FRANCE, CENTRE NATIONAL DE RECHERCHES METEOROLOGIQUES
C 
CCC
C 
C Purpose : MOCAGE describes the distribution and evolution of
C           the main tropospheric and stratospheric species using
C           a comprehensive chemical scheme coupled to a semi-
C           lagragian transport model and parameterizations of 
C           turbulent diffusion, convection, wet and dry depositions
C           and of surface emissions. Temperature, winds and humidity
C           driving both chemistry and physics are taken from
C           ARPEGE operational analyses. MOCAGE is an extension
C           of the REPROBUS 3D-CTM of the stratosphere [Lefevre et
C           al., 1994, 1998]. 
C
C Original : REPROBUS lib. 37
C
C Last Modification : V.-H. Peuch, CNRM/GMGEC/CATS, 02/2003
C		      P. Moinat and H. Le Berre Modification in forcage subroutine arguments  10/2006
C
CCC
C
      USE PARADI_MOD
      USE ALLOC_MOD
      USE MOCF_MOD
      USE MRAT_MOD
      USE EXPE_MOD
      USE CSTES_MOD
      USE UNDCOLMT_MOD
      USE SRC_MOD
      USE OUTNETCDF_MOD
      USE FORM_MOD
      USE FORMT_MOD
#ifdef PALM_RESEARCH
      USE palmlib
#endif
C
      IMPLICIT NONE
C
#ifdef MPI
#include "mpif.h"
#endif
C
#ifdef PALM_RESEARCH
C
C PALM VARIABLES DECLARATION
C
      CHARACTER(LEN=PL_LNAME) cl_name, cl_space
      CHARACTER(LEN=8) cl_domname
      INTEGER il_time, il_usrtag, il_err, il_err2
      INTEGER il_day, il_month, il_year, il_hour, il_min, il_sec
C
#endif
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: INICDF,IB,IDTMIN,IDT0MIN,NEND,NNDAYS,NNHOURS
      INTEGER :: IB_K,ISYNC,NCYCL,ISRC,II,NSECCY
      INTEGER :: NHCY,NOUT,NDOM,ierror
      INTEGER :: SELAPS,DELAPS,HELAPS,MELAPS
      REAL (KIND=8) :: DAYNUM,DAYNUM0,DAYNUMCTBTO
      CHARACTER(LEN=10) :: CHARAC1,CHARAC2
      LOGICAL :: LASTTIME
      INTEGER, DIMENSION(12) :: nbjour
      INTEGER, DIMENSION(5) :: idate0,idate1,idate00
C
c>>>
c
ccccc
c     __________________
c    |                  |
c    | 0. PREPARATIONS  |
c    |__________________|
C
C
#ifdef MPI
#ifndef PALM_RESEARCH
      CALL MPI_INIT(IERR)
#endif
#endif
C
#ifdef IBM
Cccccccccccccccccccccccccccccccccccccccc
#ifndef MPI
!$    CALL MPI_INIT(IERROR)
#endif
C
!$OMP PARALLEL
!$OMP CRITICAL
!$    call thread_bind()
!$OMP END CRITICAL
!$OMP END PARALLEL
C
Cccccccccccccccccccccccccccccccccccccccc      
#endif
C
#ifdef MPI
#ifdef PALM_RESEARCH
      CALL MPI_COMM_DUP(PL_COMM_EXEC,MPI_COMM_MOCAGE,IERR)
#else
      CALL MPI_COMM_DUP(MPI_COMM_WORLD,MPI_COMM_MOCAGE,IERR)
#endif
      CALL MPI_COMM_RANK(MPI_COMM_MOCAGE,MYID,IERR)
      CALL MPI_COMM_SIZE(MPI_COMM_MOCAGE,NPROCS,IERR)
C
      PRINT *,'======================================================'
      PRINT *,' PROCESSEUR ',MYID+1,' over ',NPROCS,' Rank:', MYID
      PRINT *,'======================================================'
#else
      NPROCS=1
      MYID=0
#endif
      MASTER=0
C
C
      nbjour= (/31,28,31,30,31,30,31,31,30,31,30,31/)
C
C	PARAMETER SETTING
C
C       BRANCH DIFFERENTIATION
#ifdef PALM_RESEARCH
      ig_run_mode = DIR_MODEL
#else
      ig_run_mode = DRY_RUN
#endif
C
      ierror = 0
      CALL LOAD_PARADI(ierror)
C
      IF (ierror .ne. 0) THEN
          WRITE(*,*) '!!! Error in load paradi'
          CALL MOCAGE_STOP
      ENDIF
C			
c     ___________________
c    |                   |
c    | 1. READ NAMELIST  |
c    |___________________| 
C
C
      open( unit = 66,
     $      file = 'namelist',
     $      form = 'formatted')
      read (66,*) idate00(1)
      read (66,*) idate00(2)
      read (66,*) idate00(3)
      read (66,*) idate00(4)
      read (66,*) idate1(1)
      read (66,*) idate1(2)
      read (66,*) idate1(3)
      read (66,*) idate1(4)
      read (66,*) NHCY
      read (66,*) NOUT
      read (66,*) NDOM
      read (66,*) NAMDOM1
      read (66,*) NAMDOM2
      read (66,*) NAMDOM3
      read (66,*) NAMDOM4
      read (66,*) CHEMSCHEME
      read (66,*) SOLID
      read (66,*) SURF
      read (66,*) PRMTRSP
      read (66,*) CLOUD
      read (66,*) OBS
      read (66,*) SRC
      read (66,*) TRANSINV
C
      read (66,*) NCDF
      if (NCDF .gt. 0) THEN 
          read (66,*) INICDF
          read (66,*) KTOTC
          READ (66,*) KNLEVELS
          IF (KNLEVELS .gt. 0) THEN
            ALLOCATE (iga_level(KNLEVELS))
            DO ib=1,KNLEVELS
              READ(66,*) iga_level(ib)
            END DO
          ELSE 
            KNLEVELS=nivm1
            ALLOCATE (iga_level(KNLEVELS)) ; iga_level(:) = 0
            DO ib_k = 1, nivm1
              iga_level(ib_k) = ib_k
            END DO
          END IF
          READ (66,*) KNSHORT
          IF (KNSHORT .gt. 0) THEN
            ALLOCATE (SPECIES(KNSHORT))
            DO ib=1,KNSHORT
              READ(66,*) SPECIES(ib)
            END DO
          ELSE
            ALLOCATE (SPECIES(1))
          ENDIF
        ENDIF
C END
      close(66)
C
C     Chemscheme sanity check
C
#ifdef CARIOLLE
      IF (TRIM(CHEMSCHEME) .ne. 'CARIOLLE') THEN
#endif
#ifdef REPROBUS
      IF (TRIM(CHEMSCHEME) .ne. 'REPROBUS') THEN
#endif
#ifdef RELACS
      IF (TRIM(CHEMSCHEME) .ne. 'RELACS') THEN
#endif
#ifdef RACMOBUS
      IF (TRIM(CHEMSCHEME) .ne. 'RACMOBUS') THEN
#endif
#ifdef ACCIDENT
      IF (TRIM(CHEMSCHEME) .ne. 'ACCIDENT') THEN
#endif
            
      WRITE(*,*) 'Chemscheme in namelist differs from compilation'
      CALL MOCAGE_STOP
      ENDIF
C
#ifdef PALM_RESEARCH
C
C     Level sanity check
C
#ifdef NIV_91
      IF ( NIVM1 .ne. 91 ) THEN
        WRITE(PL_OUT,*) 'Model compiled for 91 levels but invoked for ',
#endif
#ifdef NIV_60
      IF ( NIVM1 .ne. 60 ) THEN
        WRITE(PL_OUT,*) 'Model compiled for 60 levels but invoked for ',
#endif
#ifdef NIV_47
      IF ( NIVM1 .ne. 47 ) THEN
        WRITE(PL_OUT,*) 'Model compiled for 47 levels but invoked for ',
#endif
     +        NIVM1
        CALL PALM_Abort(il_err)
      ENDIF
#endif
c
c Dates :
c
c  - idate00 : YYYY,MM,DD,HH,MIN (begin)
c  - idate1  : YYYY,MM,DD,HH,MIN (end)
c
c At present, inputs/outputs are only possible
c for full hours (MIN=0)
c
      idate00(5)=0
      idate1(5)=0
c
c Flags :
c
c  - IFLAG# : if .T., take domain # into account
c  - IDGRI# : if .T., model domain # = forcings domain #
c 
      WRITE(*,*) '>> MOCAGE RUN :'
      WRITE(*,*) '>> CHEMICAL SCHEME is ',CHEMSCHEME
      IF (ig_run_mode .eq. DRY_RUN) WRITE (*,*) '>> RUN MODE is DRY RUN'
      IF (ig_run_mode .eq. DIR_MODEL)
     $    WRITE (*,*) '>> RUN MODE is DIRECT MODEL for ASSIMILATION'
      WRITE(*,100) '>> ',NDOM,' DOMAINS ARE CONSIDERED'
      IFLAG1=.TRUE.
      IDGRI1=.TRUE.
      WRITE(*,101) ' > DOMAIN 1 IS : ',NAMDOM1
      if (NDOM.GE.2) then
        IFLAG2=.TRUE.
        IDGRI2=.TRUE.
        WRITE(*,101) ' > DOMAIN 2 IS : ',NAMDOM2
      else
        IFLAG2=.FALSE.
        IDGRI2=.FALSE.
      endif
      if (NDOM.GE.3) then
        IFLAG3=.TRUE.
        IDGRI3=.TRUE.
        WRITE(*,101) ' > DOMAIN 3 IS : ',NAMDOM3
      else
        IFLAG3=.FALSE.
        IDGRI3=.FALSE.
      endif
      if (NDOM.GE.4) then
        IFLAG4=.TRUE.
        IDGRI4=.TRUE.
        WRITE(*,101) ' > DOMAIN 4 IS : ',NAMDOM4
      else
        IFLAG4=.FALSE.
        IDGRI4=.FALSE.
      endif
c
 100  FORMAT(A3,I1,A23)
 101  FORMAT(A23)
c
C       MEMORY ALLOCATION
C
      CALL ALLOC_GLOBAL
c
c Logical units for I/O :
c
c  - IFRED# : input history "HM" files for domain #
c  - IFORC# : forcing "FM" files for domain #
c  - IFSOR# : output history "HM" files for domain #
c  - IFSFX# : surface "SM" fluxes for domain # 
c
      IFRED1=11
      IFORC1=21
      IFSOR1=31
      IFSFX1=41
      IFRED2=12
      IFORC2=22
      IFSOR2=32
      IFSFX2=42
      IFRED3=13
      IFORC3=23
      IFSOR3=33
      IFSFX3=43
      IFRED4=14
      IFORC4=24
      IFSOR4=34
      IFSFX4=44
C
      IF (OBS) THEN
        IFOBS=50
        IFDIAG1=51
        IFDIAG2=52
        IFDIAG3=53
        IFDIAG4=54
      ENDIF
c
c Timestep controls :
c
c  - DT : timestep for physics/chemistry (15 min = 900 s)
c  - DTCONV : timestep for parameterized transport (30 min)
c  - NSCY : cycle length for 3D forcings (s)
c  - NITERT : number of iterations of dynamics/physics per cycle
c  - DT0 : timestep for resolved scale transport (s)
c  - NITAP : number of iterations for semi-lag. back-trajectories
c  - NITRA : number of points on the semi-lag. back-trajectories
c  - DTN : timestep for the semi-lag. back-trajectories 
c
      DT=15.*60.
c
c one call to convection every 2 timestep 
c
      DTCONV=2.*DT
      NSCY=NHCY*3600
      NITERT=NHCY
      DT0=REAL(NSCY)/REAL(NITERT)
      NITAP=2
      NITRA=3
      DTN=DT0/REAL(NITRA)
c
      IDTMIN=int(DT)/60
      IDT0MIN=int(DT0)/60
      write(*,204) '>> TIMESTEP FOR PHYSICS/CHEMISTRY IS : '
     $             ,IDTMIN,' (min)'
      write(*,205) '>> TIMESTEP FOR DYNAMICS IS : '
     $             ,IDT0MIN,' (min)'
c
c Simulation controls :
c
c  - NOUT : frequency of output of HM history files (s)
c  - NCDF : frequency of output of NetCDF files (s) (<0 for no output)
c  - NEND : length of the simulation (s)
c 
      write(*,200) '>> SIMULATION BEGINS ON : ',idate00(1),
     $             idate00(2),idate00(3),idate00(4)
#ifdef PALM_RESEARCH
c If the model is working as direct model for assimilation, it can only
c be a forward model and its duration is set by assimilation
      NEND = ip_assimsize
      write(*,*) '>> AND LASTS ',NEND, 'seconds'
#else
      write(*,200) '>> SIMULATION ENDS ON   : ',idate1(1),
     $             idate1(2),idate1(3),idate1(4)
C
      IF (.NOT.TRANSINV) THEN
        CALL TIMESEC(idate00,idate1,NEND)
        IF (NEND.LE.0) THEN
          WRITE(*,*) '>> END IS BEFORE BEGINNING: ABORT!'
          CALL MOCAGE_STOP  
        ENDIF
      ELSE
        CALL TIMESEC(idate1,idate00,NEND)
        WRITE(*,*) '>> MOCAGE IS RUNNING BACKWARDS!'
        IF (NEND.LE.0) THEN 
          WRITE(*,*) '>> BEGINNING IS BEFORE END: ABORT!'
          CALL MOCAGE_STOP 
        ENDIF
      ENDIF
#endif
C
      NNDAYS=NEND/86400
      NNHOURS=mod(NEND,86400)/3600
      write(*,201) '>> LENGTH : ',NNDAYS,' (d) AND '
     $            ,NNHOURS,' (h)'
      write(*,202) '>> OUTPUT HISTORY FILE EVERY ',NOUT,
     $             '(h) AND/OR AT THE END'
      NOUT=NOUT*3600
C
      IF (NCDF .gt. 0) THEN
          write(*,202) '>> OUTPUT NetCDF  FILE EVERY ',NCDF,
     $             '(h) AND/OR AT THE END'
          NCDF=NCDF*3600
          IF (KTOTC.ne.0) WRITE(*,*) '>> Total columns in NetCDF files'
      ENDIF
C
      LASTTIME=.FALSE.
c
c Synchronize simulation beginning and forcing cycles
c 
c  - idate0 : date of first forcing cycle
c  - NCYCLT : total number of cycles
c
      idate0(1)=idate00(1)
      idate0(2)=idate00(2)
      idate0(3)=idate00(3)
      idate0(4)=idate00(4)
      idate0(5)=idate00(5)
C
      ISYNC=MOD(idate0(4),NHCY)
      IF (ISYNC.NE.0) THEN
        IF (.NOT.TRANSINV) THEN
          idate0(4)=idate0(4)-ISYNC
        ELSE
          ISYNC=NHCY-ISYNC
          idate0(4)=idate0(4)+ISYNC
          IF (idate0(4).GE.24) THEN
            idate0(4)=idate0(4)-24
            idate0(3)=idate0(3)+1
          ENDIF
          IF (mod(idate0(1),4).EQ.0) THEN
            nbjour(2)=29
          ENDIF 
          IF (idate0(3).GT.nbjour(idate0(2))) THEN 
            idate0(3)=idate0(3)-nbjour(idate0(2))
            idate0(2)=idate0(2)+1
          ENDIF 
          IF (idate0(2).GT.12) THEN
            idate0(1)=idate0(1)+1
            idate0(2)=1
          ENDIF 
        ENDIF
      ENDIF
C
      daynum0=ISYNC*3600
      if (ISYNC.NE.0) then
        NCYCLT=MAX(1,NEND/NSCY+1)
      else
        NCYCLT=MAX(1,NEND/NSCY)
      endif
      NCYCL=1
c
      print*,'NCYCLT=',NCYCLT
c
 200  FORMAT(A26,I4.4,I2.2,I2.2,I2.2)
 201  FORMAT(A12,I1.1,A9,I2.2,A4)
 202  FORMAT(A29,I2.2,A21)
 203  FORMAT(A14,I1.1,A5,I2.2,A5,I2.2,A21)
 204  FORMAT(A39,I2.2,A6)
 205  FORMAT(A30,I2.2,A6)
 206  FORMAT(A45,I2.2,A1,I2.2,A4,I2.2,A1,I2.2)
C
C 3D-MODEL
C
      L1D=.FALSE.
C
C TAKE POINT SOURCES INTO ACCOUNT (OPTIONAL)
C
      IF (SRC) THEN
#ifdef ACCIDENT
      CALL INSOURCE
#endif              
      ENDIF
c
ccccc
c     ________________________________
c    |                                |
c    | 2. INITIALIZE TRANSPORT MODEL  |
c    |________________________________|
c
ccccc
c
      CALL INICSTE
C
#ifdef PALM_RESEARCH
C
C The interface of FORCAGE has one argument more in case of assimilation
C
      IF (.NOT.TRANSINV) THEN
        CALL FORCAGE(-REAL(NSCY),idate0,.false.)
      ELSE
        CALL FORCAGE(0.,idate0,.false.)
      ENDIF
#else
C
      IF (.NOT.TRANSINV) THEN
        CALL FORCAGE(-REAL(NSCY),idate0)
      ELSE
        CALL FORCAGE(0.,idate0)
      ENDIF
#endif
C
      CALL INITRANS
c
ccccc
c     ___________________________________________________________________
c    |                                                                   |
c    | 3. INITIALIZE CHEMISTRY, TRACER 3D DISTRIBUTIONS & TOP BOUNDARIES |
c    |___________________________________________________________________|
c
ccccc
c
#ifdef CARIOLLE
          if (CHEMSCHEME.EQ.'CARIOLLE') call INICARIO3(idate00)
#else
          if ((CHEMSCHEME.NE.'TRACERS0').AND.
     +        (CHEMSCHEME.NE.'TRACERCO').AND.
     +        (CHEMSCHEME.NE.'ACCIDENT').AND.
     +        (CHEMSCHEME.NE.'AEROSOLS').AND.
     +        (CHEMSCHEME.NE.'CARIOLLE')) call INIPHOT
#endif
C
      call INICT3D(idate00)
C      
#ifndef MIDATM
          call INITOP
#endif
#ifndef ACCIDENT
          call INIHENRY
#endif
c
ccccc
c     _______________________________________
c    |                                       |
c    | 4. READ-IN INITIAL SUFACE FORCINGS    |
c    |_______________________________________|
c
ccccc
c
          if (SURF) call SURFLUX(0.,idate0)
c
ccccc
c     _______________________________________
c    |                                       |
c    | 4. READ-IN INITIAL OBSERVATIONS FILE  |
c    |_______________________________________|
c
ccccc
c
      IF (OBS) THEN
          IF (myid .eq. master) CALL LECTOBS(0.,idate0)
      END IF
C
#ifdef PALM_RESEARCH
C
C Final puts of the INIT unit
#include "palm_iniputs.h"
C Begin of the MODEL unit
#include "palm_modelinit.h"
C
#else
c
ccccc
c     _______________
c    |               |
c    | 6. MAIN LOOP  |
c    |_______________|
c
ccccc
      IF ((NCDF .gt. 0 .and. INICDF .ne. 0).or.OBS) THEN
C          
        IF (IFLAG1) THEN
          XPAM1   = XPAM1A 
          XPHFM1  = XPHFM1A
          TGRIM1  = TGRIM1A
          UGRIM1  = UGRIM1A
          VGRIM1  = VGRIM1A
          WGRIM1  = WGRIM1A
          PVGRIM1 = PVGRIM1A 
          PSOLM1  = PSOLM1A
        ENDIF
C
        IF (IFLAG2) THEN
          XPAM2   = XPAM2A 
          XPHFM2  = XPHFM2A
          TGRIM2  = TGRIM2A 
          UGRIM2  = UGRIM2A 
          VGRIM2  = VGRIM2A 
          WGRIM2  = WGRIM2A 
          PVGRIM2 = PVGRIM2A
          PSOLM2  = PSOLM2A
        ENDIF
C
        IF (IFLAG3) THEN
          XPAM3   = XPAM3A 
          XPHFM3  = XPHFM3A
          TGRIM3  = TGRIM3A
          UGRIM3  = UGRIM3A
          VGRIM3  = VGRIM3A
          WGRIM3  = WGRIM3A
          PVGRIM3 = PVGRIM3A
          PSOLM3  = PSOLM3A
        ENDIF
C
        IF (IFLAG4) THEN
          XPAM4   = XPAM4A 
          XPHFM4  = XPHFM4A
          TGRIM4  = TGRIM4A 
          UGRIM4  = UGRIM4A 
          VGRIM4  = VGRIM4A 
          WGRIM4  = WGRIM4A 
          PVGRIM4 = PVGRIM4A
          PSOLM4  = PSOLM4A
        ENDIF
C
      END IF
c
      IF (NCDF .gt. 0 .and. INICDF .ne. 0)
     $        CALL OUTNETCDF(daynum0,idate0,idate00,-1)
C
      daynum=daynum0-DT
      daynumCTBTO=-DT

C
      IF (OBS .and. myid .eq. master)
     $    CALL MADONA(daynum0,idate0,.FALSE.)
c
  600 continue
c
ccc   avance temporelle
c
      daynum = daynum+DT
      daynumCTBTO=daynumCTBTO+DT

c
      SELAPS=int(daynum-daynum0)
      DELAPS=SELAPS/86400
      HELAPS=mod(SELAPS,86400)/3600
      MELAPS=mod(SELAPS,3600)/60 
c
#ifdef SBL
#ifdef NIV_91
      NIVBL=78
#endif
#ifdef NIV_60
      NIVBL=49
#endif
#ifdef NIV_47
      NIVBL=41
#endif
      if (MELAPS.EQ.0) then
        if (mod(HELAPS,6).EQ.0) then
          NIVBL=NIVM1
        endif
      endif
#else
      NIVBL=NIVM1
#endif
      WRITE(*,203) '>> TIMESTEP : ',DELAPS,' (d) '
     $      ,HELAPS,' (h) ',MELAPS
     $      ,' (min) FROM BEGINNING'
C
#endif
C
c
c  6.1 Check if new forcing files are needed 
c
#ifdef PALM_RESEARCH
C
C The interface of FORCAGE has one argument more in case of assimilation
C
      IF  ((int(daynum).GT.0).AND.(int(daynum).NE.pastassimtime)) THEN
        call FORCAGE(daynum,idate0,.TRUE.)
      ENDIF  
#else
      IF (daynum.GT.0.) THEN
          call FORCAGE(daynum,idate0)
      END IF

#endif
c
      IF (SURF) THEN
          IF (daynum.GT.0.) call SURFLUX(daynum,idate0)
      ENDIF
c
c  6.2 Semi-lagrangian advection
c
      NSECCY=mod(int(daynum),NSCY)
      if (mod(NSECCY,NSCY/NITERT).EQ.0) then
        NITER=NSECCY/(NSCY/NITERT)+1
        write(*,206)
     $  ' > SEMI-LAGRANGIAN ADVECTION IS PERFORMED : I',
     $  NITER,'/',NITERT,' - C',NCYCL,'/',NCYCLT
        if (NITER.EQ.NITERT) NCYCL=NCYCL+1 
c
        CALL ADVECSL
      endif
c
c  6.3 3-D interface with parameterizations
c
      call IOPC3D(daynum,idate0)
C
#ifdef PALM_RESEARCH
C
 206  FORMAT(A45,I2.2,A1,I2.2,A4,I2.2,A1,I2.2)
C
C include the PALM_Puts
C
#include "palm_modelputs.h"
C
#else
c
c  6.4 Diagnostics and data assimilation
c
      IF (mod(NSECCY+int(DT),NSCY/NITERT).EQ.0.and.myid.eq.master) THEN
        IF ((SELAPS+DT0).GE.NEND) LASTTIME=.TRUE.
        IF (OBS) CALL MADONA(daynum-DT0+DT,idate0,LASTTIME)
      ENDIF
c
ccccc
c     __________________________________________
c    |                                          |
c    | 7. OUTPUT HISTORY AND DIAGNOSTICS FILES  |
c    |__________________________________________|
c
ccccc
C
      IF (mod(SELAPS+int(DT),NOUT).eq.0.)
     $     call OUTHISTO(daynum+DT,idate0,-1)
#ifdef ACCIDENT
      IF (.NOT.TRANSINV) THEN
        IF (mod(SELAPS+int(DT),NOUT).eq.0.)
     $     CALL OUTSOURCE(daynum+DT,idate0)
      ELSE
       CALL OUTCTBTO(daynumCTBTO+DT,NOUT,IDATE00,IDATE1)
      ENDIF
#endif      
c
      IF (NCDF .gt. 0 .and. mod(SELAPS+int(DT),NCDF).eq.0.)
     $     call OUTNETCDF(daynum+DT,idate0,idate00,-1)
C
      IF ((SELAPS+int(DT)).LT.NEND) goto 600
c
c Save history file for later restart
c
      IF (mod(SELAPS+int(DT),NOUT).ne.0.)
     $    call OUTHISTO(daynum+DT,idate0,-1)
c
      IF (NCDF .gt. 0 .and. mod(SELAPS+int(DT),NCDF).ne.0.)
     $    call OUTNETCDF(daynum+DT,idate0,idate00,-1)
c
c Write diagnostic file
c
#ifdef MPI
      IF ((MYID.EQ.MASTER)) THEN
#endif
          IF (OBS) CALL WRTOBS(daynum+DT0,idate0,.TRUE.)
C
#ifdef MPI
      ENDIF
#endif
C
#endif
c
c>>>
c
C
C	FREE DYNAMIC MEMORY
C	
#ifdef PALM_RESEARCH
C
C We have to define here a NEW UNIT
C to clean up what INIT has allocated once at the end
C
#else
C
        CALL DEALLOC_GLOBAL
        if (NCDF .gt. 0) THEN
            DEALLOCATE (SPECIES)
            DEALLOCATE (iga_level)
        ENDIF
C
#ifdef MPI
        CALL MPI_FINALIZE(IERROR)
        STOP
#else
#ifdef IBM
!$      CALL MPI_FINALIZE(IERROR)
#endif
        STOP
#endif		
C
#endif
C
      END
