      SUBROUTINE NEBUL(TGRIM,QGRIM,XPAM,XPHFM,LPFLX,SPFLX,
     +                 FCC,ZNEBC,QLIQ,QICE,
     +                 DT,LON,LAT,NIVDEB,NIV)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Physical parameterizations
C   Purpose    : Compute fractional cloud cover at, below and
C                above each level
C   Called by  :
C
CCC
C
C  Original : V.-H. Peuch, 09/2000
C             (based upon ACNEBN by J.-M Piriou, CNRM/GMAP)
C
C  Modified : 
C
CCC
	USE CSTES_MOD
C
	IMPLICIT NONE
C
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: LON,LAT,NIV,NIVDEB
      REAL (KIND=8) :: DT
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: TGRIM,QGRIM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: XPAM,XPHFM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: LPFLX,SPFLX
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: FCC,ZNEBC
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: QLIQ,QICE
C TGRIM : TEMPERATURE (K) AT MODEL LEVEL
C QGRIM : SPECIFIC HUMIDITY (kg/kg) AT MODEL LEVEL
C XPAM  : PRESSURE (Pa) AT MODEL LEVEL
C XPHFM : PRESSURE (Pa) AT HALF MODEL LEVEL
C LPFLX : LIQUID CONVECTIVE PRECIPITATION FLUX
C SPFLX : SOLID CONVECTIVE PRECIPITATION FLUX
C FCC     : 3D CLOUD COVER
C ZNEBC   : 3D CONVECTIVE CLOUD COVER
C QLIQ    : LIQUID WATER CONTENT (kg/kg)
C QH2O    : SOLID WATER CONTENT (kg/kg)
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: I,J,J1,J2
      REAL (KIND=8) :: ALPHA,PCP,TCVQSAT,ZDSN,ZD3
      REAL (KIND=8) :: ZMASK1,ZMASK2,ZFRAC,ZDELTAP,ZQSS,ZNEBS
      REAL (KIND=8) :: ZQSC,ZQS,ZDELTA,ZPLS
      REAL (KIND=8) , ALLOCATABLE :: ZSDELP(:,:,:),ZQSSTPP(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: PAPHIF(:,:,:),PAPHIH(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: ZS(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: XQSAT(:,:,:)
C
      ALLOCATE (ZSDELP(LON,LAT,NIV)) ; ZSDELP(:,:,:) =0.0
      ALLOCATE (ZQSSTPP(lON,LAT,NIV)) ; ZQSSTPP(:,:,:) =0.0
      ALLOCATE (PAPHIF(LON,LAT,NIV)) ; PAPHIF(:,:,:) =0.0
      ALLOCATE (PAPHIH(LON,LAT,NIV)) ; PAPHIH(:,:,:) =0.0
      ALLOCATE (XQSAT(LON,LAT,NIV)) ; XQSAT(:,:,:) =0.0
      ALLOCATE (ZS(LON,LAT,NIV)) ; ZS(:,:,:) =0.0
C
C COMPUTE GEOPOTENTIAL AT FULL LEVELS
C
      DO I=1,LON*LAT
        PAPHIH(I,1,NIV) = 0.
        PAPHIF(I,1,NIVDEB) = 0.
      ENDDO
C    
      DO J=NIV-1,NIVDEB,-1
        DO I=1,LON*LAT
          PAPHIH(I,1,J) = PAPHIH(I,1,J+1)
     +                 +(RD+QGRIM(I,1,J+1)*(RV-RD))*TGRIM(I,1,J+1)
     +                  *LOG(XPHFM(I,1,J+1)/XPHFM(I,1,J))
          ALPHA= (XPHFM(I,1,J+1)-XPAM(I,1,J))
     +          /(XPHFM(I,1,J+1)-XPHFM(I,1,J))
          PAPHIF(I,1,J+1)=ALPHA*PAPHIH(I,1,J)+(1.-ALPHA)*PAPHIH(I,1,J+1)
        ENDDO
      ENDDO
C
C COMPUTE DRY STATIC ENERGY
C
      DO J=NIVDEB,NIV
        DO I=1,LON*LAT
          PCP        = CPD + CPV*QGRIM(I,1,J)/(1.-QGRIM(I,1,J))
          ZS(I,1,J)    = PCP*TGRIM(I,1,J)+PAPHIF(I,1,J)
          ZQSSTPP(I,1,J) = 0.
          ZSDELP(I,1,J)  = 0.
        ENDDO
      ENDDO
C
C COMPUTE SATURATION POINT
C
      DO J=NIVDEB,NIV
        DO I=1,LON*LAT
          XQSAT(I,1,J)=TCVQSAT(TGRIM(I,1,J),XPAM(I,1,J))
        ENDDO
      ENDDO
C
C CLOUD CONTENTS AND CLOUDINESS DIAGNOSTIC
C
      DO J1=NIV,NIVDEB+1,-1
        DO J2=J1,NIV
          DO I=1,LON*LAT
            ZDSN        = (ZS(I,1,J1)-ZS(I,1,J2))/MAX(ZEPSDS,QSSC)
            ZD3         = 1.+ZDSN*ZDSN*(2.*ZDSN-3.)
            ZMASK1      = MAX(0.,SIGN(1.,-ZDSN))
            ZMASK2      = MAX(0.,SIGN(1.,1.-ZDSN))
            ZFRAC       = ZMASK1+(1-ZMASK1)*ZMASK2*ZD3
            ZDELTAP     = ZFRAC*(XPHFM(I,1,J2)-XPHFM(I,1,J2-1))
            ZQSSTPP(I,1,J1) = ZQSSTPP(I,1,J1)+QSSUSS*
     +                    MAX(0.,QGRIM(I,1,J2)-PHUC(J1)*XQSAT(I,1,J1))
     +                   *ZDELTAP
            ZSDELP(I,1,J1)  = ZSDELP(I,1,J1)+ZDELTAP
          ENDDO
        ENDDO
C
        DO I=1,LON*LAT
C
C STRATIFORM PART
C
          ZQSS  = ZQSSTPP(I,1,J1)/ZSDELP(I,1,J1)
          ZQSS  = QSUSX*(1.-EXP(-ZQSS/QSUSX))
          ZNEBS = MAX(ZEPSNEB,MIN(1.-ZEPSNEB,
     +                QSNEBS*SQRT(ZQSS/XQSAT(I,1,J1))))
C
C CONVECTIVE PART
C
          ZQSC      = MAX(0.,LPFLX(I,1,J1)+SPFLX(I,1,J1)
     +                      -LPFLX(I,1,J1-1)-SPFLX(I,1,J1-1))
     +                /(XPHFM(I,1,J1)-XPHFM(I,1,J1-1))          
     +                *GG*QSSUSC*DT
          ZQSC      = QSUSX*(1.-EXP(-ZQSC/QSUSX))
          ZNEBC(I,1,J1) = MAX(ZEPSNEB,MIN(1.-ZEPSNEB,QSNEBC*ZQSC))
C
C TOTAL
C
          ZQS       = ZQSS+ZQSC
          FCC(I,1,J1) = ZNEBS+(1.-ZNEBS)*ZNEBC(I,1,J1)
C
C SNOW
C
          ZDELTA     = MAX(0.,SIGN(1.,RTT-TGRIM(I,1,J1)))
          ZPLS       = ZDELTA*
     +                 (1.-EXP(-(RTT-TGRIM(I,1,J1))**2*ZGAUSS))
          QLIQ(I,1,J1) = ZQS*(1-ZPLS)
          QICE(I,1,J1) = ZQS*ZPLS
C
C QLIQ AND QICE APPLY TO CLOUDS AND NOT TO THE WHOLE GRIDCELL
C
          QLIQ(I,1,J1) = QLIQ(I,1,J1)/FCC(I,1,J1)
          QICE(I,1,J1) = QICE(I,1,J1)/FCC(I,1,J1)
        ENDDO
      ENDDO
C
	DEALLOCATE (ZSDELP)
	DEALLOCATE (ZQSSTPP)
	DEALLOCATE (PAPHIF)
	DEALLOCATE (PAPHIH)
	DEALLOCATE (XQSAT)
	DEALLOCATE (ZS)
C
C
      RETURN
      END
