      SUBROUTINE O3ELCALC(LON,LAT,NIV,NTHETA,
     +                    TGRIM,XPAM,XGRIT,
     +                    ELGRIT)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Data assimilation
C   Purpose    : Compute gridded Equivalent Latitude and locate
C                observations in (theta,EL) coordinates
C   Called by  :
C
CCC
C
C Original  : M.-L. Cathala & V.-H. Peuch, CNRM/GMGEC/ERAM, 01/2002
C
C Modified  : 
C
CCC
C
	USE PARADI_MOD
	USE CSTES_MOD
	USE MOCF_MOD
	USE GRIDS_MOD
	USE DATOBS_MOD
	USE THELOBS_MOD
C	
	IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: LON,LAT,NIV,NTHETA
	REAL (KIND=8) , DIMENSION(LON,LAT,NIV) :: TGRIM,XPAM
	REAL (KIND=8) , DIMENSION(LON,LAT,NTHETA) :: XGRIT
	REAL (KIND=8) , DIMENSION (LON,LAT,NTHETA):: ELGRIT
C     OBS IN (THETA,EQUIVALENT LATITUDE) COORDINATES
C     ELGRIT : Equivalent latitude to X in (lon,lat,theta) coordinates 
C	     
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: DIM_TEMP
C HLB DIM_TEMP pour dimensionner des tableaux de dimension 1 non correctement
C		   allous dans les versions prcdentes de mocage.
	INTEGER :: LOLANI,LOLA,ITHETA,ILON,I,ILAT,IBOX,ISUM,IGOOD
	INTEGER :: LOBS,K,ICONST
	REAL (KIND=8) :: A,XMAXN,XMINN,XMAXS,XMINS,DELTAXN,DELTAXS
	REAL (KIND=8) , ALLOCATABLE :: NTH(:),THGRIM(:,:,:)
	REAL (KIND=8) , ALLOCATABLE :: ELboxN(:),ELboxS(:)
	REAL (KIND=8) , ALLOCATABLE :: THGRIM_BIG(:,:,:,:)
	REAL (KIND=8) , ALLOCATABLE :: XELUP(:,:),XELDN(:,:)
	REAL (KIND=8) , ALLOCATABLE :: XUP(:),XDN(:)
	REAL (KIND=8) , ALLOCATABLE :: XX0(:),YY0(:),P0(:),XINTER(:,:)
	REAL (KIND=8) , ALLOCATABLE :: ELGRITN(:,:,:),ELGRITS(:,:,:)
	INTEGER , ALLOCATABLE :: INDX(:),INDY(:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: ALFX1(:),ALFX2(:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: ALFX3(:),ALFX4(:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: ALFY1(:),ALFY2(:)
	REAL (KIND=8) , ALLOCATABLE , SAVE :: ALFY3(:),ALFY4(:)
C
C
CCC 0. INITIALIZATIONS
C
	DIM_TEMP=1
C
        ALLOCATE (INDX(DIM_TEMP))	  ; INDX(:) = 0
        ALLOCATE (INDY(DIM_TEMP))	  ; INDY(:) = 0
        ALLOCATE (NTH(NTHETA))
        ALLOCATE (THGRIM(LON,LAT,NIV))
        ALLOCATE (THGRIM_BIG(LON,LAT,NIV,nbconstmax))
                  THGRIM_BIG(:,:,:,:) = 0.0
        ALLOCATE (ELboxN(NBOX))	  ; ELboxN(:) = 0.0
        ALLOCATE (ELboxS(NBOX))	  ; ELboxS(:) = 0.0
        ALLOCATE (XELUP(LON,LAT))	  ; XELUP(:,:) = 0.0
        ALLOCATE (XELDN(LON,LAT))	  ; XELDN(:,:) = 0.0
        ALLOCATE (XUP(DIM_TEMP))	  ; XUP(:) = 0.0
        ALLOCATE (XDN(DIM_TEMP))	  ; XDN(:) = 0.0
        ALLOCATE (XX0(DIM_TEMP))	  ; XX0(:) = 0.0
        ALLOCATE (YY0(DIM_TEMP))	  ; YY0(:) = 0.0
        ALLOCATE (P0(nbnivpresmax))	  ; P0(:) = 0.0
        ALLOCATE (XINTER(nbnivpresmax,nbconstmax))	  ; XINTER(:,:) = 0.0
        ALLOCATE (ELGRITN(LON,LAT,NTHETA))
        ALLOCATE (ELGRITS(LON,LAT,NTHETA))
        ALLOCATE (ALFX1(DIM_TEMP))	  ; ALFX1(:) = 0.0
        ALLOCATE (ALFX2(DIM_TEMP))	  ; ALFX2(:) = 0.0
        ALLOCATE (ALFX3(DIM_TEMP))	  ; ALFX3(:) = 0.0
        ALLOCATE (ALFX4(DIM_TEMP))	  ; ALFX4(:) = 0.0
        ALLOCATE (ALFY1(DIM_TEMP))	  ; ALFY1(:) = 0.0
        ALLOCATE (ALFY2(DIM_TEMP))	  ; ALFY2(:) = 0.0
        ALLOCATE (ALFY3(DIM_TEMP))	  ; ALFY3(:) = 0.0
        ALLOCATE (ALFY4(DIM_TEMP))	  ; ALFY4(:) = 0.0
C
      LOLANI = LON*LAT*NIV
      LOLA   = LON*LAT
      DO itheta=1,NTHETA
        NTH(itheta)=THETAMIN+REAL(itheta-1)*DELTATH
      ENDDO
C
CCC 1. COMPUTE POTENTIAL TEMPERATURE
C
      DO ilon = 1,LOLANI
        THGRIM(ilon,1,1)= TGRIM(ilon,1,1)*
     +                    (100000./XPAM(ilon,1,1))**(2./7.)
      ENDDO
C
      DO i=NIV-1,2,-1
        DO ilon=1,LOLA
          IF ((THGRIM(ilon,1,i).LT.THGRIM(ilon,1,i+1)).AND.
     +        (THGRIM(ilon,1,i).LT.THGRIM(ilon,1,i-1))) THEN
             a=(XPAM(ilon,1,i+1)-XPAM(ilon,1,i))/
     +         (XPAM(ilon,1,i+1)-XPAM(ilon,1,i-1))
             THGRIM(ilon,1,i)=     a*THGRIM(ilon,1,i-1)+
     +                        (1. -a)*THGRIM(ilon,1,i+1)
          ENDIF
        ENDDO
      ENDDO
C
CCC 2. COMPUTE GRIDDED EQUIVALENT LATITUDE 
C      WARNING : GLOBAL DOMAIN ONLY, YET
C
      DO ilon=1,LON*LAT*NTHETA
         ELGRITN(ilon,1,1)=0.
         ELGRITS(ilon,1,1)=0.
      ENDDO
C
C LOOP ON THETA LEVELS
C
      DO itheta=1,NTHETA
        xmaxN=0.
        xminN=1.
        xmaxS=0.
        xminS=1.
C
C NORTHERN HEMISPHERE (NH)
C
        DO ilat=LADEMI+1,LAT
        DO ilon=1,LON
          IF (xmaxN.LT.XGRIT(ilon,ilat,itheta))
     +        xmaxN=XGRIT(ilon,ilat,itheta)
          IF (xminN.GT.XGRIT(ilon,ilat,itheta))
     +        xminN=XGRIT(ilon,ilat,itheta)
        ENDDO
        ENDDO 
C
C SOUTHERN HEMISPHERE (SH)
C
        DO ilat=1,LADEMI
        DO ilon=1,LON
          IF (xmaxS.LT.XGRIT(ilon,ilat,itheta))
     +        xmaxS=XGRIT(ilon,ilat,itheta)
          IF (xminS.GT.XGRIT(ilon,ilat,itheta))
     +        xminS=XGRIT(ilon,ilat,itheta)
        ENDDO
        ENDDO
C
        deltaxN=(xmaxN-xminN)/real(NBOX)
        deltaxS=(xmaxS-xminS)/real(NBOX)
C
CCC 2.1 COMPUTE EQUIVALENT SURFACE FOR EACH EQUIVALENT LATITUDE BOX
C
        DO ibox=1,NBOX
          ELboxN(ibox)=0.
          ELboxS(ibox)=0.
        ENDDO
C
C HN
C
        DO ilat=LADEMI+1,LAT
        DO ilon=1,LON   
          ibox=MIN(INT((XGRIT(ilon,ilat,itheta)
     +         -xminN)/deltaxN)+1,NBOX)
          ELboxN(ibox)=ELboxN(ibox)+CLAT1(ilat)
        ENDDO
        ENDDO
C
C HS
C
        DO ilat=1,LADEMI
        DO ilon=1,LON
          ibox=MIN(INT((XGRIT(ilon,ilat,itheta)
     +         -xminS)/deltaxS)+1,NBOX)
          ELboxS(ibox)=ELboxS(ibox)+CLAT1(ilat)
        ENDDO
        ENDDO
C
CCC 2.2 INTEGRATION OF SURFACE AREAS
C
        DO ibox=1,NBOX-1
          DO isum=ibox+1,NBOX
            ELboxN(ibox)=ELboxN(ibox)+ELboxN(isum)
            ELboxS(ibox)=ELboxS(ibox)+ELboxS(isum)
          ENDDO
        ENDDO 
C
        DO ibox=NBOX,1,-1
          ELboxN(ibox)=MIN(ELboxN(ibox)/ELboxN(1),1.)
          ELboxS(ibox)=MIN(ELboxS(ibox)/ELboxS(1),1.)
          ELboxN(ibox)=ASIN(1.-ELboxN(ibox))
          ELboxS(ibox)=ASIN(1.-ELboxS(ibox))
        ENDDO
C
CCC 2.3 COMPUTE O3EL
C
C HN
C
        DO ilat=LADEMI+1,LAT
        DO ilon=1,LON
          igood=MIN(INT((XGRIT(ilon,ilat,itheta)-xminN)/
     +                  deltaxN)+1,NBOX)
          ELGRITN(ilon,ilat,itheta)=ELboxN(igood)*90./XPIH
        ENDDO
        ENDDO
C
C HS
C
        DO ilat=1,LADEMI
        DO ilon=1,LON
          igood=MIN(INT((XGRIT(ilon,ilat,itheta)-xminS)/
     +                  deltaxS)+1,NBOX)
          ELGRITS(ilon,ilat,itheta)=-ELboxS(igood)*90./XPIH
        ENDDO
        ENDDO
C
        DO ilat=1,LADEMI-1
          DO ilon=1,LON
            ELGRIT(ilon,ilat,itheta)=ELGRITS(ilon,ilat,itheta)
          ENDDO
        ENDDO
C
        DO ilon=1,LON
          ELGRIT(ilon,LADEMI,itheta)=0.
          ELGRIT(ilon,LADEMI+1,itheta)=0.
        ENDDO
C
        DO ilat=LADEMI+2,LAT
          DO ilon=1,LON
            ELGRIT(ilon,ilat,itheta)=ELGRITN(ilon,ilat,itheta)
          ENDDO
        ENDDO
C
C END OF LOOP ON THETA LEVELS
C
      ENDDO
C
CCC 3. COMPUTE THETA AND EQUIVALENT LATITUDE FOR OBS. POINTS
C
      DO i=1,nobsmax*nbnivpresmax
        THobs(i,1)=-999.
        ELobs(i,1)=-999.
      ENDDO
C
CCC 3.0 COMPUTE THETA : 3D SPATIAL INTERPOLATION
C
      DO lobs=1,nobs
      IF (typemesure(lobs).EQ.'FLIGHT') THEN
C
        DO k=1,nbnivpres(lobs)
          P0(k)=nivpres(lobs,k)
        ENDDO
C
	  do iconst=1,nbconstmax
	    THGRIM_BIG(:,:,:,iconst)=THGRIM(:,:,:)
	  enddo
C
        CALL HOBS3D(THGRIM_BIG,LON,LAT,NIV,1,
     +              xobslon(lobs),xobslat(lobs),P0,
     +              nbnivpres(lobs),
     +              1,
     +              XINTER)
C
        DO k=1,nbnivpres(lobs)
          THobs(lobs,k)=XINTER(k,1)
        ENDDO
C
CCC 3.1 COMPUTE COEFFICIENTS FOR HORIZONTAL INTERPOLATION
C
        XX0=xobslon(lobs)/CONV
        YY0=xobslat(lobs)/CONV
C
        CALL POIDREG(XX0,RLONM1(1)-DRLONM1,DRLONM1
     +              ,INDX,ALFX1,ALFX2,ALFX3,ALFX4,1)
        CALL POIDREG(YY0,RLATM1(1)-DRLATM1,DRLATM1
     +              ,INDY,ALFY1,ALFY2,ALFY3,ALFY4,1)
C
CCC 3.2 LOOP ON OBS. VERTICAL LEVELS
C
        DO k=1,nbnivpres(lobs)
C
C FIND NEIGHBOURING THETA LEVELS
C
          igood=1
          DO itheta=2,NTHETA-1
            IF (THobs(lobs,k).GE.NTH(itheta)) igood=itheta
          ENDDO
          igood=MIN(igood,NTHETA-1)
          DO ilon=1,LOLA
            XELUP(ilon,1)=ELGRIT(ilon,1,igood+1)
            XELDN(ilon,1)=ELGRIT(ilon,1,igood)
          ENDDO
C
C COMPUTE EQUIVALENT LATITUDE ON UPPER THETA LEVEL
C
          CALL INTER2D(XELUP,LON,LAT,
     +                 INDX,INDY,
     +                 ALFX1,ALFX2,ALFX3,ALFX4,
     +                 ALFY1,ALFY2,ALFY3,ALFY4,
     +                 XUP,1,+1.,.FALSE.,1)
C
C COMPUTE EQUIVALENT LATITUDE ON LOWER THETA LEVEL
C
          CALL INTER2D(XELDN,LON,LAT,
     +                 INDX,INDY,
     +                 ALFX1,ALFX2,ALFX3,ALFX4,
     +                 ALFY1,ALFY2,ALFY3,ALFY4,
     +                 XDN,1,+1.,.FALSE.,1)
C
C LINEAR INTERPOLATION BETWEEN THETA LEVELS
C
          a=-(NTH(igood)-THobs(lobs,k))/(NTH(igood+1)-NTH(igood))
          ELobs(lobs,k)=a*XUP(1)+(1.-a)*XDN(1)
        ENDDO
      ENDIF
      ENDDO
C
CCC
C
	DEALLOCATE (INDX)
	DEALLOCATE (INDY)
	DEALLOCATE (NTH)
	DEALLOCATE (THGRIM)
	DEALLOCATE (THGRIM_BIG)
	DEALLOCATE (ELboxN)
	DEALLOCATE (ELboxS)
	DEALLOCATE (XELUP)
	DEALLOCATE (XELDN)
	DEALLOCATE (XUP)
	DEALLOCATE (XDN)
	DEALLOCATE (XX0)
	DEALLOCATE (YY0)
	DEALLOCATE (P0)
	DEALLOCATE (XINTER)
	DEALLOCATE (ELGRITN)
	DEALLOCATE (ELGRITS)
	DEALLOCATE (ALFX1)
	DEALLOCATE (ALFX2)
	DEALLOCATE (ALFX3)
	DEALLOCATE (ALFX4)
	DEALLOCATE (ALFY1)
	DEALLOCATE (ALFY2)
	DEALLOCATE (ALFY3)
	DEALLOCATE (ALFY4)
C
C
      RETURN
      END
