module cstes_phot
      USE PHOTO_MOD

      ! SOLAR ZENITHAL ANGLE

      REAL (KIND=8) , dimension(nsza) :: dsza = (/ 0., 20., 30., 40., 50., 60., &
                                                  70., 75., 80., 82., 84., 86., &
                                                  88., 90., 91., 92., 93., 94. /)

      ! STANDARD OZONE COLUMN OVERHEAD (300 DU)

      REAL (KIND=8) , dimension(0:79) :: o3up =          &
            (/   8.06100E+18, 7.99462E+18, 7.93299E+18,  &
                 7.87640E+18, 7.82456E+18, 7.77509E+18,  &
                 7.72605E+18, 7.67619E+18, 7.62288E+18,  &
                 7.55935E+18, 7.47512E+18, 7.35867E+18,  &
                 7.20383E+18, 7.02010E+18, 6.82213E+18,  &
                 6.60994E+18, 6.37186E+18, 6.09497E+18,  &
                 5.77106E+18, 5.40316E+18, 5.00378E+18,  &
                 4.58758E+18, 4.16749E+18, 3.75388E+18,  &
                 3.35406E+18, 2.97193E+18, 2.61180E+18,  &
                 2.28056E+18, 1.98340E+18, 1.72074E+18,  &
                 1.48913E+18, 1.28297E+18, 1.09794E+18,  &
                 9.32756E+17, 7.86977E+17, 6.58450E+17,  &
                 5.45450E+17, 4.47977E+17, 3.65469E+17,  &
                 2.96289E+17, 2.38581E+17, 1.91052E+17,  &
                 1.52537E+17, 1.21657E+17, 9.69863E+16,  &
                 7.72759E+16, 6.15336E+16, 4.89398E+16,  &
                 3.88474E+16, 3.07951E+16, 2.43773E+16,  &
                 1.93481E+16, 1.55141E+16, 1.25080E+16,  &
                 1.00583E+16, 8.08306E+15, 6.51296E+15,  &
                 5.24082E+15, 4.17979E+15, 3.30623E+15,  &
                 2.59925E+15, 2.02695E+15, 1.56351E+15,  &
                 1.18802E+15, 8.82908E+14, 6.45413E+14,  &
                 4.70041E+14, 3.46896E+14, 2.65532E+14,  &
                 2.05609E+14, 1.55031E+14, 1.17098E+14,  &
                 9.29087E+13, 7.53165E+13, 5.99234E+13,  &
                 4.67292E+13, 3.68336E+13, 2.91371E+13,  &
                 2.30897E+13, 1.86917E+13  /)

      ! RATIO OF OZONE COLUMN OVER STANDARD OZONE COLUMN 

      REAL (KIND=8) , dimension(nozo) :: xv3 = (/0.4, 0.6, 0.8, 1.0, 1.2, 1.4, 1.6/)

      ! SURFACE ALBEDO
      REAL (KIND=8) , dimension(nalb) :: albd = (/0.01, 0.2, 0.4, 0.6/)

end module cstes_phot

SUBROUTINE PHOT(day,SZA,O3T,PALBS,TJM,XPAM,IMAX,niv,SZAlim)
!
!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!!! DATE : 15/05/2005
!
!   Type       : Chemistry
!   Purpose    : Compute photodissociation frequencies (J values, in
!                s-1) by log-linear interpolation from look-up tables
!   Called by  : CHEM
!
!  Original : F. Lefevre (REPROBUS, LIB37)
!
!  Modified : V.-H.Peuch (inclusion dans MOCAGE), 08/1999
!             V.-H.Peuch (nouvelle version), 11/2000
!             P.Simon(optimisation),03/2003
!             D.Charpin(version Fortran-90),08/2006
!
! INPUT  
!           XPAM  = pressure (Pa)
!           SZA   = solar zenith angle
!           O3T   = ozone column
!           PALBS = surface albedo
!
!  OUTPUT
!           TJM  = photodiss. freq. (up to 49 photolyses)
!
USE PHOTO_MOD
USE CSTES_MOD
USE cstes_phot
!
IMPLICIT NONE
!
!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
INTEGER :: IMAX,NIV
REAL (KIND=8) :: SZAlim
REAL (KIND=8) , DIMENSION (IMAX,niv) :: XPAM
REAL (KIND=8) , DIMENSION (IMAX,niv,nphot) :: TJM  
REAL (KIND=8) , DIMENSION (IMAX,niv) :: O3T
REAL (KIND=8) , DIMENSION (IMAX) :: SZA,PALBS
LOGICAL , DIMENSION (IMAX) :: day
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!
INTEGER :: I,IVEC,INIV,J,K,L,M,NN,IMAX_
REAL (KIND=8) :: SC,V3STD
!     REAL (KIND=8) :: sum
REAL (KIND=8) :: col2,z1,z1a,z1b,cizin,cialb,cisza,cio3t,dummy
REAL (KIND=8) , ALLOCATABLE :: zphot(:)
REAL (KIND=8) , ALLOCATABLE :: zin(:,:), v3rat(:,:)
REAL (KIND=8) , ALLOCATABLE :: weight(:,:,:,:,:,:)
REAL (KIND=8) , ALLOCATABLE :: SZA_(:,:),PALBS_(:,:)
LOGICAL       , ALLOCATABLE :: day_(:,:)
INTEGER , ALLOCATABLE :: is(:,:)
INTEGER , ALLOCATABLE :: iv(:,:)
INTEGER , ALLOCATABLE :: iz(:,:)
INTEGER , ALLOCATABLE :: ia(:,:)
!
ALLOCATE (zphot(nz))
if ( mod(imax,2) == 0 ) then
  IMAX_ = IMAX + 1
else
  IMAX_ = IMAX
endif
ALLOCATE (zin   (IMAX_,NIV))
ALLOCATE (v3rat (IMAX_,NIV))
ALLOCATE (day_  (IMAX_,NIV))
ALLOCATE (SZA_  (IMAX_,NIV))
ALLOCATE (PALBS_(IMAX_,NIV))
ALLOCATE (weight(IMAX_,NIV,2,2,2,2))
ALLOCATE (is(IMAX_,NIV))
ALLOCATE (iv(IMAX_,NIV))
ALLOCATE (iz(IMAX_,NIV))
ALLOCATE (ia(IMAX_,NIV))
!
! 1. INITIALIZATION
!
DO i=1,nz
  zphot(i)=2.*REAL(i-1)
ENDDO
!
!$omp parallel &
!$omp default(none) &
!$omp shared(NIV,IMAX,O3T,PALBS,PALBS_,PREF,SZA,SZA_,TJM,XPAM) &
!$omp shared(ajl,albd,day,day_,dsza,ia,iphotno,is,iv,iz) &
!$omp shared(o3up,v3rat,weight,xv3,zin,zphot,SZAlim,CONV) &
!$omp private(cialb,cio3t,cisza,cizin,col2,dummy,i,iniv,ivec,j,k,l,m,nn,sc,v3std,z1,z1a,z1b)
!
!$omp do
DO iniv = 1,NIV
  DO ivec = 1,IMAX
    day_  (ivec,iniv) = day  (ivec)
    SZA_  (ivec,iniv) = SZA  (ivec)
    PALBS_(ivec,iniv) = PALBS(ivec)
  ENDDO
  DO k = 1,nphot
  DO ivec = 1,IMAX
    TJM(ivec,iniv,k) = 0.
  ENDDO
  ENDDO
ENDDO
!$omp end do
!
! 2. MAIN LOOP
!
!$omp do
DO iniv = niv,1,-1
!
! 2.1 PREPARE INTERPOLATION
!
  DO ivec = 1,IMAX
    dummy = XPAM(ivec,iniv)/PREF
    dummy = -6.8*alog(dummy)
    dummy = MAX(dummy,0.)
    zin(ivec,iniv) = MIN(dummy,78.9)
!
    i = INT(zin(ivec,iniv))
    v3std = o3up(i)-(zin(ivec,iniv)-real(i))*(o3up(i)-o3up(i+1))
    dummy = O3T(ivec,iniv)/v3std
    dummy = max(dummy,0.4)
    v3rat(ivec,iniv) = min(dummy,1.6)
!
    is(ivec,iniv) = nsza - 1
    iv(ivec,iniv) = nozo - 1
    ia(ivec,iniv) = nalb - 1
!
    IF(SZA_(ivec,iniv) <= SZAlim) THEN
      day_(ivec,iniv) = .TRUE.
      SZA_(ivec,iniv) = MAX(SZA_(ivec,iniv),0.)
    ELSE
      day_(ivec,iniv) = .FALSE.
    ENDIF
!
! 2.2 COMPUTE INDEXES
!
    iz(ivec,iniv) = INT(zin(ivec,iniv)/2.) + 1
!
    PALBS_(ivec,iniv)=MIN(PALBS_(ivec,iniv),albd(nalb))
    PALBS_(ivec,iniv)=MAX(PALBS_(ivec,iniv),albd(1))
  ENDDO
!
  DO i = 1,nalb
    DO ivec = 1,IMAX
      IF (albd(i) >= PALBS_(ivec,iniv)) THEN
        ia(ivec,iniv) = MIN(ia(ivec,iniv),i-1)
      ENDIF
    ENDDO
  ENDDO
!
  DO i = 1,nsza
    DO ivec = 1,IMAX
      IF (dsza(i) >= SZA(ivec)) THEN
        is(ivec,iniv) = MIN(is(ivec,iniv),i-1)
      ENDIF
    ENDDO
  ENDDO
!
  DO i = 1,nozo
    DO ivec = 1,IMAX
      IF (xv3(i) >= v3rat(ivec,iniv)) THEN
        iv(ivec,iniv) = MIN(iv(ivec,iniv),i-1)
      ENDIF
    ENDDO
  ENDDO
!
! 2.3 COMPUTE WEIGHTS
!
  DO ivec = 1,IMAX
    IF (day_(ivec,iniv)) THEN
      j = MAX(iz(ivec,iniv),1)
      k = MAX(is(ivec,iniv),1)
      l = MAX(iv(ivec,iniv),1)
      m = MAX(ia(ivec,iniv),1)
      cizin = (zin(ivec,iniv)-zphot(j))/(zphot(j+1)-zphot(j))
      cisza = (SZA_(ivec,iniv)-dsza(k))/(dsza(k+1)-dsza(k))
      cio3t = (v3rat(ivec,iniv)-xv3(l))/(xv3(l+1)-xv3(l))
      cialb = (PALBS_(ivec,iniv)-albd(m))/(albd(m+1)-albd(m))
      weight(ivec,iniv,1,1,1,1) = (1.-cizin)*(1.-cialb)*(1.-cisza)*(1.-cio3t)
      weight(ivec,iniv,1,1,1,2) = (1.-cizin)*(1.-cialb)*(1.-cisza)*cio3t
      weight(ivec,iniv,1,1,2,1) = (1.-cizin)*(1.-cialb)*cisza*(1.-cio3t)
      weight(ivec,iniv,1,1,2,2) = (1.-cizin)*(1.-cialb)*cisza*cio3t
      weight(ivec,iniv,2,1,1,1) = cizin*(1.-cialb)*(1.-cisza)*(1.-cio3t)
      weight(ivec,iniv,2,1,1,2) = cizin*(1.-cialb)*(1.-cisza)*cio3t
      weight(ivec,iniv,2,1,2,1) = cizin*(1.-cialb)*cisza*(1.-cio3t)
      weight(ivec,iniv,2,1,2,2) = cizin*(1.-cialb)*cisza*cio3t
      weight(ivec,iniv,1,2,1,1) = (1.-cizin)*cialb*(1.-cisza)*(1.-cio3t)
      weight(ivec,iniv,1,2,1,2) = (1.-cizin)*cialb*(1.-cisza)*cio3t
      weight(ivec,iniv,1,2,2,1) = (1.-cizin)*cialb*cisza*(1.-cio3t)
      weight(ivec,iniv,1,2,2,2) = (1.-cizin)*cialb*cisza*cio3t
      weight(ivec,iniv,2,2,1,1) = cizin*cialb*(1.-cisza)*(1.-cio3t)
      weight(ivec,iniv,2,2,1,2) = cizin*cialb*(1.-cisza)*cio3t
      weight(ivec,iniv,2,2,2,1) = cizin*cialb*cisza*(1.-cio3t)
      weight(ivec,iniv,2,2,2,2) = cizin*cialb*cisza*cio3t
    ENDIF
  ENDDO
!
! 2.4 INTERPOLATE
!
  DO nn = 1,nphot
    DO ivec = 1,IMAX
      IF (day_(ivec,iniv)) THEN
        j = MAX(iz(ivec,iniv),1)
        k = MAX(is(ivec,iniv),1)
        l = MAX(iv(ivec,iniv),1)
        m = MAX(ia(ivec,iniv),1)
        TJM(ivec,iniv,nn) = &
                     weight(ivec,iniv,1,1,1,1)*ajl(nn,j  ,m  ,k  ,l  )  &
                   + weight(ivec,iniv,1,1,1,2)*ajl(nn,j  ,m  ,k  ,l+1)  &
                   + weight(ivec,iniv,1,1,2,1)*ajl(nn,j  ,m  ,k+1,l  )  &
                   + weight(ivec,iniv,1,1,2,2)*ajl(nn,j  ,m  ,k+1,l+1)  &
                   + weight(ivec,iniv,2,1,1,1)*ajl(nn,j+1,m  ,k  ,l  )  &
                   + weight(ivec,iniv,2,1,1,2)*ajl(nn,j+1,m  ,k  ,l+1)  &
                   + weight(ivec,iniv,2,1,2,1)*ajl(nn,j+1,m  ,k+1,l  )  &
                   + weight(ivec,iniv,2,1,2,2)*ajl(nn,j+1,m  ,k+1,l+1)  &
                   + weight(ivec,iniv,1,2,1,1)*ajl(nn,j  ,m+1,k  ,l  )  &
                   + weight(ivec,iniv,1,2,1,2)*ajl(nn,j  ,m+1,k  ,l+1)  &
                   + weight(ivec,iniv,1,2,2,1)*ajl(nn,j  ,m+1,k+1,l  )  &
                   + weight(ivec,iniv,1,2,2,2)*ajl(nn,j  ,m+1,k+1,l+1)  &
                   + weight(ivec,iniv,2,2,1,1)*ajl(nn,j+1,m+1,k  ,l  )  &
                   + weight(ivec,iniv,2,2,1,2)*ajl(nn,j+1,m+1,k  ,l+1)  &
                   + weight(ivec,iniv,2,2,2,1)*ajl(nn,j+1,m+1,k+1,l  )  &
                   + weight(ivec,iniv,2,2,2,2)*ajl(nn,j+1,m+1,k+1,l+1)
      ENDIF
    ENDDO
  ENDDO
!
! 2.5 COMPUTE PHOTOLYSIS FOR NO (nphot=49 in master table,
!       iphotNO in current chemical scheme)
!
  DO ivec = 1,IMAX
    IF (day_(ivec,iniv)) THEN
      sc = 1./cos(min(SZA_(ivec,iniv),85.)/CONV)
      col2 = 4.43e+19*XPAM(ivec,iniv)*sc
      z1a = 1.e-8*(col2**0.38)
      z1b = 5.e-19*O3T(ivec,iniv)*sc
      z1 = z1a + z1b
      TJM(ivec,iniv,iphotNO) =4.5e-6*exp(-z1)
    ENDIF
  ENDDO
!
ENDDO                       ! END OF MAIN LOOP
!$omp end do
!
!$omp do
DO iniv = 1,niv
DO k = 1,nphot
DO ivec = 1,IMAX
  TJM(ivec,iniv,k) = MAX(TJM(ivec,iniv,k),1.e-30)
ENDDO
ENDDO
ENDDO
!$omp end do
!
!$omp do
DO ivec = 1,IMAX
  day  (ivec) = day_  (ivec,1)
  PALBS(ivec) = PALBS_(ivec,1)
ENDDO
!$omp end do
!
!$omp end parallel
DEALLOCATE (zphot)
DEALLOCATE (zin)
DEALLOCATE (v3rat)
DEALLOCATE (weight)
DEALLOCATE (is)
DEALLOCATE (iv)
DEALLOCATE (iz)
DEALLOCATE (ia)
DEALLOCATE (day_)
DEALLOCATE (SZA_)
DEALLOCATE (PALBS_)
!
RETURN
END SUBROUTINE PHOT
