      SUBROUTINE POIDGAU(Y,YREF,YCOEF,LAT
     +                  ,IND,ALF1,ALF2,ALF3,ALF4,NDIM)
C
CCCCC
C
C   Modele de transport semi-lagrangien 3D
C   - Calcul des poids pour une interpolation
C     cubique par rapport a une grille de Gauss -
C
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C
C   Centre National de Recherches Meteorologiques
C   Groupe de Modelisation de Grande Echelle et du Climat
C   Equipe de Recherches sur l'Atmosphere Moyenne
C
CCCCC
C
C  Original : P. Simon, 03/2003
C  Modified : H. Le Berre 06/2006  Fortran 90 compliant
CCCCC
C
      USE CSTES_MOD
C
      IMPLICIT NONE
C
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: LAT,NDIM
      INTEGER , DIMENSION (NDIM) :: IND
      REAL (KIND=8) , DIMENSION (NDIM) :: ALF1,ALF2,ALF3,ALF4
      REAL (KIND=8) , DIMENSION (NDIM) :: Y
      REAL (KIND=8) , DIMENSION (-1:LAT+2) :: YREF
      REAL (KIND=8) , DIMENSION (0:LAT,4)  :: YCOEF
C
C      
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C        
      INTEGER :: K,N
      REAL (KIND=8) :: DRLAT,DYM1,DYP0,DYP1,DYP2
C
C
C
C  Positionnement:
C
C  a) premiere approximation (grille reguliere)
C
      DRLAT=XPI/REAL(LAT+1)
      DO N=1,NDIM
        IND(N)=INT((Y(N)+XPIH)/DRLAT)
      ENDDO
C
C  b) correction pour la grille de Gauss (faiblement irreguliere)
C
      DO N=1,NDIM
        K=IND(N)
        IF (Y(N).LE.0.) THEN
           IF (Y(N).GE.YREF(K+1)) THEN
             IND(N)=K+1
           ELSE
             IND(N)=K
           ENDIF
         ELSE
           IF (Y(N).GE.YREF(K)) THEN
             IND(N)=K
           ELSE
             IND(N)=K-1
           ENDIF
         ENDIF
      ENDDO
C
C  Calcul des poids proprement dit
C
      DO N=1,NDIM
        K=IND(N)
        DYM1=Y(N)-YREF(K-1)
        DYP0=Y(N)-YREF(K  )
        DYP1=Y(N)-YREF(K+1)
        DYP2=Y(N)-YREF(K+2)
        ALF1(N)=YCOEF(K,1)*DYP0*DYP1*DYP2
        ALF2(N)=YCOEF(K,2)*DYM1*DYP1*DYP2
        ALF3(N)=YCOEF(K,3)*DYM1*DYP0*DYP2
        ALF4(N)=YCOEF(K,4)*DYM1*DYP0*DYP1
      ENDDO
C
C
C      
      RETURN
      END
