#ifdef _OPENMP
  module poidz_mod
      integer, parameter :: nbthreadsmax = 8
  end module poidz_mod
#endif

      SUBROUTINE POIDZ(ZZ,PP           &
                      ,AL,BL,NIV       &
                      ,INDZ            &
                      ,ALFZ1,ALFZ2,ALFZ3,ALFZ4,NDIM)


!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!CC DATE : 15/05/2005
!
!   Type      : 3D semi-lagrangian transport scheme
!   Purpose   : Compute weights for cubic vertical interpolation 
!   Called by : 
!
!CC
!
!  Original : P. Simon, 03/1999
!
!  Modified : V.-H. Peuch (inclusion dans MOCAGE), 04/1999
!             H. Teyssedre (version etendue au-dessus
!                           du niveau 1),          03/2001


#ifdef _OPENMP
        use poidz_mod
#endif
        IMPLICIT NONE


!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~

        INTEGER :: NIV,NDIM
        INTEGER , DIMENSION (NDIM) :: INDZ
        REAL (KIND=8) , DIMENSION (NDIM)  :: ZZ,PP
        REAL (KIND=8) , DIMENSION (0:NIV) :: AL,BL      
        REAL (KIND=8) , DIMENSION (NDIM) :: ALFZ1,ALFZ2,ALFZ3,ALFZ4

!     Variables locales.
!     ~~~~~~~~~~~~~~~~~

        INTEGER :: N,L,L1,L2,L3,L4
        REAL (KIND=8) :: XPALEVEL
        REAL (KIND=8) :: DP,DP1,DP2,DP3,DP4
        REAL (KIND=8) :: DP41,DP42,DP43,DP44
        REAL (KIND=8) :: DP21,DP31,DP32
        REAL (KIND=8) :: PRES,PL1,PL2,PL3,PL4,PLSUP
        REAL (KIND=8) :: XPAPLUS ,XPAMOIN
        REAL (KIND=8) :: XPAPLUS1,XPAMOIN1
        REAL (KIND=8) :: XPAPLUS2,XPAMOIN2
        REAL (KIND=8) :: XPAPLUS3,XPAMOIN3
        REAL (KIND=8) :: XPAPLUS4,XPAMOIN4

        LOGICAL, DIMENSION(NDIM,2:NIV) :: CONDITION_ZZ

#ifdef _OPENMP
      INTEGER :: ndim_size, MOI, NBTHREADS, N1, N2
      INTEGER :: omp_get_num_threads
      INTEGER :: omp_get_thread_num
#endif

#ifdef NIV_91
      PLSUP=0.5
#endif
#ifdef NIV_60
      PLSUP=5.
#endif
#ifdef NIV_47
      PLSUP=400.
#endif

#ifdef FTRACE
call ftrace_region_begin("poidz01")
#endif

      !  determination de la pression PL immediatement inferieure a zz

!$omp parallel do &
!$omp default(none) &
!$omp shared(NDIM,AL,BL,PP,ZZ,PLSUP,INDZ) &
!$omp private(N,PRES)
      DO N=1,NDIM
#ifdef MIDATM
         PRES=0.5*(AL(1)+BL(1)*PP(N))
#else
#ifdef ACCIDENT
         PRES=0.5*(AL(1)+BL(1)*PP(N))
#else
         PRES=EXP(ALOG(AL(1)+BL(1)*PP(N))-1.)
#endif
#endif
         INDZ(N)=-1
         IF(ZZ(N) >= PLSUP) THEN
           INDZ(N)=0
         ENDIF
         IF(ZZ(N) >= PRES) THEN
           INDZ(N)=1
         ENDIF
      ENDDO
!$omp end parallel do

#ifdef FTRACE
call ftrace_region_end("poidz01")
#endif

#ifdef FTRACE
call ftrace_region_begin("poidz02")
#endif

!$omp parallel do &
!$omp default(none) &
!$omp private(L,N,XPAPLUS,XPAMOIN,PRES) &
!$omp shared(NDIM,NIV,AL,BL,PP,ZZ,CONDITION_ZZ)
      DO L=2,NIV
         DO N=1,NDIM
            XPAPLUS=AL(L  )+BL(L  )*PP(N)
            XPAMOIN=AL(L-1)+BL(L-1)*PP(N)
#ifdef MIDATM
            PRES=0.5*(XPAPLUS+XPAMOIN)
#else
#ifndef ACCIDENT
            PRES=EXP( (XPAPLUS*ALOG(XPAPLUS)-XPAMOIN*ALOG(XPAMOIN))/(XPAPLUS-XPAMOIN) - 1. )
#else
            PRES=0.5*(XPAPLUS+XPAMOIN)
#endif
#endif
            CONDITION_ZZ(N,L) = (ZZ(N) >= PRES)
         END DO
      END DO
!$omp end parallel do

#ifdef FTRACE
call ftrace_region_end("poidz02")
#endif

#ifdef FTRACE
call ftrace_region_begin("poidz03")
#endif

#ifdef _OPENMP
!$omp parallel &
!$omp default(none) &
!$omp private(NBTHREADS,ndim_size,MOI,N1,N2,N,L) &
!$omp shared(NDIM,NIV,CONDITION_ZZ,INDZ)
      NBTHREADS = omp_get_num_threads()
      ndim_size = NDIM / NBTHREADS
      MOI = omp_get_thread_num()
      N1 = 1 + MOI*ndim_size
      IF (MOI == NBTHREADS-1) THEN
         N2 = NDIM
      ELSE
         N2 = N1 + ndim_size - 1
      END IF
#endif
      DO L=2,NIV
#ifdef _OPENMP
         DO N=N1,N2
#else
         DO N=1,NDIM
#endif
            IF( CONDITION_ZZ(N,L) ) THEN
              INDZ(N)=L
            ENDIF
         END DO
      END DO
!$omp end parallel

#ifdef FTRACE
call ftrace_region_end("poidz03")
#endif

      !  calcul des poids proprement dit

#ifdef FTRACE
call ftrace_region_begin("poidz04")
#endif

!$omp parallel do &
!$omp default(none) &
!$omp shared(NDIM,NIV,INDZ,ALFZ1,ALFZ2,ALFZ3,ALFZ4,AL,BL,PP,PLSUP,ZZ) &
!$omp private(N,L,L1,L2,L3,L4,PL1,PL2,PL3,PL4,DP,DP1,DP2,DP3,DP4) &
!$omp private(DP41,DP42,DP43,DP31,DP32,DP21) &
!$omp private(XPAPLUS1,XPAPLUS2,XPAPLUS3,XPAPLUS4,XPALEVEL) &
!$omp private(XPAMOIN1,XPAMOIN2,XPAMOIN3,XPAMOIN4)
      DO 2 N=1,NDIM

         L=INDZ(N)

         IF(L == -1) THEN

!  valeur constante au-dessus du niveau 0

           ALFZ1(N)=0.
           ALFZ2(N)=0.
           ALFZ3(N)=1.
           ALFZ4(N)=0.

         ELSEIF(L.EQ.0) THEN

           !  interpolation lineaire entre 0 et 1

#ifdef MIDATM
           XPAPLUS1=AL(L+1)+BL(L+1)*PP(N)
           
           PL2=PLSUP
           PL3=0.5*XPAPLUS1
#else
#ifndef ACCIDENT
           L2=L
           L3=L+1

           PL2=PLSUP
           PL3=EXP(ALOG(AL(L3)+BL(L3)*PP(N)) - 1.)
#else
           XPAPLUS1=AL(L+1)+BL(L+1)*PP(N)

           PL2=PLSUP
           PL3=0.5*XPAPLUS1
#endif
#endif

           DP2=ZZ(N)-PL2
           DP3=PL3-ZZ(N)
           DP =PL3  -PL2

           ALFZ1(N)=0.
           ALFZ2(N)=+DP3/DP
           ALFZ3(N)=+DP2/DP
           ALFZ4(N)=0.

         ELSEIF(L == 1) THEN

#ifdef MIDATM
           XPALEVEL=AL(L  )+BL(L  )*PP(N)
           XPAPLUS1=AL(L+1)+BL(L+1)*PP(N)
           XPAPLUS2=AL(L+2)+BL(L+2)*PP(N)

           PL1=PLSUP
           PL2=0.5* XPALEVEL
           PL3=0.5*(XPAPLUS1+XPALEVEL)
           PL4=0.5*(XPAPLUS2+XPAPLUS1)
#else
#ifndef ACCIDENT
           L1=L-1
           L2=L
           L3=L+1
           L4=L+2

           PL1=PLSUP
           PL2=EXP(ALOG(AL(L2)+BL(L2)*PP(N)) - 1.)
           XPAPLUS3=AL(L3)+BL(L3)*PP(N)
           XPAMOIN3=AL(L3-1)+BL(L3-1)*PP(N)
           PL3=EXP( (XPAPLUS3*ALOG(XPAPLUS3)-XPAMOIN3*ALOG(XPAMOIN3)) &
                    /(XPAPLUS3-XPAMOIN3) - 1. )
           XPAPLUS4=AL(L4)+BL(L4)*PP(N)
           XPAMOIN4=AL(L4-1)+BL(L4-1)*PP(N)
           PL4=EXP( (XPAPLUS4*ALOG(XPAPLUS4)-XPAMOIN4*ALOG(XPAMOIN4)) &
                    /(XPAPLUS4-XPAMOIN4) - 1. )
#else
           XPALEVEL=AL(L  )+BL(L  )*PP(N)
           XPAPLUS1=AL(L+1)+BL(L+1)*PP(N)
           XPAPLUS2=AL(L+2)+BL(L+2)*PP(N)

           PL1=PLSUP
           PL2=0.5* XPALEVEL
           PL3=0.5*(XPAPLUS1+XPALEVEL)
           PL4=0.5*(XPAPLUS2+XPAPLUS1)
#endif
#endif

           DP1=ZZ(N)-PL1
           DP2=ZZ(N)-PL2
           DP3=ZZ(N)-PL3
           DP4=ZZ(N)-PL4

           DP41=PL4-PL1
           DP42=PL4-PL2
           DP43=PL4-PL3
           DP31=PL3-PL1
           DP32=PL3-PL2
           DP21=PL2-PL1

           ALFZ1(N)=-(DP2*DP3*DP4)/(DP21*DP31*DP41)
           ALFZ2(N)= (DP1*DP3*DP4)/(DP21*DP32*DP42)
           ALFZ3(N)=-(DP1*DP2*DP4)/(DP31*DP32*DP43)
           ALFZ4(N)= (DP1*DP2*DP3)/(DP41*DP42*DP43)

         ELSEIF (L == 2) THEN

           !  cas general: interpolation cubique

#ifdef MIDATM
           XPAMOIN1=AL(L-1)+BL(L-1)*PP(N)
           XPALEVEL=AL(L  )+BL(L  )*PP(N)
           XPAPLUS1=AL(L+1)+BL(L+1)*PP(N)
           XPAPLUS2=AL(L+2)+BL(L+2)*PP(N)

           PL1=0.5* XPAMOIN1
           PL2=0.5*(XPALEVEL+XPAMOIN1)
           PL3=0.5*(XPAPLUS1+XPALEVEL)
           PL4=0.5*(XPAPLUS2+XPAPLUS1)
#else
#ifndef ACCIDENT
           L1=L-1
           L2=L
           L3=L+1
           L4=L+2

           PL1=EXP(ALOG(AL(L1)+BL(L1)*PP(N)) - 1.)
           XPAPLUS2=AL(L2)+BL(L2)*PP(N)
           XPAMOIN2=AL(L2-1)+BL(L2-1)*PP(N)
           PL2=EXP( (XPAPLUS2*ALOG(XPAPLUS2)-XPAMOIN2*ALOG(XPAMOIN2)) &
                    /(XPAPLUS2-XPAMOIN2) - 1. )
           XPAPLUS3=AL(L3)+BL(L3)*PP(N)
           XPAMOIN3=AL(L3-1)+BL(L3-1)*PP(N)
           PL3=EXP( (XPAPLUS3*ALOG(XPAPLUS3)-XPAMOIN3*ALOG(XPAMOIN3)) &
                    /(XPAPLUS3-XPAMOIN3) - 1. )
           XPAPLUS4=AL(L4)+BL(L4)*PP(N)
           XPAMOIN4=AL(L4-1)+BL(L4-1)*PP(N)
           PL4=EXP( (XPAPLUS4*ALOG(XPAPLUS4)-XPAMOIN4*ALOG(XPAMOIN4)) &
                    /(XPAPLUS4-XPAMOIN4) - 1. )
#else
           XPAMOIN1=AL(L-1)+BL(L-1)*PP(N)
           XPALEVEL=AL(L  )+BL(L  )*PP(N)
           XPAPLUS1=AL(L+1)+BL(L+1)*PP(N)
           XPAPLUS2=AL(L+2)+BL(L+2)*PP(N)

           PL1=0.5* XPAMOIN1
           PL2=0.5*(XPALEVEL+XPAMOIN1)
           PL3=0.5*(XPAPLUS1+XPALEVEL)
           PL4=0.5*(XPAPLUS2+XPAPLUS1)
#endif
#endif

           DP1=ZZ(N)-PL1
           DP2=ZZ(N)-PL2
           DP3=ZZ(N)-PL3
           DP4=ZZ(N)-PL4

           DP41=PL4-PL1
           DP42=PL4-PL2
           DP43=PL4-PL3
           DP31=PL3-PL1
           DP32=PL3-PL2
           DP21=PL2-PL1

           ALFZ1(N)=-(DP2*DP3*DP4)/(DP21*DP31*DP41)
           ALFZ2(N)= (DP1*DP3*DP4)/(DP21*DP32*DP42)
           ALFZ3(N)=-(DP1*DP2*DP4)/(DP31*DP32*DP43)
           ALFZ4(N)= (DP1*DP2*DP3)/(DP41*DP42*DP43)

         ELSEIF(L == NIV-1) THEN

           !  interpolation lineaire entre NIV-1 et NIV

#ifdef MIDATM
           XPAMOIN1=AL(L-1)+BL(L-1)*PP(N)
           XPALEVEL=AL(L  )+BL(L  )*PP(N)
           XPAPLUS1=AL(L+1)+BL(L+1)*PP(N)

           PL2=0.5*(XPALEVEL+XPAMOIN1)
           PL3=0.5*(XPAPLUS1+XPALEVEL)
#else
#ifndef ACCIDENT
           L2=L
           L3=L+1

           XPAPLUS2=AL(L2)+BL(L2)*PP(N)
           XPAMOIN2=AL(L2-1)+BL(L2-1)*PP(N)
           PL2=EXP( (XPAPLUS2*ALOG(XPAPLUS2)-XPAMOIN2*ALOG(XPAMOIN2)) &
                    /(XPAPLUS2-XPAMOIN2) - 1. )
           XPAPLUS3=AL(L3)+BL(L3)*PP(N)
           XPAMOIN3=AL(L3-1)+BL(L3-1)*PP(N)
           PL3=EXP( (XPAPLUS3*ALOG(XPAPLUS3)-XPAMOIN3*ALOG(XPAMOIN3)) &
                    /(XPAPLUS3-XPAMOIN3) - 1. )
#else
           XPAMOIN1=AL(L-1)+BL(L-1)*PP(N)
           XPALEVEL=AL(L  )+BL(L  )*PP(N)
           XPAPLUS1=AL(L+1)+BL(L+1)*PP(N)

           PL2=0.5*(XPALEVEL+XPAMOIN1)
           PL3=0.5*(XPAPLUS1+XPALEVEL)
#endif
#endif

           DP2=ZZ(N)-PL2
           DP3=PL3-ZZ(N)
           DP =PL3  -PL2

           ALFZ1(N)=0.
           ALFZ2(N)=+DP3/DP
           ALFZ3(N)=+DP2/DP
           ALFZ4(N)=0.

         ELSEIF(L == NIV) THEN

           !  valeur constante au-dessous du niveau NIVM1

           ALFZ1(N)=0.
           ALFZ2(N)=1.
           ALFZ3(N)=0.
           ALFZ4(N)=0.

         ELSE

           !  cas general: interpolation cubique

#ifdef MIDATM
           XPAMOIN2=AL(L-2)+BL(L-2)*PP(N)
           XPAMOIN1=AL(L-1)+BL(L-1)*PP(N)
           XPALEVEL=AL(L  )+BL(L  )*PP(N)
           XPAPLUS1=AL(L+1)+BL(L+1)*PP(N)
           XPAPLUS2=AL(L+2)+BL(L+2)*PP(N)

           PL1=0.5*(XPAMOIN1+XPAMOIN2)
           PL2=0.5*(XPALEVEL+XPAMOIN1)
           PL3=0.5*(XPAPLUS1+XPALEVEL)
           PL4=0.5*(XPAPLUS2+XPAPLUS1)
#else
#ifndef ACCIDENT
           L1=L-1
           L2=L
           L3=L+1
           L4=L+2

           XPAPLUS1=AL(L1)+BL(L1)*PP(N)
           XPAMOIN1=AL(L1-1)+BL(L1-1)*PP(N)
           PL1=EXP( (XPAPLUS1*ALOG(XPAPLUS1)-XPAMOIN1*ALOG(XPAMOIN1)) &
                    /(XPAPLUS1-XPAMOIN1) - 1. )
           XPAPLUS2=AL(L2)+BL(L2)*PP(N)
           XPAMOIN2=AL(L2-1)+BL(L2-1)*PP(N)
           PL2=EXP( (XPAPLUS2*ALOG(XPAPLUS2)-XPAMOIN2*ALOG(XPAMOIN2)) &
                    /(XPAPLUS2-XPAMOIN2) - 1. )
           XPAPLUS3=AL(L3)+BL(L3)*PP(N)
           XPAMOIN3=AL(L3-1)+BL(L3-1)*PP(N)
           PL3=EXP( (XPAPLUS3*ALOG(XPAPLUS3)-XPAMOIN3*ALOG(XPAMOIN3)) &
                    /(XPAPLUS3-XPAMOIN3) - 1. )
           XPAPLUS4=AL(L4)+BL(L4)*PP(N)
           XPAMOIN4=AL(L4-1)+BL(L4-1)*PP(N)
           PL4=EXP( (XPAPLUS4*ALOG(XPAPLUS4)-XPAMOIN4*ALOG(XPAMOIN4)) &
                    /(XPAPLUS4-XPAMOIN4) - 1. )
#else
           XPAMOIN2=AL(L-2)+BL(L-2)*PP(N)
           XPAMOIN1=AL(L-1)+BL(L-1)*PP(N)
           XPALEVEL=AL(L  )+BL(L  )*PP(N)
           XPAPLUS1=AL(L+1)+BL(L+1)*PP(N)
           XPAPLUS2=AL(L+2)+BL(L+2)*PP(N)

           PL1=0.5*(XPAMOIN1+XPAMOIN2)
           PL2=0.5*(XPALEVEL+XPAMOIN1)
           PL3=0.5*(XPAPLUS1+XPALEVEL)
           PL4=0.5*(XPAPLUS2+XPAPLUS1)
#endif
#endif

           DP1=ZZ(N)-PL1
           DP2=ZZ(N)-PL2
           DP3=ZZ(N)-PL3
           DP4=ZZ(N)-PL4

           DP41=PL4-PL1
           DP42=PL4-PL2
           DP43=PL4-PL3
           DP31=PL3-PL1
           DP32=PL3-PL2
           DP21=PL2-PL1

           ALFZ1(N)=-(DP2*DP3*DP4)/(DP21*DP31*DP41)
           ALFZ2(N)= (DP1*DP3*DP4)/(DP21*DP32*DP42)
           ALFZ3(N)=-(DP1*DP2*DP4)/(DP31*DP32*DP43)
           ALFZ4(N)= (DP1*DP2*DP3)/(DP41*DP42*DP43)

         ENDIF

 2    CONTINUE
!$omp end parallel do

#ifdef FTRACE
call ftrace_region_end("poidz04")
#endif

      RETURN
      END
