      SUBROUTINE PSCSED(NLON,NLAT,NIV,NTRA,DT,DLAT,
     +                  partH2Os,partHNO3s,VSED,
     +                  XPAM,XPHFM,TGRIM,QQQM,TRANAM)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Chemistry
C   Purpose    : Compute gravitational sedimentation of PSC particles
C   Called by  :
C
CCC
C
C  Original :  F. Lefevre, 1997
C
C  Modified : H. Teyssedre (version climat), 03/2003
C             H. Teyssedre (nouvelle version), 06/2004
C
      USE CSTES_MOD
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: NLON,NLAT,NIV,NTRA
      REAL (KIND=8) :: DT
      REAL (KIND=8) , DIMENSION (NLAT) :: DLAT
      REAL (KIND=8) , DIMENSION (NLON,NLAT,NIV,NTRA) :: QQQM
      REAL (KIND=8) , DIMENSION (NLON,NLAT,NIV) :: XPAM
      REAL (KIND=8) , DIMENSION (NLON,NLAT,NIV) :: XPHFM,TGRIM
      REAL (KIND=8) , DIMENSION (NLON,NLAT,NIV) :: partH2Os,partHNO3s
      REAL (KIND=8) , DIMENSION (NLON,NLAT,NIV) :: VSED
      REAL (KIND=8) , DIMENSION (NLON,NLAT,NIV) :: DELTAP
      CHARACTER(LEN=15) , DIMENSION(NTRA) :: TRANAM
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: INIV,ILON,ILAT,NIVHAU,NIVHAU2,NIVBAS
      INTEGER :: NOK,ITRA,IH2O,IHNO3,INOY
      INTEGER :: IPSC,IPSCNIVM1,IPSCNIV
      REAL (KIND=8) :: COEFF,COEFFRACM1,COEFFRAC,ZPAM,FRACM1,FRAC
C
C DETERMINE USEFUL INDEXES
C
      DO iniv=1,NIV
        ZPAM=0.
        DO ilon=1,NLON*NLAT
          ZPAM=ZPAM+XPAM(ilon,1,iniv)/REAL(NLON*NLAT)
        ENDDO
        IF (ZPAM.LE. 1000.) NIVHAU = iniv
        IF (ZPAM.LE.45000.) NIVBAS = iniv
      ENDDO
C
      NIVHAU = MAX(NIVHAU,   1)
      NIVBAS = MIN(NIVBAS, niv)
C
      DO ILAT=1,NLAT
        DO ILON=1,NLON
          DELTAP(ILON,ILAT,1)=XPHFM(ILON,ILAT,1)
        ENDDO
      ENDDO
C
      DO INIV=2,NIV
        DO ILAT=1,NLAT
          DO ILON=1,NLON
            DELTAP(ILON,ILAT,INIV)=XPHFM(ILON,ILAT,INIV)-
     +                             XPHFM(ILON,ILAT,INIV-1)
          ENDDO
        ENDDO
      ENDDO
C
      NOK=0
C
      DO itra = 1,NTRA
C H_2O
        IF (TRANAM(itra).EQ.'H_2O') THEN
          iH2O=itra
          NOK=NOK+1
        ENDIF
C HNO_3
        IF (TRANAM(itra).EQ.'HNO_3') THEN
          iHNO3=itra
          NOK=NOK+1
        ENDIF
C PSC
        IF (TRANAM(itra).EQ.'PSC') THEN
          iPSC=itra
          NOK=NOK+1
        ENDIF
#ifdef MASSy
C NOy
        IF (TRANAM(itra).EQ.'NO_y') THEN
          iNOy=itra
          NOK=NOK+1
        ENDIF
#endif
      ENDDO
C
#ifndef MASSy
      IF (NOK.NE.3) THEN
#else
      IF (NOK.NE.4) THEN
#endif
        WRITE(*,*)
     +  '>> CONSTITUENT(S) MISSING FOR PSC SEDIMENTATION : ABORT!'
        CALL MOCAGE_STOP
      ENDIF
C
      NIVHAU2=MAX(2,NIVHAU)
      COEFF=DT*GG*MAIR/8.314517
C
C APPLY SEDIMENTATION FOR PSCs
C
      DO INIV=NIVHAU2,NIVBAS
      DO ILAT=1,NLAT
        IF (ABS(DLAT(ILAT)*CONV).GE.45.0) THEN
          DO ILON=1,NLON
            COEFFRACM1 = VSED(ILON,ILAT,INIV-1)*COEFF
            COEFFRAC   = VSED(ILON,ILAT,INIV  )*COEFF
            iPSCNIVM1 =
     +         MAX(0,INT(QQQM(ILON,ILAT,INIV-1,iPSC)/1.999))
            iPSCNIV   =
     +         MAX(0,INT(QQQM(ILON,ILAT,INIV  ,iPSC)/1.999))
            FRACM1 = COEFFRACM1/ DELTAP(ILON,ILAT,INIV-1)
            FRAC   = COEFFRAC  / DELTAP(ILON,ILAT,INIV)
            QQQM(ILON,ILAT,INIV,iH2O ) = QQQM(ILON,ILAT,INIV,iH2O )
     +                            + QQQM(ILON,ILAT,INIV-1,iH2O )
     +                              *REAL(iPSCNIVM1)
     +                              *partH2Os(ILON,ILAT,INIV-1)
     +                              *FRACM1*XPAM(ILON,ILAT,INIV-1)
     +                              /TGRIM(ILON,ILAT,INIV-1)
     +                            - QQQM(ILON,ILAT,INIV,iH2O )
     +                              *REAL(iPSCNIV)
     +                              *partH2Os(ILON,ILAT,INIV)
     +                              *FRAC*XPAM(ILON,ILAT,INIV)
     +                              /TGRIM(ILON,ILAT,INIV)
#ifdef MASSy
            QQQM(ILON,ILAT,INIV,iNOy ) = QQQM(ILON,ILAT,INIV,iNOy )
     +                            + QQQM(ILON,ILAT,INIV-1,iHNO3)
     +                              *REAL(iPSCNIVM1)
     +                              *partHNO3s(ILON,ILAT,INIV-1)
     +                              *FRACM1*XPAM(ILON,ILAT,INIV-1)
     +                              /TGRIM(ILON,ILAT,INIV-1)
     +                            - QQQM(ILON,ILAT,INIV,iHNO3)
     +                              *REAL(iPSCNIV)
     +                              *partHNO3s(ILON,ILAT,INIV)
     +                              *FRAC*XPAM(ILON,ILAT,INIV)
     +                              /TGRIM(ILON,ILAT,INIV)
#endif
            QQQM(ILON,ILAT,INIV,iHNO3) = QQQM(ILON,ILAT,INIV,iHNO3)
     +                            + QQQM(ILON,ILAT,INIV-1,iHNO3)
     +                              *REAL(iPSCNIVM1)
     +                              *partHNO3s(ILON,ILAT,INIV-1)
     +                              *FRACM1*XPAM(ILON,ILAT,INIV-1)
     +                              /TGRIM(ILON,ILAT,INIV-1)
     +                            - QQQM(ILON,ILAT,INIV,iHNO3)
     +                              *REAL(iPSCNIV)
     +                              *partHNO3s(ILON,ILAT,INIV)
     +                              *FRAC*XPAM(ILON,ILAT,INIV)
     +                              /TGRIM(ILON,ILAT,INIV)
            QQQM(ILON,ILAT,INIV,iH2O ) = 
     +              MAX(1.E-30, QQQM(ILON,ILAT,INIV,iH2O ))
            QQQM(ILON,ILAT,INIV,iHNO3) = 
     +              MAX(1.E-30, QQQM(ILON,ILAT,INIV,iHNO3))
#ifdef MASSy
            QQQM(ILON,ILAT,INIV,iNOy ) = 
     +              MAX(1.E-30, QQQM(ILON,ILAT,INIV,iNOy ))
#endif
          ENDDO
        ENDIF
      ENDDO
      ENDDO
C
      RETURN
      END
C
CCC
C
