      SUBROUTINE RELAX(NLON,NLAT,NIV,NTRA,NSLS,DT,
     +                 XPAM,QQQM,HHHM,TRANAM,SLSNAM)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Chemistry
C   Purpose    : Relaxation towards uniform tropospheric values (REPROBUS)
C   Called by  :
C
CCC
C
C  Original :  H. Teyssedre, 06/2001
C
CCC
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: NLON,NLAT,NIV,NTRA,NSLS
      REAL (KIND=8) :: DT
      REAL (KIND=8) , DIMENSION (NLON,NLAT,NIV,NTRA) :: QQQM
      REAL (KIND=8) , DIMENSION (NLON,NLAT,NIV,NSLS) :: HHHM 
      REAL (KIND=8) , DIMENSION (NLON,NLAT,NIV) :: XPAM 
      CHARACTER(LEN=15) , DIMENSION (NTRA) :: TRANAM
      CHARACTER(LEN=15) , DIMENSION (NSLS) :: SLSNAM
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
#ifdef NIV_91
#define CPP_NIVBAS 67
#endif
#ifdef NIV_60
#define CPP_NIVBAS 41
#endif
#ifdef NIV_47
#define CPP_NIVBAS 30
#endif
      INTEGER :: NIVBAS,NOK,iN2O,iCH4,iNOx,iClOx,iOx,iCFC11,iCFC12,
     +           iCFC113,iCCl4,iCH3CCl3,iCH3Cl,iHCFC22,iCH3Br,
     +           iH1211,iH1301,iCO
      INTEGER :: ITRA,IO3,INIV,ILON
      PARAMETER (NIVBAS=CPP_NIVBAS)      
      REAL (KIND=8) :: DTT,XLIM
      REAL (KIND=8) , dimension(14), parameter :: TRALIM = (/
     +             316.0000 E-09,   1.7537 E-06,  93.9000 E-09,
     +             262.6000 E-12, 540.5000 E-12,  81.7000 E-12,
     +              97.9000 E-12,  89.0000 E-12, 550.7000 E-12,
     +             141.6000 E-12,   9.3000 E-12,   3.9700 E-12,
     +               2.8200 E-12,  50.0000 E-12/ )
C
CCC
C
C DETERMINE USEFUL INDEXES
C
      NOK=0
C
      DO itra = 1,NTRA
C N_2O
        IF (TRANAM(itra).EQ.'N_2O') THEN
          NOK=NOK+1
          iN2O=itra
        ENDIF
C CH_4
        IF (TRANAM(itra).EQ.'CH_4') THEN
          NOK=NOK+1
          iCH4=itra
        ENDIF
C CO
        IF (TRANAM(itra).EQ.'CO') THEN
          NOK=NOK+1
          iCO=itra
        ENDIF
C CFC11
        IF (TRANAM(itra).EQ.'CFC11') THEN
          NOK=NOK+1
          iCFC11=itra
        ENDIF
C CFC12
        IF (TRANAM(itra).EQ.'CFC12') THEN
          NOK=NOK+1
          iCFC12=itra
        ENDIF
C CFC113
        IF (TRANAM(itra).EQ.'CFC113') THEN
          NOK=NOK+1
          iCFC113=itra
        ENDIF
C CCl_4
        IF (TRANAM(itra).EQ.'CCl_4') THEN
          NOK=NOK+1
          iCCl4=itra
        ENDIF
C CH_3CCl3
        IF (TRANAM(itra).EQ.'CH_3CCl_3') THEN
          NOK=NOK+1
          iCH3CCl3=itra
        ENDIF
C CH_3Cl
        IF (TRANAM(itra).EQ.'CH_3Cl') THEN
          NOK=NOK+1
          iCH3Cl=itra
        ENDIF
C HCFC22
        IF (TRANAM(itra).EQ.'HCFC22') THEN
          NOK=NOK+1
          iHCFC22=itra
        ENDIF
C CH_3Br
        IF (TRANAM(itra).EQ.'CH_3Br') THEN
          NOK=NOK+1
          iCH3Br=itra
        ENDIF
C H1211
        IF (TRANAM(itra).EQ.'H1211') THEN
          NOK=NOK+1
          iH1211=itra
        ENDIF
C H1301
        IF (TRANAM(itra).EQ.'H1301') THEN
          NOK=NOK+1
          iH1301=itra
        ENDIF
C ClOx
        IF (TRANAM(itra).EQ.'ClO_x') THEN
          NOK=NOK+1
          iClOx=itra
        ENDIF
C O_x
        IF (TRANAM(itra).EQ.'O_x') THEN
          NOK=NOK+1
          iOx=itra
        ENDIF
C NO_x
        IF (TRANAM(itra).EQ.'NO_x') THEN
          NOK=NOK+1
          iNOx=itra
        ENDIF
      ENDDO
C
      DO itra=1,NSLS
C O_3
        IF (SLSNAM(itra).EQ.'O_3') THEN
          NOK=NOK+1
          iO3=itra
        ENDIF
      ENDDO
C
      IF (NOK.NE.17) THEN
        WRITE(*,*)
     +  '>> CONSTITUENT(S) MISSING FOR RELAXATION : ABORT!'
       CALL MOCAGE_STOP
      ENDIF

C
#ifdef REPROBUS
      DTT=DT/86400.
#else
      DTT=DT/(7.*86400.)
#endif
C
C  GENERAL
C
      DO iniv=NIVBAS,NIV
        DO ilon=1,NLON*NLAT
C
          QQQM(ilon,1,iniv,iN2O    ) = MAX(0. , TRALIM( 1) +
     +   (QQQM(ilon,1,iniv,iN2O    ) - TRALIM( 1))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iCH4    ) = MAX(0. , TRALIM( 2) +
     +   (QQQM(ilon,1,iniv,iCH4    ) - TRALIM( 2))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iCO     ) = MAX(0. , TRALIM( 3) +
     +   (QQQM(ilon,1,iniv,iCO     ) - TRALIM( 3))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iCFC11  ) = MAX(0. , TRALIM( 4) +
     +   (QQQM(ilon,1,iniv,iCFC11  ) - TRALIM( 4))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iCFC12  ) = MAX(0. , TRALIM( 5) +
     +   (QQQM(ilon,1,iniv,iCFC12  ) - TRALIM( 5))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iCFC113 ) = MAX(0. , TRALIM( 6) +
     +   (QQQM(ilon,1,iniv,iCFC113 ) - TRALIM( 6))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iCCl4   ) = MAX(0. , TRALIM( 7) +
     +   (QQQM(ilon,1,iniv,iCCl4   ) - TRALIM( 7))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iCH3CCl3) = MAX(0. , TRALIM( 8) +
     +   (QQQM(ilon,1,iniv,iCH3CCl3) - TRALIM( 8))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iCH3Cl  ) = MAX(0. , TRALIM( 9) +
     +   (QQQM(ilon,1,iniv,iCH3Cl  ) - TRALIM( 9))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iHCFC22 ) = MAX(0. , TRALIM(10) +
     +   (QQQM(ilon,1,iniv,iHCFC22 ) - TRALIM(10))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iCH3Br  ) = MAX(0. , TRALIM(11) +
     +   (QQQM(ilon,1,iniv,iCH3Br  ) - TRALIM(11))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iH1211  ) = MAX(0. , TRALIM(12) +
     +   (QQQM(ilon,1,iniv,iH1211  ) - TRALIM(12))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iH1301  ) = MAX(0. , TRALIM(13) +
     +   (QQQM(ilon,1,iniv,iH1301  ) - TRALIM(13))*EXP(-DTT) )
C
          QQQM(ilon,1,iniv,iClOx   ) = MAX(0. , TRALIM(14) +
     +   (QQQM(ilon,1,iniv,iClOx   ) - TRALIM(14))*EXP(-DTT) )
C
        ENDDO
      ENDDO
#ifdef REPROBUS
C
C  SPECIFIC TREATMENT FOR Ox, O3 AND NOx
C
      DO iniv=NIVBAS,NIV
C
        DO ilon=1,NLON*NLAT
          IF (XPAM(ilon,1,iniv).GT.37000.) THEN
            XLIM = -2.3825E-08*ALOG(XPAM(ilon,1,iniv)/100.)
     +           + 1.9457E-07
          ELSE
            XLIM = QQQM(ilon,1,iniv,iOx)
          ENDIF
          QQQM(ilon,1,iniv,iOx) = MAX(0. , XLIM +
     +             (QQQM(ilon,1,iniv,iOx) - XLIM)*EXP(-DTT) )
        ENDDO
C
        DO ilon=1,NLON*NLAT
          HHHM(ilon,1,iniv,iO3) = QQQM(ilon,1,iniv,iOx)
        ENDDO
C
      ENDDO
C
      DO iniv=NIVBAS,NIV
        XLIM = 0.4E-09 - (0.4E-09 - 0.1E-09)/REAL(NIV - NIVBAS)*
     +                   REAL(iniv - NIVBAS)
        DO ilon=1,NLON*NLAT
          QQQM(ilon,1,iniv,iNOx) = MAX(0. , XLIM +
     +             (QQQM(ilon,1,iniv,iNOx) - XLIM)*EXP(-DTT) )
        ENDDO
      ENDDO
#endif
C
      RETURN
      END
