      SUBROUTINE SIS(ndim,ntra,DT,var0,var,p,d)


!   __  __   _____   _____   _____   _____   _____
!  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
!  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
!  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
!
!   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
!   VERSION_RELEASE_BUGFIX : 0_1_1 
!
!CC DATE : 15/05/2005
!
!   Type      : Chemistry 
!   Purpose   : Semi-implicit scheme for chemistry
!   Called by : CHEM
!
!  Compute var(t+1), knowing var0(t-1) and production and
!  first order destruction rates. (semi-implicit solver)
!
!
!
!  Original : F. Lefevre, 1994
!
!  Modified : H. Teyssedre, 08/2000 (adaptation au format MOCAGE)



#ifdef MPI
	USE CSTES_MOD
#endif

	IMPLICIT NONE

!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~

	INTEGER :: ndim, ntra
	REAL (KIND=8) :: DT
	REAL (KIND=8) , DIMENSION (ndim,ntra) :: p,d,var0,var

!     Variables locales.
!     ~~~~~~~~~~~~~~~~~

      INTEGER :: I,J,i1,i2

#ifdef MPI
      i1=i_deb(myid) ; i2=i_fin(myid)
#else
      i1=1           ; i2=ndim
#endif

!$omp parallel do &
!$omp default(none) private(i,j) shared(i1,i2,ntra,var,var0,p,d,dt)
      do j=1,ntra
      do i=i1,i2
        var(i,j)=MAX( (var0(i,j) + DT*p(i,j))/(1. + DT*d(i,j)),1.e-10)
      enddo
      enddo
!$omp end parallel do

      RETURN
      END
