      SUBROUTINE SOURCE(XLON,XLAT,QTTY,PTOP,PBOT,POLL,UNITE,
     +                  RLAT,DRLAT,RLON,DRLON,
     +                  TGRIM,XPHFM,XPAM,QQQM,LON,LAT,NIV,NTRA,
     +                  POLLUTNAME,DEMIVIE,MOLWT,IGR)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Point source release model version
C   Purpose    : Emission from point sources
C   Called by  : IOPC3D
C
CCC
C
C  Original : B. Josse, M. Martet, 
C             L.-G. Razafindrakoto et V.-H. Peuch, 02/2003
C
C  Last Modified :
C
CCC
C
	USE CSTES_MOD
        USE EXPE_MOD
C
	IMPLICIT NONE	
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
C XLON : SOURCE LONGITUDES (Rad)
C XLAT : SOURCE LATITUDES (Rad) 
C QTTY : REJECT (BECQUEREL/HOUR OR g/s)
C PTOP : UPPER LEVEL OF EMISSION (>0 if in Pa ; <0 if in m)
C PBOT : LOWER LEVEL OF EMISSION (>0 if in Pa ; <0 if in m)
C POLL : POLLUTANT INDEX (INTEGER)
C POLLUTNAME : POLLUTANT NAME
C DEMIVIE : POLLUTANT HALF-LIFE (s)
C MOLWT    : POLLUTANT MOLECULAR WEIGHT (g/mol)
C
      INTEGER :: LON,LAT,NIV,NTRA,POLL,IGR
      REAL (KIND=8) :: XLAT,XLON,DRLON,DRLAT,PBOT,PTOP,QTTY
      CHARACTER(LEN=6) :: UNITE
      REAL (KIND=8) , DIMENSION (LON) :: RLON
      REAL (KIND=8) , DIMENSION (LAT) :: RLAT
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: TGRIM,XPHFM,XPAM
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV,NTRA) :: QQQM
      CHARACTER*15 POLLUTNAME
      REAL DEMIVIE
      REAL MOLWT
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
C HLB DIM_TEMP pour dimensionner des tableaux de dimension 1 non correctement
C                  alloues dans les versions precedentes de mocage.
C
	INTEGER :: DIM_TEMP
	INTEGER :: K,ILON,LEV,INDZBOT,INDZTOP,J,JM,I,IM
	REAL (KIND=8) :: TAU,QTY,DP,VOLTOT,DS,ALFX0,ALFY0
	REAL (KIND=8) :: ZEPSI,XFACT,S,V,XMASAIR,XMOLDAIR,XR
	REAL (KIND=8) :: QMOL,XINTER,SOMTOT,CSTSURF
      INTEGER , ALLOCATABLE :: INDX(:),INDY(:)
      REAL (KIND=8) , ALLOCATABLE :: ALFX1(:),ALFX2(:),ALFX3(:),ALFX4(:)
      REAL (KIND=8) , ALLOCATABLE :: ALFY1(:),ALFY2(:),ALFY3(:),ALFY4(:)      
      REAL (KIND=8) , ALLOCATABLE :: XTEMP(:,:)	
      REAL (KIND=8) , ALLOCATABLE :: XPH(:),XT(:),XP(:),XDZ(:),XZ(:)
      REAL (KIND=8) , ALLOCATABLE :: VOL(:),XMOLLEV(:)
      REAL (KIND=8) , ALLOCATABLE :: QQM(:,:,:)
      REAL (KIND=8) , ALLOCATABLE :: ZLEV(:),DZLEV(:)
C
C
	DIM_TEMP=1
C	
	ALLOCATE (ALFX1(DIM_TEMP))
	ALLOCATE (ALFX2(DIM_TEMP))
	ALLOCATE (ALFX3(DIM_TEMP))
	ALLOCATE (ALFX4(DIM_TEMP))
	ALLOCATE (ALFY1(DIM_TEMP))
	ALLOCATE (ALFY2(DIM_TEMP))
	ALLOCATE (ALFY3(DIM_TEMP))
	ALLOCATE (ALFY4(DIM_TEMP))
	ALLOCATE (INDX(DIM_TEMP))
	ALLOCATE (INDY(DIM_TEMP))
	ALLOCATE (XTEMP(LON,LAT))
	ALLOCATE (XPH(NIV))
	ALLOCATE (XT(NIV))
	ALLOCATE (XP(NIV))
	ALLOCATE (XDZ(NIV))
	ALLOCATE (XZ(NIV))
	ALLOCATE (VOL(NIV))
	ALLOCATE (XMOLLEV(NIV))
	ALLOCATE (QQM(4,4,NIV))
	ALLOCATE (ZLEV(NIV))
	ALLOCATE (DZLEV(NIV))

       CSTSURF=RTER*RTER*DRLON*DRLAT
C	
	
CCC 1. WEIGHTS FOR HORIZONTAL INTERPOLATION
C
      CALL POIDREG(XLON,RLON(1)-DRLON,DRLON
     +            ,INDX,ALFX1,ALFX2,ALFX3,ALFX4,1)
C
C Test sur la position du point source dans le domaine
C      
      IF ( (INDX(1).GE.1) .AND. (INDX(1).LE.LON) ) THEN
              
      CALL POIDREG(XLAT,RLAT(1)-DRLAT,DRLAT
     +            ,INDY,ALFY1,ALFY2,ALFY3,ALFY4,1)

      IF ( (INDY(1).GE.1) .AND. (INDY(1).LE.LAT) ) THEN

C
CCC 2. LOOP ON VERTICAL LEVELS
C     
      DO k=1,NIV
C
CCC 2.1 INTERPOLATE PRESSURE (HALF)
C
        DO ilon=1,LON*LAT
          XTEMP(ilon,1)=XPHFM(ilon,1,k)
        ENDDO
        CALL INTER2D(XTEMP,LON,LAT,
     +               INDX,INDY,
     +               ALFX1,ALFX2,ALFX3,ALFX4,
     +               ALFY1,ALFY2,ALFY3,ALFY4,
     +               XINTER,1,+1.,.FALSE.,IGR)
        XPH(k)=XINTER
C
CCC 2.2 INTERPOLATE TEMPERATURE (FULL)
C
        DO ilon=1,LON*LAT
          XTEMP(ilon,1)=TGRIM(ilon,1,k)
        ENDDO
        CALL INTER2D(XTEMP,LON,LAT,
     +               INDX,INDY,
     +               ALFX1,ALFX2,ALFX3,ALFX4,
     +               ALFY1,ALFY2,ALFY3,ALFY4,
     +               XINTER,1,+1.,.FALSE.,IGR)
        XT(k)=XINTER
C
CCC 2.3 INTERPOLATE PRESSURE (FULL)
C
        DO ilon=1,LON*LAT
          XTEMP(ilon,1)=XPAM(ilon,1,k)
        ENDDO
        CALL INTER2D(XTEMP,LON,LAT,
     +               INDX,INDY,
     +               ALFX1,ALFX2,ALFX3,ALFX4,
     +               ALFY1,ALFY2,ALFY3,ALFY4,
     +               XINTER,1,+1.,.FALSE.,IGR)
        XP(k)=XINTER             
      ENDDO
C
      IF (UNITE.EQ.'BQPERH') THEN
C
CCC  CONVERSION :  BECQUEREL TO NUMBER OF MOLECULE EMITTERS
C     
        TAU = DEMIVIE  / LOG(2.)
        QMOL = QTTY*TAU*DT/3600.

      ELSEIF (UNITE.EQ.'MOLPRS' ) THEN
        QMOL=QTTY*DT*XAVOG
        PRINT*,'QMOL=',QMOL
C        
      ELSEIF (UNITE.EQ.'GRPERS') THEN
C
CCC  CONVERT :  g.s-1 TO NUMBER OF MOLECULES EMITTED
C
        IF (MOLWT.GT.0.) THEN
          QMOL = QTTY*XAVOG*DT/MOLWT
        ELSE
          WRITE(*,*) '>> WARNING : NO KNOWN MOL. WEIGHT'
          WRITE(*,*) '>  NOTHING IS EMITTED...'
          QMOL = 0.
        ENDIF        
      ELSE
        WRITE(*,*) '>> WARNING : ',UNITE,
     +             ' IS NOT A KNOWN UNIT !'
      ENDIF
C
CCC 3. VERTICAL DISTRIBUTION
C
CCC 3.1 DETERMINE EMITTING LAYER(s)
C
CCC 3.1.1 MODEL LEVELS 
C 
      ZLEV(NIV)=0
      DO lev=NIV,2,-1
        DP=XPH(lev-1)-XPH(lev)
        DZLEV(lev)=-RA*DP*XT(lev)/(GG*XP(lev))
        ZLEV(lev-1)=ZLEV(lev)+DZLEV(lev)
      ENDDO
C
CCC 3.1.2 VERTICAL LEVEL OF PBOT
C
      IF (PBOT .gt. 0) THEN   
        INDZBOT=NIV
        DO WHILE ( PBOT .LT. XPH(INDZBOT) )
          INDZBOT=INDZBOT-1 
        ENDDO
      ELSE                           
        INDZBOT=NIV
        DO WHILE (ABS(PBOT) .GT. ZLEV(INDZBOT))
          INDZBOT=INDZBOT-1
        ENDDO
      ENDIF
      INDZBOT=INDZBOT+1 
      INDZBOT=(MIN(NIV,MAX(INDZBOT,2)))
C
CCC 3.1.3 VERTICAL LEVEL OF PTOP
C
      IF (PTOP .gt. 0) THEN
        INDZTOP=NIV
        DO WHILE ( PTOP .LT. XPH(INDZTOP) )
	  INDZTOP=INDZTOP-1 
	ENDDO
      ELSE
	INDZTOP=NIV
	DO WHILE (ABS(PTOP) .GT. ZLEV(INDZTOP))
	  INDZTOP=INDZTOP-1
	ENDDO
      ENDIF
      INDZTOP=INDZTOP+1
      INDZTOP=(MIN(NIV,MAX(INDZTOP,2)))

      PRINT*,'INDZTOP=',INDZTOP
      PRINT*,'INDZBOT=',INDZBOT

C
CCC 3.2 DISTRIBUTION FOR EACH LAYER BETWEEN INDZBOT AND INDZTOP
C
      VOLTOT=0.
      DS=CSTSURF*COS(RLAT(INDY(1)))
      INDZTOP=MAX(MIN(INDZTOP,INDZBOT-1),2)

      DO lev=INDZBOT,INDZTOP,-1
        DP=XPH(lev)-XPH(lev-1)
        VOL(lev)=DS*DZLEV(lev)
        VOLTOT=VOLTOT+VOL(lev)
        PRINT*,'COUCOU B1'
      ENDDO
C        
      DO lev=INDZBOT,INDZTOP,-1
        XMOLLEV(lev)=QMOL*VOL(lev)/VOLTOT
      ENDDO
C
CCC 3.3 HORIZONTAL DISTRIBUTION :=> INTERPOLATION WEIGHTS < 0 ?
C
CCC 3.3.1 MINIMUM OF ALFX AND ALFY
C
      ALFX0=MIN(ALFX1(1),MIN(ALFX2(1),MIN(ALFX3(1),ALFX4(1))))
      ALFY0=MIN(ALFY1(1),MIN(ALFY2(1),MIN(ALFY3(1),ALFY4(1))))

C
CCC 3.3.2 TRANSLATION OF ALFX AND ALFY INTO A POSITIVE VALUE
C
      IF (ALFX0 .LT. 0.) THEN
        ALFX1(1)=ALFX1(1)-ALFX0
        ALFX2(1)=ALFX2(1)-ALFX0
        ALFX3(1)=ALFX3(1)-ALFX0
        ALFX4(1)=ALFX4(1)-ALFX0
      ENDIF
      IF (ALFY0 .LT. 0.) THEN
        ALFY1(1)=ALFY1(1)-ALFY0
        ALFY2(1)=ALFY2(1)-ALFY0
        ALFY3(1)=ALFY3(1)-ALFY0
        ALFY4(1)=ALFY4(1)-ALFY0
      ENDIF
C
CCC 3.3.3 EACH ALFX AND ALFY .LT. ZEPSI=1.E-4 TAKEN AS 0.
C
      ZEPSI=1.E-4
      IF (ALFX1(1).LT.ZEPSI) ALFX1=0
      IF (ALFX2(1).LT.ZEPSI) ALFX2=0
      IF (ALFX3(1).LT.ZEPSI) ALFX3=0
      IF (ALFX4(1).LT.ZEPSI) ALFX4=0
      IF (ALFY1(1).LT.ZEPSI) ALFY1=0
      IF (ALFY2(1).LT.ZEPSI) ALFY2=0
      IF (ALFY3(1).LT.ZEPSI) ALFY3=0
      IF (ALFY4(1).LT.ZEPSI) ALFY4=0     
C
CCC 3.3.4 EQUIVALENT QUANTITY IN EACH CUBE-ELEMENT (4*4) ARROUND THE SOURCE 
C         
      DO k=INDZBOT,INDZTOP,-1         
C
        XFACT=XMOLLEV(k)/(ALFX1(1)*ALFY1(1)+ALFX2(1)*ALFY1(1)
     +                   +ALFX3(1)*ALFY1(1)+ALFX4(1)*ALFY1(1)
     +                   +ALFX1(1)*ALFY2(1)+ALFX2(1)*ALFY2(1)
     +                   +ALFX3(1)*ALFY2(1)+ALFX4(1)*ALFY2(1)
     +                   +ALFX1(1)*ALFY3(1)+ALFX2(1)*ALFY3(1)
     +                   +ALFX3(1)*ALFY3(1)+ALFX4(1)*ALFY3(1)
     +                   +ALFX1(1)*ALFY4(1)+ALFX2(1)*ALFY4(1)
     +                   +ALFX3(1)*ALFY4(1)+ALFX4(1)*ALFY4(1))
C
        QQM(1,1,k)=ALFX1(1)*ALFY1(1)*XFACT
        QQM(2,1,k)=ALFX2(1)*ALFY1(1)*XFACT
        QQM(3,1,k)=ALFX3(1)*ALFY1(1)*XFACT
        QQM(4,1,k)=ALFX4(1)*ALFY1(1)*XFACT
        QQM(1,2,k)=ALFX1(1)*ALFY2(1)*XFACT
        QQM(2,2,k)=ALFX2(1)*ALFY2(1)*XFACT
        QQM(3,2,k)=ALFX3(1)*ALFY2(1)*XFACT
        QQM(4,2,k)=ALFX4(1)*ALFY2(1)*XFACT
        QQM(1,3,k)=ALFX1(1)*ALFY3(1)*XFACT
        QQM(2,3,k)=ALFX2(1)*ALFY3(1)*XFACT
        QQM(3,3,k)=ALFX3(1)*ALFY3(1)*XFACT
        QQM(4,3,k)=ALFX4(1)*ALFY3(1)*XFACT
        QQM(1,4,k)=ALFX1(1)*ALFY4(1)*XFACT
        QQM(2,4,k)=ALFX2(1)*ALFY4(1)*XFACT
        QQM(3,4,k)=ALFX3(1)*ALFY4(1)*XFACT
        QQM(4,4,k)=ALFX4(1)*ALFY4(1)*XFACT

C
CCC 3.4.  CONVERSION IN MIXING RATIO AND INJECTION INTO QQQM
C         
C                  QQM/NAvogadro
C    XR =  _____________________________
C          n_air= ( ( P/(RaT)).V  )/Mair
C
        DO j = 1,4
          jM= INDY(1)+j-2
          jM=MIN(LAT,MAX(jM,1))
          S = CSTSURF*COS(RLAT(jm)) 
          DO i = 1,4
            iM= INDX(1)+i-2
            iM=MIN(LON,MAX(iM,1))
            XMASAIR=S * (XPHFM(im,jm,k) - XPHFM(im,jm,k-1)) / GG
            XMOLDAIR=XMASAIR*XAVOG/MAIR
            XR = QQM(i,j,k)/XMOLDAIR
            QQQM(iM,jM,k,POLL) = 
     +        QQQM(iM,jM,k,POLL) + XR
          ENDDO
        ENDDO
C
CCC   NEXT LEVEL
C
      ENDDO 
C
      ENDIF
C     
      ENDIF
C
	DEALLOCATE (ALFX1)
	DEALLOCATE (ALFX2)
	DEALLOCATE (ALFX3)
	DEALLOCATE (ALFX4)
	DEALLOCATE (ALFY1)
	DEALLOCATE (ALFY2)
	DEALLOCATE (ALFY3)
	DEALLOCATE (ALFY4)
	DEALLOCATE (INDX)
	DEALLOCATE (INDY)
	DEALLOCATE (XTEMP)
	DEALLOCATE (XPH)
	DEALLOCATE (XT)
	DEALLOCATE (XP)
	DEALLOCATE (XDZ)
	DEALLOCATE (XZ)
	DEALLOCATE (VOL)
	DEALLOCATE (XMOLLEV)
	DEALLOCATE (QQM)
	DEALLOCATE (ZLEV)
	DEALLOCATE (DZLEV)
C
C
      RETURN
      END
C
CCC 
C
