      SUBROUTINE SUMVERT(LON,LAT,NIV,RP,RPLT,RX,XINT)
C
CCCCC
C
C   VERTICAL INTEGRATION (CUBIC)
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 09/06/2005
C
C   Type      : Chemistry 
C   Purpose   : Compute a total column
C   Called by : 
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 04/2000
C
C  Modified :
C
CCCCC
C	
	IMPLICIT NONE
C
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: LON,LAT,NIV
	REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: RP,RPLT,RX,XINT
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
C CONSTANTS      
      INTEGER :: I,K,L
	REAL(KIND=8) :: MAIR,GG,AVO
C EXTENSION OF FIELD X ON THE VERTICAL              
	REAL (KIND=8) , ALLOCATABLE   :: XEXT(:)
C NUMBER OF MOLES IN EACH MODEL CELL
      REAL (KIND=8) , ALLOCATABLE   :: XMOL(:)
C WEIGHTS FOR NEIGHBORING LEVELS       	
	REAL (KIND=8), ALLOCATABLE   :: ALFZ1MAS(:,:,:),ALFZ2MAS(:,:,:)
	REAL (KIND=8), ALLOCATABLE   :: ALFZ3MAS(:,:,:),ALFZ4MAS(:,:,:)
	REAL (KIND=8), ALLOCATABLE   :: ALFZ5MAS(:,:,:)
C
C
	ALLOCATE (ALFZ1MAS(LON,LAT,NIV)) ; ALFZ1MAS(:,:,:) = 0.0
	ALLOCATE (ALFZ2MAS(LON,LAT,NIV)) ; ALFZ2MAS(:,:,:) = 0.0
	ALLOCATE (ALFZ3MAS(LON,LAT,NIV)) ; ALFZ3MAS(:,:,:) = 0.0
	ALLOCATE (ALFZ4MAS(LON,LAT,NIV)) ; ALFZ4MAS(:,:,:) = 0.0
	ALLOCATE (ALFZ5MAS(LON,LAT,NIV)) ; ALFZ5MAS(:,:,:) = 0.0
	ALLOCATE (XEXT(-1:NIV+2)) ; XEXT(:) = 0.0
	ALLOCATE (XMOL(NIV)) ; XMOL(:) = 0.0
C
C 1. INITIALIZATIONS
C
      DO L=1,NIV
!       XMOL(L)=0.
        DO K=1,LAT
        DO I=1,LON
          XINT(I,K,L)=0.
        ENDDO
        ENDDO
      ENDDO
C
      MAIR=0.029
      GG=9.80665
      AVO=6.02E23
C
C 2. COMPUTE WEIGHTS
C        
      CALL POIDZMASSE(LON,LAT,NIV,RP,RPLT
     +               ,ALFZ1MAS,ALFZ2MAS
     +               ,ALFZ3MAS,ALFZ4MAS
     +               ,ALFZ5MAS)
C
C 3. COMPUTE NUMBER OF MOLES IN EACH MODEL
C    CELL (CUBIC INTERPOLATION)
C    
      XEXT(   -1)=0.
      XEXT(    0)=0.
      XEXT(NIV+1)=0.
      XEXT(NIV+2)=0.
C        
      DO K=1,LAT
      DO I=1,LON
        DO L=1,NIV
          XEXT(L)=RX(I,K,L)
        ENDDO
        DO L=1,NIV          
          XMOL(L)=ALFZ1MAS(I,K,L)*XEXT(L-2)
     +           +ALFZ2MAS(I,K,L)*XEXT(L-1)
     +           +ALFZ3MAS(I,K,L)*XEXT(L  )
     +           +ALFZ4MAS(I,K,L)*XEXT(L+1)
     +           +ALFZ5MAS(I,K,L)*XEXT(L+2)
        ENDDO
C
C SUM ON THE VERTICAL
C          
#ifdef NIV_60
        XINT(I,K,1)=0.5*RP(I,K,1)*RX(I,K,1)/GG/MAIR*AVO
     +             +RX(I,K,1)*(RPLT(I,K,1)-RP(I,K,1))/GG/MAIR*AVO
C        XINT(I,K,1)=XMOL(1)/GG/MAIR*AVO
#else
        XINT(I,K,1)=0.75*RP(I,K,1)*RX(I,K,1)/GG/MAIR*AVO
     +             +RX(I,K,1)*(RPLT(I,K,1)-RP(I,K,1))/GG/MAIR*AVO
#endif
        DO L=2,NIV
          XINT(I,K,L)=XINT(I,K,L-1)+XMOL(L)/GG/MAIR*AVO
        ENDDO
      ENDDO
      ENDDO
C
	DEALLOCATE (ALFZ1MAS)
	DEALLOCATE (ALFZ2MAS)
	DEALLOCATE (ALFZ3MAS)
	DEALLOCATE (ALFZ4MAS)
	DEALLOCATE (ALFZ5MAS)
	DEALLOCATE (XEXT)
	DEALLOCATE (XMOL)
C
      END
C
CCC
C
      SUBROUTINE POIDZMASSE(LON,LAT,NIV
     +                     ,PL,PLT
     +                     ,ALFZ1MAS,ALFZ2MAS
     +                     ,ALFZ3MAS,ALFZ4MAS
     +                     ,ALFZ5MAS)
C     
	IMPLICIT NONE
C	
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: LON,LAT,NIV
	REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: PL,PLT
	REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: ALFZ1MAS,ALFZ2MAS
	REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: ALFZ3MAS,ALFZ4MAS
	REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: ALFZ5MAS
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~      
C
	INTEGER :: I,J,K,L,LP1
	REAL (KIND=8) :: PL1,PL2,PLT0,PLT1,PLT2,COEF,DP2
	REAL (KIND=8) :: RPN,RPNM1,RPTN,RPTNM1
	REAL (KIND=8) :: RP1,RP2,RP3,RP4,DP1
	REAL (KIND=8) :: RP12,RP13,RP14,RP23,RP24,RP34
	REAL (KIND=8) , ALLOCATABLE :: INVPROD(:,:),SOMPROD(:,:)
	REAL (KIND=8) , ALLOCATABLE :: PROD(:,:),SOM(:,:)
	REAL (KIND=8) , ALLOCATABLE :: ZCOEF(:)	
C        
	ALLOCATE (INVPROD(NIV,4)) ; INVPROD(:,:) = 0
	ALLOCATE (SOMPROD(NIV,4)) ; SOMPROD(:,:) = 0.0
	ALLOCATE (PROD(NIV,4)); PROD(:,:) = 0.0
	ALLOCATE (SOM(NIV,4)); SOM(:,:) = 0.0
	ALLOCATE (ZCOEF(4))
C
      DO 1 I=1,LON
      DO 1 K=1,LAT
C
C 1er cas: maille superieure L=1
C Valeur prise constante dans la premiere moitie de maille,
C puis interpolation lineaire entre L=1 et L~=1
C
        PL1=PL(I,K,1)
        PL2=PL(I,K,2)
        PLT1=PLT(I,K,1)
        PLT0=0.
        COEF=((PLT1-PL1)**2)/(PL2-PL1)
        COEF=COEF/2.
        ALFZ1MAS(I,K,1)=0.
        ALFZ2MAS(I,K,1)=0.
        ALFZ3MAS(I,K,1)=(PLT1-PLT0)-COEF
        ALFZ4MAS(I,K,1)=COEF          
        ALFZ5MAS(I,K,1)=0.                    
C                  
C 2 eme cas: maille L=2.
C Interpolation lineaire dans la premiere moitie de maille
C puis cubique dans la seconde moitie (les coefficients
C beta correspondant a la seconde moitie sont calcules plus loin)
C                
        COEF=((PL2-PLT1)**2)/2.     
        COEF=COEF/(PL2-PL1)
        ALFZ1MAS(I,K,2)=0.
        ALFZ2MAS(I,K,2)=COEF
        ALFZ3MAS(I,K,2)=(PL2-PLT1)-COEF
        ALFZ4MAS(I,K,2)=0.
C
C Cas general: interpolation cubique dans les deux moities de mailles
C coefficients de la moitie superieure (courbe passant par les points
C L-2,L-1,L et L+1):ALF
C
C La masse dans la maille sera calculee par:
C  ALFZ1 X(L-2) + (ALFZ2)X(L-1) + (ALFZ3) X(L)
C + (ALFZ4) X(L+1) + ALFZ5 X(L+2)
C
C Calcul de ALF(Gauche) de L=3 a NIV-1
C Calcul de ALF(Droite) de L=2 a NIV-2
C
        DO 10 L=3,NIV-1
          RP1=PL(I,K,L-2)
          RP2=PL(I,K,L-1)
          RP3=PL(I,K,L  )
          RP4=PL(I,K,L+1)
          DP1=PLT(I,K,L-1)
          RP12=RP1*RP2
          RP13=RP1*RP3
          RP14=RP1*RP4
          RP23=RP2*RP3
          RP24=RP2*RP4
          RP34=RP3*RP4
          INVPROD(L,1)=1./((RP1-RP2)*(RP1-RP3)*(RP1-RP4))
          INVPROD(L,2)=1./((RP2-RP1)*(RP2-RP3)*(RP2-RP4))
          INVPROD(L,3)=1./((RP3-RP1)*(RP3-RP2)*(RP3-RP4))
          INVPROD(L,4)=1./((RP4-RP1)*(RP4-RP2)*(RP4-RP3))
          SOM(L,1)=RP2+RP3+RP4
          SOM(L,2)=RP1+RP3+RP4
          SOM(L,3)=RP1+RP2+RP4
          SOM(L,4)=RP1+RP2+RP3
          PROD(L,1)=RP2*RP3*RP4
          PROD(L,2)=RP1*RP3*RP4
          PROD(L,3)=RP1*RP2*RP4
          PROD(L,4)=RP1*RP2*RP3
          SOMPROD(L,1)=RP23+RP24+RP34
          SOMPROD(L,2)=RP13+RP14+RP34
          SOMPROD(L,3)=RP12+RP14+RP24
          SOMPROD(L,4)=RP12+RP13+RP23
          DO 101 J=1,4
            ZCOEF(J)=             (RP3**4-DP1**4)/4.
     +              -SOM    (L,J)*(RP3**3-DP1**3)/3.
     +              +SOMPROD(L,J)*(RP3**2-DP1**2)/2.
     +              -PROD   (L,J)*(RP3   -DP1   )
            ZCOEF(J)=INVPROD(L,J)*ZCOEF(J)
 101      CONTINUE
          ALFZ1MAS(I,K,L)=ZCOEF(1)
          ALFZ2MAS(I,K,L)=ZCOEF(2)
          ALFZ3MAS(I,K,L)=ZCOEF(3)
          ALFZ4MAS(I,K,L)=ZCOEF(4)
 10     CONTINUE
C
        DO 11 L=2,NIV-2
           LP1=L+1
           RP3=PL(I,K,L  )
           RP4=PL(I,K,L+1)
           DP2=PLT(I,K,L)
           DO 111 J=1,4
             ZCOEF(J)=               (DP2**4-RP3**4)/4.
     +               -SOM    (LP1,J)*(DP2**3-RP3**3)/3.
     +               +SOMPROD(LP1,J)*(DP2**2-RP3**2)/2.
     +               -PROD   (LP1,J)*(DP2   -RP3   )
             ZCOEF(J)=INVPROD(LP1,J)*ZCOEF(J)
 111       CONTINUE
           ALFZ2MAS(I,K,L)=ALFZ2MAS(I,K,L)+ZCOEF(1)
           ALFZ3MAS(I,K,L)=ALFZ3MAS(I,K,L)+ZCOEF(2)
           ALFZ4MAS(I,K,L)=ALFZ4MAS(I,K,L)+ZCOEF(3)
           ALFZ5MAS(I,K,L)=ZCOEF(4)
 11     CONTINUE
C
C Avant-derniere maille (pres du sol: L=NIV-1)
C Meme principe que pour L=2
C Les ALFZ ont ete calcules plus haut
C
        RPN   =PL (I,K,NIV  )
        RPNM1 =PL (I,K,NIV-1)
        RPTN  =PLT(I,K,NIV  )
        RPTNM1=PLT(I,K,NIV-1)
        COEF=(RPTNM1**2-RPNM1**2)/2.-RPN*(RPTNM1-RPNM1)
        COEF=COEF/(RPNM1-RPN)         
        ALFZ3MAS(I,K,NIV-1)=ALFZ3MAS(I,K,NIV-1)+               COEF
        ALFZ4MAS(I,K,NIV-1)=ALFZ4MAS(I,K,NIV-1)+(RPTNM1-RPNM1)-COEF
        ALFZ5MAS(I,K,NIV-1)=0.
C
C Derniere maille (L=NIV)
C meme principe que pour L=1
C
        COEF=(RPN**2-RPTNM1**2)/2.-RPN*(RPN-RPTNM1)
        COEF=COEF/(RPNM1-RPN)
C
C essai d'interpolation lineaire jusqu'au sol
C sans correction en cas de passage sous 0
C
        ALFZ1MAS(I,K,NIV)=0.
        ALFZ2MAS(I,K,NIV)=           -COEF
        ALFZ3MAS(I,K,NIV)=RPTN-RPTNM1+COEF
        ALFZ4MAS(I,K,NIV)=0.         
        ALFZ5MAS(I,K,NIV)=0.
C         
 1    CONTINUE     
C
	DEALLOCATE (INVPROD)
	DEALLOCATE (SOMPROD)
	DEALLOCATE (PROD)
	DEALLOCATE (SOM)
	DEALLOCATE (ZCOEF)
C
C
	END  
