      SUBROUTINE TCVCALSUB(J,LON,LAT,NIV,NTRA,
     +                 SURFM,
     +                 PL,PLT,PSOL,EJ1,
     +                 U3D,V3D,T3D,Q3D,
     +                 DIVQ3D,DIVU3D,
     +                 QQQM,LPFLX,
     +                 IDEB,IFIN,LDEB,LFIN,ICALCONV,
     +                 BIGG,IBIGIP)     
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Convection and Diffusion (Tiedtke/Louis) 
C   Purpose   : Compute cloud ent/detrainment fluxes and vertical diffusion
C               coefficients from model fields. This version uses
C               hermite cubics for the vertical interpolation 
C   Called by : 
C
C     Externals
C     ---------
C     SUBROUTINES:   TCVCONVMA
C                    TCVSUBSCAL
C                    TCVCONZOM
C
	USE CSTES_MOD
C
	IMPLICIT NONE	
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: J,LON,LAT,NIV,NTRA
      INTEGER :: IDEB,IFIN,LDEB,LFIN
      LOGICAL :: ICALCONV
	REAL (KIND=8) , DIMENSION (LON,LAT) :: SURFM,PSOL,EJ1
	REAL (KIND=8), DIMENSION (LON,LAT,NIV) :: PL,U3D,V3D,T3D,Q3D
	REAL (KIND=8), DIMENSION (LON,LAT,NIV) :: DIVQ3D,DIVU3D
	REAL (KIND=8), DIMENSION (LON,LAT,NIV) :: LPFLX,IBIGIP
      REAL (KIND=8), DIMENSION (LON,LAT,0:NIV) :: PLT
      REAL (KIND=8), DIMENSION (LON,LAT,NIV,NTRA) :: QQQM
      REAL (KIND=8), DIMENSION (LON,LAT,NIV,NIV) :: BIGG
	
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: I,L,NC,ITAU,N,M1,I1,J1,J1P1,J1M1,I1P1,I1M1,K,LH,IS,ICON
	REAL (KIND=8) :: PLLZ,PLUZ,ALPHA,BETA,CZETA,CZETAK
	REAL (KIND=8) :: AMMIN,AMAXDIFF
      PARAMETER (AMMIN   =1.0E-10)
      PARAMETER (AMAXDIFF=1.0E5)
C
      REAL (KIND=8) , ALLOCATABLE :: ENTGM(:,:),DETGM(:,:),DKGM(:,:)
      REAL (KIND=8) , ALLOCATABLE :: ENTG(:,:),DETG(:,:),DKG(:,:)
      REAL (KIND=8) , ALLOCATABLE :: ZENTG(:),ZDETG(:),ZDKG(:)
	REAL (KIND=8) , ALLOCATABLE :: ZPQP(:),QEVAP(:)
C
C Valeurs des parametres sur les niveaux du modele
C
      REAL (KIND=8) , ALLOCATABLE :: PRESS(:,:),TTEMP(:,:),QTEMP(:,:)
      REAL (KIND=8) , ALLOCATABLE :: UTEMP(:,:),VTEMP(:,:)
      REAL (KIND=8) , ALLOCATABLE :: DIVU(:,:),DIVQ(:,:)
C
C Valeurs des parametres sur les niveaux intercouches du modele
C
      REAL (KIND=8) , ALLOCATABLE :: WZ(:,:),WT(:,:),WQ(:,:)
      REAL (KIND=8) , ALLOCATABLE :: WU(:,:),WV(:,:)
      REAL (KIND=8) , ALLOCATABLE :: QAC(:,:),QAM(:,:)
C
C     Pressure of CTM interlevels
      REAL (KIND=8) , ALLOCATABLE :: TMPRESS(:,:)

      REAL (KIND=8) , ALLOCATABLE :: AM(:),ENTU(:),DETU(:)
      REAL (KIND=8) , ALLOCATABLE :: ENTD(:),DETD(:),AK(:)
      REAL (KIND=8) , ALLOCATABLE :: G(:,:)
      REAL (KIND=8) , ALLOCATABLE :: VW(:)
      INTEGER , ALLOCATABLE :: IP(:)
C     
      REAL (KIND=8) , ALLOCATABLE :: SM(:,:,:)
C
C INITIALISATIONS
C
      ALLOCATE (ENTGM(LDEB:NIV,IFIN))	 ; ENTGM(:,:) = 0.0
      ALLOCATE (DETGM(LDEB:NIV,IFIN))	 ; DETGM(:,:) = 0.0
      ALLOCATE (DKGM (LDEB:NIV,IFIN))	 ; DKGM (:,:) = 0.0
      ALLOCATE (ENTG (LDEB:NIV,IFIN))	 ; ENTG (:,:) = 0.0
      ALLOCATE (DETG(LDEB:NIV,IFIN))	 ; DETG(:,:) = 0.0
      ALLOCATE (DKG  (LDEB:NIV,IFIN))	 ; DKG  (:,:) = 0.0
      ALLOCATE (ZENTG(LDEB:NIV))	 ; ZENTG(:) = 0.0
      ALLOCATE (ZDETG(LDEB:NIV))	 ; ZDETG(:) = 0.0
      ALLOCATE (ZDKG(LDEB:NIV))	 ; ZDKG(:) = 0.0
      ALLOCATE (ZPQP(0:NIV))	 ; ZPQP(:) = 0.0
      ALLOCATE (QEVAP(IFIN))      	 ; QEVAP(:) = 0.0      
      ALLOCATE (PRESS(NIV,IFIN))	 ; PRESS(:,:) = 0.0
      ALLOCATE (TTEMP(NIV,IFIN))	 ; TTEMP(:,:) = 0.0
      ALLOCATE (QTEMP(NIV,IFIN))	 ; QTEMP(:,:) = 0.0
      ALLOCATE (UTEMP(NIV,IFIN))	 ; UTEMP(:,:) = 0.0
      ALLOCATE (VTEMP(NIV,IFIN))	 ; VTEMP(:,:) = 0.0
      ALLOCATE (DIVU(NIV,IFIN))	 ; DIVU(:,:) = 0.0
      ALLOCATE (DIVQ(NIV,IFIN))	 ; DIVQ(:,:) = 0.0
      ALLOCATE (WZ(0:NIV,IFIN))	 ; WZ(:,:) = 0.0
      ALLOCATE (WT(0:NIV,IFIN))	 ; WT(:,:) = 0.0
      ALLOCATE (WQ(0:NIV,IFIN))	 ; WQ(:,:) = 0.0
      ALLOCATE (WU(0:NIV,IFIN))	 ; WU(:,:) = 0.0
      ALLOCATE (WV(0:NIV,IFIN))	 ; WV(:,:) = 0.0
      ALLOCATE (QAC(0:NIV,IFIN))	 ; QAC(:,:) = 0.0
      ALLOCATE (QAM(0:NIV,IFIN))	 ; QAM(:,:) = 0.0
      ALLOCATE (TMPRESS(0:NIV,IFIN))	 ; TMPRESS(:,:) = 0.0
      ALLOCATE (AM(LDEB:NIV))	 ; AM(:) = 0.0
      ALLOCATE (ENTU(LDEB:NIV))	 ; ENTU(:) = 0.0
      ALLOCATE (DETU(LDEB:NIV))	 ; DETU(:) = 0.0
      ALLOCATE (ENTD(LDEB:NIV))	 ; ENTD(:) = 0.0
      ALLOCATE (DETD(LDEB:NIV))	 ; DETD(:) = 0.0
      ALLOCATE (AK(LDEB-1:NIV))	 ; AK(:) = 0.0
      ALLOCATE (G(LDEB:NIV,LDEB:NIV))	 ; G(:,:) = 0.0
      ALLOCATE (VW(LDEB:NIV))	 ; VW(:) = 0.0
      ALLOCATE (IP(LDEB:NIV))     	 ; IP(:) = 0.0     
      ALLOCATE (SM(LON,LAT,NIV))	 ; SM(:,:,:) = 0.0
C
       DO L=LDEB,NIV
         DO I=IDEB,IFIN
          ENTGM(L,I)=0.
          DETGM(L,I)=0.
          DKGM(L,I)=0.
         ENDDO
       ENDDO     
C
C Loop over analysis times to average
C                          
       NC=0
       DO 10 ITAU=1,1     
C       
        DO 100 I=IDEB,IFIN 
C         
             N=IFIN*(J-1)+I
             M1=(N-1)/LON
C             
             I1=N-LON*M1
             J1=M1+1         
C             
             J1P1=MIN(LAT,J1+1)
             J1M1=MAX(1  ,J1-1)
             I1P1=MIN(LON,I1+1)
             I1M1=MAX(1  ,I1-1)
C             
C Pressure at level interfaces
C
          DO 1001 L=0,NIV
     
            TMPRESS(L,I)=PLT(I1,J1,L)
            
 1001     CONTINUE
 
C  
          DO 1002 L=1,NIV
         
             PRESS(L,I)=PL (I1,J1,L)
             TTEMP(L,I)=T3D(I1,J1,L) 
             QTEMP(L,I)=Q3D(I1,J1,L)
             UTEMP(L,I)=U3D(I1,J1,L)
             VTEMP(L,I)=V3D(I1,J1,L)
             DIVQ(L,I)=DIVQ3D(I1,J1,L)
             DIVU(L,I)=DIVU3D(I1,J1,L)
             SM   (I1,J1,L)=ABS(PLT(I1,J1,L)-PLT(I1,J1,L-1))
     +                     *SURFM(I1,J1)/GG
 1002     CONTINUE
C
C Interpolate variables onto interlevels of CTM grid 
C

C         Calcul de Z
                   
          WZ(NIV,I)=RA*TTEMP(NIV,I)/GG*
     +              MAX(0.,ALOG(PREF/TMPRESS(NIV,I)))
C          
C Passage 1D du flux d'evaporation
C
          QEVAP(I)=EJ1(I1,J1)
C
 100      CONTINUE
C
          IF (ICALCONV) THEN 
C
          DO 101 L=NIV-1,0,-1
            DO 1010 I=IDEB,IFIN
            K=MAX(L,1)       
            PLLZ=TMPRESS(L+1,I) 
            PLUZ=TMPRESS(K  ,I) 
            WZ(L,I)=WZ(L+1,I) + RA*TTEMP(L+1,I)*ALOG(PLLZ/PLUZ)/GG
 1010      CONTINUE          
 101      CONTINUE
C
C Calcul des parametres sur les niveaux intercouches de 1 a NIV-1
C
          DO 102 L=1,NIV-1
           DO 1020 I=IDEB,IFIN
C  
           ALPHA=(TMPRESS(L,I)-PRESS(L,I))/(TMPRESS(L+1,I)-PRESS(L,I))
           BETA=1.-ALPHA
C         
           WU(L,I) =ALPHA*UTEMP(L+1,I)+BETA*UTEMP(L,I)
           WV(L,I) =ALPHA*VTEMP(L+1,I)+BETA*VTEMP(L,I)
           WQ(L,I) =ALPHA*QTEMP(L+1,I)+BETA*QTEMP(L,I)
           WT(L,I) =ALPHA*TTEMP(L+1,I)+BETA*TTEMP(L,I)
           QAC(L,I)=ALPHA*DIVQ (L+1,I)+BETA*DIVQ(L,I)
           QAM(L,I)=ALPHA*DIVU (L+1,I)+BETA*DIVU(L,I)
 1020      CONTINUE        
 102      CONTINUE 
C
C Extrapolation pour les niveaux intercouches 0 et NIV
C
       DO 103 I=IDEB,IFIN
C
C 1)   Niveau intercouche NIV
C         
          ALPHA=(TMPRESS(NIV,I)-TMPRESS(NIV-1,I))
     +           /(PRESS(NIV,I)-TMPRESS(NIV-1,I))
          BETA=1.-ALPHA
          WU(NIV,I) =ALPHA*UTEMP(NIV,I)+BETA* WU(NIV-1,I)
          WV(NIV,I) =ALPHA*VTEMP(NIV,I)+BETA* WV(NIV-1,I)    
          WQ(NIV,I) =ALPHA*QTEMP(NIV,I)+BETA* WQ(NIV-1,I)  
          WT(NIV,I) =ALPHA*TTEMP(NIV,I)+BETA* WT(NIV-1,I)  
          QAC(NIV,I)=ALPHA*DIVQ (NIV,I)+BETA*QAC(NIV-1,I)   
          QAM(NIV,I)=ALPHA*DIVU (NIV,I)+BETA*QAM(NIV-1,I) 
C        
C 2) Niveau intercouche 0
C
          ALPHA=(TMPRESS(1,I)-TMPRESS(0,I))/(TMPRESS(1,I)-PRESS(1,I))
          BETA=1.-ALPHA
          WU(0,I) =ALPHA*UTEMP(1,I)+BETA* WU(1,I)
          WV(0,I) =ALPHA*VTEMP(1,I)+BETA* WV(1,I)  
          WQ(0,I) =ALPHA*QTEMP(1,I)+BETA* WQ(1,I)
          WT(0,I) =ALPHA*TTEMP(1,I)+BETA* WT(1,I)
          QAC(0,I)=ALPHA*DIVQ (1,I)+BETA*QAC(1,I)
          QAM(0,I)=ALPHA*DIVU (1,I)+BETA*QAM(1,I)
C                
 103     CONTINUE
C          
C Calcul des taux d'entrainement et de detrainement, 
C  et des coefficients de diffusion
C
        DO 104 I=IDEB,IFIN
C    
          CALL TCVSUBSCAL(NIV
     +                ,WZ(0,I),TMPRESS(0,I),WU(0,I)
     +                ,WV(0,I),WT(0,I),WQ(0,I)
     +                ,QAC(0,I),QAM(0,I),QEVAP(I)
     +                ,ZENTG,ZDETG,ZDKG,ZPQP  
     +                ,LDEB,LFIN)
C
C Stockage					
C
          N=IFIN*(J-1)+I
          M1=(N-1)/LON
          I1=N-LON*M1
          J1=M1+1
          DO L=1,NIV
            LPFLX(I1,J1,L)=ZPQP(L)
          ENDDO
          DO 1040 L=LDEB,NIV
            ENTGM(L,I)=ENTGM(L,I) + ZENTG(L)
            DETGM(L,I)=DETGM(L,I) + ZDETG(L)
            DKGM (L,I)=DKGM (L,I) + ZDKG (L)
 1040     CONTINUE
 104      CONTINUE
C
          NC=NC+1
C
        ENDIF
C
 10     CONTINUE
 
C
       IF (ICALCONV) THEN      
C
       DO 11 L=LDEB,NIV
         DO 110 I=IDEB,IFIN
C
         ENTG(L,I)=ENTGM(L,I)/REAL(NC)   
         DETG(L,I)=DETGM(L,I)/REAL(NC)     
         DKG (L,I)=DKGM (L,I)/REAL(NC)
C        
 110     CONTINUE
 11    CONTINUE 
C
       ENDIF
C          
C     Ent/detrainment rates go from top to bottom! 
C     make airmass per level and m**2 also go from top to bottom.
C     Scale rates and coefficients by czeta, resp. czetaK
C     We make en/detrainment rates explicitely positive 
C    (they might have become slighly negative from grib coding)
C     Calculate convection matrix
C              
      CZETA =1.0
      CZETAK=1.0
C              
       DO 12 I=IDEB,IFIN
             N=IFIN*(J-1)+I
             M1=(N-1)/LON
             I1=N-LON*M1
             J1=M1+1  
C
       IF (ICALCONV) THEN
C
       DO 120 L=LDEB,NIV
            AM  (L)=MAX(AMMIN,SM(I1,J1,L)/SURFM(I1,J1))
            ENTU(L)=MAX(0.,ENTG(L,I))*CZETA
            DETU(L)=MAX(0.,DETG(L,I))*CZETA
            ENTD(L)=0.
            DETD(L)=0.
            AK  (L)=MAX(0.,DKG(L,I))*CZETAK
 120    CONTINUE  
 
C             
C Calcul des coefficients de la matrice I-M*dt, NON INVERSEE
C                          
       CALL TCVCONVMA(NIV,LDEB,AM,ENTU,DETU,ENTD,DETD,AK,G)
C
C
       LH=NIV-LDEB+1
       CALL DALU(G,LH,LH,0.,IP,IS,VW,ICON)

       DO L=LDEB,NIV
        IBIGIP(I1,J1,L)=IP(L)
       DO K=LDEB,NIV
         BIGG(I1,J1,K,L)=G(K,L)
       ENDDO
       ENDDO
C
       ELSE
C
       DO L=LDEB,NIV
        IP(L)=IBIGIP(I1,J1,L)
       DO K=LDEB,NIV
        G(K,L)=BIGG(I1,J1,K,L)
       ENDDO
       ENDDO
C
       ENDIF
C
C Calcul des nouvelles masses d'air et de traceur
C
       CALL TCVCONZOM(LON,LAT,NIV,LDEB,NTRA,I1,J1,G,QQQM,SM,IP)
C
 12    CONTINUE
C       
       DEALLOCATE (ENTGM)
       DEALLOCATE (DETGM)
       DEALLOCATE (DKGM)
       DEALLOCATE (ENTG)
       DEALLOCATE (DETG)
       DEALLOCATE (DKG)
       DEALLOCATE (ZENTG)
       DEALLOCATE (ZDETG)
       DEALLOCATE (ZDKG)
       DEALLOCATE (ZPQP)
       DEALLOCATE (QEVAP)
       DEALLOCATE (PRESS)
       DEALLOCATE (TTEMP)
       DEALLOCATE (QTEMP)
       DEALLOCATE (UTEMP)
       DEALLOCATE (VTEMP)
       DEALLOCATE (DIVU)
       DEALLOCATE (DIVQ)
       DEALLOCATE (WZ)
       DEALLOCATE (WT)
       DEALLOCATE (WQ)
       DEALLOCATE (WU)
       DEALLOCATE (WV)
       DEALLOCATE (QAC)
       DEALLOCATE (QAM)
       DEALLOCATE (TMPRESS)
       DEALLOCATE (AM)
       DEALLOCATE (ENTU)
       DEALLOCATE (DETU)
       DEALLOCATE (ENTD)
       DEALLOCATE (DETD)
       DEALLOCATE (AK)
       DEALLOCATE (G)
       DEALLOCATE (VW)
       DEALLOCATE (IP)
       DEALLOCATE (SM)
C
C	
       RETURN
       END
