      SUBROUTINE TCVSUBSCAL(NIV,
     +                   PZ,PPRESS,PU,PV,PT,PQ,PQAC,PQAM,PEVAP,
     +                   PENTG,PDETG,PDKG,ZPQP,
     +                   LDEB,LFIN)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Convection and Diffusion (Tiedtke/Louis)
C   Purpose   : Compute sub-gridscale parameters 
C   Called by : 
C
	USE CSTES_MOD
C
	IMPLICIT NONE	
C
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: NIV,LDEB,LFIN
	REAL (KIND=8) :: PEVAP
      REAL (KIND=8) , DIMENSION (0:NIV) :: PZ,PPRESS,PT
      REAL (KIND=8) , DIMENSION (0:NIV) :: PU,PV
      REAL (KIND=8) , DIMENSION (0:NIV) :: PQ,PQAC,PQAM      
C     Output variable on centre of CTM vertical grid
C     Entrainment and detrainment rates
      REAL (KIND=8) , DIMENSION (LDEB:NIV) :: PENTG,PDETG,PDKG
	REAL (KIND=8) , DIMENSION (0:NIV) :: ZPQP
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
	INTEGER :: JL,LCT
	REAL (KIND=8) :: ZAM
	REAL (KIND=8) , ALLOCATABLE :: ZAMU(:),ZPTC(:),ZPQC(:),ZPLC(:)
      REAL (KIND=8) , ALLOCATABLE :: ZPDK(:)
      REAL (KIND=8) , ALLOCATABLE :: ZPAM(:),ZPEU(:),ZPDU(:)
	REAL (KIND=8) , ALLOCATABLE :: ZPED(:),ZPDD(:)      	
C     switches for model
      LOGICAL :: LCONV,LVDIF
      PARAMETER (LCONV=.TRUE.,LVDIF=.TRUE.) 
C
C      
      ALLOCATE (ZAMU(0:NIV))	 ; ZAMU(:) = 0.0
      ALLOCATE (ZPTC(0:NIV))	 ; ZPTC(:) = 0.0
      ALLOCATE (ZPQC(0:NIV))	 ; ZPQC(:) = 0.0
      ALLOCATE (ZPLC(0:NIV))	 ; ZPLC(:) = 0.0
      ALLOCATE (ZPDK(LDEB-1:NIV))	 ; ZPDK(:) = 0.0
      ALLOCATE (ZPAM(NIV))	 ; ZPAM(:) = 0.0
      ALLOCATE (ZPEU(NIV))	 ; ZPEU(:) = 0.0
      ALLOCATE (ZPDU(NIV))	 ; ZPDU(:) = 0.0
      ALLOCATE (ZPED(NIV))	 ; ZPED(:) = 0.0
      ALLOCATE (ZPDD(NIV))	 ; ZPDD(:) = 0.0
C
C
C  1. Calculate convection by clouds
C
      IF (LCONV) THEN
        CALL TCVCLOUD(NIV,
     +             PZ,PPRESS,PT,PQ,PQAC,PQAM,PEVAP,
     +             ZAMU,ZPTC,ZPQC,ZPLC,ZPQP,
     +             ZPAM,ZPEU,ZPDU,ZPED,ZPDD,
     +             LDEB,LFIN)
     
      ENDIF
C 
C  2. Calculate turbulent diffusion
C
      IF (LVDIF) THEN
        CALL TCVLOUIS(NIV,LDEB,PZ,PT,PU,PV,ZPDK)
      ENDIF      
C
C
C     Put entrainment/detrainment rates on tmg vertical coordinate:
      
      DO 1 JL=LDEB,NIV
        PENTG(JL)=ZPEU(JL)
        PDETG(JL)=ZPDU(JL)
 1    CONTINUE
C
C     CORRECT MASSFLUXES ON TMG GRID TO CONSERVE MASS
C     CORRECTION LEVEL IS UPPERMOST LEVEL WITH NONZERO DETRAINMENT
C
      LCT=0
      DO 2 JL=LDEB,NIV-1
        IF(PDETG(JL).LE.0.) THEN
          LCT=JL
        ENDIF
2     CONTINUE
      LCT=LCT+1
C
      ZAM=0.
      DO 3 JL=NIV,LDEB,-1
        ZAM=ZAM + PENTG(JL) - PDETG(JL)
 3    CONTINUE
C
      PDETG(LCT)=PDETG(LCT) + ZAM
C
C     Diffusion coefficients on CTM vertical coordinate:
C
      DO 4 JL=LDEB,NIV-1
        PDKG(JL)=AMAX1(0.,ZPDK(JL)*2.0*(PPRESS(JL+1)-PPRESS(JL-1))
     +    /(GG*((PZ(JL-1)-PZ(JL+1))**2)))
 4    CONTINUE
      PDKG(NIV)=0.0
C
      DEALLOCATE (ZAMU)
	DEALLOCATE (ZPTC)
	DEALLOCATE (ZPQC)
	DEALLOCATE (ZPLC)
      DEALLOCATE (ZPDK)
      DEALLOCATE (ZPAM)
	DEALLOCATE (ZPEU)
	DEALLOCATE (ZPDU)
	DEALLOCATE (ZPED)
	DEALLOCATE (ZPDD)
C
C
      RETURN
      END
