          SUBROUTINE WETBULB(LON,LAT,NIV,
     +                       XPAM,QGRIM,TGRIM,
     +                       LNEIGE,
     +                       QWGRIM)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type       : Physical parameterizations
C   Purpose    : Compute wet bulb specific humidity
C   Called by  :
C
CCC
C
C  Original : B. Josse & V.-H. Peuch, 02/2003
C
C  Modified :
C
CCC
C       
	USE CSTES_MOD
C
	IMPLICIT NONE	
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
	INTEGER :: LON,LAT,NIV
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: XPAM,QGRIM,TGRIM 
      LOGICAL :: LNEIGE
C     (wet bulb specific humidity)    
      REAL (KIND=8) , DIMENSION (LON,LAT,NIV) :: QWGRIM
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: L,K,I
	REAL (KIND=8) :: RR,RMD,RMV,RCPV,RCW,RCS,RLVTT,RLSTT,RGAMW
	REAL (KIND=8) :: RETV,RESTT,RBETW,RALPW,RGAMS,RBETS,RALPS,RGAMD
	REAL (KIND=8) :: RBETD,RALPD,ZCPVMW,ZCPVMS,RAP
        INTEGER :: NBITER,JITER
	REAL (KIND=8) :: ZEW,ZESP,ZQW,ZODLEW,ZDQW,ZDCP,ZDELQ,ZDELT
	REAL (KIND=8) , ALLOCATABLE :: PLH(:,:)
C     (latent heat at T)   
      REAL (KIND=8) , ALLOCATABLE :: PTW(:,:)
C     (wet bulb temperature)
      REAL (KIND=8) , ALLOCATABLE :: ZDELTA(:,:),ZCP(:,:)
!     REAL (KIND=8) , ALLOCATABLE :: ZLH(:,:),ZQWI(:,:)
      REAL (KIND=8) , ALLOCATABLE :: ZLH(:,:)
C
C
C 1. INITIALIZATIONS
C
      ALLOCATE (PLH(LON,LAT))
      ALLOCATE (PTW(LON,LAT))
      ALLOCATE (ZDELTA(LON,LAT))
      ALLOCATE (ZCP(LON,LAT))
      ALLOCATE (ZLH(LON,LAT))
!     ALLOCATE (ZQWI(LON,LAT))
C
      RR=8.31
      RMD=28.9644
      RMV=18.0153
      RTT=273.16
      RV=1000.*RR/RMV
      RD=1000.*RR/RMD
      CPD=3.5*RA
      CPV=4.*RV      
      RCPV=CPV
      RCW =4218.
      RCS=2106.
      RLVTT=2.5008E+6
      RLSTT=2.8345E+6
      RETV=RV/RD-1.
      RESTT=611.14
      RGAMW=(RCW-RCPV)/RV
      RBETW=RLVTT/RV+RGAMW*RTT
      RALPW=LOG(RESTT)+RBETW/RTT+RGAMW*LOG(RTT)
      RGAMS=(RCS-RCPV)/RV
      RBETS=RLSTT/RV+RGAMS*RTT
      RALPS=LOG(RESTT)+RBETS/RTT+RGAMS*LOG(RTT)
      RGAMD=RGAMS-RGAMW
      RBETD=RBETS-RBETW
      RALPD=RALPS-RALPW
      ZCPVMW=RCPV-RCW
      ZCPVMS=RCPV-RCS       
C
C 2. LOOP ON VERTICAL LEVELS
C           
      DO L=1,NIV      
C
C 2.1 Computation of PLH (FOLH function from ARPEGE)
C                   
       IF (LNEIGE) THEN      
        DO K=1,LAT
        DO I=1,LON  
	 ZDELTA(I,K)=MAX(0.,SIGN(1.,RTT-TGRIM(I,K,L)))
        ENDDO
        ENDDO
       ELSE
        DO K=1,LAT
	DO I=1,LON
	 ZDELTA(I,K)=0.
	ENDDO
	ENDDO
       ENDIF
C       
       DO K=1,LAT
       DO I=1,LON
        PLH(I,K)=RV * ( 
     +         ( RBETW+ZDELTA(I,K)*RBETD )
     +       - ( RGAMW+ZDELTA(I,K)*RGAMD ) * TGRIM(I,K,L) )
       ENDDO
       ENDDO       
C
C 2.2 Initialization of the variables in the Newton loop
C    
       DO K=1,LAT
       DO I=1,LON
        QWGRIM(I,K,L)=QGRIM(I,K,L)
	rap=QGRIM(I,K,L)/(1.-QGRIM(I,K,L))

	ZCP(I,K)=(CPD+rap*CPV)/(1.+rap)
	PTW(I,K)=TGRIM(I,K,L)
	ZLH(I,K)=PLH(I,K)

       ENDDO
       ENDDO
C
C 2.3 Newton loop
C       
       NBITER=2
C       
       DO JITER=1,NBITER
C       
        IF (LNEIGE) THEN      
         DO K=1,LAT
         DO I=1,LON  
	  ZDELTA(I,K)=MAX(0.,SIGN(1.,RTT-PTW(I,K)))
         ENDDO
         ENDDO
        ELSE
         DO K=1,LAT
	 DO I=1,LON
	  ZDELTA(I,K)=0.
	 ENDDO
	 ENDDO
        ENDIF     
C
C 2.4 General loop on horizontal indexes
C	
	DO K=1,LAT
	DO I=1,LON
C
C ARPEGE FOEW function
C	
	 ZEW = EXP (
     +       ( RALPW + ZDELTA(I,K)*RALPD )
     +     - ( RBETW + ZDELTA(I,K)*RBETD ) /      PTW(I,K)
     +     - ( RGAMW + ZDELTA(I,K)*RGAMD ) * LOG (PTW(I,K)) )
         ZESP = ZEW/XPAM(I,K,L)
C
C ARPEGE FOQS function
C	 
	 ZQW = ZESP /
     +       (1. + RETV *MAX(0., (1.-ZESP) ) )
C
C ARPEGE FODLEW function
C	   
         ZODLEW = (
     +            ( RBETW + ZDELTA(I,K)*RBETD ) 
     +          - ( RGAMW + ZDELTA(I,K)*RGAMD ) * PTW(I,K) )
     +          / ( PTW(I,K) * PTW(I,K) )
C
C ARPEGE FODQS function
C      
         ZDQW = ( ZQW *  (1. - ZQW) * ZODLEW ) 
     +        / ( 1. -ZESP )
C
C Incrementations
C
         ZDCP = ZCPVMW + ZDELTA (I,K) * (ZCPVMS - ZCPVMW)
C	 
	 ZDELQ = (ZQW - QWGRIM (I,K,L ) )
     +         *  ZCP (I,K) 
     +         / (ZCP (I,K) + ZLH (I,K) * ZDQW )
C     
         ZCP (I,K) = ZCP (I,K) + ZDCP * ZDELQ
C	 
	 ZDELT = -ZDELQ * ZLH(I,K) / ZCP(I,K)
C	 
	 ZLH (I,K)= ZLH (I,K) + ZDCP * ZDELT
C	 
	 QWGRIM (I,K,L) = QWGRIM (I,K,L) + ZDELQ
	 PTW    (I,K  ) = PTW    (I,K  ) + ZDELT
C
C End of loops on horizontal indexes
C	 
        ENDDO
	ENDDO
C
C END of Newton loop
C	 
       ENDDO
	 
C
C END OF LOOP ON VERTICAL LEVELS
C      
      ENDDO
C
      DEALLOCATE (PLH)
      DEALLOCATE (PTW)
      DEALLOCATE (ZDELTA)
      DEALLOCATE (ZCP)
      DEALLOCATE (ZLH)
!     DEALLOCATE (ZQWI)
C
C
      RETURN 
      END
