      SUBROUTINE ZENITH( DLON,DLAT,XSDAY,XDAY,sza,
     +                   nlon,nlat)
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : Chemistry 
C   Purpose   : Compute solar zenithal angle
C   Called by : 
C
CCC
C
C  Original : F. Lefevre (REPROBUS, LIB37)
C
C  Modified : V.-H.Peuch (inclusion dans MOCAGE), 08/1999
C
CCC
C
      USE CSTES_MOD
C	
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
      INTEGER :: NLON,NLAT
      REAL (KIND=8) :: XSDAY,XDAY
      REAL (KIND=8) , DIMENSION (nlat) :: DLAT
      REAL (KIND=8) , DIMENSION (nlon) :: DLON      
      REAL (KIND=8) , DIMENSION (nlon,nlat) :: sza
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C
      INTEGER :: ILAT,ILON
      REAL (KIND=8) :: ut
      REAL (KIND=8) :: lbut,lzut
      REAL (KIND=8) :: tz, rdecl, eqr, eqh, zpt
      REAL (KIND=8) :: csz, zr
      REAL (KIND=8) :: sintz, costz, sin2tz, cos2tz, sin3tz, cos3tz
C
      ut = XSDAY/3600.
C
C Equation 3.8 for "day-angle"
C
      tz = XPI2*XDAY
C
C Calculate sine and cosine from addition theoremes for
C better performance;  the computation of sin2tz,
C sin3tz, cos2tz and cos3tz is about 5-6 times faster
C than the evaluation of the intrinsic functions
C
C It is SIN(x+y) = SIN(x)*COS(y)+COS(x)*SIN(y)
C and   COS(x+y) = COS(x)*COS(y)-SIN(x)*SIN(y)
C
C sintz  = SIN(tz)      costz  = COS(tz)
C sin2tz = SIN(2.*tz)   cos2tz = SIN(2.*tz)
C sin3tz = SIN(3.*tz)   cos3tz = COS(3.*tz)
C
      sintz = SIN(tz)
      costz = COS(tz)
      sin2tz = 2.*sintz*costz
      cos2tz = costz*costz-sintz*sintz
      sin3tz = sintz*cos2tz + costz*sin2tz
      cos3tz = costz*cos2tz - sintz*sin2tz
C
C Equation 3.7 for declination in radians
C
      rdecl = 0.006918 - 0.399912*costz  + 0.070257*sintz
     +                 - 0.006758*cos2tz + 0.000907*sin2tz
     +                 - 0.002697*cos3tz + 0.001480*sin3tz
C
C Equation 3.11 for Equation of time  in radians
C
      eqr   = 0.000075 + 0.001868*costz  - 0.032077*sintz
     +                 - 0.014615*cos2tz - 0.040849*sin2tz
C
C convert equation of time to hours:
C
      eqh = eqr*24./XPI2
C
      DO ilat = 1,nlat
      DO ilon = 1,nlon
C
C       calculate local hour angle (hours):
C
C       lbut = 12. - eqh - real(ilon-1)*(360./real(nlon))*24./360.
C       (change for limited area domains)
C
C       lbut = 12. - DLONMIN*24./XPI2 - eqh
C    +  - real(ilon-1)*((DLONMIN-DLONMAX)/real(nlon-1))*24./XPI2
C
        lbut = 12. - eqh - DLON(ilon)*24./XPI2
C
C       convert to angle from UT
C
        lzut = 15.*(ut - lbut)
        zpt = lzut/CONV
C
C       Equation 2.4 for cosine of zenith angle
C
        csz = SIN(DLAT(ilat))*SIN(rdecl)
     +      + COS(DLAT(ilat))*COS(rdecl)*COS(zpt)
        zr = ACOS(csz)
        sza(ilon,ilat) = zr*CONV
      ENDDO
      ENDDO
C
      RETURN
      END
C
CCC
C
