module cstes_analytic
  real(kind=8), parameter :: r = 8.205e-5
  real(kind=8), parameter :: sigma = 1.8
  real(kind=8), dimension(3) :: a1 = (/  12.37208932   , 11.820654354   , -180.06541028    /)
  real(kind=8), dimension(3) :: b1 = (/  -0.16125516114, -0.20786404244 ,   -0.38601102592 /)
  real(kind=8), dimension(3) :: c1 = (/ -30.490657554  , -4.807306373   ,  -93.317846778   /)
  real(kind=8), dimension(3) :: d1 = (/  -2.1133114241 , -5.1727540348  ,  273.88132245    /)
  real(kind=8), dimension(3) :: a2 = (/  13.455394705  , 12.891938068   , -176.95814097    /)
  real(kind=8), dimension(3) :: b2 = (/  -0.1921312255 , -0.232338847708,   -0.36257048154 /)
  real(kind=8), dimension(3) :: c2 = (/ -34.285174607  , -6.4261237757  ,  -90.469744201   /)
  real(kind=8), dimension(3) :: d2 = (/  -1.7620073078 , -4.9005471319  ,  267.45509988    /)

  ! carslaw et al., 1995

  real(kind=8), dimension(10) :: qn = (/ 14.5734   , 0.0615994, -1.14895 , 0.691693 , -0.098863,  &
                                  0.0051579, 0.123472 , -0.115574, 0.0110113,  0.0097914 /)
  real(kind=8), dimension(10) :: qs = (/ 14.4700   , 0.0638795, -3.29597 , 1.778224 , -0.223244,  &
                                  0.0086486, 0.536695 , -0.335164, 0.0265153,  0.0157550 /)
  real(kind=8), dimension( 7) :: kn = (/ -39.136, 6358.4,83.29, -17650.0, 198.53 , -11948.0, -28.469  /)
  real(kind=8), dimension( 7) :: ks = (/ -21.661, 2724.2,51.81, -15732.0,  47.004,  -6969.0,  -4.6183 /)

  ! pruppacher and klett, 1988

  real(kind=8), dimension(0:6) :: bnre = (/ -3.18657    ,  0.992696   , -0.153193e-2, &
                                    -0.987059e-3, -0.578878e-3,  0.855176e-4, -0.327815e-5 /)
  character(len=15), dimension(10) :: NAMSPEC = (/ 'H_2O           ','HNO_3          ', &
                                                   'H_2SO_4        ','HCl            ', &
                                                   'HBr            ','HOCl           ', &
                                                   'HOBr           ','ClONO_2        ', &
                                                   'N_2O_5         ','PSC            ' /)
end module cstes_analytic

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

subroutine analytic(solid,temp,ptot,hnm,ck,vsed,  &
                    het,                          &
                    parthno3,parth2o,             &
                    parthcl, parthbr,             &
                    parthno3sl,parth2os,          &
                    lola,nbcon,nhet,tranam)

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      USE CSTES_MOD
      USE cstes_analytic

      implicit none

!     Subroutine arguments.
!     ~~~~~~~~~~~~~~~~~~~~
!
  integer :: lola,nbcon,nhet
  real (kind=8) , dimension (lola) :: parthno3,parth2o,parthcl
  real (kind=8) , dimension (lola) :: parthbr,parthno3sl,parth2os
  real (kind=8) , dimension (lola) :: vsed
  real (kind=8) , dimension (lola) :: temp,ptot,hnm
  real (kind=8) , dimension (lola,nbcon) :: ck
  real (kind=8) , dimension (lola,nhet)  :: het 
  character(len=15) , dimension(nbcon) :: tranam(nbcon)
  logical :: solid
      
!
!     Variables locales.
!     ~~~~~~~~~~~~~~~~~
!  
  integer :: i,nok,itra,ih2o,ihno3,ih2so4,ihcl,ihbr,ihocl,ihobr
  integer :: iclono2,in2o5,ipsc,ilon
  real (kind=8) :: a, b, det, msb,mnb,mnf,msf
  real (kind=8) :: pi,amu,qh2o,qhno3,t,pn0t,zmt,zbt,zhno3eq,pn
  real (kind=8) :: hno3s,zh2oeq,ph2o,h2os,qh2so4,c,xsb,wts,tf
  real (kind=8) :: tt,tr,pr,hsb,xnb,hnb,phi,wsf,wnf,vliq,rmode
  real (kind=8) :: ph2o0,qhcl,phcl1,sclono2,shocl,rpart,rhopart
  real (kind=8) :: ynre,shapek,vsedsol
  real (kind=8) :: ndrop
  real (kind=8) :: mh2so4c, bh2so4, ph2so4, muh2so4
!
  real (kind=8) :: k1(lola), k2(lola), k3(lola), &
                 k4(lola), k5(lola), k6(lola), &
                 k7(lola), k8(lola), k9(lola)  
!      
!
  real (kind=8) , allocatable :: parthno3l(:),parthno3s(:)
  real (kind=8) , allocatable :: dens(:),ns(:)
  real (kind=8) , allocatable :: pn0(:),phcl0(:)
  real (kind=8) , allocatable :: pw(:),ms(:),mn(:),ws(:),wn(:)
  real (kind=8) , allocatable :: hhcl(:),hhocl(:),hhobr(:)
  real (kind=8) , allocatable :: hclono2(:),hhbr(:),tice(:)
  integer , allocatable :: lnat(:),lice(:),condliq(:)
!
!
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     ajouts jpl 2000
!ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
  real (kind=8) , allocatable :: aw(:)
  real (kind=8) , allocatable :: wt(:), mh2so4(:)
  real (kind=8) :: m, y1, y2, bigx
!
!
!
  real (kind=8) , allocatable :: rmean(:)
  real (kind=8) , allocatable :: aliq(:)
!
  real (kind=8) :: nnat_small_max, vnat_small_max
  real (kind=8) , allocatable :: nnats(:)
  real (kind=8) , allocatable :: nnatl(:)
  real (kind=8) :: rnats, rnatl
  real (kind=8) , allocatable :: anat(:) 
  real (kind=8) :: vnat, vnatl
!
  real (kind=8) , allocatable :: nice(:)
  real (kind=8) :: rice
  real (kind=8) , allocatable :: aice(:)
  real (kind=8) :: vice
!
!     sedimentation
!
  
  real (kind=8) :: rhoair, rhoice, rhonat
  real (kind=8) :: netha, lambda, lambda0, nre, us, cdnre2
  real (kind=8) :: tcel
!
  logical :: pscliq, pscnat, pscice
!
!
  integer :: index
  real (kind=8) :: cnst1,cnst2,cnst3,cnst4
  real (kind=8) :: wtsm40,a_2,a_3,t2,t3,wt2
  real (kind=8) :: x__,x__1,x__2,x__3,x__4,x__5,x__6
  real, dimension(lola) :: t_, msb_, mnf_, msf_, hnb_ , hsb_
      
  allocate (parthno3l(lola))
  allocate (parthno3s(lola))
  allocate (dens(lola))
  allocate (ns(lola))
  allocate (pn0(lola))
  allocate (phcl0(lola))
  allocate (pw(lola))
  allocate (ms(lola))
  allocate (mn(lola))
  allocate (ws(lola))
  allocate (wn(lola))
  allocate (hhcl(lola))
  allocate (hhocl(lola))
  allocate (hhobr(lola))
  allocate (hclono2(lola))
  allocate (hhbr(lola))
  allocate (tice(lola))
  allocate (lnat(lola))
  allocate (lice(lola))
  allocate (condliq(lola))
  allocate (aw(lola))
  allocate (wt(lola))
  allocate (mh2so4(lola))
  allocate (rmean(lola))
  allocate (aliq(lola))
  allocate (nnats(lola))
  allocate (nnatl(lola))
  allocate (anat(lola))
  allocate (nice(lola))
  allocate (aice(lola))
	


! DETERMINE USEFUL INDEXES

NOK=0

#ifdef FTRACE
call ftrace_region_begin ("loop01")
#endif
DO itra = 1,NBCON
   ! H_20
   IF (TRANAM(itra) == NAMSPEC(1)) THEN
     ih2o=itra
     NOK=NOK+1
   ENDIF
   ! HNO_3
   IF (TRANAM(itra) == NAMSPEC(2)) THEN
     ihno3=itra
     NOK=NOK+1
   ENDIF
   ! H_2SO_4
   IF (TRANAM(itra) == NAMSPEC(3)) THEN
     ih2so4=itra
     NOK=NOK+1
   ENDIF
   ! HCl
   IF (TRANAM(itra) == NAMSPEC(4)) THEN
     ihcl=itra
     NOK=NOK+1
   ENDIF
   ! HBr
   IF (TRANAM(itra) == NAMSPEC(5)) THEN
     ihbr=itra
     NOK=NOK+1
   ENDIF
   ! HOCl
   IF (TRANAM(itra) == NAMSPEC(6)) THEN
     ihocl=itra
     NOK=NOK+1
   ENDIF
   ! HOBr
   IF (TRANAM(itra) == NAMSPEC(7)) THEN
     ihobr=itra
     NOK=NOK+1
   ENDIF
   ! ClONO_2
   IF (TRANAM(itra) == NAMSPEC(8)) THEN
     iclono2=itra
     NOK=NOK+1
   ENDIF
   ! N_2O_5
   IF (TRANAM(itra) == NAMSPEC(9)) THEN
     in2o5=itra
     NOK=NOK+1
   ENDIF
   ! PSC
   IF (TRANAM(itra) == NAMSPEC(10)) THEN
     ipsc=itra
     NOK=NOK+1
   ENDIF
ENDDO
#ifdef FTRACE
call ftrace_region_end ("loop01")
#endif

IF (NOK /= 10) THEN
   WRITE(*,*) '>> CONSTITUENT(S) MISSING FOR PSC CHEMISTRY : ABORT!'
   STOP
ENDIF

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     choix du schema de psc:
!   
!     pscliq = true : autoriser les aerosols liquides
!     pscnat = true : autoriser le nat
!     pscice = true : autoriser la glace
!
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

pscliq = .true.
pscnat = .true.
pscice = .true.

ndrop = 10.                 ! densite d'aerosols liquides (cm-3)

nnat_small_max  = 1.        ! densite maximale de nat (cm-3)
rnats           = 0.5e-4    ! rayon du small mode de nat (cm)
rnatl           = 6.5e-4    ! rayon du large mode de nat (cm)

rice = 10.e-4               ! rayon de la glace

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     initialisations
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

pi = 2.*asin(1.0)
amu  = 1.66e-24
vnat = 0.
vnatl= 0.
vice = 0.

#ifdef FTRACE
call ftrace_region_begin ("loop02")
#endif
do ilon = 1,lola
   rmean(ilon)      = 0.
   aliq(ilon)       = 0.
   lnat(ilon)       = 0
   nnats(ilon)      = 0.
   nnatl(ilon)      = 0.
   anat(ilon)       = 0.
   lice(ilon)       = 0
   nice(ilon)       = 0.
   aice(ilon)       = 0.
   vsed(ilon)       = 0.
   condliq(ilon)    = 0
   parthno3(ilon)   = 1.
   parthno3l(ilon)  = 1.
   parthno3s(ilon)  = 1.
   parthno3sl(ilon) = 1.
   parth2o(ilon)    = 1.
   parth2os(ilon)   = 1.         
   parthcl(ilon)    = 1.
end do
#ifdef FTRACE
call ftrace_region_end ("loop02")
#endif

#ifdef FTRACE
call ftrace_region_begin ("loop03")
#endif
do ilon = 1,lola
   qh2o = ck(ilon,ih2o)/hnm(ilon)
   pw(ilon) = qh2o*ptot(ilon)/PREF
   pw(ilon) = min(pw(ilon),2.e-1/PREF)
   pw(ilon) = max(pw(ilon),2.e-3/PREF)

   qhno3 = ck(ilon,ihno3)/hnm(ilon)
   pn0(ilon)= qhno3*ptot(ilon)/PREF
end do
#ifdef FTRACE
call ftrace_region_end ("loop03")
#endif

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     traitement des aerosols solides
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

!c    traitement du nat

if (pscnat) then
#ifdef FTRACE
call ftrace_region_begin ("loop04")
#endif

  cnst1 = 4./3.*XPI*rnats**3.
  cnst2 = 4./3.*XPI*rnatl**3.
  cnst3 = 4*XPI*rnats**2
  cnst4 = 4*XPI*rnatl**2
!$omp parallel do default(none) &
!$omp private(hno3s,ilon,pn,pn0t,t,vnat,vnat_small_max,vnatl,zbt,zhno3eq,zmt) &
!$omp shared(amu,anat,hnm,lnat,lola,nnatl,nnats,parthno3s,parthno3sl,pn0,pref,ptot,pw) &
!$omp shared(cnst1,cnst2,cnst3,cnst4,nnat_small_max,temp,xpi)
  do ilon = 1,lola
     t = min(temp(ilon),273.)
     pn0t = pn0(ilon)*PREF/100.*0.75
     zmt = -2.7836 - 0.00088*t
     zbt = 38.9855 - 11397.0/t + 0.009179*t
     zhno3eq = 10.0**(zmt*log10(pw(ilon)*PREF/100.*0.75) + zbt)
     if (pn0t/zhno3eq >= 1.) then
        lnat(ilon) = 1
        pn = zhno3eq/(0.75*PREF/100.)
        hno3s = (pn0(ilon) - pn)*PREF*hnm(ilon)/ptot(ilon)
        hno3s = max(hno3s,0.)

        !              volume de nat condense

        vnat = hno3s*amu*117./1.35

        !              volume maximal du small mode

        vnat_small_max = nnat_small_max * cnst1

        !              densite de nat (cm-3) pour les deux modes

        if (vnat > vnat_small_max) then
           nnats(ilon) = nnat_small_max
           nnatl(ilon) = (vnat - vnat_small_max) / cnst2
           vnatl = vnat - vnat_small_max
        else
           nnats(ilon) = vnat / cnst1
           nnatl(ilon) = 0.
           vnatl = 0.
        end if

        !              surface du nat

        anat(ilon) = nnats(ilon)*cnst3 + nnatl(ilon)*cnst4

        !              parthno3s: part en phase gazeuse

        parthno3s(ilon) = 1.0 - (pn0(ilon)-pn)/pn0(ilon)
        parthno3s(ilon) = max(parthno3s(ilon),0.0)
        parthno3s(ilon) = min(parthno3s(ilon),1.)

        !              parthno3sl: part non sedimentee 
        !                          (gaz + petit mode de nat)

        parthno3sl(ilon) = 1.0 - (pn0(ilon)-pn)*vnatl/vnat/pn0(ilon)
        parthno3sl(ilon) = max(parthno3sl(ilon),0.0)
        parthno3sl(ilon) = min(parthno3sl(ilon),1.)
     end if
  end do
!$omp end parallel do

#ifdef FTRACE
call ftrace_region_end ("loop04")
#endif
end if

!    traitement de la glace

if (pscice) then
#ifdef FTRACE
call ftrace_region_begin ("loop05")
#endif
   do ilon = 1,lola
      t = min(temp(ilon),273.)
      tice(ilon) = 2668.70/(10.4310-(log(pw(ilon))+log(760.0))/log(10.0))
      if (t <= tice(ilon)) then
         lice(ilon) = 1
         lnat(ilon) = 0
         anat(ilon) = 0.
         zh2oeq = 10.0**(-2668.7/t + 10.4310)
         ph2o = zh2oeq/760.0
         h2os = max(pw(ilon) - ph2o,0.)*PREF*hnm(ilon)/ptot(ilon)

!              volume de glace condensee

         vice = h2os*amu*18./0.928

!              densite de glace (cm-3)

         nice(ilon) = vice/(4./3.*XPI*rice**3.)

!              surface de la glace

         aice(ilon) = nice(ilon)*4*XPI*rice**2

!              parth2o: part en phase gazeuse

         parth2o(ilon) = 1.0 - max(pw(ilon)-ph2o,0.)/pw(ilon)
         parth2o(ilon) = max(parth2o(ilon),0.)
         parth2o(ilon) = min(parth2o(ilon),1.)
         parth2os(ilon) = parth2o(ilon) 

!              on retire hno3 de la phase gazeuse sous forme de nat

         zmt = -2.7836 - 0.00088*t
         zbt = 38.9855 - 11397.0/t + 0.009179*t
         zhno3eq = 10.0**(zmt*log10(pw(ilon)*parth2o(ilon)*PREF/100.*0.75)+ zbt)
         pn = zhno3eq/(0.75*PREF/100.)

!              parthno3s: part en phase gazeuse

         parthno3s(ilon) = 1.0 - (pn0(ilon)-pn)/pn0(ilon)
         parthno3s(ilon) = max(parthno3s(ilon),0.0)
         parthno3s(ilon) = min(parthno3s(ilon),1.)
         parthno3sl(ilon) = parthno3s(ilon)
      end if
   end do
#ifdef FTRACE
call ftrace_region_end ("loop05")
#endif
end if

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     traitement des aerosols liquides
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

if (pscliq) then

#ifdef FTRACE
call ftrace_region_begin ("loop06")
#endif

!$omp parallel &
!$omp default(none) &
!$omp private(a,a_2,a_3,b,c,det,hnb,hsb,ilon,mnb,mnf,msb,msf,muh2so4,ph2so4,phi,pn,pr) &
!$omp private(qh2o,qh2so4,qhno3,t,tf,tr,tt,wnf,wsf,wts,wtsm40,xnb,xsb) &
!$omp shared(ck,condliq,hnb_,hnm,hsb_,ih2o,ih2so4,ihno3,kn,ks,lola) &
!$omp shared(mn,mnf_,ms,msb_,msf_,ns,parth2o,parthno3l,parthno3s,pn0) &
!$omp shared(pref,ptot,pw,qn,qs,t_,temp,tice,wn,ws,xpi)

!$omp do
   do ilon = 1,lola
      condliq(ilon) = 1
      t = max(temp(ilon),max(tice(ilon)-3.0,185.))
      t_(ilon) = t
      qh2o = ck(ilon,ih2o)*parth2o(ilon)/hnm(ilon)
      pw(ilon) = qh2o*ptot(ilon)/PREF
      pw(ilon) = min(pw(ilon),2.e-1/PREF)
      pw(ilon) = max(pw(ilon),2.e-3/PREF)

      qh2so4 = max(ck(ilon,ih2so4)/hnm(ilon),1.e-18)
      ns(ilon) = ptot(ilon)*qh2so4/8.314/t

      qhno3 = ck(ilon,ihno3)*parthno3s(ilon)/hnm(ilon)
      pn0(ilon)= ptot(ilon)/PREF*qhno3
      IF ( pn0(ilon) == 0.) THEN
         condliq(ilon)=0
      ENDIF

      !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      !cc         aerosols binaires h2so4/h2o 
      !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      a = ks(3) + ks(4)/t
      b = ks(1) + ks(2)/t
      c = ks(5) + ks(6)/t + ks(7)*log(t) - log(pw(ilon))

      det = b*b - 4.*a*c

      if (det > 0.) then
         xsb = (-b - sqrt(det))/(2.*a)
         msb = 55.51*xsb/(1.0 - xsb)
      else
         msb = 0.
         condliq(ilon) = 0
      end if
      msb_(ilon) = msb

      ms(ilon) = msb
      ws(ilon) = msb*0.098076/(1.0 + msb*0.098076)

      mn(ilon) = 0.0
      wn(ilon) = 0.0

      !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      !           pression saturante de h2so4
      !          
      !           d'apres ayers et al., grl, 1980
      !           et giauque et al., j. am. chem. soc., 1960
      !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      wts = max(41., ws(ilon)*100.)
      wtsm40 = wts - 40.
      muh2so4 = 4.184*(1.514e4 - 286.*(wtsm40)   &
              + 1.08*(wtsm40)*(wtsm40)           &
              - 3941./(wtsm40)**0.1)

      !        pression saturante en atmosphere

      ph2so4 = exp(-10156./t + 16.2590 - muh2so4/(8.314*t)) 

      !ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      !
      ! Tests sur les variables pour eviter la division par zero
      !
      !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      if (    qh2so4*ptot(ilon)/PREF <= ph2so4     .or.    &
              (kn(3)+kn(4)/t) == 0.  ) then
         ms(ilon) = 0.
         ws(ilon) = 0.
         condliq(ilon) = 0
      endif
   end do
!$omp end do
           
   !-----------------------------------------------------------------

!$omp do
   do ilon = 1,lola
      t = t_(ilon)
      !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      !cc         aerosols ternaires hno3/h2so4/h2o
      !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      if (condliq(ilon) == 1 .and. t < 215.) then

         tf = t
         tt = r*tf*ns(ilon)
         tr = 1.0e4/tf-43.4782608
         pr = log(pw(ilon))+18.4

         hsb = qs(1)+qs(2)*tr**2+(qs(3)+qs(4)*tr+qs(5)*tr**2   &
              +qs(6)*tr**3)*pr + (qs(7)+qs(8)*tr+qs(9)*tr**2)  &
              *pr**2+qs(10)*tr*pr**3
         hsb = exp(hsb)
         xnb = 1.0/(2.0*(kn(3)+kn(4)/tf))*(-kn(1)-kn(2)/tf-    &
              ((kn(1)+kn(2)/tf)**2-4.0*(kn(3)+kn(4)/tf)        &
              *(kn(5)+kn(6)/tf +kn(7)*log(tf)                  &
              -log(pw(ilon))))**0.5)
         mnb = 55.51*xnb/(1.0-xnb)
         hnb = qn(1)+qn(2)*tr**2+(qn(3)+qn(4)*tr+qn(5)*tr**2   &
              +qn(6)*tr**3)*pr + (qn(7)+qn(8)*tr+qn(9)*tr**2)  &
              *pr**2+qn(10)*tr*pr**3
         hnb = exp(hnb)

         msb = msb_(ilon)
         a = (tt*hnb*mnb**2 - tt*hsb*mnb*msb - 2.0*mnb**2*msb  &
              + mnb*msb**2 + hnb*mnb*msb*pn0(ilon)             &
             - hsb*msb**2*pn0(ilon))/(mnb**2 - mnb*msb)
         b = msb*(-2.0*tt*hnb*mnb+tt*hsb*msb+mnb*msb           &
            -hnb*msb*pn0(ilon))/(mnb-msb)
         c = (tt*hnb*mnb*msb**2)/(mnb - msb)

         a_2 = a*a ; a_3 = a_2*a
         phi = atan(sqrt( max(4.0*(a_2-3.0*b)**3-(-2.0*a_3+9.0*a*b   &
                         -27.0*c)**2,0.))/(-2.0*a_3+9.0*a*b-27.0*c) )
         if (phi < 0.) then
            phi = phi + XPI
         endif

         msf = -1.0/3.0*(a+2.0*sqrt(a**2-3.0*b)*cos((XPI+phi)/3.0))
         msf = max(msf,0.)            

         mnf = mnb*(1.0-msf/msb)
         mnf = max(mnf,0.)

         if (msf == 0.   .or.   pn0(ilon) == 0.) then
            condliq(ilon)=0
            ms(ilon)=0.
            ws(ilon)=0.
         endif
         mnf_(ilon) = mnf
         msf_(ilon) = msf
         hnb_(ilon) = hnb
         hsb_(ilon) = hsb

      endif
   end do
!$omp end do

   !-----------------------------------------------------------------

!$omp do
   do ilon = 1,lola
      t = t_(ilon)
      if (condliq(ilon) == 1 .and. t < 215.) then
         mnf = mnf_(ilon)
         msf = msf_(ilon)
         hnb = hnb_(ilon)
         hsb = hsb_(ilon)

         pn = mnf/(hnb*mnf/(mnf+msf)+hsb*msf/(mnf+msf))
         wsf = msf*0.098076/(1.0+msf*0.098076+mnf*0.063012)
         wnf = mnf*0.063012/(1.0+msf*0.098076+mnf*0.063012)

         ms(ilon) = msf        
         mn(ilon) = mnf
         ws(ilon) = wsf
         wn(ilon) = wnf

         ! parthno3l: part en phase gazeuse

         parthno3l(ilon) = 1.0 - (pn0(ilon)-pn)/pn0(ilon)
         parthno3l(ilon) = min(parthno3l(ilon),1.)

      endif

      ! dilution minimale de h2so4: 0.5%

      ws(ilon) = max(ws(ilon), 0.005)
   end do
!$omp end do

!$omp end parallel

#ifdef FTRACE
call ftrace_region_end ("loop06")
#endif

   !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
   !cc   densite des aerosols ternaires (g/cm3)
   !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

   call density(ws,wn,temp,dens,lola)

   !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
   !cc      total specific liquid aerosol volume (dimensionless)
   !cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

#ifdef FTRACE
call ftrace_region_begin ("loop07")
#endif

    x__ = log(sigma) ; x__ = x__*x__

!$omp parallel do &
!$omp default(none) &
!$omp private(bh2so4,bigx,index,ilon,m,mh2so4c,ph2o0,phcl1,qhcl,rmode,sclono2,shocl) &
!$omp private(t,t2,t3,vliq,wt2,y1,y2) &
!$omp shared(PREF,XPI,a1,a2,aliq,aw,b1,b2,c1,c2,ck,condliq,d1,d2,dens,hclono2) &
!$omp shared(hhbr,hhcl,hhobr,hhocl,hnm,ihcl,lola,mh2so4,ms,ndrop,ns) &
!$omp shared(parthcl,phcl0,ptot,pw,rmean,temp,tice,ws,wt,x__)
   do ilon = 1,lola
     if (condliq(ilon) == 1) then
       vliq = ns(ilon)*98.076e-6/ws(ilon)/dens(ilon)
 
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!cc         liquid aerosol radius and surface area
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!           rmean=mean radius of liquid droplets (units = cm)
!           this varies with the total liquid volume per unit volume of air
!           (called vliq in main program).  rmean can be calculated
!           from vliq using
!
!           rmean=rmode*exp(0.5*(log(sigma))**2), where
!
!           rmode=(3.0*vliq/(4.*XPI*ndrop)*
!                 exp(-9.0/2.0*(log(sigma)**2)))**(1./3.)
!
!           ndrop=number of droplets per cm3 air
!           sigma=width of lognormal (use sigma=1.8).

       rmode=(3.0*vliq/(4.*XPI*ndrop)*exp(-4.5*x__))**(1./3.)
       rmean(ilon)=rmode*exp(0.5*x__)

       aliq(ilon)=4.0*XPI*rmode*rmode*ndrop*exp(2.0*x__)

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!cc         the solubility of hcl
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!           hhcl est calcule directement a partir de jpl 2000,
!           lui-meme utilisant carslaw et al., j. phys. chem., 1995.
!           l'ensemble des calculs suppose un aerosol binaire:
!           la molalite de h2so4 (wt) est donc recalculee dans
!           ces conditions.

       t = max(temp(ilon),max(tice(ilon)-3.0,185.)) ; t2 = t*t ; t3 = t2*t

       !  ph2o0 : saturation water vapor pressure, hpa

       ph2o0 = exp(18.452406985 - 3505.1578807/t - 330918.55082/t2 + 12725068.262/t3)

       !  aw : water activity
       aw(ilon) = pw(ilon)*PREF/100./ph2o0
       aw(ilon) = min(aw(ilon), 1.)

       if (aw(ilon) <= 0.05) then
          index = 1
       else if (aw(ilon) < 0.85) then
          index = 2
       else
          index = 3
       endif
       y1 = a1(index)*aw(ilon)**b1(index) + c1(index)*aw(ilon) + d1(index)
       y2 = a2(index)*aw(ilon)**b2(index) + c2(index)*aw(ilon) + d2(index)

       !  m: h2so4 molality, mol/kg

       m = y1 + (t - 190.)*(y2 - y1)/70.

       !  wt: h2so4 weight percentage, %

       wt(ilon) = 9800.*m/(98.*m + 1000.)
       wt(ilon) = max(wt(ilon), 0.5)

       !  mh2so4: h2so4 molarity, mol l-1

       mh2so4(ilon) = dens(ilon)*wt(ilon)/9.8

       bigx = wt(ilon)/(wt(ilon) + (100. - wt(ilon))*98./18.)

       hhcl(ilon) = (0.094 - 0.61*bigx + 1.2*bigx*bigx) * exp(-8.68 + (8515. - 10718.*(bigx**0.7))/t)

       !  phcl1 : pression de hcl restant en phase gazeuse
       !  ns/ms : kilogrammes (litres) d'eau par metre cube 

       qhcl = ck(ilon,ihcl)/hnm(ilon)
       IF (qhcl == 0.) THEN
          condliq(ilon)=0
       else
           
         phcl0(ilon) = qhcl*ptot(ilon)/PREF

         phcl1 = (phcl0(ilon)/(r*t)) / (1./(r*t) + hhcl(ilon)*ns(ilon)/ms(ilon)) 

         parthcl(ilon) = 1.0-(phcl0(ilon)-phcl1)/phcl0(ilon)

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!cc         the solubility of clono2
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!           on peut negliger la diminution de la concentration
!           en phase gazeuse.

         sclono2 = 0.306 + 24./t     ! sclono2 : setchenow coefficient m-1

         hclono2(ilon) = 1.6e-6*exp(4710./t) * exp(-sclono2*mh2so4(ilon)) ! hclono2 : shi et al., m atm-1

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!cc         the solubility of hocl
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!           hhocl d'apres jpl 2000 (shi et al.)
!           on peut negliger la diminution de la concentration
!           en phase gazeuse.

         shocl = 0.0776 + 59.18/t       ! shocl : setchenow coefficient m-1

         hhocl(ilon) = 1.91e-6*exp(5862.4/t) * exp(-shocl*mh2so4(ilon)) ! hhocl : shi et al.

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!cc         the solubility of hobr
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!           hhobr d'apres waschewsky and abbatt, j. phys. chem. a,
!           5312-5320, 1999.
!           on peut negliger la diminution de la concentration
!           en phase gazeuse.

         hhobr(ilon) = 4.6e-4*exp(4.5e3/t) 

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!cc         the solubility of hbr
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!
!           hhbr d'apres kleffmann et al., j. phys. chem. a,
!           8489-8495, 2000. attention a l'erreur typographique
!           contenue dans la version imprimee de l'article
!           (m3 = - 4.445 imprime au lieu de m3 = + 4.445)

         wt2 = wt(ilon)*wt(ilon)
         mh2so4c = -1.977e-4*wt2 -2.096e-2*wt(ilon) + 4.445
         bh2so4  = -8.979e-5*wt2 +2.141e-2*wt(ilon) - 6.067

         hhbr(ilon) = 10.**(mh2so4c*1000./t + bh2so4)
       end if
     end if
   end do
!$omp end parallel do

#ifdef FTRACE
call ftrace_region_end ("loop07")
#endif

end if

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     surface totale des psc
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

    
!         do ilon = 1,lola
!            surfarea(ilon) = aliq(ilon)
!     $                       + anat(ilon)*lnat(ilon)      
!     $                       + aice(ilon)*lice(ilon)      
!         end do

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     partition totale de hno3
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

#ifdef FTRACE
call ftrace_region_begin ("loop08")
#endif
do ilon = 1,lola
   parthno3(ilon) = parthno3l(ilon)*parthno3s(ilon)
   parthno3(ilon) = max(parthno3(ilon), 0.)
   parthno3(ilon) = min(parthno3(ilon), 1.)
end do
#ifdef FTRACE
call ftrace_region_end ("loop08")
#endif

!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
!     sedimentation
!cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

rhoice   = 0.928*1000.0
rhonat   = 1.350*1000.0

#ifdef FTRACE
call ftrace_region_begin ("loop09")
#endif
do ilon = 1,lola
   if (lice(ilon) == 1 .or. lnat(ilon) == 1) then

      t = temp(ilon)

      rpart = (lice(ilon)*rice + lnat(ilon)*rnatl)*1.e-2
      rpart = min(rpart,535.e-6)

      if (rpart >= 0.5e-6) then

         rhopart = lice(ilon)*rhoice + lnat(ilon)*rhonat
         rhoair=(hnm(ilon)*28.97/6.022e23)*1000.0

         tcel = t - 273.15
         netha = (1.718 + 0.0049*tcel-1.2e-5*tcel*tcel)*1.e-5

         cdnre2 = 32.0*(rpart)**3*(rhopart-rhoair)*rhoair*9.80665/(3.0*(netha)**2)

         !     pruppacher and klett 1998

         x__1 = log(cdnre2)
         x__2 = x__1 * x__1
         x__3 = x__1 * x__2
         x__4 = x__1 * x__3
         x__5 = x__1 * x__4
         x__6 = x__1 * x__5
         ynre = bnre(0)+bnre(1)*x__1  &
                       +bnre(2)*x__2  &
                       +bnre(3)*x__3  &
                       +bnre(4)*x__4  &
                       +bnre(5)*x__5  &
                       +bnre(6)*x__6            
         nre = exp(ynre)

         if (nre >= 1.e-2) then

            !     big particles
            !    (1.e-2 < reynolds number < 300)
            !    (10.0  < radius  < 535 microns)

            vsed(ilon) = netha*nre/(2*rhoair*rpart)

         else

            !     small particules
            !     (1.e-6 < reynolds number < 1.e-2)
            !     (0.5   < radius     < 10 microns)

            !     lambda0 : libre parcours moyen (m) a 1013.25 hpa
            !               et 293.15k. voir pruppacher and klett
            !               page 416.

            lambda0 = 6.6e-8
            lambda  = lambda0*(PREF/ptot(ilon))*(t/293.15)
 
            us = 2.0*rpart*rpart*9.80665*(rhopart-rhoair)/(9.0*netha)

            !     solid particles:

            !     niels larsen - 1991
            !     columnar or elipsoid shape
            !     with a shape factor of 1.2 (shapek)

            shapek = 1.2

            x__ = lambda/rpart
            vsedsol = us/shapek*(1.0+1.246*x__+0.42*x__*exp(-0.87/x__))

            vsed(ilon) = vsedsol
         end if
      end if
   end if
end do
#ifdef FTRACE
call ftrace_region_end ("loop09")
#endif

!     only the parameters that are needed in 'hetero' are 
!     included in the variable list.  output variables should
!     be included in the list as needed!

call hetero(aw,temp,ptot,hnm,ck,ws,       &
      hhcl,hhocl,hhobr,hclono2,hhbr,      &
      rice,nice,rnats,rnatl,nnats,nnatl,  &
      aliq,rmean,condliq,lnat,lice,       &
      k1,k2,k3,k4,k5,k6,k7,k8,k9,wt,      &
      ih2o,ihno3,ihcl,ihbr,ihocl,ihobr,   &
      iclono2,in2o5,lola,nbcon)

#ifdef FTRACE
call ftrace_region_begin ("loop10")
#endif
DO I=1,LOLA
   het(I,1)=k1(I)
   het(I,2)=k2(I)
   het(I,3)=k3(I)
   het(I,4)=k4(I)
   het(I,5)=k5(I)
   het(I,6)=k6(I)
   het(I,7)=k7(I)
   het(I,8)=k8(I)
   het(I,9)=k9(I)
ENDDO
#ifdef FTRACE
call ftrace_region_end ("loop10")
#endif
!      DO I=1,LOLA
!       DO J=1,9
!        het(I,J)=0.
!       ENDDO
!      ENDDO

!  diagnostic d'occurence des PSC et de leur type

#ifdef FTRACE
call ftrace_region_begin ("loop11")
#endif
DO i=1,LOLA
  ck(i,ipsc)=0.
  if (lnat(i) == 1) ck(i,ipsc)=1.0*hnm(i)
  if (lice(i) == 1) ck(i,ipsc)=2.0*hnm(i)
ENDDO
#ifdef FTRACE
call ftrace_region_end ("loop11")
#endif

  deallocate (parthno3l)
  deallocate (parthno3s)
  deallocate (dens)
  deallocate (ns)
  deallocate (pn0)
  deallocate (phcl0)
  deallocate (pw)
  deallocate (ms)
  deallocate (mn)
  deallocate (ws)
  deallocate (wn)
  deallocate (hhcl)
  deallocate (hhocl)
  deallocate (hhobr)
  deallocate (hclono2)
  deallocate (hhbr)
  deallocate (tice)
  deallocate (lnat)
  deallocate (lice)
  deallocate (condliq)
  deallocate (aw)
  deallocate (wt)
  deallocate (mh2so4)
  deallocate (rmean)
  deallocate (aliq)
  deallocate (nnats)
  deallocate (nnatl)
  deallocate (anat)
  deallocate (nice)
  deallocate (aice)
	
return
end subroutine analytic
