      SUBROUTINE INITRANS                                               
C
CCC
C   __  __   _____   _____   _____   _____   _____
C  |  \/  | /     \ /     \ /  _  \ /  ___\ |  ___|
C  |      | |  |  | |  <--< |  _  | |  \_ \ |  ___|
C  |_|\/|_| \_____/ \_____/ \_/ \_/ \_____/ |_____|
C
C   MODELE DE CHIMIE ATMOSPHERIQUE A GRANDE ECHELLE
C   VERSION_RELEASE_BUGFIX : 0_1_1 
C
CCC DATE : 15/05/2005
C
C   Type      : 3D semi-lagrangian transport scheme 
C   Purpose   : Initializations
C   Called by : 
C
CCC                                    
C 
C  Original : P. Simon, 03/1999
C
C  Modified : V.-H. Peuch (inclusion dans MOCAGE), 04/1999
C
CCC
C
      USE PARADI_MOD
      USE EXPE_MOD
      USE MOCF_MOD
      USE GRIDS_MOD
      USE CSTES_MOD
#ifdef GAUSS
      USE GAUSS_MOD
#endif
C
      IMPLICIT NONE
C
C     Subroutine arguments.
C     ~~~~~~~~~~~~~~~~~~~~
C
C     Variables locales.
C     ~~~~~~~~~~~~~~~~~
C		
      INTEGER :: J,K,KK,LATCHECK
      REAL (KIND=8) :: SL,CSTESURF,RLATDEB,RLATFIN
      REAL (KIND=8) :: XLAT1,XLAT2,XLAT3,XLAT4
#ifdef GAUSS
      REAL (KIND=8),ALLOCATABLE :: GMU(:),GW(:),GMUT(:)
#endif
      LOGICAL :: IIMPR
      CHARACTER(LEN=32) :: TEXT
C HLB 07/04/2006
C  Probleme de precision dans la definition des positions relatives des grilles imbriquees
C  Le resultat de la transforamtion en entier du calcul de la position de la grille peut differer
C  selon les machines
C  le passage par un reel simple precision permet de resoudre ce probleme
      REAL (KIND=4) :: tmpx12,tmpx23,tmpx34,
     +                 tmpy12,tmpy23,tmpy34
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                       
C  Impression des grilles modele / forcage
C
      IIMPR=.FALSE.
C                                                                       
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                       
C  Initialisation des differentes grilles (forcage et transport)        
C                                                                       
C  DLON* : longitudes des points de grille (en allant vers l'Est)       
C  DLAT* : latitudes  des points de grille (du S vers le N)             
C                                                                       
C  1. Initialisation des grilles de forcage                             
C                                                                       
C  1.1. Initialisation de la grille de forcage globale (F1)             
C                                                                       
      IF(IFLAG1) THEN                                                   
C                                                                       
C  RLONF1: longitudes (radians) des points de grille (en allant vers l'Est)
C                                                                       
        DRLONF1=(RLONF1MAX-RLONF1MIN)/REAL(LONF1)
#ifndef GAUSS      
        DO J=1,LONF1                                                  
          RLONF1(J)=RLONF1MIN+DRLONF1*(REAL(J)-.5)
        ENDDO         
#else
        DO J=1,LONF1
          RLONF1(J)=RLONF1MIN+DRLONF1*REAL(J-1)
        ENDDO
#endif  
C                                                                       
C  RLATF1: latitudes (radians) des points de grille (du S vers le N)    
C
#ifndef GAUSS      
        DRLATF1=(RLATF1MAX-RLATF1MIN)/REAL(LATF1)
        DO K=1,LATF1
          RLATF1(K)=RLATF1MIN+DRLATF1*(REAL(K)-.5)
        ENDDO
#endif         
C                                                                       
C  sinus et cosinus des points de la grille                             
C                                                                       
        DO K=1,LATF1
          CLAT1(K)=COS(RLATF1(K))
          SLAT1(K)=SIN(RLATF1(K))
        ENDDO
C                                                                       
        DO J=1,LONF1
         SLON1(J)=SIN(RLONF1(J))
         CLON1(J)=COS(RLONF1(J))
        ENDDO
C                                                                       
C  idem dans le repere auxiliaire                                       
C  (voir BACKTRA pour les explications relatives a ce repere)           
C     
        DO K=1,LATF1
        DO J=1,LONF1
          SLAT1R(J,K)=-CLON1(J)*CLAT1(K)
          CLAT1R(J,K)=SQRT(1.-SLAT1R(J,K)*SLAT1R(J,K))
          CLON1R(J,K)=MAX(-1.,MIN(SLAT1(K)/CLAT1R(J,K),1.))
          SL=SQRT(1.-CLON1R(J,K)*CLON1R(J,K))
          SLON1R(J,K)=SIGN(SL,SLON1(J))
        ENDDO
        ENDDO
C                                                                       
      ENDIF
C                                                                       
C  1.2. Initialisation de la grille de forcage continentale (F2)        
C                                                                       
      IF(IFLAG2) THEN                                                   
C                                                                       
C  RLONF2: longitudes (radians) des points de grille (en allant vers l'E
C                                                                       
        DRLONF2=(RLONF2MAX-RLONF2MIN)/REAL(LONF2)
        DO J=1,LONF2
          RLONF2(J)=RLONF2MIN+DRLONF2*(REAL(J)-.5)
        ENDDO
C                                                                       
C  RLATF2: latitudes (radians) des points de grille (du S vers le N)    
C                                                                       
        DRLATF2=(RLATF2MAX-RLATF2MIN)/REAL(LATF2)
        DO K=1,LATF2
          RLATF2(K)=RLATF2MIN+DRLATF2*(REAL(K)-.5)
        ENDDO
C                                                                       
C  sinus et cosinus des points de la grille                             
C                                                                       
        DO K=1,LATF2
          CLAT2(K)=COS(RLATF2(K))
          SLAT2(K)=SIN(RLATF2(K))
        ENDDO
C
        DO J=1,LONF2
          SLON2(J)=SIN(RLONF2(J))
          CLON2(J)=COS(RLONF2(J))
        ENDDO
C                                                                       
C  idem dans le repere auxiliaire                                       
C  (voir BACKTRA pour les explications relatives a ce repere)           
C                                                                       
        DO K=1,LATF2
        DO J=1,LONF2
          SLAT2R(J,K)=-CLON2(J)*CLAT2(K)
          CLAT2R(J,K)=SQRT(1.-SLAT2R(J,K)*SLAT2R(J,K))
          CLON2R(J,K)=MAX(-1.,MIN(SLAT2(K)/CLAT2R(J,K),1.))
          SL=SQRT(1.-CLON2R(J,K)*CLON2R(J,K))
          SLON2R(J,K)=SIGN(SL,SLON2(J))
        ENDDO
        ENDDO
C
      ENDIF
C                                                                       
C  1.3. Initialisation de la grille de forcage regionale    (F3)        
C                                                                       
      IF(IFLAG3) THEN                                                   
C                                                                       
C  RLONF3: longitudes (radians) des points de grille (en allant vers l'E
C                                                                       
        DRLONF3=(RLONF3MAX-RLONF3MIN)/REAL(LONF3)
        DO J=1,LONF3
          RLONF3(J)=RLONF3MIN+DRLONF3*(REAL(J)-.5)
        ENDDO
C                                                                       
C  RLATF3: latitudes (radians) des points de grille (du S vers le N)    
C                                                                       
        DRLATF3=(RLATF3MAX-RLATF3MIN)/REAL(LATF3)
        DO K=1,LATF3
          RLATF3(K)=RLATF3MIN+DRLATF3*(REAL(K)-.5)
        ENDDO
C                                                                       
C  sinus et cosinus des points de la grille                             
C                                                                       
        DO K=1,LATF3
          CLAT3(K)=COS(RLATF3(K))
          SLAT3(K)=SIN(RLATF3(K))
        ENDDO
C                                                                       
        DO J=1,LONF3
          SLON3(J)=SIN(RLONF3(J))
          CLON3(J)=COS(RLONF3(J))
        ENDDO
C                                                                       
C  idem dans le repere auxiliaire                                       
C  (voir BACKTRA pour les explications relatives a ce repere)           
C                                                                       
        DO K=1,LATF3
        DO J=1,LONF3
          SLAT3R(J,K)=-CLON3(J)*CLAT3(K)
          CLAT3R(J,K)=SQRT(1.-SLAT3R(J,K)*SLAT3R(J,K))
          CLON3R(J,K)=MAX(-1.,MIN(SLAT3(K)/CLAT3R(J,K),1.))
          SL=SQRT(1.-CLON3R(J,K)*CLON3R(J,K))
          SLON3R(J,K)=SIGN(SL,SLON3(J))
        ENDDO
        ENDDO
C
      ENDIF
C
C  1.4 Initialization for zoom domain (forcings grids : F4)
C
      IF(IFLAG4) THEN
C
C  RLONF4: longitudes (rad ; W -> E)
C
        DRLONF4=(RLONF4MAX-RLONF4MIN)/REAL(LONF4)
        DO J=1,LONF4
          RLONF4(J)=RLONF4MIN+DRLONF4*(REAL(J)-.5)
        ENDDO
C
C  RLATF4: latitudes (rad ; S -> N)
C
        DRLATF4=(RLATF4MAX-RLATF4MIN)/REAL(LATF4)
        DO K=1,LATF4
         RLATF4(K)=RLATF4MIN+DRLATF4*(REAL(K)-.5)
        ENDDO
C
        DO K=1,LATF4
          CLAT4(K)=COS(RLATF4(K))
          SLAT4(K)=SIN(RLATF4(K))
        ENDDO
C
        DO J=1,LONF4
         SLON4(J)=SIN(RLONF4(J))
         CLON4(J)=COS(RLONF4(J))
        ENDDO
C
        DO K=1,LATF4
        DO J=1,LONF4
          SLAT4R(J,K)=-CLON4(J)*CLAT4(K)
          CLAT4R(J,K)=SQRT(1.-SLAT4R(J,K)*SLAT4R(J,K))
          CLON4R(J,K)=MAX(-1.,MIN(SLAT4(K)/CLAT4R(J,K),1.))
          SL=SQRT(1.-CLON4R(J,K)*CLON4R(J,K))
          SLON4R(J,K)=SIGN(SL,SLON4(J))
        ENDDO
        ENDDO
C
      ENDIF
C                                                                       
C  2. Initialisation des grilles de transport                           
C                                                                       
C  2.1. Initialisation de la grille de transport globale (M1)           
C                                                                       
      IF(IFLAG1) THEN                                                   
C                                                                       
C  RLONM1: longitudes (radians) des points de grille (en allant vers l'Est)
C                                                                       
        IF(IDGRI1) THEN
          IF(LONM1.NE.LONF1) THEN
            PRINT *,'ERREUR !!! ',LONM1,LONF1
            CALL MOCAGE_STOP
          ENDIF
        ENDIF
C
        DRLONM1=(RLONM1MAX-RLONM1MIN)/REAL(LONM1)
#ifndef GAUSS      
        DO J=1,LONM1                                                  
          RLONM1(J)=RLONM1MIN+DRLONM1*(REAL(J)-.5)
        ENDDO
#else
        DO J=1,LONM1
          RLONM1(J)=RLONM1MIN+DRLONM1*REAL(J-1)
        ENDDO
#endif
C                                                                       
C  RLATM1: latitudes (radians) des points de grille (du S vers le N)    
C                                                                       
        IF(IDGRI1) THEN
          IF(LATM1.NE.LATF1) THEN
            PRINT *,'ERREUR !!! ',LATM1,LATF1
            CALL MOCAGE_STOP
          ENDIF
        ENDIF
C                 
#ifndef GAUSS      
        DRLATM1=(RLATM1MAX-RLATM1MIN)/REAL(LATM1)
        DO K=1,LATM1
          RLATM1(K)=RLATM1MIN+DRLATM1*(REAL(K)-.5)
          DLATM1(K)=RLATM1MIN+DRLATM1*REAL(K-1)
        ENDDO
        DLATM1(LATM1+1)=RLATM1MAX
C                                                                       
C  SURFM1 : grid-cell areas
C
        CSTESURF=RTER*RTER*DRLONM1*DRLATM1
        DO K=1,LATM1
        DO J=1,LONM1
          SURFM1(J,K)=CSTESURF*COS(RLATM1(K))
        ENDDO
        ENDDO
C
#else
C
C  RLATM1 : latitudes (radians) des points de grille (du S vers le N)
C  DLATM1 : latitudes (radians) des interfaces (du S vers le N)
C           pour la grille MOCAGE (grille de Gauss T MI0)
C
        ALLOCATE(GMU (LATM1  ))
        ALLOCATE(GW  (LATM1  ))
        ALLOCATE(GMUT(LATM1+1))
C
        OPEN (94,FILE='Gauss',form='unformatted',STATUS='OLD')
        READ(94) LATCHECK
        IF (LATCHECK.NE.LATM1) THEN
          PRINT*,"WRONG GAUSS FILE: ABORT!"
          CALL MOCAGE_STOP
        ENDIF
        READ(94) GMU
        READ(94) GW
        CLOSE (94)
C
C  Latitudes des interfaces
C
        GMUT  (1)=-1.
        DLATM1(1)=-XPIH
        DO K=2,LATM1/2
          GMUT  (K)=GMUT(K-1)+2.*GW(K-1)
          DLATM1(K)=ASIN(GMUT(K))
        ENDDO
        DLATM1(LATM1/2+1)=0.
C
C  Symetrie par rapport a l'equateur
C
        DO K=1,LATM1/2
          KK=LATM1+2-K
          DLATM1(KK)=-DLATM1(K)
        ENDDO
C
C  SURFM1 : grid-cell areas
C
        CSTESURF=RTER*RTER*DRLONM1
        DO K=1,LATM1
          DO J=1,LONM1
            SURFM1(J,K)=CSTESURF*(DLATM1(K+1)-DLATM1(K))*
     +                  COS(RLATM1(K))
          ENDDO
        ENDDO
C
C  Coefficients pour les interpolations cubiques en latitude
C
        DO K=1,LATM1
          XLAT(K)=RLATM1(K)
        ENDDO
C
        XLAT(     -1)=-XPI-XLAT(2)
        XLAT(      0)=-XPI-XLAT(1)
        XLAT(LATM1+1)=+XPI-XLAT(LATM1)
        XLAT(LATM1+2)=+XPI-XLAT(LATM1-1)
C
C  Denominateurs precalcules pour le calcul des poids en latitude
C
        DO K=0,LATM1
          XLAT1=XLAT(K-1)
          XLAT2=XLAT(K  )
          XLAT3=XLAT(K+1)
          XLAT4=XLAT(K+2)
          YCOEF(K,1)=1./ ((XLAT1-XLAT2)*(XLAT1-XLAT3)*(XLAT1-XLAT4))
          YCOEF(K,2)=1./ ((XLAT2-XLAT1)*(XLAT2-XLAT3)*(XLAT2-XLAT4))
          YCOEF(K,3)=1./ ((XLAT3-XLAT1)*(XLAT3-XLAT2)*(XLAT3-XLAT4))
          YCOEF(K,4)=1./ ((XLAT4-XLAT1)*(XLAT4-XLAT2)*(XLAT4-XLAT3))
        ENDDO
C
        DEALLOCATE(GMU)
        DEALLOCATE(GW)
        DEALLOCATE(GMUT)
#endif      
      ENDIF                                                             
C                                                                       
C  2.2. Initialisation de la grille de transport continentale (M2)      
C                                                                       
      IF(IFLAG2) THEN                                                   
C                                                                       
C  RLONM2: longitudes (radians) des points de grille (en allant vers l'E
C                                                                       
        IF(IDGRI2) THEN
          IF(LONM2.NE.LONF2) THEN
            PRINT *,'ERREUR !!! ',LONM2,LONF2
            CALL MOCAGE_STOP
          ENDIF
        ENDIF
C                                                                       
        DRLONM2=(RLONM2MAX-RLONM2MIN)/REAL(LONM2)
        DO J=1,LONM2                                                  
          RLONM2(J)=RLONM2MIN+DRLONM2*(REAL(J)-.5)
        ENDDO
C                                                                       
C  RLATM2: latitudes (radians) des points de grille (du S vers le N)    
C                                                                       
        IF(IDGRI2) THEN
          IF(LATM2.NE.LATF2) THEN
            PRINT *,'ERREUR !!! ',LATM2,LATF2
            CALL MOCAGE_STOP
          ENDIF
        ENDIF
C
        DRLATM2=(RLATM2MAX-RLATM2MIN)/REAL(LATM2)
        DO K=1,LATM2
          RLATM2(K)=RLATM2MIN+DRLATM2*(REAL(K)-.5)
          DLATM2(K)=RLATM2MIN+DRLATM2*REAL(K-1)
        ENDDO
        DLATM2(LATM2+1)=RLATM2MAX
C                         
C  SURFM2 : grid-cell areas
C
        CSTESURF=RTER*RTER*DRLONM2*DRLATM2
        DO K=1,LATM2
        DO J=1,LONM2
          SURFM2(J,K)=CSTESURF*COS(RLATM2(K))
        ENDDO
        ENDDO
C                                                                       
      ENDIF                                                             
C                                                                       
C  2.3. Initialisation de la grille de transport regionale    (M3)      
C                                                                       
      IF(IFLAG3) THEN                                                   
C                                                                       
C  RLONM3: longitudes (radians) des points de grille (en allant vers l'E
C                                                                       
        IF(IDGRI3) THEN
          IF(LONM3.NE.LONF3) THEN
            PRINT *,'ERREUR !!! ',LONM3,LONF3
            CALL MOCAGE_STOP
          ENDIF
        ENDIF
C
        DRLONM3=(RLONM3MAX-RLONM3MIN)/REAL(LONM3)
        DO J=1,LONM3
          RLONM3(J)=RLONM3MIN+DRLONM3*(REAL(J)-.5)
        ENDDO
C                                                                       
C  RLATM3: latitudes (radians) des points de grille (du S vers le N)    
C                                                                       
        IF(IDGRI3) THEN
          IF(LATM3.NE.LATF3) THEN
            PRINT *,'ERREUR !!! ',LATM3,LATF3
            CALL MOCAGE_STOP
          ENDIF
        ENDIF
C
        DRLATM3=(RLATM3MAX-RLATM3MIN)/REAL(LATM3)
        DO K=1,LATM3
          RLATM3(K)=RLATM3MIN+DRLATM3*(REAL(K)-.5)
          DLATM3(K)=RLATM3MIN+DRLATM3*REAL(K-1)
        ENDDO
        DLATM3(LATM3+1)=RLATM3MAX
C
C  SURFM3 : grid-cell areas
C
        CSTESURF=RTER*RTER*DRLONM3*DRLATM3
        DO K=1,LATM3
        DO J=1,LONM3
          SURFM3(J,K)=CSTESURF*COS(RLATM3(K))
        ENDDO
        ENDDO
C                                                                       
      ENDIF                                                             
C
C  2.4. Initialization for zoom domain (model grid : M4)
C 
      IF(IFLAG4) THEN
C 
C  RLONM4: longitudes (rad ; W -> E)
C 
        IF(IDGRI4) THEN
          IF(LONM4.NE.LONF4) THEN
            PRINT *,'ERREUR !!! ',LONM4,LONF4
            CALL MOCAGE_STOP
          ENDIF
        ENDIF
C 
        DRLONM4=(RLONM4MAX-RLONM4MIN)/REAL(LONM4)
        DO J=1,LONM4
          RLONM4(J)=RLONM4MIN+DRLONM4*(REAL(J)-.5)
        ENDDO
C 
C  RLATM4: latitudes (rad ; S -> N)
C 
        IF(IDGRI4) THEN
          IF(LATM4.NE.LATF4) THEN
            PRINT *,'ERREUR !!! ',LATM4,LATF4
            CALL MOCAGE_STOP
          ENDIF
        ENDIF
C 
        DRLATM4=(RLATM4MAX-RLATM4MIN)/REAL(LATM4)
        DO K=1,LATM4
          RLATM4(K)=RLATM4MIN+DRLATM4*(REAL(K)-.5)
          DLATM4(K)=RLATM4MIN+DRLATM4*REAL(K-1)
        ENDDO
        DLATM4(LATM4+1)=RLATM4MAX
C
C  SURFM4 : grid-cell areas
C
        CSTESURF=RTER*RTER*DRLONM4*DRLATM4
        DO K=1,LATM4
        DO J=1,LONM4
          SURFM4(J,K)=CSTESURF*COS(RLATM4(K))
        ENDDO
        ENDDO
C 
      ENDIF
C                                                                       
C  limite pour changer de repere                                        
C                                                                       
      RLATLIM=45./CONV                                                  
C                                                                       
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                       
C  3. Relative positions of grids
C                                                                       
C  3.1 M1 versus M2
C                                                                      
      DO J=1,LONM1*LATM1
        IGRID12(J,1)=0
      ENDDO
C
      IF(IFLAG2) THEN
        DO K=1,LATM1
        DO J=1,LONM1
          IF((RLONM1(J).GE.RLONM2MIN).AND.
     +       (RLONM1(J).LE.RLONM2MAX).AND.
     +       (RLATM1(K).GE.RLATM2MIN).AND.
     +       (RLATM1(K).LE.RLATM2MAX)) THEN
            IGRID12(J,K)=1
          ENDIF
        ENDDO
        END DO
C
C
        DO J=1,LONM1
          TMPX12=(RLONM1(J)-(RLONM2(1)-DRLONM2))/DRLONM2
          IX12(J)=INT(TMPX12)
        ENDDO
C
        DO K=1,LATM1
          TMPY12=(RLATM1(K)-(RLATM2(1)-DRLATM2))/DRLATM2
          IY12(K)=INT(TMPY12)
        ENDDO
C
C
      ENDIF
C                                                                       
C  3.2 M2 versus M3
C                                                                       
      DO J=1,LONM2*LATM2
        IGRID23(J,1)=0
      ENDDO
C
      IF(IFLAG3) THEN
        DO K=1,LATM2
        DO J=1,LONM2
          IF((RLONM2(J).GE.RLONM3MIN).AND.
     +       (RLONM2(J).LE.RLONM3MAX).AND.
     +       (RLATM2(K).GE.RLATM3MIN).AND.
     +       (RLATM2(K).LE.RLATM3MAX)) THEN
            IGRID23(J,K)=1
          ENDIF
        ENDDO
        ENDDO
C
C
        DO J=1,LONM2
          TMPX23=(RLONM2(J)-(RLONM3(1)-DRLONM3))/DRLONM3
          IX23(J)=INT(TMPX23)
        ENDDO
C
        DO K=1,LATM2
          TMPY23=(RLATM2(K)-(RLATM3(1)-DRLATM3))/DRLATM3
          IY23(K)=INT(TMPY23)
        ENDDO
C
C
      ENDIF
C
C  3.3 M3 versus M4 
C
      DO J=1,LONM3*LATM3
        IGRID34(J,1)=0
      ENDDO
C
      IF(IFLAG4) THEN
        DO K=1,LATM3
        DO J=1,LONM3
          IF((RLONM3(J).GE.RLONM4MIN).AND.
     +       (RLONM3(J).LE.RLONM4MAX).AND.
     +       (RLATM3(K).GE.RLATM4MIN).AND.
     +       (RLATM3(K).LE.RLATM4MAX)) THEN
            IGRID34(J,K)=1
          ENDIF
        ENDDO
        ENDDO
C
C
        DO J=1,LONM3
          TMPX34=(RLONM3(J)-(RLONM4(1)-DRLONM4))/DRLONM4
          IX34(J)=INT(TMPX34)
        ENDDO
C
        DO K=1,LATM3
          TMPY34=(RLATM3(K)-(RLATM4(1)-DRLATM4))/DRLATM4
          IY34(K)=INT(TMPY34)
        ENDDO
C
C
      ENDIF
C                                                                       
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                       
C  4. Diagnostics
C                                                                       
      PRINT *,'  MODELE DE TRANSPORT 3D SEMI-LAGRANGIEN'                
      IF(IFLAG1) PRINT *,'  POUR ',NTRA1,'ESPECES   (Grille glob.)'    
      IF(IFLAG2) PRINT *,'  POUR ',NTRA2,'ESPECES   (Grille cont.)'    
      IF(IFLAG3) PRINT *,'  POUR ',NTRA3,'ESPECES   (Grille reg. )'    
      IF(IFLAG4) PRINT *,'  POUR ',NTRA4,'ESPECES   (Grille zoom)'
      PRINT *,'  PAS DE TEMPS NOMINAL: ',DT0,'SECONDES'                
      PRINT *,'  PAS DE TEMPS POUR LES TRAJECTOIRES: '        
     +       ,DTN,'SECONDES'                                           
      PRINT *,'      (CALCUL UTILISANT ',NITAP,'ITERATIONS)'           
      PRINT *                                                           
      PRINT *,'  SIMULATION DE ',NCYCLT,'CYCLES DE ',NITERT            
     +       ,'ITERATIONS'                                             
C
      IF(IIMPR) THEN
C                                                                       
C  4.1 GLOBAL DOMAIN
C                                                                       
      IF(IFLAG1) THEN                                                   
C                                                                       
      TEXT='Grille de forcage globale       '                           
      CALL IMPGRI(RLONF1,RLATF1,ALF1,BLF1,TEXT,LONF1,LATF1,NIVF1)       
C                                                                       
      TEXT='Grille de transport globale     '                           
      CALL IMPGRI(RLONM1,RLATM1,ALM1,BLM1,TEXT,LONM1,LATM1,NIVM1)       
C                                                                       
      ENDIF                                                             
C                                                                       
C  4.2 CONTINENTAL DOMAIN
C                                                                       
      IF(IFLAG2) THEN                                                   
C                                                                       
      TEXT='Grille de forcage continentale  '                           
      CALL IMPGRI(RLONF2,RLATF2,ALF2,BLF2,TEXT,LONF2,LATF2,NIVF2)       
C                                                                       
      TEXT='Grille de transport continentale'                           
      CALL IMPGRI(RLONM2,RLATM2,ALM2,BLM2,TEXT,LONM2,LATM2,NIVM2)       
C                                                                       
      ENDIF                                                             
C                                                                       
C  4.3 REGIONAL DOMAIN
C                                                                       
      IF(IFLAG3) THEN                                                   
C                                                                       
      TEXT='Grille de forcage regionale     '                           
      CALL IMPGRI(RLONF3,RLATF3,ALF3,BLF3,TEXT,LONF3,LATF3,NIVF3)       
C                                                                       
      TEXT='Grille de transport regionale   '                           
      CALL IMPGRI(RLONM3,RLATM3,ALM3,BLM3,TEXT,LONM3,LATM3,NIVM3)       
C                                                                       
      ENDIF                                                             
C
C  4.4 ZOOM DOMAIN
C
      IF(IFLAG4) THEN
C
      TEXT='Grille de forcage zoom'
      CALL IMPGRI(RLONF4,RLATF4,ALF4,BLF4,TEXT,LONF4,LATF4,NIVF4)
C
      TEXT='Grille de transport zoom'
      CALL IMPGRI(RLONM4,RLATM4,ALM4,BLM4,TEXT,LONM4,LATM4,NIVM4)
C
      ENDIF
C
      ENDIF
C
      RLATDEB=(-60.)/CONV
      RLATFIN=(75.)/CONV
C
      IF (IFLAG1) THEN  
C
       KCVDEB1=1
       KCVFIN1=LATM1
       DO K=1,LATM1
         IF (RLATM1(K).LE.RLATDEB) THEN
           KCVDEB1=K
C           PRINT*,'KCVDEB=',KCVDEB1
         ENDIF
       ENDDO
       DO K=LATM1,1,-1
        IF (RLATM1(K).GE.RLATFIN) THEN
         KCVFIN1=K
C         PRINT*,'KCVFIN1=',KCVFIN1
        ENDIF
       ENDDO
C
      ENDIF
      IF (IFLAG2) THEN
C
       KCVDEB2=1
       KCVFIN2=LATM2
       DO K=1,LATM2
        IF (RLATM2(K).LE.RLATDEB) THEN
          KCVDEB2=K
        ENDIF
       ENDDO
       DO K=LATM2,1,-1
        IF (RLATM2(K).GE.RLATFIN) THEN
         KCVFIN2=K
        ENDIF
       ENDDO
C
      ENDIF
      IF (IFLAG3) THEN
C
       KCVDEB3=1
       KCVFIN3=LATM3
       DO K=1,LATM3
        IF (RLATM3(K).LE.RLATDEB) THEN
          KCVDEB3=K
        ENDIF
       ENDDO
       DO K=LATM3,1,-1
        IF (RLATM3(K).GE.RLATFIN) THEN
         KCVFIN3=K
        ENDIF
       ENDDO
C
      ENDIF
      IF (IFLAG4) THEN
C
       KCVDEB4=1
       KCVFIN4=LATM4
       DO K=1,LATM4
         IF (RLATM4(K).LE.RLATDEB) THEN
           KCVDEB4=K
         ENDIF
        ENDDO
        DO K=LATM4,1,-1
         IF (RLATM4(K).GE.RLATFIN) THEN
          KCVFIN4=K
         ENDIF
        ENDDO
C
      ENDIF
C
      RETURN                                                            
      END
